//---------------------------------------------------------------------------
//
//	$FileName	::	Debugger.cpp
//	$Purpose	::	DLL for project debugging in PSX IDE
//				to be used in conjunction with the emulator dll
//
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#ifndef DEBUGGER_DLL_C
#define DEBUGGER_DLL_C
#include "DebuggerDLL.h"
#endif
//---------------------------------------------------------------------------
USERES("Debugger.res");
USELIB("..\Emulator.lib");
USEFORM("DebugForm.cpp", DBGForm);
//---------------------------------------------------------------------------
#define	CLASSNAME	"DEBUGGER"
//---------------------------------------------------------------------------
__declspec( dllexport )	static	bool	fDebuggerPresent = false;
__declspec( dllexport ) static	bool	fDebuggerActive  = false;
__declspec( dllexport ) TDebugger	*lpDebugger = NULL;
static	HINSTANCE	hInstance = NULL;
//---------------------------------------------------------------------------
LRESULT	CALLBACK	DebugWndProc( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam );
//---------------------------------------------------------------------------
int 	WINAPI 	DllEntryPoint( HINSTANCE hinst, unsigned long reason, void *TlsIdent )
{
        switch ( reason ) {
        	case ( DLL_PROCESS_ATTACH ) : 	//	Process attachment
                case ( DLL_THREAD_ATTACH ) : {
                	DebugCreate();
                        break;
                }
                case ( DLL_PROCESS_DETACH ) : 	//	Process detachment
                case ( DLL_THREAD_DETACH ) : {
                	DebugTerminate();
                        break;
                }
        }
        return( true );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	DebugCreate( void )
{
        if ( lpDebugger == NULL )
        	lpDebugger = new TDebugger( true );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	DebugInitialise( void )
{
	if ( fDebuggerPresent == false )
        	DebugCreate();
        if ( lpDebugger != NULL )
        	fDebuggerActive = lpDebugger->Initialise();
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	DebugExecute( void )
{
	if ( ( !fDebuggerPresent ) || ( !fDebuggerActive ) )
        	DebugInitialise();
        if ( lpDebugger != NULL )
        	lpDebugger->Resume();
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	DebugConfigure( void )
{
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	DebugTerminate( void )
{
	if ( lpDebugger != NULL )
        	lpDebugger->Terminate();

	fDebuggerActive = false;
        fDebuggerPresent = false;
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	SystemMessage( LPSTR lpszMessage )
{
        printf( "%s\n", lpszMessage );
}
//---------------------------------------------------------------------------
__fastcall	TDebugger :: TDebugger( bool CreateSuspended ) : TThread( CreateSuspended )
{
	//	Load the emulator
        hEmulator = LoadLibrary( "Emulator.dll" );

        //	get instance
        hInstance = GetModuleHandle( NULL );

        //	register with windows messaging sub-system

        ZeroMemory( &WndClass, sizeof( WNDCLASS ) );

        WndClass.hInstance = hInstance;
        WndClass.lpszClassName = CLASSNAME;
        WndClass.lpfnWndProc = (WNDPROC)DebugWndProc;

        RegisterClass( &WndClass );

        fDebuggerPresent = true;

        if ( !CreateSuspended ) {
        	fDebuggerActive = true;
        	EmulatorExecute();
        }
}
//---------------------------------------------------------------------------
__fastcall	TDebugger :: ~TDebugger( void )
{
	delete this;
}
//---------------------------------------------------------------------------
int	__fastcall	TDebugger :: Initialise( void )
{
        if ( CPU != NULL )
        	CPU->CP0.CtrlRegs.Debug.DM = 1;	//	Tell the R3000A core that the debugger is in control

        fDebuggerActive = true;

        return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	TDebugger :: Terminate( void )
{
	UnregisterClass( CLASSNAME, &WndClass );
}
//---------------------------------------------------------------------------
void	__fastcall	TDebugger :: Reset( void )
{
}
//---------------------------------------------------------------------------
LRESULT	CALLBACK	DebugWndProc( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch ( uMsg ) {

        	case (WM_DBG_ACTIVATE	   ) : {
                	if ( !fDebuggerPresent )
                        	DebuggerCreate();
                	lpDebugger->Resume();
                        return( 0 );
                }

        	case (WM_DBG_DEACTIVATE    ) : {
                	lpDebugger->Suspend();
                	return( 0 );
                }

        	case ( WM_DBG_SET_BREAK     ) : {
                //	CPU->CP0.dwGenRegs[ v0 ] = Address
                //	CPU->CP0.dwGenRegs[ v1 ] = Flags
                	return( 0 );
                }

        	case (WM_DBG_CLR_BREAK     ) : {
                	return( 0 );
                }
        	case (WM_DBG_CLR_ALL_BREAK ) : {
                	return( 0 );
                }
        	case (WM_DBG_SINGLE        ) : {
                	return( 0 );
                }
        	case (WM_DBG_EXECUTE       ) : {
                	return( 0 );
                }
        	case (WM_DBG_CORE_DUMP     ) : {
                	return( 0 );
                }
        	case (WM_DBG_TERMINATE     ) : {
                	return( 0 );
                }
        	case (WM_DBG_READ          ) : {
                	return( 0 );
                }
        	case (WM_DBG_WRITE         ) : {
                	return( 0 );
                }
        	case (WM_DBG_TRIP_BREAK	   ) : {
                	return( 0 );
                }

        	default : {
                	return( DefWindowProc( wHandle, uMsg, wParam, lParam ) );
                }
        }
}
//---------------------------------------------------------------------------
