//---------------------------------------------------------------------------
//
//	$FileName	::	DebuggerDLL.h
//	$Purpose	::	WIN32 Debugger for PSX IDE
//				Allows for debugging of PlayStation code on
//				WIN32 via either Emulator (supplied) or
//				direct serial/parallel connection with console
//
//---------------------------------------------------------------------------
#ifndef	DEBUGGER_DLL_H
#define DEBUGGER_DLL_H
//---------------------------------------------------------------------------
#define	CPUMODE_EJTAGDEBUG_IMPLEMENTED
//---------------------------------------------------------------------------
#ifdef CONSOLE_TEST_APP
#undef DEBUGGER_DLL_C
#undef EMULATOR_DLL_C
#endif
//---------------------------------------------------------------------------
#ifdef  DEBUGGER_DLL_C
#define DEBUGGER_API	__declspec( dllexport )
#else
#define DEBUGGER_API	__declspec( dllimport )
#endif	//	DEBUGGER_API
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <Classes.hpp>
#include <System.hpp>
#include <SysUtils.hpp>
//---------------------------------------------------------------------------
#include "DebugForm.h"
//---------------------------------------------------------------------------
#ifndef	EMULATOR_DLL_H
#include "..\Emulator\EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
#define WM_DBG_ACTIVATE		WM_USER + 0x80000001
#define WM_DBG_DEACTIVATE       WM_USER + 0x80000002
#define WM_DBG_SET_BREAK        WM_USER + 0x80000003
#define WM_DBG_CLR_BREAK        WM_USER + 0x80000004
#define WM_DBG_CLR_ALL_BREAK    WM_USER + 0x80000005
#define WM_DBG_SINGLE           WM_USER + 0x80000006
#define WM_DBG_EXECUTE          WM_USER + 0x80000007
#define WM_DBG_CORE_DUMP        WM_USER + 0x80000008
#define WM_DBG_TERMINATE        WM_USER + 0x80000009
#define WM_DBG_READ             WM_USER + 0x8000000A
#define WM_DBG_WRITE            WM_USER + 0x8000000B
#define WM_DBG_TRIP_BREAK	WM_USER + 0x8000000C
//---------------------------------------------------------------------------
#define	SDBP_ACTIVATE			0x0001	//	Activates the debugger in the emulator
#define SDBP_DEACTIVATE			0x0002	//	Deactivate
#define SDBP_SET_BREAKPOINT		0x0003	//	Set break point address
#define SDBP_CLEAR_BREAKPOINT		0x0004	//	Clear a break point address
#define SDBP_CLEAR_ALL_BREAKPOINTS	0x0005
#define SDBP_SINGLE_STEP		0x0006
#define SDBP_EXECUTE			0x0007
#define SDBP_CORE_DUMP			0x0008
#define SDBP_TERMINATE			0x0009
#define SDBP_READ			0x000A
#define SDBP_WRITE			0x000B
#define SDBP_TRIP_BREAKPOINT		0x000C
//---------------------------------------------------------------------------
#define SDBP_FLAGS_READ		0x00000001	//	Read flag, breakpoint triggered on read
#define SDBP_FLAGS_WRITE	0x00000002	//	on write
#define SDBP_FLAGS_EXEC		0x00000004	//	on exec
#define SDBP_FLAGS_ENABLED	0x01000000	//	breakpoint is enabled
#define SDBP_FLAGS_DISABLED	0x00000000	//	breakpoint is disabled
//---------------------------------------------------------------------------
typedef	struct	_sdbp_breakpoint_def_	{
	DWORD	vAddr;				//	Address of breakpoint
        DWORD	Flags;				//	flags for breakpoint
        } SDBP_BreakPoint, *LPSDBP_BreakPoint;
//---------------------------------------------------------------------------
__declspec( dllimport )	bool	fDebuggerPresent;
__declspec( dllimport ) bool	fDebuggerActive;
//---------------------------------------------------------------------------
__declspec( dllimport ) void	__fastcall	DebugCreate( void );
__declspec( dllimport ) void	__fastcall	DebugInitialise( void );
__declspec( dllimport ) void	__fastcall	DebugExecute( void );
__declspec( dllimport ) void	__fastcall	DebugConfigure( void );
__declspec( dllimport ) void	__fastcall	DebugTerminate( void );
//---------------------------------------------------------------------------
class	TDebugger : public TThread
	{
        	protected:
                	void	__fastcall	Execute	( void );
                private:
                	WNDCLASS	WndClass;	//	used by windows
                        HANDLE		hEmulator;	//	link to emulator dll

                        LPSDBP_BreakPoint	*lpBreakPoints;
                        DWORD			dwNumBreakPoints;
                public:
                	__fastcall	TDebugger	( bool CreateSuspended );
                        __fastcall	~TDebugger	( void );

                        int	__fastcall	Initialise	( void );
                        void	__fastcall	Terminate	( void );
                        void	__fastcall	Reset		( void );
                __published:
        };
//---------------------------------------------------------------------------
__declspec( dllimport ) TDebugger *lpDebugger;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
