//---------------------------------------------------------------------------
//
//	$FileName	::	BIOS.h
//	$Purpose	::	Header for Internal Emulator PlayStation BIOS for PSXIDE
//
//---------------------------------------------------------------------------
#ifndef BIOSH
#define BIOSH
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
//!	??? What is this for ? This is defined in the original emulator from
//! PCSX team It is unknown as to the purpose of this structure. Does is have
//! something to do with the PAD's?
//---------------------------------------------------------------------------
typedef struct {
	long next;
	long func1;
	long func2;
	long pad;
} SysRPst;
//---------------------------------------------------------------------------
//	MIPS 32 ISA BIOS Emulation Layer. This is at the top of the emulator core
// and the only thing above this would be things like KERNEL, FILESYSTEMS, etc
//---------------------------------------------------------------------------
class	BIOS : public TObject
	{
        	friend	class	PSX_MEM;	//	Virtual Memory <--> BIOS (ROM) Interfacing
        	//	Allows interaction between the CPU and BIOS
        	friend	class	R3000A;
                //	Kernel <--> BIOS interface
                friend	class	PSX_Kernel;
        	protected:
                	void	__fastcall	Interrupt	( void );	//!< Interrupt handler for BIOS
                        void	__fastcall	Exception	( void );	//!< Exception handler for BIOS

                        //!	BIOS Event Management

                        void	__fastcall	OpenEvent	( void );
                private:
                	DWORD	*lpPsxBIOS;	//	BIOS file contents 512K

                	void	(*BIOSA0[ 256 ])();		//!<	BIOS A0 Calls (OSCORE RUNTIME LIBRARY)
                        void	(*BIOSB0[ 256 ])();		//!<	BIOS B0 Calls (KERNEL LIBRARY)
                        void	(*BIOSC0[ 256 ])();		//!<	BIOS C0 Calls (BIOS LIBRARY)

                        TList	*lpEventList;	//	VCL List for holding the psx events list

			static 	DWORD	*jmp_int;
			static 	WORD	*pad_buf;

                        DWORD	__fastcall	GROM	( DWORD Index );

                public:
                		__fastcall	BIOS		( void );	//!<	Constructor
                        	__fastcall	~BIOS   	( void );	//!<	Destructor

                	int	__fastcall	Initialise	( void );	//!<	Initialisation method
                        void	__fastcall	Terminate	( void );	//!<	Termination method

                        void	__fastcall	Reset		( void );	//!<	Reset method

                        int	__fastcall	LoadBIOS	( AnsiString asFileName );

                        friend	int	__fastcall	SaveState	( LPSTR lpszFileName );
                        friend	int	__fastcall	LoadState	( LPSTR lpszFileName );

                        friend	void	bios_malloc		( void );
                        friend	void	bios_InitHeap		( void );
                        friend	void	bios_OpenEvent		( void );
                        friend	void	bios_CloseEvent		( void );
                        friend	void	bios_WaitEvent		( void );
                        friend	void	bios_TestEvent		( void );
                        friend	void	bios_EnableEvent	( void );
                        friend	void	bios_DisableEvent	( void );
                        friend	void 	bios_ResetEntryInt	( void );
                        friend	void	bios_HookEntryInt	( void );
                __published:
                	__property	DWORD	ROM[ DWORD Index ] = { read=GROM };
        };
//---------------------------------------------------------------------------
__declspec( dllimport )	BIOS	*PsxBIOS;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
