//---------------------------------------------------------------------------
//
//	$FileName	::	CDROM.cpp
//	$Purpose	::	CDROM Support for PlayStation Emulator
//
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "CDROM.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
//---------------------------------------------------------------------------
//	The actual driver pointer, un-initialised, forces system to use dynamic
// memory allocation
//---------------------------------------------------------------------------
__declspec( dllexport ) PSX_CDROM	*CDROM	= NULL;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//	Command Names for CD-ROM Driver ( NOT REFERENCED )
//---------------------------------------------------------------------------
//char *CmdName[0x100]= {
//	"CdlSync",    "CdlNop",       "CdlSetloc",  "CdlPlay",
//	"CdlForward", "CdlBackward",  "CdlReadN",   "CdlStandby",
//	"CdlStop",    "CdlPause",     "CdlInit",    "CdlMute",
//	"CdlDemute",  "CdlSetfilter", "CdlSetmode", "CdlGetmode",
//	"CdlGetlocL", "CdlGetlocP",   "Cdl18",      "CdlGetTN",
//	"CdlGetTD",   "CdlSeekL",     "CdlSeekP",   NULL,
//	NULL,         "CdlTest",      "CdlID",      "CdlReadS",
//	"CdlReset",   NULL,           "CDlReadToc", NULL
//};
//---------------------------------------------------------------------------
//	Emulator <--> Windows CDROM driver Interface
//---------------------------------------------------------------------------
__fastcall	PSX_CDROM :: PSX_CDROM( void ) : TObject()
{
        // ....	Load the ASPI Manager
	hASPI32 = LoadLibrary( "WNASPI32" );	//	WNAPSPI32.DLL available ?

        if ( !hASPI32 ) {
        	// Handle ASPI load error here. Usually this involves the display of an
                // informative message based on the results of a call to GetLastError().

                // ....	NOPE!	just get the hell out of here with failing exception
                SystemMessage( "Unable to load WNASPI32.DLL\n" );
                throw "ASPI DLL MISSING.";
        }

        // ....	Yes, loaded and available. Now resolve the function addresses.

	lpfnGetASPI32SupportInfo	= GetProcAddress( hinstWNASPI32, "GetASPI32SupportInfo" );
	lpfnSendASPI32Command 		= GetProcAddress( hinstWNASPI32, "SendASPI32Command" );
	lpfnGetASPI32Buffer 		= GetProcAddress( hinstWNASPI32, "GetASPI32Buffer" );
	lpfnFreeASPI32Buffer 		= GetProcAddress( hinstWNASPI32, "FreeASPI32Buffer" );
	lpfnTranslateASPI32Address 	= GetProcAddress( hinstWNASPI32, "TranslateASPI32Address" );

        // ....	reset aspi interface and try to identify the device we want
        CDState.Status = lpfnGetASPISupportInfo();

        // .... number of hardware adapters is stored in the bits 0-7 of CDState.Status
        bNumHA = CDState.Status & 0x000000FF;

        if ( bNumHA == ) {	// ....	No adapters are available
        	MessageBox( NULL, "ARGH! No Hardware Adapters are available (ASPI)", "PSX_CDROM", MB_OK | MB_ICONEXCLAMATION );
                throw "PSX_CDROM FAILED";
        }

        // ....	at least one hardware adapter is present, scan all the adapters
        // for cdrom drives ignoring all other types of devices.

        for ( int iHA=0; iHA<bNumHA; iHA++ ) {
        	PSRB_HAInquiry	lpInquiry = new SRB_Inquiry;
                DWORD	dwState;

                ZeroMemory( lpInquiry, sizeof( SRB_Inquiry ) );

                // ....	setup for hardware adapter enquiry

                lpInquiry->SRB_Cmd = SC_HA_INQUIRY;
                lpInquiry->SRB_HaId = iHA;

                dwState = lpfnSendASPI32Command( lpInquiry );

        }

}
//---------------------------------------------------------------------------
//	$DESTRUCTOR	::	~PSX_CDROM
//	$Purpose	::	Destruction of the PSX_CDROM class
//---------------------------------------------------------------------------
__fastcall	PSX_CDROM :: ~PSX_CDROM( void )
{
	if ( hDevice != NULL )
        	Terminate();

        //	Disable CDROM IRQ

        PsxMem->RAM32[ IRQ_MASK ] |= ~0x08;	//	Memory-->Hardware interface
        CPU->CP0.CtrlRegs.Status.IM |= ~0x08;	//	CPU Interface

        //	Turn Off CD ROM Event
        lpCDEvent->status = EvStUNUSED;
        lpCDEvent->fhandler = NULL;
        lpCDEvent = NULL;

        // ....	Unload the ASPI dll
        if ( ( hASPI32 != NULL ) && ( hASPI32 != INVALID_HANDLE_VALUE ) )
        	FreeLibrary( hASPI32 );

        delete this;
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_CDROM :: Reset( void )
{	int 	i;

	memset( &cdr, 0, sizeof( cdr ) );

	cdr.CurTrack = 1;
	cdr.File     = 1;
        cdr.Channel  = 1;
	CurIrqQ      = -1;
	IrqQueue     = 0;
	CurCmdQ      = -1;
	CmdQueue     = 0;

	for ( i=0; i<8; i++ ) {
		CmdQ[ i ].Cmd = 0xff;
		IrqQ[ i ].cdIrqn = -1;
	}

        if ( dwFlags != 0 ) {
        	dev_cd_close();
        }
}
//---------------------------------------------------------------------------
int	__fastcall	PSX_CDROM :: Initialise( void )
{
        return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: Terminate( void )
{
	Reset();	//	Reset device
}
//---------------------------------------------------------------------------
//	Open the actual device ( MIPS --> WINDOWS )
//---------------------------------------------------------------------------
int	__fastcall	PSX_CDROM :: dev_cd_open( DWORD dwMode )
{
	//	We need to open the device at the windows level first

	return( 0 );
}
//---------------------------------------------------------------------------
int	__fastcall	PSX_CDROM :: dev_cd_read( LPVOID lpBuffer, DWORD Count, int fHandle )
{
	if ( !fHandle )
        	return( -1 );
	return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: dev_cd_close( void )
{

	dwShareMode = 0;
        dwAccessMode = 0;
        dwFlags = 0;

        ZeroMemory( &CDState, sizeof( CDState ) );

	return;
}
//---------------------------------------------------------------------------
int	__fastcall	PSX_CDROM :: dev_cd_firstfile( LPVOID lpSearchRec )
{
	return( 0 );
}
//---------------------------------------------------------------------------
int	__fastcall	PSX_CDROM :: dev_cd_nextfile( LPVOID lpSearchRec )
{
	return( 0 );
}
//---------------------------------------------------------------------------
int	__fastcall	PSX_CDROM :: dev_cd_chdir( LPSTR lpszDirName )
{
	return( 0 );
}
//---------------------------------------------------------------------------
//	add a command to the queue for processing
//---------------------------------------------------------------------------
void 	__fastcall	PSX_CDROM :: AddCmdQueue( unsigned char cmd, unsigned long ecycle )
{	int 	q;
	int	first = 0;

	if (CurCmdQ == -1) {
		CurCmdQ = 0; CmdQueue = 1;
		q = 0; first = 1;
	} else {
		q = CmdQueue + CurCmdQ;
		if (q > 7) q -= 8;
		if (CmdQueue < 8) CmdQueue++;
	}

	if (first) CDR_INT(ecycle)
	else {
		if ((--q) < 0) q += 8;
		if ((CmdQ[q].eCycle > ecycle) && (CmdQ[q].Cmd != 0xff)) {
			/* this irq happens before the last one */
			if ((--q) < 0) q += 8;
			CDR_INT(ecycle);
			if ((--CurCmdQ) < 0) CurCmdQ += 8;
		} else if ((++q) > 7) q -= 8;
	}

	CmdQ[q].Cmd = cmd;
	CmdQ[q].eCycle = ecycle;
}
//---------------------------------------------------------------------------
//	We probably need to add this as a friend method as direct inclusion
// will fail.
//---------------------------------------------------------------------------
void 	AddIrqQueue( int irqn, ... )
{	va_list	vl;
	int 	q, c, first = 0;

	if ( CDROM->CurIrqQ == -1 ) { // start queue
		CDROM->CurIrqQ = 0;
                CDROM->IrqQueue = 1;
		q = 0;
                first = 1;
	} else {
		q = CDROM->IrqQueue + CDROM->CurIrqQ;
		if ( q > 7 ) q -= 8;
		if ( CDROM->IrqQueue < 8 )
                	CDROM->IrqQueue++;
	}

	CDROM->IrqQ[ q ].cdIrqn = irqn;
	CDROM->IrqQ[ q ].count = 0;
	c = 0;

	va_start( vl, irqn );

	while ( irqn-- ) {
		CDROM->IrqQ[ q ].eCycle[ c ] = va_arg( vl, unsigned long );
		CDROM->IrqQ[ q ].Irq[ c++ ] = va_arg( vl, unsigned char );
	}
	va_end( vl );

	if ( first )
        	CDROM->AddCmdQueue( ASYNC, CDROM->IrqQ[ q ].eCycle[ 0 ] );

        //	Trigger Interrupt request at CPU level

        lpHW->dwHWM32[ 0x1070 ] |= 0x08;

        DWORD dw = lpHW->dwHWM32[ 0x1070 ];
        CPU->SignalException( CAUSE_INT, (void*)&dw );
}
//---------------------------------------------------------------------------
//	This processes the command queue
//---------------------------------------------------------------------------
void 	__fastcall	PSX_CDROM :: CDInterrupt( void )
{	unsigned char 	Cmd;

	CmdQueue--;
	Cmd = CmdQ[ CurCmdQ ].Cmd;
	CmdQ[ CurCmdQ ].Cmd = 0xff;

	if ( ++CurCmdQ == 8 )
        	CurCmdQ = 0;
	if ( CmdQueue ) {
		CDR_INT( CmdQ[ CurCmdQ ].eCycle );
	} else
        	CurCmdQ = -1;

	switch ( Cmd ) {

		case CdlSync:
			//cdr.Stat = NoIntr;
			break;

		case CdlPlay:
			//cdr.Stat = Complete;
			cdr.Result[ 0 ] = cdr.StatP;
			SetResultSize( 1 );
			if ( ( cdr.Mode & 0x5 ) == 0x5 )
                        	AddCmdQueue( REPPLAY, cdReadTime );
			break;

		case CdlID:
			//cdr.Stat = Complete;
			SetResultSize( 8 );
			break;

		case READ:
			if ( !cdr.Reading )
                        	break;
			DoCDReading();
			break;

		case REPPLAY_ACK:
			//cdr.Stat = Acknowledge;
			cdr.Result[ 0 ] = cdr.StatP;
			SetResultSize( 1 );
			AddCmdQueue( REPPLAY, cdReadTime );
			break;

		case REPPLAY:

			if ( ( cdr.Mode & 5 ) != 5 )
                        	break;

			//if ( CDR_getStatus( (CdrStat*)&CDState ) == -1 ) {
				cdr.Result[ 0 ] = 0;
				cdr.Result[ 1 ] = 0;
				cdr.Result[ 2 ] = 0;
				cdr.Result[ 3 ] = 0;
				cdr.Result[ 4 ] = 0;
				cdr.Result[ 5 ] = 0;
				cdr.Result[ 6 ] = 0;
				cdr.Result[ 7 ] = 0;
			//} else
                        	//memcpy( cdr.Result, &CDState. Track, 8 );
			cdr.Stat = 1;
			SetResultSize( 8 );
			AddCmdQueue( REPPLAY_ACK, cdReadTime );
			break;

		case ASYNC:
			cdr.Stat = IrqQ[ CurIrqQ ].Irq[ IrqQ[ CurIrqQ ].count ];

			if ( ( ++IrqQ[ CurIrqQ ].count ) < IrqQ[ CurIrqQ ].cdIrqn ) {
				AddCmdQueue( ASYNC, ( IrqQ[ CurIrqQ ].eCycle[ IrqQ[ CurIrqQ ].count ] ) );

			} else { // last irq from the current queue

				IrqQ[ CurIrqQ ].cdIrqn = -1; // reset irq
				if ( --IrqQueue ) {
					if ( ( ++CurIrqQ ) == 8 )
                                        	CurIrqQ = 0;

					AddCmdQueue( ASYNC, IrqQ[ CurIrqQ ].eCycle[ 0 ] );

				} else
                                	CurIrqQ = -1; // queue end
			}
			break;

		default:
			//cdr.Stat = Complete;
			break;
	}

	if ( cdr.Stat != 0 )
        	lpHW->dwHWM[ 0x1070 ] |= 0x4;	//	Set irq flag for CDROM
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_CDROM :: hwDma3( unsigned long madr, unsigned long bcr, unsigned long chcr )
{	unsigned long	cdaddr, cdsize;

	if ( ( chcr & 0x01000000 ) == 0x01000000 ) {
		cdaddr = madr & 0x1fffff;
		cdsize = bcr & 0xffff;

		if ( cdr.Readed == 1 ) {
			cdr.Readed = 0xff;
			cdr.pTransfer = cdr.Transfer;

			switch ( cdr.Mode & 0x70 ) {
				case 0x40:
                                case 0x60:
				case 0x00:
                                	cdr.pTransfer += 12;
                                        break;
				default:
                                	break;
			}
			printf ("cddma %x, %x\n", cdr.Mode, cdsize);
			if ( cdsize != 0x3 )
                        	cdr.pTransfer+=12;
		}

		//memcpy( (unsigned char*)PC_MEM( cdaddr ), cdr.pTransfer, cdsize * 4 );
                //ZeroMemory( (void*)PC_MEM( cdaddr ), cdsize );
		cdr.pTransfer += cdsize * 4;
	}
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: CDRefresh( void )
{
	if ( cdr.ResultReady && cdr.ResultC ) {
                lpHW->dwHWM8[ 0x1800 ] |= CDROM_CONTROL_CMD_COMPLETE;
                lpHW->dwHWM8[ 0x1801 ] = cdr.Result[ cdr.ResultP ];
        } else
        	cdr.Ctrl &= ~CDROM_CONTROL_CMD_COMPLETE;
}
//---------------------------------------------------------------------------
//
//!	Method		::	CDRead0
//
//!	bit 0 - 0 REG1 command send / 1 REG1 data read
//!	bit 1 - 0 data transfer finish / 1 data transfer ready/in progress
//!	bit 2 - unknown
//!	bit 3 - unknown
//!	bit 4 - unknown
//!	bit 5 - 1 result ready
//!	bit 6 - 1 dma ready
//!	bit 7 - 1 command being processed
//
//---------------------------------------------------------------------------
BYTE	__fastcall	PSX_CDROM :: CDRead0	( void )
{	static	bool	fCommandMode = ( !( lpHW->dwHWM8[ 0x1800 ] & 0x01 ) );
	//	REG0	CONTROL
	CDRefresh();			//	Refresh the cd CDStateus values

        if ( fCommandMode ) {
        	if ( cdr.OCUP )
                	cdr.Ctrl |= 0x40;
                else	cdr.Ctrl &= ~0x40;

                if ( cdr.Reg1Mode )
                	cdr.Ctrl |= 0x01;
                else	cdr.Ctrl &= ~0x01;

        	if ( cdr.DataTransfer )
                	cdr.Ctrl |= 0x02;
                else	cdr.Ctrl &= ~0x02;
        }

        lpHW->dwHWM8[ 0x1800 ] = cdr.Ctrl;

        // 	what means the 0x10 and the 0x08 bits? i only saw it used by the bios
        cdr.Ctrl |= 0x18;

        return( lpHW->dwHWM8[ 0x1800 ] );
}
//---------------------------------------------------------------------------
BYTE	__fastcall	PSX_CDROM :: CDRead1	( void )
{	//	REG1 COMMAND / DATA
	//	Refresh CD Status
	CDRefresh();

        //	which mode is REG1 in?
        if ( cdr.ResultReady && cdr.Reg1Mode ) {	//	Data
        	lpHW->dwHWM8[ 0x1801 ] = cdr.ResultP;
		cdr.ResultP++;
		cdr.DataTransfer = 1;
		if ( cdr.ResultP == cdr.ResultC )
                	cdr.ResultReady = 0;
	} else						//	Command
        	cdr.DataTransfer = 0;
	return lpHW->dwHWM8[ 0x1801 ];
}
//---------------------------------------------------------------------------
BYTE	__fastcall	PSX_CDROM :: CDRead2 ( void )
{	//	REG2	ERROR
	return( lpHW->dwHWM8[ 0x1802 ] );
}
//---------------------------------------------------------------------------
BYTE	__fastcall	PSX_CDROM :: CDRead3 ( void )
{	//	REG3 STATUS
	lpHW->dwHWM8[ 0x1803 ] = cdr.Stat;
	return( lpHW->dwHWM8[0x1803] );
}
//---------------------------------------------------------------------------
//	$Method		::	CDWrite0	sets the mode for reg1 command/data
//	$Parameter(s)	::	BYTE Value
//				Bit 0 = 0 - to send a command
//					1 - to get the result
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: CDWrite0	( BYTE Value )
{
	CDRefresh();

        if ( Value & 0x01 )	{	//	Read the result
        	CDState.Status = CDRead0();
                Value = CDState.Status;
                return;
        }

        AddCmdQueue( Value, psx_counters->ReadCount( 0 ) );
}
//---------------------------------------------------------------------------
//	Write to the CDROM REG 1 lpHW->dwHWM8[ 0x1801 ]
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: CDWrite1	( BYTE Value )
{	static	bool	fCommandMode = ( !( lpHW->dwHWM8[ 0x1800 ] & 0x01 ) );
	int 	i;
	//	REG 1 in command or data mode ?
	CDRefresh();

        if ( fCommandMode ) 	{	//	REG1	[   C O M M A N D   M O D E   ]

	lpHW->dwHWM8[ 0x1801 ] = Value;	//	set value in hardware mapped memory area

        cdr.Cmd = Value;
	cdr.OCUP = 0;

    	switch( cdr.Cmd ) {

        	case( CdlSync ) : {
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
//        		cdr.Stat = Complete;
                        break;
                }

                case( CdlNop ) : {
			SetResultSize( 1 );
        		cdr.Result[ 0 ] = cdr.StatP;
//        		cdr.Stat = Acknowledge;
			//i = CDState.Status;
        		//if ( CDR_getStatus( (CdrStat*)&CDState ) != -1 ) {
				if ( CDState.Type == 0xff )
/*                                	cdr.Stat = DiskError*/;
				if ( CDState.Status & 0x10 ) {
//					cdr.Stat = DiskError;
					cdr.Result[ 0 ] |=  0x11;
					cdr.Result[ 0 ] &= ~0x02;
				}
				else if ( i & 0x10 ) {
                                	cdr.StatP |= 0x2;
                                        cdr.Result[ 0 ] |= 0x2;
                                }
			//}
                        break;
                }

                case( CdlSetloc ) : {
                	for ( i=0; i<3; i++ )
                        	cdr.SetSector[ i ] = btoi( cdr.Param[ i ] );

                        cdr.SetSector[ 3 ] = 0;

                        if ( ( cdr.SetSector[ 0 ] | cdr.SetSector[ 1 ] | cdr.SetSector[ 2 ] ) == 0 ) {
                        	*( unsigned long *)cdr.SetSector = *(unsigned long *)cdr.SetSectorSeek;
                        }
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
                        cdr.Result[ 0 ] = cdr.StatP;
//                        cdr.Stat = NoIntr;
                        AddIrqQueue(1, 0x200);//, Acknowledge);
			if ( cdr.Reading ) {
				StopReading();
			}
                        break;
                }

                case( CdlPlay ) : {
                	if ( !cdr.SetSector[ 0 ] & !cdr.SetSector[ 1 ] & !cdr.SetSector[ 2 ] ) {
                        	//if ( CDR_getTN( cdr.ResultTN ) != -1 ) {
                                	if ( cdr.CurTrack > cdr.ResultTN[ 1 ] )
                                        	cdr.CurTrack = cdr.ResultTN[ 1 ];
                                        //if ( CDR_getTD( ( unsigned char)(cdr.CurTrack + 1), cdr.ResultTD ) != -1 ) {
                                        	int tmp = cdr.ResultTD[ 2 ];
                                                cdr.ResultTD[ 2 ] = cdr.ResultTD[ 0 ];
						cdr.ResultTD[ 0 ] = tmp;
                                                //if ( !Config.Cdda )
                                                //	CDR_play( cdr.ResultTD );
                                        //}
                                //}
                        } //else if ( !Config.Cdda )
                        	//CDR_play( cdr.SetSector );

                        SetResultSize( 1 );
			cdr.StatP |= 0x2;
                        cdr.Result[ 0 ] = cdr.StatP | 0x80;
//                        cdr.Stat = NoIntr;
                        cdr.Play = 1;
                        AddCmdQueue( cdr.Cmd, 0x400 );
                        break;
                }

                case( CdlForward ) : {
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
//        		cdr.Stat = NoIntr;
        		if ( cdr.CurTrack < 0xaa )
                        	cdr.CurTrack++;
        		AddIrqQueue( 1, 0x200);//, Complete );
        		break;
                }

                case( CdlBackward ) : {
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
        		//cdr.Stat = NoIntr;
        		if ( cdr.CurTrack > 1 )
                        	cdr.CurTrack--;
                        AddIrqQueue( 1, 0x200);//, Complete );
                	break;
                }

                case( CdlReadN ) : {
			SetResultSize( 1 );
                        cdr.StatP |= 0x22;
			cdr.Result[ 0 ] = cdr.StatP;
                        //cdr.Stat = NoIntr;
			StartReading( 1 );
                        break;
                }

                case( CdlStandby ) : {
                	if ( cdr.Play ) {
                        	//if ( !Config.Cdda )
                                //	CDR_stop();
                                cdr.StatP &= ~0x80;
                                cdr.Play = 0;
                        }
			StopReading();
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
                        cdr.Result[ 0 ] = cdr.StatP;
			//cdr.Stat = NoIntr;
        		AddIrqQueue( 1, 0x200);//, Complete );
                        break;
                }

                case( CdlStop ) : {
                	if ( cdr.Play ) {
                        	//if ( !Config.Cdda )
                                //	CDR_stop();
                                cdr.StatP &= ~0x80;
                                cdr.Play = 0;
                        }
			StopReading();
			SetResultSize( 1 );
                        cdr.StatP &= ~0x2;
			cdr.Result[ 0 ] = cdr.StatP;
        		//cdr.Stat = NoIntr;
        		AddIrqQueue(1, 0x200);//, Complete);
	    		AddIrqQueue(2, 0x200, 0x512);//Complete, 0x512, DataEnd);
        		break;
                }

                case( CdlPause ) : {
                	if ( cdr.Play ) {
                        	//if ( !Config.Cdda )
                                //	CDR_stop();
                                cdr.StatP &= ~0x80;
                                cdr.Play = 0;
                        }
			StopReading();
			SetResultSize( 1 );
                        cdr.StatP &= ~0x20;
			cdr.StatP |= 0x2;
			cdr.Result[ 0 ] = cdr.StatP;
                        //cdr.Stat = NoIntr;
                        AddIrqQueue( 1, 0x200);//, Complete );
                        AddIrqQueue( 2, 0x200,0x600);// Complete, 0x600, DataEnd );
                        break;
                }

                case( CdlReset ) :
                case( CdlInit ) : {
			SetResultSize( 1 );
                        if ( cdr.Play ) {
                        	//if ( !Config.Cdda )
                                //	CDR_stop();
                                cdr.Play = 0;
                        }
                        cdr.StatP = 0x2;
			cdr.Result[ 0 ] = cdr.StatP;
                        //cdr.Stat = Complete;
                        break;
                }

                case( CdlMute ) : {
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
        		//cdr.Stat = Complete;
        		cdr.Muted = 0;
        		break;
                }

                case( CdlDemute ) : {
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
                	//cdr.Stat = Complete;
        		cdr.Muted = 1;
        		break;
                }

                case( CdlSetfilter ) : {
                	cdr.File = cdr.Param[ 0 ];
                        cdr.Channel = cdr.Param[ 1 ];
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
        		//cdr.Stat = Complete;
	        	break;
                }

                case( CdlSetmode ) : {
	        	cdr.Mode = cdr.Param[ 0 ];
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
        		cdr.Result[ 0 ] = cdr.StatP;
	        	//cdr.Stat = Acknowledge;
	        	break;
                }

                case( CdlGetmode ) : {
			SetResultSize( 6 );
			cdr.StatP |= 0x2;
	        	cdr.Result[ 0 ] = cdr.StatP;
	        	//cdr.Stat = Complete;
	        	cdr.Result[ 1 ] = cdr.Mode;
	        	cdr.Result[ 2 ] = cdr.File;
	        	cdr.Result[ 3 ] = cdr.Channel;
	        	cdr.Result[ 4 ] = 0;
	        	cdr.Result[ 5 ] = 0;
	        	break;
                }

                case( CdlGetlocL ) : {
			SetResultSize( 8 );
                        //cdr.Stat = Complete;
                        for ( i=0; i<8; i++ )
                        	cdr.Result[ i ] = itob( cdr.Transfer[ i ] );
                        break;
                }

                case( CdlGetlocP ) : {
			SetResultSize( 8 );
	        	//cdr.Stat = Complete;
	        	cdr.Result[ 0 ] = 1;
	        	cdr.Result[ 1 ] = 0;
	        	cdr.Result[ 2 ] = cdr.Prev[ 0 ];
	        	cdr.Result[ 3 ] = itob( ( btoi( cdr.Prev[ 1 ] ) ) - 2 );
	        	cdr.Result[ 4 ] = cdr.Prev[ 2 ];
		    	memcpy( cdr.Result + 5, cdr.Prev, 3 );
	        	break;
                }

                case( CdlGetTN ) : {
			SetResultSize( 3 );
			cdr.StatP |= 0x2;
	        	cdr.Result[ 0 ] = cdr.StatP;
	        	//cdr.Stat = Complete;
	        	//if ( CDR_getTN( cdr.ResultTN ) == -1 ) {
                        	cdr.Result[ 1 ] = 1;
                                cdr.Result[ 2 ] = 1;
	        	//} else {
                        //	cdr.Result[ 1 ] = itob( cdr.ResultTN[ 0 ] );
                        //        cdr.Result[ 2 ] = itob( cdr.ResultTN[ 1 ] );
	        	//}
	        	break;
                }

                case( CdlGetTD ) : {
	        	cdr.Track = btoi( cdr.Param[ 0 ] );
			SetResultSize( 3 );
			cdr.StatP |= 0x2;
	        	cdr.Result[ 0 ] = cdr.StatP;
	        	//cdr.Stat = Complete;
	        	//if ( CDR_getTD( (unsigned char)cdr.Track, cdr.ResultTD ) == -1 ) {
                        	cdr.Result[ 1 ] = 0;
		    		cdr.Result[ 2 ] = 0;
	        	//} else {
		    	//	cdr.Result[ 1 ] = itob( cdr.ResultTD[ 2 ] );
	        	//    	cdr.Result[ 2 ] = itob( cdr.ResultTD[ 1 ] );
		    	//}
	        	break;
                }

                case( CdlSeekL ) : {
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
	        	cdr.Result[ 0 ] = cdr.StatP;
	        	//cdr.Stat = NoIntr;
	        	((unsigned long *)cdr.SetSectorSeek)[ 0 ] = ((unsigned long *)cdr.SetSector)[ 0 ];
	        	AddIrqQueue( 1, 0x400);//, Complete );
			AddIrqQueue( 2, 0x200, 0x400);//Acknowledge, 0x400, Complete );
	        	break;
                }

                case( CdlSeekP ) : {
                	( (unsigned long *)cdr.SetSectorSeek)[ 0 ] = ((unsigned long *)cdr.SetSector)[ 0 ];
			SetResultSize( 1 );
			cdr.StatP |= 0x2;
                        cdr.Result[ 0 ] = cdr.StatP;
                        //cdr.Stat = NoIntr;
                        AddIrqQueue(1, 0x200);//, Complete);
                        AddIrqQueue(2, 0x200, 0x200);//Acknowledge, 0x200, Complete);
                        break;
                }

                case( CdlTest ) : {
			SetResultSize( 4 );
                        //cdr.Stat = Acknowledge;
                        switch ( cdr.Param[ 0 ] ) {
                        	case( 0x20 ) : {	//	System Controller ROM Version - is this ok?
                                	cdr.Result[ 0 ] = 0x03;
                                        cdr.Result[ 1 ] = 0x06;
                                        cdr.Result[ 2 ] = 0x94;
                                        cdr.Result[ 3 ] = 0x00;
                                        break;
                                }
                        }
                        break;
                }

                case( CdlID ) : {

			SetResultSize( 8 );
                        //cdr.Stat = NoIntr;
                        //if ( CDR_getStatus( (CdrStat*)&CDState ) == -1 ) {
                        	cdr.Result[ 0 ] = 0x00; // 0x08 and cdr.Result[1]|0x10 : audio cd, enters cd player
                                cdr.Result[ 1 ] = 0x00; // 0x80 leads to the menu in the bios, else loads CD
                        //} else {
                        	if ( CDState.Type == 2 ) {
                                	cdr.Result[ 0 ] = 0x08;
                                        cdr.Result[ 1 ] = 0x10;
                                } else {
                                	cdr.Result[ 0 ] = 0x00;
                                        cdr.Result[ 1 ] = 0x00;
                                }
                        //}

                        //if ( !LoadCdBios )
                        //	cdr.Result[ 1 ] |= 0x80;

                        cdr.Result[ 2 ] = 0x00;
                        cdr.Result[ 3 ] = 0x00;

                        strncpy( (char *)&cdr.Result[ 4 ], "PCSX", 4 );

                        AddCmdQueue( cdr.Cmd, 0x200 );

                        break;
                }

                case( CdlReadS ) : {
			SetResultSize( 1 );
                        cdr.StatP |= 0x22;
                        cdr.Result[ 0 ] = cdr.StatP;
                        //cdr.Stat = NoIntr;
			StartReading( 2 );
                        break;
                }


                //!	CD Read Table Of Contents (TOC).
                case( CdlReadToc ) : {

			SetResultSize( 1 );
			cdr.StatP |= 0x2;
                        cdr.Result[ 0 ] = cdr.StatP;
                        //cdr.Stat = Complete;

                        break;
                }

                default : {
                	return;
                }

        }

        } else {	//	REG 1 MODE	[   D A T A   M O D E   ]

        }

	if ( cdr.Stat != 0 )
        	lpHW->dwHWM[ 0x1070 ] |= 0x4;
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: CDWrite2	( BYTE Value )
{
	if ( ( cdr.Reset == 2 ) && ( Value == 0x07 ) ) {
        	cdr.ParamP = 0;
		cdr.ParamC = 0;
		cdr.ResultReady = 0;
		cdr.Reset = 0;
                cdr.Ctrl = cdr.Stat = 0;
        } else {
        	cdr.Reset = 0;
		cdr.Param[ cdr.ParamP++ ] = Value;
		cdr.ParamC++;
    		if ( cdr.ParamP > 7 ) {
                	cdr.ParamP = 7;
                        cdr.ParamC = 7;
                }
        }
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM :: CDWrite3	( BYTE Value )
{
	if ( ( Value == 0x07 ) && ( cdr.Reset == 1 ) )
        	cdr.Reset = 2;
        else
        	cdr.Reset = 0;
        if ( cdr.ResultReady && ( Value == 7 ) ) {
        	cdr.ResultC = 0;
                cdr.Stat = 0;
        }
}
//---------------------------------------------------------------------------
//	THIS ALL IS FOR THE CDROM INTERRUPTIONS HANDLING
//---------------------------------------------------------------------------
void	__fastcall	PSX_CDROM ::  DoCDReading( void )
{	unsigned char	tst[ 4 ];

	if ( cdr.Readed ) {
		cdr.Prev[ 0 ] = itob( cdr.SetSector[ 0 ] );
		cdr.Prev[ 1 ] = itob( cdr.SetSector[ 1 ] );
		cdr.Prev[ 2 ] = itob( cdr.SetSector[ 2 ] );

		memcpy( tst, cdr.Prev, 3 );
		tst[ 3 ] = 0;

		//cdr.RErr = CDR_readTrack( tst );

		cdr.OCUP = 0;
		SetResultSize( 1 );
		cdr.Result[ 0 ] = cdr.StatP | 0x2;
		//cdr.Stat = Acknowledge;

		cdr.Synk = CPU->lpIrqQueue[ CPU->CurIrq ].cycle + ( ( cdr.Mode & 0x80 ) ? ( cdReadTime / 2 ) : cdReadTime );

		lpHW->dwHWM[ 0x1070 ] |= 0x08; //here we SET UP the interruption
		cdr.Readed = 0;

		if ( cdr.Mode & 0x4 ) {	//	NON BLOCKING MODE
			// i'm not sure about this, but now
			// voices in tales of destiny 2 stops
			// ok (linuzappz)
			cdr.ResultP = 0; cdr.ResultC = 8; cdr.ResultReady=1;
			cdr.Result[ 1 ] = 0;
			cdr.Result[ 2 ] = cdr.Prev[ 0 ];
			cdr.Result[ 3 ] = itob( ( btoi( cdr.Prev[ 1 ] ) ) - 2 );
			cdr.Result[ 4 ] = cdr.Prev[ 2 ];
			memcpy( cdr.Result + 5, cdr.Prev, 3 );
		}
		return;
        }

        cdr.OCUP = 1;
	SetResultSize( 1 );
        cdr.Result[ 0 ] = cdr.StatP | 0x2;
        //cdr.Stat = DataReady;

        cdr.Synk = CPU->lpIrqQueue[ CPU->CurIrq ].cycle + ( ( cdr.Mode & 0x80 ) ? ( cdReadTime / 2 ) : cdReadTime );

        lpHW->dwHWM[ 0x1070 ] |= 0x08; //	here we SET UP the interruption, Flag CD interrupt well hardware

        if ( cdr.RErr == -1 ) {
		memset( cdr.Transfer, 0, 2340 );
		//cdr.Stat = DiskError;
		cdr.Result[ 0 ]|= 0x01;
	}// else
        //	memcpy( cdr.Transfer, CDR_getBuffer(), 2340 );

        if ( ( cdr.Muted == 1 ) && ( cdr.Mode & 0x40 ) /*&& ( !Config.Xa )*/ && ( cdr.FirstSector != -1 ) ) { // CD-XA
        	if ( ( cdr.Transfer[ 4 + 2 ] & 0x4 ) &&
			( ( cdr.Mode & 0x8 ) ? ( cdr.Transfer[ 4 + 1 ] == cdr.Channel ) : 1 ) &&
			( cdr.Transfer[4+0] == cdr.File ) ) {
			int ret = DecodeXA->DecodeSector( (xa_decode_t*)&cdr.Xa, (DWORD)(cdr.Transfer + 4), cdr.FirstSector );

                        if ( !ret ) {
				//SPU_playADPCMchannel( (xa_decode_t*)&cdr.Xa );
				cdr.FirstSector = 0;
			} else
                        	cdr.FirstSector = -1;
                }
        }

        cdr.SetSector[ 2 ]++;
        if ( cdr.SetSector[ 2 ] == 75 ) {
        	cdr.SetSector[ 2 ] = 0;
                cdr.SetSector[ 1 ]++;
                if ( cdr.SetSector[ 1 ] == 60 ) {
                	cdr.SetSector[ 1 ] = 0;
                        cdr.SetSector[ 0 ]++;
                }
        }

        cdr.Readed = 1;

        //				ASYNC MODE
        if ( ( cdr.Transfer[ 4 + 2 ] & 0x80 ) && ( cdr.Mode & 0x2 ) ) { // EOF
		StopReading();
		AddIrqQueue(1, 0x400);//, DataEnd);
	}
}
//---------------------------------------------------------------------------
