//---------------------------------------------------------------------------
//!	PlayStation CDROM Emulation Layer
//
//!	$FileName	::	CDROM.h
//!	$Author		::	Chris Brooklyn
//!	$Purpose	::	To provide a low-level driver for emulation of the
//!				PlayStation CDROM driver
//
//---------------------------------------------------------------------------
#ifndef CDROMH
#define CDROMH
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
#include "Kernel.h"				//	Force inclusion of the kernel header, even if already included
#include "DecodeXA.h"                           //	and again for xa-decoding
//---------------------------------------------------------------------------
//	Interface to ASPI	Taken directly from adaptec's aspi win32 ref manual
//---------------------------------------------------------------------------
// ....	Error/Status Codes	Returned by SendASPI32Command and is stored in PSRB->SRB_Status also
//---------------------------------------------------------------------------
#define SS_PENDING			0x00	//	.
#define SS_COMP				0x01    //      .
#define SS_ABORTED			0x02    //      .
#define SS_ERR				0x04    //      .
#define SS_INVALID_CMD			0x80    //      .
#define SS_INVALID_HA			0x81    //      .
#define SS_NO_DEVICE			0x82    //      .
#define SS_INVALID_SRB			0xE0    //      .
#define SS_BUFFER_ALIGN			0xE1    //      .
#define SS_ILLEGAL_MODE			0xE2    //      .
#define SS_NO_ASPI			0xE3    //      .
#define SS_FAILED_INIT			0xE4    //      .
#define SS_ASPI_IS_BUSY			0xE5    //      .
#define SS_BUFFER_TO_BIG		0xE6    //      .
#define SS_MISMATCHED_COMPONENTS	0xE7    //      .
#define SS_NO_ADAPTERS			0xE8    //      .
#define SS_INSUFFICIENT_RESOURCES	0xE9    //      .
//---------------------------------------------------------------------------
// ....	Commands to be set in the _srb_header_
//---------------------------------------------------------------------------
#define	SC_HA_INQUIRY 			0x00	//	Queries ASPI for information on specific host adapters.
#define	SC_GET_DEV_TYPE 		0x01	//	Requests the SCSI device type for a specific SCSI target.
#define	SC_EXEC_SCSI_CMD 		0x02	//	Sends a SCSI command (arbitrary CDB) to a SCSI target.
#define	SC_ABORT_SRB 			0x03	//	Requests that ASPI cancel a previously submitted request.
#define	SC_RESET_DEV 			0x04	//	Sends a BUS DEVICE RESET message to a SCSI target.
#define	SC_GET_DISK_INFO 		0x06	//	Returns BIOS information for a SCSI target (Win95 only).
#define	SC_RESCAN_SCSI_BUS 		0x07	//	Requests a rescan of a host adapters SCSI bus.
#define	SC_GETSET_TIMEOUTS 		0x08	//	Sets SRB timeouts for specific SCSI targets.
//---------------------------------------------------------------------------
// ....	device types
//---------------------------------------------------------------------------
#define	DTYPE_DASD 			0x00 	//	Direct-access device (e.g. magnetic disk)
#define	DTYPE_SEQD     			0x01 	//	Sequential-access device (e.g. magnetic tape)
#define	DTYPE_PRNT			0x02 	//	Printer device
#define	DTYPE_PROC			0x03 	//	Processor device
#define	DTYPE_WORM			0x04 	//	Write-once device (e.g. some optical disks)
#define	DTYPE_CDROM			0x05 	//	CD-ROM device
#define	DTYPE_SCAN 			0x06 	//	Scanner device
#define	DTYPE_OPTI 			0x07 	//	Optical memory device (e.g. some optical disks)
#define	DTYPE_JUKE 			0x08 	//	Medium changer device (e.g. jukeboxes)
#define	DTYPE_COMM 			0x09 	//	Communication device
//---------------------------------------------------------------------------
#define SRB_POSTING 			0x01 	//	Enable posting. See Waiting for Completion for more information. This flag and SRB_EVENT_NOTIFY are mutually exclusive.
#define SRB_ENABLE_RESIDUAL_COUNT 	0x04 	//	Enables residual byte counting assuming it is supported. Whenever a data underrun occurs the SRB_BufLen field is updated to reflect the remaining bytes to transfer.
#define SRB_DIR_IN 			0x08 	//	Data transfer is from SCSI target to host. Mutually exclusive with SRB_DIR_OUT.
#define SRB_DIR_OUT 			0x10 	//	Data transfer is from host to SCSI target. Mutually exclusive with SRB_DIR_IN.
#define SRB_EVENT_NOTIFY 		0x40 	//	Enable event notification. See Waiting for Completion for more infomration. This flag and SRB_POSTING are mutually exclusive.
//---------------------------------------------------------------------------
//	These ones are reserved
//---------------------------------------------------------------------------
//#define	N/A 0x0A-0x0B Defined by ASC IT8 (Graphic arts pre-press devices)
//#define	N/A 0x0C-0x1E Reserved
//---------------------------------------------------------------------------
#define	DTYPE_UNKNOWN 			0x1F 	//	Unknown or no device type
//---------------------------------------------------------------------------
#define	HASTAT_OK 			0x00	//	Host adapter did not detect an error.
#define	HASTAT_TIMEOUT 			0x09 	//	The time allocated for a bus transaction ran out.
#define	HASTAT_COMMAND_TIMEOUT 		0x0B 	//	SRB expired while waiting to be processed.
#define	HASTAT_MESSAGE_REJECT 		0x0D 	//	MESSAGE REJECT received while processing SRB.
#define	HASTAT_BUS_RESET 		0x0E 	//	A bus reset was detected.
#define	HASTAT_PARITY_ERROR 		0x0F 	//	A parity error was detected.
#define	HASTAT_REQUEST_SENSE_FAILED	0x10 	//	The adapter failed in issuing a Request Sense after a check condition was reported by the target device.
#define	HASTAT_SEL_TO 			0x11 	//	Selection of target timed out.
#define	HASTAT_DO_DU 			0x12 	//	Data overrun.
#define	HASTAT_BUS_FREE 		0x13 	//	Unexpected Bus Free.
#define	HASTAT_PHASE_ERR 		0x14 	//	Target Bus phase sequence failure.
//---------------------------------------------------------------------------
#define	STATUS_GOOD 			0x00 	//	No target status.
#define	STATUS_CHKCOND 			0x02 	//	Check status (sense data is in SenseArea).
#define	STATUS_BUSY 			0x08 	//	Specified Target/LUN is busy.
#define	STATUS_RESCONF 			0x18 	//	Reservation conflict.
//---------------------------------------------------------------------------
#define	SENSE_LEN			18
//---------------------------------------------------------------------------
//	ASPI structures
//---------------------------------------------------------------------------
typedef struct _srb_header_ {
	BYTE	SRB_Cmd; 	// 	ASPI command code
        BYTE 	SRB_Status; 	// 	ASPI command status byte
        BYTE 	SRB_HaId; 	// 	ASPI host adapter number
        BYTE 	SRB_Flags; 	// 	ASPI request flags
        DWORD 	SRB_Hdr_Rsvd; 	// 	Reserved, MUST = 0
} SRB_Header, *PSRB_Header;
//---------------------------------------------------------------------------
typedef struct _srb_hainquiry_ {
	BYTE SRB_Cmd; // ASPI command code = SC_HA_INQUIRY
        BYTE SRB_Status; // ASPI command status byte
        BYTE SRB_HaId; // ASPI host adapter number
        BYTE SRB_Flags; // Reserved, MUST = 0
        DWORD SRB_Hdr_Rsvd; // Reserved, MUST = 0
        BYTE HA_Count; // Number of host adapters present
        BYTE HA_SCSI_ID; // SCSI ID of host adapter
        BYTE HA_ManagerId[16]; // String describing the manager
        BYTE HA_Identifier[16]; // String describing the host adapter
        BYTE HA_Unique[16]; // Host Adapter Unique parameters
        WORD HA_Rsvd1; // Reserved, MUST = 0
        } SRB_HAInquiry, *PSRB_HAInquiry;
//---------------------------------------------------------------------------
typedef struct _srb_gdevblock_ {
	BYTE SRB_Cmd; // ASPI command code = SC_GET_DEV_TYPE
        BYTE SRB_Status; // ASPI command status byte
        BYTE SRB_HaId; // ASPI host adapter number
        BYTE SRB_Flags; // Reserved, MUST = 0
        DWORD SRB_Hdr_Rsvd; // Reserved, MUST = 0
        BYTE SRB_Target; // Target's SCSI ID
        BYTE SRB_Lun; // Target's LUN number
        BYTE SRB_DeviceType; // Target's peripheral device type
        BYTE SRB_Rsvd1; // Reserved, MUST = 0
        } SRB_GDEVBlock, *PSRB_GDEVBlock;
//---------------------------------------------------------------------------
typedef struct _srb_exec_scsi_cmd_ {
	BYTE SRB_Cmd; // ASPI command code = SC_EXEC_SCSI_CMD
        BYTE SRB_Status; // ASPI command status byte
        BYTE SRB_HaId; // ASPI host adapter number
        BYTE SRB_Flags; // ASPI request flags
        DWORD SRB_Hdr_Rsvd; // Reserved, MUST = 0
        BYTE SRB_Target; // Target's SCSI ID
        BYTE SRB_Lun; // Target's LUN number
        WORD SRB_Rsvd1; // Reserved for Alignment
        DWORD SRB_BufLen; // Data Allocation Length
        LPBYTE SRB_BufPointer; // Data Buffer Pointer
        BYTE SRB_SenseLen; // Sense Allocation Length
        BYTE SRB_CDBLen; // CDB Length
        BYTE SRB_HaStat; // Host Adapter Status
        BYTE SRB_TargStat; // Target Status
        LPVOID SRB_PostProc; // Post routine
        BYTE SRB_Rsvd2[20]; // Reserved, MUST = 0
        BYTE CDBByte[16]; // SCSI CDB
        BYTE SenseArea[SENSE_LEN+2]; // Request Sense buffer
        } SRB_ExecSCSICmd, *PSRB_ExecSCSICmd;
//---------------------------------------------------------------------------
#define	CD_BLOCKSIZE_RAW0	2468		//	FORM 2 MODE 1
#define CD_BLOCKSIZE_RAW	2352		//	FORM 2 MODE 2
#define CD_BLOCKSIZE		2048		//	Data Block
#define CD_SECTORSIZE		CD_BLOCKSIZE
//---------------------------------------------------------------------------
#define CDF_ACCESS_COOKED	0
#define CDF_ACCESS_RAW0		1
#define CDF_ACCESS_RAW1		2
//---------------------------------------------------------------------------
#define	CD_CACHE_SIZE		( CD_BLOCKSIZE_RAW0 * 64 )	//	Should be about 150K
//---------------------------------------------------------------------------
#define ISODCL(from, to) (to - from + 1)	//	ISO Type Declaration
//---------------------------------------------------------------------------
typedef	struct _iso_primary_descriptor_def_ {	//	ISO Primary descriptor
	char type			[ISODCL (  1,   1)]; /* 711 */
	char id				[ISODCL (  2,   6)];
	char version			[ISODCL (  7,   7)]; /* 711 */
	char unused1			[ISODCL (  8,   8)];
	char system_id			[ISODCL (  9,  40)]; /* achars */
	char volume_id			[ISODCL ( 41,  72)]; /* dchars */
	char unused2			[ISODCL ( 73,  80)];
	char volume_space_size		[ISODCL ( 81,  88)]; /* 733 */
	char unused3			[ISODCL ( 89, 120)];
	char volume_set_size		[ISODCL (121, 124)]; /* 723 */
	char volume_sequence_number	[ISODCL (125, 128)]; /* 723 */
	char logical_block_size		[ISODCL (129, 132)]; /* 723 */
	char path_table_size		[ISODCL (133, 140)]; /* 733 */
	char type_l_path_table		[ISODCL (141, 144)]; /* 731 */
	char opt_type_l_path_table	[ISODCL (145, 148)]; /* 731 */
	char type_m_path_table		[ISODCL (149, 152)]; /* 732 */
	char opt_type_m_path_table	[ISODCL (153, 156)]; /* 732 */
	char root_directory_record	[ISODCL (157, 190)]; /* 9.1 */
	char volume_set_id		[ISODCL (191, 318)]; /* dchars */
	char publisher_id		[ISODCL (319, 446)]; /* achars */
	char preparer_id		[ISODCL (447, 574)]; /* achars */
	char application_id		[ISODCL (575, 702)]; /* achars */
	char copyright_file_id		[ISODCL (703, 739)]; /* 7.5 dchars */
	char abstract_file_id		[ISODCL (740, 776)]; /* 7.5 dchars */
	char bibliographic_file_id	[ISODCL (777, 813)]; /* 7.5 dchars */
	char creation_date		[ISODCL (814, 830)]; /* 8.4.26.1 */
	char modification_date		[ISODCL (831, 847)]; /* 8.4.26.1 */
	char expiration_date		[ISODCL (848, 864)]; /* 8.4.26.1 */
	char effective_date		[ISODCL (865, 881)]; /* 8.4.26.1 */
	char file_structure_version	[ISODCL (882, 882)]; /* 711 */
	char unused4			[ISODCL (883, 883)];
	char application_data		[ISODCL (884, 1395)];
	char unused5			[ISODCL (1396, 2048)];/* sector fill */
	} ISOTOC, toc, *lpISOTOC;
//---------------------------------------------------------------------------
#define btoi(b)		((b)/16*10 + (b)%16)		/* BCD to u_char */
#define itob(i)		((i)/10*16 + (i)%10)		/* u_char to BCD */
//---------------------------------------------------------------------------
// 	1x = 150kb or 65 sectors (2352b) per second
// 	PSXCLK = 1 sec in the ps
// 	so ( PSXCLK / 65 ) / BIAS = cdr read time ( linuzappz )
//---------------------------------------------------------------------------
#define cdReadTime ( ( PSXCLK / 65 ) / BIAS / 2 )
//---------------------------------------------------------------------------
#define CDR_INT( eCycle ) { \
	CPU->lpIrqQueue[ CPU->CurIrq ].interrupt |= 0x08; \
	CPU->lpIrqQueue[ CPU->CurIrq ].intCycle[2] = CPU->lpIrqQueue[ CPU->CurIrq ].cycle + eCycle; \
}
//---------------------------------------------------------------------------
#define StartReading(type) { \
   	cdr.Reading = type; \
   	cdr.Synk = CPU->lpIrqQueue[ CPU->CurIrq ].cycle + ((cdr.Mode & 0x80) ? (cdReadTime / 2) : cdReadTime); \
  	cdr.FirstSector = 1; \
  	cdr.Readed = 0xff; \
}
//---------------------------------------------------------------------------
#define StopReading() { \
	cdr.Synk = 0xffffffff; \
	cdr.Reading = 0; \
}
//---------------------------------------------------------------------------
#define SetResultSize(size) { \
    	cdr.ResultP = 0; \
	cdr.ResultC = size; \
	cdr.ResultReady = 1; \
}
//---------------------------------------------------------------------------
#define cdRefresh() { \
	if (cdr.ResultReady && cdr.ResultC) { \
		cdr.Ctrl|= 0x20; \
		lpHW->dwHWM8[ 0x1801 ] = cdr.Result[ cdr.ResultP ]; \
	} else cdr.Ctrl&=~0x20; \
}
//---------------------------------------------------------------------------
//	CDROM	CONTROL	REGISTER 0
//---------------------------------------------------------------------------
#define CDROM_CONTROL_REG1MODE		0x01	//	Bit 0	1/0 REG1 Command mode / data mode
#define CDROM_CONTROL_DATAXFER		0x02	//	Bit 1	Data transfer flag
#define CDROM_CONTROL_ACKNOWLEDGE	0x04	//	Bit 2	REG1 Acknowledge Flag
#define CDROM_CONTROL_DATAEND		0x08	//	Bit 3	Data End Flag
#define CDROM_CONTROL_DISK_ERROR	0x10	//	Bit 4	Disk Error Flag
#define CDROM_CONTROL_CMD_COMPLETE	0x20	//	Bit 5	Result Ready Flag
#define CDROM_CONTROL_DMA_READY		0x40	//	Bit 6	DMA Flag
#define CDROM_CONTROL_BUSY		0x80	//	Bit 7	Device Busy Flag
//---------------------------------------------------------------------------
//	CDROM	STATUS	REGISTER 3
//---------------------------------------------------------------------------
//#define CDROM_STATUS_
//---------------------------------------------------------------------------
#define	CDROM_ACCESS_TYPE_BLOCK	1
#define CDROM_ACCESS_TYPE_TIMED	2
//---------------------------------------------------------------------------
//!	CDROM Command values, Note this must be written to the CDROM command register
//!	@ virtual address 0x1F801080 in the PSX memory map and then the parameters
//!	if any are written in sequence to the data port @ 0x1F801084
//---------------------------------------------------------------------------
#define CdlSync         	0	//	Sync operation
#define CdlNop	        	1       //	Nop operation
#define CdlSetloc		2       //	Set Location, which location and where ?
#define CdlPlay	        	3       //	Play CD (audio)
#define CdlForward		4       //	Skip forware time++
#define CdlBackward		5       //	skip back time--
#define CdlReadN		6       //	Read N sectors
#define CdlStandby		7       //	standby
#define CdlStop	        	8       //	stop all operations
#define CdlPause        	9       //	pause operations
#define CdlInit 		10      //	Initialise
#define CdlMute	        	11      //	Mute cd-audio
#define CdlDemute		12      //	demute audio
#define CdlSetfilter		13      //	set audio filter
#define CdlSetmode		14      //	set mode of operation
#define CdlGetmode      	15      //	get mode of operation
#define CdlGetlocL		16      //	get location
#define CdlGetlocP		17      //	get location
#define Cdl18     		18      //	???
#define CdlGetTN		19      //	get TN	?
#define CdlGetTD		20      //	get TD	?
#define CdlSeekL		21      //	Seek L ?
#define CdlSeekP		22      //	Seek P ?
#define CdlTest 		25      //	test
#define CdlID   		26      //	ID disc
#define CdlReadS		27      //	read sectors
#define CdlReset		28      //	reset device
#define CdlReadToc      	30      //	read Table Of Contents
//---------------------------------------------------------------------------
#define READ_ACK		250     //	read Acknowledge
#define READ			251     //	read mode
#define REPPLAY_ACK		252     //	replay acknowledge
#define REPPLAY			253     //	replay
#define ASYNC			254     //	async tranfers
//---------------------------------------------------------------------------
//	don't set 255, it's reserved
//---------------------------------------------------------------------------
typedef struct _cdrom_device_ {
	unsigned char OCUP;
	unsigned char Reg1Mode;
	unsigned char DataTransfer;
	unsigned char Ctrl;
	unsigned char Stat;

	unsigned char StatP;

	unsigned char Transfer[2352];
	unsigned char *pTransfer;

	unsigned char Prev[4];
	unsigned char Param[8];
	unsigned char Result[8];

	unsigned char ParamC;
	unsigned char ParamP;
	unsigned char ResultC;
	unsigned char ResultP;
	unsigned char ResultReady;
	unsigned char Cmd;
	unsigned char Readed;
	unsigned long Reading;

	unsigned char ResultTN[6];
	unsigned char ResultTD[4];
	unsigned char SetSector[4];
	unsigned char SetSectorSeek[4];
	int Play;
	int CurTrack;
	int Mode, File, Channel, Track, Muted;
	int Reset;
	int RErr;
	int FirstSector;

//	xa_decot_t_def_ *Xa;
	void	*Xa;

	unsigned long Synk;

        lpISOTOC	lpTOC;		//	If Cd is present then this holds a TOC ofr the disc, else NULL
        LPPSX_DIRENT	lpRootDir;	//	Root directory of files on disc
} cdrStruct;
//---------------------------------------------------------------------------
typedef struct {
	unsigned char Irq[4];
	unsigned long eCycle[4];
	int cdIrqn;
	int count;
} cdIrqQ;
//---------------------------------------------------------------------------
typedef struct {
	unsigned char Cmd;
	unsigned long eCycle;
} cdCmdQ;
//---------------------------------------------------------------------------
//	WIN32 Device Name, used in CreateFile
//---------------------------------------------------------------------------
#define DeviceName	"\\\\.\\G:"	//!	Device name for first cd-rom device
//---------------------------------------------------------------------------
#define NUM_HANDLES		16		//	Not all are used, allows for expansion
#define PSX_FILENAME_LENGTH	20		//	Maximum filename length on PSX disc
//---------------------------------------------------------------------------
//	This is probably wrong, but allows for accessing the cd rom is a non LBA
// format, ie you specify which track and 'when' to read from, eg read min 2 onwards
// for 2 seconds etc
//---------------------------------------------------------------------------
typedef	struct	_cdtime_def_	{	//	For accessing the cd-rom
	BYTE	Min;	//	in the format min( 0 - 59 )
        BYTE	Sec;	//		sec   ( 0 - 59 )
        BYTE	Frame;	//		frame ( sec / 75 )
        BYTE	Trk;	//		track ( 0 - 99 )
} CDTime, *lpCDTime;
//---------------------------------------------------------------------------
typedef	DWORD	CDLBA;
typedef	DWORD	*LPCDLBA;
//---------------------------------------------------------------------------
class	PSX_CDROM : public TObject
	{
        	friend	class	PSX_MEM;	//	Memory interface
        	friend	class	HW;		//	Hardware interface
                friend	class	CPSX_DMA;	//	DMA Interface
                friend	class	R3000A;		//	CPU Interface
        	protected:
                	void	__fastcall	CDRefresh	( void );
                	BYTE	__fastcall	CDRead0		( void );
                        BYTE	__fastcall	CDRead1		( void );
                        BYTE	__fastcall	CDRead2		( void );
                        BYTE	__fastcall	CDRead3 	( void );
                        void	__fastcall	CDWrite0	( BYTE Value );
                        void	__fastcall	CDWrite1	( BYTE Value );
                        void	__fastcall	CDWrite2	( BYTE Value );
                        void	__fastcall	CDWrite3	( BYTE Value );
                	void	__fastcall	CDInterrupt	( void );	//	Handle CD-ROM Interrupts NOT PC
                private:
                	void	__fastcall	AddCmdQueue	( unsigned char cmd, unsigned long ecycle );
                        void	__fastcall	DoCDReading	( void );
                        void 	__fastcall	hwDma3		( unsigned long madr, unsigned long bcr, unsigned long chcr );

                        DWORD 		(*lpfnGetASPI32SupportInfo)( void );
			DWORD 		(*lpfnSendASPI32Command)( LPVOID lpSRB );
			BOOL 		(*lpfnGetASPI32Buffer)( LPVOID PASPI32BUFF );
			BOOL 		(*lpfnFreeASPI32Buffer)( LPVOID PASPI32BUFF );
			BOOL 		(*lpfnTranslateASPI32Address)( PDWORD, PDWORD );


                	HANDLE		hDevice;	//	Win32 device handle
                        HINSTANCE 	hASPI32;
                        DWORD		dwAccessMode;	//	accessing mode
                        DWORD		dwShareMode;	//	sharing mode
                        DWORD		dwCreateMode;	//	creation mode
                        DWORD		dwFlags;	//	flags and attributes

                        BYTE		bNumHA;		//	Number of hardware adapters installed in PC
                        BYTE		bLUNID;		//	ID Number of Logical CDROM drive

                        cdrStruct	cdr;            //	cd-rom structure
			cdIrqQ 		IrqQ[ 8 ];	//	irq queue
			int 		IrqQueue;	//	current max num irqs in queue
			int 		CurIrqQ;	//	current irq
			cdCmdQ 		CmdQ[ 8 ];	//	repeat for command queue
			int 		CmdQueue;
			int 		CurCmdQ;

			struct 	{
                        	DWORD	Type;
                                DWORD	Status;
                                BYTE	Time[ 3 ];
                        } CDState;		//	status value


                        //	Event Management for CDROM non-blocking async operations
                        EvCB	*lpCDEvent;

                        //	I/O OVERLAPPED Management
                        OVERLAPPED	*lpOverlapping;

                        int	__fastcall	dev_cd_open		( DWORD dwMode );
                        int	__fastcall	dev_cd_read		( LPVOID lpBuffer, DWORD Count, int fHandle );
                        void	__fastcall	dev_cd_close		( void );
                        int	__fastcall	dev_cd_firstfile	( LPVOID lpSearchRec );
                        int	__fastcall	dev_cd_nextfile		( LPVOID lpSearchRec );
                        int	__fastcall	dev_cd_chdir		( LPSTR lpszDirName );
                public:
                			__fastcall	PSX_CDROM	( void );
                        		__fastcall	~PSX_CDROM	( void );

                                int	__fastcall	Initialise	( void );
                                void	__fastcall	Terminate       ( void );

                                void	__fastcall	Reset		( void );

                                friend	void 		AddIrqQueue	( int irqn, ... );
        };
//---------------------------------------------------------------------------
__declspec( dllimport )	PSX_CDROM	*CDROM;			//	Interface to emulated CDROM drive
//---------------------------------------------------------------------------
#endif /* __CDROM_H__ */
//---------------------------------------------------------------------------
