//---------------------------------------------------------------------------
//
//	$FileName	::	Counters.cpp
//	$Purpose	::	PlayStation Root Counter Emulation
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "Counters.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
__declspec( dllexport )	PSX_Counter	*psx_counters = NULL;
__declspec( dllexport )	unsigned long	NextCounter;
//---------------------------------------------------------------------------
__fastcall	PSX_Counter :: PSX_Counter( void ) : TObject()
{
	fprintf( stderr, "PlayStation Counters Emulation Layer Construction\n" );
	lpCounters = new Counter[ MAX_COUNTERS ];

        if ( lpCounters == NULL )
        	throw "Counter construction failure.";

        ZeroMemory( lpCounters, sizeof( Counter ) * MAX_COUNTERS );
}
//---------------------------------------------------------------------------
__fastcall	PSX_Counter :: ~PSX_Counter( void )
{
	if ( lpCounters != NULL )
        	delete []lpCounters;

        delete this;
}
//---------------------------------------------------------------------------
int	__fastcall	PSX_Counter :: Initialise( void )
{
	NextCounter = 0;

	ZeroMemory( lpCounters, sizeof( Counter ) );

	lpCounters[ 0 ].rate = 1; lpCounters[ 0 ].interrupt = 16;
	lpCounters[ 1 ].rate = 1; lpCounters[ 1 ].interrupt = 32;
	lpCounters[ 2 ].rate = 1; lpCounters[ 2 ].interrupt = 64;

	lpCounters[ 3 ].interrupt = 1;
	lpCounters[ 3 ].mode = 0x58; // The VSync counter mode
	lpCounters[ 3 ].target = 1;

	//if ( Config.PsxType ) // ntsc - 0 | pal - 1
	//    lpCounters[ 3 ].rate = PSXCLK / 50;
	//else
        //	lpCounters[ 3 ].rate = PSXCLK / 60;//550000;// / 2;

	//UpDate( 0 );
        //UpDate( 1 );
        //UpDate( 2 );
        //UpDate( 3 );

	//Set();

	return( true );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Counter :: Terminate( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Counter :: ResetAll( void )
{
	for ( int i=0; i<MAX_COUNTERS; i++ )
        	Reset( i );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Counter :: Reset( int Index )
{
	lpCounters[ Index ].count = 0;
	lpCounters[ Index ].sCycle = CPU->lpIrqQueue[ CPU->CurIrq ].cycle;
        
	if ( lpCounters[ Index ].mode & 0x08 ) { // Count to target
		lpCounters[ Index ].eCycle = lpCounters[ Index ].sCycle + ( lpCounters[ Index ].target - lpCounters[ Index ].count) * lpCounters[ Index ].rate / BIAS;
	} else { // Count to 0xffff
		lpCounters[ Index ].eCycle = lpCounters[ Index ].sCycle + ( 0xffff - lpCounters[ Index ].count) * lpCounters[ Index ].rate / BIAS;
	}

	if ( lpCounters[ Index ].mode & 0x50 ) {
        	DWORD d = lpHW->dwHWM32[ 0x1070 ];
                d |= lpCounters[ Index ].interrupt;
        	lpHW->dwHWM32[ 0x1070 ] = d;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Counter :: UpDate( int Index )
{
	lpCounters[ Index ].sCycle = CPU->lpIrqQueue[ CPU->CurIrq ].cycle;

	if ( ( !( lpCounters[ Index ].mode & 1 ) ) || ( Index != 2 ) ) {
		if ( lpCounters[ Index ].mode & 0x08 ) { // Count to target
			lpCounters[ Index ].eCycle = lpCounters[ Index ].sCycle + ( lpCounters[ Index ].target - lpCounters[ Index ].count ) * lpCounters[ Index ].rate / BIAS;
		} else { // Count to 0xffff
			lpCounters[ Index ].eCycle = lpCounters[ Index ].sCycle + ( 0xffff - lpCounters[ Index ].count ) * lpCounters[ Index ].rate / BIAS;
		}
	} else lpCounters[ Index ].eCycle = 0xffffffff;

	if ( CPU->lpIrqQueue[ CPU->CurIrq ].cycle >= lpCounters[ 3 ].eCycle ) {
		lpCounters[ 3 ].count = 0;
		Reset( 3 );

//		GPU_updateLace(); // updateGPU

//		SysUpdate();
	}

        if ( CPU->lpIrqQueue[ CPU->CurIrq ].cycle >= lpCounters[ 0 ].eCycle ) {
		Reset( 0 );
	}

	if ( CPU->lpIrqQueue[ CPU->CurIrq ].cycle >= lpCounters[ 1 ].eCycle ) {
		Reset( 1 );
	}

	if ( CPU->lpIrqQueue[ CPU->CurIrq ].cycle >= lpCounters[ 2 ].eCycle ) {
		Reset( 2 );
	}

	Set();
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Counter :: Set( void )
{	unsigned long 	minTarget = 0xffffffff;
	int 		i;

	for (i = 0; i < 4; i++) {
		if ( lpCounters[ i ].eCycle > CPU->lpIrqQueue[ CPU->CurIrq ].cycle && lpCounters[ i ].eCycle < minTarget ) {
			minTarget = lpCounters[ i ].eCycle;
		}
		// CPU->lpIrqQueue[ CPU->CurIrq ].cycle is about reaching 0
		if ( ( lpCounters[ i ].eCycle + CPU->lpIrqQueue[ CPU->CurIrq ].cycle ) < CPU->lpIrqQueue[ CPU->CurIrq ].cycle && lpCounters[ i ].eCycle < minTarget )
			minTarget = lpCounters[ i ].eCycle;
	}
	NextCounter = minTarget;
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_Counter :: UpdateVSyncRate( void )
{
	//if ( Config.PsxType ) // ntsc - 0 | pal - 1
	//    	lpCounters[ 3 ].rate = PSXCLK / 50;
	//else
        lpCounters[ 3 ].rate = PSXCLK / 60;//550000;// / 2;
	UpDate( 3 );
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_Counter :: WriteCount( int Index, DWORD Value )
{
//	printf ("writeCcount[%d] = %x\n", index, value);
	lpCounters[ Index ].count = Value;
	UpDate( Index );
	Set();
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_Counter :: WriteMode( int Index, DWORD Value )
{
//	printf ("writeCmode[%ld] = %lx\n", index, value);
	lpCounters[ Index ].mode = Value;
	if( Index == 0 ) {
		switch ( Value & 0x300 ) {
			case 0x100:
				lpCounters[ Index ].rate = lpCounters[ 3 ].rate / 240 / 320; // ok???
				break;
			default:
				lpCounters[ Index ].rate = 1;
		}
	}
	else if( Index == 1 ) {
		switch ( Value & 0x300 ) {
			case 0x100:
				lpCounters[ Index ].rate = lpCounters[ 3 ].rate / 240; // ok??
				break;
			default:
				lpCounters[ Index ].rate = 1;
		}
	}
	else if( Index == 2 ) {
		switch ( Value & 0x300 ) {
			case 0x200:
				lpCounters[ Index ].rate = 8; // 1/8 speed
				break;
			default:
				lpCounters[ Index ].rate = 1; // normal speed
		}
	}

	// Need to set a rate and target
	UpDate( Index );
	Set();
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_Counter :: WriteTarget( int Index, DWORD Value )
{
//	printf ("writeCtarget[%ld] = %lx\n", index, value);
	lpCounters[ Index ].target = Value;
	UpDate( Index );
	Set();
}
//---------------------------------------------------------------------------
DWORD	__fastcall	PSX_Counter :: ReadCount( int Index )
{
	if ( ( !( lpCounters[ Index ].mode & 1 ) ) || ( Index != 2 ) ) {
		if ( lpCounters[ Index ].rate )
			return ( lpCounters[ Index ].count + BIAS * BIAS * ( CPU->lpIrqQueue[ CPU->CurIrq ].cycle - lpCounters[ Index ].sCycle ) / lpCounters[ Index ].rate ) & 0xffff;
		else
                	return ( lpCounters[ Index ].count + BIAS * BIAS * ( CPU->lpIrqQueue[ CPU->CurIrq ].cycle - lpCounters[ Index ].sCycle ) ) & 0xffff;
	} else
        	return( 0 );
}
//---------------------------------------------------------------------------
