//---------------------------------------------------------------------------
//
//	$FileName	::	DMA.cpp
//	$Purpose	::	DMA specifications for the PlayStation
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "DMA.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
__declspec( dllexport )	CPSX_DMA	*PSX_DMA = NULL;
static FILE *fDMALog = NULL;
//---------------------------------------------------------------------------
__fastcall	CPSX_DMA :: CPSX_DMA( void ) : TObject()
{
	fDMALog = fopen( "DMA.LOG", "wt" );
        if ( fDMALog == NULL ) {
        	fprintf( stderr, "Unable to create DMA logfile.\n" );
                fDMALog = stderr;
        }
	fprintf( fDMALog, "PlayStation DMA Emulation Layer Construction\n" );

        lpDMAChannels = new DMAChannel[ NUM_DMA_CHANNELS ];
        ZeroMemory( lpDMAChannels, sizeof( DMAChannel ) * NUM_DMA_CHANNELS );
}
//---------------------------------------------------------------------------
__fastcall	CPSX_DMA :: ~CPSX_DMA( void )
{
	if ( lpDMAChannels != NULL )
        	delete []lpDMAChannels;

	delete this;
}
//---------------------------------------------------------------------------
DWORD	__fastcall	CPSX_DMA :: GAddress( int Index )
{
	if ( ( Index >= 0 ) && ( Index < NUM_DMA_CHANNELS ) )
        	return( NULL );

        if ( lpDMAChannels == NULL )
        	return( NULL );

        return( lpDMAChannels[ Index ].Addr );
}
//---------------------------------------------------------------------------
DWORD	__fastcall	CPSX_DMA :: GCount( int Index )
{
	if ( ( Index < 0 ) || ( Index >= NUM_DMA_CHANNELS ) )
        	return( NULL );

        if ( lpDMAChannels == NULL )
        	return( NULL );

	return( lpDMAChannels[ Index ].Count );
}
//---------------------------------------------------------------------------
DWORD	__fastcall	CPSX_DMA :: GControl( int Index )
{
	if ( ( Index < 0 ) || ( Index >= NUM_DMA_CHANNELS ) )
        	return( NULL );

        if ( lpDMAChannels == NULL )
        	return( NULL );

	return( lpDMAChannels[ Index ].Control );
}
//---------------------------------------------------------------------------
void	__fastcall	CPSX_DMA :: SAddress( int Index, DWORD Value )
{
        if ( lpDMAChannels == NULL )
        	return;

        if ( lpDMAChannels[ Index ].Addr != Value )
        	lpDMAChannels[ Index ].Addr = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	CPSX_DMA :: SCount( int Index, DWORD Value )
{
        if ( lpDMAChannels == NULL )
        	return;

        if ( lpDMAChannels[ Index ].Count != Value )
        	lpDMAChannels[ Index ].Count = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	CPSX_DMA :: SControl( int Index, DWORD Value )
{
        if ( lpDMAChannels == NULL )
        	return;

        if ( lpDMAChannels[ Index ].Control != Value )
        	lpDMAChannels[ Index ].Control = Value;

        if ( lpDMAChannels[ Index ].Control & 0x40000000 )
        	DoDMA( Index );
}
//---------------------------------------------------------------------------
int	__fastcall	CPSX_DMA :: Initialise( void )
{
	if ( lpDMAChannels == NULL )
        	return( -1 );

        ZeroMemory( lpDMAChannels, sizeof( DMAChannel ) * NUM_DMA_CHANNELS );
	return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	CPSX_DMA :: Terminate( void )
{
	if ( lpDMAChannels != NULL )
        	delete []lpDMAChannels;

        lpDMAChannels = NULL;
}
//---------------------------------------------------------------------------
void	__fastcall	CPSX_DMA :: Reset( int Channel )
{
	if ( Channel == -1 ) {	//	Reset all
        	for ( int i=0; i<NUM_DMA_CHANNELS; i++ ) {
                	lpDMAChannels[ i ].Control &= ~0x01000000;	//	Turn off this channel
                }
                PsxMem->RAM32[ DMA_IRQ_CTRL ] = 0;
        } else	{		//	reset specific
        	lpDMAChannels[ Channel ].Control &= ~0x01000000;
                PsxMem->RAM32[ DMA_IRQ_CTRL ] &= ~Channel;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	CPSX_DMA :: DoDMA( int Index )
{
	if ( ( Index < 0 ) || ( Index >= NUM_DMA_CHANNELS ) )
        	return;

        if ( !( PsxMem->RAM32[ DMA_IRQ_CTRL ] & Index ) )	{
        	return;
        }

        if ( !( PsxMem->RAM32[ DMA_IRQ_CTRL ] & PsxMem->RAM32[ DMA_IRQ_MASK ] ) )
        	return;

        switch ( Index ) {
        	case ( 0 ) : {	//	MDEC in
                	PSX_MDEC->hwDMA0();
                        return;
                }
                case ( 1 ) : {	//	MDEC out
                	PSX_MDEC->hwDMA1();
                        return;
                }
                case ( 2 ) : {	//	GPU
//void hwDma2(unsigned long madr, unsigned long bcr, unsigned long chcr) { // GPU
//	switch(chcr) {
//		case 0x01000200: // vram2mem
//			GPU_readDataMem((unsigned long *)&RAM[madr & 0x1fffff], (bcr >> 16) * (bcr & 0xffff));
			//Cpu->Clear(madr, ((bcr >> 16) * (bcr & 0xffff)) / 4);
//			break;

//		case 0x01000201: // mem2vram
//			GPU_writeDataMem((unsigned long *)&RAM[madr & 0x1fffff], (bcr >> 16) * (bcr & 0xffff));
//			break;

//		case 0x01000401: // dma chain, proceed to execute ordering table using
                // dma to transfer to gpu
//			GPU_dmaChain((unsigned long *)RAM, madr & 0x1fffff);
//			break;
//	}
//}
//---------------------------------------------------------------------------
                	//lpGTE->hwDma2();
                	return;
                }
                case ( 3 ) : {	//	CDROM
                	//CDROM->hwDma3();
                	return;
                }
                case ( 4 ) : {	//	SPU
//	dma 4 moved to PSX_SPU
//void hwDma4(unsigned long madr, unsigned long bcr, unsigned long chcr) { // SPU
//	switch (chcr) {
//		case 0x01000201: //cpu to spu transfer
//			SPU_writeDMAMem((unsigned short *)&RAM[madr & 0x1fffff], (bcr >> 16) * (bcr & 0xffff));
//			break;
//		case 0x01000200: //spu to cpu transfer
//    		SPU_readDMAMem ((unsigned short *)&RAM[madr & 0x1fffff], (bcr >> 16) * (bcr & 0xffff));
//			break;
//	}
//}
                	return;
                }
        	case ( 5 ) : {	//	???
                	//Unknown->hwDma5();
                        return;
                }
                case ( 6 ) : {	//	GTE Ordering Table
//void	hwDma6( unsigned long madr, unsigned long bcr, unsigned long chcr )
//{
//	unsigned long *mem = (unsigned long *)&RAM[madr & 0x1fffff];
//	if (chcr == 0x11000002) {
//		while (bcr--) {
//			*mem-- = (madr - 4) & 0xffffff;
//			madr -= 4;
//		}
//		mem++; *mem = 0xffffff;
//	} else {
//		// Unknown option
//	}
//}
                	return;
                }
        }
//        CPU->SignalException( CAUSE_INT, NULL );
        Reset( Index );
}
//---------------------------------------------------------------------------
