//---------------------------------------------------------------------------
//
//	$FileName	::	DecodeXA.h
//	$Purpose	::	to decode xa audio
//
//---------------------------------------------------------------------------
#ifndef DecodeXAH
#define DecodeXAH
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"	//	This MUST be included in ALL units that are part of the Emulator
#endif
//---------------------------------------------------------------------------
typedef	struct	_xa_raw_sector_def_	{	//	CD-XA RAW sector 2352 bytes
	BYTE	sync[ 12 ];
        // 	the sync pattern is not decoded on all CD-ROM drives
	struct	{
		BYTE	minutes;	// timecode relative to start of disk
		BYTE	seconds;	// timecode relative to start of disk
		BYTE	sectors;	// timecode relative to start of disk
		BYTE	mode;		// Form 2 for ADPCM audio sectors
	} header;
	struct  {
		BYTE	file_number;	// used to identify sectors belonging to the same file
		BYTE	channel;	// 0-15 for ADPCM audio
		struct	{
			unsigned end_of_record : 1;	// identifies end of audio frame
                        unsigned video : 1;		// dependant on sector type, 0 for ADPCM sector
                        unsigned audio : 1;		// dependant on sector type, 1 for ADPCM sector
                        unsigned data : 1;		// dependant on sector type, 0 for ADPCM sector
			unsigned trigger : 1;		// used for application
                        unsigned form : 1;		// 1 for ADPCM sector (form 2, 0 for form 1)?
                        unsigned real_time : 1; 	// 1 for real time mode, see above description
                        unsigned eof_marker : 1;	// 0 for all sectors except last sector of file
		} submode;
		struct	{
                        unsigned mono_stereo : 2;	// 00=mono 01=stereo, other values reserved
                        unsigned sample_rate : 2;	// 00=37.8kHz (A,B format) 01=18.9kHz
			unsigned bits_per_sample : 2;	// 00=4bits (B,C format) 01=8bits
                        unsigned emphasis : 1;
                        unsigned reserved : 1;		// =0 ?
		} coding_info;
		BYTE	copy_of_file_number;	// =file_number
		BYTE	copy_of_channel;	// =channel
		BYTE	copy_of_submode;	// =submode
		BYTE	copy_of_coding_info;	// =coding_info
	} subheader;
	struct	{
		BYTE	sound_parameters[ 16 ];		// see below
		BYTE	audio_sample_bytes[ 112 ];	// see below
	} soundgroups[ 18 ];
	BYTE	unused_for_adpcm[ 20 ];
	BYTE	edc[4];				// error correction code or 0
} adpcm_raw_sector;				// 2352 bytes total
//---------------------------------------------------------------------------
typedef struct {
	long	y0, y1;
} ADPCM_Decode_t;
//---------------------------------------------------------------------------
typedef struct _xa_decode_t_def_ {
	int		freq;
	int		nbits;
	int		stereo;
	int		nsamples;
	ADPCM_Decode_t	left, right;
	short		pcm[ 16384 ];
} xa_decode_t, *pxa_decode_t, XADecode, *LPXADecode;
//---------------------------------------------------------------------------
//__declspec( dllimport ) long 	__fastcall	XADecodeSector( xa_decode_t *xdp, unsigned char *sectorp, int is_first_sector );
//---------------------------------------------------------------------------
class	PSX_XADecode : public TObject
	{
        	friend	class	PSX_CDROM;
        	protected:
                	DWORD	__fastcall	DecodeSector	( LPVOID lpBuffer, DWORD& dwSector, bool fIsFirstSector );
                private:
			adpcm_raw_sector	*lpSectorCache;
                public:
                		__fastcall	PSX_XADecode	( void );
                                __fastcall	~PSX_XADecode	( void );

                	int	__fastcall	Initialise	( void );
                        void	__fastcall	Terminate	( void );
                        void	__fastcall	Reset		( void );
        };
//---------------------------------------------------------------------------
__declspec( dllimport )	PSX_XADecode	*DecodeXA;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
