//---------------------------------------------------------------------------
//
//	$FileName	::	Emulator.cpp
//	$Purpose	::	Emulator DLL for KonDev
//	$Extra		::	This emulator is based on the PCSX emulator and
//				the MIPS 32 ISA documentation
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_C						//	We must tell the header file
#define EMULATOR_DLL_C						//	that this is the dll
#include "EmulatorDLL.h"        				//	General emulator stuff
#undef EMULATOR_DLL_C
#endif
//---------------------------------------------------------------------------
#ifndef DLL_VERSION
#define DLL_VERSION	"0.01"
#endif
//---------------------------------------------------------------------------
//	Header, used for identification of configuration files. Layout as follows:
//	16 Byte Header	==	DLLHeader ( "EDLL PCSX V0.01\0" )
//	EmulatorConfig struct
//		....
//		depends on the structure contents, but if the flag is set the the
//	contents of the memory area corresponding to the flag is stored / retrieved
//	using the configuration file. Otherwise this section is not included in the
//	file.
//		....
//	Note that this file is just under 9 MB long, very big is the memory map for the playstation
//---------------------------------------------------------------------------
const char DLLHeader[ 16 ] = { "EDLL PCSX v0.01\0" };	//	Header for DLL
//---------------------------------------------------------------------------
#define	CLASSNAME	"PlayStation"	         	        //	Classname for the emulator as a whole
#define REVISION        "Revision: 0.3"                         //      Software Revision Number
#define VERSION         "Id: emulator.cpp 0.30 2002/10/10"      //      Software Id and version number
#define AUTHOR		"Chris Brooklyn"			//	Author
//---------------------------------------------------------------------------
USEUNIT("ELF.cpp");
USEUNIT("Memory.cpp");
USEUNIT("HW.cpp");
USEUNIT("Counters.cpp");
USEUNIT("DMA.cpp");
USEUNIT("MDec.cpp");
USEUNIT("SIO.cpp");
USEUNIT("SPU.cpp");
USEUNIT("DecodeXA.cpp");
USEUNIT("CDROM.cpp");
USEUNIT("GTE.cpp");
USEUNIT("R3000A.cpp");
USEUNIT("BIOS.cpp");
USEUNIT("Plugins.cpp");
USERES("Emulator.res");
USEUNIT("Kernel.cpp");
USEUNIT("OSCore.cpp");
USERES("Emulator.res");
USELIB("..\Debugger.lib");
USEUNIT("GLTexture.cpp");
//---------------------------------------------------------------------------
LRESULT	CALLBACK	DLLWndProc	( HWND, UINT, WPARAM, LPARAM );
	WNDCLASSEX			WndClass;			//	Class to register the emulator with windows
	static	HINSTANCE		hInstance;			//	Instance of this DLL
__declspec( dllexport )	static	LPSTR	lpszExecFileName;		//	filename of executable from psxide
__declspec( dllexport )	static	bool	fExecLoaded;			//	flag has emulator got the exec file is ram ?
__declspec( dllexport ) EmulatorConfig 	Config;				//	Configuration details
__declspec( dllexport )	static	bool	fEmulatorPresent = false;	//	Emulator Initialised Flag
__declspec( dllexport ) static	bool	fEmulatorActive = false;	//	Emulator Active Flag
__declspec( dllexport ) char 		CdromId[256];
int Log = 0;
long jumpMode;
char ostr[256];
//---------------------------------------------------------------------------
int	__fastcall	CheckCdrom( void );
int 	__fastcall	Load( char *ExePath );
//---------------------------------------------------------------------------
int 	__fastcall	SaveState( char *FileName );
int 	__fastcall	LoadState( char *FileName );
//---------------------------------------------------------------------------
//	Names of registers (GENERAL)
//---------------------------------------------------------------------------
char *disRNameGPR[] = {
	"r0", "at", "v0", "v1", "a0", "a1","a2", "a3",
	"t0", "t1", "t2", "t3", "t4", "t5","t6", "t7",
	"s0", "s1", "s2", "s3", "s4", "s5","s6", "s7",
	"t8", "t9", "k0", "k1", "gp", "sp","fp", "ra"};
//---------------------------------------------------------------------------
//	...	(CONTROL COP0 ONLY)
//---------------------------------------------------------------------------
char *disRNameCOP0[] = {
	"Index"     , "Random"    , "EntryLo0", "EntryLo1", "Context" , "PageMask"  , "Wired"     , "*Check me*",
	"BadVAddr"  , "Count"     , "EntryHi" , "Compare" , "Status"  , "Cause"     , "ExceptPC"  , "PRevID"    ,
	"Config"    , "LLAddr"    , "WatchLo" , "WatchHi" , "XContext", "*RES*"     , "*RES*"     , "*RES*"     ,
	"*RES*"     , "*RES* "    , "PErr"    , "CacheErr", "TagLo"   , "TagHi"     , "ErrorEPC"  , "*RES*"     };
//---------------------------------------------------------------------------
//!	Windows Message Handler
//---------------------------------------------------------------------------
LRESULT	CALLBACK	DLLWndProc( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam );
//---------------------------------------------------------------------------
//!	Initialise the emulator
//---------------------------------------------------------------------------
void	__fastcall	EMU_Initialise( void );
//---------------------------------------------------------------------------
//!	Terminate the emulator
//---------------------------------------------------------------------------
void	__fastcall	EMU_Terminate( void );
//---------------------------------------------------------------------------
//!	Configure emulator
//---------------------------------------------------------------------------
void	__fastcall	EMU_Configure( void );
//---------------------------------------------------------------------------
void	__fastcall	EMU_Create( void );
//---------------------------------------------------------------------------
//!	DLL Entry Point.
//---------------------------------------------------------------------------
int 	WINAPI 	DllEntryPoint( HINSTANCE hinst, unsigned long reason, void *pBuffer )
{
	hInstance = hinst;

	switch ( reason ) {
        	case ( DLL_PROCESS_ATTACH ) :
                case ( DLL_THREAD_ATTACH ) : {
                	if ( !fEmulatorPresent )
                        	EMU_Create();
                	EMU_Initialise();
                	return( true );
                }

                case ( DLL_PROCESS_DETACH ) :
                case ( DLL_THREAD_DETACH ) : {
                	EMU_Terminate();
                        return( true );
                }
        }
	return( false );
}
//---------------------------------------------------------------------------
//	Emulator Creation routine, creates all the classes in a non-operational
// mode. This must be called first.
//---------------------------------------------------------------------------
void	__fastcall	EMU_Create( void )
{
	if ( ( !fEmulatorPresent ) && ( !fEmulatorActive ) ) {
        	//	Only create if not all ready create or have been initialised
        	memset( &WndClass, 0, sizeof( WNDCLASSEX ) );

	        WndClass.cbSize = sizeof( WNDCLASSEX );
	        WndClass.hInstance = hInstance;
	        WndClass.lpszClassName = "EMULATOR";
	        WndClass.lpfnWndProc = (WNDPROC) DLLWndProc;

                //!	Register the emulator as a whole with Windows
	        if ( !RegisterClassEx( &WndClass ) )	{	//	Register class with windows
	        	MessageBox( hInstance, "Unable to register class with windows.", "EMULATOR", MB_OK );
                        fEmulatorPresent = false;
	                return;
	        }

	        //!	Class registered, start to create and then initialise the
                //! emulator the sequence in which the system is initialised may
                //! need to be changed to allow the emulator to initialise correctly.

                //!	Firstly allocate and initialise the basic memory manager
                //! which in turn will initiate a cascade for creating the hardware
                //! interface and various other sub-systems. See the constructor
                //! for the class 'PSX_MEM' for further information.
                if ( PsxMem == NULL )
                	PsxMem = new PSX_MEM;

                // ... prior to creating the CP2 coprocessor

                //	Initialise CP 2
                if ( lpGTE == NULL )
                	lpGTE = new GTE( true );

                // ... and then the cpu, but do not start execution of any of the
                // threads yet

                if ( CPU == NULL )
                	CPU = new R3000A( true );

                //	Create the 'fake' BIOS
	        PsxBIOS = new BIOS;

                // ....	kernel
                lpKernel = new PSX_Kernel;
                // .... and basic os support
                lpOSCore = new PSX_OSCore;

                fEmulatorPresent = true;	//	Emulator has been created, but not active
                fEmulatorActive = false;
                // ie not initialised or running

                //	Now safe for initialisation, but not safe for execution
	}
}
//---------------------------------------------------------------------------
//	Emulator Initialisation routine. To be called AFTER the EMU_Create
//---------------------------------------------------------------------------
void	__fastcall	EMU_Initialise( void )
{

	if ( !fEmulatorPresent )	//	Have we been created?
        	EMU_Create();

	if ( ( fEmulatorPresent ) && ( !fEmulatorActive ) ) {
                //	If we are here, with no serious errors, then the emulator
                // is created but not initialised.

                //	Now we need to repeat the sequence for initialisation of the emulator
                PsxMem->Initialise();
                lpGTE->Initialise();
                //CPU->Reset();	//... reset the cpu first, auto calls the initialise method anyhow
                CPU->Initialise();
                PsxBIOS->Initialise();
                fEmulatorActive = true;
        }

        // Emulator can execute code now, from location of RAM[ PC ]

        //	RAM translates as PsxMem->RAM32[ <32-bit index> ]
        //	PC translates as CP0.PC ie the current 'program counter' for the CPU
}
//---------------------------------------------------------------------------
//	Deregister emulator with windows
//---------------------------------------------------------------------------
void	__fastcall	EMU_Terminate( void )
{
	CPU->DumpRegs();
	if ( fEmulatorPresent ) {
		if ( hInstance )
	        	UnregisterClass( "EMULATOR", hInstance );	//	De-Register class with windows
        	fEmulatorPresent = false;
        }
}
//---------------------------------------------------------------------------
void	__declspec( dllexport ) __fastcall	EmulatorConfigure( void )
{
	EMU_Configure();
}
//---------------------------------------------------------------------------
__declspec( dllexport )	void	__fastcall	EmulatorTransfer( LPSTR lpszFileName )
{	AnsiString	Mess;

	if ( !lpszFileName )
        	Mess = "Bad parameter for transfer to emulator.";

        else if ( !FileExists( AnsiString( lpszFileName ) ) )
        	Mess = "File " + AnsiString( lpszFileName ) + " not found.";

        else
        	Mess = "Transferring " + AnsiString( lpszFileName ) + " to emulator.";

        if ( Mess.Length() > 0 )
        	MessageBox( NULL, Mess.c_str(), "EMULATOR", MB_OK );

        if ( Load( lpszFileName ) == -1 )
        	printf( "Failed to load file %s for execution.\n", lpszFileName );
        else
        	printf( "File %s loaded.\n", lpszFileName );
}
//---------------------------------------------------------------------------
void	__declspec( dllexport )	__fastcall	EmulatorExecute( void )
{
	fprintf( stderr, "\nExecuting from 0x%08x.\n", CPU->PC );
	if ( !fEmulatorPresent )	//	Emulator has not been created
        	EMU_Create();

        if ( !fEmulatorActive )		//	Emulator has not been initialised yet
        	EMU_Initialise();

        CPU->Resume();		//	Resume execution of CP0/CP1
        lpGTE->Resume();        //	Resume execution of CP2, this may need to be changed
        PsxSPU->Resume();	//	Resume execution for SPU

        while ( ( !PsxSPU->Terminated ) && ( !lpGTE->Terminated ) && ( !CPU->Terminated ) ) {
        	;
        }
}
//---------------------------------------------------------------------------
//	Display configure dialog
//---------------------------------------------------------------------------
void	__fastcall	EMU_Configure( void )
{
        MessageBox( NULL, "Configuration not available at present", CLASSNAME, MB_OK );
}
//---------------------------------------------------------------------------
void	__declspec( dllexport )	__fastcall	EmulatorLoadConfig( LPSTR lpszConfigFile )
{	LPSTR	lpszMess;
	if ( !FileExists( lpszConfigFile ) ) {
        	lpszMess = strcat( strcat( "Configuration file ", lpszConfigFile ), " not found." );
        	if ( lpszMess )
                	MessageBox( NULL, lpszMess, CLASSNAME, MB_OK );
                return;
        }
        LoadState( lpszConfigFile );
}
//---------------------------------------------------------------------------
void	__declspec( dllexport )	__fastcall	EmulatorSaveConfig( LPSTR lpszConfigFile )
{	LPSTR	lpszMess;

	if ( FileExists( lpszConfigFile ) )	{

        	lpszMess =strcat( strcat( "Overwrite file ", lpszConfigFile ), "?" );

        	int mrCode = MessageBox( NULL, lpszMess, CLASSNAME, MB_YESNO );

                if ( mrCode == IDNO )
                	return;
        }

        DeleteFile( lpszConfigFile );	//	Delete the file and then save it

        SaveState( lpszConfigFile );
}
//---------------------------------------------------------------------------
void	__fastcall	EMUDebug( void )
{
//	BroadcastSystemMessage( NULL, NULL, MSG_SUSPEND, NULL, NULL );

//        SaveState( "state.cfg" );

//        BroadcastSystemMessage( NULL, NULL, MSG_RESUME, NULL, NULL );
	fprintf( stderr, "EMULATOR DEBUGGER INTERFACE.\n" );
}
//---------------------------------------------------------------------------
void	__declspec( dllexport ) __fastcall	EmulatorDebug( void )
{
	EMUDebug();
}
//---------------------------------------------------------------------------
//	Windows message handler		EMULATOR IN GENERAL
//---------------------------------------------------------------------------
LRESULT	CALLBACK	DLLWndProc( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam )
{

	//	Work out what windows wants to do

	switch ( uMsg ) {

        	case ( WM_CREATE ) : 	{	//!	Create.
                //! Creates the emulator, but do not initialise anything with-in it, therfore
                //! it will be in a completely blank state.
                	EMU_Create();
                        return( 0 );
                }

        	case ( MSG_INITIALISE ) : {	//!	Initialise.
                //! Initialise the emulator ready for execution, this sets up the CPU
                //! registers, hardware states, root counters, exceptions, BIOS, etc, etc
                //! in preparation for application load or cd boot.

                	if ( !fEmulatorPresent )	{	//	EEK!
                        //	Emulator has NOT been created, or creation failed. recreate it
                        	EMU_Create();
                        }

                        EMU_Initialise();

                        return( 0 );
                }

        	case ( MSG_CONFIGURE ) : {	//!	Configure.
                //!	Not available at present. User can load/save a state using the
                //! 'LoadState' and 'SaveState' methods respectively.
                	EMU_Configure();
                        return( 0 );
                }

                case ( MSG_TRANSFER_READ ) :
                case ( MSG_TRANSFER_WRITE ) : {	//!	Transfer. READ or WRITE
                //! Request to transfer program code from ide or debugger to the emulator, or
                //! from a file, or even the other way
                	MessageBox( NULL, "Transfer requested. Not implemented yet.", CLASSNAME, MB_OK );
                	return( 0 );
                }

                case ( MSG_EXECUTE ) : {	//!	Execute.
                //!	Execute the emulator from the current value in _PC_. Note
                //! that this will either be a program entry point or the BIOS
                //! reset vector, in which case the emulator behaves as though
                //! power has been turned on in the actual hardware, and will
                //! attempt to boot as a PlayStation would.
                	EmulatorExecute();
                        lpKernel->ConsoleMode();
                        return( 0 );
                }

                case ( MSG_TERMINATE ) : {	//!	Terminate.
                //!	Mmm, wonder what happens here eh?
                	if ( ( fEmulatorPresent ) || ( fEmulatorActive ) )
                        	EMU_Terminate();
                        return( 0 );
                }

                case ( MSG_RESET ) : {		//!	Reset.
                //!	Reset the emulator to a known state. If the 'wParam' parameter
                //! is equal to 0 the emulator is reset as though you pushed the
                //! reset button on the PlayStation. Else is it reset to a known state
                //! and execution is suspended.

                	//	Reset the emulator
                        fEmulatorActive = false;

                        //	Suspend everything
                        CPU->Suspend();
                        lpGTE->Suspend();
                        PsxSPU->Suspend();

                        //----	REMOVED
                        //
                        // ... now reset it, well actually re-initialise it, so that
                        // we know that it is in a 'known-state'
                        BroadcastSystemMessage( 0, NULL, MSG_INITIALISE, NULL, NULL );

                        //	Now we actually reset it all
                        //PsxMem->Reset();

                        //CPU->Reset();
                        //lpGTE->Reset();
                        //PsxSPU->Reset();

			// ... now we re-execute it
                        CPU->Resume();
                        lpGTE->Resume();
                        PsxSPU->Resume();

                        return( 0 );
                }

                //	Resume suspended execution
                case ( MSG_RESUME ) : {
                        CPU->Resume();
                        lpGTE->Resume();
                        PsxSPU->Resume();
                        return( 0 );
                }

                //	go to sleep for a while
                case ( MSG_SUSPEND ) : {
                	CPU->Suspend();
                        lpGTE->Suspend();
                        PsxSPU->Suspend();
                        return( 0 );
                }

                case ( MSG_DEBUGGER ) : {
                	//BroadcastSystemMessage( NULL, NULL, MSG_SUSPEND, NULL, NULL );
                        EMUDebug();
                        //BroadcastSystemMessage( NULL, NULL, MSG_RESUME, NULL, NULL );
                        return( 0 );
                }

/*		case WM_COMMAND : {
			switch (LOWORD(wParam)) {
				case ID_FILE_EXIT:
					SysClose();
					PostQuitMessage(0);
					exit(0);
					return TRUE;

				case ID_FILE_RUN_CD:
					LoadCdBios = 0;
					SetMenu(hWnd, NULL);
					OpenPlugins(hWnd);
					SysReset();
					NeedReset = 0;
					CheckCdrom();
					cpuRegs.pc = cpuRegs.GPR.n.ra;
					Cpu->Execute();
					return TRUE;

				case ID_FILE_RUNCDBIOS:
					LoadCdBios = 1;
					SetMenu(hWnd, NULL);
					OpenPlugins(hWnd);
					CheckCdrom();
					SysReset();
					NeedReset = 0;
					Cpu->Execute();
					return TRUE;

				case ID_EMULATOR_RUN:
					SetMenu(hWnd, NULL);
					OpenPlugins(hWnd);
					if (NeedReset) { SysReset(); NeedReset = 0; }
					Cpu->Execute();
					return TRUE;

				case ID_EMULATOR_RESET:
					NeedReset = 1;
					return TRUE;

				case ID_CONFIGURATION_GRAPHICS:
					GPU_configure();
					return TRUE;

				case ID_CONFIGURATION_SOUND:
					SPU_configure();
					return TRUE;

				case ID_CONFIGURATION_CONTROLLERS:
					PAD1_configure();
					if (strcmp(Config.Pad1, Config.Pad2)) PAD2_configure();
					return TRUE;

				case ID_CONFIGURATION_CDROM:
				    CDR_configure();
					return TRUE;

				case ID_CONFIGURATION_MEMORYCARDMANAGER:
					DialogBox(gApp.hInstance, MAKEINTRESOURCE(IDD_MCDCONF), hWnd, (DLGPROC)ConfigureMcdsDlgProc);
					return TRUE;

				case ID_CONFIGURATION_CPU:
					DialogBox(gApp.hInstance, MAKEINTRESOURCE(IDD_CPUCONF), hWnd, (DLGPROC)ConfigureCpuDlgProc);
					return TRUE;

				case ID_CONFIGURATION:
					ConfigurePlugins(hWnd);
					return TRUE;

				case ID_FILE_RUN_EXE:
					{
					char File[256];
					if (!Open_File_Proc(File)) return TRUE;
					SetMenu(hWnd, NULL);
					OpenPlugins(hWnd);
					SysReset();
					NeedReset = 0;
					Load(File);
					}
					Cpu->Execute();
					return TRUE;

				case ID_HELP_HELP:
					ShellExecute(NULL, "open", "Readme.txt", NULL, NULL, SW_SHOWNORMAL);
					return TRUE;

				case ID_HELP_ABOUT:
					DialogBox(gApp.hInstance, MAKEINTRESOURCE(ABOUT_DIALOG), hWnd, (DLGPROC)AboutDlgProc);
					return TRUE;
			}
			break;
                }

		case WM_KEYDOWN:
			PADhandleKey(wParam);
			break;

		case WM_DESTROY:
			if (!AccBreak) {
				SysClose();
				PostQuitMessage(0);
				exit(0);
			}
			else AccBreak = 0;

			return TRUE;

		case WM_CREATE:
			gApp.hMenu = LoadMenu(gApp.hInstance, MAKEINTRESOURCE(IDR_MENU1));
			SetMenu(hWnd, gApp.hMenu);
			if (Config.PsxOut) OpenConsole();
			break;

		case WM_QUIT:
			if (Config.PsxOut) CloseConsole();
			exit(0);
			break;
*/
		default :
			return DefWindowProc( wHandle, uMsg, wParam, lParam);
        }
}
//---------------------------------------------------------------------------
// LOAD STUFF
//---------------------------------------------------------------------------
int 	__fastcall	CheckCdrom( void )
{	unsigned char time[4],*buf;

	time[ 0 ] = itob( 0 );			//	Time index for accessing a particular point of the CD
        time[ 1 ] = itob( 2 );
        time[ 2 ] = itob( 0x10 );

	//if ( CDR_readTrack( time ) == -1 )
        //	return -1;

	//buf = CDR_getBuffer();
        return( false );

        strncpy( CdromId, buf + 52, 10 );

//	if ( !( Config.PsxAuto ) ) { // autodetect system (pal or ntsc)
//		if ( strstr( CdromId, "ES" ) != NULL )
//			Config.PsxType = 1; // pal
//		else Config.PsxType = 0; // ntsc
//	}
	//psx_counters->UpdateVSyncRate();

	return 0;
}
//---------------------------------------------------------------------------
static	int	PSXGetFileType( FILE *f )
{	unsigned long current;
	unsigned long mybuf[2048];
        EXE_HEADER *exe_hdr;
        FILHDR *coff_hdr;

        current = ftell(f);
        fseek(f,0L,SEEK_SET);
        fread(mybuf,2048,1,f);
        fseek(f,current,SEEK_SET);

        exe_hdr = (EXE_HEADER *)mybuf;
        if (memcmp(exe_hdr->id,"PS-X EXE",8)==0)
        	return( PSX_EXE );

        if (mybuf[0]=='C' && mybuf[1]=='P' && mybuf[2]=='E')
        	return( CPE_EXE );

        if (mybuf[0]== '\177' && mybuf[1]=='E' && mybuf[2]=='L' && mybuf[3]=='F')
        	return( ELF_EXE );

        coff_hdr = (FILHDR *)mybuf;
        //if (coff_hdr->f_magic == 0x0162)
        if ( mybuf[ 0 ] == (char)0x01 && mybuf[ 1 ] == (char)0x62 )
        	return( COFF_EXE );

        return( INVALID_EXE );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) int	__fastcall	LoadECOFF( char *ExeFilePath )
{
	return( -1 );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) int	__fastcall	LoadCOFF( char *ExeFilePath )
{
	return( -1 );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) int	__fastcall	LoadCPE( char *ExeFilePath )
{
	return( -1 );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) int	__fastcall	LoadPSXEXE( char *ExeFilePath )
{
	//	Attempt to load a playstation executable, really elf, sort of
	return( -1 );
}
//---------------------------------------------------------------------------
//	Loads an executable for execution in the emulator. The following file
// formats are supported.
//
//		PSX_EXE,		Standard PlayStation Executable
//		ELF,			ELF Format binary executable
//		CPE,			Compressed PSX_EXE ?
//		ECOFF			ECOFF format binary executable
//		and COFF		COFF format binary executable
//
//	Other requirements the binary file must be compiled for the MIPS32 ISA
// and R3000 series CPU, also must be in little endian format at present, as
// big endian is not supported at present, it is not native to the ix86 cores
//---------------------------------------------------------------------------
__declspec( dllexport ) int	__fastcall	Load( char *ExePath )
{	FILE		*tmpFile;
	EXE_HEADER 	tmpHead;
	int 		type;

        if ( ExePath == NULL )
        	return( -1 );

	strcpy( CdromId, "SLUS_999.99" );

        tmpFile = fopen( ExePath, "rb" );

	if ( tmpFile == NULL ) {
                return( -1 );
        }

        type = PSXGetFileType( tmpFile );

        switch ( type ) {

        	case ( PSX_EXE ) : {
                	//	Standard PlayStaion Executable

                	//fread( &tmpHead, sizeof( EXE_HEADER ), 1, tmpFile );
                        //fseek( tmpFile, 0x800, SEEK_SET );
                        //fread( (void *)PC_MEM( tmpHead.t_addr ), tmpHead.t_size, 1, tmpFile );
			//fclose( tmpFile );

                        //	Set program entry point (playstation)
			//CPU->PC = tmpHead.EntryPoint;

			//CPU->GenRegs[ 28 ] = tmpHead.gp0;
			//CPU->GenRegs[ 29 ] = tmpHead.s_addr;

                        //strcpy( (char*)&RAM[ 0xB070 ], "PS-X EXE" );	//	support for pec :)

                        LoadPSXEXE( ExePath );
                        break;
                }

                case ( ELF_EXE ) : {
                	fclose( tmpFile );
                	return( LoadELF( ExePath ) );
                }

                case ( CPE_EXE ) : {
			fclose( tmpFile );
                        return( LoadCPE( ExePath ) );
                }

                case ( COFF_EXE ) : {
                	fclose( tmpFile );

                	//if ( f_magic == ECOFF )
                        //	return( LoadECOFF( ExePath ) )
                        //else if ( f_magic == COFF )
                        //	return( LoadCOFF( ExePath ) );

			return( -1 );
                }

                default : {
                	fclose( tmpFile );
                	return( -1 );
                }

        }
}
//---------------------------------------------------------------------------
//	Save the current state of the emulator to disk
//---------------------------------------------------------------------------
__declspec( dllexport )	int __fastcall	SaveState( LPSTR lpszFileName )
{       FILE	*fHandle;
	DWORD	*dwTmpHWM;

	if ( lpszFileName == NULL )
        	return( -1 );

        fHandle = fopen( lpszFileName, "wb" );

        if ( fHandle == NULL )
        	return( -1 );

        fprintf( stderr, "Saving configuration, %s ", lpszFileName );

        //	Save the current state to the file
        fwrite( DLLHeader, 1, 16, fHandle );	//	Write a header to we know that this is an image file

        fprintf( stderr, "." );

        //	Now write the current configuration to disk
        fwrite( &Config, 1, sizeof( EmulatorConfig ), fHandle );

        fprintf( stderr, "." );

        if ( Config.RAMImageFlag )
        	fwrite( PsxMem->lpPsxRAM, sizeof( DWORD ), RAM_SIZE, fHandle );
        fprintf( stderr, "." );
        if ( Config.ROMImageFlag )	{	//	Save ROM Image ?
        	if ( ( Config.InternalBIOS ) && ( Config.lpszBIOSFileName == NULL ) )
        		fwrite( PsxBIOS->lpPsxBIOS, sizeof( DWORD ), 0x00080000, fHandle );
        }
        fprintf( stderr, "." );

        if ( Config.PPMImageFlag )
        	fwrite( PsxMem->lpPsxPPM, sizeof( DWORD ), 0x00010000, fHandle );

        fprintf( stderr, "." );

        if ( Config.HWMImageFlag )	{
        	//dwTmpHWM = new DWORD[ 0x00010000 ];
                //if ( dwTmpHWM != NULL ) {
                //	ZeroMemory( dwTmpHWM, 0x00010000 );
                //        fwrite( dwTmpHWM, sizeof( DWORD ), 0x00010000, fHandle );
                //        delete []dwTmpHWM;
                //}
                fprintf( stderr, "." );
        }
        //	switch ( Config.MMUType ) {
        //		case ( 0 )
        //		case ( 1 ) FIXED lpMemLUT
        //if ( Config.TLBImageFlag )
        //	fwrite( PsxMem->lpMemLUT, sizeof( DWORD ), 0x00010000, fHandle );
        fprintf( stderr, "." );
        //		case ( 2 ) TLB MIN
        //		case ( 3 ) TLB FULL
        //		default ERROR
        //	}
        if ( Config.CP0ImageFlag )
        	fwrite( &CPU->CP0, sizeof( CoProcessor0 ), 1, fHandle );
        fprintf( stderr, "." );
        if ( Config.CP1ImageFlag )
        	fwrite( &CPU->CP1, sizeof( CoProcessor1 ), 1, fHandle );
        fprintf( stderr, "." );
        if ( Config.CP2ImageFlag )
        	fwrite( &lpGTE->Regs, sizeof( CP2Regs ), 1, fHandle );
        fprintf( stderr, "." );
	fclose( fHandle );
	fprintf( stderr, ".\n" );
	return( 0 );
}
//---------------------------------------------------------------------------
__declspec( dllexport )	int 	__fastcall	LoadState( LPSTR lpszFileName )
{	char		*lpHeader = NULL;
	TFileStream	*fStream = NULL;

	if ( lpszFileName == NULL )
        	return( -1 );

        fStream = new TFileStream( lpszFileName, fmOpenRead | fmShareExclusive );
        if ( fStream == NULL )
        	return( -1 );

        lpHeader = new char[ 16 ];
        ZeroMemory( lpHeader, 16 );
        fStream->Read( lpHeader, 16 );

        if ( memcmp( lpHeader, DLLHeader, 16 ) != 0 ) {		//	Bye, bye ?
        	SystemMessage( AnsiString( lpszFileName ) + " is not an emulator configuration file." );
        	delete fStream;
                return( -1 );
        }

        //	Load the configuration

        printf( "Loading the configuration, ..." );

        ZeroMemory( &Config, sizeof( EmulatorConfig ) );

        fStream->Read( &Config, sizeof( EmulatorConfig ) );

        printf( "." );

        if ( Config.RAMImageFlag )	fStream->Read( PsxMem->lpPsxRAM, sizeof( DWORD ) * RAM_SIZE );
        printf( "." );
        if ( Config.ROMImageFlag )	{
        	if ( ( Config.InternalBIOS ) && ( Config.lpszBIOSFileName == NULL ) )
                	fStream->Read( PsxBIOS->lpPsxBIOS, sizeof( DWORD ) * 0x00080000 );
                else    {
                	PsxBIOS->LoadBIOS( Config.lpszBIOSFileName );
                }
        }
        printf( "." );
        if ( Config.PPMImageFlag )	fStream->Read( PsxMem->lpPsxPPM, sizeof( DWORD ) * 0x00010000 );
        printf( "." );
        if ( Config.HWMImageFlag )
        	printf( "<>" );
                //for ( WORD w=0; w<=0xFFFF; w++ )
                //	fStream->Read( (void*)lpHW->dwHWM[ w ], sizeof( DWORD ) );
        printf( "." );
        //if ( Config.TLBImageFlag )	fStream->Read( &PsxMem->lpMemLUT, sizeof( DWORD ) * 0x00010000 );
	printf( "." );
        if ( Config.CP0ImageFlag )	fStream->Read( &CPU->CP0, sizeof( CoProcessor0 ) );
        printf( "." );
        if ( Config.CP1ImageFlag ) 	fStream->Read( &CPU->CP1, sizeof( CoProcessor1 ) );
        printf( "." );
        if ( Config.CP2ImageFlag )	fStream->Read( &lpGTE->Regs, sizeof( CP2Regs ) );
        printf( "\nCompleted.\n" );

        delete fStream;

	return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	Message( FILE *lpHandle, LPSTR lpszMessage )
{
	if ( ( lpHandle == NULL ) || ( lpszMessage == NULL ) )
        	return;
        fprintf( lpHandle, "%s", lpszMessage );
}
//---------------------------------------------------------------------------
void	__fastcall	SystemMessage( AnsiString asMessage )
{
	if ( asMessage.Length() <= 0 )
        	return;
        fprintf( stderr, "%s\n", asMessage.c_str() );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	BitDump( DWORD *lpData, DWORD BitCount )
{
	for ( DWORD d=0; d<BitCount; d++ ) {
        	if ( lpData[ d ] & ( 1 << d ) )
                	fprintf( stderr, "1" );
                else	fprintf( stderr, "0" );
        }
}
//---------------------------------------------------------------------------
