//---------------------------------------------------------------------------
//
//	$FileName	::	EmulatorDLL.h
//	$Purpose	::	Emulator for PSXIDE (PlayStation ONLY)
//				Based on the emulator from the Pcsx Team, thanx :)
//
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#define EMULATOR_DLL_H
//---------------------------------------------------------------------------
#ifdef	CONSOLE_TEST_APP
#undef EMULATOR_DLL_C
#undef DEBUGGER_DLL_C
#endif
//---------------------------------------------------------------------------
#define	WIN32_LEAN_AND_MEAN				//	Force Windows header to strip excess/unwanted crap
//---------------------------------------------------------------------------
#ifdef EMULATOR_DLL_C                                   //	EMULATOR_DLL_C
		#define EMULATOR_API	__declspec( dllexport )
	#else
		#define EMULATOR_API	__declspec( dllimport )
#endif							//	EMULATOR_DLL_C
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <windows.h>
#include <Classes.hpp>
#include <System.hpp>
#include <SysUtils.hpp>
//---------------------------------------------------------------------------
#define ISA_MIPS32		//	Tell the rest of the emulator we are emulating MIPS 32 ISA
//---------------------------------------------------------------------------
#ifndef DEBUGGER_DLL_H
#include "..\Debugger\DebuggerDLL.h"
#endif
//---------------------------------------------------------------------------
//#include "System.h"
//#include <libpng/zlib.h>
//---------------------------------------------------------------------------
#if defined(__WIN32__)					//	Running on Windows ? (WIN32)
		#include <windows.h>
	#elif defined (__LINUX__)			//	else running on Linux ?
		#include <sys/types.h>
#endif
//---------------------------------------------------------------------------
//!	Include support for ELF format binary executable files. Note that only
//! little endian format files are supported at present
//---------------------------------------------------------------------------
#ifndef ELFH						//	Support for ELF binaries
#include "elf.h"
#endif
//---------------------------------------------------------------------------
#define	MIN( a, b )	( ( (a) < (b) ) ? (a) : (b) )	//	return minimum of a and b
#define MAX( a, b )	( ( (a) > (b) ) ? (a) : (b) )	//	return maximum of a and b
//---------------------------------------------------------------------------
//	Turn a flag on/off at a specified address
//---------------------------------------------------------------------------
#define	FlagOn	( ADDR, FLAG )	( PsxMem->RAM32[ ADDR ] |= FLAG )
#define FlagOff ( ADDR, FLAG )	( PsxMem->RAM32[ ADDR ] &= ~FLAG )
//---------------------------------------------------------------------------
//	malloc info, this is for psx only!, and used by the bios / kernel / oscore
//---------------------------------------------------------------------------
typedef struct _malloc_chunk {
	DWORD			stat;	//	Status of this chunk, see below
	DWORD			size;	//	size of this chunk
	struct _malloc_chunk 	*fd;	//	forward (next) pointer in malloc chain
	struct _malloc_chunk 	*bk;	//	backward (prev) pointer in malloc chain
} malloc_chunk;
//---------------------------------------------------------------------------
#define INUSE 	0x1	//	Malloc chunk in use else free
#define NOTUSED	~INUSE	//	Malloc chunk not in use
//---------------------------------------------------------------------------
//!	Event Control Buffers
//---------------------------------------------------------------------------
typedef struct _event_buffer_def_ {
	DWORD desc;		//!	Description
	DWORD status;		//!	Status, active, ready, waiting, unused
	DWORD spec;		//!	Specified value to give ? variable
	DWORD mode;		//!	Mode of operation, i/o wait, timed
	DWORD fhandler;		//!	function handler
        struct _event_buffer_def_ *lpNext;	//!	Next in chain
} EvCB, *lpEvCB;
//---------------------------------------------------------------------------
//!	Event status flags
//---------------------------------------------------------------------------
#define EvStUNUSED	0x0000	//!	Event not is use, free for assignment.
#define EvStASSIGNED	0x0001	//!	Event has been assigned.
#define EvStWAIT    	0x1000	//!	Wait for this event.
#define EvStACTIVE  	0x2000	//!	Event is currently active.
#define EvStALREADY 	0x4000	//!	Ready for triggering.
//---------------------------------------------------------------------------
#define MAX_EVENTS	0x1f
#define EVENT_MASK	0x1f000000	//	Event = r4 & EVENT_MASK < MAX_EVENTS
//---------------------------------------------------------------------------
//	Individual Event Codes, but what these pertain to is unknown
//---------------------------------------------------------------------------
#define EVENTCODE_0	0x00
#define EVENTCODE_1	0x01
#define EVENTCODE_2	0x02
#define EVENTCODE_3	0x03
#define EVENTCODE_4	0x04
#define EVENTCODE_5	0x05
#define EVENTCODE_6	0x06
#define EVENTCODE_7	0x07
#define CDROM_EVENTCODE	0x08	//	EVENT CODE 0x08	CDROM ATTENTION IS REQUIRED
#define EVENTCODE_9	0x09
#define EVENTCODE_A	0x0A
#define EVENTCODE_B	0x0B
#define EVENTCODE_C	0x0C
#define EVENTCODE_D	0x0D
#define EVENTCODE_E	0x0E
#define EVENTCODE_F	0x0F
#define EVENTCODE_10	0x10
#define EVENTCODE_11	0x11
#define EVENTCODE_12	0x12
#define EVENTCODE_13	0x13
#define EVENTCODE_14	0x14
#define EVENTCODE_15	0x15
#define EVENTCODE_16	0x16
#define EVENTCODE_17	0x17
#define EVENTCODE_18	0x18
#define EVENTCODE_19	0x19
#define EVENTCODE_1A	0x1A
#define EVENTCODE_1B	0x1B
#define EVENTCODE_1C	0x1C
#define EVENTCODE_1D	0x1D
#define EVENTCODE_1E	0x1E
#define EVENTCODE_1F	0x1F
//---------------------------------------------------------------------------
__declspec( dllimport )	void	__fastcall	SystemMessage	( AnsiString asMessage );
//__declspec( dllimport ) void	__fastcall	Message		( FILE *lpHandle, LPSTR lpszMessage );
__declspec( dllimport ) void	__fastcall	GLPrintf	( int x, int y, const char *fmt = NULL );
//---------------------------------------------------------------------------
//	These need to be converted to point to the relevant section of the emulator
// ie 0xA0 calls must be mapped to the relevant Windows / Linux call, same with the
// BIOS, and KERNEL syscalls. A0 = OSCORE B0 = KERNEL C0 = BIOS
//---------------------------------------------------------------------------
const	char sysCalls[ 3 ][ 256 ][ 256 ] = {

	//	0xA0 calls, run-time library
	{/*0x0000 */ "int open(char *name , int mode)",
		/*0x0001 */ "int lseek(int fd , int offset , int whence)",
		/*0x0002 */ "int read(int fd , char *buf , int nbytes)",
		/*0x0003 */ "int write(int fd , char *buf , int nbytes)",
		/*0x0004 */ "close(int fd)",
		/*0x0005 */ "int ioctl(int fd , int cmd , int arg)",
		/*0x0006 */ "exit()",
		/*0x0007 */ "sys_b0_39()",
		/*0x0008 */ "char getc(int fd)",
		/*0x0009 */ "putc(char c , int fd)",
		/*0x000a */ "todigit",
		/*0x000b */ "double atof(char *s)",
		/*0x000c */ "long strtoul(char *s , char **ptr , int base)",
		/*0x000d */ "unsigned long strtol(char *s , char **ptr , int base)",
		/*0x000e */ "int abs(int val)",
		/*0x000f */ "long labs(long lval)",
		/*0x0010 */ "long atoi(char *s)",
		/*0x0011 */ "int atol(char *s)",
		/*0x0012 */ "atob",
		/*0x0013 */ "int setjmp(jmp_buf *ctx)",
		/*0x0014 */ "longjmp(jmp_buf *ctx , int value)",
		/*0x0015 */ "char *strcat(char *dst , char *src)",
		/*0x0016 */ "char *strncat(char *dst , char *src , int n)",
		/*0x0017 */ "int strcmp(char *dst , char *src)",
		/*0x0018 */ "int strncmp(char *dst , char *src , int n)",
		/*0x0019 */ "char *strcpy(char *dst , char *src)",
		/*0x001a */ "char *strncpy(char *dst , char *src , int n))",
		/*0x001b */ "int strlen(char *s)",
		/*0x001c */ "int index(char *s , int c)",
		/*0x001d */ "int rindex(char *s , int c)",
		/*0x001e */ "char *strchr(char *c , int c)",
		/*0x001f */ "char *strrchr(char *c , int c)",
		/*0x0020 */ "char *strpbrk(char *dst , *src)",
		/*0x0021 */ "int strspn(char *s , char *set)",
		/*0x0022 */ "int strcspn(char *s , char *set)",
		/*0x0023 */ "strtok(char *s , char *set)",
		/*0x0024 */ "strstr(char *s , char *set)",
		/*0x0025 */ "int toupper(int c)",
		/*0x0026 */ "int tolower(int c)",
		/*0x0027 */ "void bcopy(void *src , void *dst , int len)",
		/*0x0028 */ "void bzero(void *ptr , int len)",
		/*0x0029 */ "int bcmp(void *ptr1 , void *ptr2 , int len)",
		/*0x002a */ "memcpy(void *dst , void *src , int n)",
		/*0x002b */ "memset(void *dst , char c , int n)",
		/*0x002c */ "memmove(void *dst , void *src , int n)",
		/*0x002d */ "memcmp(void *dst , void *src , int n)",
		/*0x002e */ "memchr(void *s , int c , int n)",
		/*0x002f */ "int rand()",
		/*0x0030 */ "void srand(unsigned int seed)",
		/*0x0031 */ "void qsort(void *base , int nel , int width , int (*cmp)*void *,void *))",
		/*0x0032 */ "double strtod(char *s , char *endptr)",
		/*0x0033 */ "void *malloc(int size)",
		/*0x0034 */ "free(void *buf)",
		/*0x0035 */ "void *lsearch(void *key , void *base , int belp , int width , int (*cmp)(void * , void *))",
		/*0x0036 */ "void *bsearch( void *key , void *base , int nel , int size , int (*cmp)(void * , void *))",
		/*0x0037 */ "void *calloc(int size , int n)",
		/*0x0038 */ "void *realloc(void *buf , int n)",
		/*0x0039 */ "InitHeap(void *block , int n)",
		/*0x003a */ "_exit()",
		/*0x003b */ "char getchar(int fd) ",
		/*0x003c */ "putchar(char c , int fd)",
		/*0x003d */ "char *gets(char *s)",
		/*0x003e */ "puts(char *s)",
		/*0x003f */ "printf(char *fmt , ...)",
		/*0x0040 */ "",
		/*0x0041 */ "LoadTest(char *name , XF_HDR *header)",
		/*0x0042 */ "Load(char *name , XF_HDR *header)",
		/*0x0043 */ "Exec(struct EXEC *header , int argc , char **argc)",
		/*0x0044 */ "FlushCache()",
		/*0x0045 */ "void InstallInterruptHandler()",
		/*0x0046 */ "GPU_dw",
		/*0x0047 */ "",
		/*0x0048 */ "int SetGPUStatus(int status)",
		/*0x0049 */ "GPU_cw",
		/*0x004a */ "GPU_cwb (not sure)",
		/*0x004b */ "",
		/*0x004c */ "",
		/*0x004d */ "int GetGPUStatus()",
		/*0x004e? */ "GPU_sync",
		/*0x004f */ "",
		/*0x0050 */ "",
		/*0x0051 */ "LoadExec(char *name , int , int)",
		/*0x0052 */ "GetSysSp()",
		/*0x0053 */ "",
		/*0x0054 */ "_96_init()",
		/*0x0055 */ "_bu_init()",
		/*0x0056 */ "_96_remove()",
		/*0x0057 */ "return_0",
		/*0x0058 */ "return_0",
		/*0x0059 */ "return_0",
		/*0x005a */ "return_0",
		/*0x005b */ "dev_tty_init",
		/*0x005c */ "dev_tty_open",
		/*0x005d */ "",
		/*0x005e */ "dev_tty_ioctl",
		/*0x005f */ "dev_cd_open",
		/*0x0060 */ "dev_cd_read",
		/*0x0061 */ "dev_cd_close",
		/*0x0062 */ "dev_cd_firstfile",
		/*0x0063 */ "dev_cd_nextfile",
		/*0x0064 */ "dev_cd_chdir",
		/*0x0065 */ "dev_card_open",
		/*0x0066 */ "dev_card_read",
		/*0x0067 */ "dev_card_write",
		/*0x0068 */ "dev_card_close",
		/*0x0069 */ "dev_card_firstfile",
		/*0x006a */ "dev_card_nextfile",
		/*0x006b */ "dev_card_erase",
		/*0x006c */ "dev_card_undelete",
		/*0x006d */ "dev_card_format",
		/*0x006e */ "dev_card_rename",
		/*0x006f */ "",
		/*0x0070 */ "_bu_init",
		/*0x0071 */ "_96_init",
		/*0x0072 */ "_96_remove",
		/*0x0073 */ "",
		/*0x0074 */ "",
		/*0x0075 */ "",
		/*0x0076 */ "",
		/*0x0077 */ "",
		/*0x0078 */ "_96_CdSeekL",
		/*0x0079 */ "",
		/*0x007a */ "",
		/*0x007b */ "",
		/*0x007c */ "_96_CdGetStatus",
		/*0x007d */ "",
		/*0x007e */ "_96_CdRead",
		/*0x007f */ "",
		/*0x0080 */ "",
		/*0x0081 */ "",
		/*0x0082 */ "",
		/*0x0083 */ "",
		/*0x0084 */ "",
		/*0x0085 */ "_96_CdStop",
		/*0x0086 */ "",
		/*0x0087 */ "",
		/*0x0088 */ "",
		/*0x0089 */ "",
		/*0x008a */ "",
		/*0x008b */ "",
		/*0x008c */ "",
		/*0x008d */ "",
		/*0x008e */ "",
		/*0x008f */ "",
		/*0x0090 */ "",
		/*0x0091 */ "",
		/*0x0092 */ "",
		/*0x0093 */ "",
		/*0x0094 */ "",
		/*0x0095 */ "",
		/*0x0096 */ "AddCDROMDevice()",
		/*0x0097 */ "AddMemCardDevice()",
		/*0x0098 */ "DisableKernelIORedirection()",
		/*0x0099 */ "EnableKernelIORedirection()",
		/*0x009a */ "",
		/*0x009b */ "",
		/*0x009c */ "GetConf(int Event , int TCB , int Stack)",
		/*0x009d */ "GetConf(int *Event , int *TCB , int *Stack)",
		/*0x009e */ "",
		/*0x009f */ "SetMem(int size)",
		/*0x00a0 */ "_boot",
		/*0x00a1 */ "SystemError",
		/*0x00a2 */ "EnqueueCdIntr",
		/*0x00a3 */ "DequeueCdIntr",
		/*0x00a4 */ "",
		/*0x00a5 */ "ReadSector(count,sector,buffer)",
		/*0x00a6 */ "get_cd_status ??",
		/*0x00a7 */ "bufs_cb_0",
		/*0x00a8 */ "bufs_cb_1",
		/*0x00a9 */ "bufs_cb_2",
		/*0x00aa */ "bufs_cb_3",
		/*0x00ab */ "_card_info",
		/*0x00ac */ "_card_load",
		/*0x00ad */ "_card_auto",
		/*0x00ae */ "bufs_cb_4",
		/*0x00af */ "",
		/*0x00b0 */ "",
		/*0x00b1 */ "",
		/*0x00b2 */ "do_a_long_jmp()",
		/*0x00b3 */ "",
		/*0x00b4 */ "",
		/*0x00b5 */ "",
		/*0x00b6 */ "",
		/*0x00b7 */ "",
		/*0x00b8 */ "",
		/*0x00b9 */ "",
		/*0x00ba */ "",
		/*0x00bb */ "",
		/*0x00bc */ "",
		/*0x00bd */ "",
		/*0x00be */ "",
		/*0x00be */ "",
		/*0x00c0 */ "",
		/*0x00c1 */ "",
		/*0x00c2 */ "",
		/*0x00c3 */ "",
		/*0x00c4 */ "",
		/*0x00c5 */ "",
		/*0x00c6 */ "",
		/*0x00c7 */ "",
		/*0x00c8 */ "",
		/*0x00c9 */ "",
		/*0x00ca */ "",
		/*0x00cb */ "",
		/*0x00cc */ "",
		/*0x00cd */ "",
		/*0x00ce */ "",
		/*0x00cf */ "",
		/*0x00d0 */ "",
		/*0x00d1 */ "",
		/*0x00d2 */ "",
		/*0x00d3 */ "",
		/*0x00d4 */ "",
		/*0x00d5 */ "",
		/*0x00d6 */ "",
		/*0x00d7 */ "",
		/*0x00d8 */ "",
		/*0x00d9 */ "",
		/*0x00da */ "",
		/*0x00db */ "",
		/*0x00dc */ "",
		/*0x00dd */ "",
		/*0x00de */ "",
		/*0x00de */ "",
		/*0x00q0 */ "",
		/*0x00e1 */ "",
		/*0x00e2 */ "",
		/*0x00e3 */ "",
		/*0x00e4 */ "",
		/*0x00e5 */ "",
		/*0x00e6 */ "",
		/*0x00e7 */ "",
		/*0x00e8 */ "",
		/*0x00e9 */ "",
		/*0x00ea */ "",
		/*0x00eb */ "",
		/*0x00ec */ "",
		/*0x00ed */ "",
		/*0x00ee */ "",
		/*0x00ee */ "",
		/*0x00f0 */ "",
		/*0x00f1 */ "",
		/*0x00f2 */ "",
		/*0x00f3 */ "",
		/*0x00f4 */ "",
		/*0x00f5 */ "",
		/*0x00f6 */ "",
		/*0x00f7 */ "",
		/*0x00f8 */ "",
		/*0x00f9 */ "",
		/*0x00fa */ "",
		/*0x00fb */ "",
		/*0x00fc */ "",
		/*0x00fd */ "",
		/*0x00fe */ "",
		/*0x00ff */ ""},

        //	0xB0 call, BIOS only
	{/*0x0000 */ "SysMalloc",
		/*0x0001 */ "",
		/*0x0002 */ "",
		/*0x0003 */ "",
		/*0x0004 */ "",
		/*0x0005 */ "",
		/*0x0006 */ "",
		/*0x0007 */ "DeliverEvent(class , event)",
		/*0x0008 */ "OpenEvent(class , spec , mode , func)",
		/*0x0009 */ "CloseEvent",
		/*0x000a */ "WaitEvent",
		/*0x000b */ "TestEvent",
		/*0x000c */ "EnableEvent",
		/*0x000d */ "DisableEvent",
		/*0x000e */ "OpenTh",
		/*0x000f */ "CloseTh",
		/*0x0010 */ "ChangeTh",
		/*0x0011 */ "",
		/*0x0012 */ "InitPad",
		/*0x0013 */ "StartPad",
		/*0x0014 */ "StopPAD",
		/*0x0015 */ "PAD_init",
		/*0x0016 */ "u_long PAD_dr()",
		/*0x0017 */ "ReturnFromException",
		/*0x0018 */ "ResetEntryInt",
		/*0x0019 */ "HookEntryInt",
		/*0x001a */ "",
		/*0x001b */ "",
		/*0x001c */ "",
		/*0x001d */ "",
		/*0x001e */ "",
		/*0x001f */ "",
		/*0x0020 */ "UnDeliverEvent(class , event)",
		/*0x0021 */ "",
		/*0x0022 */ "",
		/*0x0023 */ "",
		/*0x0024 */ "",
		/*0x0025 */ "",
		/*0x0026 */ "",
		/*0x0027 */ "",
		/*0x0028 */ "",
		/*0x0029 */ "",
		/*0x002a */ "",
		/*0x002b */ "",
		/*0x002c */ "",
		/*0x002d */ "",
		/*0x002e */ "",
		/*0x002f */ "",
		/*0x0030 */ "",
		/*0x0031 */ "",
		/*0x0032 */ "int open(char *name,int access)",
		/*0x0033 */ "int lseek(int fd,long pos,int seektype)",
		/*0x0034 */ "int read(int fd,void *buf,int nbytes)",
		/*0x0035 */ "int write(int fd,void *buf,int nbytes)",
		/*0x0036 */ "close(int fd)",
		/*0x0037 */ "int ioctl(int fd , int cmd , int arg)",
		/*0x0038 */ "exit(int exitcode)",
		/*0x0039 */ "",
		/*0x003a */ "char getc(int fd)",
		/*0x003b */ "putc(int fd,char ch)",
		/*0x003c */ "char getchar()",
		/*0x003d */ "putchar(char ch)",
		/*0x003e */ "char *gets(char *s)",
		/*0x003f */ "puts(char *s)",
		/*0x0040 */ "cd",
		/*0x0041 */ "format",
		/*0x0042 */ "firstfile",
		/*0x0043 */ "nextfile",
		/*0x0044 */ "rename",
		/*0x0045 */ "delete",
		/*0x0046 */ "undelete",
		/*0x0047 */ "AddDevice",
		/*0x0048 */ "RemoveDevice",
		/*0x0049 */ "PrintInstalledDevices",
		/*0x004a */ "InitCARD",
		/*0x004b */ "StartCARD",
		/*0x004c */ "StopCARD",
		/*0x004d */ "",
		/*0x004e */ "_card_write",
		/*0x004f */ "_card_read",
		/*0x0050 */ "_new_card",
		/*0x0051 */ "Krom2RawAdd",
		/*0x0052 */ "",
		/*0x0053 */ "",
		/*0x0054 */ "long _get_errno(void)",
		/*0x0055 */ "long _get_error(long fd)",
		/*0x0056 */ "GetC0Table",
		/*0x0057 */ "GetB0Table",
		/*0x0058 */ "_card_chan",
		/*0x0059 */ "",
		/*0x005a */ "",
		/*0x005b */ "ChangeClearPad(int)",
		/*0x005c */ "_card_status",
		/*0x005d */ "_card_wait",
		/*0x005e */ "",
		/*0x005f */ "",
		/*0x0060 */ "",
		/*0x0061 */ "",
		/*0x0062 */ "",
		/*0x0063 */ "",
		/*0x0064 */ "",
		/*0x0065 */ "",
		/*0x0066 */ "",
		/*0x0067 */ "",
		/*0x0068 */ "",
		/*0x0069 */ "",
		/*0x006a */ "",
		/*0x006b */ "",
		/*0x006c */ "",
		/*0x006d */ "",
		/*0x006e */ "",
		/*0x006f */ "",
		/*0x0070 */ "",
		/*0x0071 */ "",
		/*0x0072 */ "",
		/*0x0073 */ "",
		/*0x0074 */ "",
		/*0x0075 */ "",
		/*0x0076 */ "",
		/*0x0077 */ "",
		/*0x0078 */ "",
		/*0x0079 */ "",
		/*0x007a */ "",
		/*0x007b */ "",
		/*0x007c */ "",
		/*0x007d */ "",
		/*0x007e */ "",
		/*0x007f */ "",
		/*0x0080 */ "",
		/*0x0081 */ "",
		/*0x0082 */ "",
		/*0x0083 */ "",
		/*0x0084 */ "",
		/*0x0085 */ "",
		/*0x0086 */ "",
		/*0x0087 */ "",
		/*0x0088 */ "",
		/*0x0089 */ "",
		/*0x008a */ "",
		/*0x008b */ "",
		/*0x008c */ "",
		/*0x008d */ "",
		/*0x008e */ "",
		/*0x008f */ "",
		/*0x0090 */ "",
		/*0x0091 */ "",
		/*0x0092 */ "",
		/*0x0093 */ "",
		/*0x0094 */ "",
		/*0x0095 */ "",
		/*0x0096 */ "",
		/*0x0097 */ "",
		/*0x0098 */ "",
		/*0x0099 */ "",
		/*0x009a */ "",
		/*0x009b */ "",
		/*0x009c */ "",
		/*0x009d */ "",
		/*0x009e */ "",
		/*0x009f */ "",
		/*0x00a0 */ "",
		/*0x00a1 */ "",
		/*0x00a2 */ "",
		/*0x00a3 */ "",
		/*0x00a4 */ "",
		/*0x00a5 */ "",
		/*0x00a6 */ "",
		/*0x00a7 */ "",
		/*0x00a8 */ "",
		/*0x00a9 */ "",
		/*0x00aa */ "",
		/*0x00ab */ "",
		/*0x00ac */ "",
		/*0x00ad */ "",
		/*0x00ae */ "",
		/*0x00af */ "",
		/*0x00b0 */ "",
		/*0x00b1 */ "",
		/*0x00b2 */ "",
		/*0x00b3 */ "",
		/*0x00b4 */ "",
		/*0x00b5 */ "",
		/*0x00b6 */ "",
		/*0x00b7 */ "",
		/*0x00b8 */ "",
		/*0x00b9 */ "",
		/*0x00ba */ "",
		/*0x00bb */ "",
		/*0x00bc */ "",
		/*0x00bd */ "",
		/*0x00be */ "",
		/*0x00be */ "",
		/*0x00c0 */ "",
		/*0x00c1 */ "",
		/*0x00c2 */ "",
		/*0x00c3 */ "",
		/*0x00c4 */ "",
		/*0x00c5 */ "",
		/*0x00c6 */ "",
		/*0x00c7 */ "",
		/*0x00c8 */ "",
		/*0x00c9 */ "",
		/*0x00ca */ "",
		/*0x00cb */ "",
		/*0x00cc */ "",
		/*0x00cd */ "",
		/*0x00ce */ "",
		/*0x00cf */ "",
		/*0x00d0 */ "",
		/*0x00d1 */ "",
		/*0x00d2 */ "",
		/*0x00d3 */ "",
		/*0x00d4 */ "",
		/*0x00d5 */ "",
		/*0x00d6 */ "",
		/*0x00d7 */ "",
		/*0x00d8 */ "",
		/*0x00d9 */ "",
		/*0x00da */ "",
		/*0x00db */ "",
		/*0x00dc */ "",
		/*0x00dd */ "",
		/*0x00de */ "",
		/*0x00de */ "",
		/*0x00q0 */ "",
		/*0x00e1 */ "",
		/*0x00e2 */ "",
		/*0x00e3 */ "",
		/*0x00e4 */ "",
		/*0x00e5 */ "",
		/*0x00e6 */ "",
		/*0x00e7 */ "",
		/*0x00e8 */ "",
		/*0x00e9 */ "",
		/*0x00ea */ "",
		/*0x00eb */ "",
		/*0x00ec */ "",
		/*0x00ed */ "",
		/*0x00ee */ "",
		/*0x00ee */ "",
		/*0x00f0 */ "",
		/*0x00f1 */ "",
		/*0x00f2 */ "",
		/*0x00f3 */ "",
		/*0x00f4 */ "",
		/*0x00f5 */ "",
		/*0x00f6 */ "",
		/*0x00f7 */ "",
		/*0x00f8 */ "",
		/*0x00f9 */ "",
		/*0x00fa */ "",
		/*0x00fb */ "",
		/*0x00fc */ "",
		/*0x00fd */ "",
		/*0x00fe */ "",
		/*0x00ff */ ""},

        //	0xC0 calls KERNEL only
	{/*0x0000 */ "InitRCnt",
		/*0x0001 */ "InitException",
		/*0x0002 */ "SysEnqIntRP(int index , long *queue)",
		/*0x0003 */ "SysDeqIntRP(int index , long *queue)",
		/*0x0004 */ "get_free_EvCB_slot()",
		/*0x0005 */ "get_free_TCB_slot()",
		/*0x0006 */ "ExceptionHandler",
		/*0x0007 */ "InstallExceptionHandlers",
		/*0x0008 */ "SysInitMemory",
		/*0x0009 */ "SysInitKMem",
		/*0x000a */ "ChangeClearRCnt",
		/*0x000b */ "SystemError",
		/*0x000c */ "InitDefInt",
		/*0x000d */ "",
		/*0x000e */ "",
		/*0x000f */ "",
		/*0x0010 */ "",
		/*0x0011 */ "",
		/*0x0012 */ "InstallDevices",
		/*0x0013 */ "FlushStdInOutPut",
		/*0x0014 */ "return 0",
		/*0x0015 */ "_cdevinput",
		/*0x0016 */ "_cdevscan",
		/*0x0017 */ "char _circgetc(struct device_buf *circ)",
		/*0x0018 */ "_circputc(char c , struct device_buf *circ)",
		/*0x0019 */ "ioabort(char *str)",
		/*0x001a */ "",
		/*0x001b */ "KernelRedirect(int flag)",
		/*0x001c */ "PatchA0Table",
		/*0x001d */ "",
		/*0x001e */ "",
		/*0x001f */ "",
		/*0x0020 */ "",
		/*0x0021 */ "",
		/*0x0022 */ "",
		/*0x0023 */ "",
		/*0x0024 */ "",
		/*0x0025 */ "",
		/*0x0026 */ "",
		/*0x0027 */ "",
		/*0x0028 */ "",
		/*0x0029 */ "",
		/*0x002a */ "",
		/*0x002b */ "",
		/*0x002c */ "",
		/*0x002d */ "",
		/*0x002e */ "",
		/*0x002f */ "",
		/*0x0030 */ "",
		/*0x0031 */ "",
		/*0x0032 */ "",
		/*0x0033 */ "",
		/*0x0034 */ "",
		/*0x0035 */ "",
		/*0x0036 */ "",
		/*0x0037 */ "",
		/*0x0038 */ "",
		/*0x0039 */ "",
		/*0x003a */ "",
		/*0x003b */ "",
		/*0x003c */ "",
		/*0x003d */ "",
		/*0x003e */ "",
		/*0x003f */ "",
		/*0x0040 */ "",
		/*0x0041 */ "",
		/*0x0042 */ "",
		/*0x0043 */ "",
		/*0x0044 */ "",
		/*0x0045 */ "",
		/*0x0046 */ "",
		/*0x0047 */ "",
		/*0x0048 */ "",
		/*0x0049 */ "",
		/*0x004a */ "",
		/*0x004b */ "",
		/*0x004c */ "",
		/*0x004d */ "",
		/*0x004e */ "",
		/*0x004f */ "",
		/*0x0050 */ "",
		/*0x0051 */ "",
		/*0x0052 */ "",
		/*0x0053 */ "",
		/*0x0054 */ "",
		/*0x0055 */ "",
		/*0x0056 */ "",
		/*0x0057 */ "",
		/*0x0058 */ "",
		/*0x0059 */ "",
		/*0x005a */ "",
		/*0x005b */ "",
		/*0x005c */ "",
		/*0x005d */ "",
		/*0x005e */ "",
		/*0x005f */ "",
		/*0x0060 */ "",
		/*0x0061 */ "",
		/*0x0062 */ "",
		/*0x0063 */ "",
		/*0x0064 */ "",
		/*0x0065 */ "",
		/*0x0066 */ "",
		/*0x0067 */ "",
		/*0x0068 */ "",
		/*0x0069 */ "",
		/*0x006a */ "",
		/*0x006b */ "",
		/*0x006c */ "",
		/*0x006d */ "",
		/*0x006e */ "",
		/*0x006f */ "",
		/*0x0070 */ "",
		/*0x0071 */ "",
		/*0x0072 */ "",
		/*0x0073 */ "",
		/*0x0074 */ "",
		/*0x0075 */ "",
		/*0x0076 */ "",
		/*0x0077 */ "",
		/*0x0078 */ "",
		/*0x0079 */ "",
		/*0x007a */ "",
		/*0x007b */ "",
		/*0x007c */ "",
		/*0x007d */ "",
		/*0x007e */ "",
		/*0x007f */ "",
		/*0x0080 */ "",
		/*0x0081 */ "",
		/*0x0082 */ "",
		/*0x0083 */ "",
		/*0x0084 */ "",
		/*0x0085 */ "",
		/*0x0086 */ "",
		/*0x0087 */ "",
		/*0x0088 */ "",
		/*0x0089 */ "",
		/*0x008a */ "",
		/*0x008b */ "",
		/*0x008c */ "",
		/*0x008d */ "",
		/*0x008e */ "",
		/*0x008f */ "",
		/*0x0090 */ "",
		/*0x0091 */ "",
		/*0x0092 */ "",
		/*0x0093 */ "",
		/*0x0094 */ "",
		/*0x0095 */ "",
		/*0x0096 */ "",
		/*0x0097 */ "",
		/*0x0098 */ "",
		/*0x0099 */ "",
		/*0x009a */ "",
		/*0x009b */ "",
		/*0x009c */ "",
		/*0x009d */ "",
		/*0x009e */ "",
		/*0x009f */ "",
		/*0x00a0 */ "",
		/*0x00a1 */ "",
		/*0x00a2 */ "",
		/*0x00a3 */ "",
		/*0x00a4 */ "",
		/*0x00a5 */ "",
		/*0x00a6 */ "",
		/*0x00a7 */ "",
		/*0x00a8 */ "",
		/*0x00a9 */ "",
		/*0x00aa */ "",
		/*0x00ab */ "",
		/*0x00ac */ "",
		/*0x00ad */ "",
		/*0x00ae */ "",
		/*0x00af */ "",
		/*0x00b0 */ "",
		/*0x00b1 */ "",
		/*0x00b2 */ "",
		/*0x00b3 */ "",
		/*0x00b4 */ "",
		/*0x00b5 */ "",
		/*0x00b6 */ "",
		/*0x00b7 */ "",
		/*0x00b8 */ "",
		/*0x00b9 */ "",
		/*0x00ba */ "",
		/*0x00bb */ "",
		/*0x00bc */ "",
		/*0x00bd */ "",
		/*0x00be */ "",
		/*0x00be */ "",
		/*0x00c0 */ "",
		/*0x00c1 */ "",
		/*0x00c2 */ "",
		/*0x00c3 */ "",
		/*0x00c4 */ "",
		/*0x00c5 */ "",
		/*0x00c6 */ "",
		/*0x00c7 */ "",
		/*0x00c8 */ "",
		/*0x00c9 */ "",
		/*0x00ca */ "",
		/*0x00cb */ "",
		/*0x00cc */ "",
		/*0x00cd */ "",
		/*0x00ce */ "",
		/*0x00cf */ "",
		/*0x00d0 */ "",
		/*0x00d1 */ "",
		/*0x00d2 */ "",
		/*0x00d3 */ "",
		/*0x00d4 */ "",
		/*0x00d5 */ "",
		/*0x00d6 */ "",
		/*0x00d7 */ "",
		/*0x00d8 */ "",
		/*0x00d9 */ "",
		/*0x00da */ "",
		/*0x00db */ "",
		/*0x00dc */ "",
		/*0x00dd */ "",
		/*0x00de */ "",
		/*0x00de */ "",
		/*0x00q0 */ "",
		/*0x00e1 */ "",
		/*0x00e2 */ "",
		/*0x00e3 */ "",
		/*0x00e4 */ "",
		/*0x00e5 */ "",
		/*0x00e6 */ "",
		/*0x00e7 */ "",
		/*0x00e8 */ "",
		/*0x00e9 */ "",
		/*0x00ea */ "",
		/*0x00eb */ "",
		/*0x00ec */ "",
		/*0x00ed */ "",
		/*0x00ee */ "",
		/*0x00ee */ "",
		/*0x00f0 */ "",
		/*0x00f1 */ "",
		/*0x00f2 */ "",
		/*0x00f3 */ "",
		/*0x00f4 */ "",
		/*0x00f5 */ "",
		/*0x00f6 */ "",
		/*0x00f7 */ "",
		/*0x00f8 */ "",
		/*0x00f9 */ "",
		/*0x00fa */ "",
		/*0x00fb */ "",
		/*0x00fc */ "",
		/*0x00fd */ "",
		/*0x00fe */ "",
		/*0x00ff */ ""}
};
//---------------------------------------------------------------------------
//	PlayStation Executable file-types recognised by the emulator
//---------------------------------------------------------------------------
#define PSX_EXE     	1	//	Standard PlayStation Executable
#define CPE_EXE     	2	//	Compressed Executable
#define COFF_EXE    	3	//	COFF or ECOFF Executable
#define ELF_EXE		4	//	ELF Executable
#define INVALID_EXE 	5	//	Unknown, not recognised
//---------------------------------------------------------------------------
#define btoi(b)		((b)/16*10 + (b)%16)		/* BCD to u_char */
#define itob(i)		((i)/10*16 + (i)%10)		/* u_char to BCD */
//---------------------------------------------------------------------------
__declspec( dllimport )	void	__fastcall	COP_LW	( int z, int rt, DWORD Value );
__declspec( dllimport ) void	__fastcall	COP_LD	( int z, int rt, __int64 Value );
//---------------------------------------------------------------------------
#define	METHOD_ID( method )	fprintf( stderr, "Method ID : %s :\n", (char*)method );
//---------------------------------------------------------------------------
//	The actual messages that ALL of the plugins recognise, and most of the
// emulator itself. If you do not want to implement the feature that is entirely
// up to you.
//---------------------------------------------------------------------------
#define MSG_CREATE		WM_USER + 0x08000000	//	Create the emulator, but do not initialise
#define MSG_INITIALISE		WM_USER + 0x08000001	//	Initialise the emulator ready for execution
#define MSG_CONFIGURE		WM_USER + 0x08000002	//	Display the Configure dialog box, if available
#define MSG_TRANSFER_READ	WM_USER + 0x08000003	//	Transfer program to emulator, do not execute though
#define MSG_TRANSFER_WRITE	WM_USER + 0x08000004	//	Transfer information from emulator
#define MSG_EXECUTE		WM_USER	+ 0x08000005	//	Execute emulator if program loaded else load then run a cd
#define MSG_RESET		WM_USER + 0x08000006	//	Reset emulator, removes programs from ram
#define MSG_SUSPEND		WM_USER + 0x08000007	//	Suspend execution of emulator
#define	MSG_RESUME		WM_USER + 0x08000008	//	Resume execution of emulator
#define MSG_TERMINATE		WM_USER + 0x08000009	//	Terminate
#define MSG_DEBUGGER		WM_USER + 0x88008800	//	Debugger wants attention, or emulator needs to call the debugger
//---------------------------------------------------------------------------
//	Error Codes
//---------------------------------------------------------------------------
#define	ENO_ERROR		0			//	No errors detected
#define EFAIL_CREATE_DEVICE	-1			//	win 32 device creation failure
#define EFAIL_CREATE_EVENT	-2			//	win 32 event creation failure
//---------------------------------------------------------------------------
// Basic types
//---------------------------------------------------------------------------
#if defined(__WIN32__)					//	WIN32 specific
//---------------------------------------------------------------------------
		typedef __int8  	int8;		//	Signed 8 bit value
		typedef __int16 	int16;		//	Signed 16 bit value
		typedef __int32 	int32;          //      Signed 32 bit value
		typedef __int64 	int64;          //      Signed 64 bit value
//---------------------------------------------------------------------------
		typedef unsigned __int8  uint8;		//	Unsigned 8 bit value
		typedef unsigned __int16 uint16;	//	Unsigned 16 bit value
		typedef unsigned __int32 uint32;        //	Unsigned 32 bit value
		typedef unsigned __int64 uint64;        //	Unsigned 64 bit value
//---------------------------------------------------------------------------
#elif defined(__LINUX__)				//	Linux specific
//---------------------------------------------------------------------------
		typedef char 			int8;	//	As above but for linux
		typedef short 			int16;
		typedef int 			int32;
		typedef long long 		int64;
//---------------------------------------------------------------------------
		typedef unsigned char 		uint8;
		typedef unsigned short 		uint16;
		typedef unsigned int 		uint32;
		typedef unsigned long long 	uint64;
//---------------------------------------------------------------------------
		typedef int64 			__int64;
		typedef uint64 			__uint64;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
typedef struct _EXE_HEADER_ {		//	PlayStation executable header PSX_EXE
	unsigned char id[8];            //	id field
    	unsigned long text;             //	text (code) segment, file offset
    	unsigned long data;             //	data segment, file offset
    	unsigned long EntryPoint;	//	entry point             (PC-->)
    	unsigned long gp0;              //	base pointer 		(gp-->)
    	unsigned long t_addr;           //	code start		(vAddr)
    	unsigned long t_size;           //	code size
    	unsigned long d_addr;           //	data start		(vAddr)
    	unsigned long d_size;           //	data size
    	unsigned long b_addr;           //	heap start		(data start+size)
    	unsigned long b_size;           //	heap size
    	unsigned long s_addr;		//	stack 			(sp-->)
    	unsigned long s_size;           //	stack size
    	unsigned long SavedSP;		//	Saved Stack Pointer	(<--SP)
    	unsigned long SavedFP;		//	Saved Frame Pointer	(<--FP)
    	unsigned long SavedGP;		//	Saved base pointer	(<--GP)
    	unsigned long SavedRA;		//	Saved Return Address ?	(<--PC)
    	unsigned long SavedS0;		//	Saved S0
	} EXE_HEADER, *PEXE_HEADER, *lpExecHeader;
//---------------------------------------------------------------------------
//!	Configuration Definition.
//!	This is from the actual configuration struct from PCSX, it has NOT been
//! modified yet but will undoubtably will need to be too as I have not stuck
//! to the code that is in pcsx2 that much
//---------------------------------------------------------------------------
typedef struct _emu_config_def_ {	//	Emulator DLL configuration settings

	unsigned	RAMImageFlag	: 1;	//	Is a copy of the RAM array in file ?
        unsigned	ROMImageFlag	: 1;	//		ROM
        unsigned	PPMImageFlag	: 1;	//		PPM
        unsigned	HWMImageFlag	: 1;	//		HWM
        unsigned	TLBImageFlag	: 1;	//		TLB

        unsigned	CP0ImageFlag	: 1;	//	CP0 state
        unsigned	CP1ImageFlag	: 1;	//	CP1 state
        unsigned	CP2ImageFlag	: 1;	//	CP2 state

        unsigned	MMUType		: 2;	//	What type of MMU are we using ? NONE, FIXED, TLB MIN, TLB FULL
        unsigned	PSXType		: 1;	//	PAL / NTSC
        unsigned 	Console		: 1;	//	Is I/O console available ?

        unsigned	InternalBIOS	: 1;	//	Are we using the internal BIOS class or an external image ?

        char		*lpszBIOSFileName;	//	if internal BIOS class is to be used this must be equal to NULL

	//char *Gpu;			//	GPU name, ie if the this file present load as dll for GPU/GTE support
	//char *Spu;         		//      SPU
	//char *Cdr;          		//      CD-ROM
	//char *Pad1;         		//      Pad 0
	//char *Pad2;         		//      Pad 1
	//char *Mcd1;         		//      Memory Card 0
	//char *Mcd2;         		//      Memory Card 1
	//char *Bios;         		//      BIOS

	//long Xa;			//	XA ( Mode 2 Data ie Movies )
	//long Sio;			//	SIO ( Serial I/O Controller ) NOT PADS!
	//long Mdec;		     	//	MDEC ( Memory decoder )
	//long PsxAuto;			//	resolution ?
	//long PsxType; 		// 	ntsc - 0 | pal - 1
	//long QKeys;			//	??
	//long Cdda;			//	??
	//long HLE;			//	Internal BIOS flag
	//long Cpu;			//	Compiled or Interpreted mode ?
	//long PsxOut;                  //	Enable console output flag
	//long SpuIrq;			//	SPUIRQ ( SPU IRQ present ) ?

	} EmulatorConfig, *LPEmulatorConfig;
//---------------------------------------------------------------------------
__declspec( dllimport ) EmulatorConfig 	Config;	//	Current configuration
//---------------------------------------------------------------------------
__declspec( dllimport )	char		CdromId[ 256 ];	//	Current discs' title if any
//---------------------------------------------------------------------------
//!	ELF FILE HEADER.
//---------------------------------------------------------------------------
typedef struct _external_filehdr_ {	//!	ELF/ ECOFF
	unsigned short	f_magic;	//!	magic number
	unsigned short	f_nscns;	//! 	number of sections
	unsigned long 	f_timdat;	//! 	time & date stamp
	unsigned long 	f_symptr;	//! 	file pointer to symtab
	unsigned long 	f_nsyms;	//! 	number of symtab entries
        unsigned short 	f_opthdr;	//! 	sizeof( optional hdr )
	unsigned short 	f_flags;	//! 	flags
} FILHDR, ELF_HEADER, *lpELF_HEADER;
//---------------------------------------------------------------------------
#define	FILHDR	struct external_filehdr
#define	FILHSZ	sizeof(FILHDR)
//---------------------------------------------------------------------------
//	Device definition, only a guess though
//---------------------------------------------------------------------------
typedef	struct	_psx_device_def_	{
	WORD	wMajor;
        WORD	wMinor;
	DWORD	dwFlags;
        DWORD	dwStatus;
        LPVOID	lpEvent;
        } DEVICE, *LPDEVICE;
//---------------------------------------------------------------------------
//!	PlayStation Directory Entry.
//!	This is the definition of the PC's FAT entry, (FAT File Allocation Table)
//! and as the PlayStation uses an ISO9660 format discs, with some modifications
//! this structure allows reading of the TOC and any files on the disc.
//---------------------------------------------------------------------------
typedef	struct	_psx_dirent_	{
	char			Name[ 20 ];	//!	Filename
        DWORD			dwAttr;		//!	Attributes
        DWORD			dwSize;		//!	Size of file in blocks or bytes?
        struct	_psx_dirent_	*lpNext;	//!	next entry in queue
        DWORD			dwHead;		//!	Head
        char			System[ 4 ];	//!	System identifier
        } PSX_DIRENT, *LPPSX_DIRENT;
//---------------------------------------------------------------------------
#define BIAS			4
#define PSXCLK			33868800	/* 33.8688 Mhz */
//---------------------------------------------------------------------------
#define PSX_NTSC		0
#define PSX_PAL			1
//---------------------------------------------------------------------------
//!	System Memory Sizes.
//---------------------------------------------------------------------------
#define PSX_RAMSIZE		2048000			//!	2MB system memory TOTAL
#define PSX_VIDSIZE		1024000			//!	1MB Video memory
#define PSX_AUDSIZE		512000			//!	512K Audio memory
#define PSX_CDSIZE		65535			//!	64K CD ROM memory
//---------------------------------------------------------------------------
__declspec( dllimport ) void	__fastcall	EMU_Create		( void );
__declspec( dllimport ) void	__fastcall	EMU_Initialise		( void );
__declspec( dllimport )	void	__fastcall	EmulatorConfigure	( void );
__declspec( dllimport )	void	__fastcall	EmulatorTransfer	( LPSTR lpszFileName );
__declspec( dllimport )	void	__fastcall	EmulatorExecute		( void );
__declspec( dllimport )	void	__fastcall	EmulatorDebug		( void );
//---------------------------------------------------------------------------
//	Load configuration
//---------------------------------------------------------------------------
__declspec( dllimport )	void	__fastcall	EmulatorLoadConfig	( LPSTR lpszConfigFile );
//---------------------------------------------------------------------------
//	Save configuration
//---------------------------------------------------------------------------
__declspec( dllimport )	void	__fastcall	EmulatorSaveConfig	( LPSTR lpszConfigFile );
//---------------------------------------------------------------------------
__declspec( dllimport ) void	__fastcall	BitDump( DWORD *lpData, DWORD BitCount );
//---------------------------------------------------------------------------
//
//!	The following is from PSEMU_Pro_Plugindefs, not mine, I just modified it
//!	for the needs of PSXIDE (KONDEV)
//
//!	PSEmu Plugin Developer Kit Header definition
//
//!	(C)1998 Vision Thing
//!	Modifications (C)2002 Liquid Silicon Developments
//
//!	This file can be used only to develop PSEmu Plugins
//!	Other usage is highly prohibited.
//
//!	Sorry! :-) I borrowed (pinched) it
//
//---------------------------------------------------------------------------
// 	IMPORTANT!!!
//---------------------------------------------------------------------------
// 	if you want to add return codes (any errors or warnings) just drop mail to
// 	plugin@psemu.com
//---------------------------------------------------------------------------
// 	header version
//---------------------------------------------------------------------------
#define _PPDK_HEADER_VERSION		3
#define DLL_HEADER_VERSION		1	//	Version of the DLL Emulator
//---------------------------------------------------------------------------
#define PLUGIN_VERSION			1
//---------------------------------------------------------------------------
// plugin type returned by PSEgetLibType (types can be merged if plugin is multi type!)
//---------------------------------------------------------------------------
#define PSE_LT_CDR			1	//	Plugin is a CDROM Driver
#define PSE_LT_GPU			2	//	Plugin is a GPU/GTE Driver
#define PSE_LT_SPU			4	//	Plugin is an SPU Driver
#define PSE_LT_PAD			8	//	Plugin is a PAD Driver
//---------------------------------------------------------------------------
// 	every function in DLL if completed sucessfully should return this value
//---------------------------------------------------------------------------
#define PSE_ERR_SUCCESS			0
//---------------------------------------------------------------------------
// 	undefined error but fatal one, that kills all functionality
//---------------------------------------------------------------------------
#define PSE_ERR_FATAL			-1
//---------------------------------------------------------------------------
// 	XXX_Init return values
// 	Those return values apply to all libraries
// 	currently obsolete - preserved for compatibilty
//---------------------------------------------------------------------------
// 	initialization went OK
//---------------------------------------------------------------------------
#define PSE_INIT_ERR_SUCCESS		PSE_ERR_SUCCESS
//---------------------------------------------------------------------------
// this driver is not configured
//---------------------------------------------------------------------------
#define PSE_INIT_ERR_NOTCONFIGURED	-2
//---------------------------------------------------------------------------
// this driver can not operate properly on this hardware or hardware is not detected
//---------------------------------------------------------------------------
#define PSE_INIT_ERR_NOHARDWARE		-3
//---------------------------------------------------------------------------
//	GPU PlugIn
//---------------------------------------------------------------------------
//  	GPU_Test return values
//---------------------------------------------------------------------------
// 	sucess, everything configured, and went OK.
//---------------------------------------------------------------------------
#define PSE_GPU_ERR_SUCCESS			PSE_ERR_SUCCESS
//---------------------------------------------------------------------------
// ERRORS
// this error might be returned as critical error but none of below
//---------------------------------------------------------------------------
#define PSE_GPU_ERR				-20
//---------------------------------------------------------------------------
// this driver is not configured
//---------------------------------------------------------------------------
#define PSE_GPU_ERR_NOTCONFIGURED		PSE_GPU_ERR - 1
//---------------------------------------------------------------------------
// this driver failed Init
//---------------------------------------------------------------------------
#define PSE_GPU_ERR_INIT			PSE_GPU_ERR - 2
//---------------------------------------------------------------------------
// WARNINGS
// this warning might be returned as undefined warning but allowing driver to continue
//---------------------------------------------------------------------------
#define PSE_GPU_WARN				20
//---------------------------------------------------------------------------
//	Here we need to define some structures to allow for the matching of the
// GPU/GTE capabilities with what the PC has to offer, ie Direct X or Open GL?
// Does the PC have a 3D accelerator, if so what can it do ?
//---------------------------------------------------------------------------
//  GPU_Query		- will be implemented soon
//---------------------------------------------------------------------------
typedef struct _cpu_info_ {
	unsigned long	flags;
	unsigned long	status;
	HWND		window;
	unsigned char 	reserved[100];
} gpuQueryS;
//---------------------------------------------------------------------------
// gpuQueryS.flags
// if driver can operate in both modes it must support GPU_changeMode();
// this driver can operate in fullscreen mode
//---------------------------------------------------------------------------
#define PSE_GPU_FLAGS_FULLSCREEN		1
//---------------------------------------------------------------------------
// this driver can operate in windowed mode
//---------------------------------------------------------------------------
#define PSE_GPU_FLAGS_WINDOWED			2
//---------------------------------------------------------------------------
// gpuQueryS.status
// this driver cannot operate in this windowed mode
//---------------------------------------------------------------------------
#define PSE_GPU_STATUS_WINDOWWRONG		1
//---------------------------------------------------------------------------
//  GPU_Query	End	- will be implemented in v2
//---------------------------------------------------------------------------
/*         CDR PlugIn          */
//---------------------------------------------------------------------------
//	CDR_Test return values
//---------------------------------------------------------------------------
// sucess, everything configured, and went OK.
//---------------------------------------------------------------------------
#define PSE_CDR_ERR_SUCCESS			PSE_ERR_SUCCESS
//---------------------------------------------------------------------------
// general failure (error undefined)
//---------------------------------------------------------------------------
#define PSE_CDR_ERR_FAILURE			-1
//---------------------------------------------------------------------------
// ERRORS
//---------------------------------------------------------------------------
#define PSE_CDR_ERR 				-40
//---------------------------------------------------------------------------
// this driver is not configured
//---------------------------------------------------------------------------
#define PSE_CDR_ERR_NOTCONFIGURED		PSE_CDR_ERR - 0
//---------------------------------------------------------------------------
// if this driver is unable to read data from medium
//---------------------------------------------------------------------------
#define PSE_CDR_ERR_NOREAD			PSE_CDR_ERR - 1
//---------------------------------------------------------------------------
// WARNINGS
//---------------------------------------------------------------------------
#define PSE_CDR_WARN 				40
//---------------------------------------------------------------------------
// if this driver emulates lame mode ie. can read only 2048 tracks and sector header is emulated
// this might happen to CDROMS that do not support RAW mode reading - surelly it will kill many games
//---------------------------------------------------------------------------
#define PSE_CDR_WARN_LAMECD			PSE_CDR_WARN + 0
//---------------------------------------------------------------------------
/*         SPU PlugIn          */
//---------------------------------------------------------------------------
// some info retricted (now!)
//---------------------------------------------------------------------------
// sucess, everything configured, and went OK.
//---------------------------------------------------------------------------
#define PSE_SPU_ERR_SUCCESS 			PSE_ERR_SUCCESS
//---------------------------------------------------------------------------
// ERRORS
// this error might be returned as critical error but none of below
//---------------------------------------------------------------------------
#define PSE_SPU_ERR				-60
//---------------------------------------------------------------------------
// this driver is not configured
//---------------------------------------------------------------------------
#define PSE_SPU_ERR_NOTCONFIGURED		PSE_SPU_ERR - 1
//---------------------------------------------------------------------------
// this driver failed Init
//---------------------------------------------------------------------------
#define PSE_SPU_ERR_INIT			PSE_SPU_ERR - 2
//---------------------------------------------------------------------------
// WARNINGS
// this warning might be returned as undefined warning but allowing driver to continue
//---------------------------------------------------------------------------
#define PSE_SPU_WARN				60
//---------------------------------------------------------------------------
/*         PAD PlugIn          */
//---------------------------------------------------------------------------
/*

  functions that must be exported from PAD Plugin

  long	PADinit(long flags);		// 	called only once when PSEmu Starts
  void	PADshutdown(void);		// 	called when PSEmu exits
  long	PADopen(PadInitS *);		// 	called when PSEmu is running program
  long	PADclose(void);
  long	PADconfigure(void);
  void  PADabout(void);
  long  PADtest(void);			// 	called from Configure Dialog and after PADopen();
  long	PADquery(void);

  long	PADreadPort1(PadDataS *);
  long	PADreadPort2(PadDataS *);

*/
//---------------------------------------------------------------------------
// PADquery responses (notice - values ORed)
// PSEmu will use them also in PADinit to tell Plugin which Ports will use
// notice that PSEmu will call PADinit and PADopen only once when they are from
// same plugin
//---------------------------------------------------------------------------
// might be used in port 1 (must support PADreadPort1() function)
//---------------------------------------------------------------------------
#define PSE_PAD_USE_PORT1			1
//---------------------------------------------------------------------------
// might be used in port 2 (must support PADreadPort2() function)
//---------------------------------------------------------------------------
#define PSE_PAD_USE_PORT2			2
//---------------------------------------------------------------------------
// MOUSE SCPH-1030
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_MOUSE			1
//---------------------------------------------------------------------------
// NEGCON - 16 button analog controller SLPH-00001
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_NEGCON			2
//---------------------------------------------------------------------------
// GUN CONTROLLER - gun controller SLPH-00014 from Konami
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_GUN			3
//---------------------------------------------------------------------------
// STANDARD PAD SCPH-1080, SCPH-1150
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_STANDARD			4
//---------------------------------------------------------------------------
// ANALOG JOYSTICK SCPH-1110
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_ANALOGJOY			5
//---------------------------------------------------------------------------
// GUNCON - gun controller SLPH-00034 from Namco
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_GUNCON			6
//---------------------------------------------------------------------------
// ANALOG CONTROLLER SCPH-1150
//---------------------------------------------------------------------------
#define PSE_PAD_TYPE_ANALOGPAD			7
//---------------------------------------------------------------------------
// sucess, everything configured, and went OK.
//---------------------------------------------------------------------------
#define PSE_PAD_ERR_SUCCESS			PSE_ERR_SUCCESS
//---------------------------------------------------------------------------
// general plugin failure (undefined error)
//---------------------------------------------------------------------------
#define PSE_PAD_ERR_FAILURE			-1
//---------------------------------------------------------------------------
// ERRORS
// this error might be returned as critical error but none of below
//---------------------------------------------------------------------------
#define PSE_PAD_ERR				-80
//---------------------------------------------------------------------------
// this driver is not configured
//---------------------------------------------------------------------------
#define PSE_PAD_ERR_NOTCONFIGURED		PSE_PAD_ERR - 1
//---------------------------------------------------------------------------
// this driver failed Init
//---------------------------------------------------------------------------
#define PSE_PAD_ERR_INIT			PSE_PAD_ERR - 2
//---------------------------------------------------------------------------
// WARNINGS
// this warning might be returned as undefined warning but allowing driver to continue
//---------------------------------------------------------------------------
#define PSE_PAD_WARN				80
//---------------------------------------------------------------------------
typedef struct
{
	// controler type - fill it withe predefined values above
	unsigned char controllerType;

	// status of buttons - every controller fills this field
	unsigned short buttonStatus;

	// for analog pad fill those next 4 bytes
	// values are analog in range 0-255 where 128 is center position
	unsigned char rightJoyX, rightJoyY, leftJoyX, leftJoyY;

	// for mouse fill those next 2 bytes
	// values are in range -128 - 127
	unsigned char moveX, moveY;

	unsigned char reserved[91];

} PadDataS;
//---------------------------------------------------------------------------
//extern  int  		SysInit();					// 	Init mem and plugins
//extern  void 		SysReset();					// 	Resets mem
//extern  void 		SysPrintf(char *fmt);				// 	Printf used by bios syscalls
//extern  void 		SysMessage(char *fmt, ...);			// 	Message used to print msg to users
//extern  void 		*SysLoadLibrary(char *lib);			// 	Loads Library
//extern  void 		*SysLoadSym(void *lib, char *sym);		// 	Loads Symbol from Library
//extern  char 		*SysLibError();					// 	Gets previous error loading sysbols
//extern  void 		SysCloseLibrary(void *lib);			// 	Closes Library
//extern  void 		SysUpdate();					// 	Called on VBlank (to update i.e. pads)
//extern  void 		SysRunGui();					// 	Returns to the Gui
//extern 	void 		SysClose();					// 	Close mem and plugins
//extern	unsigned char	self_sram[];					//	Presume this is a copy of the BIOS, ie its the internal HLE BIOS ?
//---------------------------------------------------------------------------
__declspec( dllimport )	int 	__fastcall	CheckCdrom	( void );
__declspec( dllimport )	int 	__fastcall	Load		( char *ExePath );
//---------------------------------------------------------------------------
__declspec( dllimport )	int	__fastcall	SaveState	( char *FileName );
__declspec( dllimport ) int 	__fastcall	LoadState	( char *FileName );
//---------------------------------------------------------------------------
//!	Note the order that these are included are important.
//---------------------------------------------------------------------------
//!	The sequence is as follows, it mirrors the creation, initialisation sequence
//! during emulator execution : Memory, HW, counters, dma, mdec, cdrom, decode xa,
//! spu, sio, cpu, cop2, bios.
//
//---------------------------------------------------------------------------
#ifndef MEMORYH
#include "Memory.h"		//	Allows for direct memory access, soon will
#endif				//	be the ability for 3-tier type virtual memory
//---------------------------------------------------------------------------
#ifndef HWH
#include "hw.h"
#endif
//---------------------------------------------------------------------------
#ifndef COUNTERSH
#include "Counters.h"		//	Root counters for system timing
#endif
//---------------------------------------------------------------------------
#ifndef DMAH
#include "DMA.h"		//	DMA (Direct Memory Access) for devices
#endif
//---------------------------------------------------------------------------
#ifndef MDECH
#include "Mdec.h"		//	MDEC (decoding for hardware layer) MPG/XA-CDPCM
#endif
//---------------------------------------------------------------------------
#ifndef CDROMH
#include "CDROM.h"		//	CDROM
#endif
//---------------------------------------------------------------------------
#ifndef DECODEXAH
#include "DecodeXA.h"		//	Decoding XA cdrom data
#endif
//---------------------------------------------------------------------------
#ifndef SPUH
#include "SPU.h"		//	SPU (Sound Processing Unit)
#endif
//---------------------------------------------------------------------------
#ifndef	SIOH
#include "Sio.h"		//	SIO (Serial I/O)
#endif
//---------------------------------------------------------------------------
#ifndef R3000AH
#include "R3000A.h"		//	R3000A CPU Emulation Layer MIPS32 ISA
#endif
//---------------------------------------------------------------------------
#ifndef GTEH
#include "GTE.h"		//	GTE (Graphics Transfrom Engine) (3D)
#endif				//	Also incorporates GPU (Graphics Processing Unit) (2D)
//---------------------------------------------------------------------------
#ifndef BIOSH
#include "BIOS.h"		//	BIOS
#endif
//---------------------------------------------------------------------------
#ifndef KernelH			//	Simple Level Kernel, based on linux
#include "Kernel.h"
#endif
//---------------------------------------------------------------------------
#ifndef OSCoreH			//	simple os core functionality
#include "OSCore.h"
#endif
//---------------------------------------------------------------------------
#ifndef PLUGINSH
#include "Plugins.h"		//	Extras
#endif
//---------------------------------------------------------------------------
#ifndef GLTextureH
#include "GLTexture.h"
#endif
//---------------------------------------------------------------------------
#endif		//	EMULATOR_DLL_H
//---------------------------------------------------------------------------
