//---------------------------------------------------------------------------
//
//	$FileName	::	GTE.cpp
//	$Author		::	Chris Brooklyn
//	$Purpose	::	Emulation of PlayStation GPU/GTE chips
//				Converts GPU / GTE commands to open gl,
//				or directx equivalent. Note that the GTE
//				is a fast 3d maths processor and the GPU
//				is the actual polygon rendering device
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "GTE.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//!	Using Direct X or Open GL.
//---------------------------------------------------------------------------
#undef USE_DIRECTX		//	NOT using Direct X/3D not supported yet
#define USE_OPENGL		//	Using Open GL
//---------------------------------------------------------------------------
//	Note, if you do set to using DIRECT X then the calls, at present, are
// mirrored to the Open GL equivalent otherwise is just ignored.
//---------------------------------------------------------------------------
//	To change to using Direct X, simply define USE_DIRECTX and undefine USE_OPENGL
//---------------------------------------------------------------------------
__declspec( dllexport )	GTE	*lpGTE = NULL;
__declspec( dllexport ) WORD	glNextTID = 256;	//	used in texture generation, first 255 are for fonts
static	FILE	*fGTELog = NULL;
//---------------------------------------------------------------------------
static DWORD	VSyncCounter = 0;
//---------------------------------------------------------------------------
#define	CLASSNAME	"GTE"
//---------------------------------------------------------------------------
//!	Windows Message Handler for GTE/GPU emulation
//
//!	This handles the messages that the emulator as a whole broadcasts using
//! the Windows Message Subsystem. It responds only to the message it needs to
//! the actual emulation of the GTE/GPU functions are decoded at an instruction
//! level by the R3000A class and the GTE class
//---------------------------------------------------------------------------
LRESULT	CALLBACK	GTEWndProc( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch ( uMsg )	{

        	case ( WM_SIZE ) : {	//	User wants to resize the window
                	lpGTE->GLResize( HIWORD( lParam ), LOWORD( lParam ) );
                        return( 0 );
                }

                case ( WM_PAINT ) : {	//	Repaint the window
                	if ( ( lpGTE->lpOT != NULL ) && ( lpGTE->dwOTSize > 0 ) )
                        	lpGTE->GPU_DrawOT( lpGTE->lpOT );
                        else
                        	lpGTE->GPU_DrawSync();
                        return( 0 );
                }

                //case ( MSG_EXECUTE ) : {
                //	lpGTE->Execute();
                //	return( 0 );
                //}

                //case ( MSG_INITIALISE ) :
        	case ( GTE_INITIALISE ) : {	//	Initialise Message
                	//	Request to initialise the GTE, called when the
                	//	R3000 thread starts executing the BIOS

                	//	lParam = pointer to buffer containg int, int, int, bool
                        //	denoting x, y, c, f,
                        //	x = width, y = height, c = bits per pixel, f = fullscreen
                        //
                        //	note resolution is limited we should really check the available
                        //	modes that the video card is capable of to what the playstation
                        //	is capable of.

                        return( lpGTE->Initialise() );
                }

                //case ( MSG_TERMINATE ) :
                case ( GTE_SHUTDOWN ) :	{	//	Shutdown called when the R3000 thread terminates, on a PSX this is NEVER called
                	lpGTE->Terminate();
                        break;
                }

                case ( GTE_CONFIGURE ) : {	//	Configure the driver, ie resolution, colour depth, full screen mode, 3d accelerators ( PC ONLY )
                	//lpGTE->Configure();
                	break;
                }

                case ( GTE_TEST ) : {		//	Test current configuration
                	break;
                }

                case ( GTE_ABOUT ) :	{	//	Display a dialog displaying the driver information
                	if ( lpGTE->GPUInfo.lpszGPUDriver )
                        	MessageBox( NULL, lpGTE->GPUInfo.lpszGPUDriver, CLASSNAME, MB_OK );
                	break;
                }

                //case ( MSG_RESUME ) :
                case ( GTE_OPEN ) :	{	//	open the gte, ie let it run as COP2
                				//	Note when not operating as COP2 the the GPU is in effect ie 2D only
                                                //	else running as GTE 2D/3D mode
                	if ( lpGTE->Suspended )
                        	lpGTE->Resume();
                        return( 0 );
                }

                //case ( MSG_SUSPEND ) :
                case ( GTE_CLOSE ) :	{	//	close the GTE, ie stop it running as COP2
                	lpGTE->Suspend();
                        break;
                }

                case ( GTE_READSTATUS ) : {	//	Read the current status of the GTE
                	break;
                }

                case ( GTE_READDATA ) : {	//	Read from the data port of the GTE
                	break;
                }

                case ( GTE_READDATAMEM ) : {	//	Read from the video memory of the GTE
                	break;
                }

                case ( GTE_WRITESTATUS ) : {	//	update the status of the GTE
                	break;
                }

                case ( GTE_WRITEDATA ) : {	//	write data to the GTE data port, ie send it some 3d vectors etc
                	break;
                }

                case ( GTE_WRITEDATAMEM ) : {	//	write data to the GTE video memory, ie give it a texture map, not drawing on the screen
                	break;
                }

                case ( GTE_DMACHAIN ) : {	//	Access and control of the DMA chain used by the GTE thread
                	break;
                }

                case ( GTE_KEYPRESSED ) : {	//	updated the keypressed buffer information
                	break;
                }

                case ( GTE_DISPLAYTEXT ) : {	//	output text in 2d on a 2d or 3d mode display
                	break;
                }

                case ( GTE_MAKESNAPSHOT ) : {	//	takes a copy of the video memory and saves to disk PC ONLY
                	break;
                }

                case ( GTE_GETSCREENPIC ) : {	//	copies the current screen into system memory
                	break;
                }

                case ( GTE_SHOWSCREENPIC ) : {	//	copy an image from system memory to the screen
                	break;
                }

                case ( GTE_UPDATELACE ) : {	//	VSync update, ie swap the buffers
                	VSyncCounter = 0;
                        if ( lpGTE->hDC )
                        	SwapBuffers( lpGTE->hDC );
                        return( 0 );
                }

                case ( GTE_SETFONTNAME ) : {	//	change the font used for outputting text only
                	break;
                }

                case ( GTE_SETFONTSIZE ) : {	//	change the font size only
                	break;
                }

                case ( GTE_GETFONTNAME ) : {	//	returns the current font name
                	break;
                }

                case ( GTE_GETFONTSIZE ) : {	//	returns the current font size
                	break;
                }
                default : {
                	return( DefWindowProc( wHandle, uMsg, wParam, lParam ) );
                }
        }

        return( 0 );
}
//---------------------------------------------------------------------------
__fastcall 	GTE :: GTE( bool CreateSuspended ) : TThread( CreateSuspended )
{
	fGTELog = fopen( "GTE.LOG", "wt" );
        if ( fGTELog == NULL ) {
        	fprintf( stderr, "Unable to create GTE logfile.\n" );
                fGTELog = fGTELog;
        }

	fprintf( fGTELog, "PlayStation GPU / GTE Emulation Layer Construction\n" );

	fActive = CreateSuspended;

        pDataPort = new BYTE[ 64 ];
        pCommandPort = new BYTE[ 64 ];

        ZeroMemory( pDataPort, 64 );
        ZeroMemory( pCommandPort, 64 );

        Buffer = 1;
        VSyncCounter = 0;

	//	Initialise thread
        hInstance = GetModuleHandle( NULL );	//	Get Module Handle, used later

        memset( &WndClass, 0, sizeof( WNDCLASSEX ) );
        memset( &dmScreenSettings, 0, sizeof( DEVMODE ) );
        memset( &WinRect, 0, sizeof( RECT ) );

        dwOTSize = 0;	//	Ordering Table size, null empty at present

        lpOT = NULL;

        //	Set Driver Information
        GPUInfo.lpszGPUDriver = CLASSNAME;
        GPUInfo.dwGPUVersion = 0x00010001;

        //	Register the GTE class with windows
        WndClass.cbSize = sizeof( WNDCLASSEX );
        WndClass.hInstance = hInstance;
        WndClass.lpfnWndProc = (WNDPROC) GTEWndProc;
        WndClass.lpszClassName = CLASSNAME;

        RegisterClassEx( &WndClass );

        //	Create Event Object
        hEvent = CreateEvent( NULL, false, false, "CP2" );	//	Create event handle auto-reset named 'CP2'
        if ( ( hEvent == NULL ) || ( hEvent == INVALID_HANDLE_VALUE ) )
        	throw "CP2 event creation failure.";

        //	Class registered, but still not active finish initialising the
        //	class ready for execution

        dwFPS = 0xFFFFFFFF;		//	Not defined yet

        iWidth = DEFAULT_XSIZE;
        iHeight = DEFAULT_YSIZE;
        iCDepth = DEFAULT_CDEPTH;
        fWindowed = true;		//	Default 640 x 256 x 15

        fPAL = true;			//	PAL emulation mode

        if ( !CreateSuspended )	{

                if ( GLInit( iWidth, iHeight, iCDepth, fWindowed ) < 0 )
                	throw "Unable to initialise GTE <--> Open GL Interface.";

                if ( !fActive )
                	fActive = true;

                Resume();
        }
}
//---------------------------------------------------------------------------
__fastcall	GTE :: ~GTE( void )
{
	if ( fActive )
        	GLShutDown();

        delete this;
}
//---------------------------------------------------------------------------
//	Generic initialisation method. Initialises Open Gl to a default state.
//---------------------------------------------------------------------------
int	__fastcall	GTE :: Initialise( void )
{
	return( GLInit( iWidth, iHeight, iCDepth, fWindowed ) );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: Terminate( void )
{
	GLShutDown();
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: Reset( void )
{
	Initialise();
}
//---------------------------------------------------------------------------
//	Emulated GPU Data Port
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GDataPort( void )
{	DWORD	Value = 0x00000000;

	// ....	Increment the data port output point, note increase by the size
        // of a read (32-bit)
        bdout += sizeof( DWORD );

        // .... Make sure it is in the valid buffer address space
        if ( bdout >= 64 )
        	bdout = 0;

        if ( bdout >= bdin )
        	bdout = bdin;

        Value = pDataPort[ bdout ];

	return( Value );
}
//---------------------------------------------------------------------------
//	Reads the status port of the GTE/GPU. Note just returns the commands
// given when written to the command port. Should really translate the Open GL
// status values to something that resembles what the GTE/GPU would be doing.
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GStatusPort( void )
{
	if ( bcout >= 64 )	bcout = 0;
        if ( bcout >= bcin )	bcout = bcin - 4;
        bcout += 4;
        return( (DWORD)pCommandPort[ ( bcout - 4 ) ] | 0x80000000 );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: SDataPort( DWORD Value )
{
	bdin += 4;
        if ( bdin >= 64 )	bdin = 0;
        if ( bdin >= bdout )	bdin = bdout;
	pDataPort[ bdin ] = Value;
}
//---------------------------------------------------------------------------
//	Emulates the Command Port for the GTE / GPU thread.
//---------------------------------------------------------------------------
void	__fastcall	GTE :: SCommandPort( DWORD Value )
{	BYTE	bCmd	= ( Value >> 24 );
	// ...	What do we want to do, send command or just data ?

        switch ( bCmd ) {

        	//	0XE1		Draw Mode
		case ( GPUCMD_PACKET_DMODE_SETTING ) : {	//	0xE0xxxxxx

                	// ...	Sets the drawing mode, NTSC/PAL, interlaced/non-interlaced
                        BYTE	bWidth = ( Value & 0x000000C0 );	//	Bits 6 and 7
                        BYTE	bHeight = ( Value & 0x00000004 );	//	Bit 2

                        switch ( bWidth ) {
                        	case ( 0 ) : {	//	240
                                	iWidth = 240;
                                	break;
                                }
                                case ( 1 ) : {	//	320
                                	iWidth = 320;
                                	break;
                                }
                                case ( 2 ) : {	//	384
                                	iWidth = 384;
                                	break;
                                }
                                case ( 3 ) : {	//	512
                                	iWidth = 512;
                                	break;
                                }
                                case ( 4 ) : {	//	640
                                	iWidth = 640;
                                	break;
                                }
                                default : {
                                	iWidth = 320;
                                        break;
                                }
                        }

                        switch ( bHeight ) {
                        	case ( 0 ) : {	//	240
                                	iHeight = 240;
                                	break;
                                }
                                case ( 1 ) : {	//	480
                                	iHeight = 480;
                                	break;
                                }
                                default : {
                                	iHeight = 240;
                                        break;
                                }
                        }

                	break;
                }

		case ( GPUCMD_PACKET_TXTRE_SETTING ) : {	//0XE2		Texture Area
                	break;
                }
		case ( GPUCMD_PACKET_DAREA_TLEFT   ) : {	//0XE3		Drawing Area Top Left
                	break;
                }

		case ( GPUCMD_PACKET_DAREA_BRIGHT  ) : {	//0XE4		Drawing Area Bottom Right
                	break;
                }
		case ( GPUCMD_PACKET_DRAW_OFFSET   ) : {	//0XE5		Offset in VRAM where FrameBuffer starts
                	break;
                }
		case ( GPUCMD_PACKET_DRAW_MASK	   ) : {	//0XE6		Drawing mask, transparency, and, or etc
                	break;
                }

                case ( GPUCMD_PACKET_CLEAR	   ) : {	//0x01		Clear cache, frame-buffer, OT
                	break;
                }
		case ( GPUCMD_PACKET_FBUFF_RECT	   ) : {	//0x02		frame buffer rectangle drawing, ie bit-blit operation
                	break;
                }
		case ( GPUCMD_PACKET_MOVEIN	   ) : {	//0x80		Move image in framebuffer
                	break;
                }
		case ( GPUCMD_PACKET_MOVETO	   ) : {	//0xA0		move image to frame buffer from RAM
                	break;
                }
		case ( GPUCMD_PACKET_MOVEFROM	   ) : {	//0xC0		move image from frame buffer to RAM
                	break;
                }

		//	0x20		3 point polygon monochrome ( single colour NOT black and white )
		case ( GPUCMD_PACKET_MPOLY3	   ) : {
                // .... Process the data port buffer into a flat shaded triangle primitive
                	GPU_SortTriangle( lpOT, (float)(Value & 0x0000FFFF), (LPTRI)&pDataPort[ bdin ] );
                	break;
                }
		case ( GPUCMD_PACKET_TPOLY3 	   ) : {	//0x24		3 point polygon textured
                	break;
                }
		case ( GPUCMD_PACKET_MPOLY4	   ) : {	//0x28		4 point polygon monochrome
                	break;
                }
		case ( GPUCMD_PACKET_TPOLY4	   ) : {	//0x2C		4 point polygon textured
                	break;
                }
		case ( GPUCMD_PACKET_SPOLY3	   ) : {	//0x30		3 point polygon shaded
                	break;
                }
		case ( GPUCMD_PACKET_STPOLY3	   ) : {	//0x34		3 point polygon textured shaded
                	break;
                }
		case ( GPUCMD_PACKET_SPOLY4	   ) : {	//0x38		4 point polygon shaded
                	break;
                }
		case ( GPUCMD_PACKET_STPOLY4	   ) : {	//0x3C		4 point polygon textured shaded
                	break;
                }
		case ( GPUCMD_PACKET_MLINE	   ) : {	//0x40		monochrome line
                	break;
                }
		case ( GPUCMD_PACKET_MPOLYL	   ) : {	//0x48		monochrome poly line
                	break;
                }
		case ( GPUCMD_PACKET_SLINE	   ) : {	//0x50		shaded line
                	break;
                }
		case ( GPUCMD_PACKET_SPOLYL	   ) : {	//0x58		shaded polyline
                	break;
                }
		case ( GPUCMD_PACKET_RECT	   ) : {	//0x60		rectangle
                	break;
                }
		case ( GPUCMD_PACKET_SPRITE	   ) : {	//0x64		sprite
                	break;
                }
		case ( GPUCMD_PACKET_DOT	   ) : {	//0x68		dot
                	break;
                }
		case ( GPUCMD_PACKET_RECT8	   ) : {	//0x70		8*8 rectangle
                	break;
                }
		case ( GPUCMD_PACKET_SPRITE8	   ) : {	//0x74		8*8 sprite
                	break;
                }
		case ( GPUCMD_PACKET_RECT16	   ) : {	//0x78		16*16 rectangle
                	break;
                }
		case ( GPUCMD_PACKET_SPRITE16	   ) : {	//0x7C		16*16 sprite
                	break;
                }
        }
}
//---------------------------------------------------------------------------
//	Terminates the Interface between GTE and Open GL. Note does not delete
// textures
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLShutDown( void )
{
        if ( fActive )
        	Suspend();

	//	If fullscreen then switch batch to windows desktop

        if ( !fWindowed )	{	//	if not windowed ie fullscreen
        //	Change back to the windows desktop
        	ChangeDisplaySettings( NULL, 0 );
        	ShowCursor( true );
        }

        //	Release and delete the Open GL Rendering Context
	if ( glRC )	{
        	wglMakeCurrent( NULL, NULL );	//	Release
                wglDeleteContext( glRC );	//	Delete
        	glRC = NULL;
        }

        //	Release the Windows Device Context
        if ( hDC )	{
        	ReleaseDC( wHandle, hDC );
                hDC = NULL;
        }

        //	Release the Window
        if ( wHandle )	{
        	DestroyWindow( wHandle );
                wHandle = NULL;
        }

        //	Unregister GTE Thread
        if ( hInstance )	{
        	UnregisterClass( CLASSNAME, hInstance );
                hInstance = NULL;
        }

        fActive = false;
}
//---------------------------------------------------------------------------
//	Initialises the GTE / Open GL Interface
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GLInit( int xSize, int ySize, int CDepth, bool FullScreen )
{
	if ( iWidth != xSize )		iWidth = xSize;
        if ( iHeight != ySize )		iHeight = ySize;
        if ( iCDepth != CDepth )	iCDepth = CDepth;

        if ( FullScreen )	fWindowed = false;
        else			fWindowed = true;

        WinRect.left = (long)0;
        WinRect.top = (long)0;
        WinRect.right = (long)iWidth;
        WinRect.bottom = (long)iHeight;

        if ( !fWindowed )	{	//	Running in FullScreen mode ?
        	dmScreenSettings.dmSize = sizeof( DEVMODE );
                dmScreenSettings.dmPelsWidth = iWidth;
                dmScreenSettings.dmPelsHeight = iHeight;
                dmScreenSettings.dmBitsPerPel = iCDepth;
                dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
                if ( ChangeDisplaySettings( &dmScreenSettings, CDS_FULLSCREEN ) != DISP_CHANGE_SUCCESSFUL )	{
                	int mrCode = MessageBox( NULL, "Unable to change to fullscreen mode at this resolution, use window?", CLASSNAME, MB_YESNO );
                	if ( mrCode == ID_NO )	{
                        	GLShutDown();
                                return( -2 );
                        }
                        fWindowed = true;
                }
        }

        if ( !fWindowed )	{
        	dwExStyle = WS_EX_APPWINDOW;
                dwStyle = WS_POPUP;
                ShowCursor( false );
        } else {
        	dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
                dwStyle = WS_OVERLAPPEDWINDOW;
        }

        AdjustWindowRectEx( &WinRect, dwStyle, false, dwExStyle );

        if ( !fWindowed )	{
        	wHandle = CreateWindowEx( dwExStyle, CLASSNAME, "GTE", WS_SYSMENU | WS_BORDER | WS_CAPTION | WS_VISIBLE,
                			0, 0, WinRect.right - WinRect.left, WinRect.bottom - WinRect.top,
                                        NULL, NULL, hInstance, NULL );
                if ( !wHandle )	{
                	GLShutDown();
                        return( -3 );
                }
        } else	{
        	wHandle = CreateWindowEx( dwExStyle, CLASSNAME, "GTE", WS_POPUP | WS_VISIBLE,
                			0, 0, WinRect.right - WinRect.left, WinRect.bottom - WinRect.top,
                                        NULL, NULL, hInstance, NULL );
                if ( !wHandle )	{
                	GLShutDown();
                        return( -3 );
                }
        }

        static	PIXELFORMATDESCRIPTOR	pfd = {
        	sizeof( PIXELFORMATDESCRIPTOR ),
                1, PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
                PFD_TYPE_COLORINDEX/*PFD_TYPE_RGBA*/, iCDepth, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                16, 0, 0, PFD_MAIN_PLANE, 0, 0, 0, 0 };

        // ....	If running in 16/24-bit mode then PFD_TYPE_COLORINDEX should be
        // PFD_TYPE_RGBA
        if ( ( iCDepth == 16 ) || ( iCDepth == 24 ) )	{ //	16 or 24-bit mode
	        // ....	needs to be changed then
                pfd.iPixelType = PFD_TYPE_RGBA;
        }	// ....	Otherwise running 4/8-bit colour lookup

        hDC = GetDC( wHandle );
        if ( !hDC )	{
        	GLShutDown();
                return( -4 );
        }

        //	Is pixel format supported ?
        PixelFormat = ChoosePixelFormat( hDC, &pfd );
        if ( !PixelFormat )	{
        	GLShutDown();
                return( -5 );
        }

        //	and can be set ?
        if ( !SetPixelFormat( hDC, PixelFormat, &pfd ) )	{
        	GLShutDown();
                return( -6 );
        }

        //	create render context for open gl
        glRC = wglCreateContext( hDC );
        if ( !glRC )	{
        	GLShutDown();
                return( -7 );
        }

        //	and activate
        if ( !wglMakeCurrent( hDC, glRC ) )	{
        	GLShutDown();
                return( -8 );
        }

        //	Windows. Activate and set to top-most active focus window
        ShowWindow( wHandle, SW_SHOW );
        SetForegroundWindow( wHandle );
        SetFocus( wHandle );

        //	Open GL specific initialisation
        switch ( iCDepth ) {	//	What colour depth are we running at?

        	case ( 4 ) :	//	4-bit or
                case ( 8 ) : {	//	8-bit colour mode
                	glDisable( GL_ALPHA_TEST );
                        glDisable( GL_BLEND );
                        break;
                }

                case ( 15 ) :	//	15-bit or
                case ( 16 ) :   //	16-bit colour mode
                default : {	//	also default colour depth
                	glDisable( GL_ALPHA_TEST );
                	glEnable( GL_BLEND );
                        break;
                }

                case ( 24 ) :	//	24-bit or
                case ( 32 ) : {	//	32-bit colour mode, although psx does not support 32-bit mode
                	glEnable( GL_ALPHA_TEST );
                        glEnable( GL_BLEND );
                        break;
                }
        }

        //	Enable texture mapping but do not actual load any textures
        // the game will do that, and then (and only then) we transfer to
        // open gl

        glEnable( GL_TEXTURE_2D );			//	Enable texture mapping
        if ( ( iCDepth == 16 ) || ( iCDepth == 24 ) )	//	16/24-bit Shading Model ?
        	glShadeModel( GL_SMOOTH );		//	Smooth, more accurate and used by psx in 24bit mode only
        else	//	4/8-bit
        	glShadeModel( GL_FLAT );		//	flat, speed only

        glEnable( GL_DEPTH_TEST );			//	Depth clipping
        glDepthFunc( GL_LEQUAL );			//	Less or equal, same as PlayStation
        // ie render if less than or equal to Regs.Data.otz (FAR)
        // Regs.Control.zsf4 (NEAR), this I think is wrong, further checking will be done

        glDepthRange( Regs.Control.zsf4, Regs.Data.otz );

        //	Enable Open GL specific 'hints'

        glHint( GL_FOG_HINT, GL_FASTEST );			//	Select fog generation type, ie try to get vertex processing not pixel processing

        glHint( GL_LINE_SMOOTH_HINT, GL_FASTEST );              //	select fastest method of smoothing for lines

        glHint( GL_POINT_SMOOTH_HINT, GL_FASTEST );		//	Again for points

        glHint( GL_POLYGON_SMOOTH_HINT, GL_FASTEST );		//	 and for polygons

        glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST );	//	and for perspective correction calculations

        //	Set blending function type
        glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

        //	Set fog
        glHint( GL_FOG, GL_FASTEST );
        glFogf( GL_FOG_DENSITY, 0.0f );		//	no density fog
        glFogi( GL_FOG_MODE, GL_LINEAR );	// 	linear mapping
        glFogi( GL_FOG_START, Regs.Data.otz );	//	near fog value
        glFogi( GL_FOG_END, Regs.Control.zsf4 );//	far fog value

        if ( ( iCDepth == 16 ) || ( iCDepth == 24 ) )
        	glFogiv( GL_FOG_COLOR, (int*)&Regs.Data.rgb );
        else
        	glFogi( GL_FOG_INDEX, 0 );

        glEnable( GL_FOG );	//	and enable it

        //	Set Lights 0, 1 and 2, colour. Note that Open GL supports
        // setting the AMBIENT, DIFFUSE AND SPECULAR colours, whether the
        // PlayStation supports all of these is unknown at present, so I
        // am only setting the AMBIENT values from the data stored in the
        // CP2 Regs Data rgb0, rgb1, and rgb2 colour structures

        //	Set Colour for light 0
        glLightiv( GL_LIGHT0, GL_AMBIENT, (int*)&Regs.Data.rgb0 );
        //	and enable
        glEnable( GL_LIGHT0 );
        //	again for light 1
        glLightiv( GL_LIGHT1, GL_AMBIENT, (int*)&Regs.Data.rgb1 );
        //	enable
        glEnable( GL_LIGHT1 );
        //	and light 2
        glLightiv( GL_LIGHT2, GL_AMBIENT, (int*)&Regs.Data.rgb2 );
        //	enable it
        glEnable( GL_LIGHT2 );

        //	Now enable Open GL lighting, in general
        glEnable( GL_LIGHTING );

        //	Initialise font subsystem, this is the only thing that loads a
        // texture with being prompted to do so by the game that is under emulation
        GLFontInit( FONTNAME, GLFONT_TYPE_DEFAULT );

        GLResize( iWidth, iHeight );

        fActive = true;

        return( fActive );
}
//---------------------------------------------------------------------------
//	Open GL SPECIFIC	::	Reconfigures Open GL to match the values
//					stored in the CP2 Control and Data Registers
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLUpdate( void )
{
        //	Foreground colour
        glColor3f( Regs.Control.rfc, Regs.Control.bfc, Regs.Control.gfc );

        //	Clear colour black
        glClearColor( Regs.Control.rbk, Regs.Control.bbk, Regs.Control.gbk, 0.0f );

        //	Reset Camera mode
        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();

        //	recalculate aspect ratio
        gluPerspective( /*Field Of View*/ 45.0f, /* Width / Height */ ( GLfloat ) iWidth / ( GLfloat ) iHeight,
        	/* Z Clipping value NEAR */ Regs.Control.zsf4, /* Current z-pos / 4 */
                /* Z Clipping value FAR  */ Regs.Data.otz ); /* Current z-pos */
                // this means that objects in the range > Regs.Control.zsf4 & < Regs.Data.otz will be displayed

        //	switch back to model mode
        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity();

        //	set lights
        //	set clipping
}
//---------------------------------------------------------------------------
//	Initialise Open GL font sub-system
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLFontInit( char *font_name, int font_type )
{	/*TEXTURE 	font;
	float 		cx=0;
	float 		cy=0;
	int 		loop;

	font.LoadTGA("Art/font.tga", GL_LINEAR, GL_LINEAR);
	fontTexture=font.ID;

	base=glGenLists(95);						//Creating a display list that can hold 95
	glBindTexture(GL_TEXTURE_2D, fontTexture);	//Bind our font texture
	for(loop=0; loop<95; loop++)				//Loop through the display list
		{
		cx=float(loop%16)/16.0f;				//X position of current character
		cy=float(loop/16)/8.0f;					//Y position of current character

		glNewList(base+loop,GL_COMPILE);		//Start building a list
			glBegin(GL_TRIANGLE_STRIP);			//Use a quad for each character
				//Top right (tex/vertex) coordinates
				glTexCoord2f( (cx+0.0830f), (1.0f-cy));
				glVertex2i(16,16);
				//Top left (tex/vertex) coordinates
				glTexCoord2f( (cx),			(1.0f-cy));
				glVertex2i(0,16);
				//Bottom right (tex/vertex) coordinates
				glTexCoord2f( (cx+0.0830f), (1.0f-cy-0.120f));
				glVertex2i(16,0);
				//Bottom left (tex/vertex) coordinates
				glTexCoord2f( (cx),			(1.0f-cy-0.120f));
				glVertex2i(0,0);
			glEnd();							//Done Building Our Quad (Character)
			glTranslated(10,0,0);				//Move To The Right Of The Character
		glEndList();							//Done Building The Display List
		}															// Loop Until All 256 Are Built

        */

        //	Set font type, name and size, in general
        GLFont.glFontType = font_type;
        GLFont.lpszFontName = new char[ strlen( font_name ) ];
        strncpy( GLFont.lpszFontName, font_name, strlen( font_name ) );
        GLFont.glFontSize = GLFONT_SIZE_DEFAULT;

        // ....	now initialise the font sub-system
        if ( GLFont.glFontType & GLFONT_TYPE_2D ) {		//	initialise 2D font sub-system

        	//	Transfer font name
        	GLFont.lpszFont2DName = new char[ strlen( GLFont.lpszFontName ) ];
                strncpy( GLFont.lpszFont2DName, GLFont.lpszFontName, strlen( GLFont.lpszFontName ) );

                //	obtain window handle to font if possible
                GLFont.hFont2D = CreateFont( -( GLFont.glFontSize * 2 ), 0, 0, 0, FW_NORMAL, 0, 0,
                			0, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
                                        DEFAULT_QUALITY, DEFAULT_PITCH, GLFont.lpszFontName );

                //	set open gl font list base
                GLFont.glFont2D = glGenLists( 96 );
                GLFont.glFontBase = GLFont.glFont2D;

                //	Select the font object
                SelectObject( hDC, GLFont.hFont2D );

                //	and build the list
                wglUseFontBitmaps( hDC, 32, 96, GLFont.glFontBase );


        } else {
        	if ( GLFont.glFontType & GLFONT_TYPE_3D ) {	//	else 3D font sub-system
	        	//	Transfer font name
	        	GLFont.lpszFont3DName = new char[ strlen( GLFont.lpszFontName ) ];
	                strncpy( GLFont.lpszFont3DName, GLFont.lpszFontName, strlen( GLFont.lpszFontName ) );
	                //	obtain a windows handle to font if possible
	                GLFont.hFont3D = CreateFont( -( GLFont.glFontSize * 2 ), 0, 0, 0, FW_NORMAL, 0, 0,
	                			0, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
	                                        DEFAULT_QUALITY, DEFAULT_PITCH, GLFont.lpszFontName );
	                //	set open gl font list base
	                GLFont.glFont3D = glGenLists( 96 );
	                GLFont.glFontBase = GLFont.glFont3D;
	                //	Select the font object
	                SelectObject( hDC, GLFont.hFont3D );
	                //	and build the list
	                wglUseFontBitmaps( hDC, 32, 96, GLFont.glFontBase );
	        } else {
                	// .... Otherwise bad font
                	GLFont.glFontBase = NULL;
                        GLFont.lpszFontName = NULL;
                        GLFont.glFontSize = 0;
                        GLFont.glFontType = 0;
	        }
        }

}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLFontShutDown( void )
{

	if ( GLFont.glFont2D ) {	//	Delete 2D font information
        	if ( ( GLFont.hFont2D != NULL ) && ( GLFont.hFont2D != INVALID_HANDLE_VALUE ) )
                	DeleteObject( GLFont.hFont2D );
                if ( GLFont.lpszFont2DName != NULL )
                	delete []GLFont.lpszFont2DName;
                glDeleteLists( GLFont.glFont2D, 96 );
        }

        if ( GLFont.glFont3D ) {	//	and again for 3D font information
        	if ( ( GLFont.hFont3D != NULL ) && ( GLFont.hFont3D != INVALID_HANDLE_VALUE ) )
                	DeleteObject( GLFont.hFont3D );
                if ( GLFont.lpszFont3DName != NULL )
                	delete []GLFont.lpszFont3DName;
                glDeleteLists( GLFont.glFont3D, 96 );
        }

        GLFont.hFont2D = NULL;
        GLFont.hFont3D = NULL;
        GLFont.glFont2D = NULL;
        GLFont.glFont3D = NULL;

        // ....	now delete any general information
        if ( GLFont.lpszFontName != NULL )
        	delete []GLFont.lpszFontName;

        GLFont.lpszFontName = NULL;
        GLFont.glFontBase = NULL;
        GLFont.glFontSize = 0;
        GLFont.glFontType = 0;
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLFontSetFontName( char *font_name )
{
	if ( GLFont.lpszFontName != font_name )	{
        	GLFontShutDown();
                GLFontInit( GLFont.lpszFontName, GLFont.glFontType );
        }
}
//---------------------------------------------------------------------------
char*	__fastcall	GTE :: GLFontGetFontName( void )
{
        return( GLFont.lpszFontName );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLFontSetFontSize( int font_size )
{
	if ( GLFont.glFontSize != font_size ) {
        	GLFontShutDown();	//	shut down the font system
                GLFontInit( GLFont.lpszFontName, GLFont.glFontType );
        }
}
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GLFontGetFontSize( void )
{
        return( GLFont.glFontSize );
}
//---------------------------------------------------------------------------
//	Resizes the GL view makes sure the perspective is correct when resizing
//	the window, usually only called when the driver is in windowed mode
//	and the user resizes the display, or when switching from windowed to
//	fullscreen or resolution change
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GLResize( int xSize, int ySize )
{
	if ( ySize == 0 )							//	Prevent a divide by zero (bad)
		ySize = 1;							//	Making height equal one

        if ( iWidth != xSize )	iWidth = xSize;
        if ( iHeight != ySize ) iHeight = ySize;

	glViewport( 0, 0, iWidth, iHeight );					//	Reset the current viewport

	glMatrixMode( GL_PROJECTION );						//	Select the projection matrix
	glLoadIdentity();							//	Reset the projection matrix

	//	Calculate the aspect ratio of the window
        gluPerspective( 45.0f, ( GLfloat ) iWidth / ( GLfloat ) iHeight, Regs.Data.otz, 100.0f );

	glMatrixMode( GL_MODELVIEW );						//	Select the modelview matrix
	glLoadIdentity();							//	Reset The modelview matrix
}
//---------------------------------------------------------------------------
//!	METHOD		::	GLCompileOrderingTable.
//!	Compiles the Ordering Table into an Open GL compiled list, for quicker
//! rendering. Hopefully.
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GLCompileOrderingTable( LPGsOT pOT )
{
	if ( pOT )	return( false );
        else 		return( true );
}
//---------------------------------------------------------------------------
//
//	DIRECT X SPECIFIC SECTION
//
//---------------------------------------------------------------------------
int	__fastcall	GTE :: DXInit			( int xSize, int ySize, int CDepth, bool FullScreen )
{
#ifdef	USE_DIRECTX
	return( false );
#endif
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXShutDown		( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXFontInit		( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXFontInit		( char *font_name, int font_size )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXFontShutDown		( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXResize		( int xSize, int ySize )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXFontSetFontName	( char *font_name )
{
}
//---------------------------------------------------------------------------
char*	__fastcall	GTE :: DXFontGetFontName	( void )
{
	return( NULL );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: DXFontSetFontSize	( int font_size )
{
}
//---------------------------------------------------------------------------
int	__fastcall	GTE :: DXFontGetFontSize	( void )
{
	return( 0 );
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//
//	INTERFACE SECTION
//
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//	Sets the display to PAL or NTSC, depending on the fPALFlag
//	ie if true run as PAL,
//	if false run as NTSC
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SetPAL		( bool fPALFlag )	//	Set display to emulate PAL else default to NTSC
{
	/*

0000000000000468 <SetVideoMode>:
     468:	3c050000 	lui	$a1,0x0				468: R_MIPS_HI16	.data
     46c:	8ca50c40 	lw	$a1,3136($a1)			46c: R_MIPS_LO16	.data
     470:	3c010000 	lui	$at,0x0				470: R_MIPS_HI16	.data
     474:	ac240c40 	sw	$a0,3136($at)			474: R_MIPS_LO16	.data
     478:	28840002 	slti	$a0,$a0,2
     47c:	1480000a 	bnez	$a0,4a8 <SetVideoMode+40>
     480:	00000000 	nop
     484:	3c03bfc8 	lui	$v1,0xbfc8
     488:	8063ff52 	lb	$v1,-174($v1)
     48c:	24020045 	li	$v0,69
     490:	3c010000 	lui	$at,0x0      			490: R_MIPS_HI16	.data
     494:	14620004 	bne	$v1,$v0,4a8 <SetVideoMode+40>
     498:	ac200c40 	sw	$zero,3136($at)                 498: R_MIPS_LO16	.data
     49c:	24020001 	li	$v0,1
     4a0:	3c010000 	lui	$at,0x0                         4a0: R_MIPS_HI16	.data
     4a4:	ac220c40 	sw	$v0,3136($at)                   4a4: R_MIPS_LO16	.data
     4a8:	03e00008 	jr	$ra
     4ac:	00a01021 	move	$v0,$a1

	*/
}
//---------------------------------------------------------------------------
//	Reset the GPU to a known mode state, and return previous state.
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GPU_Reset		( int Mode )
{
	if ( Mode && GPUMODE_PAL )	{	//	Switch to PAL mode
        	return( GPUInfo.CurrentMode );
        }

        if ( Mode && GPUMODE_NTSC )	{	//	Switch to NTSC mode
        	return( GPUInfo.CurrentMode );
        }

	return( 0 );
}
//---------------------------------------------------------------------------
//	We pinched the interlaced flag to determine whether or not to run in
//	windowed mode or not. Note that this is supposed to initialise only the
//	2d section of COP2, however Open GL IS primarily a 3D api, but can do some 2d
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GPU_Init			(
		int xSize,		//	Width of display area
                int ySize,		//	Height of display area
                bool fInterlaced,	//	must be set is running 480 or 512 height
                bool f24Bit )		//	using dithered mode (24-bit or 16-bit)
{
	if ( fActive )	{
                GLShutDown();	//	Shutdown gl interface
                fActive = false;
        }

        if ( iWidth != xSize )	iWidth = xSize;

        if ( iHeight != ySize )	iHeight = ySize;

        if ( fWindowed != fInterlaced )	fWindowed = fInterlaced;

        if ( f24Bit )	iCDepth = 24;	//	24-bit depth wanted ?
        else iCDepth = 15;

        if ( ( iHeight == 480 ) || ( iHeight == 512 ) || ( fInterlaced ) )
        	fInterlacedFlag = true;
        else
        	fInterlacedFlag = false;

        return( GLInit( iWidth, iHeight, iCDepth, fWindowed ) );
}
//---------------------------------------------------------------------------
//	Define the display buffers start locations.
//	Does not do anything
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_DefDispBuff		( int x0, int y0, int x1, int y1 )
{
}
//---------------------------------------------------------------------------
//	Get Active Buffer
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GPU_GetActiveBuff	( void )
{
	return( Buffer );
}
//---------------------------------------------------------------------------
//	Returns 0 if VSYNC in progress else waits and returns 1
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GPU_VSync		( void )
{
	if ( VSyncCounter != 0 ) {
        	dwFPS = ( VSyncCounter / 50 );
        	do {
                	;
                } while ( VSyncCounter > 0 );
                return( 1 );
        } else	return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_EnableDisplay	( bool fEnable )
{
	if ( fActive != fEnable )	{
        	if ( ( !fEnable ) && ( fActive ) )	{	//	Stop ?
                	Suspend();
                }
                if ( ( fEnable ) && ( !fActive ) )	{	//	Start ?
                	Resume();
                }
                fActive = fEnable;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_FlipDisplay		( void )
{
	//if ( fActive )	{		//	Only swap buffers if any rendering is required
        	if ( Buffer == 1 )
                	Buffer = 3;
                else if ( Buffer == 3 )
                	Buffer = 1;
        	SwapBuffers( hDC );
//        }
}
//---------------------------------------------------------------------------
LPTIMCLUT	__fastcall	GTE :: GPU_TIMGetClut		( LPTIMHEADER lpTim )
{	LPTIMCLUT	lpClut;

	if ( lpTim == NULL )
        	return( NULL );

        if ( lpTim->id != 0x00000010 )	{	//	Not a TIM image
        	return( NULL );
        }

        lpClut = new TIMCLUT;

        if ( lpClut == NULL )
        	return( NULL );

        ZeroMemory( lpClut, sizeof( TIMCLUT ) );

        return( lpClut );
}
//---------------------------------------------------------------------------
LPTIMIMAGE	__fastcall	GTE :: GPU_TIMGetImage		( LPTIMHEADER lpTim )
{
	return( NULL );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_LoadImageData	( RECT *Area, void *Data )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SetCmdWorkSpace	( void *pBuffer )
{
	if ( pBuffer == NULL )
        	return;

	//pDataPort = (void*)pBuffer;
}
//---------------------------------------------------------------------------
//	Generic sorting, actually just transfer the current z-values to Open GL
// then gets Open GL to sort them for use and do all the clipping
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GsSortOt( LPGsOT lpOT, unsigned long offset, unsigned long pos )
{       LPGsOTTAG	lpTAG = NULL;
	if ( lpOT == NULL )
        	return;

        lpOT->Offset = offset;
        lpOT->Position = pos;

        lpTAG = lpOT->lpHead;

        if ( lpTAG == NULL )
        	return;

        // ... cycle through this list and resort the polygons if needed
        //do {
        //        lpTAG = (LPGsOTTAG)lpTAG->pNext;
        //} while ( lpTAG->pNext != NULL );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_ClearOT		( unsigned int offset, unsigned int point, LPGsOT lpOT )
{
}
//---------------------------------------------------------------------------
//	merge a box fill, bit blit, into the ordering table at the z-value stored
// in the parameter 'pos'
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortBoxFill		( LPGsOT lpOT, unsigned long pos, LPBOXFILL lpBoxFill )
{       LPGsOTTAG	lpTAG = NULL;

	if ( ( lpOT == NULL ) || ( lpBoxFill == NULL ) )
        	return;

	if ( lpOT->lpHead == NULL )
        	return;

        //	quick z-value check
        if ( pos > lpOT->Position )
        	return;

        lpTAG = lpOT->lpHead;

        // ...	find end.
        while ( lpTAG->pNext != NULL )
        	lpTAG = (LPGsOTTAG)lpTAG->pNext;

        // ...	now at end of list, add this operation
        lpTAG->pNext = (unsigned)new BOXFILL;
        lpTAG->Length = ( sizeof( GsOTTAG ) + sizeof( BOXFILL ) );

        ZeroMemory( (void*)lpTAG->pNext, sizeof( BOXFILL ) );

        // ...	now copy into list
        memcpy( (void*)lpTAG->pNext, lpBoxFill, sizeof( BOXFILL ) );

        // ...	and sort
        GsSortOt( lpOT, lpOT->Offset, pos );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortClear		( LPGsOT lpOT, unsigned long Index, LPCOLOUR lpColour )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortLine		( LPGsOT lpOT, unsigned long pos, LPLINE lpLine )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortCiLine		( LPGsOT lpOT, unsigned long pos, LPCILINE lpCiLine )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortTriangle		( LPGsOT lpOT, unsigned long pos, LPTRI lpTri )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortCiTriangle	( LPGsOT lpOT, unsigned long pos, LPCITRI lpCiTri )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortTexTriangle	( LPGsOT lpOT, unsigned long pos, LPTEXTRI lpTexTi )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortCiTexTriangle	( LPGsOT lpOT, unsigned long pos, LPCITEXTRI lpCiTexTri )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortQuad		( LPGsOT lpOT, unsigned long pos, LPQUAD lpQuad )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortCiQuad		( LPGsOT lpOT, unsigned long pos, LPCIQUAD lpCiQuad )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortTexQuad		( LPGsOT lpOT, unsigned long pos, LPTEXQUAD lpTexQuad )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_SortCiTexQuad	( LPGsOT lpOT, unsigned long pos, LPCITEXQUAD lpCiTexQuad )
{
}
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GPU_CalcClutID		( int x, int y )
{
	return( (int)GetPixel( hDC, x, y ) );	//	returns the pixel colour at the position
}
//---------------------------------------------------------------------------
//	CLUT MODES 4, 8, 16, 24-bit colour translation
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GPU_CalcTexturePage	( int x, int y, int ClutMode )
{
	return( 0 );
}
//---------------------------------------------------------------------------
//	This draw sthe Ordering Table, well actually calls a lot of pre-compiled
//	open gl drawing lists. All textures, data is loaded in ram and compiled
//	into a GsOTTAG (Open GL Compiled Command List), provides for a lot faster
//	drawing.
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GPU_DrawOT		( LPGsOT lpOrderingTable )
{       LPGsOTTAG	lpTag = NULL;

	if ( lpOrderingTable == NULL )
        	return;

        glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
	GLPrintf( 0, 0, "HELLO!" );

        //	Transfer attention to the new ordering table
        lpOT = lpOrderingTable;

        lpTag = lpOT->lpHead;

        do {

        	glBegin( GL_QUADS );
                	glVertex3f( 0.0f, 0.0f, -1.0f );
                        glVertex3f( 1.0f, 0.0f, -1.0f );
                        glVertex3f( 1.0f, 1.0f, -1.0f );
                        glVertex3f( 0.0f, 1.0f, -1.0f );
                glEnd();

        	switch ( pCommandPort[ bcout++ ] ) {
			case ( GPUCMD_PACKET_MPOLY3   ) : {
                        //	3 point polygon monochrome (single colour)
                        LPTRI	lpPoly = (LPTRI)(lpTag + sizeof( GsOT ) );

                        	glColor3b( lpPoly->color.r, lpPoly->color.g, lpPoly->color.b );

                                //	draw polygon
                                glBegin( GL_TRIANGLES );
                                	glVertex3i( lpPoly->vert0.x, lpPoly->vert0.y, lpPoly->vert0.z );
                                        glVertex3i( lpPoly->vert1.x, lpPoly->vert1.y, lpPoly->vert1.z );
                                        glVertex3i( lpPoly->vert2.x, lpPoly->vert2.y, lpPoly->vert2.z );
                                	//	Finish this polygon, but do NOT flush the gl queue
                                glEnd();

                        	break;
                        }
			case ( GPUCMD_PACKET_TPOLY3   ) : {		//	3 point polygon textured
                        LPTEXTRI	lpPoly = (LPTEXTRI)(lpTag + sizeof( GsOT ) );

                        	break;
                        }
			case ( GPUCMD_PACKET_MPOLY4   ) : {		//	4 point polygon monochrome
                        	break;
                        }
			case ( GPUCMD_PACKET_TPOLY4   ) : {		//	4 point polygon textured
                        	break;
                        }
			case ( GPUCMD_PACKET_SPOLY3   ) : {		//	3 point polygon shaded
                        	break;
                        }
			case ( GPUCMD_PACKET_STPOLY3  ) : {		//	3 point polygon textured shaded
                        	break;
                        }
			case ( GPUCMD_PACKET_SPOLY4   ) : {		//	4 point polygon shaded
                        	break;
                        }
			case ( GPUCMD_PACKET_STPOLY4  ) : {		//	4 point polygon textured shaded
                        	break;
                        }
			case ( GPUCMD_PACKET_MLINE    ) : {		//	monochrome line
                        	break;
                        }
			case ( GPUCMD_PACKET_MPOLYL   ) : {		//	monochrome poly line
                        	break;
                        }
			case ( GPUCMD_PACKET_SLINE    ) : {		//	shaded line
                        	break;
                        }
			case ( GPUCMD_PACKET_SPOLYL   ) : {		//	shaded polyline
                        	break;
                        }
			case ( GPUCMD_PACKET_RECT     ) : {		//	rectangle
                        	break;
                        }
			case ( GPUCMD_PACKET_SPRITE   ) : {		//	sprite
                        	break;
                        }
			case ( GPUCMD_PACKET_DOT      ) : {		//	dot
                        	break;
                        }
			case ( GPUCMD_PACKET_RECT8    ) : {		//	8*8 rectangle
                        	break;
                        }
			case ( GPUCMD_PACKET_SPRITE8  ) : {		//	8*8 sprite
                        	break;
                        }
			case ( GPUCMD_PACKET_RECT16   ) : {		//	16*16 rectangle
                        	break;
                        }
			case ( GPUCMD_PACKET_SPRITE16 ) : {		//	16*16 sprite
                        	break;
                        }
                }

                if ( bcout >= bcin )
                	bcout = bcin;

                if ( bcout >= 64 )
                	bcout = 0;

                if ( bcin >= 64 )
                	bcin = 0;

                lpTag += ( lpTag->Length + sizeof( GsOT ) );

        } while ( lpTag->pNext != NULL );

        //	now end and flush the Open GL rendering pipeline
	glEnd();	//	Force all drawing to stop
	glFlush();	//	and be flushed through the pipeline
}
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GPU_DrawSync		( void )
{
	if ( lpOT != NULL )
        	GPU_DrawOT( lpOT );
        dwFPS = ( VSyncCounter / 50 );
        VSyncCounter++;
	return( dwFPS );
}
//---------------------------------------------------------------------------
//	Initialise a transfer of a bitmap between either main memory and the
// video ram or the video ram to video ram, or even video ram to main memory
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GPU_LoadImage		( LPRECT lpRect, void *pData )
{       int	XStart, XEnd;
	int	YStart, YEnd;
        int	Width, Height;

	if ( ( pData == NULL ) || ( lpRect == NULL ) )
        	return( NULL );	//	Bad data

        // ...	Now what area are we looking at in the video memory, texture,
        // enviroment, or display area ?

        //	Width
        XStart = lpRect->left;
        XEnd = lpRect->right;
        Width = ( XEnd - XStart );

        //	Height
        YStart = lpRect->top;
        YEnd = lpRect->bottom;
        Height = ( YEnd - YStart );

        // ...	Should really initiate some form of DMA transfer to/from the GPU

	return( 0 );
}
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GPU_ClearImage		( LPRECT lpRect, LPCOLOUR lpColour )
{
	return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_Init			( int Width, int Height )
{
        GLInit( Width, Height, iCDepth, fWindowed );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_SetOffset		( int x, int y )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_SetFogColour		( unsigned char r, unsigned char g, unsigned char b )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_SV3Scale		( LPSVECTOR3D lpVectorOut, LPSVECTOR3D lpVectorIn, int Scale, int Quantity )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_SV3XForm		( LPSVECTOR3D lpVectorOut, LPSVECTOR3D lpVectorIn, LPMATRIX lpMatrix, int Quantity )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_V3XForm		( LPSVECTOR3D lpVectorOut, LPSVECTOR3D lpVectorIn, LPMATRIX lpMatrix, int Quantity )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_MatMul		( LPMATRIX lpMatrixOut, LPMATRIX lpMatrix1, LPMATRIX lpMatrix2 )
{
}
//---------------------------------------------------------------------------
//	Determines which is the front polgyon, ie 1, 2, 3 or 0 on equal
//---------------------------------------------------------------------------
int	__fastcall	GTE :: GTE_IsPolyFront		( LPSVECTOR3D lpVector1, LPSVECTOR3D lpVector2, LPSVECTOR3D lpVector3 )
{
	if ( ( lpVector1->z == lpVector2->z ) && ( lpVector1->z == lpVector3->z ) &&
        	( lpVector2->z == lpVector3->z ) )
                	return( 0 );
        return( MIN( lpVector1->z, MIN( lpVector2->z, lpVector3->z ) ) );
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_ParallelProject( LPVECTOR3D lpVertOut, LPVECTOR3D lpVectIn, DWORD dwQuantity )
{
}
//---------------------------------------------------------------------------
void	__fastcall	GTE :: GTE_MatrixSetIdentity(MATRIX *matrix)
{
}
//---------------------------------------------------------------------------
DWORD		__fastcall	GTE :: GTE_AverageZ3		( unsigned long v1, unsigned long v2, unsigned long v3 )
{
	return( ( v1 + v2 + v3 ) / 3 );
}
//---------------------------------------------------------------------------
DWORD		__fastcall	GTE :: GTE_AverageZ4		( unsigned long v1, unsigned long v2, unsigned long v3, unsigned long v4 )
{
	return( ( v1 + v2 + v3 + v4 ) / 4 );
}
//---------------------------------------------------------------------------
int		__fastcall	GTE :: GTE_Sin			( int Angle )
{
	return( (int)sin( (float)Angle ) );
}
//---------------------------------------------------------------------------
int		__fastcall	GTE :: GTE_Cos			( int Angle )
{
	return( (int)cos( (float)Angle ) );
}
//---------------------------------------------------------------------------
//!	METHOD		::	Execute	THREAD FUNCTION.
//!	This runs as a thread for decoding of COP2 opcodes, allows for faster
//!	emulation (hopefully) as it is not executed unless a specific COP2
//!	is encountered in the instruction pipeline
//---------------------------------------------------------------------------
void 	__fastcall 	GTE :: Execute( void )
{	DWORD		dwObject = 0;
	PCPUInstr	lpInstr = NULL;
	//---- Place thread code here ----
        while ( ( !Terminated ) || ( !Suspended ) ) {

        	//	Wait for CP0, or another part of the system to signal that
                // a CP2 instruction is needed to be decoded
                dwObject = WaitForSingleObject( hEvent, INFINITE );

                if ( dwObject == WAIT_TIMEOUT )	//	Timed out ?
                	Suspend();		//	go back to bed, zzz

        	GPU_DrawSync();		//	Make sure the display is updated correctly
        }
}
//---------------------------------------------------------------------------
//!	METHOD		::	GGenRegs.
//!	Returns the current value in the data register set specified by 'Index'.
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GGenRegs( int Index )
{	DWORD	dwValue = 0x00000000;

	// ...	Valid register?
        if ( ( Index < 0 ) || ( Index > 31 ) ) {
        	SystemMessage( "Bad COP2 General Register on read " + IntToStr( Index ) );
                return( 0 );
        }


        switch ( Index ) {

        	case ( 0 ) : {	//	v0.x and v0.y
                	dwValue = ( Regs.Data.v0.x << 16 ) | Regs.Data.v0.y;
                        return( dwValue );
                }

                case ( 1 ) : {	//	v0.z
                	return( Regs.Data.v0.z );
                }

                //	...	repeat for v1
                case ( 2 ) : {
                	dwValue = ( Regs.Data.v1.x << 16 ) | Regs.Data.v1.y;
                        return ( dwValue );
                }

                case ( 3 ) : {
                	return( Regs.Data.v1.z );
                }

                //	...	and v2
                case ( 4 ) : {
                	dwValue = ( Regs.Data.v2.x << 16 ) | Regs.Data.v2.y;
                        return( dwValue );
                }

                case ( 5 ) : {
                	return( Regs.Data.v2.z );
                }

                //	...	return colour, also last command value
                case ( 6 ) : {
                	dwValue = ( Regs.Data.rgb.a << 24 ) | ( Regs.Data.rgb.r << 16 ) |
                        	( Regs.Data.rgb.g << 8 ) | Regs.Data.rgb.b;
                	return( dwValue );
                }

                case ( 7 ) : {
                	return( Regs.Data.otz );
                }

                case ( 8 ) : {
                	return( Regs.Data.ir0 );
                }

                case ( 9 ) : {
                	return( Regs.Data.ir1 );
                }

                case ( 10 ) : {
                	return( Regs.Data.ir2 );
                }

                case ( 11 ) : {
                	return( Regs.Data.ir3 );
                }

                case ( 12 ) : {
                	dwValue = ( Regs.Data.sxy0.x << 16 ) | Regs.Data.sxy0.y;
                        return( dwValue );
                }

                case ( 13 ) : {
                	dwValue = ( Regs.Data.sxy1.x << 16 ) | Regs.Data.sxy1.y;
                        return( dwValue );
                }

                case ( 14 ) : {
                	dwValue = ( Regs.Data.sxy2.x << 16 ) | Regs.Data.sxy2.y;
                        return( dwValue );
                }

                case ( 15 ) : {
                	return( Regs.Data.sxyp );
                }

                case ( 16 ) : {
                	return( Regs.Data.sz0 );
                }

                case ( 17 ) : {
                	return( Regs.Data.sz1 );
                }

                case ( 18 ) : {
                	return( Regs.Data.sz2 );
                }

                case ( 19 ) : {
                	return( Regs.Data.sz3 );
                }

                case ( 20 ) : {
                	dwValue = ( Regs.Data.rgb0.a << 24 ) | ( Regs.Data.rgb0.r << 16 ) |
                        	( Regs.Data.rgb0.g << 8 ) | Regs.Data.rgb0.b;
                        return( dwValue );
                }

                case ( 21 ) : {
                	dwValue = ( Regs.Data.rgb1.a << 24 ) | ( Regs.Data.rgb1.r << 16 ) |
                        	( Regs.Data.rgb1.g << 8 ) | Regs.Data.rgb1.b;
                        return( dwValue );
                }

                case ( 22 ) : {
                	dwValue = ( Regs.Data.rgb2.a << 24 ) | ( Regs.Data.rgb2.r << 16 ) |
                        	( Regs.Data.rgb2.g << 8 ) | Regs.Data.rgb2.b;
                        return( dwValue );
                }

                case ( 23 ) : {
                	return( 0 );
                }

                case ( 24 ) : {
                	return( Regs.Data.mac0 );
                }

                case ( 25 ) : {
                	return( Regs.Data.mac1 );
                }

                case ( 26 ) : {
                	return( Regs.Data.mac2 );
                }

                case ( 27 ) : {
                	return( Regs.Data.mac3 );
                }

                case ( 28 ) : {
                	return( Regs.Data.irgb );
                }

                case ( 29 ) : {
                	return( Regs.Data.orgb );
                }

                case ( 30 ) : {
                	return( Regs.Data.lzcs );
                }

                case ( 31 ) : {
                	return( Regs.Data.lzcr );
                }

                default : {
                	return( 0 );
                }
        }
}
//---------------------------------------------------------------------------
//!	METHOD		::	GControlRegs.
//!	Returns the control register for coprocessor 2 as specified in 'Index'
//---------------------------------------------------------------------------
DWORD	__fastcall	GTE :: GCtlRegs( int Index )
{	DWORD	dwValue = 0x00000000;


	//!	Valid register ?
	if ( ( Index < 0 ) || ( Index > 31 ) )
        	return( 0 );	//!	No

        //!	Which particular register are we interested in?
        switch ( Index ) {

        	case ( 0 ) : {	//!	0 (ZERO)	Rotation Matrix
                	dwValue = ( Regs.Control.rMatrix.m[ 1 ][ 1 ] << 16 ) | Regs.Control.rMatrix.m[ 1 ][ 2 ];
                        return( dwValue );
                }

                case ( 1 ) : {	//!	1		Rotaion Matrix
                	dwValue = ( Regs.Control.rMatrix.m[ 1 ][ 3 ] << 16 ) | Regs.Control.rMatrix.m[ 2 ][ 1 ];
                        return( dwValue );
                }

                case ( 2 ) : {	//!	2		Rotation Matrix
                	dwValue = ( Regs.Control.rMatrix.m[ 2 ][ 2 ] << 16 ) | Regs.Control.rMatrix.m[ 2 ][ 3 ];
                        return( dwValue );
                }

                case ( 3 ) : {	//!	3		Rotation Matrix
                	dwValue = ( Regs.Control.rMatrix.m[ 3 ][ 1 ] << 16 ) | Regs.Control.rMatrix.m[ 3 ][ 2 ];
                        return( dwValue );
                }

                case ( 4 ) : {	//!	4		Rotation Matrix
                	return( Regs.Control.rMatrix.m[ 3 ][ 3 ] );
                }

                case ( 5 ) : {	//!	5		Translation Vector X
                	return( Regs.Control.trX );
                }

                case ( 6 ) : {	//!	6		Translation Vector Y
                	return( Regs.Control.trY );
                }

                case ( 7 ) : {	//!	7		Translation Vector Z
                	return( Regs.Control.trZ );
                }

                case ( 8 ) : {	//!	8		Light Direction Vector
                	dwValue = ( Regs.Control.lMatrix.m[ 1 ][ 1 ] << 16 ) | Regs.Control.lMatrix.m[ 1 ][ 2 ];
                        return( dwValue );
                }

                case ( 9 ) : {	//!	9		Light Direction Vector
                	dwValue = ( Regs.Control.lMatrix.m[ 1 ][ 3 ] << 16 ) || Regs.Control.lMatrix.m[ 2 ][ 1 ];
                        return( dwValue );
                }

                case ( 10 ) : {	//!	10		Light Direction Vector
                	dwValue = ( Regs.Control.lMatrix.m[ 2 ][ 2 ] << 16 ) | Regs.Control.lMatrix.m[ 2 ][ 3 ];
                        return( dwValue );
                }

                case ( 11 ) : {	//!	11		Light Direction Vector
                	dwValue = ( Regs.Control.lMatrix.m[ 3 ][ 1 ] << 16 ) | Regs.Control.lMatrix.m[ 3 ][ 2 ];
                        return( dwValue );
                }

                case ( 12 ) : {	//!	12		Light Direction Vector
                	return( Regs.Control.lMatrix.m[ 3 ][ 3 ] );
                }

                case ( 13 ) : {	//!	13		Background colour r
                	return( Regs.Control.rbk );
                }

                case ( 14 ) : {	//!	14		Background colour g
                	return( Regs.Control.gbk );
                }

                case ( 15 ) : {	//!	15		Background colour b
                	return( Regs.Control.bbk );
                }

                case ( 16 ) : {	//!	16		Light Source Colour
                	dwValue = ( Regs.Control.cMatrix.m[ 1 ][ 1 ] << 16 ) | Regs.Control.cMatrix.m[ 1 ][ 2 ];
                        return( dwValue );
                }

                case ( 17 ) : {	//!	17		Light Source Colour
                	dwValue = ( Regs.Control.cMatrix.m[ 1 ][ 3 ] << 16 ) | Regs.Control.cMatrix.m[ 2 ][ 1 ];
                        return( dwValue );
                }

                case ( 18 ) : {	//!	18		Light Source Colour
                	dwValue = ( Regs.Control.cMatrix.m[ 2 ][ 2 ] << 16 ) | Regs.Control.cMatrix.m[ 2 ][ 3 ];
                        return( dwValue );
                }

                case ( 19 ) : {	//!	19		Light Source Colour
                	dwValue = ( Regs.Control.cMatrix.m[ 3 ][ 1 ] << 16 ) | Regs.Control.cMatrix.m[ 3 ][ 2 ];
                        return( dwValue );
                }

                case ( 20 ) : {	//!	20		Light Source Colour
                	return( Regs.Control.cMatrix.m[ 3 ][ 3 ] );
                }

                case ( 21 ) : {	//!	21		Foreground Colour r
                	return( Regs.Control.rfc );
                }

                case ( 22 ) : {	//!	22		Foreground Colour g
                	return( Regs.Control.gfc );
                }

                case ( 23 ) : {	//!	23		Foreground Colour b
                	return( Regs.Control.bfc );
                }

                case ( 24 ) : {	//!	24		Video Ram display offset X
                	return( Regs.Control.ofx );
                }

                case ( 25 ) : {	//!	25			...	and Y
                	return( Regs.Control.ofy );
                }

                case ( 26 ) : {	//!	26		Video Display Height
                	return( Regs.Control.h );
                }

                case ( 27 ) : {	//!	27		Depth Parameter a ( Co-efficient )
                	return( Regs.Control.dqa );
                }

                case ( 28 ) : {	//!	28		Depth Parameter b ( Offset )
                	return( Regs.Control.dqb );
                }

                case ( 29 ) : {	//!	29		1/3 Z Average Scale Factor
                	return( Regs.Control.zsf3 );
                }

                case ( 30 ) : {	//!	30		1/4 Z Average Scale Factor
                	return( Regs.Control.zsf4 );
                }

                case ( 31 ) : {	//!	31		Flags
                	return( Regs.Control.flag );
                }


        	default : {
                	return( 0 );
                }
        }
}
//---------------------------------------------------------------------------
//!	METHOD		::	SGenRegs.
//!	Store a 'Value' into the Data Register Set specified by 'Index'
//---------------------------------------------------------------------------
void	__fastcall	GTE :: SGenRegs( int Index, DWORD Value )
{

	// ...	Is it a valid register ie 0-31 ?
	if ( ( Index < 0 ) || ( Index > 31 ) ) {
        	SystemMessage( "Bad COP2 General Register on write " + IntToStr( Index ) + " : " + IntToStr( Value ) );
        	return;	//	Bye bye, do nothing, no error reported. MIPS 32 does not allow for this type of error reporting
        }

        switch ( Index ) {	//	Which register are we writing to?

        	case ( 0 ) : {	//	0 (ZERO)	V0.X and V0.Y
                	Regs.Data.v0.x = HIWORD( Value );
                        Regs.Data.v0.y = LOWORD( Value );
                	return;
                }

                case ( 1 ) : {	//	1 		v0.z
                	Regs.Data.v0.z = HIWORD( Value );
                        return;
                }

                case ( 2 ) : {	//	2		v1.x and v1.y
                	Regs.Data.v1.x = HIWORD( Value );
                        Regs.Data.v1.y = LOWORD( Value );
                        return;
                }

                case ( 3 ) : {	//	3
                	Regs.Data.v1.z = HIWORD( Value );
                        return;
                }

                case ( 4 ) : {	//	4
                	Regs.Data.v2.x = HIWORD( Value );
                        Regs.Data.v2.y = LOWORD( Value );
                        return;
                }

                case ( 5 ) : {	//	5
                	Regs.Data.v2.z = HIWORD( Value );
                        return;
                }

                case ( 6 ) : {	//	6	rgb + gte instruction

                	Regs.Data.rgb.a = Value & GPUCMD_MASK;
                        Regs.Data.rgb.r = Value & 0x00FF0000;
                        Regs.Data.rgb.g = Value & 0x0000FF00;
                        Regs.Data.rgb.b = Value & 0x000000FF;
                	return;
                }

                case ( 7 ) : {	//	7	z component average value ( focal point z value ? )
                	Regs.Data.otz = Value;
                        return;
                }

                case ( 8 ) : {	//	8	ir0
                	Regs.Data.ir0 = Value;
                        return;
                }

                case ( 9 ) : { //	9	ir1
                	Regs.Data.ir1 = Value;
                        return;
                }

                case ( 10 ) : {	//	10	ir2
                	Regs.Data.ir2 = Value;
                        return;
                }

                case ( 11 ) : {	//	11	ir3
                	Regs.Data.ir3 = Value;
                        return;
                }

                case ( 12 ) : {	//	12	calc result 0 x,y	sxy0
                	Regs.Data.sxy0.x = HIWORD( Value );
                        Regs.Data.sxy0.y = LOWORD( Value );
                        return;
                }

                case ( 13 ) : {	//	13	sxy1
                	Regs.Data.sxy1.x = HIWORD( Value );
                        Regs.Data.sxy1.y = LOWORD( Value );
                        return;
                }

                case ( 14 ) : {	//	14	sxy2
                	Regs.Data.sxy2.x = HIWORD( Value );
                        Regs.Data.sxy2.y = LOWORD( Value );
                        return;
                }

                case ( 15 ) : {
                	Regs.Data.sxyp = Value;
                        return;
                }

                case ( 16 ) : {
                	Regs.Data.sz0 = Value;
                        return;
                }

                case ( 17 ) : {
                	Regs.Data.sz1 = Value;
                        return;
                }

                case ( 18 ) : {
                	Regs.Data.sz2 = Value;
                        return;
                }

                case ( 19 ) : {
                	Regs.Data.sz3 = Value;
                        return;
                }

                case ( 20 ) : {
                	Regs.Data.rgb0.r = Value & 0x00FF0000;
                        Regs.Data.rgb0.g = Value & 0x0000FF00;
                        Regs.Data.rgb0.b = Value & 0x000000FF;
                        return;
                }

                case ( 21 ) : {
                	Regs.Data.rgb1.r = Value & 0x00FF0000;
                        Regs.Data.rgb1.g = Value & 0x0000FF00;
                        Regs.Data.rgb1.b = Value & 0x000000FF;
                        return;
                }

                case ( 22 ) : {
                	Regs.Data.rgb2.r = Value & 0x00FF0000;
                        Regs.Data.rgb2.g = Value & 0x0000FF00;
                        Regs.Data.rgb2.b = Value & 0x000000FF;
                        return;
                }

                case ( 23 ) : {	//	RESERVED IGNORE THIS
                	return;
                }

                case ( 24 ) : {
                	Regs.Data.mac0 = Value;
                        return;
                }

                case ( 25 ) : {
                	Regs.Data.mac1 = Value;
                        return;
                }

                case ( 26 ) : {
                	Regs.Data.mac2 = Value;
                        return;
                }

                case ( 27 ) : {
                	Regs.Data.mac3 = Value;
                        return;
                }

                case ( 28 ) : {
                	Regs.Data.irgb = Value;
                        return;
                }

                case ( 29 ) : {
                	Regs.Data.orgb = Value;
                        return;
                }

                case ( 30 ) : {
                	Regs.Data.lzcs = Value;
                        return;
                }

                case ( 31 ) : {
                	Regs.Data.lzcr = Value;
                        return;
                }

                //! ...	Otheriwse ignore it
        }
}
//---------------------------------------------------------------------------
//!	METHOD		::	SControlRegs.
//!	Sets a control register 'Index' with 'Value'.
//---------------------------------------------------------------------------
void	__fastcall	GTE :: SCtlRegs( int Index, DWORD Value )
{
	if ( ( Index < 0 ) || ( Index > 31 ) )
        	return;

        //!  ...	Which register are we interested in ?
        switch ( Index ) {

        	case ( 0 ) : {	//!	0 	Rotaion Matrix
                	Regs.Control.rMatrix.m[ 1 ][ 1 ] = Value & 0xFFFF0000;
                        Regs.Control.rMatrix.m[ 1 ][ 2 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 1 ) : {	//!	1	Rotation Matrix
                	Regs.Control.rMatrix.m[ 1 ][ 3 ] = Value & 0xFFFF0000;
                        Regs.Control.rMatrix.m[ 2 ][ 1 ] = Value & 0x0000FFFF;
                	return;
                }

                case ( 2 ) : {	//!	2	Rotation Matrix
                	Regs.Control.rMatrix.m[ 2 ][ 2 ] = Value & 0xFFFF0000;
                        Regs.Control.rMatrix.m[ 2 ][ 3 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 3 ) : {	//!	3	Rotation Matrix
                	Regs.Control.rMatrix.m[ 3 ][ 1 ] = Value & 0xFFFF0000;
                        Regs.Control.rMatrix.m[ 3 ][ 2 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 4 ) : {	//!	4	Rotation Matrix
                	Regs.Control.rMatrix.m[ 3 ][ 3 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 5 ) : {	//!	5	Translation Vector X
                	Regs.Control.trX = Value;
                        return;
                }

                case ( 6 ) : {	//!	6	Translation Vector Y
                	Regs.Control.trY = Value;
                        return;
                }

                case ( 7 ) : {	//!	7	Translation Vector Z
                	Regs.Control.trZ = Value;
                        return;
                }

                case ( 8 ) : {	//!	8	Light Source Direction Vector
                	Regs.Control.lMatrix.m[ 1 ][ 1 ] = Value & 0xFFFF0000;
                        Regs.Control.lMatrix.m[ 1 ][ 2 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 9 ) : {	//!	9	Light Source Direction Vector
                	Regs.Control.lMatrix.m[ 1 ][ 3 ] = Value & 0xFFFF0000;
                        Regs.Control.lMatrix.m[ 2 ][ 1 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 10 ) : {	//!	10	Light Source Direction Vector
                	Regs.Control.lMatrix.m[ 2 ][ 2 ] = Value & 0xFFFF0000;
                        Regs.Control.lMatrix.m[ 2 ][ 3 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 11 ) : {	//!	11	Light Source Direction Vector
                	Regs.Control.lMatrix.m[ 3 ][ 1 ] = Value & 0xFFFF0000;
                        Regs.Control.lMatrix.m[ 3 ][ 2 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 12 ) : {	//!	12	Light Source Direction Vector
                	Regs.Control.lMatrix.m[ 3 ][ 3 ] = Value & 0x0000FFFF;
                }

                case ( 13 ) : {	//!	13	Background Colour r
                	Regs.Control.rbk = Value;
                        glColor3b( Regs.Control.rbk, Regs.Control.gbk, Regs.Control.bbk );
                        return;
                }

                case ( 14 ) : {	//!	14	Background Colour g
                	Regs.Control.gbk = Value;
                        glColor3b( Regs.Control.rbk, Regs.Control.gbk, Regs.Control.bbk );
                        return;
                }

                case ( 15 ) : {	//!	15	Background Colour b
                	Regs.Control.bbk = Value;
                        glColor3b( Regs.Control.rbk, Regs.Control.gbk, Regs.Control.bbk );
                        return;
                }

                case ( 16 ) : {	//!	16	Light Colour Matrix
                	Regs.Control.cMatrix.m[ 1 ][ 1 ] = Value & 0xFFFF0000;
                        Regs.Control.cMatrix.m[ 1 ][ 2 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 17 ) : {	//!	17	Light Colour Matrix
                	Regs.Control.cMatrix.m[ 1 ][ 3 ] = Value & 0xFFFF0000;
                        Regs.Control.cMatrix.m[ 2 ][ 1 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 18 ) : {	//!	18	Light Colour Matrix
                	Regs.Control.cMatrix.m[ 2 ][ 2 ] = Value & 0xFFFF0000;
                        Regs.Control.cMatrix.m[ 2 ][ 3 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 19 ) : {	//!	19	Light Colour Matrix
                	Regs.Control.cMatrix.m[ 3 ][ 1 ] = Value & 0xFFFF0000;
                        Regs.Control.cMatrix.m[ 3 ][ 2 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 20 ) : {	//!	20	Light Colour Marix
                	Regs.Control.cMatrix.m[ 3 ][ 3 ] = Value & 0x0000FFFF;
                        return;
                }

                case ( 21 ) : {	//!	21	Foreground Colour r
                	Regs.Control.rfc = Value;
                        return;
                }

                case ( 22 ) : {	//!	22	Foreground Colour g
                	Regs.Control.gfc = Value;
                        return;
                }

                case ( 23 ) : {	//!	23	Foreground Colour b
                	Regs.Control.bfc = Value;
                	return;
                }

                case ( 24 ) : {	//!	24	Screen Offset X
                	Regs.Control.ofx = Value;
                        return;
                }

                case ( 25 ) : {	//!	25	Screen Offset Y
                	Regs.Control.ofy = Value;
                        return;
                }

                case ( 26 ) : {	//!	26	Screen Height
                	Regs.Control.h = Value;
                        //iHeight = Regs.Control.h;
                        //GLResize( iWidth, iHeight );
                        return;
                }

                case ( 27 ) : {	//!	27	Depth Parameter a ( Co-efficient )
                	Regs.Control.dqa = Value;
                        return;
                }

                case ( 28 ) : {	//!	28	Depth Parameter b ( Offset )
                	Regs.Control.dqb = Value;
                        return;
                }

                case ( 29 ) : {	//!	29	1/3 Z Average Scale Factor
                	Regs.Control.zsf3 = Value;
                        return;
                }

                case ( 30 ) : {	//!	30	1/4 Z Average Scale Factor
                	Regs.Control.zsf4 = Value;
                        return;
                }

                case ( 31 ) : {	//!	31	flags
                	Regs.Control.flag |= Value;
                	return;
                }
        }
}
//---------------------------------------------------------------------------
//	Allows for text output to the video driver
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	GLPrintf( int x, int y, const char *fmt )
{	char		text[256];						// Holds Our String
	va_list		ap;							// Pointer To List Of Arguments
	if (fmt == NULL)							// If There's No Text
		return;								// Do Nothing
	va_start(ap, fmt);							// Parses The String For Variables
	    vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);								// Results Are Stored In Text
	glPushAttrib(GL_LIST_BIT);						// Pushes The Display List Bits
	glListBase(lpGTE->GLFont.glFontBase - 32);					// Sets The Base Character to 32
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);			// Draws The Display List Text
	glPopAttrib();								// Pops The Display List Bits
}
//---------------------------------------------------------------------------
#ifdef __WIN32__
#pragma warning(disable:4244)
#endif
/*/---------------------------------------------------------------------------
void cp2MFC2() {
//	//::MessageBox(0, "MFC2", "GTE", 0);
//	cpuRegs.GPR.r[_Rt_] =	cpuRegs.CP2D.r[_Rd_];

}
//---------------------------------------------------------------------------
void cp2CFC2() {
//	//::MessageBox(0, "CFC2", "GTE", 0);
//	cpuRegs.GPR.r[_Rt_] =	cpuRegs.CP2C.r[_Rd_];
}
//---------------------------------------------------------------------------
void cp2MTC2() {
//	//::MessageBox(0, "MTC2", "GTE", 0);
	switch(_Rd_) {
		case 8: case 9: case 10: case 11:
//			cpuRegs.CP2D.r[_Rd_] = (short)cpuRegs.GPR.r[_Rt_];
			break;
		case 30:
			{
				int a, b;

//				cpuRegs.CP2D.r[30] = cpuRegs.GPR.r[_Rt_];

//				a = cpuRegs.CP2D.r[30];
				if(a > 0)
				{
#if defined(__WIN32__)
//					__asm
//					{
//						mov eax, a;
//						bsr eax, eax;
//						mov b, eax;
//					}
#elif defined(__LINUX__)
//					__asm__ (
//						"movl %1, %%eax\n"
//						"bsrl %%eax, %%eax\n"
//						"movl %%eax, %0\n"
//					: "=m" (b)
//					: "m" (a)
//					);
#endif
					b = 31 - b;
				}
				else if(a < 0)
				{
#if defined (__WIN32__)
//					__asm
//					{
//						mov eax, a;
//						xor eax, 0xffffffff;
//						bsr eax, eax;
//						mov b, eax;
//					}
#elif defined (__LINUX__)
//					__asm__ (
//						"movl %1, %%eax\n"
//						"xorl %2, %%eax\n"
//						"bsrl %%eax, %%eax\n"
//						"movl %%eax, %0\n"
//					: "=m" (b)
//					: "m" (a), "long" (0xffffffff)
//					);
#endif
					b = 31 - b;
				}
				else
				{
					b = 32;
				}

//				cpuRegs.CP2D.r[31] = b;
			}
			break;

		default:
//			cpuRegs.CP2D.r[_Rd_] = cpuRegs.GPR.r[_Rt_];
			break;
	}

}
//---------------------------------------------------------------------------
void cp2CTC2() {
//	//::MessageBox(0, "CTC2", "GTE", 0);
//	cpuRegs.CP2C.r[_Rd_] = cpuRegs.GPR.r[_Rt_];
}
//---------------------------------------------------------------------------
/////LIMITATIONS AND OTHER STUFF************************************
//---------------------------------------------------------------------------
#define MAC2IR \
if (gteMAC1 < (signed long)(-32768)) {gteIR1=(signed long)(-32768); gteFLAG|=1<<24;} \
else \
if (gteMAC1 > (signed long)( 32767)) { gteIR1=(signed long)( 32767); gteFLAG|=1<<24;} \
else gteIR1=(signed long)gteMAC1; \
if (gteMAC2 < (signed long)(-32768)) { gteIR2=(signed long)(-32768); gteFLAG|=1<<23;} \
else \
if (gteMAC2 > (signed long)( 32767)) { gteIR2=(signed long)( 32767); gteFLAG|=1<<23;} \
else gteIR2=(signed long)gteMAC2; \
if (gteMAC3 < (signed long)(-32768)) { gteIR3=(signed long)(-32768); gteFLAG|=1<<22;} \
else \
if (gteMAC3 > (signed long)( 32767)) { gteIR3=(signed long)( 32767); gteFLAG|=1<<22;} \
else gteIR3=(signed long)gteMAC3;
//---------------------------------------------------------------------------
#define MAC2IR1() \
{           \
	if (gteMAC1 < (signed long)0) { gteIR1=(signed long)0; gteFLAG|=1<<24;}  \
	else if (gteMAC1 > (signed long)(32767)) { gteIR1=(signed long)(32767); gteFLAG|=1<<24;} \
	else gteIR1=(signed long)gteMAC1;                                                         \
	if (gteMAC2 < (signed long)0) { gteIR2=(signed long)0; gteFLAG|=1<<23;}      \
	else if (gteMAC2 > (signed long)(32767)) { gteIR2=(signed long)(32767); gteFLAG|=1<<23;}    \
	else gteIR2=(signed long)gteMAC2;                                                            \
	if (gteMAC3 < (signed long)0) { gteIR3=(signed long)0; gteFLAG|=1<<22;}         \
	else if (gteMAC3 > (signed long)(32767)) { gteIR3=(signed long)(32767); gteFLAG|=1<<22;}       \
	else gteIR3=(signed long)gteMAC3; \
}
//---------------------------------------------------------------------------
#define MAGIC  (((65536. * 65536. * 16) + (65536.*.5)) * 65536.)
//---------------------------------------------------------------------------
static __inline long float2int(double d)
{
	double dtemp = MAGIC + d;
	return (*(long *)&dtemp)-0x80000000;
}
//---------------------------------------------------------------------------
__inline double NC_OVERFLOW1(double x)
{
	if (x<-2147483648.0) {gteFLAG |= 1<<29;}
	else
	if (x> 2147483647.0) {gteFLAG |= 1<<26;}

	return x;
}
//---------------------------------------------------------------------------
__inline double NC_OVERFLOW2(double x)
{
	if (x<-2147483648.0) {gteFLAG |= 1<<28;}
	else
	if (x> 2147483647.0) {gteFLAG |= 1<<25;}

	return x;
}
//---------------------------------------------------------------------------
__inline double NC_OVERFLOW3(double x)
{
	if (x<-2147483648.0) {gteFLAG |= 1<<27;}
	else
	if (x> 2147483647.0) {gteFLAG |= 1<<24;}

	return x;
}
//---------------------------------------------------------------------------
__inline double NC_OVERFLOW4(double x)
{
	if (x<-2147483648.0) {gteFLAG |= 1<<16;}
	else
	if (x> 2147483647.0) {gteFLAG |= 1<<15;}

	return x;
}
//---------------------------------------------------------------------------
__inline double EDETEC1(double data)
{
	if (data<(double)-2147483647) {gteFLAG|=1<<30; return (double)-2147483647;}
	else
	if (data>(double) 2147483647) {gteFLAG|=1<<27; return (double) 2147483647;}

	else return data;
}
//---------------------------------------------------------------------------
__inline double EDETEC2(double data)
{
	if (data<(double)-2147483647) {gteFLAG|=1<<29; return (double)-2147483647;}
	else
	if (data>(double) 2147483647) {gteFLAG|=1<<26; return (double) 2147483647;}

	else return data;
}
//---------------------------------------------------------------------------
__inline double EDETEC3(double data)
{
	if (data<(double)-2147483647) {gteFLAG|=1<<28; return (double)-2147483647;}
	else
	if (data>(double) 2147483647) {gteFLAG|=1<<25; return (double) 2147483647;}

	else return data;
}
//---------------------------------------------------------------------------
__inline double EDETEC4(double data)
{
	if (data<(double)-2147483647) {gteFLAG|=1<<16; return (double)-2147483647;}
	else
	if (data>(double) 2147483647) {gteFLAG|=1<<15; return (double) 2147483647;}

	else return data;
}
//---------------------------------------------------------------------------
double LimitAU(double fraction,unsigned long bitIndex) {
	if (fraction <     0.0) { fraction =     0.0; gteFLAG |= (1<<bitIndex); }
	else
	if (fraction > 32767.0) { fraction = 32767.0; gteFLAG |= (1<<bitIndex); }

	return (fraction);
}
//---------------------------------------------------------------------------
double LimitAS(double fraction,unsigned long bitIndex) {
	if (fraction <-32768.0) { fraction =-32768.0; gteFLAG |= (1<<bitIndex); }
    else
	if (fraction > 32767.0) { fraction = 32767.0; gteFLAG |= (1<<bitIndex); }

	return (fraction);
}
//---------------------------------------------------------------------------
double LimitB (double fraction,unsigned long bitIndex) {
	if (fraction <     0.0) { fraction =     0.0; gteFLAG |= (1<<bitIndex); }
	else
	if (fraction >   255.0) { fraction =   255.0; gteFLAG |= (1<<bitIndex); }

	return (fraction);
}
//---------------------------------------------------------------------------
double LimitC (double fraction,unsigned long bitIndex) {
	if (fraction <     0.0) { fraction =     0.0; gteFLAG |= (1<<bitIndex); }
	else
	if (fraction > 65535.0) { fraction = 65535.0; gteFLAG |= (1<<bitIndex); }

	return (fraction);
}
//---------------------------------------------------------------------------
double LimitD (double fraction,unsigned long bitIndex) {
	if (fraction < -1024.0) { fraction = -1024.0; gteFLAG |= (1<<bitIndex); }
	else
	if (fraction >  1023.0) { fraction =  1023.0; gteFLAG |= (1<<bitIndex); }

	return (fraction);
}
//---------------------------------------------------------------------------
double LimitE (double fraction,unsigned long bitIndex) {
	if (fraction <     0.0) { fraction =     0.0; gteFLAG |= (1<<bitIndex); }
	else
	if (fraction >  1023.0) { fraction =  1023.0; gteFLAG |= (1<<bitIndex); }

	return (fraction);
}
//---------------------------------------------------------------------------
double LIMIT(double data,double MIN,double MAX,int FLAG)
{
	if (data<MIN) {gteFLAG|=1<<FLAG; return MIN;}
	else
	if (data>MAX) {gteFLAG|=1<<FLAG; return MAX;}

	else return data;
}
//---------------------------------------------------------------------------
double ALIMIT(double data,double MIN,double MAX)
{
	if (data<MIN) return MIN;
	else
	if (data>MAX) return MAX;

	else return data;
}
//---------------------------------------------------------------------------
double OLIMIT(double data)
{
	data=(data);

	if (data<(double)-2147483647) {return (double)-2147483647;}
	else
	if (data>(double) 2147483647) {return (double) 2147483647;}

	else return data;
}
//---------------------------------------------------------------------------
//********END OF LIMITATIONS**********************************
//---------------------------------------------------------------------------
void cp2RTPS() {
	double SSX0,SSY0,SSZ0;
	double SZ,DSZ;

#ifdef GTE_LOG
	GTE_LOG("GTE_RTPS\n");
#endif

	gteFLAG = 0;

	SSX0 = NC_OVERFLOW1((double)gteTRX + ((double)(gteVX0*gteR11) + (double)(gteVY0*gteR12) + (double)(gteVZ0*gteR13))/4096.0);
	SSY0 = NC_OVERFLOW2((double)gteTRY + ((double)(gteVX0*gteR21) + (double)(gteVY0*gteR22) + (double)(gteVZ0*gteR23))/4096.0);
	SSZ0 = NC_OVERFLOW3((double)gteTRZ + ((double)(gteVX0*gteR31) + (double)(gteVY0*gteR32) + (double)(gteVZ0*gteR33))/4096.0);

	SZ   = LIMIT(SSZ0,(double)0,(double)65535,18);
	DSZ  = ((double)gteH/SZ);

	if ((DSZ>(double)2147483647)) {DSZ=(double)2; gteFLAG|=1<<17;}

	gteSZ0  = gteSZ1;
	gteSZ1  = gteSZ2;
	gteSZ2  = gteSZx;
	gteSZx  = (unsigned short)float2int(SZ);

	cpuRegs.CP2D.r[12]= cpuRegs.CP2D.r[13];
	cpuRegs.CP2D.r[13]= cpuRegs.CP2D.r[14];

	gteSX2  = (signed short)float2int(LIMIT((double)(gteOFX)/65536.0f + (LimitAS(SSX0,24)*DSZ),(double)-1024,(double)1024,14));
	gteSY2  = (signed short)float2int(LIMIT((double)(gteOFY)/65536.0f + (LimitAS(SSY0,23)*DSZ),(double)-1024,(double)1024,13));

	gteMAC1 = (signed long)(SSX0);
	gteMAC2 = (signed long)(SSY0);
	gteMAC3 = (signed long)(SSZ0);

	MAC2IR;

	gteMAC0 = (signed long)float2int(OLIMIT((((double)gteDQB/(double)16777216) + (((double)gteDQA/(double)256)*DSZ))*16777216));
	gteIR0  = (signed long)float2int(LIMIT(((((double)gteDQB/(double)16777216) + (((double)gteDQA/(double)256)*DSZ))*4096),(double)0,(double)4095,12));

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2RTPT() {
	double SSX0,SSY0,SSZ0;
	double SZ,DSZ;

#ifdef GTE_LOG
	GTE_LOG("GTE_RTPT\n");
#endif

	gteFLAG = 0;

	gteSZ0  = gteSZx;

	SSX0 = NC_OVERFLOW1((double)gteTRX + ((double)(gteVX0 * gteR11) + (double)(gteVY0 * gteR12) + (double)(gteVZ0 * gteR13)) / 4096.0);
	SSY0 = NC_OVERFLOW2((double)gteTRY + ((double)(gteVX0 * gteR21) + (double)(gteVY0 * gteR22) + (double)(gteVZ0 * gteR23)) / 4096.0);
	SSZ0 = NC_OVERFLOW3((double)gteTRZ + ((double)(gteVX0 * gteR31) + (double)(gteVY0 * gteR32) + (double)(gteVZ0 * gteR33)) / 4096.0);

	SZ   = LIMIT(SSZ0, (double)0, (double)65535, 18);
	DSZ  = ((double)gteH / SZ);

	if ((DSZ>(double)2147483647)) {DSZ=(double)2; gteFLAG|=1<<17;}

	gteSZ1 = (unsigned short)float2int(SZ);
	gteSX0 = (signed short)float2int(LIMIT((double)(gteOFX)/65536.0f + (LimitAS(SSX0,24)*DSZ),(double)-1024,(double)1023,14));
	gteSY0 = (signed short)float2int(LIMIT((double)(gteOFY)/65536.0f + (LimitAS(SSY0,23)*DSZ),(double)-1024,(double)1023,13));

	SSX0 = NC_OVERFLOW1((double)gteTRX + ((double)(gteVX1*gteR11) + (double)(gteVY1*gteR12) + (double)(gteVZ1*gteR13))/4096.0);
	SSY0 = NC_OVERFLOW2((double)gteTRY + ((double)(gteVX1*gteR21) + (double)(gteVY1*gteR22) + (double)(gteVZ1*gteR23))/4096.0);
	SSZ0 = NC_OVERFLOW3((double)gteTRZ + ((double)(gteVX1*gteR31) + (double)(gteVY1*gteR32) + (double)(gteVZ1*gteR33))/4096.0);

	SZ   = LIMIT(SSZ0,(double)0,(double)65535,18);
	DSZ  = ((double)gteH/SZ);

	if ((DSZ>(double)2147483647)) {DSZ=(double)2; gteFLAG|=1<<17;}

	gteSZ2 = (unsigned short)float2int(SZ);
	gteSX1 = (signed short)float2int(LIMIT((double)(gteOFX)/65536.0f + (LimitAS(SSX0,24)*DSZ),(double)-1024,(double)1023,14));
	gteSY1 = (signed short)float2int(LIMIT((double)(gteOFY)/65536.0f + (LimitAS(SSY0,23)*DSZ),(double)-1024,(double)1023,13));

	SSX0 = NC_OVERFLOW1((double)gteTRX + ((double)(gteVX2*gteR11) + (double)(gteVY2*gteR12) + (double)(gteVZ2*gteR13))/4096.0);
	SSY0 = NC_OVERFLOW2((double)gteTRY + ((double)(gteVX2*gteR21) + (double)(gteVY2*gteR22) + (double)(gteVZ2*gteR23))/4096.0);
	SSZ0 = NC_OVERFLOW3((double)gteTRZ + ((double)(gteVX2*gteR31) + (double)(gteVY2*gteR32) + (double)(gteVZ2*gteR33))/4096.0);

	SZ   = LIMIT(SSZ0,(double)0,(double)65535,18);
	DSZ  = ((double)gteH/SZ);

	if ((DSZ>(double)2147483647)) {DSZ=(double)2; gteFLAG|=1<<17;}

	gteSZx = (unsigned short)float2int(SZ);
	gteSX2 = (signed short)float2int(LIMIT((double)(gteOFX)/65536.0f + (LimitAS(SSX0,24)*DSZ),(double)-1024,(double)1023,14));
	gteSY2 = (signed short)float2int(LIMIT((double)(gteOFY)/65536.0f + (LimitAS(SSY0,23)*DSZ),(double)-1024,(double)1023,13));

	gteMAC1 = (signed long)float2int(SSX0);
	gteMAC2 = (signed long)float2int(SSY0);
	gteMAC3 = (signed long)float2int(SSZ0);

	MAC2IR;

	gteMAC0 = (signed long)float2int(OLIMIT((((double)gteDQB/(double)16777216) + (((double)gteDQA/(double)256)*DSZ))*16777216));
	gteIR0  = (signed long)float2int(LIMIT(((((double)gteDQB/(double)16777216) + (((double)gteDQA/(double)256)*DSZ))*4096),(double)0,(double)4095,12));

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
#define gte_C11 gteLR1
#define gte_C12 gteLG1
#define gte_C13 gteLB1
#define gte_C21 gteLR2
#define gte_C22 gteLG2
#define gte_C23 gteLB2
#define gte_C31 gteLR3
#define gte_C32 gteLG3
#define gte_C33 gteLB3
//---------------------------------------------------------------------------
#define _MVMVA_FUNC(_v0, _v1, _v2, mx) { \
    	SSX = ((double)_v0) * mx##11 + ((double)_v1) * mx##12 + ((double)_v2) * mx##13; \
        SSY = ((double)_v0) * mx##21 + ((double)_v1) * mx##22 + ((double)_v2) * mx##23; \
        SSZ = ((double)_v0) * mx##31 + ((double)_v1) * mx##32 + ((double)_v2) * mx##33; \
}
//---------------------------------------------------------------------------
void cp2MVMVA() {
	double SSX, SSY, SSZ;

#ifdef GTE_LOG
	GTE_LOG("GTE_MVMVA %lx\n", cpuRegs.code & 0x1ffffff);
#endif

	switch (cpuRegs.code & 0x78000) {
		case 0x00000: // V0 * R
			_MVMVA_FUNC(gteVX0, gteVY0, gteVZ0, gteR); break;
		case 0x08000: // V1 * R
			_MVMVA_FUNC(gteVX1, gteVY1, gteVZ1, gteR); break;
		case 0x10000: // V2 * R
			_MVMVA_FUNC(gteVX2, gteVY2, gteVZ2, gteR); break;
		case 0x18000: // IR * R
			_MVMVA_FUNC((short)gteIR1, (short)gteIR2, (short)gteIR3, gteR); break;
		case 0x20000: // V0 * L
			_MVMVA_FUNC(gteVX0, gteVY0, gteVZ0, gteL); break;
		case 0x28000: // V1 * L
			_MVMVA_FUNC(gteVX1, gteVY1, gteVZ1, gteL); break;
		case 0x30000: // V2 * L
			_MVMVA_FUNC(gteVX2, gteVY2, gteVZ2, gteL); break;
		case 0x38000: // IR * L
			_MVMVA_FUNC((short)gteIR1, (short)gteIR2, (short)gteIR3, gteL); break;
		case 0x40000: // V0 * C
			_MVMVA_FUNC(gteVX0, gteVY0, gteVZ0, gte_C); break;
		case 0x48000: // V1 * C
			_MVMVA_FUNC(gteVX1, gteVY1, gteVZ1, gte_C); break;
		case 0x50000: // V2 * C
			_MVMVA_FUNC(gteVX2, gteVY2, gteVZ2, gte_C); break;
		case 0x58000: // IR * C
			_MVMVA_FUNC((short)gteIR1, (short)gteIR2, (short)gteIR3, gte_C); break;
		default:
			SSX = SSY = SSZ = 0;
	}

	if (cpuRegs.code & 0x80000) {
		SSX /= 4096.0; SSY /= 4096.0; SSZ /= 4096.0;
	}

	switch (cpuRegs.code & 0x6000) {
		case 0x0000: // Add TR
			SSX+= (double)gteTRX;
			SSY+= (double)gteTRY;
			SSZ+= (double)gteTRZ;
			break;
		case 0x2000: // Add BK
			SSX+= (double)gteRBK;
			SSY+= (double)gteGBK;
			SSZ+= (double)gteBBK;
			break;
		case 0x4000: // Add FC
			SSX+= (double)gteRFC;
			SSY+= (double)gteGFC;
			SSZ+= (double)gteBFC;
			break;
	}

	gteFLAG = 0;
	gteMAC1 = (long)SSX;
	gteMAC2 = (long)SSY;
	gteMAC3 = (long)SSZ;
	if (cpuRegs.code & 0x400)
		MAC2IR1()
	else MAC2IR

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2NCLIP() {
#ifdef GTE_LOG
	GTE_LOG("GTE_NCLIP\n");
#endif

	gteFLAG = 0;

	gteMAC0 = (signed long)float2int(EDETEC4(
		((double)gteSX0*((double)gteSY1-(double)gteSY2))+
		((double)gteSX1*((double)gteSY2-(double)gteSY0))+
		((double)gteSX2*((double)gteSY0-(double)gteSY1))));

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2AVSZ3() {
	unsigned long SS;
	double SZ1,SZ2,SZ3;
	double ZSF3;

#ifdef GTE_LOG
	GTE_LOG("GTE_AVSZ3\n");
#endif

	gteFLAG = 0;

	SS = cpuRegs.CP2D.r[17] & 0xffff; SZ1  = (double)SS;
	SS = cpuRegs.CP2D.r[18] & 0xffff; SZ2  = (double)SS;
	SS = cpuRegs.CP2D.r[19] & 0xffff; SZ3  = (double)SS;
	SS = cpuRegs.CP2C.r[29] & 0xffff; ZSF3 = (double)SS/(double)4096;

	cpuRegs.CP2D.r[24] = (signed long)float2int(EDETEC4(((SZ1+SZ2+SZ3)*ZSF3)));
	cpuRegs.CP2D.r[7]  = (unsigned short)float2int(LimitC(((SZ1+SZ2+SZ3)*ZSF3),18));

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2AVSZ4() {
	unsigned long SS;
	double SZ0,SZ1,SZ2,SZ3;
	double ZSF4;

#ifdef GTE_LOG
	GTE_LOG("GTE_AVSZ4\n");
#endif

	gteFLAG = 0;

	SS = cpuRegs.CP2D.r[16] & 0xffff; SZ0  = (double)SS;
	SS = cpuRegs.CP2D.r[17] & 0xffff; SZ1  = (double)SS;
	SS = cpuRegs.CP2D.r[18] & 0xffff; SZ2  = (double)SS;
	SS = cpuRegs.CP2D.r[19] & 0xffff; SZ3  = (double)SS;
	SS = cpuRegs.CP2C.r[30] & 0xffff; ZSF4 = (double)SS/(double)4096;

	cpuRegs.CP2D.r[24] = (signed long)float2int(EDETEC4(((SZ0+SZ1+SZ2+SZ3)*ZSF4)));
	cpuRegs.CP2D.r[7]  = (unsigned short)float2int(LimitC(((SZ0+SZ1+SZ2+SZ3)*ZSF4),18));

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2SQR() {
	double SSX0,SSY0,SSZ0;

#ifdef GTE_LOG
	GTE_LOG("GTE_SQR %lx\n", cpuRegs.code & 0x1ffffff);
#endif

	gteFLAG = 0;

	SSX0 = (double)gteIR1 * gteIR1;
  	SSY0 = (double)gteIR2 * gteIR2;
	SSZ0 = (double)gteIR3 * gteIR3;

	if (cpuRegs.code & 0x80000) {
		SSX0 /= 4096.0; SSY0 /= 4096.0; SSZ0 /= 4096.0;
	}

	gteMAC1 = (long)SSX0;
	gteMAC2 = (long)SSY0;
	gteMAC3 = (long)SSZ0;

	MAC2IR1();

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
#define GTE_NCCS(vn) { \
	RR0 = ((double)gteL11 * gteVX##vn + (double)gteL12 * gteVY##vn + (double)gteL13 * gteVZ##vn)/4096.0; \
	GG0 = ((double)gteL21 * gteVX##vn + (double)gteL22 * gteVY##vn + (double)gteL23 * gteVZ##vn)/4096.0; \
	BB0 = ((double)gteL31 * gteVX##vn + (double)gteL32 * gteVY##vn + (double)gteL33 * gteVZ##vn)/4096.0; \
	t1 = LimitAU(RR0,24); \
	t2 = LimitAU(GG0,23); \
	t3 = LimitAU(BB0,22); \
 \
	RR0 = (double)gteRBK + ((double)gteLR1 * t1 + (double)gteLR2 * t2 + (double)gteLR3 * t3)/4096.0; \
	GG0 = (double)gteGBK + ((double)gteLG1 * t1 + (double)gteLG2 * t2 + (double)gteLG3 * t3)/4096.0; \
	BB0 = (double)gteBBK + ((double)gteLB1 * t1 + (double)gteLB2 * t2 + (double)gteLB3 * t3)/4096.0; \
	t1 = LimitAU(RR0,24); \
	t2 = LimitAU(GG0,23); \
	t3 = LimitAU(BB0,22); \
 \
	RR0 = ((double)gteR * t1)/256.0; \
	GG0 = ((double)gteG * t2)/256.0; \
	BB0 = ((double)gteB * t3)/256.0; \
 \
	gteIR1 = (long)LimitAU(RR0,24); \
	gteIR2 = (long)LimitAU(GG0,23); \
	gteIR3 = (long)LimitAU(BB0,22); \
 \
	gteCODE0 = gteCODE1; gteCODE1 = gteCODE2; gteCODE2 = gteCODE; \
	gteR0 = gteR1; gteR1 = gteR2; gteR2 = (unsigned char)LimitB(RR0/16.0,21); \
	gteG0 = gteG1; gteG1 = gteG2; gteG2 = (unsigned char)LimitB(GG0/16.0,20); \
	gteB0 = gteB1; gteB1 = gteB2; gteB2 = (unsigned char)LimitB(BB0/16.0,19); \
 \
	gteMAC1 = (long)RR0; \
	gteMAC2 = (long)GG0; \
	gteMAC3 = (long)BB0; \
}
//---------------------------------------------------------------------------
void cp2NCCS()  {
	double RR0,GG0,BB0;
	double t1, t2, t3;

#ifdef GTE_LOG
	GTE_LOG("GTE_NCCS\n");
#endif

	gteFLAG = 0;

	GTE_NCCS(0);

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2NCCT() {
	double RR0,GG0,BB0;
	double t1, t2, t3;

#ifdef GTE_LOG
	GTE_LOG("GTE_NCCT\n");
#endif

	gteFLAG = 0;

	GTE_NCCS(0);
	GTE_NCCS(1);
	GTE_NCCS(2);

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2NCDS() {
	double tRLT,tRRLT;
	double tGLT,tGGLT;
	double tBLT,tBBLT;
	double tRR0,tL1,tLL1;
	double tGG0,tL2,tLL2;
	double tBB0,tL3,tLL3;
	unsigned long C,R,G,B;

#ifdef GTE_LOG
	GTE_LOG("GTE_NCDS\n");
#endif

	gteFLAG = 0;

	R = ((gteRGB)&0xff);
	G = ((gteRGB>> 8)&0xff);
	B = ((gteRGB>>16)&0xff);
	C = ((gteRGB>>24)&0xff);

	tLL1 = (gteL11/4096.0 * gteVX0/4096.0) + (gteL12/4096.0 * gteVY0/4096.0) + (gteL13/4096.0 * gteVZ0/4096.0);
    tLL2 = (gteL21/4096.0 * gteVX0/4096.0) + (gteL22/4096.0 * gteVY0/4096.0) + (gteL23/4096.0 * gteVZ0/4096.0);
    tLL3 = (gteL31/4096.0 * gteVX0/4096.0) + (gteL32/4096.0 * gteVY0/4096.0) + (gteL33/4096.0 * gteVZ0/4096.0);

	tL1 = LimitAU(tLL1,24);
	tL2 = LimitAU(tLL2,23);
	tL3 = LimitAU(tLL3,22);

    tRRLT = gteRBK/4096.0 + (gteLR1/4096.0 * tL1) + (gteLR2/4096.0 * tL2) + (gteLR3/4096.0 * tL3);
    tGGLT = gteGBK/4096.0 + (gteLG1/4096.0 * tL1) + (gteLG2/4096.0 * tL2) + (gteLG3/4096.0 * tL3);
    tBBLT = gteBBK/4096.0 + (gteLB1/4096.0 * tL1) + (gteLB2/4096.0 * tL2) + (gteLB3/4096.0 * tL3);

    tRLT = LimitAU(tRRLT,24);
	tGLT = LimitAU(tGGLT,23);
	tBLT = LimitAU(tBBLT,22);

    tRR0 = (R * tRLT) + (gteIR0/4096.0 * LimitAS(gteRFC/16.0 - (R * tRLT),24));
    tGG0 = (G * tGLT) + (gteIR0/4096.0 * LimitAS(gteGFC/16.0 - (G * tGLT),23));
    tBB0 = (B * tBLT) + (gteIR0/4096.0 * LimitAS(gteBFC/16.0 - (B * tBLT),22));

    gteMAC1 = (long)(tRR0 * 16.0); gteIR1 = (long)LimitAU((tRR0*16.0),24);
    gteMAC2 = (long)(tGG0 * 16.0); gteIR2 = (long)LimitAU((tGG0*16.0),23);
    gteMAC3 = (long)(tBB0 * 16.0); gteIR3 = (long)LimitAU((tBB0*16.0),22);

	R = (unsigned long)LimitB(tRR0,21); if (R>255) R=255; else if (R<0) R=0;
	G = (unsigned long)LimitB(tGG0,20); if (G>255) G=255; else if (G<0) G=0;
	B = (unsigned long)LimitB(tBB0,19); if (B>255) B=255; else if (B<0) B=0;

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteRGB2 = R|(G<<8)|(B<<16)|(C<<24);

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2NCDT() {
	double tRLT,tRRLT;
	double tGLT,tGGLT;
	double tBLT,tBBLT;
	double tRR0,tL1,tLL1;
	double tGG0,tL2,tLL2;
	double tBB0,tL3,tLL3;
	unsigned long C,R,G,B;

#ifdef GTE_LOG
	GTE_LOG("GTE_NCDT\n");
#endif

	gteFLAG = 0;

	R = ((gteRGB)&0xff);
	G = ((gteRGB>> 8)&0xff);
	B = ((gteRGB>>16)&0xff);
	C = ((gteRGB>>24)&0xff);

	tLL1 = (gteL11/4096.0 * gteVX0/4096.0) + (gteL12/4096.0 * gteVY0/4096.0) + (gteL13/4096.0 * gteVZ0/4096.0);
    tLL2 = (gteL21/4096.0 * gteVX0/4096.0) + (gteL22/4096.0 * gteVY0/4096.0) + (gteL23/4096.0 * gteVZ0/4096.0);
    tLL3 = (gteL31/4096.0 * gteVX0/4096.0) + (gteL32/4096.0 * gteVY0/4096.0) + (gteL33/4096.0 * gteVZ0/4096.0);

	tL1 = LimitAU(tLL1,24);
	tL2 = LimitAU(tLL2,23);
	tL3 = LimitAU(tLL3,22);

    tRRLT = gteRBK/4096.0 + (gteLR1/4096.0 * tL1) + (gteLR2/4096.0 * tL2) + (gteLR3/4096.0 * tL3);
    tGGLT = gteGBK/4096.0 + (gteLG1/4096.0 * tL1) + (gteLG2/4096.0 * tL2) + (gteLG3/4096.0 * tL3);
    tBBLT = gteBBK/4096.0 + (gteLB1/4096.0 * tL1) + (gteLB2/4096.0 * tL2) + (gteLB3/4096.0 * tL3);

    tRLT = LimitAU(tRRLT,24);
	tGLT = LimitAU(tGGLT,23);
	tBLT = LimitAU(tBBLT,22);

    tRR0 = (R * tRLT) + (gteIR0/4096.0 * LimitAS(gteRFC/16.0 - (R * tRLT),24));
    tGG0 = (G * tGLT) + (gteIR0/4096.0 * LimitAS(gteGFC/16.0 - (G * tGLT),23));
    tBB0 = (B * tBLT) + (gteIR0/4096.0 * LimitAS(gteBFC/16.0 - (B * tBLT),22));

    gteMAC1 = (long)(tRR0 * 16.0); gteIR1 = (long)LimitAU((tRR0*16.0),24);
    gteMAC2 = (long)(tGG0 * 16.0); gteIR2 = (long)LimitAU((tGG0*16.0),23);
    gteMAC3 = (long)(tBB0 * 16.0); gteIR3 = (long)LimitAU((tBB0*16.0),22);

	R = (unsigned long)LimitB(tRR0,21); if (R>255) R=255; else if (R<0) R=0;
	G = (unsigned long)LimitB(tGG0,20); if (G>255) G=255; else if (G<0) G=0;
	B = (unsigned long)LimitB(tBB0,19); if (B>255) B=255; else if (B<0) B=0;

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteRGB2 = R|(G<<8)|(B<<16)|(C<<24);

	R = ((gteRGB)&0xff);
	G = ((gteRGB>> 8)&0xff);
	B = ((gteRGB>>16)&0xff);
	C = ((gteRGB>>24)&0xff);

	tLL1 = (gteL11/4096.0 * gteVX1/4096.0) + (gteL12/4096.0 * gteVY1/4096.0) + (gteL13/4096.0 * gteVZ1/4096.0);
    tLL2 = (gteL21/4096.0 * gteVX1/4096.0) + (gteL22/4096.0 * gteVY1/4096.0) + (gteL23/4096.0 * gteVZ1/4096.0);
    tLL3 = (gteL31/4096.0 * gteVX1/4096.0) + (gteL32/4096.0 * gteVY1/4096.0) + (gteL33/4096.0 * gteVZ1/4096.0);

    tL1 = LimitAU(tLL1,24);
	tL2 = LimitAU(tLL2,23);
	tL3 = LimitAU(tLL3,22);

    tRRLT = gteRBK/4096.0 + (gteLR1/4096.0 * tL1) + (gteLR2/4096.0 * tL2) + (gteLR3/4096.0 * tL3);
    tGGLT = gteGBK/4096.0 + (gteLG1/4096.0 * tL1) + (gteLG2/4096.0 * tL2) + (gteLG3/4096.0 * tL3);
    tBBLT = gteBBK/4096.0 + (gteLB1/4096.0 * tL1) + (gteLB2/4096.0 * tL2) + (gteLB3/4096.0 * tL3);

    tRLT = LimitAU(tRRLT,24);
	tGLT = LimitAU(tGGLT,23);
	tBLT = LimitAU(tBBLT,22);

    tRR0 = (R * tRLT) + (gteIR0/4096.0 * LimitAS(gteRFC/16.0 - (R * tRLT),24));
    tGG0 = (G * tGLT) + (gteIR0/4096.0 * LimitAS(gteGFC/16.0 - (G * tGLT),23));
    tBB0 = (B * tBLT) + (gteIR0/4096.0 * LimitAS(gteBFC/16.0 - (B * tBLT),22));

    gteMAC1 = (long)(tRR0 * 16.0); gteIR1 = (long)LimitAU((tRR0*16.0),24);
    gteMAC2 = (long)(tGG0 * 16.0); gteIR2 = (long)LimitAU((tGG0*16.0),23);
    gteMAC3 = (long)(tBB0 * 16.0); gteIR3 = (long)LimitAU((tBB0*16.0),22);

    R = (unsigned long)LimitB(tRR0,21); if (R>255) R=255; else if (R<0) R=0;
	G = (unsigned long)LimitB(tGG0,20); if (G>255) G=255; else if (G<0) G=0;
	B = (unsigned long)LimitB(tBB0,19); if (B>255) B=255; else if (B<0) B=0;

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteRGB2 = R|(G<<8)|(B<<16)|(C<<24);

	R = ((gteRGB)&0xff);
	G = ((gteRGB>> 8)&0xff);
	B = ((gteRGB>>16)&0xff);
	C = ((gteRGB>>24)&0xff);

	tLL1 = (gteL11/4096.0 * gteVX2/4096.0) + (gteL12/4096.0 * gteVY2/4096.0) + (gteL13/4096.0 * gteVZ2/4096.0);
    tLL2 = (gteL21/4096.0 * gteVX2/4096.0) + (gteL22/4096.0 * gteVY2/4096.0) + (gteL23/4096.0 * gteVZ2/4096.0);
    tLL3 = (gteL31/4096.0 * gteVX2/4096.0) + (gteL32/4096.0 * gteVY2/4096.0) + (gteL33/4096.0 * gteVZ2/4096.0);

    tL1 = LimitAU(tLL1,24);
	tL2 = LimitAU(tLL2,23);
	tL3 = LimitAU(tLL3,22);

    tRRLT = gteRBK/4096.0 + (gteLR1/4096.0 * tL1) + (gteLR2/4096.0 * tL2) + (gteLR3/4096.0 * tL3);
    tGGLT = gteGBK/4096.0 + (gteLG1/4096.0 * tL1) + (gteLG2/4096.0 * tL2) + (gteLG3/4096.0 * tL3);
    tBBLT = gteBBK/4096.0 + (gteLB1/4096.0 * tL1) + (gteLB2/4096.0 * tL2) + (gteLB3/4096.0 * tL3);

    tRLT = LimitAU(tRRLT,24);
	tGLT = LimitAU(tGGLT,23);
	tBLT = LimitAU(tBBLT,22);

    tRR0 = (R * tRLT) + (gteIR0/4096.0 * LimitAS(gteRFC/16.0 - (R * tRLT),24));
    tGG0 = (G * tGLT) + (gteIR0/4096.0 * LimitAS(gteGFC/16.0 - (G * tGLT),23));
    tBB0 = (B * tBLT) + (gteIR0/4096.0 * LimitAS(gteBFC/16.0 - (B * tBLT),22));

	gteMAC1 = (long)(tRR0 * 16.0); gteIR1 = (long)LimitAU((tRR0*16.0),24);
    gteMAC2 = (long)(tGG0 * 16.0); gteIR2 = (long)LimitAU((tGG0*16.0),23);
    gteMAC3 = (long)(tBB0 * 16.0); gteIR3 = (long)LimitAU((tBB0*16.0),22);

    R = (unsigned long)LimitB(tRR0,21); if (R>255) R=255; else if (R<0) R=0;
	G = (unsigned long)LimitB(tGG0,20); if (G>255) G=255; else if (G<0) G=0;
	B = (unsigned long)LimitB(tBB0,19); if (B>255) B=255; else if (B<0) B=0;

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteRGB2 = R|(G<<8)|(B<<16)|(C<<24);

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2OP() {
	double SSX0=0,SSY0=0,SSZ0=0;

#ifdef GTE_LOG
	GTE_LOG("GTE_OP %lx\n", cpuRegs.code & 0x1ffffff);
#endif

	gteFLAG=0;

	switch (cpuRegs.code & 0x1ffffff) {
		case 0x178000C://op12
			SSX0 = EDETEC1((gteR22*(short)gteIR3 - gteR33*(short)gteIR2)/(double)4096);
			SSY0 = EDETEC2((gteR33*(short)gteIR1 - gteR11*(short)gteIR3)/(double)4096);
			SSZ0 = EDETEC3((gteR11*(short)gteIR2 - gteR22*(short)gteIR1)/(double)4096);
			break;
		case 0x170000C:
			SSX0 = EDETEC1((gteR22*(short)gteIR3 - gteR33*(short)gteIR2));
			SSY0 = EDETEC2((gteR33*(short)gteIR1 - gteR11*(short)gteIR3));
			SSZ0 = EDETEC3((gteR11*(short)gteIR2 - gteR22*(short)gteIR1));
			break;
	}

	gteMAC1 = (long)float2int(SSX0);
	gteMAC2 = (long)float2int(SSY0);
	gteMAC3 = (long)float2int(SSZ0);

	MAC2IR;

	if (gteIR1<0) gteIR1=0;
	if (gteIR2<0) gteIR2=0;
	if (gteIR3<0) gteIR3=0;

    if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2DCPL() {
	unsigned long C,R,G,B;

#ifdef GTE_LOG
	GTE_LOG("GTE_DCPL\n");
#endif

	gteFLAG = 0;

	R = ((gteRGB)&0xff);
	G = ((gteRGB>> 8)&0xff);
	B = ((gteRGB>>16)&0xff);
	C = ((gteRGB>>24)&0xff);

	gteMAC1 = (signed long)((double)(R*gteIR1) + (double)(gteIR0*LimitAS(gteRFC-(double)(R*gteIR1),24))/4096.0);
	gteMAC2 = (signed long)((double)(G*gteIR2) + (double)(gteIR0*LimitAS(gteGFC-(double)(G*gteIR2),23))/4096.0);
	gteMAC3 = (signed long)((double)(B*gteIR3) + (double)(gteIR0*LimitAS(gteBFC-(double)(B*gteIR3),22))/4096.0);

	MAC2IR

	R = (unsigned long)LimitB(gteMAC1,21); if (R>255) R=255; else if (R<0) R=0;
	G = (unsigned long)LimitB(gteMAC2,20); if (G>255) G=255; else if (G<0) G=0;
	B = (unsigned long)LimitB(gteMAC3,19); if (B>255) B=255; else if (B<0) B=0;

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteRGB2 = R|(G<<8)|(B<<16)|(C<<24);

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2GPF() {
	double ipx, ipy, ipz;

#ifdef GTE_LOG
	GTE_LOG("GTE_GPF %lx\n", cpuRegs.code & 0x1ffffff);
#endif

	gteFLAG = 0;

	ipx = (double)((short)gteIR0) * ((short)gteIR1);
	ipy = (double)((short)gteIR0) * ((short)gteIR2);
	ipz = (double)((short)gteIR0) * ((short)gteIR3);

	// same as mvmva
	if (cpuRegs.code & 0x80000) {
		ipx /= 4096.0; ipy /= 4096.0; ipz /= 4096.0;
	}

	gteMAC1 = (long)ipx;
	gteMAC2 = (long)ipy;
	gteMAC3 = (long)ipz;

	gteIR1 = (long)LimitAS(ipx,24);
	gteIR2 = (long)LimitAS(ipy,23);
	gteIR3 = (long)LimitAS(ipz,22);

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteC2 = gteCODE;
	gteR2 = (unsigned char)LimitB(ipx,21);
	gteG2 = (unsigned char)LimitB(ipy,20);
	gteB2 = (unsigned char)LimitB(ipz,19);
}
//---------------------------------------------------------------------------
void cp2GPL() {
    double IPX=0,IPY=0,IPZ=0;
    unsigned long C,R,G,B;

#ifdef GTE_LOG
	GTE_LOG("GTE_GPL %lx\n", cpuRegs.code & 0x1ffffff);
#endif

	gteFLAG=0;
	switch(cpuRegs.code & 0x1ffffff) {
		case 0x1A8003E:
			IPX = EDETEC1((double)gteMAC1 + ((double)gteIR0*(double)gteIR1)/4096.0f);
	        IPY = EDETEC2((double)gteMAC2 + ((double)gteIR0*(double)gteIR2)/4096.0f);
	        IPZ = EDETEC3((double)gteMAC3 + ((double)gteIR0*(double)gteIR3)/4096.0f);
			break;

		case 0x1A0003E:
	       IPX = EDETEC1((double)gteMAC1 + ((double)gteIR0*(double)gteIR1));
           IPY = EDETEC2((double)gteMAC2 + ((double)gteIR0*(double)gteIR2));
	       IPZ = EDETEC3((double)gteMAC3 + ((double)gteIR0*(double)gteIR3));
			break;
	}
	gteIR1  = (short)float2int(LimitAS(IPX,24));
	gteIR2  = (short)float2int(LimitAS(IPY,23));
	gteIR3  = (short)float2int(LimitAS(IPZ,22));

	gteMAC1 = (int)float2int(IPX);
	gteMAC2 = (int)float2int(IPY);
	gteMAC3 = (int)float2int(IPZ);

	C = gteRGB & 0xff000000;
	R = float2int(ALIMIT(IPX,0,255));
	G = float2int(ALIMIT(IPY,0,255));
	B = float2int(ALIMIT(IPZ,0,255));

	gteRGB0 = gteRGB1;
	gteRGB1 = gteRGB2;
	gteRGB2 = C|R|(G<<8)|(B<<16);
}
//---------------------------------------------------------------------------
#define GTE_DPCS() { \
	RR0 = (double)R + (gteIR0*LimitAS((double)(gteRFC - R),24))/4096.0; \
	GG0 = (double)G + (gteIR0*LimitAS((double)(gteGFC - G),23))/4096.0; \
	BB0 = (double)B + (gteIR0*LimitAS((double)(gteBFC - B),22))/4096.0; \
 \
	gteIR1 = (long)LimitAS(RR0,24); \
	gteIR2 = (long)LimitAS(GG0,23); \
	gteIR3 = (long)LimitAS(BB0,22); \
 \
	gteRGB0 = gteRGB1; \
	gteRGB1 = gteRGB2; \
	gteC2 = C; \
	gteR2 = (unsigned char)LimitB(RR0/16.0,21); \
	gteG2 = (unsigned char)LimitB(GG0/16.0,20); \
	gteB2 = (unsigned char)LimitB(BB0/16.0,19); \
 \
	gteMAC1 = (long)RR0; \
	gteMAC2 = (long)GG0; \
	gteMAC3 = (long)BB0; \
}
//---------------------------------------------------------------------------
void cp2DPCS() {
	unsigned long C,R,G,B;
	double RR0,GG0,BB0;

#ifdef GTE_LOG
	GTE_LOG("GTE_DPCS\n");
#endif

	gteFLAG = 0;

	C = gteCODE;
	R = gteR * 16.0;
	G = gteG * 16.0;
	B = gteB * 16.0;

	GTE_DPCS();

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2DPCT() {
	unsigned long C,R,G,B;
	double RR0,GG0,BB0;

#ifdef GTE_LOG
	GTE_LOG("GTE_DPCT\n");
#endif

	gteFLAG = 0;

	C = gteCODE0;
	R = gteR0 * 16.0;
	G = gteG0 * 16.0;
	B = gteB0 * 16.0;

	GTE_DPCS();

	C = gteCODE0;
	R = gteR0 * 16.0;
	G = gteG0 * 16.0;
	B = gteB0 * 16.0;

	GTE_DPCS();

	C = gteCODE0;
	R = gteR0 * 16.0;
	G = gteG0 * 16.0;
	B = gteB0 * 16.0;

	GTE_DPCS();

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
#define GTE_NCS(vn) { \
	RR0 = ((double)gteVX##vn * gteL11 + (double)gteVY##vn * (double)gteL12 + (double)gteVZ##vn * gteL13) / 4096.0; \
	GG0 = ((double)gteVX##vn * gteL21 + (double)gteVY##vn * (double)gteL22 + (double)gteVZ##vn * gteL23) / 4096.0; \
	BB0 = ((double)gteVX##vn * gteL31 + (double)gteVY##vn * (double)gteL32 + (double)gteVZ##vn * gteL33) / 4096.0; \
	t1 = LimitAU(RR0, 24); \
	t2 = LimitAU(GG0, 23); \
	t3 = LimitAU(BB0, 22); \
 \
	RR0 = (double)gteRBK + ((double)gteLR1 * t1 + (double)gteLR2 * t2 + (double)gteLR3 * t3) / 4096.0; \
	GG0 = (double)gteGBK + ((double)gteLG1 * t1 + (double)gteLG2 * t2 + (double)gteLG3 * t3) / 4096.0; \
	BB0 = (double)gteBBK + ((double)gteLB1 * t1 + (double)gteLB2 * t2 + (double)gteLB3 * t3) / 4096.0; \
	t1 = LimitAU(RR0, 24); \
	t2 = LimitAU(GG0, 23); \
	t3 = LimitAU(BB0, 22); \
 \
	gteRGB0 = gteRGB1; gteRGB1 = gteRGB2; \
	gteR2 = (unsigned char)LimitB(RR0/16.0, 21); \
	gteG2 = (unsigned char)LimitB(GG0/16.0, 20); \
	gteB2 = (unsigned char)LimitB(BB0/16.0, 19); \
	gteCODE2=gteCODE0; \
}
//---------------------------------------------------------------------------
void cp2NCS() {
	double RR0,GG0,BB0;
	double t1, t2, t3;

#ifdef GTE_LOG
	GTE_LOG("GTE_NCS\n");
#endif

	gteFLAG = 0;

	GTE_NCS(0);

	gteMAC1=(long)RR0;
	gteMAC2=(long)GG0;
	gteMAC3=(long)BB0;

	gteIR1=(long)t1;
	gteIR2=(long)t2;
	gteIR3=(long)t3;

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2NCT() {
	double RR0,GG0,BB0;
	double t1, t2, t3;

#ifdef GTE_LOG
	GTE_LOG("GTE_NCT\n");
#endif

	gteFLAG = 0;

//V0
	GTE_NCS(0);
//V1
	GTE_NCS(1);
//V2
	GTE_NCS(2);

	gteMAC1=(long)RR0;
	gteMAC2=(long)GG0;
	gteMAC3=(long)BB0;

	gteIR1=(long)t1;
	gteIR2=(long)t2;
	gteIR3=(long)t3;

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2CC() {
	double RR0,GG0,BB0;
	double t1,t2,t3;

#ifdef GTE_LOG
	GTE_LOG("GTE_CC\n");
#endif

	gteFLAG = 0;

	RR0 = (double)gteRBK + ((double)gteLR1 * gteIR1 + (double)gteLR2 * gteIR2 + (double)gteLR3 * gteIR3) / 4096.0;
	GG0 = (double)gteGBK + ((double)gteLG1 * gteIR1 + (double)gteLG2 * gteIR2 + (double)gteLG3 * gteIR3) / 4096.0;
	BB0 = (double)gteBBK + ((double)gteLB1 * gteIR1 + (double)gteLB2 * gteIR2 + (double)gteLB3 * gteIR3) / 4096.0;
	t1 = LimitAU(RR0, 24);
	t2 = LimitAU(GG0, 23);
	t3 = LimitAU(BB0, 22);

	RR0=((double)gteR * t1)/256.0;
	GG0=((double)gteG * t2)/256.0;
	BB0=((double)gteB * t3)/256.0;
	gteIR1 = (long)LimitAU(RR0,24);
	gteIR2 = (long)LimitAU(GG0,23);
	gteIR3 = (long)LimitAU(BB0,22);

	gteCODE0=gteCODE1; gteCODE1=gteCODE2;
	gteC2 = gteCODE0;
	gteR2 = (unsigned char)LimitB(RR0/16.0, 21);
	gteG2 = (unsigned char)LimitB(GG0/16.0, 20);
	gteB2 = (unsigned char)LimitB(BB0/16.0, 19);

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
//	TEST OPCODES
//---------------------------------------------------------------------------
//	Boudaries
//---------------------------------------------------------------------------
#define OB_31Pos    (long)2147483647					//31 bit: 2^31 Positive
#define OB_31Neg    (long)-2147483648					//31 bit: 2^31 Negative
#define OB_24Pos    (long)16777215						//24 bit: 2^24 Positive
#define OB_24Neg    (long)-16777216						//24 bit: 2^24 Negative
#define OB_16Pos    (long)65535							//16 bit: 2^16 Positive
#define OB_16Neg    (long)-65536						//16 bit: 2^16 Negative
#define OB_15Pos    (long)32767							//15 bit: 2^15 Positive
#define OB_15Neg    (long)-32768						//15 bit: 2^15 Negative
#define OB_12Pos    (long)4095							//12 bit: 2^12 Positive
#define OB_12Neg    (long)-4096							//12 bit: 2^12 Negative
#define OB_10Pos    (long)1023							//10 bit: 2^10 Positive
#define OB_10Neg    (long)-1024							//10 bit: 2^10 Negative
#define OB_8Pos		(long)255							//8 bit: 2^8 Positive
#define OB_8Neg		(long)-256							//8 bit: 2^8 Negative
//---------------------------------------------------------------------------
//Boudaries converted to doubles
//---------------------------------------------------------------------------
#define OBd_43Pos    (double)8796093022207.0				//43 bit: 2^43 Positive
#define OBd_43Neg    (double)-8796093022208.0				//43 bit: 2^43 Negative
#define OBd_31Pos    (double)2147483647.0					//31 bit: 2^31 Positive
#define OBd_31Neg    (double)-2147483648.0					//31 bit: 2^31 Negative
#define OBd_24Pos    (double)16777215.0						//24 bit: 2^24 Positive
#define OBd_24Neg    (double)-16777216.0					//24 bit: 2^24 Negative
#define OBd_16Pos    (double)65535.0						//16 bit: 2^16 Positive
#define OBd_16Neg    (double)-65536.0						//16 bit: 2^16 Negative
#define OBd_15Pos    (double)32767.0							//15 bit: 2^15 Positive
#define OBd_15Neg    (double)-32768.0						//15 bit: 2^15 Negative
#define OBd_12Pos    (double)4095.0							//12 bit: 2^12 Positive
#define OBd_12Neg    (double)-4096.0						//12 bit: 2^12 Negative
#define OBd_10Pos    (double)1023.0							//10 bit: 2^10 Positive
#define OBd_10Neg    (double)-1024.0						//10 bit: 2^10 Negative
#define OBd_8Pos		(double)255.0						//8 bit: 2^8 Positive
#define OBd_8Neg		(double)-256.0						//8 bit: 2^8 Negative
//---------------------------------------------------------------------------
#define INV65536	(double)0.0000152587890625		//1.0/65536
//---------------------------------------------------------------------------
//Global Vars					 									//
//---------------------------------------------------------------------------
double dC1,dC2,dC3,dZ,dHDivZ,dSX,dSY,dTmp,dTmp1;
//---------------------------------------------------------------------------
//LIMITS
//---------------------------------------------------------------------------
#define Limit(val,min,max)							\
	( ((val)<min) ? (min) :				\
	( ((val)>max) ? (max) : ((val)) ) )	\
//---------------------------------------------------------------------------
#define Lm(val,min,max,flag)							\
	( ((val)<min) ? (gteFLAG|=(1<<flag),min) :				\
	( ((val)>max) ? (gteFLAG|=(1<<flag),max) : ((val)) ) )	\
//---------------------------------------------------------------------------
#define LmEx(val,min,max,minflag,maxflag)							\
	( ((val)<min) ? (gteFLAG|=(1<<minflag),min) :				\
	( ((val)>max) ? (gteFLAG|=(1<<maxflag),max) : ((val)) ) )	\
//---------------------------------------------------------------------------
//Limits on int
//---------------------------------------------------------------------------
#define	ILm_B1(val)		Lm(val,OB_15Neg,OB_15Pos,24)
#define	ILm_B2(val)		Lm(val,OB_15Neg,OB_15Pos,23)
#define	ILm_B3(val)		Lm(val,OB_15Neg,OB_15Pos,22)
#define	ILm1_B1(val)		Lm(val,0,OB_15Pos,24)
#define	ILm1_B2(val)		Lm(val,0,OB_15Pos,23)
#define	ILm1_B3(val)		Lm(val,0,OB_15Pos,22)
//---------------------------------------------------------------------------
#define	ILm_C1(val)		Lm(val,0,OB_8Pos,21)
#define	ILm_C2(val)		Lm(val,0,OB_8Pos,20)
#define	ILm_C3(val)		Lm(val,0,OB_8Pos,19)
//---------------------------------------------------------------------------
#define	ILm_D(val)		Lm(val,0,OB_16Pos,18)
//---------------------------------------------------------------------------
#define	ILm_F(val)		LmEx(val,OB_31Neg,OB_31Pos,15,16)
//---------------------------------------------------------------------------
#define	ILm_G1(val)		Lm(val,OB_10Neg,OB_10Pos,14)
#define	ILm_G2(val)		Lm(val,OB_10Neg,OB_10Pos,13)
#define	ILm_H(val)		Lm(val,0,OB_12Pos,12)
//---------------------------------------------------------------------------
//Limits on double
//---------------------------------------------------------------------------
#define	FLm_B1(val)		Lm(val,OBd_15Neg,OBd_15Pos,24)
#define	FLm_B2(val)		Lm(val,OBd_15Neg,OBd_15Pos,23)
#define	FLm_B3(val)		Lm(val,OBd_15Neg,OBd_15Pos,22)
#define	FLm1_B1(val)		Lm(val,0.0,OBd_15Pos,24)
#define	FLm1_B2(val)		Lm(val,0.0,OBd_15Pos,23)
#define	FLm1_B3(val)		Lm(val,0.0,OBd_15Pos,22)
//---------------------------------------------------------------------------
#define	FLm_C1(val)		Lm(val,0.0,OBd_8Pos,21)
#define	FLm_C2(val)		Lm(val,0.0,OBd_8Pos,20)
#define	FLm_C3(val)		Lm(val,0.0,OBd_8Pos,19)
//---------------------------------------------------------------------------
#define	FLm_D(val)		Lm(val,0.0,OBd_16Pos,18)
//---------------------------------------------------------------------------
#define	FLm_F(val)		LmEx(val,OBd_31Neg,OBd_31Pos,15,16)
//---------------------------------------------------------------------------
#define	FLm_G1(val)		Lm(val,OBd_10Neg,OBd_10Pos,14)
#define	FLm_G2(val)		Lm(val,OBd_10Neg,OBd_10Pos,13)
#define	FLm_H(val)		Lm(val,0,OBd_12Pos,12)
//---------------------------------------------------------------------------
//Special limits
//---------------------------------------------------------------------------
#define FLm_E(dhdivz,dz)        \
{                                   \
	if (dhdivz > 2.0) {				\
		dz=0.5*(double)gteH;     \
		 dhdivz=2.0;               \
		 gteFLAG|=(1<<17);          \
	}                              \
}
//---------------------------------------------------------------------------
__inline double FLm_G1F(double val)
{
	if((val)>OBd_31Pos) {
		gteFLAG|=(1<<16);
	} else if((val)<OBd_31Neg) {
		gteFLAG|=(1<<15);
	}

	if((val)>OBd_10Pos) {
		dTmp=OBd_10Pos;
		gteFLAG|=(1<<14);
	} else if((val)<OBd_10Neg) {
		val=OBd_10Neg;
		gteFLAG|=(1<<14);
	}
	return val;
}
//---------------------------------------------------------------------------
__inline double FLm_G2F(double val)
{
	if((val)>OBd_31Pos) {
		gteFLAG|=(1<<16);
	} else if((val)<OBd_31Neg) {
		gteFLAG|=(1<<15);
	}

	if((val)>OBd_10Pos) {
		val=OBd_10Pos;
		gteFLAG|=(1<<13);
	} else if((val)<OBd_10Neg) {
		val=OBd_10Neg;
		gteFLAG|=(1<<13);
	}
	return val;
}
//---------------------------------------------------------------------------
//OVERFLOWS
//---------------------------------------------------------------------------
#define OC(val,min,max,minflag,maxflag) \
{											\
	if((val)<min) {							\
		gteFLAG|=(1<<minflag);				\
	} else if((val)>max) {					\
		gteFLAG|=(1<<maxflag);				\
	}										\
}
//---------------------------------------------------------------------------
/*
#define OC(val,min,max,minflag,maxflag)       \
	  ( ((val)<min) ? (gteFLAG|=(1<<minflag)) :				\
	  ( ((val)>max) ? (gteFLAG|=(1<<maxflag)) )  )*
//---------------------------------------------------------------------------
//overflows on float values
//---------------------------------------------------------------------------
#define FOC_A1(val)  OC(val,OBd_43Neg,OBd_43Pos,26,29)
#define FOC_A2(val)  OC(val,OBd_43Neg,OBd_43Pos,25,28)
#define FOC_A3(val)  OC(val,OBd_43Neg,OBd_43Pos,24,27)
//---------------------------------------------------------------------------
#define FOC_F(val) OC(val,OBd_31Neg,OBd_31Pos,15,16)
//---------------------------------------------------------------------------
void cp2INTPL() {
	double dIR1,dIR2,dIR3;

#ifdef GTE_LOG
	GTE_LOG("GTE_INTP\n");
#endif

	gteFLAG=0;

	//NOTE this opcode should be done better

	dIR1=(double)gteIR1*16.0;
	dIR2=(double)gteIR2*16.0;
	dIR3=(double)gteIR3*16.0;

	dC1=dIR1+(double)gteIR0*(FLm_B1((double)gteRFC-dIR1))/4096.0;
	dC2=dIR2+(double)gteIR0*(FLm_B2((double)gteGFC-dIR2))/4096.0;
	dC3=dIR3+(double)gteIR0*(FLm_B3((double)gteBFC-dIR3))/4096.0;

	//Check overflow
	FOC_A1(dC1);
	FOC_A2(dC2);
	FOC_A3(dC3);

	gteMAC1=(signed long)dC1;
	gteMAC2=(signed long)dC2;
	gteMAC3=(signed long)dC3;

	//Assign to IR
	MAC2IR;

	gteRGB0 = gteRGB1; gteRGB1 = gteRGB2;  //step RGB fifo

	gteR2= (unsigned char)ILm_C1(gteMAC1>>4 );
	gteG2= (unsigned char)ILm_C2(gteMAC2>>4 );
	gteB2= (unsigned char)ILm_C3(gteMAC3>>4 );
	gteCODE2= gteCODE0;

	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
void cp2CDP() {
	double t1,t2,t3;
	double rir,gir,bir;

#ifdef GTE_LOG
	GTE_LOG("GTE_CDP\n");
#endif
	gteFLAG = 0;

	dC1 = (double)gteRBK + ((double)gteLR1 * (double)gteIR1 + (double)gteLR2 * (double)gteIR2 + (double)gteLR3 * (double)gteIR3) / 4096.0;
	dC2 = (double)gteGBK + ((double)gteLG1 * (double)gteIR1 + (double)gteLG2 * (double)gteIR2 + (double)gteLG3 * (double)gteIR3) / 4096.0;
	dC3 = (double)gteBBK + ((double)gteLB1 * (double)gteIR1 + (double)gteLB2 * (double)gteIR2 + (double)gteLB3 * (double)gteIR3) / 4096.0;
	t1= Limit(dC1, 0.0, OBd_31Pos);
	t2= Limit(dC2, 0.0, OBd_31Pos);
	t3= Limit(dC3, 0.0, OBd_31Pos);
	//it's 12 bit precision

	rir=(double)gteR*t1/256.0f;
	gir=(double)gteG*t2/256.0f;
	bir=(double)gteB*t3/256.0f;
	//Now it's 4 bit precision

	dC1 = rir + (double)gteIR0*(FLm_B1((double)gteRFC-rir))/4096.0;
	dC2 = gir + (double)gteIR0*(FLm_B2((double)gteGFC-gir))/4096.0;
	dC3 = bir + (double)gteIR0*(FLm_B3((double)gteBFC-bir))/4096.0;

	//Check overflow
	FOC_A1(dC1);
	FOC_A2(dC2);
	FOC_A3(dC3);

	gteRGB0 = gteRGB1; gteRGB1 = gteRGB2;  //step RGB fifo

	gteMAC1=(signed long)dC1;
	gteMAC2=(signed long)dC2;
	gteMAC3=(signed long)dC3;

	//Assign to IR
	MAC2IR;

	gteR2= (unsigned char)ILm_C1(gteMAC1>>4 );
	gteG2= (unsigned char)ILm_C2(gteMAC2>>4 );
	gteB2= (unsigned char)ILm_C3(gteMAC3>>4 );
	gteCODE2= gteCODE0;


	if (gteFLAG & 0x7f87e000) gteFLAG|=0x80000000;
}
//---------------------------------------------------------------------------
*/
//---------------------------------------------------------------------------
