//---------------------------------------------------------------------------
//
//	$FileName	::	GTE.h
//	$Purpose	::	Header for GTE / GPU emulation
//				MIPS32 ISA COP2
//
//---------------------------------------------------------------------------
#ifndef GTEH
#define GTEH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <stdarg.h>
#include <windows.h>
//---------------------------------------------------------------------------
#include <gl/gl.h>
#include <gl/glu.h>
//---------------------------------------------------------------------------
//#include "d3d8.h"
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
//	OPEN GL SPECIFIC HEADER INFORMATION
//---------------------------------------------------------------------------
#define	GLFONT_TYPE_COLOURED	0x0001	//	Flat shading
#define GLFONT_TYPE_TEXTURED	0x0002	//	texture mapped, or gourad shaded
#define GLFONT_TYPE_LIGHTING	0x0004	//	gourad lit shading (use only if 3D card!)
#define GLFONT_TYPE_2D		0x0010	//	2d font
#define GLFONT_TYPE_3D		0x0020	//	3d font
#define GLFONT_TYPE_DEFAULT	( GLFONT_TYPE_COLOURED | GLFONT_TYPE_2D )
//---------------------------------------------------------------------------
#define GLFONT_SIZE_DEFAULT	10
//---------------------------------------------------------------------------
//	DIRECT X SPECIFIC HEADER INFORMATION
//---------------------------------------------------------------------------
//	GPU commands are encoded in the colour register variable 'a'
//---------------------------------------------------------------------------
#define	GPUCMD_MASK		0xFF000000	//	Use this to & with 32-bit command to retrieve what is being requested
#define GPUCMD_SHIFT		24		//	Shift command byte left 24 bits
//---------------------------------------------------------------------------
//	These are taken from Gran Turismo 2 (Sorry!), something to do with resetting
// the GPU or initialising it
//---------------------------------------------------------------------------
#define GPUCMD_UNKNOWN0		0x14
#define GPUCMD_UNKNOWN1		0x18
//---------------------------------------------------------------------------
//	GPU Primitive packet type
//---------------------------------------------------------------------------
//	Note that monochrome does not mean black or white but one particular
// colour is used across the entire face, ie flat shading and not goraud shading
//---------------------------------------------------------------------------
#define GPUCMD_PACKET_MPOLY3		0x20	//	3 point polygon monochrome (single colour NOT black and white)
#define GPUCMD_PACKET_TPOLY3 		0x24	//	3 point polygon textured
#define GPUCMD_PACKET_MPOLY4		0x28	//	4 point polygon monochrome
#define GPUCMD_PACKET_TPOLY4		0x2C	//	4 point polygon textured
#define GPUCMD_PACKET_SPOLY3		0x30	//	3 point polygon shaded
#define GPUCMD_PACKET_STPOLY3		0x34	//	3 point polygon textured shaded
#define GPUCMD_PACKET_SPOLY4		0x38	//	4 point polygon shaded
#define GPUCMD_PACKET_STPOLY4		0x3C	//	4 point polygon textured shaded
#define GPUCMD_PACKET_MLINE		0x40	//	monochrome line
#define GPUCMD_PACKET_MPOLYL		0x48	//	monochrome poly line
#define GPUCMD_PACKET_SLINE		0x50	//	shaded line
#define GPUCMD_PACKET_SPOLYL		0x58	//	shaded polyline
#define GPUCMD_PACKET_RECT		0x60	//	rectangle
#define GPUCMD_PACKET_SPRITE		0x64	//	sprite
#define GPUCMD_PACKET_DOT		0x68	//	dot
#define GPUCMD_PACKET_RECT8		0x70	//	8*8 rectangle
#define GPUCMD_PACKET_SPRITE8		0x74	//	8*8 sprite
#define GPUCMD_PACKET_RECT16		0x78	//	16*16 rectangle
#define GPUCMD_PACKET_SPRITE16		0x7C	//	16*16 sprite
//---------------------------------------------------------------------------
//	GPU Command and Transfer packets
//---------------------------------------------------------------------------
#define GPUCMD_PACKET_CLEAR		0x01	//	Clear cache, frame-buffer, OT
#define GPUCMD_PACKET_FBUFF_RECT	0x02	//	frame buffer rectangle drawing, ie bit-blit operation
#define GPUCMD_PACKET_MOVEIN		0x80	//	Move image in framebuffer
#define GPUCMD_PACKET_MOVETO		0xA0	//	move image to frame buffer from RAM
#define GPUCMD_PACKET_MOVEFROM		0xC0	//	move image from frame buffer to RAM
//---------------------------------------------------------------------------
//	Draw mode / enviroment setting packets
//---------------------------------------------------------------------------
#define GPUCMD_PACKET_DMODE_SETTING	0XE1	//	Draw Mode
#define GPUCMD_PACKET_TXTRE_SETTING	0XE2	//	Texture Area
#define GPUCMD_PACKET_DAREA_TLEFT	0XE3	//	Drawing Area Top Left
#define GPUCMD_PACKET_DAREA_BRIGHT	0XE4	//	Drawing Area Bottom Right
#define GPUCMD_PACKET_DRAW_OFFSET	0XE5	//	Offset in VRAM where FrameBuffer starts
#define GPUCMD_PACKET_DRAW_MASK		0XE6	//	Drawing mask, transparency, and, or etc
//---------------------------------------------------------------------------
//	GPU STATUS REGISTER VALUES RETURNABLE
//---------------------------------------------------------------------------
#define	GPU_BUSY			0x00000001	//	GTE/GPU is busy
//---------------------------------------------------------------------------
#define FONTNAME			"Lucinda Console"	//	Default Font name used by Open GL do not know what the default font the psx uses
#define FONTSIZE			8			//	default screen size
#define DEFAULT_XSIZE			320			//	default x resolution
#define DEFAULT_YSIZE			256			//	default y resolution
#define DEFAULT_CDEPTH			15			//	default colour resolution 16 or 24-bit display mode
//---------------------------------------------------------------------------
//	Windows Messaging for GTE Thread, these need to be merged with the ones
//	defined in the EmulatorDLL.h file so that a common intialise message
//	can be customised to signal only certain particular messages
//	allows for greater customisation of the message system and parameter
//	passing, however message encoding/decoding take a bit longer
//---------------------------------------------------------------------------
#define GTE_INITIALISE		MSG_INITIALISE		//	Initialise GPU/GTE
#define GTE_SHUTDOWN		MSG_TERMINATE           //	Shutdown GPU/GTE
#define GTE_CONFIGURE           MSG_CONFIGURE          			//	Configure GPU/GTE emulation
#define GTE_TEST                WM_USER + 0x0000000C                  	//	Test current settings
#define GTE_ABOUT               WM_USER + 0x0000000D                  	//	Display an about dialog box
#define GTE_OPEN                WM_USER + 0x0000000E                  	//	Activate the emulation
#define GTE_CLOSE               WM_USER + 0x0000000F                  	//	deactivate the emulation
#define GTE_READSTATUS          WM_USER + 0x00000010                  	//	read status of GPU/GTE
#define GTE_READDATA            WM_USER + 0x00000011                  	//	read data port
#define GTE_READDATAMEM         WM_USER + 0x00000012                  	//	read data memory
#define GTE_WRITESTATUS         WM_USER + 0x00000013                  	//	write status port
#define GTE_WRITEDATA           WM_USER + 0x00000014                 	//	write data port
#define GTE_WRITEDATAMEM        WM_USER + 0x00000015                  	//	write data memory
#define GTE_DMACHAIN            WM_USER + 0x00000016                  	//	dma chain sequence initialisation
#define GTE_KEYPRESSED          WM_USER + 0x00000017                  	//	keypressed (PC) pad pressed (PSX)
#define GTE_DISPLAYTEXT         WM_USER + 0x00000018                  //	write some text out
#define GTE_MAKESNAPSHOT        WM_USER + 0x00000019                  //	make a copy of the current display memory and save to disk
#define GTE_FREEZE              WM_USER + 0x0000001A                  //	Pause emulation
#define GTE_GETSCREENPIC        WM_USER + 0x0000001B                  //	same as GTE_MAKESNAPSHOT
#define GTE_SHOWSCREENPIC       WM_USER + 0x0000001C                  //	display a bitmap file
#define GTE_UPDATELACE          WM_USER + 0x0000001D                  //	update interlacing
#define GTE_SETFONTNAME		WM_USER + 0x0000001E                  //	set display font face name
#define GTE_SETFONTSIZE		WM_USER + 0x0000001F                  //	set display font size
#define GTE_GETFONTNAME		WM_USER + 0x00000020                  //	get display font face name
#define GTE_GETFONTSIZE		WM_USER + 0x00000021                  //	get display font size
//---------------------------------------------------------------------------
//	GPU	Primitive type as structures
//---------------------------------------------------------------------------
//typedef	struct	_gpu_prim_mpoly3_	{	//	Monochrome 3 point polygon (triangle)
//	unsigned	S	: 1;
//        unsigned	B	: 5;
//        unsigned	G	: 5;
//        unsigned	R	: 5;		//	Colour CLUT index
//        unsigned	cmd_code: 8;            //	Would match GPUCMD_PACKAET_MPOLY3
//        unsigned	x0	: 16;		//	First x,y pairing
//        unsigned	y0	: 16;
//        unsigned	x1	: 16;		//	second x,y pairing
//        unsigned	y1	: 16;
//        unsigned	x2	: 16;		//	third x,y pairing
//        unsigned	y2	: 16;
//	} GPU_Primitive_mpoly3, *lpPrimMPoly3;
//---------------------------------------------------------------------------
//typedef	struct	_gpu_prim_tpoly3_	{	//	Texture 3 point polygon
//	unsigned	fill	: 16;
//        unsigned	BGR	: 8;
//        unsigned	cmd_code: 8;
//        unsigned	x0	: 16;
//        unsigned	y0	: 16;
//        unsigned	u0	: 8;
//        unsigned	v0	: 8;
//        unsigned	clut	: 16;
//        unsigned	x1	: 16;
//        unsigned	y1	: 16;
//        unsigned	u1	: 8;
//        unsigned	v1	: 8;
//        unsigned	tpage	: 16;
//        unsigned	x2	: 16;
//        unsigned	y2	: 16;
//        unsigned	u2	: 8;
//        unsigned	v2	: 8;
//        unsigned	fill1	: 16;
//        } GPU_Primitive_tpoly3;
//---------------------------------------------------------------------------
//	Types required by the GTE and/or GPU sub-systems
//---------------------------------------------------------------------------
typedef struct _ot_tag_	{		//	Ordering Table Tag
	unsigned	pNext : 24;	//	Next tag pointer
        unsigned	Length: 8;	//	Length of this tag, data immediately follows this
        } GsOTTAG, *LPGsOTTAG;
//---------------------------------------------------------------------------
//	GPU Ordering Table
//---------------------------------------------------------------------------
typedef struct _gsot_	{		//	Ordering Table
	unsigned int	Length;		//	bit length of ordering table
        unsigned long	Offset;		//	NEAR Z-Value for this ordering table
        unsigned long	Position;	//      FAR Z-Value for this ordering table
        LPGsOTTAG	lpHead;		//	Head of tag queue ptr to array of tags
	} GsOT, *LPGsOT;
//---------------------------------------------------------------------------
typedef	struct	_colour_ {		//	Colour r,g,b,a psx uses r,g,b a == GTE instruction
	unsigned char	a, r, g, b;
        } COLOUR, COLOR, *LPCOLOUR, *LPCOLOR;
//---------------------------------------------------------------------------
typedef	struct	_svector2d_ {		//	x, y position
	unsigned int	x, y;
        } SVECTOR2D, *LPSVECTOR2D;
//---------------------------------------------------------------------------
typedef struct _svector3d_ {		//	x, y, z position
	unsigned int	x, y, z, w;
        } SVECTOR3D, VECTOR3D, *LPSVECTOR3D, *LPVECTOR3D;
//---------------------------------------------------------------------------
//	PlayStation Pixel Format Definition
//---------------------------------------------------------------------------
typedef	struct	_psx_pixel_def_	{	//	16-bit pixel definition
	unsigned	S : 1;		//	but always read/written as a 24-bit value
        unsigned	B : 5;		//	and masked as needed, dependant upon
        unsigned	G : 5;		//	the current display mode
        unsigned	R : 5;
} PSXPIXEL, *LPPSXPIXEL;
//---------------------------------------------------------------------------
//	This is PlayStation's 4x4 matrix, used for 3d rotation, scaling and
// translation. It does not seem to match anything I have seen before for this
// type of operation. Why is it different? Any particular benefit(s)?
//---------------------------------------------------------------------------
typedef struct	_matrix_ {		//	4x4 matrix, sort of
	int	m[ 3 ][ 3 ];		//	rotation and scale values
        int	pad;			//	why this is here who knows, answers on a postcard please!
        int	t[ 3 ];			//	Translation values
	} MATRIX, *LPMATRIX;
//---------------------------------------------------------------------------
//	Rectangle ie x,y,w,h. Windows defines left, right, top, bottom
//---------------------------------------------------------------------------
typedef	RECT	*LPRECT;		//	Lets borrow windows one as its the same he he
//---------------------------------------------------------------------------
typedef struct _texture_ {		//	Texture Mapping Coordinates
    	unsigned char u, v;
        } TEXCOORDS, *LPTEXCOORDS;
//---------------------------------------------------------------------------
typedef struct _line_	{		//	Line
    	COLOR       color;
    	SVECTOR3D   vert0, vert1;
	} LINE, *LPLINE;
//---------------------------------------------------------------------------
typedef struct _ciline_	{		//	Colour indexed line
    	COLOR       color0;		//	Colour for lower left
    	SVECTOR3D   vert0;		//	lower left
    	COLOR       color1;		//	colour for uperr right
    	SVECTOR3D   vert1;		//	upper right
	} CILINE, *LPCILINE;
//---------------------------------------------------------------------------
typedef struct _tri_ 	{		//	Triangle
    	COLOR       color;
    	SVECTOR3D   vert0, vert1, vert2;
	} TRI, *LPTRI;
//---------------------------------------------------------------------------
typedef struct _citri_	{		//	Colour indexed triangle
    	COLOR       color0;
    	SVECTOR3D   vert0;
    	COLOR       color1;
    	SVECTOR3D   vert1;
    	COLOR       color2;
    	SVECTOR3D   vert2;
	} CITRI, *LPCITRI;
//---------------------------------------------------------------------------
typedef struct _textri_	{		//	Textured triangle
    	COLOR       color;
    	SVECTOR3D   vert0;
    	TEXCOORDS   texCoords0;
    	unsigned int   clutId;		//	open gl texture id values
    	SVECTOR3D   vert1;
    	TEXCOORDS   texCoords1;
    	unsigned int   texPage;	//	open gl texture id
    	SVECTOR3D   vert2;
    	TEXCOORDS   texCoords2;
        unsigned int   pad;
	} TEXTRI, *LPTEXTRI;
//---------------------------------------------------------------------------
typedef struct _citextri_	{	//	Colour indexed textured triangle
	COLOR       color0;
    	SVECTOR3D   vert0;
    	TEXCOORDS   texCoords0;
    	unsigned int   clutId;
    	COLOR       color1;
    	SVECTOR3D   vert1;
    	TEXCOORDS   texCoords1;
    	unsigned int   texPage;
    	COLOR       color2;
    	SVECTOR3D   vert2;
    	TEXCOORDS   texCoords2;
    	unsigned int   pad;
	} CITEXTRI, *LPCITEXTRI;
//---------------------------------------------------------------------------
typedef struct _quad_	{		//	Quad
    	COLOR       color;
    	SVECTOR3D   vert0, vert1, vert2, vert3;
	} QUAD, *LPQUAD;
//---------------------------------------------------------------------------
typedef struct	_ciquad_	{	//	Colour indexed quad
    	COLOR       color0;
    	SVECTOR3D   vert0;
    	COLOR       color1;
    	SVECTOR3D   vert1;
    	COLOR       color2;
    	SVECTOR3D   vert2;
    	COLOR       color3;
    	SVECTOR3D   vert3;
	} CIQUAD, *LPCIQUAD;
//---------------------------------------------------------------------------
typedef struct _texquad_	{	//	Textured quad
	COLOR       color;
    	SVECTOR3D   vert0;
    	TEXCOORDS   texCoords0;
    	unsigned int   clutId;
    	SVECTOR3D   vert1;
    	TEXCOORDS   texCoords1;
    	unsigned int   texPage;
    	SVECTOR3D   vert2;
    	TEXCOORDS   texCoords2;
    	unsigned int   pad1;
    	SVECTOR3D   vert3;
    	TEXCOORDS   texCoords3;
    	unsigned int   pad2;
	} TEXQUAD, *LPTEXQUAD;
//---------------------------------------------------------------------------
typedef struct _citexquad_	{	//	Colour indexed textured quad
    	COLOR       color0;
    	SVECTOR3D   vert0;
    	TEXCOORDS   texCoords0;
    	unsigned int   clutId;
    	COLOR       color1;
    	SVECTOR3D   vert1;
    	TEXCOORDS   texCoords1;
    	unsigned int   texPage;
    	COLOR       color2;
    	SVECTOR3D   vert2;
    	TEXCOORDS   texCoords2;
    	unsigned int   pad1;
    	COLOR       color3;
    	SVECTOR3D   vert3;
    	TEXCOORDS   texCoords3;
    	unsigned int   pad2;
	} CITEXQUAD, *LPCITEXQUAD;
//---------------------------------------------------------------------------
typedef struct _boxfill_	{	//	fill area
    	COLOR       color;
    	SVECTOR3D   pos, size;
	} BOXFILL, *LPBOXFILL;
//---------------------------------------------------------------------------
//	TIM image files, PlayStation Texture/Bitmap file format
//---------------------------------------------------------------------------
typedef struct _timhdr_	{
    	unsigned long   id;                 /* Should be 0x00000010 */
    	unsigned long   flags;              /* What kind of image is this? */
	} TIMHEADER, *LPTIMHEADER;
//---------------------------------------------------------------------------
typedef struct _timclut_	{
    	unsigned long   dataLen;            /* Len of clut block (inc. this word) */
    	RECT        	vRamPos;            /* Position clut is to be loaded into VRam */
    	unsigned int   	clutData[2];        /* May be 16 or 256 entries */
	} TIMCLUT, *LPTIMCLUT;
//---------------------------------------------------------------------------
typedef struct _tim_image_ {
    	unsigned long   dataLen;            /* Len of pixel data block (inc. this word) */
    	RECT        vRamPos;            /* Position pixel data is to be loaded into VRam */
    	unsigned int   imageData[2];       /* Is usually more than 2 entries!!! */
	} TIMIMAGE, *LPTIMIMAGE;
//---------------------------------------------------------------------------
#define TIMHDR_4BITCLUT     	0		//	4-bit Colour Lookup Table
#define TIMHDR_8BITCLUT     	1		//	8-bit Colour Lookup Table
#define TIMHDR_15BIT        	2		//	15-bit Image
#define TIMHDR_24BIT        	3		//	24-bit Image
#define TIMHDR_MIXED        	4		//	Mixed Image and Colour Data
#define TIMHDR_CLUTPRESENT  	8		//	Colour Lookup Table is present with image data
//---------------------------------------------------------------------------
//	GTE/GPU API
//---------------------------------------------------------------------------
#define MODE_PAL    		1
#define MODE_NTSC		0
#define GsNONINTER  		0
#define GsINTER     		1
//---------------------------------------------------------------------------
//	TMD Section	(3D Models usually)
//---------------------------------------------------------------------------
typedef struct _tmd_header_	{	//	TMD Header definition
	DWORD		dwIdNum;
        DWORD		dwFlags;
        DWORD		dwObjectCount;
	} TMDHEADER, *LPTMDHEADER;
//---------------------------------------------------------------------------
typedef struct _tmd_vertex_	{	//	TMD Vertex, very same as SVECTOR3D
	unsigned int	x, y, z;	//	PSX DOES NOT USE w
        } TMDVERTEX, *LPTMDVERTEX;
//---------------------------------------------------------------------------
typedef struct	_tmd_normal_	{	//	TMD Normal, same as above
	unsigned int	x, y, z;	//	PSX DOES NOT USE w
        } TMDNORMAL, *LPTMDNORMAL;
//---------------------------------------------------------------------------
typedef struct	_tmd_prim_header_ {	//	TMD Primitive header
	unsigned char	outLen;		//	Buffer Lengh of output buffer AFTER processing has occured in words (32-bits)
        unsigned char	inLen;		//	Buffer length in input data in words (32-bits)
        unsigned char	Flags;		//	Flags for primitive
        unsigned char	Mode;		//	Primitive type and options
        } TMDPRIMHEADER, *LPTMDPRIMHEADER;
//---------------------------------------------------------------------------
typedef struct	_tmd_object_header_ {	//	TMD Object Header
	LPTMDVERTEX	lpVertices;	//	Vertex List
        DWORD		dwVertexCount;	//	MMM ?
        LPTMDNORMAL	lpNormals;	//	Normals list
        DWORD		dwNormalsCount;	//
        LPTMDPRIMHEADER	lpPrimitives;	//	List of primitives
        DWORD		dwPrimCount;	//	Number of primitives in list
        DWORD		LogScale;	//	Scale 2^LogScale
	} TMDOBJECT, *LPTMDOBJECT;
//---------------------------------------------------------------------------
#define TMDHDR_FIXP         0x1L    	//	This object's addresses have been fixed up
#define TMDMODE_CODEMASK    0xE0    	//	Mask for primitive type
#define TMDMODE_OPTIONMASK  0x1F    	//	Mask for options
#define TMDMODE_CODESPRITE  0x60    	//	This is a sprite
#define TMDMODE_CODELINE    0x40    	//	This is a line
#define TMDMODE_CODEPOLY    0x20    	//	This is a triangle or quadrilateral
#define TMDFLAG_GRADUATION  0x10    	//	This is a smooth shaded polygon
#define TMDFLAG_4VERTICES   0x08    	//	This is a quadrilateral
#define TMDFLAG_TEXTURED    0x04    	//	This is a textured polygon
#define TMDFLAG_DOUBLESIDE  0x02    	//	This is a double sided polygon
#define TMDFLAG_LIT         0x01    	//	This is a lit polygon
//---------------------------------------------------------------------------
//	Need another section for interfacing with bs image streams
//---------------------------------------------------------------------------
//	PSX <--> GTE Thread methods
//---------------------------------------------------------------------------
void 	cp2MFC2()  {};	//	Move from coprocessor 2 to cpu ie GTE/GPU --> CPU
void 	cp2CFC2()  {};	//	Compare with coprocessor 2 register
void 	cp2MTC2()  {};	//	move to coprocessor 2 from cpu ie CPU --> GTE/GPU
void 	cp2CTC2()  {};	//	compare with ?
void 	cp2RTPS()  {};	//	Rotate Point ?
void 	cp2OP()    {};	//	specific operation
void 	cp2NCLIP() {};	//	normal clipping
void 	cp2DPCS()  {};  //      Depth Precision Clipping Settings ?
void 	cp2INTPL() {};  //
void 	cp2MVMVA() {};  //
void 	cp2NCDS()  {};  //
void 	cp2NCDT()  {};  //
void 	cp2CDP()   {};  //
void 	cp2NCCS()  {};  //
void 	cp2CC()    {};  //
void 	cp2NCS()   {};  //
void 	cp2NCT()   {};  //
void 	cp2SQR()   {};  //	--> GTE_Sqr
void 	cp2DCPL()  {};  //
void 	cp2DPCT()  {};  //
void 	cp2AVSZ3() {};	//	average z3 value
void 	cp2AVSZ4() {};	//	average z4 value
void 	cp2RTPT()  {};  //
void 	cp2GPF()   {};  //
void 	cp2GPL()   {};  //
void 	cp2NCCT()  {};  //
//---------------------------------------------------------------------------
//	GTE Registers DATA REGISTERS
//---------------------------------------------------------------------------
//	???
//---------------------------------------------------------------------------
#define gteVX0     ((signed short*)GenRegs)[0]
#define gteVY0     ((signed short*)GenRegs)[1]
#define gteVZ0     ((signed short*)GenRegs)[2]
//---------------------------------------------------------------------------
//	???
//---------------------------------------------------------------------------
#define gteVX1     ((signed short*)GenRegs)[4]
#define gteVY1     ((signed short*)GenRegs)[5]
#define gteVZ1     ((signed short*)GenRegs)[6]
//---------------------------------------------------------------------------
//	???
//---------------------------------------------------------------------------
#define gteVX2     ((signed short*)GenRegs)[8]
#define gteVY2     ((signed short*)GenRegs)[9]
#define gteVZ2     ((signed short*)GenRegs)[10]
//---------------------------------------------------------------------------
//	Current Colour or Current Colour Lookup Table Entry ?
//---------------------------------------------------------------------------
#define gteRGB     GenRegs[6]
//---------------------------------------------------------------------------
//	Ordering Table Z value ?
//---------------------------------------------------------------------------
#define gteOTZ     ((short*)GenRegs)[7*2]
//---------------------------------------------------------------------------
//	???
//---------------------------------------------------------------------------
#define gteIR0     ((signed long*)GenRegs)[8]
#define gteIR1     ((signed long*)GenRegs)[9]
#define gteIR2     ((signed long*)GenRegs)[10]
#define gteIR3     ((signed long*)GenRegs)[11]
//---------------------------------------------------------------------------
//	???
//---------------------------------------------------------------------------
#define gteSX0     ((short*)GenRegs)[12*2]
#define gteSY0     ((short*)GenRegs)[12*2+1]
//---------------------------------------------------------------------------
#define gteSX1     ((short*)GenRegs)[13*2]
#define gteSY1     ((short*)GenRegs)[13*2+1]
//---------------------------------------------------------------------------
#define gteSX2     ((short*)GenRegs)[14*2]
#define gteSY2     ((short*)GenRegs)[14*2+1]
//---------------------------------------------------------------------------
#define gteSXP     ((short*)GenRegs)[15*2]
#define gteSYP     ((short*)GenRegs)[15*2+1]
//---------------------------------------------------------------------------
#define gteSZ0     GenRegs[16]
#define gteSZ1     GenRegs[17]
#define gteSZ2     GenRegs[18]
#define gteSZx     GenRegs[19]
//---------------------------------------------------------------------------
//	current colour r,g,b values ?
//---------------------------------------------------------------------------
#define gteRGB0    GenRegs[20]
#define gteRGB1    GenRegs[21]
#define gteRGB2    GenRegs[22]
//---------------------------------------------------------------------------
//	??? Matrix ?
//---------------------------------------------------------------------------
#define gteMAC0    GenRegs[24]
#define gteMAC1    ((signed long*)GenRegs)[25]
#define gteMAC2    ((signed long*)GenRegs)[26]
#define gteMAC3    ((signed long*)GenRegs)[27]
//---------------------------------------------------------------------------
//	Used for CLUT ?
//---------------------------------------------------------------------------
#define gteIRGB    GenRegs[28]
#define gteORGB    GenRegs[29]
//---------------------------------------------------------------------------
//	Z Sorting ?
//---------------------------------------------------------------------------
#define gteLZCS    GenRegs[30]
#define gteLZCR    GenRegs[31]
//---------------------------------------------------------------------------
//	GPU CONTROL REGISTERS
//---------------------------------------------------------------------------
//	Note sure what the following are used for, could these be for the CLUT ?
// in various operating modes ?
//---------------------------------------------------------------------------
#define gteR     ((unsigned char*)GenRegs)[6*4]
#define gteG     ((unsigned char*)GenRegs)[6*4+1]
#define gteB     ((unsigned char*)GenRegs)[6*4+2]
#define gteCODE  ((unsigned char*)GenRegs)[6*4+3]
#define gteC     gteCODE
//---------------------------------------------------------------------------
#define gteR0    ((unsigned char*)GenRegs)[20*4]
#define gteG0    ((unsigned char*)GenRegs)[20*4+1]
#define gteB0    ((unsigned char*)GenRegs)[20*4+2]
#define gteCODE0 ((unsigned char*)GenRegs)[20*4+3]
#define gteC0    gteCODE0
//---------------------------------------------------------------------------
#define gteR1    ((unsigned char*)GenRegs)[21*4]
#define gteG1    ((unsigned char*)GenRegs)[21*4+1]
#define gteB1    ((unsigned char*)GenRegs)[21*4+2]
#define gteCODE1 ((unsigned char*)GenRegs)[21*4+3]
#define gteC1    gteCODE1
//---------------------------------------------------------------------------
#define gteR2    ((unsigned char*)GenRegs)[22*4]
#define gteG2    ((unsigned char*)GenRegs)[22*4+1]
#define gteB2    ((unsigned char*)GenRegs)[22*4+2]
#define gteCODE2 ((unsigned char*)GenRegs)[22*4+3]
#define gteC2    gteCODE2
//---------------------------------------------------------------------------
//	Rotation matrix values
//---------------------------------------------------------------------------
#define gteR11  ((signed short*)CtlRegs)[0]
#define gteR12  ((signed short*)CtlRegs)[1]
#define gteR13  ((signed short*)CtlRegs)[2]	//	X ?

#define gteR21  ((signed short*)CtlRegs)[3]	//	Y ?
#define gteR22  ((signed short*)CtlRegs)[4]
#define gteR23  ((signed short*)CtlRegs)[5]

#define gteR31  ((signed short*)CtlRegs)[6]	//	Z ?
#define gteR32  ((signed short*)CtlRegs)[7]
#define gteR33  ((signed short*)CtlRegs)[8]
//---------------------------------------------------------------------------
//	Translation matrix ?
//---------------------------------------------------------------------------
#define gteTRX  (signed long)CtlRegs[5]
#define gteTRY  (signed long)CtlRegs[6]
#define gteTRZ  (signed long)CtlRegs[7]
//---------------------------------------------------------------------------
//	Rotation/Scaling matrix ?	I think that these relate to the lighting matrix
//	as the psx has a possibility of 3 (max) light sources, hardware limit afraid.
//---------------------------------------------------------------------------
#define gteL11  ((short*)CtlRegs)[16]
#define gteL12  ((short*)CtlRegs)[17]
#define gteL13  ((short*)CtlRegs)[18]
#define gteL21  ((short*)CtlRegs)[19]
#define gteL22  ((short*)CtlRegs)[20]
#define gteL23  ((short*)CtlRegs)[21]
#define gteL31  ((short*)CtlRegs)[22]
#define gteL32  ((short*)CtlRegs)[23]
#define gteL33  ((short*)CtlRegs)[24]
//---------------------------------------------------------------------------
//	Background colour ?
//---------------------------------------------------------------------------
#define gteRBK  CtlRegs[13]
#define gteGBK  CtlRegs[14]
#define gteBBK  CtlRegs[15]
//---------------------------------------------------------------------------
#define gteLR1  ((signed short*)CtlRegs)[32]
#define gteLR2  ((signed short*)CtlRegs)[33]
#define gteLR3  ((signed short*)CtlRegs)[34]
//---------------------------------------------------------------------------
#define gteLG1  ((signed short*)CtlRegs)[35]
#define gteLG2  ((signed short*)CtlRegs)[36]
#define gteLG3  ((signed short*)CtlRegs)[37]
//---------------------------------------------------------------------------
#define gteLB1  ((signed short*)CtlRegs)[38]
#define gteLB2  ((signed short*)CtlRegs)[39]
#define gteLB3  ((signed short*)CtlRegs)[40]
//---------------------------------------------------------------------------
//	Foreground colour ?
//---------------------------------------------------------------------------
#define gteRFC  CtlRegs[21]
#define gteGFC  CtlRegs[22]
#define gteBFC  CtlRegs[23]
//---------------------------------------------------------------------------
#define gteOFX  (signed long)CtlRegs[24]
#define gteOFY  (signed long)CtlRegs[25]
//---------------------------------------------------------------------------
#define gteH    ((unsigned short*)CtlRegs)[52]
//---------------------------------------------------------------------------
#define gteDQA  ((signed short*)CtlRegs)[54]
#define gteDQB  (signed long)CtlRegs[28]
//---------------------------------------------------------------------------
//	Z 3/4 Average
//---------------------------------------------------------------------------
#define gteZSF3 ((signed short*)CtlRegs)[58]
#define gteZSF4 ((signed short*)CtlRegs)[60]
//---------------------------------------------------------------------------
//	Access the the flags register for the GTE/GPU
//---------------------------------------------------------------------------
#define gteFLAG cpuRegs.GPR.r[ 31 ]					//	CP2C.r[31]
//---------------------------------------------------------------------------
//	GPU Modes, note not GTE as the GTE is just a fast 3d maths processor
// and the GPU is the thing that actually draws the polygons
//---------------------------------------------------------------------------
#define	GPUMODE_PAL		0x01000000
#define GPUMODE_NTSC		0x02000000
#define GPUMODE_INTERLACED	0x04000000
#define GPUMODE_NONINTERLACED	0x08000000
//---------------------------------------------------------------------------
#define	DOUBLEWORD	DWORD
//---------------------------------------------------------------------------
//!	Data registers.
//!	CP2 Data Register Set (General Registers).
//---------------------------------------------------------------------------
typedef struct _cop2_data_regs_ {
	SVECTOR3D     	v0, v1, v2;		//!	Not sure what these vertices are for, model,camera,lights? (Action!)
	COLOUR        	rgb;			//!	fog colour, command value also
	long          	otz;			//!	Z Value for ordering table
	long          	ir0, ir1, ir2, ir3;	//!	Intermmiediate Result Registers
	SVECTOR2D     	sxy0, sxy1, sxy2; 	//!	??? Clipping volume ? Clipping Viewport ?
        DWORD		sxyp;
	DWORD		sz0, sz1, sz2, sz3;	//!	???
	COLOUR         	rgb0, rgb1, rgb2;	//!	Light colours for light 0-2 (1-3)
	long          	reserved;		//!	???
	long          	mac0, mac1, mac2, mac3;	//!	???
	unsigned long 	irgb, orgb;		//!	???
	long          	lzcs, lzcr;		//!	???
} CP2DRegs, *lpCP2DRegs;			//	DATA REGISTERS
//---------------------------------------------------------------------------
//!	Control Registers
//!	CP2 Control Register Set
//---------------------------------------------------------------------------
//!	Control Registers
//---------------------------------------------------------------------------
typedef struct _cop2_ctl_regs_ {
	MATRIX 	rMatrix;		//!	World rotation matrix
	long    trX, trY, trZ;		//!	World translation values
	MATRIX 	lMatrix;		//!	lighting matrix
	long    rbk, gbk, bbk;		//!	Colour background
	MATRIX 	cMatrix;		//!	Camera matrix
	long    rfc, gfc, bfc;		//!	colour foreground
	long    ofx, ofy;		//!	offset in video ram where picture generation starts, ie framebuffer start
	long    h;			//!	projection distance
	long    dqa, dqb;		//!	???
	long    zsf3, zsf4;		//!	z sorting averag 1/3 and 1/4 values, for texture mip-mapping
	long    flag;			//!	flags register
} CP2CRegs, *lpCP2CRegs;
//---------------------------------------------------------------------------
//!	COP2 register set.
//---------------------------------------------------------------------------
typedef struct _cop2_regs_ {
	union {
        	CP2DRegs	Data;
                DWORD		dwData[ 32 ];
        };
        union {
        	CP2CRegs	Control;
                DWORD		dwCtrl[ 32 ];
        };
} CP2Regs, *lpCP2Regs;
//---------------------------------------------------------------------------
//
//	$CLASS		::	GTE
//	$Purpose	::	Emulation of PlayStation's Geometry Transform Engine (GTE)
//				and Graphics Processing Unit (GPU).
//				It is intended to map PlayStation things to PC
//				Open GL or Direct X.
//
//---------------------------------------------------------------------------
class GTE : public TThread
	{
        	friend	class	BIOS;
                friend	class	R3000A;
                friend	class	HW;

		private:
                	//	Windows Specific
                	WNDCLASSEX	WndClass;		//	Windows Class registration structure
                        DEVMODE		dmScreenSettings;	//	Device Mode Settings ie screen resolution colour depth
                        HINSTANCE	hInstance;		//	Instance of this application
                        RECT		WinRect;		//	Standard rectangle x, y, w, h type
                        HWND		wHandle;		//	Handle to the window representing the playstation display
                        HDC		hDC;			//	Hardware device context, used by windows
                        DWORD		dwExStyle;		//	Windows extra style flags for window definition
                        DWORD		dwStyle;		//	Windows style flags for window definition
                        DWORD		dwFPS;			//	FrameCounter
                        int		PixelFormat;		//	Pixel format r,g,b,a
                        HANDLE		hEvent;			//	WIN32 event object for CP2

                        //	OpenGL specific
                        HGLRC		glRC;			//	Open GL rendering context

                        //	This structure holds the font definitions for Open GL
                        // it is changed every time a new font is specified but not
                        // when a new size is given
                        struct {
                        	GLuint	glFontBase;		//	open gl font base list
                                int	glFontSize;		//	Current size of font
                                int	glFontType;		//	2D/3D, single colour, textured
                                char 	*lpszFontName;		//	Current font name
                                // note, if textured then texture id is glFontType >> 8, therefore only the
                                // first 256 textures defined in Open Gl can be used to texture a font, this
                                // restriction is ONLY applied when the font is textured otherwise it is
                                // ignored. This does not matter whether the font is 2D or 3D
                                char	*lpszFont2DName;	//	Font Name for 2D Fonts
                                char	*lpszFont3DName;	//	Font Name for 3D Fonts
                                HFONT	hFont2D;		//	Windows Handles to 2D
                                HFONT	hFont3D;		//	and 3D 'Windows Fonts'
                                GLuint	glFont2D;               //	Open GL List for 2D font
                                GLuint	glFont3D;               //	and again for 3D font
                                class	GLTexture	*lpTexture2D;
                                class	GLTexture	*lpTexture3D;
                        } GLFont;

                        //	Direct X/3D Specific

                        //	...	Nothing defined at present for Direct X, still playing with the SDK (8.1)

			//	Common
                        int		iWidth;			//	width of window
                        int		iHeight;		//	height of window
                        int		iCDepth;		//	colour depth PlayStation supports either 16 or 24-bit modes
                        int		Buffer;                 //

                        bool		fWindowed;		//	Are we running in a window and NOT fullscreen, windows only
                        bool		fActive;		//	is this class defined and registered ok?
                        bool		fPAL;			//	PAL / NTSC Mode flag
                        bool		fInterlacedFlag;	//	Interlacing being used (FAKED)

                        //!	GTE/GPU Specific

                        struct	{	//!	Returned by GPUGetInfo
                        	char	*lpszGPUDriver;		//!	Name of GPU Driver
                                // either this NULL (or 'Internal') or set to filename (.dll)
                                // of module to load
                                DWORD	dwGPUVersion;		//!	Version of GPU Driver
                                int	CurrentMode;            //!	Current video mode

                                // ....	Enviroment Status Return Information
                                struct {

                                	//	Display Enviroment
                                	struct {
                                        	RECT	BackBuffer;	//	BACK BUFFER
                                                RECT	FrontBuffer;	//	FRONT BUFFER
                                                bool	fInterlaced;	//	Interlaced mode?
                                                bool	fDithering;	//	Dithering ( true if !iCDepth==24 )
	                                } Display;

                                        //	Drawing Enviroment
	                                struct {
                                        	RECT	Clipping;
                                                WORD	ofs[ 2 ];
                                                RECT	tw;
                                                BYTE	tpage;
                                                WORD	dtd;
                                                WORD	dfe;
                                                BYTE	isbg;
                                                BYTE	r0, g0, b0;
	                                } Drawing;

                                        //	Frame Buffer Enviroment
	                                struct {
	                                	WORD	wFlags;
	                                } FrameBuffer;

                                } Enviroment;

                        } GPUInfo;

                        CP2Regs		Regs;		//	CP2 Control and Data registers

                        BYTE		*pDataPort;	//	Data Port storage (64 BYTES)
                        BYTE		*pCommandPort;	//	Command Port storage (again 64 BYTES)

                        BYTE	        bdin, bdout;	//	Variables used to track the current data
                        BYTE		bcin, bcout;	//	and matching command values

                        //!	PlayStation Interface, GTE/GPU Ordering Table Emulation, and Open GL compiled version.
                        LPGsOT		lpOT;			//!	Ordering table
                        unsigned int	*lpglOT[];		//!	Open GL Compiled list for OT[i] equivalent
                        DWORD		dwOTSize;		//!	number of entries in OT, also for lpglOT[]

                        //	Windows Message Handler, deals with any messages from the rest of the
                        //	emulator. Note careful when calling ANY of these methods as this class
                        //	runs as an independant thread to the main MIPS core CP0 and CP1

                        friend	LRESULT	CALLBACK	GTEWndProc		( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam );

                        //	COP2 Specific Functions
                        void				**lpFuncTable;

                        DWORD	__fastcall	GDataPort	( void );
                        DWORD	__fastcall	GStatusPort	( void );

                        void	__fastcall	SDataPort	( DWORD Value );
                        void	__fastcall	SCommandPort	( DWORD Value );

                        DWORD	__fastcall	GGenRegs	( int Index );
                        DWORD	__fastcall	GCtlRegs	( int Index );

                        void	__fastcall	SGenRegs	( int Index, DWORD Value );
                        void	__fastcall	SCtlRegs	( int Index, DWORD Value );
		protected:
                	//	Class specific

                        //	Main execution thread for this class
			void 	__fastcall 	Execute 		( void );

                        //	Open GL specific interface methods
                        int	__fastcall	GLInit			( int xSize = 640, int ySize = 480, int CDepth = 16, bool FullScreen = false );
                        void	__fastcall	GLShutDown		( void );
                        void	__fastcall	GLFontInit		( char *font_name, int font_size );
                        void	__fastcall	GLFontShutDown		( void );
                        void	__fastcall	GLResize		( int xSize, int ySize );
                        void	__fastcall	GLFontSetFontName	( char *font_name );
                        char*	__fastcall	GLFontGetFontName	( void );
                        void	__fastcall	GLFontSetFontSize	( int font_size );
                        int	__fastcall	GLFontGetFontSize	( void );
                        int	__fastcall	GLCompileOrderingTable	( LPGsOT pOT );

                        //	Reconfigures Open GL to the values stored in
                        // the Regs structures. Called on EVERY WRITE to the
                        // data or control registers. Or should it be called
                        // every time the Ordering Table is drawn? Or should a
                        // variable be set/unset and then the drawing routine
                        // checks this variable, and call this method if needed?
                        void	__fastcall	GLUpdate		( void );

                        //	Direct X 8.1 specific interface methods;
                        int	__fastcall	DXInit			( int xSize = 640, int ySize = 480, int CDepth = 16, bool FullScreen = false );
                        void	__fastcall	DXShutDown		( void );
                        void	__fastcall	DXFontInit		( void );
                        void	__fastcall	DXFontInit		( char *font_name, int font_size );
                        void	__fastcall	DXFontShutDown		( void );
                        void	__fastcall	DXResize		( int xSize, int ySize );
                        void	__fastcall	DXFontSetFontName	( char *font_name );
                        char*	__fastcall	DXFontGetFontName	( void );
                        void	__fastcall	DXFontSetFontSize	( int font_size );
                        int	__fastcall	DXFontGetFontSize	( void );

                        void	__fastcall	GsSortOt		( LPGsOT lpOT, DWORD offset, DWORD pos );
		public:
				__fastcall 	GTE			( bool CreateSuspended = true );
                                __fastcall	~GTE			( void );

                	int	__fastcall	Initialise		( void );
                        void	__fastcall	Terminate		( void );

                        void	__fastcall	Reset			( void );

                        friend	int	__fastcall	LoadState	( LPSTR lpszFileName );
                        friend	int	__fastcall	SaveState	( LPSTR lpszFileName );

                        friend	LRESULT	CALLBACK	GTEWndProc	( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam );

        		//
                        //	GPU 2D Specific methods
                        //

                        //	Specifically set the GPU video system to PAL mode else NTSC is selected
                	void		__fastcall	GPU_SetPAL		( bool fPALFlag );
                        //	Reset the GPU sub-system
                        DWORD		__fastcall	GPU_Reset		( int Mode );
                        //	Initialise the GPU to a specific resolution and colour depth and whether interlacing is used for display
                        //	note the interlacing option is currently not supported as PC video resolutions are a LOT higher than
                        //	any games console produces
                        DWORD		__fastcall	GPU_Init		( int xSize, int ySize, bool fInterlaced, bool f24Bit );
                        //	Set the frame buffer dimensions
                        void		__fastcall	GPU_DefDispBuff		( int x0, int y0, int x1, int y1 );
                        //	get the current frame buffer (front or back)
                        int		__fastcall	GPU_GetActiveBuff	( void );
                        //	wait or check for the Vertical Retrace for display swapping and drawing
                        int		__fastcall	GPU_VSync		( void );
                        //	allows the display to actual draw what is in video ram
                        void		__fastcall	GPU_EnableDisplay	( bool fEnable );
                        //	swap between the front and back buffers and vice-versa
                        void		__fastcall	GPU_FlipDisplay		( void );
                        //	get the colour lookup table for the tim image
                        LPTIMCLUT	__fastcall	GPU_TIMGetClut		( LPTIMHEADER lpTim );
                        //	get the actual image, xfer from mem 2 vram
                        LPTIMIMAGE	__fastcall	GPU_TIMGetImage		( LPTIMHEADER lpTim );
                        //	loads an image from cd to mem
                        void		__fastcall	GPU_LoadImageData	( RECT *Area, void *Data );
                        //	not sure on this, sets the actual size of the buffer to hold the ordering table ?
                        void		__fastcall	GPU_SetCmdWorkSpace	( void *pBuffer );

                        //	clear the current ordering table			ruler z-value	z-pos		     ot to sort and merge with main ot
                        // Used for clearing images from the display, but this is not really needed on the PC
                        // as windows/open gl are already told to clear the display or redraw
                        void		__fastcall	GPU_ClearOT		( unsigned int offset, unsigned int point, LPGsOT lpOT );

                        void		__fastcall	GPU_SortOt		( LPGsOT lpOT, unsigned long offset, unsigned long pos );

                        //	add a box fill to the ordering table and sort it in repect to the 'pos' z-value
                        void		__fastcall	GPU_SortBoxFill		( LPGsOT lpOT, unsigned long pos, LPBOXFILL lpBoxFill );
                        //	clear the sort values, ie default to normal
                        void		__fastcall	GPU_SortClear		( LPGsOT lpOT, unsigned long Index, LPCOLOUR lpColour );
                        //	add a line to the ot and sort
                        void		__fastcall	GPU_SortLine		( LPGsOT lpOT, unsigned long pos, LPLINE lpLine );
                        //	add a colour indexed line to the ot and sort
                        void		__fastcall	GPU_SortCiLine		( LPGsOT lpOT, unsigned long pos, LPCILINE lpCiLine );
                        //	triangle
                        void		__fastcall	GPU_SortTriangle	( LPGsOT lpOT, unsigned long pos, LPTRI lpTri );
                        //	colour indexed triangle
                        void		__fastcall	GPU_SortCiTriangle	( LPGsOT lpOT, unsigned long pos, LPCITRI lpCiTri );
                        //	textured triangle
                        void		__fastcall	GPU_SortTexTriangle	( LPGsOT lpOT, unsigned long pos, LPTEXTRI lpTexTi );
                        //	colour indexed textured triangle
                        void		__fastcall	GPU_SortCiTexTriangle	( LPGsOT lpOT, unsigned long pos, LPCITEXTRI lpCiTexTri );
                        //	add a quad
                        void		__fastcall	GPU_SortQuad		( LPGsOT lpOT, unsigned long pos, LPQUAD lpQuad );
                        //	colour indexed quad
                        void		__fastcall	GPU_SortCiQuad		( LPGsOT lpOT, unsigned long pos, LPCIQUAD lpCiQuad );
                        //	texture quad
                        void		__fastcall	GPU_SortTexQuad		( LPGsOT lpOT, unsigned long pos, LPTEXQUAD lpTexQuad );
                        //	colour indexed textured quad
                        void		__fastcall	GPU_SortCiTexQuad	( LPGsOT lpOT, unsigned long pos, LPCITEXQUAD lpCiTexQuad );
                        //	find the clut id value for a given pixel
                        int		__fastcall	GPU_CalcClutID		( int x, int y );
                        //	calculate the clut mode for the pixel
                        int		__fastcall	GPU_CalcTexturePage	( int x, int y, int ClutMode );
                        //	actually draws the contents of the ordering table
                        void		__fastcall	GPU_DrawOT		( LPGsOT lpOrderingTable );
                        //	draws on a vsync
                        DWORD		__fastcall	GPU_DrawSync		( void );
                        //	loads an images direct to vram
                        DWORD		__fastcall	GPU_LoadImage		( LPRECT lpRect, void *pData );
                        //	clears an image in vram
                        DWORD		__fastcall	GPU_ClearImage		( LPRECT lpRect, LPCOLOUR lpColour );
                        //
                        //	GTE 3D Specific methods Fast 3D Math
                        //
                        //	initialise the 3d engine
                        void		__fastcall	GTE_Init		( int Width, int Height );
                        //	set the display start in the video ram
                        void		__fastcall	GTE_SetOffset		( int x, int y );
                        //	set the default fog colour
                        void		__fastcall	GTE_SetFogColour	( unsigned char r, unsigned char g, unsigned char b );
                        //	set the general scaling factor, usually 4096
                        void		__fastcall	GTE_SV3Scale		( LPSVECTOR3D lpVectorOut, LPSVECTOR3D lpVectorIn, int Scale, int Quantity );
                        //	not sure
                        void		__fastcall	GTE_SV3XForm		( LPSVECTOR3D lpVectorOut, LPSVECTOR3D lpVectorIn, LPMATRIX lpMatrix, int Quantity );
                        //	not sure
                        void		__fastcall	GTE_V3XForm		( LPSVECTOR3D lpVectorOut, LPSVECTOR3D lpVectorIn, LPMATRIX lpMatrix, int Quantity );
                        //	multiply a matrix
                        void		__fastcall	GTE_MatMul		( LPMATRIX lpMatrixOut, LPMATRIX lpMatrix1, LPMATRIX lpMatrix2 );
                        //	is the polygon given in front of ???
                        int		__fastcall	GTE_IsPolyFront		( LPSVECTOR3D lpVector1, LPSVECTOR3D lpVector2, LPSVECTOR3D lpVector3 );
                        //	average z value 1/3
                        DWORD		__fastcall	GTE_AverageZ3		( unsigned long v1, unsigned long v2, unsigned long v3 );
                        //	Average z value 1/4
                        DWORD		__fastcall	GTE_AverageZ4		( unsigned long v1, unsigned long v2, unsigned long v3, unsigned long v4 );
                        //	Parallel projection, convert from 3d world space to camera space
                        void		__fastcall	GTE_ParallelProject	( LPVECTOR3D lpVertOut, LPVECTOR3D lpVectIn, DWORD dwQuantity );
                        //	Set the identity matrix
                        void        	__fastcall	GTE_MatrixSetIdentity(MATRIX *matrix);
                        //	sine
                        int		__fastcall	GTE_Sin			( int Angle );
                        //	cosine
                        int		__fastcall	GTE_Cos			( int Angle );

                        friend	void	__fastcall	EmulatorExecute		( void );
                        friend	void	__fastcall	GLPrintf( int x, int y, const char *fmt );
                __published:
                	__property	DWORD	GenRegs[ int Index ] = { read=GGenRegs, write=SGenRegs };
                        __property	DWORD	CtlRegs[ int Index ] = { read=GCtlRegs, write=SCtlRegs };


                        //	Want to change these in the future to use pipes, would be better for
                        // multi-threded application you see, thus more emulated speed. Hopefully :)
                        __property	DWORD	DataPort	= { read=GDataPort, write=SDataPort };
                        __property	DWORD	CommandPort	= { write=SCommandPort };
                        __property	DWORD	StatusPort	= { read=GStatusPort };
	};
//---------------------------------------------------------------------------
__declspec( dllimport )	GTE	*lpGTE;			//	COP2 emulation
__declspec( dllimport ) DWORD	VSyncCounter;		//	Vertical Retrace Counter
//---------------------------------------------------------------------------
#endif	//	GTEH
//---------------------------------------------------------------------------
