//---------------------------------------------------------------------------
//
//	$FileName	::	HW.cpp
//	$Purpose	::	Provides access to the memory mapped hardware
//				ie CPU, GTE/GPU, SIO, SPU, etc, etc,
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "HW.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#ifdef __WIN32__
#pragma warning(disable:4244)
#endif
//---------------------------------------------------------------------------
#define	CLASSNAME	"HW"		//	Hardware Mapped Memory
//---------------------------------------------------------------------------
#define HW_DMA0_MADR lpPsxHWM[ 0x1080 ] // MDEC in DMA
#define HW_DMA0_BCR  lpPsxHWM[ 0x1084 ]
#define HW_DMA0_CHCR lpPsxHWM[ 0x1088 ]
//---------------------------------------------------------------------------
#define HW_DMA1_MADR lpPsxHWM[ 0x1090 ] // MDEC out DMA
#define HW_DMA1_BCR  lpPsxHWM[ 0x1094 ]
#define HW_DMA1_CHCR lpPsxHWM[ 0x1098 ]
//---------------------------------------------------------------------------
#define HW_DMA2_MADR lpPsxHWM[ 0x10a0 ] // GPU DMA	Address
#define HW_DMA2_BCR  lpPsxHWM[ 0x10a4 ]		//	Count
#define HW_DMA2_CHCR lpPsxHWM[ 0x10a8 ]		//	channel / mode
//--------------------------------------------------------------------------
#define HW_DMA3_MADR lpPsxHWM[ 0x10b0 ] // CDROM DMA
#define HW_DMA3_BCR  lpPsxHWM[ 0x10b4 ]
#define HW_DMA3_CHCR lpPsxHWM[ 0x10b8 ]
//---------------------------------------------------------------------------
#define HW_DMA4_MADR lpPsxHWM[ 0x10c0 ] // SPU DMA
#define HW_DMA4_BCR  lpPsxHWM[ 0x10c4 ]
#define HW_DMA4_CHCR lpPsxHWM[ 0x10c8 ]
//---------------------------------------------------------------------------
#define HW_DMA6_MADR lpPsxHWM[ 0x10e0 ] // GPU DMA (OT)
#define HW_DMA6_BCR  lpPsxHWM[ 0x10e4 ]
#define HW_DMA6_CHCR lpPsxHWM[ 0x10e8 ]
//---------------------------------------------------------------------------
#define HW_DMA_PCR   lpPsxHWM[ 0x10f0 ]	//	DMA Programmable Control Register
#define HW_DMA_ICR   lpPsxHWM[ 0x10f4 ]	//	DMA Interrupt Control Register
//---------------------------------------------------------------------------
__declspec( dllexport )	HW	*lpHW = NULL;
//---------------------------------------------------------------------------
//	$Method		::	Constructor
//---------------------------------------------------------------------------
__fastcall	HW :: HW( void ) : TObject()
{
	fprintf( stderr, "Hardware Emulation Layer Construction\n" );
	lpPsxHWM = new DWORD[ 0x00010000 ];

        ZeroMemory( lpPsxHWM, sizeof( DWORD ) * 0x00010000 );

        //	Initialise DMA
        if ( PSX_DMA == NULL )
        	PSX_DMA = new CPSX_DMA;

        //	Initialise the MPEG decoder
        if ( PSX_MDEC == NULL )
        	PSX_MDEC = new CPSX_MDEC;

	//	Initialise root counters
	if ( psx_counters == NULL )
        	psx_counters = new PSX_Counter;

        //	Initialise the serial i/o unit

        //	Initialise SPU
        if ( PsxSPU == NULL )
        	if ( CPU != NULL )
                	PsxSPU = new PSX_SPU( CPU->Suspended );
                else	PsxSPU = new PSX_SPU( false );

        //	create decode xa

        //	Initialise the CDROM
        if ( CDROM == NULL )
        	CDROM = new PSX_CDROM;
}
//---------------------------------------------------------------------------
//	$Method		::	Destructor
//---------------------------------------------------------------------------
__fastcall	HW :: ~HW( void )
{
	if ( CDROM != NULL )
        	delete CDROM;	//	Remove CDROM Emulation Layer

        if ( PsxSPU != NULL ) {
        	if ( !PsxSPU->Suspended )
                	PsxSPU->Suspend();
                delete PsxSPU;
        }

        delete PSX_MDEC;

        delete PSX_DMA;

        delete psx_counters;

	delete this;
}
//---------------------------------------------------------------------------
int	__fastcall	HW :: Initialise( void )
{

	//	Initialise the root counters
        psx_counters->Initialise();

        //	Initialse the DMA
        PSX_DMA->Initialise();

        //	Initialise the MDEC
        PSX_MDEC->Initialise();

        //	Initialise the SIO
        PsxSIO->Initialise();

        //	Initialise the PIO
        //PsxPIO->Initialise();

        //	Initialie the SPU
        PsxSPU->Initialise();

        //	Initialise the XADecode
        //PSX_XADecode->Initialise();

        //	Initialise the CDROM
        CDROM->Initialise();

        return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	HW :: Terminate( void )
{
	CDROM->Terminate();
        PsxSPU->Terminate();
        PSX_MDEC->Terminate();
        PSX_DMA->Terminate();
        psx_counters->Terminate();
}
//---------------------------------------------------------------------------
//	Reset Hardware Emulation Layer
//---------------------------------------------------------------------------
void	__fastcall	HW :: Reset( void )
{
	lpPsxHWM[ 0x1070 ] = 0;		 		//	Clear hardware interrupts
        psx_counters->ResetAll();
        PSX_DMA->Reset();
        PSX_MDEC->Reset();
        PsxSIO->Reset();
        PsxSPU->Reset();
        CDROM->Reset();
}
//---------------------------------------------------------------------------
//	Reads an 8-bit Value from hardware memory, ie what hardware are we trying
// to read? and then pass it on to them if possible. Note only passed if the hardware
// in question supports 8-bit reads.
//---------------------------------------------------------------------------
BYTE	__fastcall	HW :: HWRead8( WORD Index )
{	unsigned char 	hard = 0x00;

	switch ( Index ) {
		case 0x1040:
                	hard = sioRead8();
                        break;
      		case 0x1050:
                	hard = sioRead8();
                      break;//for use of serial port ignore for now
		case 0x1800:
                	hard = CDROM->CDRead0();
                        break;
		case 0x1801:
                	hard = CDROM->CDRead1();
                        break;
		case 0x1802:
                	hard = CDROM->CDRead2();
                        break;
		case 0x1803:
                	hard = CDROM->CDRead3();
                        break;
		default:
			hard = lpPsxHWM[ Index ];
	}
        return( hard );
}
//---------------------------------------------------------------------------
//	Same as the 8-bit version above, again only passed if 16-bit reading is
// supported else ignored
//---------------------------------------------------------------------------
WORD	__fastcall	HW :: HWRead16( WORD Index )
{	WORD	hard = 0x0000;

	switch ( Index ) {

        	//	IREG	Interrupt Request Register, actually points to CP0
                //		Control Register Cause, specifically the IP bits

		case 0x1070: //HW_LOG("IREG 16bit read %x\n", uHWM16(0x1070));
                	if ( lpPsxHWM[ 0x1070 ] != CPU->CP0.CtrlRegs.Cause.IP )
                        	lpPsxHWM[ 0x1070 ] = CPU->CP0.CtrlRegs.Cause.IP;

			return( (WORD)lpPsxHWM[ 0x1070 ] );

                //	IMASK	Interupt Mask Register, like the above it points to
                //		a CP0 Control Register Status, specifically the IM bits
		case 0x1074: // HW_LOG("IMASK 16bit read %x\n", uHWM16(0x1074));
                	if ( lpPsxHWM[ 0x1074 ] != CPU->CP0.CtrlRegs.Status.IM )
                        	lpPsxHWM[ 0x1704 ] = CPU->CP0.CtrlRegs.Status.IM;
			return( (WORD)lpPsxHWM[ 0x1074 ] );

		//Serial port stuff not support now ;P
	 	//case 0x1050: hard = serial_read16(); break;
	 	//case 0x1054: hard = serial_status_read(); break;
	 	//case 0x105a: hard = serial_control_read(); break;
	 	//case 0x105e: hard = serial_baud_read(); break;

		case 0x1100:
			hard = psx_counters->ReadCount(0);
			return hard;
		case 0x1104:
			hard = psx_counters->lpCounters[0].mode;
			return hard;
		case 0x1108:
			hard = psx_counters->lpCounters[0].target;
			return hard;
		case 0x1110:
			hard = psx_counters->ReadCount(1);
			return hard;
		case 0x1114:
			hard = psx_counters->lpCounters[1].mode;
			return hard;
		case 0x1118:
			hard = psx_counters->lpCounters[1].target;
			return hard;
		case 0x1120:
			hard = psx_counters->ReadCount(2);
			return hard;
		case 0x1124:
			hard = psx_counters->lpCounters[2].mode;
			return hard;
		case 0x1128:
			hard = psx_counters->lpCounters[2].target;
			return hard;

		//case 0x2030: hard =   //int_2000????
		//case 0x2040: hard =//dip switches...??

		default:
			if ( ( Index & 0xfff0 ) == 0x1040 ) {
				hard = sioRead16( Index );
			} else if ( Index >= 0x1c00 && Index < 0x1e00 ) {
//            	hard = SPU_readRegister( Index );
			} else {
				hard = lpPsxHWM[ Index ];
			}
                        return( hard );
	}
}
//---------------------------------------------------------------------------
DWORD	__fastcall	HW :: HWRead32( WORD Index )
{	DWORD	hard = 0x00000000;

	switch ( Index ) {

		case 0x1040:
                	hard = sioRead32();
                        break;
		case 0x1050:
                	hard = sioRead32();
                        break;//serial port

		case 0x1060:
			return lpPsxHWM[ 0x1060 ];

		case 0x1070: // HW_LOG("IREG 32bit read %x\n", dwlpPsxHWM[ 0x1070 ] );
			return( lpPsxHWM[ 0x1070 ] );

		case 0x1074: // HW_LOG("IMASK 32bit read %x\n", dwlpPsxHWM[ 0x1074 ] );
                	if ( lpPsxHWM[ 0x1074 ] != CPU->CP0.CtrlRegs.Status.IM )
                        	lpPsxHWM[ 0x1074 ] = CPU->CP0.CtrlRegs.Status.IM;
			return( lpPsxHWM[ 0x1074 ] );

		case 0x1810:
//			hard = GPU_readData();
			return hard;
		case 0x1814:
//			hard = GPU_readStatus();
			return hard;

		//	Read the pChan0 Stream
		case 0x1820:
                	hard = PSX_MDEC->Read( 0 );
                        break;

                //	read the pChan1 Stream
		case 0x1824:
                	hard = PSX_MDEC->Read( 1 );
                        break;

                //	Read individual properties of DMA0 - MDEC IN
		case 0x1080:
                	hard = PSX_MDEC->Address[ 0 ];	//	Address
                        break;

		case 0x1084:
                	hard = PSX_MDEC->Count[ 0 ];	//	Count
                        break;

		case 0x1088:
                	hard = PSX_MDEC->Control[ 0 ];	//	Control
                        break;

                //	again repeat for properties of DMA1 - MDEC OUT
		case 0x1090:
                	hard = PSX_MDEC->Address[ 1 ];
                        break;
		case 0x1094:
                	hard = PSX_MDEC->Count[ 1 ];
                        break;
		case 0x1098:
                	hard = PSX_MDEC->Control[ 1 ];
                        break;

		case 0x10a0:
			return HW_DMA2_MADR;
		case 0x10a4:
			return HW_DMA2_BCR;
		case 0x10a8:
			return HW_DMA2_CHCR;
		case 0x10b0:
			return HW_DMA3_MADR;
		case 0x10b4:
			return HW_DMA3_BCR;
		case 0x10b8:
			return HW_DMA3_CHCR;

                //	DMA Interrupt Request Control Register
		case 0x10f0:
                	if ( lpPsxHWM[ 0x10F0 ] != PSX_DMA->IrqCtrl )
                        	lpPsxHWM[ 0x10F0 ] = PSX_DMA->IrqCtrl;
		        return( lpPsxHWM[ 0x10F0 ] ); // dma rest channel

                //	DMA Interrupt Mask Control Register
		case 0x10f4:
                	if ( lpPsxHWM[ 0x10F4 ] != PSX_DMA->IrqMask )
                        	lpPsxHWM[ 0x10F4 ] = PSX_DMA->IrqMask;
			return( lpPsxHWM[ 0x10F4 ] ); // interrupt enabler?

		// time for rootpsx_counters->lpCounters :)
		case 0x1100:
			hard = psx_counters->ReadCount(0);
			return hard;
		case 0x1104:
			hard = psx_counters->lpCounters[0].mode;
			return hard;
		case 0x1108:
			hard = psx_counters->lpCounters[0].target;
			return hard;
		case 0x1110:
			hard = psx_counters->ReadCount(1);
			return hard;
		case 0x1114:
			hard = psx_counters->lpCounters[1].mode;
			return hard;
		case 0x1118:
			hard = psx_counters->lpCounters[1].target;
			return hard;
		case 0x1120:
			hard = psx_counters->ReadCount(2);
			return hard;
		case 0x1124:
			hard = psx_counters->lpCounters[2].mode;
			return hard;
		case 0x1128:
			hard = psx_counters->lpCounters[2].target;
			return hard;

		default:
			hard = lpPsxHWM[ Index ];
			return hard;
	}
}
//---------------------------------------------------------------------------
//	As with the read versions, the write ones only support the desired bit
// size accessing for individual devices
//---------------------------------------------------------------------------
void 	__fastcall	HW :: HWWrite8( WORD Index, BYTE Value )
{
	switch ( Index ) {
		case 0x1040:
                	sioWrite8( Value );
                        break;
		case 0x1050:
                	sioWrite8( Value );
                        break;//serial port
		case 0x1800:
                	CDROM->CDWrite0( Value );
                        break;
		case 0x1801:
                	CDROM->CDWrite1( Value );
                        break;
		case 0x1802:
                	CDROM->CDWrite2( Value );
                        break;
		case 0x1803:
                	CDROM->CDWrite3( Value );
                        break;
		default:
			lpPsxHWM[ Index ] = Value;
                        break;
	}
}
//---------------------------------------------------------------------------
// 1f8010f4 and 1f8010f6
//---------------------------------------------------------------------------
void 	__fastcall	HW :: HWWrite16( WORD Index, WORD Value )
{
	switch ( Index ) {
		//serial port ;P
	//  	case 0x1050: serial_write16(Value); break;
	//	case 0x105a: serial_control_write(Value);break;
	//	case 0x105e: serial_baud_write(Value); break;
	//	case 0x1054: serial_status_write(Value); break;

		case 0x1070: //HW_LOG("IREG 16bit write %x\n", Value);
			//if ( Config.Sio )
                        //	lpPsxHWM[ 0x1070 ] |= 0x80;

			//if ( Config.SpuIrq )
                        //	lpPsxHWM[ 0x1070 ] |= 0x200;

			lpPsxHWM[ 0x1070 ] = Value;
                        if ( lpPsxHWM[ 0x1070 ] & lpPsxHWM[ 0x1074 ] )
                        	CPU->SignalException( CAUSE_INT, (void*)&lpPsxHWM[ 0x1070 ] );
			return;

		case 0x1074: //HW_LOG("IMASK 16bit write %x\n", Value);
			lpPsxHWM[ 0x1074 ] = Value;
                        if ( lpPsxHWM[ 0x1070 ] & lpPsxHWM[ 0x1074 ] )
                        	CPU->SignalException( CAUSE_INT, (void*)&lpPsxHWM[ 0x1070 ] );
			return;

		case 0x1100:
			psx_counters->WriteCount(0, Value); return;
		case 0x1104:
			psx_counters->WriteMode(0, Value); return;
		case 0x1108:
			psx_counters->WriteTarget(0, Value); return;

		case 0x1110:
			psx_counters->WriteCount(1, Value); return;
		case 0x1114:
			psx_counters->WriteMode(1, Value); return;
		case 0x1118:
			psx_counters->WriteTarget(1, Value); return;

		case 0x1120:
			psx_counters->WriteCount(2, Value); return;
		case 0x1124:
			psx_counters->WriteMode(2, Value); return;
		case 0x1128:
			psx_counters->WriteTarget(2, Value); return;

		default:
			if ((Index&0xfff0) == 0x1040) {
				sioWrite16( Index, Value );
				break;
			}

			if (Index>=0x1f801c00 && Index<0x1f801e00) {
//            	SPU_writeRegister(Index, Value);
				break;
			}

			lpPsxHWM[ Index ] = Value;
	}
}
//---------------------------------------------------------------------------
#define	DMA_INTERRUPT( n )	if ( HW_DMA_ICR & ( 1 << ( 16 + n ) ) ) \
				{ \
                                	HW_DMA_ICR |= ( 1 << ( 24 + n ) ); \
                                        lpPsxHWM[ 0x1070 ] |= 8; \
                                        PSX_DMA->DoDMA( n ); \
                                }
//---------------------------------------------------------------------------
void 	__fastcall	HW :: HWWrite32( WORD Index, DWORD Value )
{
	switch (Index) {
	    case 0x1040: sioWrite32(Value); break;
	//	case 0x1050: serial_write32(Value); break;//serial port
		case 0x1060:
			lpPsxHWM[ Index] = Value;
			return; // Ram size
		case 0x1070: //HW_LOG("IREG 32bit write %lx\n", Value);
			//if ( Config.Sio )
                        //	lpPsxHWM[ 0x1070] |= 0x80;
                        //
			//if ( Config.SpuIrq )
                        //	lpPsxHWM[ 0x1070] |= 0x200;

			lpPsxHWM[ 0x1070 ] = Value;
			return;
		case 0x1074:
			lpPsxHWM[ 0x1074 ] = Value;
			return;
		case 0x1080:
			PSX_MDEC->Address[ 0 ] = Value;
                        return; // DMA0 madr
		case 0x1084:
			PSX_MDEC->Count[ 0 ] = Value;
                        return; // DMA0 bcr
		case 0x1088:
			PSX_MDEC->Control[ 0 ] = Value;        // DMA0 chcr (MDEC in DMA)
			DMA_INTERRUPT( 0 );
			return;

		case 0x1090:
			PSX_MDEC->Address[ 1 ] = Value;
                        return; // DMA1 madr
		case 0x1094:
			PSX_MDEC->Count[ 1 ] = Value;
                        return; // DMA1 bcr
		case 0x1098:
			PSX_MDEC->Control[ 1 ] = Value;        // DMA1 chcr (MDEC out DMA)
			DMA_INTERRUPT( 1 );
			return;

		case 0x10a0:
			HW_DMA2_MADR = Value; return; // DMA2 madr
		case 0x10a4:
			HW_DMA2_BCR  = Value; return; // DMA2 bcr
		case 0x10a8:
			HW_DMA2_CHCR = Value;        // DMA2 chcr (GPU DMA)
			//hwDma2(HW_DMA2_MADR, HW_DMA2_BCR, HW_DMA2_CHCR);
			HW_DMA2_CHCR &= ~0x01000000;
			DMA_INTERRUPT(2);
			return;

		case 0x10b0:
			HW_DMA3_MADR = Value; return; // DMA3 madr
		case 0x10b4:
			HW_DMA3_BCR  = Value; return; // DMA3 bcr
		case 0x10b8:
			HW_DMA3_CHCR = Value;        // DMA3 chcr (CDROM DMA)
			CDROM->hwDma3( HW_DMA3_MADR, HW_DMA3_BCR, HW_DMA3_CHCR );
			HW_DMA3_CHCR &= ~0x01000000;
			DMA_INTERRUPT(3);

			return;

		case 0x10c0:
			HW_DMA4_MADR = Value; return; // DMA4 madr
		case 0x10c4:
			HW_DMA4_BCR  = Value; return; // DMA4 bcr
		case 0x10c8:
			HW_DMA4_CHCR = Value;        // DMA4 chcr (SPU DMA)
			//hwDma4(HW_DMA4_MADR, HW_DMA4_BCR, HW_DMA4_CHCR);
			HW_DMA4_CHCR &= ~0x01000000;
			DMA_INTERRUPT(4);
			return;

		case 0x10d0: break; //DMA5write_madr();
		case 0x10d4: break; //DMA5write_bcr();
		case 0x10d8: break; //DMA5write_chcr(); // Not yet needed??
		case 0x10e0:
			HW_DMA6_MADR = Value; return; // DMA6 bcr
		case 0x10e4:
			HW_DMA6_BCR  = Value; return; // DMA6 bcr
		case 0x10e8:
			HW_DMA6_CHCR = Value;         // DMA6 chcr (OT clear)
			//hwDma6(HW_DMA6_MADR, HW_DMA6_BCR, HW_DMA6_CHCR);
			HW_DMA6_CHCR &= ~0x01000000;
			DMA_INTERRUPT(6);
			return;

                //	Programmable DMA Interrupt Request Control Register
                // is a value is written to this address then a DMA interrupt
                // is initialised
                case ( 0x10F0 ) : {
                	PSX_DMA->IrqCtrl = Value;
                        lpPsxHWM[ 0x10F0 ] = Value;
                        if ( lpPsxHWM[ 0x10F0 ] & lpPsxHWM[ 0x10F4 ] )
                        	PSX_DMA->DoDMA( Value );
                        break;
                }

                //	DMA Irq Mask register
                case ( 0x10F4 ) : {
                	PSX_DMA->IrqMask = Value;
                        lpPsxHWM[ 0x10F4 ] = Value;
                        if ( lpPsxHWM[ 0x10F4 ] & lpPsxHWM[ 0x10F0 ] )
                        	PSX_DMA->DoDMA( Value );
                        break;
                }

		case 0x1810:
//			GPU_writeData(Value); return;
		case 0x1814:
//			GPU_writeStatus(Value); return;

		//case 0x1820:
		//	mdecWrite0(Value); break;
		//case 0x1824:
		//	mdecWrite1(Value); break;

		case 0x1100:
			psx_counters->WriteCount(0, Value & 0xffff); return;
		case 0x1104:
			psx_counters->WriteMode(0, Value); return;
		case 0x1108:
			psx_counters->WriteTarget(0, Value & 0xffff); return; //  HW_DMA_ICR&= (~Value)&0xff000000;

		case 0x1110:
			psx_counters->WriteCount(1, Value & 0xffff); return;
		case 0x1114:
			psx_counters->WriteMode(1, Value); return;
		case 0x1118:
			psx_counters->WriteTarget(1, Value & 0xffff); return;

		case 0x1120:
			psx_counters->WriteCount(2, Value & 0xffff); return;
		case 0x1124:
			psx_counters->WriteMode(2, Value); return;
		case 0x1128:
			psx_counters->WriteTarget(2, Value & 0xffff); return;
		default:
			lpPsxHWM[ Index ] = Value;
	}
}
//---------------------------------------------------------------------------
