//---------------------------------------------------------------------------
//
//	$FileName	::	HW.h
//	$Purpose	::	Header file for hardware memory mapping
//
//---------------------------------------------------------------------------
#ifndef HWH
#define HWH
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
#ifndef SPUH
#include "spu.h"
#endif
//---------------------------------------------------------------------------
//	Base address for the hardware mapped memory region
//---------------------------------------------------------------------------
#define	HWM_BASE	0x1F800000
#define HWM_SIZE	0x0000FFFF
//---------------------------------------------------------------------------
//	internal descriptions
//---------------------------------------------------------------------------
#define	SCRATCH_BASE	HWM_BASE
#define SCRATCH_SIZE	1024
//---------------------------------------------------------------------------
//	Unknown areas within the HWM_SEGMENT
//---------------------------------------------------------------------------
#define	HWM_UNKNOWN0	( HWM_BASE + 0x1010 )	//	BIOS writes to this first thing
#define SPU_DELAY	( HWM_BASE + 0x1014 )
#define DV5_DELAY	( HWM_BASE + 0x1018 )
#define COM_DELAY	( HWM_BASE + 0x1020 )
//---------------------------------------------------------------------------
//	Memory Card
//---------------------------------------------------------------------------
#define MEMCARD_BASE	( HWM_BASE + 0x1040 )
#define MEMCARD_DATA	( MEMCARD_BASE )
#define MEMCARD_STATUS	( MEMCARD_BASE + 0x0004 )
#define MEMCARD_CTRL	( MEMCARD_BASE + 0x000A )
#define MEMCARD_BAUD	( MEMCARD_BASE + 0x000E )
//---------------------------------------------------------------------------
//	Serial I/O ( IF FITTED )
//---------------------------------------------------------------------------
#define SERIAL_BASE	( HWM_BASE + 0x1050 )
#define SERIAL_DATA	( SERIAL_BASE )
#define SERIAL_STATUS	( SERIAL_BASE + 0x0004 )
#define SERIAL_CTRL	( SERIAL_BASE + 0x000A )
#define SERIAL_BAUD	( SERIAL_BASE + 0x000E )
//---------------------------------------------------------------------------
//	RAM SIZE ( Usually contains 0xB88 )
//---------------------------------------------------------------------------
#define	RAM_SIZE	( HWM_BASE + 0x1060 )
//---------------------------------------------------------------------------
//	IRQ REG and IRQ MASK
// Irq is signaled if ( IRQ_REG & IRQ_MASK ) else ignored
//---------------------------------------------------------------------------
#define IRQ_REG		( HWM_BASE + 0x1070 )
#define IRQ_MASK	( HWM_BASE + 0x1074 )
//---------------------------------------------------------------------------
//	DMA Channel Control Registers   ( 0-6 )
//---------------------------------------------------------------------------
#define	DMA0_BASE	( HWM_BASE + 0x1080 )   //      MDEC IN <--
#define DMA0_ADDR	( DMA0_BASE )
#define DMA0_SIZE	( DMA0_BASE + 0x0004 )
#define DMA0_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define	DMA1_BASE	( HWM_BASE + 0x1090 )   //      MDEC OUT -->
#define DMA1_ADDR	( DMA0_BASE )
#define DMA1_SIZE	( DMA0_BASE + 0x0004 )
#define DMA1_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define	DMA2_BASE	( HWM_BASE + 0x10A0 )   //      GPU
#define DMA2_ADDR	( DMA0_BASE )
#define DMA2_SIZE	( DMA0_BASE + 0x0004 )
#define DMA2_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define	DMA3_BASE	( HWM_BASE + 0x10B0 )   //      CDROM
#define DMA3_ADDR	( DMA0_BASE )
#define DMA3_SIZE	( DMA0_BASE + 0x0004 )
#define DMA3_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define	DMA4_BASE	( HWM_BASE + 0x10C0 )   //      SPU
#define DMA4_ADDR	( DMA0_BASE )
#define DMA4_SIZE	( DMA0_BASE + 0x0004 )
#define DMA4_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define	DMA5_BASE	( HWM_BASE + 0x10D0 )   //      ???
#define DMA5_ADDR	( DMA0_BASE )
#define DMA5_SIZE	( DMA0_BASE + 0x0004 )
#define DMA5_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define	DMA6_BASE	( HWM_BASE + 0x10E0 )   //      MEMORY ( GPU Ordering Table )
#define DMA6_ADDR	( DMA0_BASE )
#define DMA6_SIZE	( DMA0_BASE + 0x0004 )
#define DMA6_CTRL	( DMA0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define DMA_IRQ_CTRL	( HWM_BASE + 0x10F0 )
#define DMA_IRQ_MASK	( HWM_BASE + 0x10F4 )
//---------------------------------------------------------------------------
//	Root Counters
//---------------------------------------------------------------------------
#define ROOTCOUNT0_BASE	( HWM_BASE + 0x1100 )
#define ROOT0_COUNT	( ROOTCOUNT0_BASE )
#define ROOT0_MODE	( ROOTCOUNT0_BASE + 0x0004 )
#define ROOT0_TARGET	( ROOTCOUNT0_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define ROOTCOUNT1_BASE	( HWM_BASE + 0x1110 )
#define ROOT1_COUNT	( ROOTCOUNT1_BASE )
#define ROOT1_MODE	( ROOTCOUNT1_BASE + 0x0004 )
#define ROOT1_TARGET	( ROOTCOUNT1_BASE + 0x0008 )
//---------------------------------------------------------------------------
#define ROOTCOUNT2_BASE	( HWM_BASE + 0x1120 )
#define ROOT2_COUNT	( ROOTCOUNT2_BASE )
#define ROOT2_MODE	( ROOTCOUNT2_BASE + 0x0004 )
#define ROOT2_TARGET	( ROOTCOUNT2_BASE + 0x0008 )
//---------------------------------------------------------------------------
//	CDROM
//---------------------------------------------------------------------------
#define CDROM_BASE	( HWM_BASE + 0x1800 )
#define CDROM_REG0	( CDROM_BASE )
#define CDROM_REG1	( CDROM_BASE + 0x0001 )
#define CDROM_REG2	( CDROM_BASE + 0x0002 )
#define CDROM_REG3	( CDROM_BASE + 0x0003 )
//---------------------------------------------------------------------------
//	GPU
//---------------------------------------------------------------------------
#define	GPU_BASE	( HWM_BASE + 0x1814 )
#define GPU_REG0	( GPU_BASE )
#define GPU_REG1	( GPU_BASE + 0x0004 )
//---------------------------------------------------------------------------
//	MDEC
//---------------------------------------------------------------------------
#define MDEC_BASE	( HWM_BASE + 0x1820 )
#define MDEC_REG0	( MDEC_BASE )
#define MDEC_REG1	( MDEC_BASE + 0x0004 )
//---------------------------------------------------------------------------
//	SPU
//---------------------------------------------------------------------------
#define SPU_BASE	( HWM_BASE + 0x1C00 )
#define SPU_VOL_LEFT	( SPU_BASE )
#define SPU_VOL_RIGHT	( SPU_BASE + 0x0002 )
//---------------------------------------------------------------------------
#define DMA_IRQ_BASE	( HWM_BASE + 0x10F0 )
#define DMA_IRQ_CTL	( DMA_IRQ_BASE + 0x0000 )
#define DMA_IRQ_MASK	( DMA_IRQ_BASE + 0x0004 )
//---------------------------------------------------------------------------
//	PlayStation Hardware Memory Mapped Emulation Layer Class
//---------------------------------------------------------------------------
class	HW : public TObject
	{
        	friend	class	PSX_MEM;
        	friend	class	CPSX_DMA;
                friend	class	CPSX_MDEC;
        	protected:
                private:
                	DWORD	*lpPsxHWM;
                	BYTE	__fastcall	HWRead8	( WORD Index );
                        WORD	__fastcall	HWRead16( WORD Index );
                        DWORD	__fastcall	HWRead32( WORD Index );
                        void	__fastcall	HWWrite8( WORD Index, BYTE Value );
                        void	__fastcall	HWWrite16( WORD Index, WORD Value );
                        void	__fastcall	HWWrite32( WORD Index, DWORD Value );
                public:
                		__fastcall	HW	( void );
                        	__fastcall	~HW	( void );

                	int	__fastcall	Initialise	( void );
                        void	__fastcall	Terminate	( void );

                        void	__fastcall	Reset	( void );	//	Resets the Hardware Emulation Layer to a known state.
                __published:
                	__property	BYTE	dwHWM8 [ WORD Index ] = { read=HWRead8,  write=HWWrite8  };
                        __property	WORD	dwHWM16[ WORD Index ] = { read=HWRead16, write=HWWrite16 };
                        __property	DWORD	dwHWM32[ WORD Index ] = { read=HWRead32, write=HWWrite32 };
                        __property	DWORD	dwHWM  [ WORD Index ] = { read=HWRead32, write=HWWrite32 };
        };
//---------------------------------------------------------------------------
//	We need to redirect the reads/writes to/from the HWM array to
//	HW::dwHWM
//---------------------------------------------------------------------------
__declspec( dllimport )	HW	*lpHW;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
