//---------------------------------------------------------------------------
//
//	$FileName	::	Kernel.cpp
//	$Purpose	::	PlayStation Kernel Emulation Layer
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "Kernel.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
__declspec( dllexport )	PSX_Kernel	*lpKernel = NULL;
__declspec( dllexport ) HANDLE		hStdIn;		//	KERNEL STDIN
__declspec( dllexport ) HANDLE		hStdOut;	//	KERNEL STDOUT
__declspec( dllexport ) HANDLE		hStdErr;	//	KERNEL STDERR
//---------------------------------------------------------------------------
__fastcall	PSX_Kernel :: PSX_Kernel( void ) : TObject()
{
	lpKMemory = new DWORD[ KERNEL_SIZE ];
        ZeroMemory( lpKMemory, sizeof( DWORD ) * KERNEL_SIZE );

        //	Create OS Core
        if ( lpOSCore == NULL )
        	lpOSCore = new PSX_OSCore;
}
//---------------------------------------------------------------------------
__fastcall	PSX_Kernel :: ~PSX_Kernel( void )
{
	if ( lpKMemory != NULL )        //	DO NOT DELETE THIS POINTER, AS THE PSX_MEM CLASS
        	lpKMemory = NULL;	//	WILL CRASH
}
//---------------------------------------------------------------------------
//	protected members
//---------------------------------------------------------------------------
//	Install Kernel / OS Core Default Exception Handlers
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: InstallExceptionHandlers( void )
{
	CPU->Initialise();	//	Initialise CPU Interface
	PsxBIOS->Initialise();	//	Initialise BIOS Interface
	lpOSCore->Initialise();	//	And OS Core
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: InstallDevices( void )
{
	lpHW->Initialise();
        CDROM->Initialise();
        PsxSIO->Initialise();
        PsxSPU->Initialise();
        lpGTE->Initialise();
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: SysInitMemory( void )
{
	if ( lpKMemory == NULL )
        	SysInitKMem();	//	Initialise Kernel specific area, first 64K

        KernelRedirection( 0 );	//	Turn off i/o redirection
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: InitDefInt( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: FlushStdInOutPut( void )
{

	//	Flush the stdin, stdout pipes

}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: KernelRedirection( int Flag )
{
}
//---------------------------------------------------------------------------
LPVOID	__fastcall	PSX_Kernel :: PatchA0Table( WORD Index, LPVOID lpPtr )
{
	return( NULL );
}
//---------------------------------------------------------------------------
//	private members
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: InitException( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: ExceptionHandler( void )
{
}
//---------------------------------------------------------------------------
//	KERNEL :: SysInitKMem.	Called from BIOS / OS CORE ONLY
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: SysInitKMem( void )
{

	if ( PsxMem == NULL )
        	PsxMem = new PSX_MEM;


        //	Set the stack
        //	frame
        //	global
        //	initialise the heap

        SysInitMemory();
}
//---------------------------------------------------------------------------
//	Add an event control buffer to the kernel event list
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: SysEnqIntRP( WORD Index, LPVOID lpBuffer )
{
}
//---------------------------------------------------------------------------
//	Remove an event control buffer from the kernel event list
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: SysDeqIntRP ( WORD Index, LPVOID lpBuffer )
{
}
//---------------------------------------------------------------------------
//	return the next event control buffer index
//---------------------------------------------------------------------------
LPVOID	__fastcall	PSX_Kernel :: get_free_EvCB_slot( void )
{
	return( NULL );
}
//---------------------------------------------------------------------------
//	return the next thread control buffer index
//---------------------------------------------------------------------------
LPVOID	__fastcall	PSX_Kernel :: get_free_TCB_slot( void )
{
	return( NULL );
}
//---------------------------------------------------------------------------
//	abort all counter operations and set to non-running state
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: ChangeClearRCnt( void )
{
}
//---------------------------------------------------------------------------
//	obtain direct data from the cd rom device
//	ie CDROM->Read( buffer, count ) at the lowest level allowed. This is
//	called from the BIOS / OS Core
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: _cdevinput( void )
{
 /*
        	//	MIPS 32 ISA BIOS :: _cdevinput	
0xBFC03190:
	BEQ	a0, zero, 0xBFC031A0
        NOP
        BNE	a1, zero, 0xBFC031A8
        NOP
0xBFC031A0:
        JR	ra
        ADDU	v0, zero, zero
0xBFC031A8:
        LB	v0, 0x0000( a0 )
        ADDU	v1, a0, zero
        BEQ	v0, zero, 0xBFC031C8
        ADDIU	a0, a0, 0x0001
0xBFC031B8:
        LB	v0, 0x0000( a0 )
        ADDIU	a0, a0, 0x0001
        BNE	v0, zero, 0xBFC031B8
        NOP
0xBFC031C8:
        LB	v0, 0x0000( a0 )
        ADDIU	a0, a0, 0xFFFF
        ADDIU	a0, a0, 0x0001
        ADDIU	a1, a1, 0x0001
        BEQ	v0, zero, 0xBFC031F4
        SB	v0, -0x0001( a0 )
0xBFC031E0:
        LB	v0, 0x0000( a1 )
        ADDIU	a0, a0, 0x0001
        ADDIU	a1, a1, 0x0001
        BNE	v0, zero, 0xBFC031E0
        SB	v0, -0x0001( a0 )
0xBFC031F4:
        ADDU	v0, v1, zero
        JR	ra
        NOP
 */
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: _cdevscan( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: ioabort( void )
{
}
//---------------------------------------------------------------------------
char	__fastcall	PSX_Kernel :: _circgetc( LPVOID lpBuffer )
{
	return( NULL );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: _circputc( char ch, LPVOID lpBuffer )
{
}
//---------------------------------------------------------------------------
//	public members
//---------------------------------------------------------------------------
int	__fastcall	PSX_Kernel :: Initialise( void )
{
	SysInitMemory();	//	Initialise the kernel interface stubs
	return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: Terminate( void )
{
	ioabort();
        FlushStdInOutPut();
        KernelRedirection( 0 );
        FlushStdInOutPut();
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: Reset( void )
{
	PsxBIOS->Reset();
}
//---------------------------------------------------------------------------
DWORD	__fastcall	PSX_Kernel :: SystemError( LPSTR lpszMessage )
{
	MessageBox( NULL, lpszMessage, "KERNEL", MB_OK | MB_ICONEXCLAMATION );
	return( 0 );
}
//---------------------------------------------------------------------------
//	Console Debugging for Kernel
//---------------------------------------------------------------------------
void	__fastcall	PSX_Kernel :: ConsoleMode( void )
{       bool	fDone = false;
	LPSTR	lpszCLI = NULL;

        fprintf( stdout, "PlayStation RunTime Kernel Command Console\n" );
        fprintf( stdout, "( C ) Copyright 2001 - 2003 Chris Brooklyn\n\n" );

        while ( !fDone ) {

        	lpszCLI = new char[ MAX_PATH ];
                if ( lpszCLI == NULL )
                	fDone = true;

                ZeroMemory( lpszCLI, MAX_PATH );

                printf( " >" );

                scanf( "%s\n", lpszCLI );

                if ( strlen( lpszCLI ) > 0 ) {	//	Line entered
                	LPSTR lpszCommand = strtok( lpszCLI, "\t\n " );
                        if ( strlen( lpszCommand ) > 0 ) {	//	Command

                        	if ( strcmp( lpszCommand, "quit" ) == 0 ) {
                                	//	Quit
                                        fDone = true;
                                }

                                if ( strcmp( lpszCommand, "diss" ) == 0 ) {
                                	printf( "disassemble\n" );
                                }
                        }
                } else {
                	delete []lpszCLI;
                }
        }
}
//---------------------------------------------------------------------------
