//---------------------------------------------------------------------------+
//
//	$FileName	::	Memory.cpp
//	$Purpose	::	PlayStation Memory Manager Unit Emulation
//	$Author		::	Chris Brooklyn
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "Memory.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#define	CLASSNAME	"MEMORY"
//---------------------------------------------------------------------------
__declspec( dllexport ) PSX_MEM	*PsxMem = NULL;
//---------------------------------------------------------------------------
static	FILE	*fMemLog = NULL;
//---------------------------------------------------------------------------
__fastcall	PSX_MEM :: PSX_MEM( void ) : TObject()
{
	fMemLog = fopen( "MEMORY.LOG", "wt" );

        if ( fMemLog == NULL ) {
        	fprintf( stderr, "Memory Emulation Layer [ FAILED TO CREATE LOG FILE ].\n" );
                fMemLog = stderr;
        }

	fprintf( fMemLog, "PlayStation Memory Emulation Layer Construction.\n" );
        lpPsxRAM = NULL;	//	RAM
        lpPsxPPM = NULL;	//	PPM	Parallel Pheripheral Mode? Has something to do with the Parallel I/O Port (CARTRIDGE?)
	//!	Create the hardware emulation layer class, which also creates things
        //! like the root counters, dma, mdec, sio, spu, cdrom, ie it starts the rest
        //! of the 'hardware' that is needed for a psx.
        lpHW = new HW;
}
//---------------------------------------------------------------------------
__fastcall	PSX_MEM :: ~PSX_MEM( void )
{
	//	Initiate cascade shutdown within hardware emulation layer
	if ( lpHW != NULL ) {
        	delete lpHW;
        }

        //	Delete any memory reserved by this class
	if ( lpPsxRAM != NULL ) {
                delete []lpPsxRAM;
        }

        if ( lpPsxPPM != NULL ) {
        	delete []lpPsxPPM;
        }

        //if ( lpPsxROM != NULL ) {
        //	delete []lpPsxROM;
        //}

//        if ( lpMemLUT != NULL ) {
//        	delete []lpMemLUT;
//        }

        fclose( fMemLog );

	delete this;
        //	The Memory Emulation Layer is now no longer
}
//---------------------------------------------------------------------------
//	Does an entry exist in the tlb array ?
//---------------------------------------------------------------------------
bool	__fastcall	PSX_MEM :: TLBProbe( void )//DWORD Index )
{	bool	found  = false;	//	Entry found in array ?
	bool	mfound = false;	//	more than one entry in array ?
        int	Index = CPU->CP0.CtrlRegs.Index.Index;

	//	Probe for the entry referenced by the CP0 control register Index and Random

        //	Bad index, below kernel/oscore first free or above the maximum
        // number of allowable entries
        if (	( CPU->CP0.CtrlRegs.Index.Index <= CPU->CP0.CtrlRegs.Wired.Wired ) ||
        	( CPU->CP0.CtrlRegs.Index.Index >= MAX_TLBENTRIES ) ) {
                CPU->SignalException( CAUSE_TLBM, (void*)&CPU->CP0.CtrlRegs.Index );
                CPU->CP0.CtrlRegs.Index.ProbeFail = true;
                return( false );
        }

        //	Also check the random register
        if (	( CPU->CP0.CtrlRegs.Random.Random <= CPU->CP0.CtrlRegs.Wired.Wired ) ||
        	( CPU->CP0.CtrlRegs.Random.Random >= MAX_TLBENTRIES ) ) {
                CPU->SignalException( CAUSE_TLBM, (void*)&CPU->CP0.CtrlRegs.Index );
                CPU->CP0.CtrlRegs.Index.ProbeFail = true;
                return( false );
        }

        //	Otherwise loop through the valid array entries, for global
        // matches, vpn2 match, pagemask match with/without asid matching

        for ( int i=0; i<MAX_TLBENTRIES; i++ ) {
        	LPTLBENTRY	lpEntry = lpTLBArray[ i ];

                if ( lpEntry != NULL ) {

                	//	Does this entry match ?
                	if ( ( ( lpEntry->vpn2 & ~ lpEntry->PageMask ) == ( CPU->CP0.CtrlRegs.EntryHi.VPN2 & ~lpEntry->PageMask ) ) &&
                    		( ( lpEntry->Global ) || ( lpEntry->ASID == CPU->CP0.CtrlRegs.EntryHi.ASID ) ) ) {

                                if ( found ) {	//	all ready found this match

                                	//	all ready have multi-match signal
                                	if ( mfound )
                                        	//	continue with loop
                                        	continue;

                                        mfound = true;
                                        CPU->CP0.CtrlRegs.Index.Index = i;

                                } else found = true;

                        }
                } else
                	//	as it says
                	continue;

        }

        //	check for match and multi-match

        //	match and not multi-match
        if ( ( found ) & ( !mfound ) )
        	CPU->CP0.CtrlRegs.Index.ProbeFail = false;
        else
        // 	else not found and either multi-match or no multi-match
        	CPU->CP0.CtrlRegs.Index.ProbeFail = true;

        //	if multi-match, whether found or not signal exception
        if ( mfound )
        	CPU->SignalException( CAUSE_TLBL, (void*)&Index );

	//	return fail or succes of operation, not needed at mips level
        // just a quick way of my code not running if the code deems the
        // access invalid, base on information given FROM the mips level.
        return( CPU->CP0.CtrlRegs.Index.ProbeFail );
}
//---------------------------------------------------------------------------
//	Read into the processor the information stored in the tlb array indexed
// by the CP0 Index Register.
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM	:: TLBRead( void )
{	LPTLBENTRY	lpEntry;

	//	As this would probably be only called from the BIOS/KERNEL/OSCORE
        // and as such the cpu would be in KERNEL mode so check for the correct
        // operating mode

        if ( CPU->CPUMode != CPUMODE_KERNEL )
        	//	no, barf out. Should be a really signal of some sort through
                // the mips layers?
        	return;

	//	Does the TLB Entry exist
        if ( !TLBProbe() )
        	return;	//	No, the TLBProbe method will cause an exception if needed, so don't check just bye

        lpEntry = lpTLBArray[ CPU->CP0.CtrlRegs.Index.Index ];
        if ( lpEntry == NULL )	//	this should never happen if we are here, but just on the safe side
        	return;
        //
        // ....	Otherwise the entry is present so proceed to load into
        // the corresponding registers in the CP0 processor.
        //


        //	Is it valid tlb entry ? IE does this entry corresponding to virtual mapping or not
        if ( !( lpEntry->V0 | lpEntry->V1 ) ) {
        	//	No signal exception
        	CPU->SignalException( CAUSE_TLBL, NULL );
        	return;
        }

        //	Read corresponding details into EntryLo0
        CPU->CP0.CtrlRegs.EntryLo0.Valid = lpEntry->V0;
        CPU->CP0.CtrlRegs.EntryLo0.Dirty = lpEntry->D0;
        CPU->CP0.CtrlRegs.EntryLo0.Global = lpEntry->Global;
        CPU->CP0.CtrlRegs.EntryLo0.PFN = lpEntry->PFN0;
        CPU->CP0.CtrlRegs.EntryLo0.CCA = lpEntry->C0;

        //	Read corresponding details into EntryLo1
        CPU->CP0.CtrlRegs.EntryLo1.Valid = lpEntry->V1;
        CPU->CP0.CtrlRegs.EntryLo1.Dirty = lpEntry->D1;
        CPU->CP0.CtrlRegs.EntryLo1.Global = lpEntry->Global;
        CPU->CP0.CtrlRegs.EntryLo1.PFN = lpEntry->PFN1;
        CPU->CP0.CtrlRegs.EntryLo1.CCA = lpEntry->C1;

        //	Read corresponding details into EntryHi
        CPU->CP0.CtrlRegs.EntryHi.ASID = lpEntry->ASID;
        CPU->CP0.CtrlRegs.EntryHi.VPN2 = lpEntry->vpn2;

        //	Read corresponding details into PageMask
        CPU->CP0.CtrlRegs.PageMask.Mask = lpEntry->PageMask;

        //	Valid details have now been loaded into the required registers
}
//---------------------------------------------------------------------------
//	Write a tlb entry referenced by the index register
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: TLBWriteIndexed( void )
{       LPTLBENTRY	lpEntry = NULL;

	//	Bad entry
	if ( !TLBProbe() )
        	return;

        lpEntry = lpTLBArray[ CPU->CP0.CtrlRegs.Index.Index ];
        if ( lpEntry == NULL )
        	return;

        //	Write corresponding information from EntryHi to tlb
        lpEntry->ASID = CPU->CP0.CtrlRegs.EntryHi.ASID;
        lpEntry->vpn2 = CPU->CP0.CtrlRegs.EntryHi.VPN2;

        //	And for the PageMask register
        lpEntry->PageMask = CPU->CP0.CtrlRegs.PageMask.Mask;

        //	Set the global bit
        lpEntry->Global = ( CPU->CP0.CtrlRegs.EntryLo0.Global & CPU->CP0.CtrlRegs.EntryLo1.Global );

        //	set from EntryLo0
        lpEntry->V0 = CPU->CP0.CtrlRegs.EntryLo0.Valid;
        lpEntry->D0 = CPU->CP0.CtrlRegs.EntryLo0.Dirty;
        lpEntry->C0 = CPU->CP0.CtrlRegs.EntryLo0.CCA;
        lpEntry->PFN0 = CPU->CP0.CtrlRegs.EntryLo0.PFN;

        //	and EntryLo1
        lpEntry->D1 = CPU->CP0.CtrlRegs.EntryLo1.Valid;
        lpEntry->V1 = CPU->CP0.CtrlRegs.EntryLo1.Dirty;
        lpEntry->C1 = CPU->CP0.CtrlRegs.EntryLo1.CCA;
        lpEntry->PFN1 = CPU->CP0.CtrlRegs.EntryLo1.PFN;

}
//---------------------------------------------------------------------------
//	As above but use the Random register
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: TLBWriteRandom( void )
{       LPTLBENTRY	lpEntry = NULL;

	//	Bad entry
	if ( !TLBProbe() )
        	return;

        lpEntry = lpTLBArray[ CPU->CP0.CtrlRegs.Random.Random ];
        if ( lpEntry == NULL )
        	return;

        //	Write corresponding information from EntryHi to tlb
        lpEntry->ASID = CPU->CP0.CtrlRegs.EntryHi.ASID;
        lpEntry->vpn2 = CPU->CP0.CtrlRegs.EntryHi.VPN2;

        //	And for the PageMask register
        lpEntry->PageMask = CPU->CP0.CtrlRegs.PageMask.Mask;

        //	Set the global bit
        lpEntry->Global = ( CPU->CP0.CtrlRegs.EntryLo0.Global & CPU->CP0.CtrlRegs.EntryLo1.Global );

        //	set from EntryLo0
        lpEntry->V0 = CPU->CP0.CtrlRegs.EntryLo0.Valid;
        lpEntry->D0 = CPU->CP0.CtrlRegs.EntryLo0.Dirty;
        lpEntry->C0 = CPU->CP0.CtrlRegs.EntryLo0.CCA;
        lpEntry->PFN0 = CPU->CP0.CtrlRegs.EntryLo0.PFN;

        //	and EntryLo1
        lpEntry->V1 = CPU->CP0.CtrlRegs.EntryLo1.Valid;
        lpEntry->D1 = CPU->CP0.CtrlRegs.EntryLo1.Dirty;
        lpEntry->C1 = CPU->CP0.CtrlRegs.EntryLo1.CCA;
        lpEntry->PFN1 = CPU->CP0.CtrlRegs.EntryLo1.PFN;
}
//---------------------------------------------------------------------------
//	Initialise the memory map.
//---------------------------------------------------------------------------
int	__fastcall	PSX_MEM :: Initialise( void )
{	int 	i;

	fprintf( fMemLog, "Initialising Virtual Memory Emulation Layer.\n" );

        lpPsxRAM = new DWORD[ RAM_SIZE ];	//	Allocate ram for the system
        lpPsxPPM = new DWORD[ 0x00010000 ];	//	Cartridge Port, this should technically be part of HW

        //	memory allocation ok, now clear it
        ZeroMemory( lpPsxRAM, RAM_SIZE );
        ZeroMemory( lpPsxPPM, 0x00010000 );

        return( lpHW->Initialise() );			//	Now initialise Hardware Mapped Memory
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: Terminate( void )
{
	//	Start cascade effect in the hardware emulation layer
	lpHW->Terminate();

        //	delete memory
	if ( lpPsxRAM != NULL )
        	delete []lpPsxRAM;

        if ( lpPsxPPM != NULL )
        	delete []lpPsxPPM;
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: Reset( void )
{
	//	Clear memory contents
	ZeroMemory( lpPsxRAM, RAM_SIZE );

	ZeroMemory( lpPsxPPM, 0x00010000 );

	if ( ( !Config.InternalBIOS ) && ( Config.lpszBIOSFileName != NULL ) ) {	//	Using internal BIOS ?
        	PsxBIOS->LoadBIOS( Config.lpszBIOSFileName );
	} else  {
        	Config.InternalBIOS = 1;
                Config.lpszBIOSFileName = NULL;
        }

	lpHW->Reset();
        PsxBIOS->Reset();	//	Need to reset the BIOS Emulation Layer, regardless of
        // using internal of an external file as the BIOS, note that the BIOS does not
        // have to be from SONY but you can use your own. Write one its fun! ;-)
}
//---------------------------------------------------------------------------
//	The following methods are used for reading and writing to memory
//	NOTE that they do NOT taken into account the alignment of the address
//	ie in MIPS 32 ISA it should produced an exception if the mem address
//	is not aligned correctly, ie read 16-bit must be on a half-word boundary
//	for further information please see the MIPS 32 ISA documentation in
//	the docs directory, or if not present goto www.mips.com and search the
//	site for documentation relating to MIPS 32/64 and the R3000 series cpus.
//
//	Also this does not take into account the different operating modes of the
//	processors ( yes multiple cpus ). MIPS 32 ISA defines that the system has
//	4 (four) cpus COP0-3 COP0 is the system control processor, COP1 the FPU unit
//	COP2 implementation specific, (PSX GTE), COP2 is also defined as the FPU
//	unit for MIPS 64 ISA, COP3 implementation specific
//---------------------------------------------------------------------------
//	8-bit memory read
//---------------------------------------------------------------------------
BYTE	__fastcall	PSX_MEM :: GRAM8	( DWORD Index )
{       WORD	wSegment = ( Index >> 16 );

        //	Is referencing ROM
        if ( ( wSegment >= 0x1fc0 ) && ( wSegment <= 0x1fc8 ) ) {
        	fprintf( fMemLog, "[ ROM 8 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%02x ]\n",
                	wSegment, Index, PsxBIOS->lpPsxBIOS[ Index & 0x0007FFFF ] );
        	return( (BYTE)PsxBIOS->ROM[ Index ] );
        }

        //	BIOS ?
        if ( ( wSegment >= 0xBFC0 ) && ( wSegment <= 0xBFC8 ) ) {
        	fprintf( fMemLog, "[ BIOS 8 READ ] [ 0x0%04 ] [ 0x%08x ]\n",
                	wSegment, Index );
                //	BIOS->FuncTable[ Index & 0x00000300 ][ Index & 0x00000FF ]
                //	or something similar
                //return( (BYTE)lpPsxROM[ ( Index & 0x0007FFFF ) ] );
                return( (BYTE)PsxBIOS->ROM[ Index ] );
        }

        //	Hardware reference ?
	if ( wSegment == 0x1f80 ) {

        	//	ScratchPad RAM Access ?
                if ( ( Index >= 0x1F800000 ) && ( Index <= 0x1F801000 ) ) {
                	fprintf( fMemLog, "[ SCRATCH 8 READ ] [ 0x%04 ] [ 0x%08x ] [ 0x%02 ]\n",
                        	wSegment, Index, lpPsxPPM[ Index & 0xFFFF ] );
                	return( lpPsxPPM[ Index & 0xFFFF ] );
                //	nope, so give to Hardware Emulation Layer
                } else {
                	fprintf( fMemLog, "[ HARD 8 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%02x ]\n",
                        	wSegment, Index, lpHW->dwHWM8[ Index & 0xFFFF ] );
			return( lpHW->dwHWM8[ Index & 0xFFFF ] );
                }
	}	//	nope, so WE need to resolve the address
        else {  //	to the general ram area, technically we need to
        // check the address for various ram areas which are used by the
        // kernel and/or oscore, mmm
        	fprintf( fMemLog, "[ RAM 8 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%02 ]n",
                	wSegment, Index, lpPsxRAM[ Index & 0x001FFFFF ] );
        	return( (BYTE)lpPsxRAM[ Index & 0x001FFFFF ] );
	}
}
//---------------------------------------------------------------------------
//	16-bit memory read ( see COP_LW in MIPS 32 ISA documentation )
//---------------------------------------------------------------------------
WORD	__fastcall	PSX_MEM :: GRAM16	( DWORD Index )
{	char 	*p;
	WORD	wSegment = ( Index >> 16 );

        //	Is referencing the ROM segments, 0xbcf0 ... 0xbfc8 ?
        if ( ( wSegment >= 0x1fc0 ) && ( wSegment <= 0x1fc8 ) || ( wSegment >= 0xBFC0 ) && ( wSegment <= 0xBFC8 ) )
        {
        	return( (WORD)PsxBIOS->ROM[ Index ] );
        }

	if ( wSegment == 0x1f80 ) {
		//if ( Index < 0x1f801000 )
		//	return uHWM16( Index & 0xffff);
		//else
		//	return HWM16( Index );
                return( lpHW->dwHWM16[ Index ] );
	} else {
                	return( 0 );
	}
}
//---------------------------------------------------------------------------
//!	32-BIT MEMORY READ.
//---------------------------------------------------------------------------
DWORD	__fastcall	PSX_MEM :: GRAM32	( DWORD Index )
{	WORD	wSegment = ( Index >> 16 );
	DWORD	dwValue = 0x00000000;

	//	As the PlayStation only supports useg, kseg0, and kseg1 we need
        // to check the segment we are addressing, but first make sure that the
        // reference is valid for the current operating mode of the processor

        //	BIOS
        if ( ( ( wSegment >= 0xBFC0 ) && ( wSegment <= 0xBFC8 ) ) ||
        	( ( wSegment >= 0x1FC0 ) && ( wSegment <= 0x1FC0 ) ) )
        {
                //	...	pass to BIOS class
                dwValue = PsxBIOS->ROM[ Index ];
	        fprintf( fMemLog, "[ BIOS 32 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%08x ]\n",
        		wSegment, Index, PsxBIOS->ROM[ ( Index & 0x0007FFFF ) ] );
        }

        //	Hardware segment
        if ( wSegment == 0x1F80 ) {
        	dwValue = lpHW->dwHWM32[ ( Index & 0x0000FFFF ) ];
	        fprintf( fMemLog, "[ HARD 32 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%08x ]\n",
        		wSegment, Index, PsxBIOS->lpPsxBIOS[ Index & 0x0000FFFF ] );
        }

        //	Cartridge port
        if ( wSegment == 0x1F00 )	{
	        //fprintf( fMemLog, "[ SCRATCH 32 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%08x ]\n",
        	//	wSegment, Index, lpPsxBIOS[ Index & 0x0000FFFF ] );
        	dwValue = lpPsxPPM[ ( Index & 0x0000FFFF ) ];
        }

        //	else main ram access
        if ( ( wSegment >= 0x0000 ) && ( wSegment <= 0x7FFF ) ) {
                dwValue = lpPsxRAM[ ( Index && 0x001FFFFF ) ];
        	fprintf( fMemLog, "[ RAM 32 READ ] [ 0x%04x ] [ 0x%08x ] [ 0x%08x ]\n",
                	wSegment, Index, lpPsxRAM[ Index && 0x001FFFFF ] );
        }

        return( dwValue );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: SRAM8	( DWORD Index, BYTE Value )
{	char	*p;
	WORD	wSegment = ( Index >> 16 );

        //	Hardware segment
	if ( wSegment == 0x1f80 ) {
        	lpHW->dwHWM8[ Index & 0xFFFF ] = Value;
                return;
	}

        //	Scratch PAD RAM ?
        if ( wSegment == 0x1F00 )	{
        	lpPsxPPM[ Index & 0xFFFF ] = Value;
                return;
        }

        //	Main RAM Access
        lpPsxRAM[ Index & 0x001FFFFF ] = Value;

        CPU->SignalException( CAUSE_TLBM, (void*)&Index );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: SRAM16	( DWORD Index, WORD Value )
{	char	*p;
	WORD	wSegment = ( Index >> 16 );

	if ( wSegment == 0x1f80 ) {
                lpHW->dwHWM16[ Index & 0xFFFF ] = Value;
                return;
	}

        if ( wSegment == 0x1F00 ) {
        	lpPsxPPM[ Index & 0xFFFF ] = Value;
                return;
        }

        lpPsxRAM[ Index & 0x001FFFFF ] = Value;

        CPU->SignalException( CAUSE_TLBM, (void*)&Index );
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_MEM :: SRAM32	( DWORD Index, DWORD Value )
{	char	*p;
	WORD	wSegment = ( Index >> 16 );

	if ( wSegment == 0x1f80 ) {		//	Segment requested is USEG / KUSEG
                lpHW->dwHWM32[ Index & 0xFFFF ] = Value;
                return;
        }

        if ( wSegment == 0x1F00 ) {
        	lpPsxPPM[ Index & 0xFFFF ] = Value;
        	return;
        }

        lpPsxRAM[ Index & 0x001FFFFF ] = Value;

        CPU->SignalException( CAUSE_TLBM, (void*)&Index );
}
//---------------------------------------------------------------------------
DWORD	*__fastcall	PSX_MEM :: GAddr( DWORD vAddr )
{	WORD	wSegment = ( vAddr >> 16 );

	//	Test for various memory references and return the required pointer
        // for said address

        //	...	Check for BIOS / ROM access first
        if ( ( wSegment >= 0xBFC0 ) && ( wSegment <= 0xBFC8 ) || ( wSegment >= 0x1FC0 ) && ( wSegment <= 0x1FC8 ) ) {
                return( (DWORD*)&PsxBIOS->lpPsxBIOS[ ( vAddr & 0x0007FFFF ) ] );
        }

        //	...	Next scratch pad ram
        if ( wSegment == 0x1F00 )
        	return( (DWORD*)&lpPsxPPM[ ( vAddr & 0x0000FFFF ) ] );

        //	...	now hardware segment
        if ( wSegment == 0x1F80 )
        	return( (DWORD*)&lpHW->lpPsxHWM[ ( vAddr & 0x0000FFFF ) ] );

        //	...	else translate to a valid RAM address
        return( (DWORD*)&lpPsxRAM[ ( vAddr & 0x001FFFFF ) ] );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) void	__fastcall	AddressTranslation( DWORD& pAddr, int& CCA, DWORD vAddr, bool LorS, bool IorD )
{	bool	found = false;
	int	EOBit = 0;
        DWORD	pfn = 0;
        bool	v = false;
        bool	c = false;
        bool	d = false;

        //	First check alignment for code
        if ( IorD ) {	//	Code Alignment check
        	if ( ( vAddr & 0x01 ) || ( vAddr & 0x02 ) || ( vAddr & 0x04 ) ) {
                	CPU->CP0.CtrlRegs.BadVAddr.BadVAddr = vAddr;
                        CPU->SignalException( CAUSE_IBE, (void*)&vAddr );
                        return;
                }
        }

        //	Else, valid as data can be loaded in 8-32bit modes

	for ( int i=0; i<CPU->CP0.CtrlRegs.Wired.Wired; i++ ) {
        	LPTLBENTRY	lpEntry = PsxMem->lpTLBArray[ i ];
                if ( ( lpEntry->vpn2 & ~lpEntry->PageMask ) == ( vAddr & ~lpEntry->PageMask ) && ( lpEntry->Global || lpEntry->ASID == CPU->CP0.CtrlRegs.EntryHi.ASID ) ) {
                	switch ( lpEntry->PageMask ) {
                        	case ( PAGESIZE_4K ) : {
                                	EOBit = 12;
                                        break;
                                }
                               	case ( PAGESIZE_16K ) : {
                                       	EOBit = 14;
                                       	break;
                                }
                               	case ( PAGESIZE_64K ) : {
                                       	EOBit = 16;
                                       	break;
                                }
                               	case ( PAGESIZE_256K ) : {
                                       	EOBit = 18;
                                       	break;
                                }
                               	case ( PAGESIZE_1MB ) : {
                                       	EOBit = 20;
                                       	break;
                                }
                               	case ( PAGESIZE_4MB ) : {
                                       	EOBit = 22;
                                       	break;
                                }
                               	case ( PAGESIZE_16MB ) : {
                                       	EOBit = 24;
                                       	break;
                                }
                               	case ( PAGESIZE_64MB ) : {
                                       	EOBit = 26;
                                       	break;
                                }
                               	case ( PAGESIZE_256MB ) : {
                                       	EOBit = 28;
                                       	break;
                                }
                                default : {
                                       	EOBit = 12;
                                        break;
                                }
                        }

                        if ( ( vAddr & ( 1 << EOBit ) ) == 0 ) {
                        	//	Even address
                                pfn = lpEntry->PFN0;
                                v = lpEntry->V0;
                                c = lpEntry->C0;
                                d = lpEntry->D0;
                        } else {
                        	//	Odd address
                                pfn = lpEntry->PFN1;
                                v = lpEntry->V1;
                                c = lpEntry->C1;
                                d = lpEntry->D0;
                        }

                        CCA = c;

                        CPU->CP0.CtrlRegs.BadVAddr.BadVAddr = vAddr;

                        if ( d & (!LorS) )
                        	CPU->SignalException( CAUSE_TLBM, (void*)&vAddr );

                        pAddr = ( ( pfn << ( EOBit - 1 ) ) | ( vAddr << ( EOBit - 1 ) ) );
                        found = true;
                        break;
                }
        }

        if ( found == false )
        	CPU->SignalException( CAUSE_TLBM, (void*)&vAddr );
}
//---------------------------------------------------------------------------
