//---------------------------------------------------------------------------
//
//	$CODENAME	::	Memory Emulation Layer ( MEL )
//	$FileName	::	Memory.h
//	$Purpose	::	MIPS 32 ISA Complient Virtual Memory Manager
//				Using Fixed Address Translation.
//				Will be expanded in the future to take into
//				account of the fact that a PC has a lot more
//				RAM than any console.
//
//---------------------------------------------------------------------------
#ifndef MemoryH
#define MemoryH
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
#define RAM_CACHED_MAPPED_BASE	0x80000000	//	Cache on, Mapped Memory
#define RAM_MAPPED_BASE		0xA0000000	//	Cache off, Mapped Memory
#define RAM_VIRTUAL_BASE	0x80000000	//	Cache on, Mapped
#define RAM_BASE		RAM_VIRTUAL_BASE
#define RAM_SIZE		0x00200000	//	2MB RAM
#define ROM_BASE		0xBFC00000	//	Base address for BIOS
#define ROM_SIZE		0x00080000	//	512K
#define CARTRIDGE_BASE		0x1F000000	//	Cartridge port (parallel)
#define CARTRIDGE_SIZE		0x00001000	//	????
#define KERNEL_BASE		0x00000000	//	Kernel Base space ( + RAM_TYPE_BASE )
#define KERNEL_SIZE		0x00004000	//	64K Kernel Size
#define STACK_BASE		0x80200000	//	Stack start at 2MB boundary, grows down remember
#define STACK_SIZE		0x00002000	//	32K Stack Size
#define HEAP_START		( RAM_BASE + KERNEL_SIZE )
#define HEAP_SIZE		0x00001000	//	16K Initial Help Size
//---------------------------------------------------------------------------
#define USEG_START		RAM_CACHED_MAPPED_BASE	//	User Segment
#define KSEG0_START		RAM_MAPPED_BASE		//	KSEG 0 see mips 32 isa prm
#define KSEG1_START		RAM_BASE		//	KSEG 1 see mips 32 isa prm
//---------------------------------------------------------------------------
//	Segment Page sizes, See CP0 PageMask register
//---------------------------------------------------------------------------
#define PAGESIZE_4K		0	//	4K    page size
#define PAGESIZE_16K		3	//	16K   page size
#define PAGESIZE_64K            15      //	64K   page size
#define PAGESIZE_256K           63      //	256K  page size
#define PAGESIZE_1MB            255     //	1MB   page size
#define PAGESIZE_4MB            1023    //	4MB   page size
#define PAGESIZE_16MB           4095    //	16MB  page size
#define PAGESIZE_64MB           16383   //	64MB  page size
#define PAGESIZE_256MB          65535   //	256MB page size
//---------------------------------------------------------------------------
#define	MMUTYPE_NONE		0	//	No address translators
#define MMUTYPE_STANDARD_TLB	1	//	TLB Based Full Virtual Memory
#define MMUTYPE_STANDARD_BAT	2	//	TLB Based minimum Block Address Translators
#define MMUTYPE_STANDARD_FIXED	3	//	Fixed Address Translators
//---------------------------------------------------------------------------
#define MAX_TLBENTRIES		96	//	This is defined in the MIPS 32 ISA documentation
//---------------------------------------------------------------------------
//	TLB Entry definition for Virtual Memory
//---------------------------------------------------------------------------
typedef struct	_tlb_entry_def_	{		//	TLB Entry definition
	//	PageMask for this entry, see MIPS 32 ISA documentation
	unsigned	PageMask	: 16;	//	Read / Written to CP0.CtrlRegs.PageMask.Mask

        //	Address Space Identifier OS supplied
        unsigned	ASID		: 8;	//	Modified by the KERNEL or OSCORE only

        //	Global bit
        unsigned	Global		: 1;	//	Entry is global	written to EntryLo0, 1, EntryHi global bit(s)

        union	{
        	unsigned	VirtualPageNum  : 19;	//	Actual Virtual Page / 2 as the following defines the even / odd pages
                unsigned	vpn2 : 19;
        };

        //	Even Pages
        unsigned	V0	: 1;	//	Valid bit
        unsigned	D0	: 1;	//	Dirty bit
        unsigned	C0	: 3;	//	Cache coherency attributes
        unsigned	PFN0	: 24;	//	Page Frame Number

        //	Odd Pages
        unsigned	V1	: 1;	//	Valid bit
        unsigned	D1	: 1;	//	Dirty bit
        unsigned	C1	: 3;	//	Cache coherency attributes
        unsigned	PFN1	: 24;	//	Page Frame Number

        }	TLBENTRY, *LPTLBENTRY, TLBEntry, *lpTLBEntry;
//---------------------------------------------------------------------------
//	Virtual Memory Manager
//---------------------------------------------------------------------------
class	PSX_MEM : public TObject
	{
        	friend	class	CPSX_MDEC;	//	MDEC <--> Memory Interface
                friend	class	R3000A;		//	CPU <--> Memory Interface
        	friend	class	BIOS;		//	BIOS Interfacing
                friend	class	PSX_Kernel;     //	Kernel interfacing
                friend	class	PSX_OSCore;	//	OSCore interfacing
        	protected:

                	//	TLB Management
                        bool	__fastcall	TLBProbe	( void ); //DWORD Index );
                        void	__fastcall	TLBRead		( void );
                        void	__fastcall	TLBWriteIndexed	( void );
                        void	__fastcall	TLBWriteRandom	( void );

                        BYTE	__fastcall	GRAM8	( DWORD Index );
                        WORD	__fastcall	GRAM16	( DWORD Index );
                        DWORD	__fastcall	GRAM32	( DWORD Index );
                        void	__fastcall	SRAM8	( DWORD Index, BYTE Value );
                        void	__fastcall	SRAM16	( DWORD Index, WORD Value );
                        void	__fastcall	SRAM32	( DWORD Index, DWORD Value );
                private:
                	DWORD		*lpPsxRAM;	//	RAM
                        DWORD		*lpPsxPPM;

                        //	TLB Entries
                        LPTLBENTRY	*lpTLBArray;

                        DWORD	*__fastcall	GAddr		( DWORD vAddr );
                public:
                		__fastcall	PSX_MEM 	( void );		//	Constructor
                        	__fastcall	~PSX_MEM	( void );		//	Destructor
                        int	__fastcall	Initialise	( void );		//	Initialise 'System Memory' to a known state, ie blank
                        void	__fastcall	Terminate	( void );		//	Remove the memory associations
                        void	__fastcall	Reset		( void );		//	reset to a blank state
                        //	Friends
                        friend	int	__fastcall	LoadState		( LPSTR lpszFileName );
                        friend	int	__fastcall	SaveState		( LPSTR lpszFileName );

                        friend	void	bios_InitHeap( void );

                        friend	void	__fastcall	AddressTranslation( DWORD& pAddr, int& CCA, DWORD vAddr, bool LorS, bool IorD );

                __published:

                        __property	BYTE	RAM8 [ DWORD Index ] = { read=GRAM8,  write=SRAM8  };	//	8  bit read / write access methods
                        __property	WORD	RAM16[ DWORD Index ] = { read=GRAM16, write=SRAM16 };	//      16 bit read / write access methods
                        __property	DWORD	RAM32[ DWORD Index ] = { read=GRAM32, write=SRAM32 };   //      32 bit read / write access methods
        };
//---------------------------------------------------------------------------
__declspec( dllimport )	PSX_MEM	*PsxMem;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
