//---------------------------------------------------------------------------
//
//	$FileName	::	R3000A.cpp
//	$Author		::	Chris Brooklyn
//      $Version	::	0.1 for PSXIDE ONLY
//	$Purpose	::	Emulation of the MIPS 32 R3000 Core Processor
//				Handles CP0 (SysControl) and CP1 (FPU) CPU Cores
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "R3000A.h"
#include <windows.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#define	CLASSNAME	"R3000A"
//---------------------------------------------------------------------------
LRESULT	CALLBACK	CPUWndProc( HWND wHandle, UINT Msg, WPARAM wParam, LPARAM lParam );
//---------------------------------------------------------------------------
__declspec( dllexport ) R3000A	*CPU	 = NULL;	//	This is main CP0/CP1 thread
__declspec( dllexport ) DWORD	dwOpCode = 0x00000000;	//	Opcode being executed
//---------------------------------------------------------------------------
//	General Register Names
//---------------------------------------------------------------------------
const	char	RegNames[][ 32 ] =	{ "zero", "at", "v0", "v1", "a0", "a1", "a2", "a3",
					  "t0",   "t1", "t2", "t3", "t4", "t5", "t6", "t7",
                                          "s0",   "s1", "s2", "s3", "s4", "s5", "s6", "s7",
                                          "t8",   "t9", "k0", "k1", "gp", "sp", "fp", "ra"
                                        };
//---------------------------------------------------------------------------
//	Control Register Names
//---------------------------------------------------------------------------
const	char	CtlRegNames[][ 35 ] =	{
			"Index",		//	0  - Index in TLB Array
                        "Random",		//	1  - Random entry in TLB Array
                        "EntryLo0",		//	2  - EntryLo0 TLB
                        "EntryLo1",		//	3  - EntryLo1 TLB
                        "Context",		//	4  - Context
                        "PageMask",		//	5  - PageMask
                        "Wired",                //      6  - Wired TLB
                        "Reserved",             //      7  - Reserved
                        "BadVAddr",             //      8  - BadVAddr
                        "Count",                //      9  - Count
                        "EntryHi",              //      10 - EntryHi (TLB)
                        "Compare",              //      11 - Compare
                        "Status",               //      12 - Status
                        "Cause",                //      13 - Cause
                        "EPC",                  //      14 - EPC
                        "PRiD",                 //      15 - PRiD
                        "Config",               //      16 - Config 0..<n>
                        "LLAddr",               //      17 - LLAddr
                        "WatchLo",              //      18 - WatchLo
                        "WatchHi",              //      19 - WatchHi
                        "XContent",             //      20 - XContext (MIPS 64)
                        "Reserved",             //      21 - Reserved
                        "Reserved",             //      22 - Reserved
                        "Debug",                //      23 - Debug
                        "DEPC",                 //      24 - DEPC
                        "PerfCnt",              //      25 - PerfCnt
                        "ErrCtl",               //      26 - ErrCtl
                        "CacheErr",		//	27 - CacheErr
                        "TagLo",                //      28 - TagLo/DataLo
                        "TagHi",                //      29 - TagHi/DataHi
                        "ErrorPC",              //      30 - ErrorPC
                        "DESAVE"                //      31 - DESAVE
                        };
//---------------------------------------------------------------------------
//int  	__declspec( dllexport )	__fastcall	CPUInitialise		( void );
//void 	__declspec( dllexport )	__fastcall	CPUReset		( void );
//void 	__declspec( dllexport )	__fastcall	CPUTerminate		( void );
//void 	__declspec( dllexport )	__fastcall	CPUTakeException	( unsigned long code, unsigned long step );
//void 	__declspec( dllexport )	__fastcall	CPUInterrupt		( unsigned long code );
//void 	__declspec( dllexport )	__fastcall	CPUBranchTest		( void );
//---------------------------------------------------------------------------
FILE	*fCPULog = NULL;
//---------------------------------------------------------------------------
//!	CONSTRUCTOR	::	R3000A Thread.
//!	Construction of the R3000A emulation thread, just basically registers it
//! with windows message system.
//---------------------------------------------------------------------------
__fastcall 	R3000A :: R3000A( bool CreateSuspended ) : TThread( CreateSuspended )
{

	//fDebugControl = false;
        //fSingleStep = false;

	fCPULog = fopen( "CPU.LOG", "wt" );
        if ( fCPULog == NULL ) {
        	fprintf( stderr, "R3000A CPU Emulation Layer [ FAILED TO CREATE LOG FILE ].\n" );
                fCPULog = stderr;
        }

	fprintf( fCPULog, "CPU Emulation Layer Construction" );

	CurIrq = -1;

        lpIrqQueue = new CPUIrq[ 8 ];

        ZeroMemory( lpIrqQueue, sizeof( CPUIrq ) * 8 );

	dwCPUCycle = 0;
        lpInstr = new CPUInstr;
        ZeroMemory( lpInstr, sizeof( CPUInstr ) );
	//	Get instance of this module
	hInstance = GetModuleHandle( NULL );

        //	Clear memory
        ZeroMemory( &WndClass, sizeof( WNDCLASSEX ) );

        //	Register this thread with windows
        WndClass.cbSize = sizeof( WNDCLASSEX );
        WndClass.lpfnWndProc = (WNDPROC) CPUWndProc;
        WndClass.lpszClassName = CLASSNAME;

        if ( !RegisterClassEx( &WndClass ) )	{
        	MessageBox( NULL, "Unable to register the R3000A class with Windows.", CLASSNAME, MB_OK | MB_ICONEXCLAMATION );
        	throw "Unable to register R3000 class with Windows.";
        }

        //	Registration with windows was a success

        //	Blank EVERYTHING! As the cpu/fpu is initialised to a known state
        // .... when the initialise, or reset, methods are called.
        ZeroMemory( &CP0, sizeof( CP0 ) );
        ZeroMemory( &CP1, sizeof( CP1 ) );

        if ( lpGTE == NULL )
        	lpGTE = new GTE( CreateSuspended );

        CP2 = lpGTE;

        if ( !CreateSuspended )	{	//	Run immediately
        	// ....	Make sure the CPU/FPU and CP2 are initialised, ie
                //	CP0, CP1 and CP2, CP3 is NOT present
        	Reset();
                lpGTE->Reset();
                // ....	and resume execution, or start as it where
        	Resume();		//	make sure the main CP0 / CP1 core is active and running
                lpGTE->Resume();	//	run cop2 (GTE/GPU)
        }

        // ....	Note if NOT autorun then you must at least intialise the class
        // before you use it. However, if you call reset then the class is
        // automatically initialised for you.

	fprintf( fCPULog, " Completed OK.\n" );

}
//---------------------------------------------------------------------------
//	Thread destructor
//---------------------------------------------------------------------------
__fastcall	R3000A :: ~R3000A( void )
{
	fprintf( fCPULog, "R3000A Destructor.\n" );

        if ( !Suspended )
        	Suspend();

        Terminate();

	if ( hInstance )
        	UnregisterClass( CLASSNAME, hInstance );

        if ( fCPULog != NULL )
        	fclose( fCPULog );

	delete this;
}
//---------------------------------------------------------------------------
//	Returns the current operating mode of the processor
//---------------------------------------------------------------------------
BYTE	__fastcall	R3000A :: GCPUMode( void )
{
        return( CP0.CtrlRegs.Status.KSU );
}
//---------------------------------------------------------------------------
//	Sets the processor mode, this is not user callable
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SCPUMode( BYTE Value )
{	//	Force reset the debug.DM bit, whether or not debug mode is implemented.
	// as the debug mode if implemented will reset this bit if needed
        CP0.CtrlRegs.Debug.DM = 0;

        //	Does not matter if the EXL or ERL bits in CP0 Status
        // are set or not for KERNEL MODE but does matter for all other
        // modes.

        //	Now which mode do we wish to switch to
	switch ( Value ) {
#ifdef	CPUMODE_EJTAGDEBUG_IMPLEMENTED
        	case ( CPUMODE_DEBUG ) : {	//	DEBUG MODE ?
                	CP0.CtrlRegs.Status.ERL = 0;	//	Force NOT ERROR LEVEL
                        CP0.CtrlRegs.Status.EXL = 0;	//	Force NOT EXCEPTION LEVEL
                	CP0.CtrlRegs.Debug.DM = 1;
                        CP0.CtrlRegs.Status.KSU = CPUMODE_DEBUG;
                	break;
                }
#endif
		case ( CPUMODE_KERNEL ) : {	//	KERNEL MODE ?
                        CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
                	break;
                }

                case ( CPUMODE_SUPERVISOR ) : {	//	SUPERVISOR MODE ?
                	CP0.CtrlRegs.Status.ERL = 0;
                        CP0.CtrlRegs.Status.EXL = 0;
                	CP0.CtrlRegs.Status.KSU = CPUMODE_SUPERVISOR;
                	break;
                }

                case ( CPUMODE_USER ) : {	//	USER MODE ?
                	CP0.CtrlRegs.Status.ERL = 0;
                        CP0.CtrlRegs.Status.EXL = 0;
                	CP0.CtrlRegs.Status.KSU = CPUMODE_USER;
                	break;
                }
	}

        //	I suppose when the processor switches modes it would probably
        // need to load / store states and tlb / thread information?
}
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GLo	( void )
{
	return( CP0.Lo );
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SLo	( DWORD Value )
{
	if ( CP0.Lo != Value )
        	CP0.Lo = Value;
}
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GHi	( void )
{
	return( CP0.Hi );
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SHi	( DWORD Value )
{
	if ( CP0.Hi != Value )
        	CP0.Hi = Value;
}
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GPC	( void )
{
	return( CP0.PC );
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SPC( DWORD Value )
{
	if ( CP0.PC != Value ) {	//	Nullify current instruction
        	NullifyCurrentInstruction();
        	CP0.PC = Value;
        }
}
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GGenRegs( int Index )
{
	if ( ( Index < 1 ) || ( Index > 31 ) )	//! 	ignore 'r0', and any other 'bad' register(s)
        	return( 0 );

        return( CP0.dwGenRegs[ Index ] );
}
//---------------------------------------------------------------------------
//	Returns a control register, note does support 'multiple select' on the
// registers, as it examines the instruction to determine the 'sel' value if
// needed. Note 'Hi' and 'Lo' are note accessable
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GCtlRegs( int Index )
{
	if ( ( Index < 0 ) || ( Index > 32 ) )
        	return( NULL );

#ifdef	LOGGING
	fprintf( fCPULog, "Ctl Reg Read %d : %s\n", Index, CtlRegNames[ Index ] );
#endif

        switch ( Index ) {
        	case (  0 ) : return( CP0.CtrlRegs.dwIndex);
                case (  1 ) : return( CP0.CtrlRegs.dwRandom );
                case (  2 ) : return( CP0.CtrlRegs.dwEntryLo0 );
                case (  3 ) : return( CP0.CtrlRegs.dwEntryLo1 );
                case (  4 ) : return( CP0.CtrlRegs.dwContext );
                case (  5 ) : return( CP0.CtrlRegs.dwPageMask );
                case (  6 ) : return( CP0.CtrlRegs.dwWired );
                case (  7 ) : return( CP0.CtrlRegs.r7 );
                case (  8 ) : return( CP0.CtrlRegs.dwBadVAddr );
                case (  9 ) : return( CP0.CtrlRegs.dwCount );
                case ( 10 ) : return( CP0.CtrlRegs.dwEntryHi );
                case ( 11 ) : return( CP0.CtrlRegs.dwCompare );
                case ( 12 ) : return( CP0.CtrlRegs.dwStatus );
                case ( 13 ) : return( CP0.CtrlRegs.dwCause );
                case ( 14 ) : return( CP0.CtrlRegs.dwEPC );
                case ( 15 ) : return( CP0.CtrlRegs.dwPRiD );
                case ( 16 ) : {
                	switch ( _Sel_ ) {
                        	case ( 0 ) : return( CP0.CtrlRegs.dwConfig );
                                //	Config 1
                                case ( 1 ) : return( CP0.CtrlRegs.dwConfig1 );
                                //	Config 2
                                case ( 2 ) : return( CP0.CtrlRegs.dwConfig2 );
                                //	Config 3
                                case ( 3 ) : return( CP0.CtrlRegs.dwConfig3 );
                        }
                        return( 0 );
                }
                case ( 17 ) : return( CP0.CtrlRegs.dwLLAddr );
                case ( 18 ) : return( CP0.CtrlRegs.dwWatchLo );
                case ( 19 ) : return( CP0.CtrlRegs.dwWatchHi );
                case ( 20 ) : return( CP0.CtrlRegs.XContext );
                case ( 21 ) : return( CP0.CtrlRegs.r21 );
                case ( 22 ) : return( CP0.CtrlRegs.r22 );
                case ( 23 ) : return( CP0.CtrlRegs.dwDebug );
                case ( 24 ) : return( CP0.CtrlRegs.DEPC );
                case ( 25 ) : return( CP0.CtrlRegs.PerfCnt );
                case ( 26 ) : return( CP0.CtrlRegs.ErrCtl );
                case ( 27 ) : return( CP0.CtrlRegs.CacheErr );
                case ( 28 ) : return( CP0.CtrlRegs.TagLo );
                case ( 29 ) : return( CP0.CtrlRegs.TagHi );
                case ( 30 ) : return( CP0.CtrlRegs.ErrorEPC );
                case ( 31 ) : return( CP0.CtrlRegs.DESAVE );
        	default : return( 0 );
        }
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SGenRegs( int Index, DWORD Value )
{
	if ( ( Index < 0 ) || ( Index > NUM_MIPS_REGS ) )
        	return;

        if ( CP0.dwGenRegs[ Index ] != Value )
        	CP0.dwGenRegs[ Index ] = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SCtlRegs( int Index, DWORD Value )
{	//	Which specific register are interested in ?
	switch ( Index ) {
        	case (  0 ) : {	//	TLB Entry Index Register ie curr TLB = TLBArray[ Index ]
                	CP0.CtrlRegs.dwRandom = Value;
                        break;
                }
                case (  1 ) : CP0.CtrlRegs.dwRandom = Value;
                case (  2 ) : CP0.CtrlRegs.dwEntryLo0 = Value;
                case (  3 ) : CP0.CtrlRegs.dwEntryLo1 = Value;
                case (  4 ) : CP0.CtrlRegs.dwContext = Value;
                case (  5 ) : CP0.CtrlRegs.dwPageMask = Value;
                case (  6 ) : CP0.CtrlRegs.dwWired = Value;
                case (  7 ) : return;
                case (  8 ) : CP0.CtrlRegs.dwBadVAddr = Value;
                case (  9 ) : CP0.CtrlRegs.dwCount = Value;
                case ( 10 ) : CP0.CtrlRegs.dwEntryHi = Value;
                case ( 11 ) : CP0.CtrlRegs.dwCompare = Value;
                case ( 12 ) : {
                	CP0.CtrlRegs.dwStatus = Value;
                        if ( ( CP0.CtrlRegs.Status.EXL ) || ( CP0.CtrlRegs.Status.ERL ) )
                        	CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
                        break;
                }
                case ( 13 ) : CP0.CtrlRegs.dwCause = Value;
                case ( 14 ) : CP0.CtrlRegs.dwEPC = Value;
                case ( 15 ) : CP0.CtrlRegs.dwPRiD = Value;

                //!	CP0 Register 16 Select <n>	Configuration Registers.
                case ( 16 ) : {
                	//	Which configuration register are we after ?
                        switch ( _Sel_ ) {
                        	case ( 0 ) : {	//!	Config.
                                	if ( CP0.CtrlRegs.dwConfig != Value )
                                        	CP0.CtrlRegs.dwConfig = Value;
                                        break;
                                }

                                case ( 1 ) : {
                                	//if ( CP0.CtrlRegs.Config.M )	//	Config1 actually implmented ?
                                        //	if ( CP0.CtrlRegs.Config1.dwConfig1 != Value )
                                        //        	CP0.CtrlRegs.Config1.dwValue = Value;
                                        break;
                                }

                                case ( 2 ) : {
                                	//if ( ( CP0.CtrlRegs.Config.M ) && ( CP0.CtrlRegs.Config1.M ) )
                                        //	if ( CP0.CtrlRegs.Config2.dwConfig2 != Value )
                                        //        	CP0.CtrlRegs.Config2.dwConfig2 = Value;
                                        break;
                                }

                                case ( 3 ) : {
                                	//if ( ( CP0.CtrlRegs.Config.M ) && ( CP0.CtrlRegs.Config1.M )
                                        //	&& ( CP0.CtrlRegs.Config2.M ) )
                                        //	if ( CP0.CtrlRegs.Config3.dwConfig3 != Value )
                                        //        	CP0.CtrlRegs.Config3.dwConfig3 = Value;
                                        break;
                                }

                        }
                        break;
                }

                case ( 17 ) : CP0.CtrlRegs.dwLLAddr = Value;
                case ( 18 ) : CP0.CtrlRegs.dwWatchLo = Value;
                case ( 19 ) : CP0.CtrlRegs.dwWatchHi = Value;
                case ( 20 ) : CP0.CtrlRegs.XContext = Value;
                case ( 21 ) : CP0.CtrlRegs.r21 = Value;
                case ( 22 ) : CP0.CtrlRegs.r22 = Value;
                case ( 23 ) : CP0.CtrlRegs.dwDebug = Value;
                case ( 24 ) : CP0.CtrlRegs.DEPC = Value;
                case ( 25 ) : CP0.CtrlRegs.PerfCnt = Value;
                case ( 26 ) : CP0.CtrlRegs.ErrCtl = Value;
                case ( 27 ) : CP0.CtrlRegs.CacheErr = Value;
                case ( 28 ) : CP0.CtrlRegs.TagLo = Value;
                case ( 29 ) : CP0.CtrlRegs.TagHi = Value;
                case ( 30 ) : CP0.CtrlRegs.ErrorEPC = Value;
                case ( 31 ) : CP0.CtrlRegs.DESAVE = Value;
        }
}
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GFPR		( int Index )
{
	if ( ( Index < 0 ) || ( Index > NUM_COPRO_REGS ) )
        	return( 0 );
        return( CP1.GenRegs[ Index ].dw );
}
//---------------------------------------------------------------------------
DWORD	__fastcall	R3000A :: GFCR		( int Index )
{
	switch ( Index ) {

        	case ( 0 ) : {	//	FPU Control Register 'FIR'
                	return( CP1.dwFIR );
                }

                case ( 25 ) : {	//	FPU Control Register 'FCCR'
                	return( CP1.dwFCCR );
                }

                case ( 26 ) : {	//	FPU Control Register 'FEXR'
                	return( CP1.dwFEXR );
                }

                case ( 28 ) : {	//	FPU Control Register 'FENR'
                	return( CP1.dwFENR );
                }

                case ( 31 ) : {	//	FPU Control Register 'FCSR'
                	return( CP1.dwFCSR );
                }

        	default : {
                	return( 0 );
                }
        }
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SFPR		( int Index, DWORD Value )
{
	if ( ( Index < 0 ) || ( Index > NUM_COPRO_REGS ) )
        	return;
        CP1.GenRegs[ Index ].dw = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SFCR		( int Index, DWORD Value )
{
	switch ( Index ) {

        	case ( 0 ) : {	//	FPU Control Register 'FIR'
                	CP1.dwFIR = Value;
                        break;
                }

                case ( 25 ) : {	//	FPU Control Register 'FCCR'
                	CP1.dwFCCR = Value;
                        break;
                }

                case ( 26 ) : {	//	FPU Control Register 'FEXR'
                	CP1.dwFEXR = Value;
                        break;
                }

                case ( 28 ) : {	//	FPU Control Register 'FENR'
                	CP1.dwFENR = Value;
                        break;
                }

                case ( 31 ) : {	//	FPU Control Register 'FCSR'
                	CP1.dwFCSR = Value;
                        break;
                }
        }
}
//---------------------------------------------------------------------------
int	__fastcall	R3000A :: Initialise( void )
{
	//	Initiate Initialisation cascade through memory and hardware levels
        //
        //----	DISABLED. Leave ALL garbage in virtual ram space
        //
        //PsxMem->Initialise();

        fprintf( fCPULog, "R3000A Initialising.\n" );

	ZeroMemory( ICache, sizeof( DWORD ) * 1024 );	//	Instruction cache
        ZeroMemory( DCache, sizeof( DWORD ) * 4096 );	//	Data cache
        //----	CACHE NOT IMPLEMENTED YET

        ZeroMemory( &CP0, sizeof( CoProcessor0 ) );
        ZeroMemory( &CP1, sizeof( CoProcessor1 ) );

        dwCPUCycle = 0;

        //!   Now we set the coprocessor 0 to a known state, ie similar to what the
        //! R3000A is set to on power up. Now go through all the control registers
        //! and set to a known default-state.

        //!   The Index register. As we are using fixed address translation
        //! various CP0 resgisters are set to zero, which we have already done,
        //! to inidicate that they are not being used, ie primarily 'Index',
        //! 'Random' and 'Wired', 'EntryLo0', 'EntryLo1', 'Context', 'PageMask',
        //! 'EntryHi'

        //! 	TLB Registers.
        //! CP Register 0 Select 0 ( Index Register )
        CP0.CtrlRegs.Index.Index = 0;	//!	No TLB entry

        //! CP Register 1 Select 0 ( Random Register )
        CP0.CtrlRegs.Random.Random = 0;	//!	No random entry

        //! CP Register 2 Select 0 ( EntryLo0 Register )
        CP0.CtrlRegs.EntryLo0.Global = 0;
        CP0.CtrlRegs.EntryLo0.Dirty = 0;
        CP0.CtrlRegs.EntryLo0.Valid = 0;
        CP0.CtrlRegs.EntryLo0.CCA = CCA_UNCACHED;
        CP0.CtrlRegs.EntryLo0.PFN = 0;

        //! CP Register 3 Select 0 ( EntryLo1 Register )
        CP0.CtrlRegs.EntryLo1.Global = 0;
        CP0.CtrlRegs.EntryLo1.Dirty = 0;
        CP0.CtrlRegs.EntryLo1.Valid = 0;
        CP0.CtrlRegs.EntryLo1.CCA = CCA_UNCACHED;
        CP0.CtrlRegs.EntryLo1.PFN = 0;

        //! CP Register 4 Select 0 ( Context Register )
        CP0.CtrlRegs.Context.PTEBase = 0;
        CP0.CtrlRegs.Context.BadVPN2 = 0;

        //! CP Register 5 Select 0 ( PageMask Register )
        CP0.CtrlRegs.PageMask.Mask = PAGESIZE_4K | PAGESIZE_16K | PAGESIZE_16K;

        //! CP Register 6 Select 0 ( Wired Register )
        CP0.CtrlRegs.Wired.Wired = 0;

        //! CP Register 7 Select 0 ( RESERVED FOR FUTURE EXTENSIONS )

        //! CP Register 8 Select 0 ( BadVAddr Register )
        CP0.CtrlRegs.BadVAddr.BadVAddr = 0;

        //! CP Register 9 Select 0 ( Count Register )
        CP0.CtrlRegs.Count.Count = 0;

        //! CP Register 10 Select 0 ( EntryHi Register )
        CP0.CtrlRegs.EntryHi.ASID = 0;
        CP0.CtrlRegs.EntryHi.VPN2 = 0;

        //! CP Register 11 Select 0 ( Compare Register )
        CP0.CtrlRegs.Compare.Compare = 0;

        //! CP Register 12 Select 0 ( Status Register )
        CP0.CtrlRegs.Status.IE = 0;	//!   Master Interrupt Enable
        CP0.CtrlRegs.Status.EXL = 1;	//!   Exception Level
        CP0.CtrlRegs.Status.ERL = 0;	//	not an error level, see psxmem for more info
        CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
        CP0.CtrlRegs.Status.UX = 0;
        CP0.CtrlRegs.Status.SX = 0;
        CP0.CtrlRegs.Status.KX = 0;
        CP0.CtrlRegs.Status.IM = 0;	//! 	NO INTERRUPTS ENABLED AT ALL, THIS IS RESET LATER
        CP0.CtrlRegs.Status.NMI = 0;	//!  Not an NMI entry to the reset vector
        CP0.CtrlRegs.Status.SR = MIPS32_HARD_RESET;	//!  Not a soft reset to the entry reset vector
        CP0.CtrlRegs.Status.TS = 0;
        CP0.CtrlRegs.Status.BEV = 1;	//	Bootstrap Exception Vectors
        CP0.CtrlRegs.Status.RE = 0;
        CP0.CtrlRegs.Status.FR = 0;
        CP0.CtrlRegs.Status.RP = 0;
        CP0.CtrlRegs.Status.CU = 7;	//!  CP0, CP1 and CP2 are present, not CP3

        //! CP Register 13 Select 0 ( Cause Register )
        CP0.CtrlRegs.Cause.ExcCode = 0;
        CP0.CtrlRegs.Cause.IP = 0;
        CP0.CtrlRegs.Cause.WP = 0;
        CP0.CtrlRegs.Cause.IV = 0;
        CP0.CtrlRegs.Cause.BD = 0;

        //	CP0 Register 14 select 0 ( EPC Register )
        CP0.CtrlRegs.dwEPC = 0;

        //! CP Register 15 Select 0 ( Processor Identification )
        CP0.CtrlRegs.PRiD.CompanyID = 1;	//	MIPS Technologies ( 1..255 )
        CP0.CtrlRegs.PRiD.ProcessorID = 2;	//	MIPS32 ISA GENERIC
        CP0.CtrlRegs.PRiD.Revision = 0x30;	//	R3000A MIPS 32 ASE SPECIFIC

        //	CP0 Register 16 Select 0 ( Config Register )
        CP0.CtrlRegs.Config.KSEG0CCA == CCA_UNCACHED;		//	KSEG0 is not cache but is mapped
        CP0.CtrlRegs.Config.MT = MMUTYPE_STANDARD_FIXED;	//	MMU TYPE FIXED
        CP0.CtrlRegs.Config.AR = 1;				//	Architecture Revision Level MIPS V (32)
        CP0.CtrlRegs.Config.AT = ISA_DEFAULT;			//	Architecture Type implemented by the processor MIPS 32
        CP0.CtrlRegs.Config.BE = ENDIAN_DEFAULT;		//	Little Endian mode
        CP0.CtrlRegs.Config.M = 1;				//	Config1 register present

        //	CP0 Register 16 Select 1 ( Config1 Register )
        CP0.CtrlRegs.Config1.FP = 1;	//	FPU Present, ie CP1/FPU

        //	This is slowly being implemented, in conjunction with the debugger.dll
#ifdef	CPUMODE_EJTAGDEBUG_IMPLEMENTED
	CP0.CtrlRegs.Config1.EP = 1;	//	EJTAG DEBUG Mode implemented
#else
        CP0.CtrlRegs.Config1.EP = 0;	//	No ETJAG Debug mode
#endif
        CP0.CtrlRegs.Config1.CA = 0;	//	No MIPS16 ASE, not even implemented yet
        CP0.CtrlRegs.Config1.WR = 0;	//	No Watch Registers, ( this makes WatchHi/Lo references to be ignored
        CP0.CtrlRegs.Config1.PC = 0;	//	No Performance Counters Registers, NOTE THIS IS NOT THE SAME AS PSX_Counters!
        CP0.CtrlRegs.Config1.C2 = 1;	//	CP2 present

        //	Cache Control data

        // note that I have not even attempted this yet, even though
        // their is a DCache and ICache arrays within this class, no idea on how to attempt
        // caches under this type of emulation as yet.
        CP0.CtrlRegs.Config1.DA = 0;	//	DCache Associativity ( NONE - DIRECT MAPPED )
        CP0.CtrlRegs.Config1.DL = 0;	//	DCache Line Size ( NO DCACHE PRESENT )
        CP0.CtrlRegs.Config1.DS = 0;	//	DCache sets per way ( 64 ) ignored

        //	Cache Control instructions
        //	as above
        CP0.CtrlRegs.Config1.IA = 0;
        CP0.CtrlRegs.Config1.IL = 0;
        CP0.CtrlRegs.Config1.IS = 0;

	//	this depends on the Config.MT value, but is forced to 0
        switch ( CP0.CtrlRegs.Config.MT ) {

        	case ( MMUTYPE_NONE ) : {
                	break;
                }

        	default : {	//	Unknown, turn on the virtual memory manager
                	CP0.CtrlRegs.Config.MT = MMUTYPE_NONE;
                	CP0.CtrlRegs.Config1.MMUSIZE = 0;
                        break;
                }
        }

        CP0.CtrlRegs.Config1.M = 0;	//	Config2 is not present

        //	CP 0 Register 17 Select 0 ( Load Linked Address Register )
        CP0.CtrlRegs.dwLLAddr = 0;

        //	All other control register have been preset to '0' (zero) to
        // indicate to the architecture that it is not present nor implemented
        // and should generate an appropriate exception at the mips level

        //	Now we need to set some general registers, global, frame pointers, stack etc

        //	_gp reg 30 (access to all virtual addresses)
        //	_fp reg 28 (access to applications virtual addresses)

        //	Set STACK
        CP0.dwGenRegs[ 29 ] = RAM_BASE + RAM_SIZE;	//	Starts at	0x80200000 ( 2MB )
        // note that it is only a 32K stack in total

        CP0.PC = 0xBFC00000;	//	Set entry point

        return( 0 );
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: Terminate( void )
{
        fprintf( fCPULog, "\nCPU Terminated Cycle Count : [ 0x%08x ]\n", dwCPUCycle );
        Suspend();
}
//---------------------------------------------------------------------------
//	Reset the cpu, if resettype is 0 then COLD else SOFT reset
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: Reset( int ResetType )
{	int	i;

	//! 	Reset the emulator to a known, state
	fprintf( fCPULog, "R3000 Reset.\n" );

        //!  Suspend current thread(s), if running
        if ( !Suspended ) {
        	PsxSPU->Suspend();
                lpGTE->Suspend();
                Suspend();
        }

        PsxMem->Reset();

        PsxSPU->Reset();
        lpGTE->Reset();

        //!  General resetting and initialisation, prior to being executed.
        //Initialise();

        //	Clear IRQ Queue
        CurIrq = 0;
	lpIrqQueue[ CurIrq ].cycle = 0;
	lpIrqQueue[ CurIrq ].interrupt = 0;

        // 	Should be ready for execution now, in theory at least, so restart
        // the threads
        PsxMem->Initialise();

        Initialise();
        lpGTE->Initialise();
        PsxSPU->Initialise();

        switch ( ResetType ) {
        	case ( MIPS32_HARD_RESET ) : {	//	Hard Reset 'COLD'
                	CP0.CtrlRegs.Status.SR = MIPS32_HARD_RESET;
                        break;
                }
                case ( MIPS32_SOFT_RESET ) : {	//	Soft Reset 'WARM'
                	CP0.CtrlRegs.Status.SR = MIPS32_SOFT_RESET;
                	break;
                }
                default : {
                	CP0.CtrlRegs.Status.SR = MIPS32_HARD_RESET;
                        break;
                }
        }

        CP0.CtrlRegs.Status.NMI = 0;	//	Not an nmi, but power on entry

        PsxSPU->Resume();
        lpGTE->Resume();
        Resume();
}
//---------------------------------------------------------------------------
//!	METHOD		::	Execute.
//!	This is the actual code loop that is executed, until a reset, or debugger
//! exception occurs. It justs simply decodes each instruction in turn, whilst
//! processing any windows messages, as well.
//---------------------------------------------------------------------------
void 	__fastcall 	R3000A :: Execute( void )
{	MSG	Msg;
	//---- Place thread code here ----

        //----	definitely make sure that we start at the bootvector entry point
        // ie if PC is not equal to ROM_BASE (0xbfc00000) and no nmi is active
        // and the processor is set to HARD_RESET
        if ( ( CP0.PC != 0xBFC00000 ) &&
             ( CP0.CtrlRegs.Status.SR == MIPS32_HARD_RESET ) &&
             ( CP0.CtrlRegs.Status.NMI == 0 )
           )
        	CP0.PC = 0xBFC00000;

        //----	execute while thread is not terminated
        while ( !Terminated ) { //&& ( !Suspended ) ) {
        	//	Any outstanding messages for us ?
                ZeroMemory( &Msg, sizeof( MSG ) );
                if ( PeekMessage( &Msg, NULL, NULL, NULL, PM_REMOVE ) != NULL ) {
                	//	if so process
                        TranslateMessage( &Msg );
                        DispatchMessage( &Msg );
                } else {
                	//	Decode the MIPS 32 Instruction Set Architecture
                        DecodeInstruction();
                        lpGTE->GPU_DrawSync();
                }
        }

        //----	Bye bye time
        PsxMem->Terminate();	//	Memory, also triggers hardware termination
        PsxSPU->Terminate();	//	Sound thread
        lpGTE->Terminate();	//	GTE/GPU thread
        Terminate();		//	and now the CPU, BIOS/KERNEL/OSCORE terminate after CPU
}
//---------------------------------------------------------------------------
//	Decodes MIPS 32 SPECIAL OpCodes, of wHich there are ONLY 64
//	If not recognised then you MUST generate a Reserved Instruction Exception
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeSpecial( void )
{
	fUpdatePC = true;
	switch ( _Funct_ ) {
        	//	MIPS32 Format	::	SLL	rd, rt, sa
                //	Actually this is classed as a NOP
        	case ( MIPS32_SPECIAL_OPCODE_SLL ) : {		//	SLL rd, rt, sa	LoGICAL SHiFT LEFT
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ] << _Sa_;
                        fprintf( fCPULog, "NOP\n" );
                        break;
                }
                
                //	MIPS32 Format	::	MOVC1.cond.fmt	fd, fs, rt
        	case ( MIPS32_SPECIAL_OPCODE_MOVC1 ) : {	//	MOVC1.cond.fmt fd, fs, rt
                	//	cond equal / not equal zero / not zero
                        // Generally if GPR[ rt ] == 0 mov fd, fs else return
                	DecodeCOP1();
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_SRL ) : {		//	SRL SHift Right Logical
			CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ] >> _Sa_;
                        fprintf( fCPULog, "SRL\t%d, %d, %d\n", _Rd_, _Rt_, _Sa_ );
                	break;
                }

                case ( MIPS32_SPECIAL_OPCODE_SRA ) : {		//	SRA SHift Right Arithemetic
                	CP0.dwGenRegs[ _Rd_ ] |= CP0.dwGenRegs[ _Rt_ ] >> _Sa_;
                        fprintf( fCPULog, "SRA\t%d, %d, %d\n", _Rd_, _Rt_, _Sa_ );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_SLLV ) : {		//	SLLV Shift Word Left Logical Variable
                	int sa = CP0.dwGenRegs[ _Rs_ ] & 0x1F;	//	Just first 5-bits please
                        CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ] << sa;
                        fprintf( fCPULog, "SLLV\t%d, %d, %d\n", _Rd_, _Rt_, _Sa_ );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_SRLV ) : {		//	SRLV	Shift word Right Logical Variable
                	int sa = CP0.dwGenRegs[ _Rs_ ] & 0x1F;	//	Just first 5-bits please
                        CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ] >> sa;
                        fprintf( fCPULog, "SRLV\t%d, %d, %d\n", _Rd_, _Rt_, _Sa_ );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_SRAV ) : {		//	SRAV	Shift Word Right Arithmetic Variable
                	int sa = CP0.dwGenRegs[ _Rs_ ] & 0x1F;	//	Just first 5-bits please
                        CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ] >> sa;
                        fprintf( fCPULog, "SRAV\t%d, %d, %d\n", _Rd_, _Rt_, _Sa_ );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_JR ) : {		//	JR	rs
                	CP0.dwGenRegs[ 31 ] = ( CP0.PC + 8 );	//	save return address
                        CP0.PC = _JumpTarget_;//CP0.dwGenRegs[ _Rs_ ];
                        fprintf( fCPULog, "JR %d\n", _Rs_ );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_JALR ) : {		//	JALR Jump And Link Register
                //	THis and the above instruction are the only ones to specify wHich register
                //	to use as the return link register, normally general register 31 (ra) is
                //	used, if no register is specified then the cpu defaults to using ra.

                	if ( ( !_Rd_ ) || ( _Rd_ > 0 ) || ( _Rd_ < 32 ) )
                        	CP0.dwGenRegs[ _Rd_ ] = ( CP0.PC + 8 );
                        else
                        	CP0.dwGenRegs[ 31 ] = ( CP0.PC + 8 );
                        CP0.PC = CP0.dwGenRegs[ _Rs_ ];	//	Load program counter with contents from the register specified
                        fprintf( fCPULog, "JALR\t%d, %d", _Rd_, _Rs_ );
                        fUpdatePC = false;
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_MOVZ ) : {		//	Move conditional on Zero, move equal zero
                //	ie if register rt = 0 then rd = rs
                	if ( CP0.dwGenRegs[ _Rt_ ] == 0 )	{
                        	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ];
                                fprintf( fCPULog, "MOVZ\t%d, %d\n", _Rd_, _Rs_ );
                        }
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_MOVN ) : {		//	Move conditional on not zero
                	if ( CP0.dwGenRegs[ _Rt_ ] != 0 )	{
                        	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ];
                                fprintf( fCPULog, "MOVN\t%d, %d\n", _Rd_, _Rs_ );
                        }
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_SYSCALL ) : {	//	Issue a syscall, BIOS / KERNEL / RUNTIME ?
        		fprintf( fCPULog, "SYSCALL\t%d\n", lpInstr->CPUSType.code );
                	SignalException( CAUSE_SYS, (void*)lpInstr->CPUSType.code );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_BREAK ) : {	//	Issue a breakpoint call bios?
                	fprintf( fCPULog, "BREAK\t%d\n", lpInstr->CPUSType.code );
                        SignalException( CAUSE_BP, (void*)lpInstr->CPUSType.code );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_SYNC ) : {		//	Syncs i-cache, d-cache and system memory
                	fprintf( fCPULog, "SYNC\n" );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_MFHI ) : {		//	Move from Hi
                	CP0.dwGenRegs[ _Rd_ ] = CP0.Hi;
                        fprintf( fCPULog, "MFHI\t%d\n", _Rd_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_MTHI ) : {		//	Move to Hi
                	CP0.Hi = CP0.dwGenRegs[ _Rs_ ];
                        fprintf( fCPULog, "MTHI\t%d\n", _Rs_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_MFLO ) : {		//	Move From Lo
                	CP0.dwGenRegs[ _Rd_ ] = CP0.Lo;
                        fprintf( fCPULog, "MFLO\t%d\n", _Rd_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_MTLO ) : {		//	Move to Lo
                	CP0.Lo = CP0.dwGenRegs[ _Rs_ ];
                        fprintf( fCPULog, "MTLO\t%d\n", _Rs_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_MULT ) : {		//	Multiply 64-bit
                //	(Hi,Lo) = ( rs x rt )
                	__int64	i64 = CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ];
                        CP0.Hi = i64 & 0xFFFFFFFF00000000;
                        CP0.Lo = i64 & 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "MULT\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_MULTU ) : {
                	__int64	i64 = CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ];
                        CP0.Hi = i64 & 0xFFFFFFFF00000000;
                        CP0.Lo = i64 & 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "MULTU\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_DIV ) : {
                	__int64	i64 = CP0.dwGenRegs[ _Rs_ ] / CP0.dwGenRegs[ _Rt_ ];
                        CP0.Hi = i64 & 0xFFFFFFFF00000000;
                        CP0.Lo = i64 & 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "DIV\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_DIVU ) : {
                	__int64	i64 = CP0.dwGenRegs[ _Rs_ ] / CP0.dwGenRegs[ _Rt_ ];
                        CP0.Hi = i64 & 0xFFFFFFFF00000000;
                        CP0.Lo = i64 & 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "DIVU\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_ADD ) : {		//	Add
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] + CP0.dwGenRegs[ _Rt_ ];
                        fprintf( fCPULog, "ADD\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
			if ( CP0.dwGenRegs[ _Rd_ ] && 0x80000000 )	{
                        	SignalException( CAUSE_OV, (void*)PC );
                        }
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_ADDU ) : {
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] + CP0.dwGenRegs[ _Rt_ ];
                        fprintf( fCPULog, "ADDU\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_SUB ) : {
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] - CP0.dwGenRegs[ _Rt_ ];
                        fprintf( fCPULog, "SUB\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                        if ( CP0.dwGenRegs[ _Rd_ ] > 0 ) {
                        	SignalException( CAUSE_OV, (void*)PC );
                        }
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_SUBU ) : {
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] - CP0.dwGenRegs[ _Rt_ ];
                	fprintf( fCPULog, "SUBU\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                        break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_AND ) : {
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] & CP0.dwGenRegs[ _Rt_ ];
                        fprintf( fCPULog, "AND\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_OR ) : {
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] | CP0.dwGenRegs[ _Rt_ ];
                        fprintf( fCPULog, "OR\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_XOR ) : {
                	CP0.dwGenRegs[ _Rd_ ] = CP0.dwGenRegs[ _Rs_ ] % CP0.dwGenRegs[ _Rt_ ];
                        fprintf( fCPULog, "XOR\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_NOR ) : {
                	CP0.dwGenRegs[ _Rd_ ] = !( CP0.dwGenRegs[ _Rs_ ] | CP0.dwGenRegs[ _Rt_ ] );
                        fprintf( fCPULog, "NOR\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_SLT ) : {
                	//	Set on less than
                        CP0.dwGenRegs[ _Rd_ ] = ( CP0.dwGenRegs[ _Rs_ ] < CP0.dwGenRegs[ _Rt_ ] );
                        fprintf( fCPULog, "SLT\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_SLTU ) : {
                	//	Set on less than unsigned
                        CP0.dwGenRegs[ _Rd_ ] = ( CP0.dwGenRegs[ _Rs_ ] < CP0.dwGenRegs[ _Rt_ ] );
                        fprintf( fCPULog, "SLTU\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_TGE ) :
        	case ( MIPS32_SPECIAL_OPCODE_TGEU ) : {
                	//	Trap on greater or equal
                        if ( CP0.dwGenRegs[ _Rs_ ] >= CP0.dwGenRegs[ _Rt_ ] ) {
                        	fprintf( fCPULog, "TGE/U\t%d, %d\n", _Rs_, _Rt_ );
                        	SignalException( CAUSE_TR, (void*)lpInstr->CPURType.code );
                        }
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_TLT ):
        	case ( MIPS32_SPECIAL_OPCODE_TLTU ) : {
                	//	Trap on less or equal
                        if ( CP0.dwGenRegs[ _Rs_ ] <= CP0.dwGenRegs[ _Rt_ ] ) {
                        	fprintf( fCPULog, "TLT/U\t%d, %d\n", _Rs_, _Rt_ );
                        	SignalException( CAUSE_TR, (void*)lpInstr->CPURType.code );
                        }
                	break;
                }

        	case ( MIPS32_SPECIAL_OPCODE_TEQ ) : {
                	//	Trap on equal
                        if ( CP0.dwGenRegs[ _Rs_ ] == CP0.dwGenRegs[ _Rt_ ] ) {
                                fprintf( fCPULog, "TEQ\t%d, %d\n", _Rs_, _Rt_ );
                        	SignalException( CAUSE_TR, (void*)lpInstr->CPURType.code );
                        }
                	break;
                }
        	case ( MIPS32_SPECIAL_OPCODE_TNE ) : {
                	//	Trap on not equal
                        if ( CP0.dwGenRegs[ _Rs_ ] != CP0.dwGenRegs[ _Rt_ ] ) {
                        	fprintf( fCPULog, "TNE\t%d, %d\n", _Rs_, _Rt_ );
                                SignalException( CAUSE_TR, (void*)lpInstr->CPURType.code );
                        }
                	break;
                }

                default : {
                        CP0.CtrlRegs.Cause.ExcCode = CAUSE_RI;
                        CP0.CtrlRegs.Status.EXL = 1;
                        CP0.CtrlRegs.Status.ERL = 1;
                        CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
                        SignalException( CAUSE_RI, (void*)&CP0.PC );
                }
        }
}
//---------------------------------------------------------------------------
//!	Decodes	Register Immediate Instructions.
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeREGIMM( void )
{
	fUpdatePC = true;
	switch ( _Funct_ ) {
                //	MIPS32 Format	::	BLTZ 	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_BLTZ    ) : {
                        if ( CP0.dwGenRegs[ _Rs_ ] < 0 ) {
                        	CP0.PC = ( CP0.PC | _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BLTZ\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        }
                        break;
                }
                //	MIPS32 Format	::	BGEZ 	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_BGEZ    ) : {
                        if ( CP0.dwGenRegs[ _Rs_ ] >= 0 ) {
                        	CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BGEZ\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        }
                        break;
                }
                //	MIPS32 Format	::	BLTZL   rs, immediate
		case ( MIPS32_REGIMM_OPCODE_BLTZL   ) : {
                        if ( CP0.dwGenRegs[ _Rs_ ] < 0 ) {
                        	CP0.dwGenRegs[ 31 ] = CP0.PC;
                        	CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BLTZL\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        }
                	break;
                }
                //	MIPS32 Format	::	BGEZL	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_BGEZL   ) : {
                        if ( CP0.dwGenRegs[ _Rs_ ] >= 0 ) {
                        	CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BGEZL\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        }
                	break;
                }
                //	MIPS32 Format	::	TGEI	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_TGEI    ) : {
                	if ( CP0.dwGenRegs[ _Rs_ ] >= _Im_ ) {
                        	fprintf( fCPULog, "TGEI\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        	SignalException( CAUSE_TR, (void*)&CP0.PC );
                        }
                	break;
                }
                //	MIPS32 Format	::	TGEIU	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_TGEIU   ) : {
                	if ( CP0.dwGenRegs[ _Rs_ ] >= _Im_ ) {
                        	fprintf( fCPULog, "TGEIU\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        	SignalException( CAUSE_TR, (void*)&CP0.PC );
                        }
                	break;
                }
        	//	MIPS32 Format	::	TLTI	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_TLTI    ) : {
                	if ( CP0.dwGenRegs[ _Rs_ ] < _Im_ ) {
                        	fprintf( fCPULog, "TLTI\t%d, %d\n", _Rs_, ( _Im_ << 2 ) );
                        	SignalException( CAUSE_TR, (void*)&CP0.PC );
                        }
                	break;
                }
                //	MIPS32 Format	::	TLTIU	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_TLTIU   ) : {
                	if ( CP0.dwGenRegs[ _Rs_ ] < _Im_ ) {
                        	fprintf( fCPULog, "TLTIU\t%d, %d\n", _Rs_, _Im_ );
                        	SignalException( CAUSE_TR, (void*)&CP0.PC );
                        }
                	break;
                }
                //	MIPS32 Format	::	TEQI	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_TEQI    ) : {
                	if ( CP0.dwGenRegs[ _Rs_ ] == _Im_ ) {
                                fprintf( fCPULog, "TEQI\t%d, %d\n", _Rs_, _Im_ );
                        	SignalException( CAUSE_TR, (void*)&CP0.PC );
                        }
                	break;
                }
                //	MIPS32 Format	::	TNEI	rs, immediate
		case ( MIPS32_REGIMM_OPCODE_TNEI    ) : {
                	if ( CP0.dwGenRegs[ _Rs_ ] != _Im_ ) {
                        	fprintf( fCPULog, "TNEI\t%d, %d\n", _Rs_, _Im_ );
                                SignalException( CAUSE_TR, (void*)&CP0.PC );
                        }
                	break;
                }
                //	MIPS32 Format	::	BLTAL	rs, offset
		case ( MIPS32_REGIMM_OPCODE_BLTZAL  ) : {
                        CP0.dwGenRegs[ 31 ] = ( CP0.PC + 8 );	//	save return address
                        if ( CP0.dwGenRegs[ _Rs_ ] < 0 ) {
                        	CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BLTAL\t%d, %d\n", _Rs_, lpInstr->CPUIType.offset );
                                return;
                        }
                	break;
                }
                //	MIPS32 Format	::	BGEZAL	rs, offset
		case ( MIPS32_REGIMM_OPCODE_BGEZAL  ) : {
                        CP0.dwGenRegs[ 31 ] = ( CP0.PC + 8 );
                        if ( CP0.dwGenRegs[ _Rs_ ] >= 0 ) {
                        	CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BGEZAL\t%d, 0x%04x\n", _Rs_, lpInstr->CPUIType.offset );
                                return;
                        }
                	break;
                }
                //	MIPS32 Format	::	BLTZALL	rs, offset
		case ( MIPS32_REGIMM_OPCODE_BLTZALL ) : {
                        bool	fCondition = ( CP0.dwGenRegs[ _Rs_ ] < 0 );
                        if ( fCondition ) {
                        	CP0.dwGenRegs[ 31 ] = CP0.PC;
                        	CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BLTZALL\t%d, 0x%04x\n", _Rs_, lpInstr->CPUIType.offset );
                                return;
                        }
                	break;
                }
                //	MIPS32 Format	::	BGEZALL	rs, offset
		case ( MIPS32_REGIMM_OPCODE_BGEZALL ) : {
                        bool	fCondition = ( CP0.dwGenRegs[ _Rs_ ] >= 0 );
                        if ( fCondition ) {
                        	CP0.dwGenRegs[ 31 ] = CP0.PC;
                                CP0.PC = ( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                fprintf( fCPULog, "BGEZALL\t%d, 0x%04x\n", _Rs_, lpInstr->CPUIType.offset );
                                return;
                        }
                	break;
                }

                default : {
                        CP0.CtrlRegs.Cause.ExcCode = CAUSE_RI;
                        CP0.CtrlRegs.Status.EXL = 1;
                        CP0.CtrlRegs.Status.ERL = 1;
                        CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
                        SignalException( CAUSE_RI, (void*)&CP0.PC );
                }

	}
}
//---------------------------------------------------------------------------
//	Decodes SPECIAL2 opcodes
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeSpecial2( void )
{
	fUpdatePC = true;
	switch ( _Funct_ ) {
		case ( MIPS32_SPECIAL2_OPCODE_MADD      ) : {		//	Multiple and Add word to Hi/Lo
                	__int64	i64;
                        i64 = ( ( CP0.Hi << 32 ) || CP0.Lo ) + ( CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ] );
                        CP0.Hi = i64 && 0xFFFFFFFF00000000;
                        CP0.Lo = i64 && 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "MADD\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
		case ( MIPS32_SPECIAL2_OPCODE_MADDU     ) : {		//	Same as above
                	__int64	i64;
                        i64 = ( ( CP0.Hi << 32 ) || CP0.Lo ) + ( CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ] );
                        CP0.Hi = i64 && 0xFFFFFFFF00000000;
                        CP0.Lo = i64 && 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "MADDU\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
		case ( MIPS32_SPECIAL2_OPCODE_MUL       ) : {
                	CP0.dwGenRegs[ _Rd_ ] = ( CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ] );
                        fprintf( fCPULog, "MUL\t%d, %d, %d\n", _Rd_, _Rs_, _Rt_ );
                	break;
                }
		case ( MIPS32_SPECIAL2_OPCODE_MSUB      ) : {		//	Multiple and subtract to Hi/Lo
                	__int64	i64;
                        i64 = ( ( CP0.Hi << 32 ) || CP0.Lo ) - ( CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ] );
                        CP0.Hi = i64 && 0xFFFFFFFF00000000;
                        CP0.Lo = i64 && 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "MSUB\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
		case ( MIPS32_SPECIAL2_OPCODE_MSUBU     ) : {
                	__int64	i64;
                        i64 = ( ( CP0.Hi << 32 ) || CP0.Lo ) - ( CP0.dwGenRegs[ _Rs_ ] * CP0.dwGenRegs[ _Rt_ ] );
                        CP0.Hi = i64 && 0xFFFFFFFF00000000;
                        CP0.Lo = i64 && 0x00000000FFFFFFFF;
                        fprintf( fCPULog, "MSUBU\t%d, %d\n", _Rs_, _Rt_ );
                	break;
                }
		case ( MIPS32_SPECIAL2_OPCODE_CLZ       ) : {		//	Count Leading Zeros
                	int	i;
                	for ( i=0; i<=NUM_MIPS_REGS; i++ )	{
                        	if ( CP0.dwGenRegs[ _Rs_ ] & ( 1 << i ) )	{	//	is tHis bit 0 or 1
                                	//	1, break the Loop
                                        break;
                                }
                        }
                        //	write the value to the destination register
                        CP0.dwGenRegs[ _Rd_ ] = i;
                        fprintf( fCPULog, "CLZ\t%d, %d\n", _Rd_, _Rs_ );
                	break;
                }
		case ( MIPS32_SPECIAL2_OPCODE_CLO	) : {		//	Count Leading Ones
                	int	i;
                	for ( i=0; i<=NUM_MIPS_REGS; i++ ) {
                        	if ( !( CP0.dwGenRegs[ _Rs_ ] & ( 1 << i ) ) ) {
                                	break;
                                }
                        }
                        CP0.dwGenRegs[ _Rd_ ] = i;
                        fprintf( fCPULog, "CLO\t%d, %d\n", _Rd_, _Rs_ );
                	break;
                }

//#ifdef	EJTAG_DEBUG_IMPLEMENTED
		//! 	This is part of the EJTAG DEBUG Mode and no further information is available to me at present.
                //! This is going to be used for the interaction between the emulator the debugger. Can also be used
                //! by the ide to allow information to flow between all three
		case ( MIPS32_SPECIAL2_OPCODE_SDBBP	) : {		//! 	SDBBP	code
                //! 	The function that is being requested is stored in the code bits of the instruction
                //! type CPUInstrSysCall. Does not do anything yet as 'Debugger.dll' is blank.
                //	switch ( lpInstr->CPUSType.code ) {
                //        	default : return;
                //        }

                	fprintf( fCPULog, "SDBBP\t%d\n", lpInstr->CPUSType.code );
	        	break;
	        }
//#endif
                default : {
                        CP0.CtrlRegs.Cause.ExcCode = CAUSE_RI;
                        CP0.CtrlRegs.Status.EXL = 1;
                        CP0.CtrlRegs.Status.ERL = 1;
                        CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
                        SignalException( CAUSE_RI, (void*)&CP0.PC );
                }

        }
}
//---------------------------------------------------------------------------
//	Decodes the basic instruction set (64 instructions), it will detected
// any special, REGIMM and CPx specific instructions and decode accordingly
// calling any required methods as needed.
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeInstruction( void )
{
        fUpdatePC = true;

        //	Which instruction is it ?
        //	Defaults to using CPUInstrIType CPU Immediate instructions
        fprintf( fCPULog, "0x%08x\t", CP0.PC );
        dwOpCode = PsxMem->RAM32[ CP0.PC ];

        fprintf( fCPULog, "0x%08x\t", dwOpCode );
        lpInstr = (PCPUInstr)&dwOpCode;

        CP0.CtrlRegs.Cause.BD = 0;	//	Default to not a branch delay slot instruction

        //	Now decode the instruction
        switch ( ( dwOpCode & 0x3F000000 ) >> 26 ) {

        	//	J	<register>/<address>	+/- 256MB
		case (  MIPS32_OPCODE_J ) : {		//	Jump instruction
                //	Jumps anywhere witHin the current 256MB page boundary
                //	PC is previous instruction address Logically or'd with the
                //	new instruction index sHifted left 2 bits

                	//	Should really save this address, then execute the instruction AFTER
                        // this instruction prior to this executionn being executed.

                        //	Save return address, DISABLED
                	//CP0.dwGenRegs[ 31 ] = ( CP0.PC + 8 );

                        //	Calculate new branch address
                        //DWORD d_Target_ = CP0.PC | ( ( dwOpCode & ~0x3F000000 ) << 2 );
                	CP0.PC = _JumpTarget_;//( dwOpCode & 0x003FFFFF ) << 2;

                        //	Tell class not to autoinc the instruction counter
                        fUpdatePC = false;

                        //	tell user (logging) what instruction is being executed
                        fprintf ( fCPULog, "J	0x%08x\n", CP0.PC );

                        //	break, so next time this method is execution the instruction
                        // at the jump destination is executed
                	break;
                }

                //	JAL	<address>	+/- 256MB
		case (  MIPS32_OPCODE_JAL ) : {		//	Jump and Link
                //	Similar to above but set the return address ie general register 31 (ra)
                	CP0.dwGenRegs[ 31 ] = CP0.PC + 8;	//	The returns address is the SECOND instruction after the jump
                        CP0.PC = _JumpTarget_;
                        fprintf( fCPULog, "JAL	0x%08x\n", CP0.PC );
                        fUpdatePC = false;
                        break;
                }

                //	BEQ	rs, rt, offset	if rs == rt J ( PC + offset )
		case (  MIPS32_OPCODE_BEQ ) : {		//	Branch if Equal
                	if ( GenRegs[ _Rs_ ] == GenRegs[ _Rt_ ] )  {
                        	GenRegs[ 31 ] = CP0.PC + 8;
                                CP0.PC = _BranchTarget_;//( CP0.PC ) || ( _Im_ << 2 );
                                fprintf( fCPULog, "BEQ\t%d, %d, 0x%08x\n", _Rs_, _Rt_, CP0.PC );
                                fUpdatePC = false;
                                break;
                        }
                        break;
                }

		case (  MIPS32_OPCODE_BNE ) : {		//	Branch not equal
                	if ( CP0.dwGenRegs[ _Rs_ ] != CP0.dwGenRegs[ _Rt_ ] )  {
                        	CP0.dwGenRegs[ 31 ] = CP0.PC + 8;
                                CP0.PC = _BranchTarget_;//( CP0.PC ) || ( _Im_ << 2 );
                                fprintf( fCPULog, "BNE\t%d, %d, 0x%08x\n", _Rs_, _Rt_, CP0.PC );
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

		case (  MIPS32_OPCODE_BLEZ ) : {		//	branch is less than 0
                	if ( CP0.dwGenRegs[ _Rs_ ] < 0 )	{
                        	CP0.dwGenRegs[ 31 ] = CP0.PC + 8;
                                CP0.PC = _BranchTarget_;//( CP0.PC ) || ( _Im_ << 2 );
                                fprintf( fCPULog, "BLEZ\t%d, 0x%08x\n", _Rs_, CP0.PC );
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

		case (  MIPS32_OPCODE_BGTZ ) : {		//	Branch is greater than 0
                	if ( CP0.dwGenRegs[ _Rs_ ] > 0 )	{
                        	CP0.dwGenRegs[ 31 ] = CP0.PC + 8;
                                CP0.PC = _BranchTarget_;//( CP0.PC ) || ( _Im_ << 2 );
                                fprintf( fCPULog, "BGTZ\t%d, 0x%08x\n", _Rs_, CP0.PC );
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

                //	ADDI	rd, rt, 16-bit signed immediate
                //		rd = rt + immediate, exception on overflow
		case (  MIPS32_OPCODE_ADDI ) : {		//	Add immediate
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] + _Im_ );
                        fprintf( fCPULog, "ADDI\t%d, %d, 0x%04\n", _Rt_, _Rs_, _Im_ );
                        if ( CP0.dwGenRegs[ _Rt_ ] & 0x80000000 )	{	//	High set ?
                        	SignalException( CAUSE_OV, (void*)&CP0.PC );
                        }
                        //	No exception occured
                	break;
                }

		//	ADDIU	rd, rt, immediate
                //		rd = rt + immediate, no exception
    		case (  MIPS32_OPCODE_ADDIU ) : {	//	Add unsigned immediate, similar to above but no overfLow can be generated
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] + _Im_ );
                        fprintf( fCPULog, "ADDIU\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
                	break;
                }

                //	SLTI	rt, rs, immediate
                //		if ( rs < immediate ) rt = 1 else rt = 0;
		case (  MIPS32_OPCODE_SLTI ) : {		//	Set on less than
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] < _Im_ );
                        fprintf( fCPULog, "SLTI\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
                	break;
                }

		case (  MIPS32_OPCODE_SLTIU ) : {
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] < _Im_ );
                        fprintf( fCPULog, "SLTIU\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
                	break;
                }

		case (  MIPS32_OPCODE_ANDI ) : {		//	Logical And
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] && CP0.dwGenRegs[ _Im_ ] );
                        fprintf( fCPULog, "ANDI\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
                	break;
                }

		case (  MIPS32_OPCODE_ORI ) : {		//	Logical OR Immediate
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] | _Im_ );
                        fprintf( fCPULog, "ORI\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
                	break;
                }

		case (  MIPS32_OPCODE_XORI ) : {		//	Exclusive OR Immediate
                	CP0.dwGenRegs[ _Rt_ ] = ( CP0.dwGenRegs[ _Rs_ ] ^ _Im_ );
                        fprintf( fCPULog, "XORI\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
                	break;
                }


                //	Format	LUI rt, immediate	Reg[ rt ] = im << 16 | 0
		case ( MIPS32_OPCODE_LUI ) : {		//	Load Upper Immediate
                	CP0.dwGenRegs[ _Rt_ ] = ( _Im_ << 16 ) | 0x0000;
                        if ( _Im_ & 0x8000 )
                        	CP0.dwGenRegs[ _Rt_ ] |= 0x80000000;
                        fprintf( fCPULog, "LUI %d, 0x%08x\n", _Rt_, ( _Im_ << 16 ) );
                	break;
                }

		case (  MIPS32_OPCODE_COP0 ) : {		//	Syscontrol specific instruction
                	DecodeCOP0();
                	break;
                }

                //	Decode instruction specific to the FPU
		case ( MIPS32_OPCODE_COP1 ) : //{		//	FPU specific instruction
                	//break;
                //}
                case ( MIPS32_OPCODE_LWC1 ) :
                case ( MIPS32_OPCODE_LDC1 ) :
                case ( MIPS32_OPCODE_SWC1 ) :
                case ( MIPS32_OPCODE_SDC1 ) : {
                	DecodeCOP1();
                	break;
                }

                //	and again for the GTE / GPU. This also signals the thread to resume execution
		case (  MIPS32_OPCODE_COP2 ) : 		//	GTE/GPU specific instruction
		case (  MIPS32_OPCODE_LDC2 ) :		//	load double word coprocessor 2
		case (  MIPS32_OPCODE_LWC2 ) :		//	Load word coprocessor 2
		case (  MIPS32_OPCODE_SWC2 ) :		//	Store Word to COP2 (32-bit)
		case (  MIPS32_OPCODE_SDC2 ) : {	//	Store Double Word to COP2 (64-bit)
                	DecodeCOP2();
                	break;
                }

                //	this is not implemented, cause exception
		case (  MIPS32_OPCODE_COP3 ) : {		//	Unknown, not implemented
                	DecodeCOP3();
                	break;
                }

                //	Branch on Equal Likely
		case (  MIPS32_OPCODE_BEQL ) : {
                //	goes something like this

                //	target_offset	<---	offset || 0 (bits 0-1)
                //	condition	<---	( GPR[ rs ] = GPR[ rt ] )
                //	if condition then
                //		PC	<---	PC + target_offset
                //	else
                //		NullifyCurrentInstruction();
                //	endif

                        bool	fCondition = ( CP0.dwGenRegs[ _Rs_ ] == CP0.dwGenRegs[ _Rt_ ] );

                        if ( fCondition )  {	//	Condition triggered ?
                        	fprintf( fCPULog, "BEQL\t%d, %d, 0x%4x\n", _Rs_, _Rt_, _Im_ );
                        	CP0.PC = ( CP0.PC + _Target_ );	//	then branch
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

                //	Branch on Not Equal Likely
		case (  MIPS32_OPCODE_BNEL ) : {
                        bool	fCondition = ( CP0.dwGenRegs[ _Rs_ ] != CP0.dwGenRegs[ _Rt_ ] );

                        if ( fCondition )  {
                        	fprintf( fCPULog, "BNEL\t%d, %d, 0x%4x\n", _Rs_, _Rt_, _Im_ );
                        	CP0.PC = _BranchTarget_;//( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

                //	Branch on Less than or Equal to Zero Likely
		case (  MIPS32_OPCODE_BLEZL ) : {
                        bool	fCondition = ( CP0.dwGenRegs[ _Rs_ ] <= 0 );

                        if ( fCondition )  {
                        	fprintf( fCPULog, "BLEZL\t%d, %d, 0x%4x\n", _Rs_, _Rt_, _Im_ );
                        	CP0.PC = _BranchTarget_;//( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

                //	Branch on Greater Than Zero Likley
		case (  MIPS32_OPCODE_BGTZL ) : {
                        bool	fCondition = ( CP0.dwGenRegs[ _Rs_ ] > 0 );

                        if ( fCondition )  {
                        	fprintf( fCPULog, "BGTZL\t%d, %d, 0x%4x\n", _Rs_, _Rt_, _Im_ );
                        	CP0.PC = _BranchTarget_;//( CP0.PC + _Target_ );
                                fUpdatePC = false;
                                break;
                        }
                	break;
                }

                //	SPECIAL 2 Class instructions
		case (  MIPS32_OPCODE_SPECIAL2 ) : {
                	DecodeSpecial2();
                	break;
                }

                //	jump to 16-bit code from 32-bit code
		//case (  MIPS32_OPCODE_JALX ) : {
                //	break;
                //}

                //	Load byte ( 8-bit )
		case (  MIPS32_OPCODE_LB ) : {
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] = PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LB\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                }

                //	Load halfword ( 16-bit )
		case (  MIPS32_OPCODE_LH ) : {
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] = PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LH\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                }

                //	Load Word Left, load unaligned 32-bit value
		case ( MIPS32_OPCODE_LWL ) :
                //	Load Word	rt, base, offset
		case ( MIPS32_OPCODE_LW ) : {
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] = PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LW/L\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                 }

		case (  MIPS32_OPCODE_LBU ) : {
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] = PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LBU\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                }

		case (  MIPS32_OPCODE_LHU ) : {
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] = PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LHU\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                }

		case (  MIPS32_OPCODE_LWR ) : {
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] =  PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LWR\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                }

                //	Store Byte 8-bit write
		case (  MIPS32_OPCODE_SB ) : {	//	SB reg, offset( reg )
                	DWORD vAddr = ( CP0.dwGenRegs[ _Rs_ ] | _Im_ );
                        PsxMem->RAM8[ vAddr ] = CP0.dwGenRegs[ _Rt_ ];
                	break;
                }

                //	Store Halfword 16-bit write
		case (  MIPS32_OPCODE_SH ) : {
                	DWORD vAddr = ( CP0.dwGenRegs[ _Rs_ ] | _Im_ );
                        PsxMem->RAM16[ vAddr ] = CP0.dwGenRegs[ _Rt_ ];
                	break;
                }

                //	Store Word Left
		case (  MIPS32_OPCODE_SWL ) : {
                	DWORD vAddr = ( CP0.dwGenRegs[ _Rs_ ] | _Im_ );
                        PsxMem->RAM32[ vAddr ] = CP0.dwGenRegs[ _Rt_ ];
                	break;
                }

                //	Store word
		case (  MIPS32_OPCODE_SW ) : {
                	DWORD vAddr = ( CP0.dwGenRegs[ _Rs_ ] | _Im_ );
                        PsxMem->RAM32[ vAddr ] = CP0.dwGenRegs[ _Rt_ ];
                	break;
                }

		case (  MIPS32_OPCODE_SWR ) : {
                	break;
                }

		case (  MIPS32_OPCODE_CACHE ) : {
                	break;
                }

		case (  MIPS32_OPCODE_LL ) : {
                	fAtomicOperation = true;
                	DWORD	vAddr = ( CP0.dwGenRegs[ lpInstr->CPUIType.base ] + lpInstr->CPUIType.offset );
                        CP0.dwGenRegs[ _Rt_ ] =  PsxMem->RAM32[ vAddr ];
                        fprintf( fCPULog, "LL\t%d, %d, 0x%04x\n", _Rt_, lpInstr->CPUIType.base, lpInstr->CPUIType.offset );
                	break;
                }

                //	Prefetch, instruction / data ?
		case (  MIPS32_OPCODE_PREF ) : {
			break;
                }

                //	Atomic store operation, clears any atmoic update that may
                // be active.
		case (  MIPS32_OPCODE_SC ) : {
                	fAtomicOperation = false;
                        //	FlushCache();	//	???
                        break;
                }

                //	SPECIAL opcode needs further decoding
        	case (  MIPS32_OPCODE_SPECIAL ) : {
        		DecodeSpecial();
                        break;
                }

                //	Register Immediate type instruction
                case (  MIPS32_OPCODE_REGIMM ) : {
                	DecodeREGIMM();
                	break;
                }
                default : {
                        CP0.CtrlRegs.Cause.ExcCode = CAUSE_RI;
                        CP0.CtrlRegs.Status.EXL = 1;
                        CP0.CtrlRegs.Status.ERL = 1;
                        CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
                        //SignalException( CAUSE_RI, (void*)&CP0.PC );
                        fprintf( fCPULog, "RI\n" );
                }

        }	//	END OF SWITCH

        //	otherwise the next instruction 'should' be executed in the
        // execution stream.

        //	Note that updating of the register and signalling of exceptions
        // etc is handled on an instruction by instruction basis. note that branch
        // delayed instructions are not processed, oops!

        if ( fUpdatePC )
        	CP0.PC += 4;

        dwCPUCycle++;
}
//---------------------------------------------------------------------------
//	Return the value stored in the fpu general register fpr in the format specified
//	ie Single or Double precision ( if supported ), Note only the Paired Single
//	Precision is supported under MIPS 64 NOT MIPS 32 (ie PlayStation )
//---------------------------------------------------------------------------
float	__fastcall	R3000A :: ValueFPR( int fpr, int fmt )
{	float	fret;
	switch ( fmt ) {
        	case ( FPU_FMT_SINGLE )  : {	//! 	float
                	fret = CP1.GenRegs[ fpr ].s;
                        break;
                }
                case ( FPU_FMT_DOUBLE ) : {	//! 	double
        		fret = CP1.GenRegs[ fpr ].d;
                        break;
                }
                case ( FPU_FMT_FIXED ) : {	//!  DWORD
                	fret = CP1.GenRegs[ fpr ].dw;
                        break;
                }
                default : {
                	//	Generate the right exception, update the FPU registers
                        // not done at present

                	CP0.CtrlRegs.Cause.CE = 1;
                	SignalException( CAUSE_FPE, (void*)fmt );
                        return( 0.0f );
                }
        }
        return( fret );
}
//---------------------------------------------------------------------------
//!	METHOD		::	StoreFPR.
//!	Writes the value into the fpu general register in the requested format.
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: StoreFPR( int fpr, int fmt, float value )
{
	switch ( fmt ) {
        	case ( FPU_FMT_SINGLE ) : {
                	CP1.GenRegs[ fpr ].s = value;
                	return;
                }
                case ( FPU_FMT_DOUBLE ) : {
                	CP1.GenRegs[ fpr ].d = value;
                        return;
                }
                case ( FPU_FMT_FIXED ) : {
                	CP1.GenRegs[ fpr ].dw = (DWORD)value;
                        return;
                }
                default : {
                	//	Again here we need to update the, registers
                        // but not done at present
                	CP0.CtrlRegs.Cause.CE = 1;
                        SignalException( CAUSE_FPE, (void*)fmt );
                        return;
                }
        }
}
//---------------------------------------------------------------------------
//	Cache Sync Operations, MIPS 32 docs very short in this area :-(
//	Presume it would allow for things like synchronising the cpu cache
//	with the actual contents of the memory
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SyncOperation( int stype )
{
	switch ( stype ) {
        	case ( SYNCTYPE_DATA ) : {	//	Sync the data cache
                	break;
                }
                case ( SYNCTYPE_CODE ) : {	//	Sync the instruction cache
                	break;
                }
        }
}
//---------------------------------------------------------------------------
//!	MIPS 32 ISA Exception Handling.
//!	Exception Handler for the CPU Emulation Layer, responds appropriately to
//! all documented exceptions. Note that the CAUSE_SD (Debugger) is implementation
//! specific, so I have created a programmers architecture to allow MIPS 32 code
//! to interact with the debugger running on the emulator under windows, to provide
//! a way of interacting with the user.
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SignalException( DWORD Exception, void *Arguments )
{	static WORD	wOffset	= 0x0000;
	static DWORD	dwBase  = 0x00000000;
	//!	This is the main entry point for ANY exception that may occur within
        //!	the emulators execution period.
        //
        //!	The first stage is to set any appropriate registers in the syscontrol
        //!	processor of CP0 ie general ones independant of what type of exception
        //!	is occuring. then with the switch statement beLow adjust them according
        //!	to the specific requirements of the exception handler's expectations.
        //
        //!	Basically we just alter any register and/or memory Location needed
        //!	to help the emulated bios handle them correctly.
        //!	IE all that is done here, is to set the registers and HWM memory area
        //! 	to what the actual hardware would be set to, if appropriate. And set
        //!	CP0.PC to the required exception vector entry point. See MIPS32 ISA
        //!	PRM for further information.
        //
        //!	Note that depending on which exception is being generated depends on the
        //! 	'entry point' and/or offset of the vector from specific locations, this
        //!	behaviour is also dependant on the current state of CP0.


        //	Note that at present it is emulating the bios, and calls the relevant
        // class method, if appropriate else is just disabled and ignored. The above will
        // be implemented when I can

        //	Save current execution point
        CP0.CtrlRegs.EPC.EPC = CP0.PC;
        CP0.CtrlRegs.ErrorEPC = CP0.PC;
        CP0.CtrlRegs.DEPC = CP0.PC;

        //	Disable IRQs, EnterCriticalSection
        CP0.CtrlRegs.Status.IE = 0;

        //	Switch to kernel mode
        if ( ( CP0.CtrlRegs.Status.ERL ) || ( CP0.CtrlRegs.Status.EXL ) &&
        	( CP0.CtrlRegs.Status.KSU != CPUMODE_KERNEL ) && ( CP0.CtrlRegs.Status.KSU != CPUMODE_DEBUG ) )
        	CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;

        //	Setup Exception Cause
        CP0.CtrlRegs.Cause.ExcCode = Exception;           	//!	This must be set, inidicates the exception type

        //	Yes we are at EXCEPTION LEVEL
        CP0.CtrlRegs.Status.EXL = 1;				//!	Set this, only 3 exceptions turn it off, so it's needed

        //! 	Which exception has been generated.
	switch ( Exception ) {

        	case ( CAUSE_INT      ) : {			//!	Interrupt is being requested

                	//	Master Interrupt Enable
                        if ( CP0.CtrlRegs.Status.IE == 0 ) {
                        	CP0.CtrlRegs.Cause.IP |= ~CAUSE_INT;
                        	break;
                        }

                        //	Individual Interrupt Enable
                        if ( CP0.CtrlRegs.Cause.IP & CP0.CtrlRegs.Status.IM ) {
                        	CP0.CtrlRegs.Cause.IP |= ~CAUSE_INT;
                        	break;
                        }

                        if ( CP0.CtrlRegs.Cause.IP & 0x01 ) {	//	Software Interrupt 0	BIOS <--> KERNEL
                        	//	BIOS->DoInterrupt();
                                //	KERNEL->DoInterrupt();
                                CP0.CtrlRegs.Cause.IP |= ~0x01;	//	Turn it off
                        }

                        if ( CP0.CtrlRegs.Cause.IP & 0x02 ) {	//	Software Interrupt 1	OSCORE <--> BIOS/KERNEL
                        	//	BIOS->DoInterrupt();
                                //	KERNEL->DoInterrupt();
                                CP0.CtrlRegs.Cause.IP |= ~0x02;	//	Turn it off
                        }

                        if ( CP0.CtrlRegs.Cause.IP & 0x04 ) {	//	Hardware Interrupt 0	DMA wants some attention
                        	PSX_DMA->DoDMA( PsxMem->RAM32[ DMA_IRQ_CTL ] );
                                CP0.CtrlRegs.Cause.IP |= ~0x04;	//	Turn it off
                        }

                        if ( CP0.CtrlRegs.Cause.IP & 0x08 ) {	//	Hardware Interrupt 1	CDROM wants some attention
                        	CDROM->DoCDReading();
                                CP0.CtrlRegs.Cause.IP |= ~0x08;	//	Turn it off
                        }
                        if ( CP0.CtrlRegs.Cause.IP & 0x10 ) {	//	Hardware Interrupt 2	SPU wants some attention
                        	//	SPU->DoInterrupt();
                                CP0.CtrlRegs.Cause.IP |= ~0x10;	//	Turn it off
                        }
                        if ( CP0.CtrlRegs.Cause.IP & 0x20 ) {	//	Hardware Interrupt 3	SIO wants some attention
                        	//	SIO->DoInterrupt();
                                CP0.CtrlRegs.Cause.IP |= ~0x20;	//	Turn it off
                        }
                        if ( CP0.CtrlRegs.Cause.IP & 0x40 ) {	//	Hardware Interrupt 4	???
                        	//	BIOS->DoInterrupt();
                                CP0.CtrlRegs.Cause.IP |= ~0x40;	//	Turn it off
                        }
                        if ( CP0.CtrlRegs.Cause.IP & 0x80 ) {	//	Hardware Interrupt 5	root counters, timers, performance counters
                        	//	BIOS->DoInterrupt();
                                CP0.CtrlRegs.Cause.IP |= ~0x80;	//	Turn it off
                        }
                }

        	case ( CAUSE_MOD      )	: {			//	TLB Modified, Write to RAM
                //	A write to current tlb addressing was detected
                	break;
                }
        	case ( CAUSE_TLBL     )	: {			//	TLB Load or Instruction Fetch
                //	Failed to Load an instruction, usually from the prefetch routine
                	break;
                }
        	case ( CAUSE_TLBS     )	: {			//	TLB Store
                //	TLB Entry has been modified
                	break;
                }
        	case ( CAUSE_ADEL     )	: {			//	Address error ( Load or instruction fetch )
                //	Arguments point to the virtual addr of the Location that caused the error
                	CP0.CtrlRegs.EPC.EPC = (DWORD)Arguments;
                	break;
                }
        	case ( CAUSE_ADES     )	: {			//      Address error ( store )
                	CP0.CtrlRegs.EPC.EPC = (DWORD)Arguments;
                	break;
                }
        	case ( CAUSE_IBE      )	: {	                //	Bus error ( instruction fetch )
                	break;
                }
        	case ( CAUSE_DBE      )	: {	                //	Bus error ( data Load or store )
                	break;
                }


        	case ( CAUSE_SYS      )	: {	                //	Syscall exception
                	DWORD	dwSysCall = (DWORD)Arguments;

                	SystemMessage( "SysCall Exception detected : " + IntToStr( dwSysCall ) );

                        switch ( dwSysCall ) {
                        	case ( 0 ) : {
                                	break;
                                }
                                default : {
                                	break;
                                }
                        }
                        PsxBIOS->Interrupt();
                        PsxBIOS->Exception();
                	break;
                }

        	case ( CAUSE_BP	      )	: {              	//	Breakpoint exception
                	//! 	BREAKPOINT	DEPENDANT SPECIFIC IMPLEMENTATION
                        PsxBIOS->Interrupt();
                        PsxBIOS->Exception();
                        //	lpDebugger->BreakPoint( (DWORD*)Arguments );
                	break;
                }

        	case ( CAUSE_RI	      )	: {             	//	Reserved instruction
                	DWORD *dwAddr = (DWORD*)Arguments;
                	break;
                }

        	case ( CAUSE_CPU      )	: {	              	//	coprocessor unusable
                //	Processor that caused the fault is pointed to by Arguments
                	CP0.CtrlRegs.Cause.CE = (int)Arguments;
                	break;
                }

        	case ( CAUSE_OV	      )	: {             	//	Integer overfLow
                	break;
                }

        	case ( CAUSE_TR	      )	: {             	//	trap, generally caused to allow emulation of fpu or other instructions in software
                        break;
                }

                //! 	Floating Point Exception.
        	case ( CAUSE_FPE      )	: {
                	//! 	Examine the CP1 Control Cause/Status Register ( FCSR )

                        //!  Firstly check for the 'Cause' to be 'Unimplemented Operation'
                        //! exception, as this is for software emulation (thank you!) and needs
                        //! to be processed. If it is not an unimplemented operation then decode
                        //! as a floating point exception.

                        //!  also note that the exception is only processed IF it is enabled
                        //! ie ( cause.exception type & enables.exception type ) == true

                        if ( CP1.FCSR.Cause.E )	{	//! 	yes so process or return
                        	CP1.FCSR.Cause.E = 0;
                        	break;
                        }

                        if ( ( CP1.FCSR.Cause.I ) & ( CP1.FCSR.Enables.I ) ) {	//! 	Invalid Operation
                        	CP1.FCSR.Cause.I = 0;
                        	break;
                        }

                        if ( ( CP1.FCSR.Cause.U ) & ( CP1.FCSR.Enables.U ) ) {	//! 	Underflow
                        	CP1.FCSR.Cause.U = 0;
                        	break;
                        }

                        if ( ( CP1.FCSR.Cause.O ) & ( CP1.FCSR.Enables.O ) ) {	//!  Overflow
                        	CP1.FCSR.Cause.O = 0;
                        	break;
                        }

                        if ( ( CP1.FCSR.Cause.Z ) & ( CP1.FCSR.Enables.Z ) ) {	//! 	Divide by zero
                        	CP1.FCSR.Cause.Z = 0;
                        	break;
                        }

                	break;
                }

        	case ( CAUSE_C2E      )	: {	              	//	precise coprocessor 2 exceptions
                	break;
                }

#ifdef	MIPS64_ISA
        	case ( CAUSE_MDMX     )	: {	              	//	reserved for MDMX
                	break;
                }
#endif
        	case ( CAUSE_WATCH    )	: {	              	//	reference to watch Hi/Lo address
                	break;
                }
        	case ( CAUSE_MCHECK   )	: {	              	//	macHine check, no info available for tHis exception
                	break;
                }
        	case ( CAUSE_CACHEERR )	: {	              	//	cache eror, relates to EJTAG mode ie debug mode
                	break;
                }

                //! 	ADDED CAUSE_TLBR
        	case ( CAUSE_TLBR     )	: {			//	TLB REFILL NOT PART OF THE MIPS 32 ISA
                	break;
                }
//#ifdef CPUMODE_EJTAGDEBUG_IMPLEMENTED
                case ( CAUSE_SDB ) : {		//	Debugger
                	DWORD	dwDebugFunction = (DWORD)Arguments;	//	obtain function
                        int	m = CP0.CtrlRegs.Status.KSU;

                	CP0.CtrlRegs.DEPC = CP0.PC;		//	Save PC

                        CP0.CtrlRegs.Debug.DM = 1;              //	Set CPU operating mode to debug
                        CP0.CtrlRegs.Status.KSU = CPUMODE_DEBUG;

                        CP0.CtrlRegs.Debug.Debug = dwDebugFunction;	//	Save debug function, for later if needed

                        switch ( dwDebugFunction ) {

                        	case ( SDBP_ACTIVATE ) : {
                                	//	Broadcast to the debugger to activate
                                        BroadcastSystemMessage( NULL, NULL, WM_DBG_ACTIVATE, NULL, NULL );
                                        break;
                                }

                                case ( SDBP_DEACTIVATE ) : {
                                	//	Broadcast to the debugger to deactivate
                                        BroadcastSystemMessage( NULL, NULL, WM_DBG_DEACTIVATE, NULL, NULL );
                                        break;
                                }

				case ( SDBP_SET_BREAKPOINT	  ) : {	//	Set break point address
                                	DWORD	Flags, Addr;

                                        Flags = CP0.dwGenRegs[ 11 ];	// a0
                                        Addr = CP0.dwGenRegs[ 12 ];	// a1

                                        BroadcastSystemMessage( NULL, NULL, WM_DBG_SET_BREAK, Flags, Addr );

                                	break;
                                }
				case ( SDBP_CLEAR_BREAKPOINT	  ) : {	//	Clear a break point address
                                	break;
                                }
				case ( SDBP_CLEAR_ALL_BREAKPOINTS ) : {
                                	break;
                                }
				case ( SDBP_SINGLE_STEP           ) : {
                                	break;
                                }
				case ( SDBP_EXECUTE               ) : {
                                	break;
                                }
				case ( SDBP_CORE_DUMP             ) : {
                                	break;
                                }
				case ( SDBP_TERMINATE             ) : {
                                	break;
                                }
				case ( SDBP_READ                  ) : {
                                	break;
                                }
				case ( SDBP_WRITE                 ) : {
                                	break;
                                }
				case ( SDBP_TRIP_BREAKPOINT       ) : {
                                	break;
                                }

                        	default : {
                                	break;
                                }
                        }

                        //	restore cpu mode
                        CP0.CtrlRegs.Status.KSU = m;
                        CP0.CtrlRegs.Debug.DM = 0;

                	break;
                }
//#endif
                //!  else ignore it
                default : {
                	//	Turn it off
                	CP0.CtrlRegs.Cause.IP &= ~Exception;
                        //	and disable it, so it it ignored
                        CP0.CtrlRegs.Status.IM &= ~Exception;
                        break;
                }
        }

        //	DISABLED
        //	Follow the MIPX 32 ISA Exception Handling

        //	CP0.DwGenRegs[ 31 ] = CP0.PC;
        //	CP0.PC = ( dwBase + wOffset );	//	Entry desired vector

        //	Switch back to user mode, and restart where exception or error
        // caught us

        if ( CP0.CtrlRegs.Status.ERL ) {
        	CP0.CtrlRegs.Status.ERL = 0;
                CP0.PC = CP0.CtrlRegs.ErrorEPC;
        } else {
        	CP0.CtrlRegs.Status.EXL = 0;
                CP0.PC = CP0.CtrlRegs.EPC.EPC;
        }

        //	Switch back to user mode, note if entered in debug mode return to kernel mode first
        if ( CP0.CtrlRegs.Status.KSU == CPUMODE_DEBUG ) {
        	CP0.PC = CP0.CtrlRegs.DEPC;
                CP0.CtrlRegs.Status.KSU = CPUMODE_KERNEL;
        } else
        	CP0.CtrlRegs.Status.KSU = CPUMODE_USER;

        //	and re-enable irqs, ExitCriticalSection
        CP0.CtrlRegs.Status.IE = 1;

        if ( CP0.CtrlRegs.Cause.IP & Exception )
        	CP0.CtrlRegs.Cause.IP &= ~Exception;
}
//---------------------------------------------------------------------------
//!	METHOD		::	NullifyCurrentInstruction.
//! No documentation is available on this, so I have just increment the PC to the
//! next logical instruction and turning off the BD bit in the cp0 cause register.
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: NullifyCurrentInstruction( void )
{
	CP0.CtrlRegs.Cause.BD = 0;
        CP0.PC += 4;
}
//---------------------------------------------------------------------------
//!	METHOD		::	CoProcessorOperation.
//! This decodes the instruction 'COPz', where 'z' is equal to 0 through 3 and
//! 'cop_fun' is an implementation dependant coprocessor specific function.
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: CoProcessorOperation( int z, int cop_fun )
{
	if ( ! ( z && CP0.CtrlRegs.Status.CU ) )	{	//	Coproccesor present ?
        	SignalException( CAUSE_CPU, (void*)z );		//	No, oh dear!
                return;
        }

        switch ( z ) {					//	Which CPU do we want ?

        	case ( 0 ) : {		//	CP0 or SysControl processor, ie access to control regs BE CAREFUL!
                	DecodeCOP0();
                	break;
                }

                case ( 1 ) : {		//	CP1 or FPU, ie maths shit BORING!
                	DecodeCOP1();
                	break;
                }

                case ( 2 ) : {		//	GPU/GTE pair, does nothing at present
                	break;
                }

                case ( 3 ) : {		//	FPU1 IF PRESENT!
                	DecodeCOP3();
                	break;
                }

                default : {
                	SignalException( CAUSE_CPU, (void*)z );
                }
        }
}
//---------------------------------------------------------------------------
//	Returns true/false depending on the instruction located at RAM[ vAddr ]
//---------------------------------------------------------------------------
bool	__fastcall	R3000A :: JumpDelaySlot( DWORD vAddr )
{	PCPUInstr	pInstr = (CPUInstr*)&PsxMem->lpPsxRAM[ ( vAddr & 0x001FFFFF ) ];

        switch ( pInstr->CPUIType.opcode ) {
        	case ( MIPS32_SPECIAL_OPCODE_JR   ) :
                case ( MIPS32_SPECIAL_OPCODE_JALR ) :
                case ( MIPS32_OPCODE_JALX 	  ) :
        	case ( MIPS32_OPCODE_JAL  	  ) :  {
                	CP0.CtrlRegs.Cause.BD = 1;
                        break;
                }
        	default : {
                	CP0.CtrlRegs.Cause.BD = 0;
                        break;
                }
        }
        return( CP0.CtrlRegs.Cause.BD );
}
//---------------------------------------------------------------------------
//!	FPU Condition Code Check.
//! Checks the status of the FPU against a supplied condition code, returns
//! true/false value. (should be integer value return?)
//---------------------------------------------------------------------------
bool	__fastcall	R3000A :: FPConditionCode( int cc )
{
	if ( cc == 0 )
        	return( CP1.FCSR.FCC0 );
        else
        	return( CP1.FCSR.FCC1 + cc );
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: SetFPConditionCode( int cc )
{
	CP1.FCSR.FCC1 |= cc;
}
//---------------------------------------------------------------------------
//	Decode specific COP0 Instructions, of which there are only 2
//	MFC0 and MTC0 both in the format of rs, rd, sel
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeCOP0( void )
{
	//	Make sure it is a COP 0 specific instruction
	if ( ( ( dwOpCode & 0x3F000000 ) >> 26 ) != MIPS32_OPCODE_COP0 ) {
        	SignalException( CAUSE_CPU, (void*)0 );
                return;
        }

        //	First check for other operation, ie tlb, exception return and debug return
        if ( _Funct_ & MIPS32_COP0_CO )	{

        	switch ( _Rs_ ) {

                	case ( MIPS32_COP0_TLBR  ) : {
                        	//	Read the TLB entry to appropriate CPU register from
                                // the tlb array indexed by the CP0 Register 'Index'

                                //	Modifies	EntryLo0, EntryLo1, EntryHi, PageMask registers
                                // well actually the memory emulation class does
                                PsxMem->TLBRead();
                                fprintf( fCPULog, "TLBR\n" );
                        	break;
                        }

                        //	Write an entry to the tlb array, use the Index register
                        case ( MIPS32_COP0_TLBWI ) : {
                        	PsxMem->TLBWriteIndexed();
                                fprintf( fCPULog, "TLBWI\n" );
                        	break;
                        }

                        //	write an entry to the tlb array, use the 'Random' register
                        case ( MIPS32_COP0_TLBWR ) : {
                        	PsxMem->TLBWriteRandom();
                                fprintf( fCPULog, "TLBWR\n" );
                        	break;
                        }

                        //	Does Index/Random entry exist ? or does EntryHi match
                        case ( MIPS32_COP0_TLBP  ) : {
                        	PsxMem->TLBProbe();
                                fprintf( fCPULog, "TLBP\n" );
                        	break;
                        }

                        //	Exception return
                        case ( MIPS32_COP0_ERET  ) : {
                        	fUpdatePC = false;
                        	CP0.PC = CP0.CtrlRegs.dwEPC;
                                fprintf( fCPULog, "ERET\n" );
                        	break;
                        }

                        //	Debug return
                        case ( MIPS32_COP0_DERET ) : {
                        	fUpdatePC = false;
                        	CP0.PC = CP0.CtrlRegs.DEPC;
                                CP0.CtrlRegs.Debug.DM = 0;
                                fprintf( fCPULog, "DERET\n" );
                        	break;
                        }

                        //	enter lower power mode
                        case ( MIPS32_COP0_WAIT  ) : {
                        	//	CP0.CtrlRegs.Config.RP = 1;
                                fprintf( fCPULog, "WAIT\n" );
                        	break;
                        }

                }

        } else {

        	//	Instruction for COP0 is located in the 'rs' field
        	switch ( _Rs_ ) {

                	//	CP0 CPR --> CPU GPR
                	case ( MIPS32_COP0_MFC0 ) : {
                        	fprintf( fCPULog, "MFC0\t%d, %d, %d\n", _Rt_, _Rd_, _Sel_ );
                        	break;
                        }

                        //	CPU GPR --> CP0 CPR
                        case ( MIPS32_COP0_MTC0 ) : {
                        	fprintf( fCPULog, "MTC0\t%d, %d, %d\n", _Rt_, _Rd_, _Sel_ );
                        	break;
                        }

                }

        }

}
//---------------------------------------------------------------------------
//	Decodes the instruction specific to the COP1 processor or FPU
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeCOP1( void )
{
	//	Is processor present and available ?

        // ....	Presence check
        if ( ( !CP0.CtrlRegs.Config.M ) || ( !CP0.CtrlRegs.Config1.FP ) ) {
        	SignalException( CAUSE_CPU, (void*)1 );
                return;
        }

        // ...	Processor available
        if ( !CP0.CtrlRegs.Status.CU & MIPS32_STATUS_COP1 )	{
        	SignalException( CAUSE_CPU, (void*)1 );
                return;
        }

        //	Else decode instruction
        switch ( _OpCode_ ) {

        	//	64-bit transfer from memory to the fpu, this is done as 2 (two)
                // 32-bit access
        	case ( MIPS32_OPCODE_LDC1 ) : {
                	GenRegs[ _Rt_ ] = PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ];
                        GenRegs[ _Rt_ + 1 ] = PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ + 4 ];
                        fprintf( fCPULog, "LDC1\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
			break;
                }

                //	as above but only 32-bit
                case ( MIPS32_OPCODE_LWC1 ) : {
                	GenRegs[ _Rt_ ] = PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ];
                        fprintf( fCPULog, "LWC1\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
			break;
                }

                //	64-bit transfer from fpu to the memory, this is done as 2 (two)
                // 32-bit access
                case ( MIPS32_OPCODE_SDC1 ) : {
                	PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ] = GenRegs[ _Rt_ ];
                        PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ + 4 ] = GenRegs[ _Rt_ + 1 ];
                        fprintf( fCPULog, "SDC1\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
			break;
                }

                //	as above but only 32-bit
                case ( MIPS32_OPCODE_SWC1 ) : {
                	PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ] = GenRegs[ _Rt_ ];
                        fprintf( fCPULog, "SWC1\t%d, %d, 0x%04x\n", _Rt_, _Rs_, _Im_ );
			break;
                }

                //	COP1 other function
                case ( MIPS32_OPCODE_COP1 ) : {
	                switch ( _Rs_ ) {
				case ( MIPS32_COP1_MF ) : {
                                	CP0.dwGenRegs[ _Rt_ ] = FPR[ _Rd_ ];
                                        fprintf( fCPULog, "MFC1\t%d, %d\n", _Rt_, _Rd_ );
					break;
                                }
				case ( MIPS32_COP1_CF ) : {
                                	CP0.dwGenRegs[ _Rt_ ] = FCR[ _Rd_ ];
                                        fprintf( fCPULog, "CFC1\t%d, %d\n", _Rt_, _Rd_ );
					break;
                                }
				case ( MIPS32_COP1_MT ) : {
                                	FPR[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ];
                                        fprintf( fCPULog, "MTC1\t%d, %d\n", _Rt_, _Rd_ );
					break;
                                }
				case ( MIPS32_COP1_CT ) : {
                                	FCR[ _Rd_ ] = CP0.dwGenRegs[ _Rt_ ];
                                        fprintf( fCPULog, "CTC1\t%d, %d\n", _Rt_, _Rd_ );
					break;
                                }
				case ( MIPS32_COP1_BC ) : {
                                	fprintf( fCPULog, "BC_CC\n" );
					break;
                                }
                                //	The following three instructions are typed to various
                                // other instructions
                                case ( MIPS32_COP1_CVT_S ) :
                                case ( MIPS32_COP1_CVT_D ) :
                                case ( MIPS32_COP1_CVT_W ) : {

                                	switch ( _Funct_ ) {

                                        	//	FPR = FPR + FPR
						case ( MIPS32_COP1_ADD ) : {
                                                	fprintf( fCPULog, "ADD" );
                                                	switch ( _Rs_ ) {
                                                        	case ( MIPS32_COP1_FMT_S ) : {
                                                                	CP1.GenRegs[ _Sa_ ].s = CP1.GenRegs[ _Rd_ ].s + CP1.GenRegs[ _Rt_ ].s;
                                                                        fprintf( fCPULog, ".S\t" );
                                                                	break;
                                                                }
                                                                case ( MIPS32_COP1_FMT_D ) : {
                                                                	CP1.GenRegs[ _Sa_ ].d = CP1.GenRegs[ _Rd_ ].d + CP1.GenRegs[ _Rt_ ].d;
                                                                        fprintf( fCPULog, ".D\t" );
                                                                	break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d, %d\n", _Sa_, _Rd_, _Rt_ );
                                                	break;
                                                }

                                                //	FPR = FPR - FPR
						case ( MIPS32_COP1_SUB ) : {
                                                	fprintf( fCPULog, "SUB" );
                                                	switch ( _Rs_ ) {
                                                        	case ( MIPS32_COP1_FMT_S ) : {
                                                                	CP1.GenRegs[ _Sa_ ].s = CP1.GenRegs[ _Rd_ ].s - CP1.GenRegs[ _Rt_ ].s;
                                                                        fprintf( fCPULog, ".S\t" );
                                                                	break;
                                                                }
                                                                case ( MIPS32_COP1_FMT_D ) : {
                                                                	CP1.GenRegs[ _Sa_ ].d = CP1.GenRegs[ _Rd_ ].d - CP1.GenRegs[ _Rt_ ].d;
                                                                        fprintf( fCPULog, ".D\t" );
                                                                	break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d, %d\n", _Sa_, _Rd_, _Rt_ );
                                                	break;
                                                }

                                                //	FPR = FPR * FPR
						case ( MIPS32_COP1_MUL ) : {
                                                	fprintf( fCPULog, "MUL" );
                                                	switch ( _Rs_ ) {
                                                        	case ( MIPS32_COP1_FMT_S ) : {
                                                                	CP1.GenRegs[ _Sa_ ].s = CP1.GenRegs[ _Rd_ ].s * CP1.GenRegs[ _Rt_ ].s;
                                                                        fprintf( fCPULog, ".S\t" );
                                                                	break;
                                                                }
                                                                case ( MIPS32_COP1_FMT_D ) : {
                                                                	CP1.GenRegs[ _Sa_ ].d = CP1.GenRegs[ _Rd_ ].d * CP1.GenRegs[ _Rt_ ].d;
                                                                        fprintf( fCPULog, ".D\t" );
                                                                	break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d, %d\n", _Sa_, _Rd_, _Rt_ );
                                                	break;
                                                }

                                                //	FPR = FPR / FPR
						case ( MIPS32_COP1_DIV ) : {
                                                	fprintf( fCPULog, "DIV" );
                                                	switch ( _Rs_ ) {
                                                        	case ( MIPS32_COP1_FMT_S ) : {
                                                                	CP1.GenRegs[ _Sa_ ].s = CP1.GenRegs[ _Rd_ ].s / CP1.GenRegs[ _Rt_ ].s;
                                                                        fprintf( fCPULog, ".S\t" );
                                                                	break;
                                                                }
                                                                case ( MIPS32_COP1_FMT_D ) : {
                                                                	CP1.GenRegs[ _Sa_ ].d = CP1.GenRegs[ _Rd_ ].d / CP1.GenRegs[ _Rt_ ].d;
                                                                	fprintf( fCPULog, ".D\t" );
                                                                        break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d, %d\n", _Sa_, _Rd_, _Rt_ );
                                                	break;
                                                }

                                                //	FPR = sqrt( FPR )
						case ( MIPS32_COP1_SQRT ) : {
                                                	fprintf( fCPULog, "SQRT" );
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = (float)sqrt( CP1.GenRegs[ _Fs_ ].s );
                                                                        fprintf( fCPULog, ".S\t" );
                                                                	break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = sqrt( CP1.GenRegs[ _Fs_ ].d );
                                                                        fprintf( fCPULog, ".D\t" );
                                                                	break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d\n", _Fd_, _Fs_ );
                                                	break;
                                                }

                                                //	FPR = abs( FPR )
						case ( MIPS32_COP1_ABS ) : {
                                                	fprintf( fCPULog, "ABS" );
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = (float)fabs( CP1.GenRegs[ _Fs_ ].d );
                                                                        fprintf( fCPULog, ".S\t" );
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = fabs( CP1.GenRegs[ _Fs_ ].d );
                                                                        fprintf( fCPULog, ".D\t" );
                                                                        break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d\n", _Fd_, _Fs_ );
                                                	break;
                                                }

                                                //	FPR = FPR
						case ( MIPS32_COP1_MOV ) : {
                                                	fprintf( fCPULog, "MOV" );
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = CP1.GenRegs[ _Fs_ ].s;
                                                                        fprintf( fCPULog, ".S\t" );
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = CP1.GenRegs[ _Fs_ ].d;
                                                                        fprintf( fCPULog, ".D\t" );
                                                                        break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d\n", _Fd_, _Fs_ );
                                                	break;
                                                }

                                                //	FPR = ~FPR
						case ( MIPS32_COP1_NEG ) : {
                                                	fprintf( fCPULog, "NEG" );
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].dw = ~CP1.GenRegs[ _Fs_ ].dw;
                                                                        fprintf( fCPULog, ".S\t" );
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].dw = ~CP1.GenRegs[ _Fs_ ].dw;
                                                                        fprintf( fCPULog, ".D\t" );
                                                                        break;
                                                                }
                                                        }
                                                        fprintf( fCPULog, "%d, %d\n", _Fd_, _Fs_ );
                                                	break;
                                                }

                                                
						case ( MIPS32_COP1_ROUND ) : {
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = CP1.GenRegs[ _Fs_ ].s;
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = CP1.GenRegs[ _Fs_ ].d;
                                                                        break;
                                                                }
                                                        }
                                                	break;
                                                }

						case ( MIPS32_COP1_TRUNC ) : {
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = CP1.GenRegs[ _Fs_ ].s;
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = CP1.GenRegs[ _Fd_ ].d;
                                                                        break;
                                                                }
                                                        }
                                                	break;
                                                }

						case ( MIPS32_COP1_CEIL ) : {
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = ceil( CP1.GenRegs[ _Fs_ ].s );
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = ceil( CP1.GenRegs[ _Fs_ ].d );
                                                                        break;
                                                                }
                                                        }
                                                	break;
                                                }

						case ( MIPS32_COP1_FLOOR ) : {
                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	CP1.GenRegs[ _Fd_ ].s = floor( CP1.GenRegs[ _Fs_ ].s );
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	CP1.GenRegs[ _Fd_ ].d = floor( CP1.GenRegs[ _Fs_ ].d );
                                                                        break;
                                                                }
                                                        }
                                                	break;
                                                }

                                                //	Move Conditional
						case ( MIPS32_COP1_MOVC ) : {

                                                	bool	fCondition = ( _Cc_ & ( ( CP1.FCSR.FCC1 << 1 ) | ( CP1.FCSR.FCC0 ) ) );

                                                	switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	if ( ( _Tf_ ) & ( fCondition ) ) {
                                                                        	CP1.GenRegs[ _Fd_ ].s = CP1.GenRegs[ _Fs_ ].s;
                                                                        }
                                                                        if ( ( !_Tf_ ) & ( !fCondition ) ) {
                                                                        	CP1.GenRegs[ _Fd_ ].s = CP1.GenRegs[ _Fs_ ].s;
                                                                        }
                                                                	break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	if ( ( _Tf_ ) & ( fCondition ) ) {
                                                                        	CP1.GenRegs[ _Fd_ ].d = CP1.GenRegs[ _Fs_ ].d;
                                                                        }
                                                                        if ( ( !_Tf_ ) & ( !fCondition ) ) {
                                                                        	CP1.GenRegs[ _Fd_ ].d = CP1.GenRegs[ _Fs_ ].d;
                                                                        }
                                                                	break;
                                                                }
                                                        }
                                                	break;
                                                }

                                                //	Move on Zero
						case ( MIPS32_COP1_MOVZ ) : {
                                                	bool	fCondition = ( CP0.dwGenRegs[ _Rt_ ] == 0 );
                                                        switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	if ( fCondition )
                                                                        	CP1.GenRegs[ _Rd_ ] = CP1.GenRegs[ _Rs_ ];
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	if ( fCondition )
                                                                        	CP1.GenRegs[ _Rd_ ] = CP1.GenRegs[ _Rs_ ];
                                                                        break;
                                                                }
                                                        }
                                                	break;
                                                }

                                                //	Move on Not Zero
						case ( MIPS32_COP1_MOVN ) : {
                                                	bool	fCondition = ( CP0.dwGenRegs[ _Rt_ ] != 0 );
                                                        switch ( _Fmt_ ) {
                                                        	case ( MIPS32_COP1_CVT_S ) : {
                                                                	if ( fCondition )
                                                                        	CP1.GenRegs[ _Rd_ ] = CP1.GenRegs[ _Rs_ ];
                                                                        break;
                                                                }
                                                                case ( MIPS32_COP1_CVT_D ) : {
                                                                	if ( fCondition )
                                                                        	CP1.GenRegs[ _Rd_ ] = CP1.GenRegs[ _Rs_ ];
                                                                        break;
                                                                }
                                                        }
                                                	break;
                                                }

						case ( MIPS32_COP1_CVT_S ) : {
                                                	CP1.GenRegs[ _Fd_ ].s = CP1.GenRegs[ _Fs_ ].d;
                                                	break;
                                                }

						case ( MIPS32_COP1_CVT_D ) : {
                                                	CP1.GenRegs[ _Fd_ ].d = CP1.GenRegs[ _Fs_ ].d;
                                                	break;
                                                }

						case ( MIPS32_COP1_CVT_W ) : {
                                                	CP1.GenRegs[ _Fd_ ].dw = CP1.GenRegs[ _Fs_ ].d;
                                                	break;
                                                }

                                        }

                                	break;
                                }

                                //case ( CC_COND ) : {	//	There are lots of instructions here also
                                //	break;
                                //}

        	        }

                }

        }

}
//---------------------------------------------------------------------------
//	decode and execute CP2 specific instruction
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeCOP2( void )
{
	switch ( _OpCode_ ) {	//	What do we want to do ?
                case ( MIPS32_OPCODE_LWC2 ) : {	//	Load word from coprocessor 2
                	lpGTE->GenRegs[ _Rt_ ] = PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ];
                        break;
                }
                case ( MIPS32_OPCODE_LDC2 ) : {	//	Load double word from coprocessor 2
                	lpGTE->GenRegs[ _Rt_ ] = PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ];
                        lpGTE->GenRegs[ _Rt_ + 1 ] = PsxMem->RAM32[ ( CP0.dwGenRegs[ _Rs_ ] + _Im_ ) + 4 ];
                        break;
                }
                case ( MIPS32_OPCODE_SWC2 ) : { //	Store word to coprocessor 2
                	PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ] = lpGTE->GenRegs[ _Rt_ ];
                        break;
                }
                case ( MIPS32_OPCODE_SDC2 ) : {	//	store double word to coprocessor 2
                	PsxMem->RAM32[ CP0.dwGenRegs[ _Rs_ ] + _Im_ ] = lpGTE->GenRegs[ _Rt_ ];
                        PsxMem->RAM32[ ( CP0.dwGenRegs[ _Rs_ ] + _Im_ ) + 4 ] = lpGTE->GenRegs[ _Rt_ + 1 ];
                        break;
                }
                case ( MIPS32_OPCODE_COP2 ) : {	//	COP2 Specific instruction, can either be a function
                // call or instruction like MTC2 or MFC2
                	switch ( _Rs_ ) {	//	Which instruction is it?
                        	//	MOVE FROM
                        	case ( MIPS32_COP2_MF ) : {	//	CP2 GPR --> CPU GPR
                                        CP0.dwGenRegs[ _Rt_ ] = lpGTE->GenRegs[ _Rd_ ];
                                	break;
                                }
                                case ( MIPS32_COP2_MT ) : {	//	CPU GPR --> CP2 GPR
                                	lpGTE->GenRegs[ _Rt_ ] = CP0.dwGenRegs[ _Rd_ ];
                                	break;
                                }
                                case ( MIPS32_COP2_CF ) : {	//	CP2 CPR --> CPU GPR
                                	CP0.dwGenRegs[ _Rt_ ] = lpGTE->CtlRegs[ _Rd_ ];
                                	break;
                                }
                                case ( MIPS32_COP2_CT ) : {	//	CPU GPR --> CP2 CPR
                                	lpGTE->CtlRegs[ _Rt_ ] = CP0.dwGenRegs[ _Rd_ ];
                                	break;
                                }
                                case ( MIPS32_COP2_BC ) : {	//	BC2 [T/F] [L]
                                //	Branch on True/False
                                	bool	fCondition = _Cc_ & lpGTE->CtlRegs[ 31 ];
                                        if ( ( _Tf_ ) & ( fCondition ) ) {	//	Branch TRUE
                                        	CP0.dwGenRegs[ 31 ] = CP0.PC;
                                                fUpdatePC = false;
                                                CP0.PC = _BranchTarget_;
                                                return;
                                        }
                                        if ( ( !_Tf_ ) & ( !fCondition ) ) {	//	Branch FALSE
                                        	CP0.dwGenRegs[ 31 ] = CP0.PC;
                                                fUpdatePC = false;
                                                CP0.PC = _BranchTarget_;
                                                return;
                                        }
                                        //	else ignored
                                	break;
                                }
                                default : {	//	Processor as a COP2 <function> instruction
                                	CoProcessorOperation( 2, _Funct_ );
                                	break;
                                }
                        }
                }
        }
}
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DecodeCOP3( void )
{
	if ( !COPPresent( 3 ) ) {
		CP0.CtrlRegs.Cause.CE = 3;
		SignalException( CAUSE_CPU, (void*)3 );
	}
}
//---------------------------------------------------------------------------
//#ifdef	CPUMODE_EJTAGDEBUG_IMPLEMENTED
//---------------------------------------------------------------------------
void	__fastcall	R3000A :: DumpRegs( void )
{
	fprintf( fCPULog, "\nEJTAG DUMP REGS\n" );
        fprintf( fCPULog, "General Registers\n\n" );

        for ( int i=0; i<NUM_MIPS_REGS; i++ )
                fprintf( fCPULog, "r%i\t:\t0x%8x\n", i, CP0.dwGenRegs[ i ] );

        //	Control registers
        fprintf( fCPULog, "\nControl Registers\n\n" );
        fprintf( fCPULog, "Index\t:\t0x%08x\n", CP0.CtrlRegs.dwIndex );
        fprintf( fCPULog, "Random\t:\t0x%08x\n",       CP0.CtrlRegs.dwRandom );
        fprintf( fCPULog, "EntryLo0\t:\t0x%08x\n",     CP0.CtrlRegs.dwEntryLo0 );
        fprintf( fCPULog, "EntryLo1\t:\t0x%08x\n",     CP0.CtrlRegs.dwEntryLo1 );
        fprintf( fCPULog, "Context\t:\t0x%08x\n",      CP0.CtrlRegs.dwContext );
        fprintf( fCPULog, "PageMask\t:\t0x%08x\n",     CP0.CtrlRegs.dwPageMask );
        fprintf( fCPULog, "Wired\t:\t0x%08x\n",        CP0.CtrlRegs.dwWired );
        fprintf( fCPULog, "BadVAddr\t:\t0x%08x\n",     CP0.CtrlRegs.dwBadVAddr );
        fprintf( fCPULog, "Count\t:\t0x%08x\n",        CP0.CtrlRegs.dwCount );
        fprintf( fCPULog, "EntryHi\t:\t0x%08x\n",      CP0.CtrlRegs.dwEntryHi );
        fprintf( fCPULog, "Compare\t:\t0x%08x\n",      CP0.CtrlRegs.dwCompare );
        fprintf( fCPULog, "Status\t:\t0x%08x\n",       CP0.CtrlRegs.dwStatus );

        fprintf( fCPULog, "Cause\t:\t0x%08x\n",        CP0.CtrlRegs.dwCause );

        fprintf( fCPULog, "Decoding CP0 Control Register 'CAUSE'\n'ExccCode'\n" );

        switch ( CP0.CtrlRegs.Cause.ExcCode ) {
        	case (  0 ) : {
                	fprintf( fCPULog, "\tInterrupt" );
                        break;
                }
                case (  1 ) : {
                	fprintf( fCPULog, "\tTLB Modification (WRITE)" );
                        break;
                }
                case (  2 ) : {
                	fprintf( fCPULog, "\tTLB Load" );
                        break;
                }
                case (  3 ) : {
                	fprintf( fCPULog, "\tTLB Store" );
                        break;
                }
                case (  4 ) : {
                	fprintf( fCPULog, "\tAddress Error Load or Instruction Fetch" );
                        break;
                }
                case (  5 ) : {
                	fprintf( fCPULog, "\tAddress Error Store" );
                        break;
                }
                case (  6 ) : {
                	fprintf( fCPULog, "\tInstruction Bus Error Load or Instruction Fetch" );
                        break;
                }
                case (  7 ) : {
                	fprintf( fCPULog, "\tData Bus Error Data Load OR Store" );
                        break;
                }
                case (  8 ) : {
                	fprintf( fCPULog, "\tSYSCALL" );
                        break;
                }
                case (  9 ) : {
                	fprintf( fCPULog, "\tBREAKPOINT" );
                        break;
                }
                case ( 10 ) : {
                	fprintf( fCPULog, "\tReserved Instruction" );
                        break;
                }
                case ( 11 ) : {
                	fprintf( fCPULog, "\tCoprocessor Unusable" );
                        break;
                }
                case ( 12 ) : {
                	fprintf( fCPULog, "\tArithmetic Overflow (INTEGER)" );
                        break;
                }
                case ( 13 ) : {
                	fprintf( fCPULog, "\tTrap" );
                        break;
                }
                case ( 15 ) : {
                	fprintf( fCPULog, "\tFloaing Point Exception" );
                        //	FCSR has the FPU Exception
                        break;
                }
                case ( 18 ) : {
                	fprintf( fCPULog, "\tPrecise CP2" );
                        break;
                }
                case ( 22 ) : {
                	fprintf( fCPULog, "\tMDMX" );	//	MIPS 64 ONLY!
                        break;
                }
                case ( 23 ) : {
                	fprintf( fCPULog, "\tWATCH" );
                        break;
                }
                case ( 24 ) : {
                	fprintf( fCPULog, "\tMACHINE CHECK" );
                        break;
                }
                case ( 30 ) : {
                	fprintf( fCPULog, "\tCACHE" );
                        break;
                }
                default : {
                	fprintf( fCPULog, "\tUnknown 0x%04x", CP0.CtrlRegs.Cause.ExcCode );
                        break;
                }
        }
        fprintf( fCPULog, "\n" );

        fprintf( fCPULog, "EPC\t:\t0x%08x\n",          CP0.CtrlRegs.dwEPC );
        fprintf( fCPULog, "PRiD\t:\t0x%08x\n",         CP0.CtrlRegs.dwPRiD );
        fprintf( fCPULog, "Config\t:\t0x%08x\n",       CP0.CtrlRegs.dwConfig );
        fprintf( fCPULog, "Config1\t:\t0x%08x\n",      CP0.CtrlRegs.dwConfig1 );
        fprintf( fCPULog, "Config2\t:\t0x%08x\n",      CP0.CtrlRegs.dwConfig2 );
        fprintf( fCPULog, "Config3\t:\t0x%08x\n",      CP0.CtrlRegs.dwConfig3 );
        fprintf( fCPULog, "LLAddr\t:\t0x%08x\n",       CP0.CtrlRegs.dwLLAddr );
        fprintf( fCPULog, "WatchLo\t:\t0x%08x\n",      CP0.CtrlRegs.dwWatchLo );
        fprintf( fCPULog, "WatchHi\t:\t0x%08x\n",      CP0.CtrlRegs.dwWatchHi );
        fprintf( fCPULog, "XContext\t:\t0x%08x\n",     CP0.CtrlRegs.XContext );
        fprintf( fCPULog, "r21\t:\t0x%08x\n",          CP0.CtrlRegs.r21 );
        fprintf( fCPULog, "r22\t:\t0x%08x\n",          CP0.CtrlRegs.r22 );
        fprintf( fCPULog, "Debug\t:\t0x%08x\n",        CP0.CtrlRegs.dwDebug );
        fprintf( fCPULog, "DEPC\t:\t0x%08x\n",         CP0.CtrlRegs.DEPC );
        fprintf( fCPULog, "PerfCnt\t:\t0x%08x\n",      CP0.CtrlRegs.PerfCnt );
        fprintf( fCPULog, "ErrCtl\t:\t0x%08x\n",       CP0.CtrlRegs.ErrCtl );
        fprintf( fCPULog, "CacheErr\t:\t0x%08x\n",     CP0.CtrlRegs.CacheErr );
        fprintf( fCPULog, "TagLo\t:\t0x%08x\n",        CP0.CtrlRegs.TagLo );
        fprintf( fCPULog, "TagHi\t:\t0x%08x\n",        CP0.CtrlRegs.TagHi );
        fprintf( fCPULog, "ErrorEPC\t:\t0x%08x\n",     CP0.CtrlRegs.ErrorEPC );
        fprintf( fCPULog, "DESAVE\t:\t0x%08x\n",       CP0.CtrlRegs.DESAVE );
        fprintf( fCPULog, "HI/LO\t:0x%08x\t0x%08x\n",  CP0.Hi, CP0.Lo );
        fprintf( fCPULog, "PC\t:\t0x%08x\n", 	     CP0.PC );
}
//---------------------------------------------------------------------------
//#endif
//---------------------------------------------------------------------------
//	Determine whether a specified coprocesor or coprocessors are present
// or not. Uses a bit mask to determine the processor(s) requeting a presence
// check
//---------------------------------------------------------------------------
bool	__fastcall	COPPresent( int z )
{
	return( CPU->CP0.CtrlRegs.Status.CU & z );
}
//---------------------------------------------------------------------------
void 	__declspec( dllexport ) __fastcall	CPUTerminate( void )
{
	//!  Initiate a cascade shutdown
	PsxBIOS->Terminate();

        lpGTE->Terminate();

        CPU->Terminate();

        lpHW->Terminate();

	PsxMem->Terminate();
}
//---------------------------------------------------------------------------
//__declspec( dllexport ) void	__fastcall	R3000A :: CPUTakeException( DWORD Code, DWORD bd )
//{
	// Set the Cause

	//cpuRegs.CP0.n.Cause &= 0xffffff83;
	//cpuRegs.CP0.n.Cause |= code;

	// Set the EPC & PC
	//if (bd) {
	//	cpuRegs.CP0.n.Cause |= 0x80000000;
	//	cpuRegs.CP0.n.EPC = (cpuRegs.pc - 4);
	//}
	//else
	//	cpuRegs.CP0.n.EPC = (cpuRegs.pc);

	//if (cpuRegs.CP0.n.Status & 0x400000)
	//	cpuRegs.pc = 0xbfc00180;
	//else
	//	cpuRegs.pc = 0x80000080;

	// Set the Status
	//cpuRegs.CP0.n.Status = (cpuRegs.CP0.n.Status & 0xffffffc0) |
	//					  ((cpuRegs.CP0.n.Status & 0xf) << 2);

	//if (Config.HLE) BIOSException();
//}
//---------------------------------------------------------------------------
void 	__declspec( dllexport ) __fastcall	CPUInterrupt(unsigned long code)
{
	//if (cpuRegs.CP0.n.Status & (1 << (code + 8)))
	//	cpuRegs.CP0.n.Cause |= (1 << (code + 8));
	//CPUTakeException(0, 0);
}
//---------------------------------------------------------------------------
void doCDREADING(void);
//---------------------------------------------------------------------------
extern unsigned short StatReg;
//---------------------------------------------------------------------------
void 	__declspec( dllexport ) __fastcall	CPUBranchTest( void )
{
	//if(cdr.Reading && cpuIrq.cycle >= cdr.Synk)
	//	doCDREADING();

	//if(cpuIrq.cycle >= nextCounter)
	//	rcntUpdate();

	//if (cpuIrq.interrupt) {
	//	if ((cpuIrq.interrupt & 0x80) && (!(Config.Sio & 0x1))) { // 	Serial I/O Interrupt
	//		if (cpuIrq.cycle >= cpuIrq.intCycle[7]) {
	//			lpHW->dwHWM[ 0x1070 ] |= 0x80;
	//			cpuIrq.interrupt &= ~0x80;
	//			//StatReg |= IRQ;
	//		}
	//	}
	//	if (cpuIrq.interrupt & 0x04) { 				//	CD-ROM interrupt
	//		if (cpuIrq.cycle >= cpuIrq.intCycle[2]) {
	//			cpuIrq.interrupt &= ~0x04;
	//			//cdrInterrupt();
	//		}
	//	}
	//}

	//if (uHWM32(0x1070) & uHWM32(0x1074)) {				//	Direct Hardware Read/Write ?
	//	if ((cpuRegs.CP0.n.Status & 0x401) == 0x401) {
	//		CPUTakeException(0x400, 0);
	//	}
	//}

	//if (Config.PsxOut || Config.HLE) {				//	Emulator outputting messages to windows console ?
	//	switch (cpuRegs.pc & 0x1fffff) {
	//		case 0x80:
	//			if (Config.HLE) BIOSException();
	//			break;
	//	    case 0xa0:
	//			if (biosA0[cpuRegs.GPR.n.t1 & 0xff])
	//		   		biosA0[cpuRegs.GPR.n.t1 & 0xff]();
	//		   	break;
	//	    case 0xb0:
	//			if (biosB0[cpuRegs.GPR.n.t1 & 0xff])
	//		   		biosB0[cpuRegs.GPR.n.t1 & 0xff]();
	//		   	break;
	//	    	case 0xc0:
	//			if (biosC0[cpuRegs.GPR.n.t1 & 0xff])
	//	    		biosC0[cpuRegs.GPR.n.t1 & 0xff]();
	//	    	break;
	//	}
	//}
}
//---------------------------------------------------------------------------
LRESULT	CALLBACK	CPUWndProc( HWND wHandle, UINT Msg, WPARAM wParam, LPARAM lParam )
{

	switch ( Msg ) {

        	case ( MSG_CREATE ) : {	//	Creation stage
                	if ( CPU == NULL )
                        	CPU = new R3000A( true );
                        return( 0 );
                }

                case ( MSG_INITIALISE ) : {	//	Initialisation Stage
                	if ( CPU != NULL ) {
                        	return( CPU->Initialise() );
                        } else
                        	return( 0 );
                }

                case ( MSG_EXECUTE ) : {
                	if ( CPU != NULL ) {
                        	if ( CPU->Suspended )
                                	CPU->Resume();
                                else
                                	CPU->Execute();
                        }
                        return( 0 );
                }

                case ( MSG_TERMINATE ) :
                case ( WM_QUIT ) : {
                	PostQuitMessage( 0 );
                        return( 0 );
                }

        	default : {
                	return( DefWindowProc( wHandle, Msg, wParam, lParam ) );
                }
        }
}
//---------------------------------------------------------------------------
