//---------------------------------------------------------------------------
//
//	$FileName	::	R3000A.h
//	$Purpose	::	Header for MIPS 32 ISA R3000 code emulator
//	$Extras		::	Note is complient with MIPS32 ISA, including
//				Virtual Memory methods.
//
//---------------------------------------------------------------------------
#ifndef R3000AH
#define R3000AH
//---------------------------------------------------------------------------
//	Only include the rest of the emulator IF we are NOT currently being
// included as part of the emulator.
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
//	Architecture Types ( as implemented by the processor ). These values are
// stored in the CP0 Register 16 Select 0 AT Bits
//---------------------------------------------------------------------------
#define	ISA_MIPS32	0		//	MIPS 32
#define ISA_MIPS64MIN	1		//	MIPS 64 with 32-bit accessing
#define ISA_MIPS64	2		//	MIPS 64
#define ISA_RESERVED	3		//	Reserved, no information available
//---------------------------------------------------------------------------
#define ISA_DEFAULT	ISA_MIPS32	//	Default Architecture Type
//---------------------------------------------------------------------------
//	CPU Endian Mode
//---------------------------------------------------------------------------
#define ENDIAN_LH	0		//	Little Endian (ie like the ix86 cores)
#define ENDIAN_HL	1		//	Big endian (like most risc cpu cores)
#define ENDIAN_DEFAULT	ENDIAN_LH	//	Little endian, force emulation to match ix86 core, (saves a lot of byte swapping)
//---------------------------------------------------------------------------
#define GPR_LEN		32			//!	Number of bits in a general register
#define GCR_LEN		GPR_LEN			//!	Number of bits in a control register
//---------------------------------------------------------------------------
#define	NUM_MIPS_REGS	32			//!	Number of general register in CPU
//---------------------------------------------------------------------------
//!	MIPS 32 ISA COPROCESSOR MODELS	SEE MIPS32 PRA FOR FURTHER INFORMATION
//---------------------------------------------------------------------------
#define	MAX_NUM_COPROS		4		//!	Maximum number of CPU's installed
#define MIN_NUM_COPROS		1		//!	FPU ( COP1 ) OPTIONAL
#define DEFAULT_NUM_COPROS	2		//!	COP0 / FPU Present
#define PSX_NUM_COPROS		4		//!	COP0 / FPU / GTE / SPU Present, GTE / SPU are specific to the PlayStation implementation of MIPS 32
//---------------------------------------------------------------------------
#define NUM_COPRO_REGS		NUM_MIPS_REGS	//	Number of registers per copro, note may have general and/or control sets available
//---------------------------------------------------------------------------
//	CPU General Registers
//---------------------------------------------------------------------------
//	these are used in conjunction with char *RegNames[], or CPU->GenRegs[ <Index> ]
//---------------------------------------------------------------------------
#define rZERO	0
#define rAT      1
#define rV0      2
#define rV1      3
#define rA0      4
#define rA1      5
#define rA2      6
#define rA3      7
#define rT0      8
#define rT1      9
#define rT2      10
#define rT3      11
#define rT4      12
#define rT5      13
#define rT6      14
#define rT7      15
#define rS0      16
#define rS1      17
#define rS2      18
#define rS3      19
#define rS4      20
#define rS5      21
#define rS6      22
#define rS7      23
#define rT8      24
#define rT9      25
#define rK0      26
#define rK1      27
#define rGP      28
#define rSP      29
#define rGP      30
#define rRA      31
//---------------------------------------------------------------------------
//!	COP0 Control Registers	note any that are not defined with a struct are either
//!	reserved for future use or are in use or on an implementation specific basis.
//---------------------------------------------------------------------------
//!	COP0 Register 0 Select 0 ( INDEX current tlb entry )
//---------------------------------------------------------------------------
typedef struct _cop0_index_reg_ {
	unsigned	Index : 31;		//!	Entry number
        unsigned	ProbeFail : 1;		//!	Probe Failure, TLB match found flag ( found == 0 / !found == 1 )
        } CP0Index, *PCP0Index;
//---------------------------------------------------------------------------
typedef struct	_cop0_random_reg_ {		//!	COP0 Register 1 Select 0 ( RANDOM )
	unsigned	Random : 32;		//!	Entry Number
        } CP0Random, *PCP0Random;
//---------------------------------------------------------------------------
//!	COP 0 Register 'EntryLo0' and 'EntryLo1'
//---------------------------------------------------------------------------
typedef struct _cop0_entrylo_reg_ {		//!	<strong>COP0 Registers 2 and 3, Select 0 ( EntryLo0 and EntryLo1 )</strong>
	unsigned	Global	: 1;		//!	<strong>Global Bit</strong>, if set then is handled differently than if not set
        unsigned	Valid	: 1;		//!	<strong>Valid Bit</strong>, Valid TLB Entry for this page
        unsigned	Dirty	: 1;		//!	<strong>Dirty Bit</strong>, Does this page need writing back to RAM
        unsigned	CCA	: 3;		//!	<strong>CCA Bits</strong>, Cache Coherency Algorithm
        unsigned	PFN	: 24;		//!	<strong>PFN</strong>, Page Frame Number
        unsigned	reserved:2;		//!	<strong>Reserved</strong> must be read as 0 (zero) ignored on write
        } CP0EntryLo0, CP0EntryLo1, *PCP0EntryLo0, *PCP0EntryLo1;
//---------------------------------------------------------------------------
#define CCA_WTHRU_NOALLOC	0		//!	Cache Write Through, No TLB Entry Allocation
#define CCA_WTHRU_ALLOC		1		//!	Cache Write Through, with TLB Entry Allocation
#define	CCA_UNCACHED		2		//!	Page is uncacheable
#define	CCA_CACHEABLE		3		//!	Page is cacheable
#define CCA_WBACK		3		//!	Cache Write Back
#define CCA_NONCOHERENT		3		//!	Non-coherent cache algorithm
#define CCA_COHERENT_XCL	4		//!	?????
#define CCA_COHERENT_XCLW	5		//!	?????
#define CCA_COHERENT_UPDATE	6		//!	Update TLB entries / RAM ?????
#define CCA_UNCACHE_ACCEL	7		//!	Uncache Accelerated ?????
//---------------------------------------------------------------------------
typedef struct	_cop0_context_reg_ {		//!	COP0 Register 4 Select 0 ( CONTEXT )
	unsigned	reserved: 4;		//!	<strong>Reserved</strong> must be read as 0 (zero) ignored on write
        unsigned	BadVPN2	: 19;		//!	<strong>Bad VPN</strong> Bad Virtal Page Number / 2
        unsigned	PTEBase	: 9;		//!	<strong>PTE Base</strong> Base for PTE Table
        } CP0Context, *PCP0Context;
//---------------------------------------------------------------------------
typedef	struct	_cop0_pagemask_reg_ {		//	COP0 Register 5 Select 0
	unsigned	reserved:13;
        unsigned	Mask : 16;
        unsigned	reserved1:3;
        } CP0PageMask, *PCP0PageMask;
//---------------------------------------------------------------------------
typedef	struct	_cop0_wired_reg_ {	//	COP0 Register 6 Select 0
	unsigned	Wired : 32;
        } CP0Wired, *PCP0Wired;
//---------------------------------------------------------------------------
//	CP0 Register 7 Select 0 Reserved in all implementations
//---------------------------------------------------------------------------
typedef	struct	_cop0_badvaddr_reg_ {	//	CP0 Register 8 Select 0
	unsigned	BadVAddr : 32;
        } CP0BadVAddr, *PCP0BadVAddr;
//---------------------------------------------------------------------------
typedef struct _cop0_count_reg_ {	//	CP0 Register 9 Select 0
	unsigned	Count : 32;
        } CP0Count, *PCP0Count;
//---------------------------------------------------------------------------
//	CP0 Register 9 Selects 6 and 7 optional implementation dependent
//---------------------------------------------------------------------------
typedef struct _cop0_entryhi_reg_ {	//	CP0 Register 10 Select 0
	unsigned	ASID	: 8;
        unsigned	reserved: 5;
        unsigned	VPN2	: 19;
        } CP0EntryHi, *PCP0EntryHi;
//---------------------------------------------------------------------------
typedef struct _cop0_compare_reg_ {	//	CP0 Register 11 Select 0
	unsigned	Compare : 32;
        } CP0Compare, *PCP0Compare;
//---------------------------------------------------------------------------
typedef struct	_cop0_status_reg_ {	//	CP0 Register 12 Select 0
	unsigned	IE	: 1;	//	Interrupts Enabled
        unsigned	EXL	: 1;	//	Exception Level
        unsigned	ERL	: 1;	//	Error Level
        union	{
         	unsigned	KSU : 2;	//	If supervisor mode is implemented then denotes the base operating mode of the processor
         	unsigned	R0  : 1;	//	reserved
         	unsigned	UM  : 1;	//	denotes kernel or user mode, see KSU field above
        };
        unsigned	UX	: 1;		//	enables access to 64-bit user mode address space on 64-bit MIPS processors not used in MIPS 32
        unsigned	SX	: 1;            //	Enables access to 64-bit supervisor mode address space
        unsigned	KX	: 1;            //	Enables access to 64-bit kernel mode address space
        unsigned	IM	: 8;            //	Interrupt mask
        unsigned	Impl	: 2;            //	Implementation specific
        unsigned	reserved: 1;            //	reserved
        unsigned	NMI 	: 1;            //	entry through reset vector due to NMI ?
        unsigned	SR 	: 1;            //	entry through reset vector due to soft reset ?
        unsigned	TS 	: 1;            //	indicates multiple tlb entry matches
        unsigned	BEV 	: 1;            //	base exception vector locations
        unsigned	PX 	: 1;            //	enables access to 64-bit operations on MIPS 46
        unsigned	MX 	: 1;            //	enables access to MDMX resources on MIPS 64
        unsigned	RE 	: 1;            //	enables reverse endian memory references
        unsigned	FR 	: 1;            //	controls the FPU register mode on MIPS 64
        unsigned	RP	: 1;            //	enables reduced power mode, implementation specific
        unsigned	CU 	: 4;            //	controls access to coprocessors 0, 1, 2, 3 respectively
        } CP0Status, *PCP0Status;
//---------------------------------------------------------------------------
#define MIPS32_STATUS_COP0	0x10000000	//	if ( CP0.CtrlRegs.Status.CU & MIPS32_STATUS_COP0 ) CP0 is available
#define MIPS32_STATUS_COP1	0x20000000	//      again for CP1
#define MIPS32_STATUS_COP2	0x40000000      //	and CP2
#define MIPS32_STATUS_COP3	0x80000000      //	CP3
//---------------------------------------------------------------------------
#define MIPS32_STATUS_COP_ALL	0xF0000000	//	Any CP<0-3>
//---------------------------------------------------------------------------
typedef	struct	_cop0_cause_reg_ {		//	CP0 Register 13 Select 0
	unsigned	reserved0:2;
        unsigned	ExcCode	: 5;		//	Exception Code
        unsigned	reserved1:1;
        unsigned	IP	: 8;		//	Indicates an external interrupt is pending and controls the request for software interrupts
        unsigned	reserved2:6;
        unsigned	WP	: 1;		//	Watch Exception was deferred
        unsigned	IV	: 1;		//	indicates whether interrupt vectors uses the genernal or a special interrupt vector
        unsigned	reserved3:4;
        unsigned	CE	: 2;		//	coprocessor unit number when a Coprocessor Unusable exception is taken
        unsigned	reserved4:1;
        unsigned	BD	: 1;		//	inidcates whether the last exception taken occured in a branch delay slot
        } CP0Cause, *PCP0Cause;
//---------------------------------------------------------------------------
//	Exception cause, CP0 Cause IP field. Any values not listed here are
// classed as reserved and are ignored , and disabled if possible.
//---------------------------------------------------------------------------
#define	CAUSE_INT		0		//	Interrupt
#define CAUSE_MOD		1		//	TLB Modified
#define CAUSE_TLBL		2		//	TLB Load or Instruction Fetch
#define CAUSE_TLBS		3		//	TLB Store
#define	CAUSE_ADEL		4		//	Address error ( load or instruction fetch )
#define CAUSE_ADES		5		//      Address error ( store )
#define CAUSE_IBE		6               //	Bus error ( instruction fetch )
#define CAUSE_DBE		7               //	Bus error ( data load or store )
#define CAUSE_SYS		8               //	Syscall exception
#define CAUSE_BP		9               //	Breakpoint exception
#define CAUSE_RI		10              //	Reserved instruction
#define CAUSE_CPU		11              //	coprocessor unusable
#define CAUSE_OV		12              //	overflow
#define CAUSE_TR		13              //	trap
#define CAUSE_FPE		15              //	floating point exception
#define CAUSE_C2E		18              //	precise coprocessor 2 exceptions
#define CAUSE_MDMX		22              //	reserved for MDMX
#define CAUSE_WATCH		23              //	reference to watch hi/lo address DEBUG
#define CAUSE_MCHECK		24              //	machine check, no info available for this exception
#define CAUSE_CACHEERR		30              //	cache eror, relates to EJTAG mode ie debug mode
//---------------------------------------------------------------------------
//	The following are not documented in the MIPS 32 ISA manuals, but are
// referred to for causes other than what MIPS 32 ISA understands. eg TLB Refill
//---------------------------------------------------------------------------
#define CAUSE_TLBR		31		//	TLB REFILL NOT PART OF THE MIPS 32 ISA
#define CAUSE_TLBM		32		//	TLB MISS NOT PART OF THE MIPS 32 ISA
#define CAUSE_SDB		33		//	Software deugger
//---------------------------------------------------------------------------
typedef	struct	_cop0_epc_reg_ {		//	CP0 Register 14 Select 0
	unsigned	EPC	: 32;
        } CP0EPC, *PCP0EPC;
//---------------------------------------------------------------------------
typedef	struct	_cop0_prid_reg_ {		//	CP0 Register 15 Sleect 0
	unsigned	Revision	: 8;
        unsigned	ProcessorID	: 8;
        unsigned	CompanyID	: 8;	//	== 0 NOT MIPS 32 or 64 CPU
        unsigned	CompanyOptions	: 8;
        } CP0PRID, *PCP0PRID;
//---------------------------------------------------------------------------
typedef struct	_cop0_config_reg_	{	//	CP0 Register 16 Select 0
	unsigned	KSEG0CCA	: 3;	//	KSEG 0 Cache Coherency Attribute
        unsigned	reserved	: 4;
        unsigned	MT		: 3;	//	MMU Type
        unsigned	AR		: 3;	//	Architecture revision
        unsigned	AT		: 2;	//	Architecture type
        unsigned	BE		: 1;	//	Base Endian Mode
        unsigned	Impl		: 15;	//	Implementation specific
        unsigned	M		: 1;	//	denotes that the config 1 register is present
        } CP0Config, *PCP0Config;
//---------------------------------------------------------------------------
typedef struct	_cop0_config1_reg_	{	//	CP0 Register 16 Select 1
	unsigned	FP	: 1;		//	FPU implemented
        unsigned	EP	: 1;            //	EJTAG implemented
        unsigned	CA      : 1;            //      Code compression
        unsigned	WR      : 1;            //      watch registers implemented
        unsigned	PC      : 1;            //	performance counters implemented
        unsigned	MD      : 1;            //	mdmx ase implemented
        unsigned	C2      : 1;            //	coprocessor 2 implemented
        unsigned	DA      : 3;            //	dcache associativity
        unsigned	DL      : 3;            //	dcache line size
        unsigned	DS      : 3;            //	dcache sets per way
        unsigned	IA      : 3;            //	icache associativity
        unsigned	IL      : 3;            //	icache line size
        unsigned	IS      : 3;            //	icache sets per way
        unsigned	MMUSIZE : 6;            //	number of entries in the tlb minus 1
        unsigned	M       : 1;            //	denotes config2 registers is implemented
        } CP0Config1, *PCP0Config1;
//---------------------------------------------------------------------------
typedef struct	_cop0_config2_reg_	{	//	CP0 Register 16 Select 2
	unsigned	TBS	: 31;
        unsigned	M	: 1;
        } CP0Config2, *PCP0Config2;
//---------------------------------------------------------------------------
typedef struct _cop0_config3_reg_	{	//	CP0 Register 16 Select 3
	unsigned	TL	: 1;
        unsigned	SM	: 1;
        unsigned	reserved:29;
        unsigned	M	: 1;
        } CP0Config3, *PCP0Config3;
//---------------------------------------------------------------------------
typedef struct _cop0_lla_reg_	{		//	CP0 Register 17 Select 0
	unsigned	PAddr	: 32;
        } CP0LLA, *PCP0LLA;
//---------------------------------------------------------------------------
typedef struct	_cop0_watchlo_reg_	{	//	CP0 Register 18
	unsigned	W	: 1;		//	watch exception enabled for stores to this address
        unsigned	R	: 1;		//	watch exception enabled for loads from this address
        unsigned	I	: 1;		//	watch exception enabled for instruction fetches from this address
        unsigned	VAddr	: 29;		//	virtual address
        } CP0WatchLo, *PCP0WatchLo;
//---------------------------------------------------------------------------
typedef struct _cop0_watchhi_reg_	{	//	CP0 Register 19
	unsigned	reserved:3;
        unsigned	Mask	: 9;
        unsigned	reserved1:4;
        unsigned	ASID	: 8;
        unsigned	reserved2:6;
        unsigned	G	: 1;
        unsigned	M	: 1;
        } CP0WatchHi, *PCP0WatchHi;
//---------------------------------------------------------------------------
//	NOTE r0	is always 0 on reads, as it is hardwired to 0 in the MIPS 32 ISA
//---------------------------------------------------------------------------
//	This struct allows you to reference all the registers as DWORD values,
// but also, in the case of the control set, can be referenced by individual
// bits within a particular cp0 register.
//---------------------------------------------------------------------------
typedef struct	_coprocessor_sys_control_	{	//	System Control Processor
	DWORD	dwGenRegs[ NUM_COPRO_REGS ];		//	General registers
        struct	{					//	Control Registers
        	union {
                	CP0Index	Index;                  //	TLB Index entry
                        DWORD		dwIndex;
                };
                union {
                	CP0Random	Random;                 //	TLB Random entry, ie next free slot
                        DWORD		dwRandom;
                };
                union	{
                	CP0EntryLo0	EntryLo0;               //	for use by TLB
                        DWORD		dwEntryLo0;
                };
                union	{
                	CP0EntryLo1	EntryLo1;               //	for use by TLB
                        DWORD		dwEntryLo1;
                };
                union	{
                	CP0Context	Context;                //	TLB base
                        DWORD		dwContext;
                };
                union	{
                	CP0PageMask	PageMask;               //	PageMask
                        DWORD		dwPageMask;
                };
                union	{
                	CP0Wired	Wired;                  //	number of fixed tlb entries
                        DWORD		dwWired;
                };
                DWORD		r7;			//	Reserved for future extensions
                union	{
                	CP0BadVAddr	BadVAddr;               //	bad virtual address
                        DWORD		dwBadVAddr;
                };
                union	{
                	CP0Count	Count;                  //	counter
                        DWORD		dwCount;
                };
                union	{
                	CP0EntryHi	EntryHi;                //	for TLB use
                        DWORD		dwEntryHi;
                };
                union	{
                	CP0Compare	Compare;                //	compare
                        DWORD		dwCompare;
                };
                union	{
                	CP0Status	Status;                 //	status register
                        DWORD		dwStatus;
                };
                union	{
                	CP0Cause	Cause;                  //	cause register
                        DWORD		dwCause;
                };
                union	{
                	CP0EPC		EPC;                    //	PC at last exception
                        DWORD		dwEPC;
                };
                union	{
                	CP0PRID		PRiD;                   //	processor id
                        DWORD		dwPRiD;
                };
                struct	{				//	Use the sel field from MFC0 and MTC0 instructions
                	union	{
                        	CP0Config	Config;         //	config register
                                DWORD		dwConfig;
                        };
                        union	{
                        	CP0Config1	Config1;        //	config1 register	the following three (3) registers are implementation specific
                                DWORD		dwConfig1;
                        };
                        union	{
                        	CP0Config2	Config2;        //	config2 register
                                DWORD		dwConfig2;
                        };
                        union	{
                        	CP0Config3	Config3;        //	config3 register
                                DWORD		dwConfig3;
                        };
                };
                union	{
                	CP0LLA		LLAddr;                 //	load linked address
                        DWORD		dwLLAddr;
                };
                union	{
                	CP0WatchLo	WatchLo;                //	watch lo
                        DWORD		dwWatchLo;
                };
                union	{
                	CP0WatchHi	WatchHi;                //	watch hi
                        DWORD		dwWatchHi;
                };
                DWORD		XContext;               //	MIPS 64
                DWORD		r21;                    //	reserved
                DWORD		r22;                    //	reserved
                union	{
                	struct	{
                        	unsigned DM : 1;
                                unsigned Debug : 31;                  //	debug register see MIPS docs for EJTAG Debug Mode
                        } Debug;
                        DWORD	dwDebug;
                };
                DWORD		DEPC;                   //	PC at last EJTAG debug exception
                DWORD		PerfCnt;                //	performance counters (optional)
                DWORD		ErrCtl;                 //	Parity/ECC error control and status
                DWORD		CacheErr;               //	cache parity error control and status
                union	{
                	DWORD		TagLo;                  //	tag lo cache tag interface
                        DWORD		DataLo;                 //	data lo cache dadat interface
                };
                union	{
                	DWORD		TagHi;                  //	tag hi
                        DWORD		DataHi;                 //	data hi
                };
                DWORD		ErrorEPC;               //	PC at last error
                DWORD		DESAVE;                 //	EJTAG debug exception save register
        } CtrlRegs;					//	Control Registers
        DWORD	Hi;					//	Hi / Lo register pair
        DWORD	Lo;
        DWORD	PC;					//	PC ( Program Counter )
        } CoProcessor0, SysControl, *PCoProcessor0, *PSysControl;
//---------------------------------------------------------------------------
//	MIPS 32 ISA complient FPU
//---------------------------------------------------------------------------
typedef struct	_fpu_fir_reg_	{	//	FIR FP Implementation and Revision Register
	unsigned	Revision : 8;
        unsigned	ProcessorID : 8;
        unsigned	S	: 1;
        unsigned	D	: 1;
        unsigned	PS	: 1;
        unsigned	_3D	: 1;
        unsigned	reserved:11;
        } CP1FIR, FPUFIR, *PCP1FIR, *PFPUFIR;
//---------------------------------------------------------------------------
typedef struct	_fpu_fcsr_reg_	{	//	FCCR FP Condition Codes register
	unsigned	RM	: 2;	//! 	Bits 0..1	Rounding Mode
        struct	{
        	unsigned	I : 1;	//! 	Bit  2
                unsigned	U : 1;  //!       3
                unsigned	O : 1;  //!       4
                unsigned	Z : 1;  //!       5
                unsigned	V : 1;  //!       6
        } Flags;
        struct	{
        	unsigned	I : 1;  //           7
                unsigned	U : 1;  //           8
                unsigned	O : 1;  //           9
                unsigned	Z : 1;  //           10
                unsigned	V : 1;  //           11
        } Enables;
        struct	{
        	unsigned	I : 1;  //           12
                unsigned	U : 1;  //           13
                unsigned	O : 1;  //           14
                unsigned	Z : 1;  //           15
                unsigned	V : 1;  //           16
                unsigned	E : 1;	//	     17
        } Cause;
        unsigned	reserved:3;     //           18  20
        unsigned	Impl	: 2;    //           21  22
        unsigned	FCC0	: 1;    //           23
        unsigned	FS	: 1;    //           24
        unsigned	FCC1	: 7;    //           25  31
        } CP1FCSR, FPUFCSR, *PCP1FCSR, *PFPUFCSR;
//---------------------------------------------------------------------------
typedef struct _fpu_fccr_reg_	{	//	FP Condition Codes register
	unsigned	FCC : 8;
        unsigned	reserved:24;
        } CP1FCCR, FPUFCCR, *PCP1FCCR, *PFPUFCCR;
//---------------------------------------------------------------------------
typedef struct _fpu_fexr_reg_ 	{	//	FP Exceptions register
	unsigned	reserved0:2;
        struct	{
        	unsigned	I : 1;
                unsigned	U : 1;
                unsigned	O : 1;
                unsigned	Z : 1;
                unsigned	V : 1;
        } Flags;
        unsigned	reserved1:5;
        struct	{
        	unsigned	I : 1;
                unsigned	U : 1;
                unsigned	O : 1;
                unsigned	Z : 1;
                unsigned	V : 1;
        } Cause;
        unsigned	reserved2:14;
        } CP1FEXR, FPUFEXR, *PCP1FEXR, *PFPUFEXR;
//---------------------------------------------------------------------------
typedef struct	_fpu_fenr_reg_	{	//	FP Enables register
	unsigned	RM : 2;
        unsigned	FS : 1;
        unsigned	reserved0:4;
        struct	{
        	unsigned	I : 1;
                unsigned	U : 1;
                unsigned	O : 1;
                unsigned	Z : 1;
                unsigned	V : 1;
        } Enables;
        unsigned	reserved1:20;
        } CP1FENR, FPUFENR, *PCP1FENT, *PFPUFENR;
//---------------------------------------------------------------------------
//	MIPS 32 FPU Supported Formats
//---------------------------------------------------------------------------
typedef	union	_fpu_fmt_ {		//	FPU Number Formats
	float	s;			//	32-bit single precision
        double	d;			//	64-bit double precision
        union	{			//	32-bit fixed precision
        	DWORD	dw;
        	struct {
        		unsigned	s : 1;	//	sign
                	unsigned	v : 31;	//	value
                } w;
        };
        } FPUFMT, *LPFPUFMT;
//---------------------------------------------------------------------------
typedef struct	_coprocessor_fpu_control_	{	//	FPU coprocessor CP1
	FPUFMT	GenRegs[ NUM_COPRO_REGS ];		//	General registers
        union {
	        struct {					//	Control registers
	        	union {
	                	FPUFIR	FIR;			//	( FIR ) Floating point Implementation and revision Register
	                        DWORD	dwFIR;
	                };
	                union {
	                	FPUFCSR	FCSR;			//	( FCSR ) Floating point Control / Status register
	                        DWORD	dwFCSR;
	                };
	                union {
			        FPUFCCR	FCCR;			//	( FCCR ) Floating point Condition Codes register
	                        DWORD	dwFCCR;
	                };
	                union {
			        FPUFEXR	FEXR;			//	( FEXR ) Floating point EXception Register
	                        DWORD	dwFEXR;
	                };
	                union {
			        FPUFENR	FENR;			//	( FENR ) Floating point ENables Register
	                        DWORD	dwFENR;
	                };
	        };
        	DWORD	dwCtlRegs[ 5 ];
        };
} CoProcessor1, FPU, *PCoProcessor1, *PFPU;
//---------------------------------------------------------------------------
//	MIPS 32 ISA CPU Instruction Formats
//---------------------------------------------------------------------------
typedef	struct	_cpu_instr_itype_	{	//	CPU Instruction Format Immediate Type
        unsigned	opcode : 6;
	union	{
	        unsigned	rs : 5;
	        unsigned	base : 5;
	};
	unsigned	rt : 5;
        union	{
		unsigned	immediate : 16;
	        unsigned	offset	  : 16;
	};
} CPUInstrIType, *PCPUInstrIType;
//---------------------------------------------------------------------------
typedef	struct	_cpu_instr_jtype_	{	//	CPU Instruction Format Jump Type
	unsigned	opcode : 6;
        unsigned	instr_index	: 26;
        } CPUInstrJType, *PCPUInstrJType;
//---------------------------------------------------------------------------
typedef	struct	_cpu_instr_rtype_	{	//	CPU Instruction Format Register Type
	unsigned	opcode  : 6;
        unsigned	rs	: 5;
        unsigned	rt	: 5;

        union {
        	struct {
                	unsigned	rd	: 5;
                        unsigned	sa 	: 5;
                };
                unsigned	code	: 10;
        };

        union	{
        	struct {
                	unsigned	res : 3;
                        unsigned	sel : 3;
                };
        	unsigned	immediate : 6;
                unsigned	function : 6;
        };

        } CPUInstrRType, *PCPUInstrRType;
//---------------------------------------------------------------------------
typedef struct _cpu_instr_syscall_type_	{	//	SysCall, and BREAK instruction
        unsigned	opcode	: 6;
        unsigned	code	: 20;
        unsigned	syscall	: 6;
        } CPUInstrSysCall, *LPCPUInstrSysCall;
//---------------------------------------------------------------------------
typedef	struct	_cpu_instr_copx_type_	{	//	COPx instruction
        unsigned	opcode	: 6;
        unsigned	co	: 2;
        unsigned	cofunc	: 24;
        } CPUInstrCType, *LPCPUInstrCType;
//---------------------------------------------------------------------------
//	MIPS 32 ISA FPU Instruction Formats
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_itype_	{	//	FPU Instruction Format Immediate Type
	unsigned	offset	: 16;
        unsigned	ft	: 5;
        unsigned	base	: 5;
        unsigned	opcode	: 6;	//	for FPU specific this would be COP1 opcode
        } FPUInstrIType, *PFPUInstrIType;
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_rtype_	{	//	FPU Instruction Format Register Type
        unsigned	opcode	: 6;	//	This is COP1 opcode for these instructions
        unsigned	fs	: 5;
        unsigned	ft	: 5;
        unsigned	function: 6;
        unsigned	fd	: 5;
        } FPUInstrRType, *PFPUInstrRType;
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_ritype_	{	//	FPU Instruction Format Register Immediate Type
        unsigned	opcode	: 6;
        unsigned	sub 	: 5;
        unsigned	rt	: 5;
        unsigned	reserved: 11;
        unsigned	fs	: 5;
        } FPUInstrRIType, *PFPUInstrRIType;
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_ccitype_	{	//	FPU Instruction Format Condition Code Immediate Type
	unsigned	offset	: 16;
        unsigned	tf	: 1;
        unsigned	nd	: 1;
        unsigned	cc	: 3;
        unsigned	bcc1	: 5;	//	Must be BCC1 opcode
        unsigned	opcode	: 6;	//	Must be COP1 opcode
        } FPUInstrCCIType, *PFPUInstrCCIType;
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_fmtcmptype_	{	//	FPU Instruction Format Formatted Compare Type
	unsigned	function	: 6;
        unsigned	reserved	: 2;
        unsigned	cc		: 3;
        unsigned	fs		: 5;
        unsigned	ft		: 5;
        unsigned	fmt		: 5;
        unsigned	opcode		: 6;
        } FPUInstrFmtCompType, *PFPUInstrFmtCompType;
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_ccmvtype_	{	//	FPU Instruction Format Conditional Register Move Type
	unsigned	movcf	: 6;
        unsigned	fd	: 5;
        unsigned	fs 	: 5;
        unsigned	tf	: 1;
        unsigned	reserved: 1;
        unsigned	cc	: 3;
        unsigned	fmt	: 5;
        unsigned	opcode	: 6;
        } FPUInstrCCMVType, *PFPUInstrCCMVType;
//---------------------------------------------------------------------------
typedef	struct	_fpu_instr_ccregitype_	{	//	FPU Instruction Format Condition code Register Integer Type
	unsigned	movc1	: 6;
        unsigned	reserved: 5;
        unsigned	rd	: 5;
        unsigned	tf	: 1;
        unsigned	reserved1:1;
        unsigned	cc	: 5;
        unsigned	rs	: 5;
        unsigned	opcode	: 6;
        } FPUInstrCCRegIntType, *PFPUInstrCCRegIntType;
//---------------------------------------------------------------------------
typedef	union	_cpu_instr_	{
	//	cpu generic instructions
        CPUInstrIType		CPUIType;	//	immediate type
        CPUInstrJType		CPUJType;	//	jump type
        CPUInstrRType		CPURType;	//	Register
        CPUInstrSysCall		CPUSType;	//	SYSCALL/BREAK
        CPUInstrCType		CPUCType;	//	COPx

        //	fpu generic instructions
        FPUInstrIType		FPUIType;	//	As above
        FPUInstrRType		FPURType;
        FPUInstrRIType		FPURIType;
        FPUInstrCCIType		FPUCCIType;
        FPUInstrFmtCompType	FPUFmtCmpType;
        FPUInstrCCMVType	FPUCCMVType;
        FPUInstrCCRegIntType	FPURegIntType;

        DWORD			dwOpCode;	//	Instruction as a 32-bit value

        } CPUInstr, *PCPUInstr;
//---------------------------------------------------------------------------
//	MIPS 32 Encoding of the Opcode Field
//---------------------------------------------------------------------------
//	If a value is NOT encoded here then it must generate a MIPS 32 ISA
//	Reserved Instruction Exception
//---------------------------------------------------------------------------
#define MIPS32_OPCODE_SPECIAL		0	//	SPECIAL, needs further decoding
#define MIPS32_OPCODE_REGIMM		1	//	REGIMM, Register Immediate
#define MIPS32_OPCODE_J			2	//	J	instr_index
#define MIPS32_OPCODE_JAL		3       //	JAL 	immediate
#define MIPS32_OPCODE_BEQ		4       //	BEQ
#define MIPS32_OPCODE_BNE		5       //	BNE
#define MIPS32_OPCODE_BLEZ		6       //	BLEZ
#define MIPS32_OPCODE_BGTZ		7       //	BGTZ
#define MIPS32_OPCODE_ADDI		8       //	ADDI
#define MIPS32_OPCODE_ADDIU		9       //	ADDIU
#define MIPS32_OPCODE_SLTI		10      //	SLTI
#define MIPS32_OPCODE_SLTIU		11      //	SLTIU
#define MIPS32_OPCODE_ANDI		12      //
#define MIPS32_OPCODE_ORI		13      //
#define MIPS32_OPCODE_XORI		14      //
#define MIPS32_OPCODE_LUI		15      //	LUI	rt, immediate
#define MIPS32_OPCODE_COP0		16      //
#define MIPS32_OPCODE_COP1		17      //! 	COP1 Specific Instruction
#define MIPS32_OPCODE_COP2		18      //
#define MIPS32_OPCODE_COP3		19      //
#define MIPS32_OPCODE_BEQL		20      //
#define MIPS32_OPCODE_BNEL		21      //
#define MIPS32_OPCODE_BLEZL		22      //
#define MIPS32_OPCODE_BGTZL		23	//
#define MIPS32_OPCODE_SPECIAL2		28	//
#define MIPS32_OPCODE_JALX              29      //
#define MIPS32_OPCODE_LB                32      //
#define MIPS32_OPCODE_LH                33      //
#define MIPS32_OPCODE_LWL               34      //
#define MIPS32_OPCODE_LW                35      //
#define MIPS32_OPCODE_LBU               36      //
#define MIPS32_OPCODE_LHU               37      //
#define MIPS32_OPCODE_LWR               38      //
#define MIPS32_OPCODE_SB                40      //
#define MIPS32_OPCODE_SH                41      //
#define MIPS32_OPCODE_SWL               42      //
#define MIPS32_OPCODE_SW                43      //
#define MIPS32_OPCODE_SWR               46      //
#define MIPS32_OPCODE_CACHE             47      //
#define MIPS32_OPCODE_LL                48      //
#define MIPS32_OPCODE_LWC1              49      //
#define MIPS32_OPCODE_LWC2              50      //
#define MIPS32_OPCODE_PREF              51      //
#define MIPS32_OPCODE_LDC1              53      //
#define MIPS32_OPCODE_LDC2              54      //
#define MIPS32_OPCODE_SC                56      //
#define MIPS32_OPCODE_SWC1              57      //
#define MIPS32_OPCODE_SWC2              58      //
#define MIPS32_OPCODE_SDC1              61      //
#define MIPS32_OPCODE_SDC2              62      //
//---------------------------------------------------------------------------
//	MIPS 32 SPECIAL Opcode Encoding of Function Field
//---------------------------------------------------------------------------
#define MIPS32_SPECIAL_OPCODE_SLL	0
#define MIPS32_SPECIAL_OPCODE_MOVC1     1
#define MIPS32_SPECIAL_OPCODE_SRL       2
#define MIPS32_SPECIAL_OPCODE_SRA       3
#define MIPS32_SPECIAL_OPCODE_SLLV      4
#define MIPS32_SPECIAL_OPCODE_SRLV      6
#define MIPS32_SPECIAL_OPCODE_SRAV      7
#define MIPS32_SPECIAL_OPCODE_JR        8
#define MIPS32_SPECIAL_OPCODE_JALR      9
#define MIPS32_SPECIAL_OPCODE_MOVZ      10
#define MIPS32_SPECIAL_OPCODE_MOVN      11
#define MIPS32_SPECIAL_OPCODE_SYSCALL   12
#define MIPS32_SPECIAL_OPCODE_BREAK     13
#define MIPS32_SPECIAL_OPCODE_SYNC      15
#define MIPS32_SPECIAL_OPCODE_MFHI      16
#define MIPS32_SPECIAL_OPCODE_MTHI      17
#define MIPS32_SPECIAL_OPCODE_MFLO      18
#define MIPS32_SPECIAL_OPCODE_MTLO      19
#define MIPS32_SPECIAL_OPCODE_MULT      24
#define MIPS32_SPECIAL_OPCODE_MULTU     25
#define MIPS32_SPECIAL_OPCODE_DIV       26
#define MIPS32_SPECIAL_OPCODE_DIVU      27
#define MIPS32_SPECIAL_OPCODE_ADD       32
#define MIPS32_SPECIAL_OPCODE_ADDU      33
#define MIPS32_SPECIAL_OPCODE_SUB       34
#define MIPS32_SPECIAL_OPCODE_SUBU      35
#define MIPS32_SPECIAL_OPCODE_AND       36
#define MIPS32_SPECIAL_OPCODE_OR        37
#define MIPS32_SPECIAL_OPCODE_XOR       38
#define MIPS32_SPECIAL_OPCODE_NOR       39
#define MIPS32_SPECIAL_OPCODE_SLT       42
#define MIPS32_SPECIAL_OPCODE_SLTU      43
#define MIPS32_SPECIAL_OPCODE_TGE       48
#define MIPS32_SPECIAL_OPCODE_TGEU      49
#define MIPS32_SPECIAL_OPCODE_TLT       50
#define MIPS32_SPECIAL_OPCODE_TLTU      51
#define MIPS32_SPECIAL_OPCODE_TEQ       52
#define MIPS32_SPECIAL_OPCODE_TNE       54
//---------------------------------------------------------------------------
//	MIPS 32 REGIMM Encoding of rt Field
//---------------------------------------------------------------------------
#define MIPS32_REGIMM_OPCODE_BLTZ	0
#define MIPS32_REGIMM_OPCODE_BGEZ       1
#define MIPS32_REGIMM_OPCODE_BLTZL      2
#define MIPS32_REGIMM_OPCODE_BGEZL      3
#define MIPS32_REGIMM_OPCODE_TGEI       8
#define MIPS32_REGIMM_OPCODE_TGEIU      9
#define MIPS32_REGIMM_OPCODE_TLTI       10
#define MIPS32_REGIMM_OPCODE_TLTIU      11
#define MIPS32_REGIMM_OPCODE_TEQI       12
#define MIPS32_REGIMM_OPCODE_TNEI       14
#define MIPS32_REGIMM_OPCODE_BLTZAL     16
#define MIPS32_REGIMM_OPCODE_BGEZAL     17
#define MIPS32_REGIMM_OPCODE_BLTZALL    18
#define MIPS32_REGIMM_OPCODE_BGEZALL    19
//---------------------------------------------------------------------------
//	MIPS 32 SPECIAL2 Encoding of Function Field
//---------------------------------------------------------------------------
#define MIPS32_SPECIAL2_OPCODE_MADD	0
#define MIPS32_SPECIAL2_OPCODE_MADDU    1
#define MIPS32_SPECIAL2_OPCODE_MUL      2
#define MIPS32_SPECIAL2_OPCODE_MSUB     4
#define MIPS32_SPECIAL2_OPCODE_MSUBU    5
#define MIPS32_SPECIAL2_OPCODE_CLZ      32
#define MIPS32_SPECIAL2_OPCODE_CLO      33
#define MIPS32_SPECIAL2_OPCODE_SDBBP    63
//---------------------------------------------------------------------------
//	MIPS 32 COP 0 Encoding of rs field	COPz where z == 0
//---------------------------------------------------------------------------
#define MIPS32_COP0_MFC0	0	//	Move From CP0
#define MIPS32_COP0_MTC0	1	//	Move To CP0
#define MIPS32_COP0_CO		0x1FF	//	CP0 TLB or other function
//---------------------------------------------------------------------------
//	MIPS 32 COP 0 Encoding of Function Field when _Rs_ == MIPS32_COP0_CO
//---------------------------------------------------------------------------
#define MIPS32_COP0_TLBR	1	//	TLB Read Indexed
#define MIPS32_COP0_TLBWI       2	//	TLB Write Indexed
#define MIPS32_COP0_TLBWR       6       //	TLB Write Random
#define MIPS32_COP0_TLBP        8       //	TLB Probe
#define MIPS32_COP0_ERET        24      //	Exception return
#define MIPS32_COP0_DERET       31      //	Debug return
#define MIPS32_COP0_WAIT        32      //	enter lower power mode
//---------------------------------------------------------------------------
//!	MIPS 32 FPU ISA Instructions.
//---------------------------------------------------------------------------
//! The MIPS 32 ISA defines the following types available and known to the FPU
//! SINGLE (float), DOUBLE (double) and FIXED, or WORD (unsigned long)
//---------------------------------------------------------------------------
#define	FPU_FMT_SINGLE		16		//! 	C / C++ Type equivalent 'float'
#define FPU_FMT_DOUBLE		17		//!  	C / C++ Type equivalent 'double'
#define FPU_FMT_FIXED		20		//!  	C / C++ Type equivalent 'unsigned long'
//---------------------------------------------------------------------------
#define MIPS32_COP1_MF		0
#define MIPS32_COP1_CF          2
#define MIPS32_COP1_MT          4
#define MIPS32_COP1_CT          6
#define MIPS32_COP1_BC          8
#define MIPS32_COP1_FMT_S	FPU_FMT_SINGLE
#define MIPS32_COP1_FMT_D	FPU_FMT_DOUBLE
#define MIPS32_COP1_FMT_W	FPU_FMT_FIXED
//---------------------------------------------------------------------------
//	Instructions if main opcode is COP1 and _Rs_ field is FMT_S/D/W
//---------------------------------------------------------------------------
#define MIPS32_COP1_ADD		0
#define MIPS32_COP1_SUB         1
#define MIPS32_COP1_MUL         2
#define MIPS32_COP1_DIV         3
#define MIPS32_COP1_SQRT        4
#define MIPS32_COP1_ABS         5
#define MIPS32_COP1_MOV         6
#define MIPS32_COP1_NEG         7
#define MIPS32_COP1_ROUND       12
#define MIPS32_COP1_TRUNC       13
#define MIPS32_COP1_CEIL        14
#define MIPS32_COP1_FLOOR       15
#define MIPS32_COP1_MOVC        17
#define MIPS32_COP1_MOVZ        18
#define MIPS32_COP1_MOVN        19
#define MIPS32_COP1_CVT_S       32
#define MIPS32_COP1_CVT_D       33
#define MIPS32_COP1_CVT_W       36
//---------------------------------------------------------------------------
//	MIPS 32 ISA COP 2 SPECIFIC INSTRUCTIONS
//---------------------------------------------------------------------------
#define MIPS32_COP2_MF		0
#define MIPS32_COP2_MT		4
#define MIPS32_COP2_CF		2
#define MIPS32_COP2_CT		6
#define MIPS32_COP2_BC		16
//---------------------------------------------------------------------------
//	Note that other opcodes are defined in the MIPS 32 ISA but not here
//	at present, will be added in the nearby future.
//---------------------------------------------------------------------------
//	CP2 is the GTE on the PlayStation ( I think )
//	CP3 is unknown (presume the SPU on Playstation?)
//---------------------------------------------------------------------------
//	CP2 specific instructions
//---------------------------------------------------------------------------
//	CPU Operating Modes	MIPS 32 PRA
//---------------------------------------------------------------------------
#define	CPUMODE_EJTAGDEBUG_IMPLEMENTED	//	Define this to allow the emulator to use debug mode else not available
//---------------------------------------------------------------------------
#define	CPUMODE_KERNEL		0	//	Kernel Mode
#define CPUMODE_SUPERVISOR	1	//	SuperVisor Mode
#define CPUMODE_USER		2	//	User Mode
#define CPUMODE_DEBUG		3	//	Debug mode, same as kernel, only used if CPUMODE_EJTAGDEBUG_IMPLEMENTED is defined else uses CPUMODE_KERNEL
//---------------------------------------------------------------------------
#define SYNCTYPE_DATA	0x00000001	//	Sync operation on data cache
#define SYNCTYPE_CODE	0x00000002	//	Sync operation on instruction cache
//---------------------------------------------------------------------------
typedef struct _cpu_irq_def_ {
	unsigned long cycle;
	unsigned long interrupt;
	unsigned long intCycle[32];
} CPUIrq, *lpCPUIrq;
//---------------------------------------------------------------------------
//__declspec( dllimport ) cpuIrqStruct cpuIrq;
//---------------------------------------------------------------------------
//#define _i32(x) (long)x
//#define _u32(x) x
//---------------------------------------------------------------------------
//#define _i16(x) (short)x
//#define _u16(x) (unsigned short)x
//---------------------------------------------------------------------------
//#define _i8(x) (char)x
//#define _u8(x) (unsigned char)x
//---------------------------------------------------------------------------
//	***** R3000A Instruction Macros *****
// For some reason my typedef's fail to decode the instruction correctly, so
// have reverted back to using the pcsx macros, ho hum. Seems to work though
//---------------------------------------------------------------------------
#define _PC_     (CPU->PC)			// 	The next PC to be executed
//---------------------------------------------------------------------------
#define _OpCode_ ((dwOpCode >> 26) & 0x3F)  //	The opcode part of the instruction register
#define _Funct_  ((dwOpCode      ) & 0x3F)  // 	The funct part of the instruction register
#define _Rd_     ((dwOpCode >> 11) & 0x1F)  // 	The rd part of the instruction register
#define _Rt_     ((dwOpCode >> 16) & 0x1F)  // 	The rt part of the instruction register
#define _Rs_     ((dwOpCode >> 21) & 0x1F)  // 	The rs part of the instruction register
#define _Sa_     ((dwOpCode >>  6) & 0x1F)  // 	The sa part of the instruction register
#define _Cc_	 ((dwOpCode >> 18) & 0x1F)  //  the cc portion of a COPZ instruction
#define _Bl_	 ((dwOpCode >> 16) & 0x01)  //  The likely flag
#define _Tf_	 ((dwOpCode >> 17) & 0x01)  //  The true/false flag
#define _Sel_	 ((dwOpCode      ) & 0x07)  //  sel field
#define _Im_     ((unsigned short)dwOpCode) // 	The immediate part of the instruction register
#define _Target_ (dwOpCode & 0x03ffffff)    // 	The target part of the instruction register
//---------------------------------------------------------------------------
//	Like above but for accessing the FPU registers
//---------------------------------------------------------------------------
#define	_Fs_	_Rd_
#define _Fd_	_Sa_
#define _Ft_	_Rt_
#define _Fmt_	_Rs_
//---------------------------------------------------------------------------
#define _Imm_	((short)dwOpCode)		// 	sign-extended immediate
#define _ImmU_	(dwOpCode&0xffff) 		// 	zero-extended immediate
//---------------------------------------------------------------------------
#define _rRs_   CP0.dwGenRegs[_Rs_]   		// 	Rs register/ also base
#define _rRt_   CP0.dwGenRegs[_Rt_]   		// 	Rt register
#define _rRd_   CP0.dwGenRegs[_Rd_]   		// 	Rd register
#define _rSa_   CP0.dwGenRegs[_Sa_]   		// 	Sa register
#define _rFs_   cpuRegs.CP0.r[_Rd_]   		// 	Fs register
//---------------------------------------------------------------------------
#define _c2dRs_ cpuRegs.CP2D.r[_Rs_]  		//	Rs cop2 data register
#define _c2dRt_ cpuRegs.CP2D.r[_Rt_]  		//	Rt cop2 data register
#define _c2dRd_ cpuRegs.CP2D.r[_Rd_]  		//	Rd cop2 data register
#define _c2dSa_ cpuRegs.CP2D.r[_Sa_]  		//	Sa cop2 data register
//---------------------------------------------------------------------------
//#define _rHi_   cpuRegs.GPR.n.hi   					// 	The HI register
//#define _rLo_   cpuRegs.GPR.n.lo   					// 	The LO register
//---------------------------------------------------------------------------
#define _JumpTarget_    ((_Target_ * 4) + (_PC_ & 0xf0000000))   	// 	Calculates the target during a jump instruction
#define _BranchTarget_  ((short)_Im_ * 4 + _PC_)                 	// 	Calculates the target during a branch instruction
//---------------------------------------------------------------------------
#define _SetLink(x)     CP0.dwGenRegs[x] = _PC_ + 4;       		// 	Sets the return address in the link register, generally x==31
//---------------------------------------------------------------------------
#define MIPS32_HARD_RESET	0
#define MIPS32_SOFT_RESET	1
//---------------------------------------------------------------------------
//	CLASS	::	R3000A
//	Purpose	::	MIPS 32 ISA CPU Emulation Layer
//---------------------------------------------------------------------------
class R3000A : public TThread
	{
        	//!	CPU <--> Memory Interface Emulation Layer
        	friend	class	PSX_MEM;
                //! 	PlayStation Counters Emulation Layer
        	friend	class	PSX_Counter;
                //!  	PlayStation CDROM Emulation Layer
                friend	class	PSX_CDROM;
                //! 	PlayStation BIOS Emulation Layer
        	friend	class	BIOS;
                //! 	PlayStation GTE/GPU Emulation Layer
        	friend	class	GTE;
                friend	class	HW;
                friend	class	CPSX_DMA;

                friend	class	TDebugger;	//	Debugger interface
		private:
                	//	Variables
                        WNDCLASSEX	WndClass;	//!  Windows Class, used to register the R3000A thread with windows
                        HINSTANCE	hInstance;	//!  instance of application attached to dll, or this class

                        CoProcessor0	CP0;		//! 	CP0 Coprocessor ( SysControl )
                        CoProcessor1	CP1;		//!  	CP1 Coprocessor ( FPU )
                        class	GTE	*CP2;		//! 	CP2 Coprocessor ( GPU/GTE )

                        PCPUInstr	lpInstr;	//! 	pointer to current instruction, ie lpInstr = (CPUInstr*)&RAM[ _PC ]
                        //  	CPU Cycle Count
                        DWORD		dwCPUCycle;

                        //	CPU Stack
                        DWORD		*lpStack;

                        //   CPU Interrupt Queue
                        lpCPUIrq	lpIrqQueue;
                        int		CurIrq;

                        //! 	Instruction Cache 	(1k)	*** NOT USED ***
                        DWORD		ICache[ 1024 ];		//!	Instruction cache, not used
                        //!  Data Cache 		(4K) 	*** NOT USED ***
                        DWORD		DCache[ 4096 ];		//!	data cache, not used

                        bool	fAtomicOperation;	//	Is a R-M-W sequence is effect ?
                        bool	fBranchDelaySlot;
                        bool	fUpdatePC;

                        //static	bool	fDebugControl;
                        //static	bool	fSingleStep;

                        //! 	Virtual Memory Manager		*** NOT USED ***
                        //	TLBENTRY	VM[ MAX_NUM_TLBENTRIES ];	//	Virtual Memory TLB Array

                	//! 	PSUEDO FUNCTIONS FROM MIPS 32 ISA PRM, see below

                        //! 	CoProcessor Access Functions for Floating Point Registers, READ / WRITE
                        float	__fastcall	ValueFPR			( int fpr, int fmt );
                        void	__fastcall	StoreFPR			( int fpr, int fmt, float value );

                        //! 	Miscellaneous Functions
                        void	__fastcall	SyncOperation			( int stype );

                        void	__fastcall	SignalException			( DWORD Exception, void *Arguments );

                        void	__fastcall	NullifyCurrentInstruction	( void );
                        void	__fastcall	CoProcessorOperation		( int z, int cop_fun );
                        bool	__fastcall	JumpDelaySlot			( DWORD vAddr );

                        bool	__fastcall	FPConditionCode			( int cc );

                        void	__fastcall	SetFPConditionCode		( int cc );

			void	__fastcall	DecodeSpecial		( void );	//	Decodes MIPS 32 SPECIAL OPCODES
                        void	__fastcall	DecodeSpecial2		( void );
                        void	__fastcall	DecodeREGIMM		( void );

                        void	__fastcall	DecodeCOP0		( void );	//	Decodes a COP0 specific instruction
                        void	__fastcall	DecodeCOP1		( void );
                        void	__inline	DecodeFPU		( void )	{ DecodeCOP1(); };
                        void	__fastcall	DecodeCOP2		( void );
                        void	__fastcall	DecodeCOP3		( void );

                        //	Read/Write Access to CPU General AND Control Registers
                        DWORD	__fastcall	GGenRegs	( int Index );
                        DWORD	__fastcall	GCtlRegs	( int Index );
                        void	__fastcall	SGenRegs	( int Index, DWORD Value );
                        void	__fastcall	SCtlRegs	( int Index, DWORD Value );

                        //	Read/Write Access to FPU General AND Control Registers
                        DWORD	__fastcall	GFPR		( int Index );
                        DWORD	__fastcall	GFCR		( int Index );
                        void	__fastcall	SFPR		( int Index, DWORD Value );
                        void	__fastcall	SFCR		( int Index, DWORD Value );

                        BYTE	__fastcall	GCPUMode	( void );
                        void	__fastcall	SCPUMode	( BYTE Value );

                        DWORD	__fastcall	GLo	( void );
                        void	__fastcall	SLo	( DWORD Value );

                        DWORD	__fastcall	GHi	( void );
                        void	__fastcall	SHi	( DWORD Value );

                        DWORD	__fastcall	GPC	( void );
                        void	__fastcall	SPC	( DWORD Value );

		protected:
                	void	__fastcall	DecodeInstruction	( void );	//	Decodes MIPS 32 OPCODES
			void 	__fastcall 	Execute			( void );

		public:
				__fastcall 	R3000A	( bool CreateSuspended );
                                __fastcall	~R3000A	( void );

                	int	__fastcall	Initialise	( void );
                        void	__fastcall	Terminate	( void );

                        void	__fastcall	Reset		( int ResetType = MIPS32_SOFT_RESET );

//#ifdef	CPUMODE_EJTAGDEBUG_IMPLEMENTED
			void	__fastcall	DumpRegs	( void );
//#endif

                        friend	int	__fastcall	LoadState	( LPSTR lpszFileName );
                        friend	int	__fastcall	SaveState	( LPSTR lpszFileName );

                	friend	bool	__fastcall	COPPresent	( int z );

                        friend	int	__fastcall	Load		( char *FileName );

                        friend	void 			AddIrqQueue	( int irqn, ... );

                        friend	void	__fastcall	AddressTranslation	( DWORD& pAddr, int& CCA, DWORD vAddr, bool LorS, bool IorD );

                        friend	void	__fastcall	EmulatorExecute		( void );

                        friend	LRESULT	CALLBACK	CPUWndProc		( HWND wHandle, UINT Msg, WPARAM wParam, LPARAM lParam );

                __published:
                	//!	Interface to CPU General Register Set
                	__property	DWORD	GenRegs[ int Index ] = { read=GGenRegs, write=SGenRegs };
                        //!	Interface to CPU Control Register Set, note that not all
                        //! registers are accessable if Reg <n> sel<n>!=0, ie can only access sel == 0
                        __property	DWORD	CtlRegs[ int Index ] = { read=GCtlRegs, write=SCtlRegs };


                        //	Interface to FPU General Registers
                        __property	DWORD	FPR[ int Index ] = { read=GFPR, write=SFPR };
                        //	and control registers
                        __property	DWORD	FCR[ int Index ] = { read=GFCR, write=SFCR };

                        //!	Current Operating Mode of CPU	EJTAG, KERNEL, SUPERVISOR or USER
                        __property	BYTE	CPUMode		     = { read=GCPUMode };

                        //!	Access to the Hi / Lo pairing registers
			__property	DWORD	Lo	= { read=GLo, write=SLo };
                        __property	DWORD	Hi	= { read=GHi, write=SHi };
                        //!	And the program counter
                        __property	DWORD	PC	= { read=GPC, write=SPC };
	};
//---------------------------------------------------------------------------
__declspec( dllimport )	R3000A	*CPU;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
