//---------------------------------------------------------------------------
//
//	$FileName	::	SPU.CPP
//	$Purpose	::	PlayStation Sound Processing Unit
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "SPU.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#define	CLASSNAME	"SPU"
//---------------------------------------------------------------------------
__declspec( dllexport ) PSX_SPU	*PsxSPU = NULL;
//---------------------------------------------------------------------------
//void 		*hSPUDriver;
//---------------------------------------------------------------------------
//long CALLBACK SPU__configure(void) { return 0; }
//void CALLBACK SPU__about(void) {}
//long CALLBACK SPU__test(void) { return 0; }
//---------------------------------------------------------------------------
//	Windows Message Handle for the PlayStation SPU Hardware Emulation
// maps to Direct Audio or Direct 3D Audio device.
//---------------------------------------------------------------------------
LRESULT	CALLBACK	SPUWndProc( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
        switch ( uMsg ) {	//	What message are we getting ?

		case ( WM_SPU_CREATE     ) : {
                	if ( PsxSPU != NULL )
                        	return( 1 );

                        PsxSPU = new PSX_SPU( true );
                        return( 0 );
                }

		case ( WM_SPU_DESTROY    ) : {
                	if ( PsxSPU == NULL )
                        	return( 1 );
                        delete PsxSPU;
                        return( 0 );
                }

		case ( WM_SPU_INITIALISE ) : {
                	if ( PsxSPU == NULL )
                        	return( 1 );
                	return( PsxSPU->Initialise() );
                }

		case ( WM_SPU_TERMINATE  ) : {
                	if ( PsxSPU == NULL )
                        	return( 1 );
                        PsxSPU->Terminate();
                	return( 0 );
                }

		case ( WM_SPU_RESET      ) : {
                	if ( PsxSPU == NULL )
                        	return( 1 );
                        PsxSPU->Reset();
                        return( 0 );
                }

		case ( WM_SPU_EXECUTE    ) : {
                	if ( PsxSPU == NULL )
                        	return( 1 );
                        PsxSPU->Resume();
                	return( 0 );
                }

		case ( WM_SPU_RESUME     ) : {
                	if ( PsxSPU == NULL )
                        	return( 1 );
                        PsxSPU->Resume();
                        return( 0 );
                }

		case ( WM_SPU_OPEN       ) : { return( 0 ); }
		case ( WM_SPU_CLOSE      ) : { return( 0 ); }

		case ( WM_SPU_READ       ) : { return( 0 ); }
		case ( WM_SPU_WRITE      ) : { return( 0 ); }

		case ( WM_SPU_PLAY       ) : { return( 0 ); }
		case ( WM_SPU_STOP       ) : { return( 0 ); }

		case ( WM_SPU_PLAYXA     ) : { return( 0 ); }

		case ( WM_SPU_LOAD       ) : { return( 0 ); }

        	default : {	//	Default action, return to windows to handle it
                	return( DefWindowProc( wHandle, uMsg, wParam, lParam ) );
                }
        }
}
//---------------------------------------------------------------------------
__fastcall	PSX_SPU :: PSX_SPU( bool CreateSuspended ) : TThread( CreateSuspended )
{
	//	Most basic initialisation
        fCreated = false;

        ZeroMemory( regArea, sizeof( WORD ) * SPU_BUFFER_SIZE );
        pBuffer = (DWORD*)&regArea[ 0x10000 ];

        lpChannels = new SPUAudioChannel[ MAX_CHANNELS ];
        ZeroMemory( lpChannels, sizeof( SPUAudioChannel ) * MAX_CHANNELS );

        //	Register class with windows and create the COM interface
	hInstance = GetModuleHandle( NULL );	//	Get instance of the applicaiton that we are attached to

        //	Obtain some memory for the SPU work area, and make sure it is cleared

        //	Register the SPU with windows, so we can handle things in the windows
        // messaging loop(s), so emulation and sound glitches should be kept to a
        // minimum, and as running as a seperate thread will be able to process sound
        // data independently of the main code and graphics threads
        ZeroMemory( &WndClass, sizeof( WNDCLASS ) );

        WndClass.hInstance = hInstance;		//	instance of the application, see above
        WndClass.lpfnWndProc = SPUWndProc;	//	which messaging procedure shall we use?
        WndClass.lpszClassName = CLASSNAME;

        //	Now try to register it with windows
        if ( !RegisterClass( &WndClass ) )
        	throw "Unable to register SPU Emulation Layer with windows.";

        //	Now we are registered and processing continues, if told to
        // create not suspended, ie run straight away

        fCreated = true;

        if ( !CreateSuspended )	{
        	Initialise();	//	Make sure that initialise is called if auto-run
        	Resume();
        }
}
//---------------------------------------------------------------------------
__fastcall	PSX_SPU :: ~PSX_SPU( void )
{
        if ( !Suspended )
        	Terminate();

	//CoUninitialise();	//

	UnregisterClass( CLASSNAME, &WndClass );

	if ( pBuffer != NULL )
        	delete []pBuffer;

	delete this;
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: SReg( DWORD Index, DWORD Value )
{ 	DWORD	r = Index & 0xfff;

	regArea[ ( r - 0xc00 ) / 2 ] = Value;

	if( r >= 0x0c00 && r < 0x0d80 ) {	//	Left / Right Channel Control 0-24
		unsigned char ch = ( r >> 4 ) - 0xc0;
		switch( r & 0x0f ) {//switch voices
			case 0:  //left volume	0-24
        			//SPU_setVolumeL(ch,value);
				return;
			case 2: //right volume	0-24
				//SPU_setVolumeR(ch,value);
				return;
			case 4:  //frequency	0-24
				//SPU_setPitch(ch,value);
				return;
			case 6://start address	0-24
	            		//SPU_setAddr(ch,value);
				return;
     //------------------------------------------------// level
//     			case 8:
//       			s_chan[ch].ADSRX.AttackModeExp  = (val&0x8000)?TRUE:FALSE;
//       			s_chan[ch].ADSRX.AttackRate     = (float)((val>>8) & 0x007f)*1000.0f/240.0f;
//       			s_chan[ch].ADSRX.DecayRate      = (float)((val>>4) & 0x000f)*1000.0f/240.0f;
//      			s_chan[ch].ADSRX.SustainLevel   = (float)((val)    & 0x000f);

//       			return;
//     			case 10:
//       			s_chan[ch].ADSRX.SustainModeExp = (val&0x8000)?TRUE:FALSE;
//       			s_chan[ch].ADSRX.ReleaseModeExp = (val&0x0020)?TRUE:FALSE;
//       			s_chan[ch].ADSRX.SustainRate    = ((float)((val>>6) & 0x007f))*R_SUSTAIN;
//       			s_chan[ch].ADSRX.ReleaseRate    = ((float)((val)    & 0x001f))*R_RELEASE;
//       			if(val & 0x4000) s_chan[ch].ADSRX.SustainModeDec=-1.0f;
//       			else             s_chan[ch].ADSRX.SustainModeDec=1.0f;
//       			return;
//     			case 12:
//       			return;
//     			case 14:
//       			s_chan[ch].pRepeat=spuMemC+((unsigned long) val<<3);
//       			return;
		}
    		return;
	}

	switch(r) {
		//case H_SPUaddr://SPU-memory address
    		//	spuAddr = (unsigned long) value<<3;
		//	spuAddr=value * 8;
    		//	return;
		//case H_SPUdata://DATA to SPU
//      		spuMem[spuAddr/2] = value;
//         		spuAddr+=2;
//        	//	if(spuAddr>0x7ffff) spuAddr=0;
		//	SPU_putOne(spuAddr,value);
    		//	spuAddr+=2;
    		//	return;
		//case H_SPUctrl://SPU control 1
    		//	spuCtrl=value;
    		//	return;
		//case H_SPUstat://SPU status
                //        spuStat=value & 0xf800;
    		//	return;
		//case H_SPUirqAddr://SPU irq
    		//	spuIrq = value;
    		//	return;
		//case H_SPUon1://start sound play channels 0-16
		//        SPU_startChannels1(value);
    		//	return;
		//case H_SPUon2://start sound play channels 16-24
    		//	SPU_startChannels2(value);
    		//	return;
		//case H_SPUoff1://stop sound play channels 0-16
    		//	SPU_stopChannels1(value);
    		//	return;
		//case H_SPUoff2://stop sound play channels 16-24
    		//	SPU_stopChannels2(value);
    		//	return;
	}
}
//---------------------------------------------------------------------------
//	Read a SPU register interface, either from the channels themselves or
// from the SPU
//---------------------------------------------------------------------------
DWORD	__fastcall	PSX_SPU :: GReg( DWORD Index )
{	WORD	wReg = ( Index & 0xFFF );

	if ( ( wReg >= 0xc00 ) || ( wReg <= 0xd80 ) )	{	//	Access Channels 0-24 L/R
        } else {
		switch ( Index & 0xfff ) {	//	Old Interface
		       	//	is the index in the range 0..0xfff
			case H_SPUctrl://spu control
	    			return spuCtrl;
			case H_SPUstat://spu status
	    			return spuStat;
			case H_SPUaddr://SPU-memory address
	                         return (unsigned short)(spuAddr>>3);
			case H_SPUdata://DATA to SPU
	    			spuAddr+=2;
	        		if(spuAddr>0x7ffff) spuAddr=0;
	        		//return spuMem[spuAddr/2];
				//return SPU_getOne(spuAddr);
                                return 0;
			case H_SPUirqAddr://spu irq
	    			return spuIrq;
			//case H_SPUIsOn1:
	    			//return IsSoundOn(0,16);
			//case H_SPUIsOn2:
	    			//return IsSoundOn(16,24);
                	default : {
                		return( Mem[ Index ] );
                        	//regArea[((add&0xfff)-0xc00)/2];
                	}
		}
	}
}
//---------------------------------------------------------------------------
DWORD	__fastcall	PSX_SPU :: GMem( DWORD Index )
{
	//	Read the SPU memory area, note that the first 0x10000 bytes
        // are used for the register interface to the SPU

        if ( ( Index < 0 ) || ( Index > 0x80000 ) )
        	return( 0 );

        if ( Index < 0x10000 ) {	//	Accessing the register interface
        	switch ( Index ) {
                	case ( H_SPUirqAddr ) : {	//	Access the Interrupt Request Register
                        	return( dwIrqReg );
                        }

                        case ( H_SPUaddr ) : {
                        	return( dwAddrReg );
                        }

                        case ( H_SPUdata ) : {
                        	dwAddrReg += 2;
                                if ( dwAddrReg > SPU_BUFFER_SIZE ) {
                                	dwAddrReg = 0;
                                        return( regArea[ SPU_BUFFER_SIZE - 2 ] );
                                }
                                return( regArea[ dwAddrReg - 2 ] );
                        }

                        case ( H_SPUctrl ) : {
                        	return( dwCommandReg );
                        }
                        case ( H_SPUstat ) : {
                        	return( dwStatusReg );
                        }
                        //case ( H_SPUon1 ) : {
                        //	return( 0 );
                        //}
                        //case ( H_SPUon2 ) : {
                        //	return( 0 );
                        //}
                        //case ( H_SPUoff1 ) : {
                        //	return( 0 );
                        //}
                        //case ( H_SPUoff2 ) : {
                        //	return( 0 );
                        //}
                        default : {
                        	return( regArea[ Index ] );
                        }
                }
        } else {			//	Accessing the actual sound buffer
        	return( regArea[ Index ] );
        }
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: SMem( DWORD Index, WORD Value )
{
	//	Write the SPU memory area, note that the first 0x10000 bytes
        // are used for the register interface to the SPU

        if ( ( Index < 0 ) || ( Index > 0x80000 ) )
        	return;

        if ( Index < 0x10000 ) {	//	Accessing the register interface
        	switch ( Index ) {
                	case ( H_SPUirqAddr ) : {	//	Access the Interrupt Request Register
                        	if ( dwIrqReg != Value )
                                	dwIrqReg = Value;
                                break;
                        }

                        case ( H_SPUaddr ) : {
                        	dwAddrReg += 2;
                                if ( dwAddrReg > SPU_BUFFER_SIZE ) {
                                	dwAddrReg = 0;
                                        regArea[ SPU_BUFFER_SIZE - 2 ] = Value;
                                }
                                break;
                        }

                        case ( H_SPUdata ) : {
                        	dwAddrReg += 2;
                                if ( dwAddrReg > SPU_BUFFER_SIZE ) {
                                	dwAddrReg = 0;
                                        regArea[ SPU_BUFFER_SIZE - 2 ] = Value;
                                }
                                break;
                        }

                        case ( H_SPUctrl ) : {
                        	dwCommandReg = Value;
                        }
                        case ( H_SPUstat ) : {
                        	dwStatusReg = Value;
                        }
                        //case ( H_SPUon1 ) : {
                        //	( 0 );
                        //}
                        //case ( H_SPUon2 ) : {
                        //	( 0 );
                        //}
                        //case ( H_SPUoff1 ) : {
                        //	( 0 );
                        //}
                        //case ( H_SPUoff2 ) : {
                        //	( 0 );
                        //}
                        default : {
                        	regArea[ Index ] = Value;
                        }
                }
        } else {			//	Accessing the actual sound buffer
        	regArea[ Index ] = Value;
        }
}
//---------------------------------------------------------------------------
//	Based on transfer direction copy DMA4 COUNT bytes from/to DMA4 ADDR
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: hwDMA4( void )
{
}
//---------------------------------------------------------------------------
//	Initialise the SPU Emulation Layer
//---------------------------------------------------------------------------
int	__fastcall	PSX_SPU :: Initialise( void )
{
	//	Clear audio buffer
	ZeroMemory( regArea, sizeof( WORD ) * SPU_BUFFER_SIZE );
        ZeroMemory( lpChannels, sizeof( SPUAudioChannel ) * MAX_CHANNELS );

        //	Initialise Direct Audio Interface
        //CoInitialise( NULL );

        //CoCreateInstance( CLSID_DirectMusicLoader, NULL, CLSCTX_INPROC, IID_IDirectMusicLoader8, (void**)&lpLoader );

        //CoCreateInstance( CLSID_DirectMusicPerformance, NULL, CLSCTX_INPROC, IID_IDirectMusicPerformance8, (void**)&lpPerformance );

        //	Initialise the audio interface
        //lpPerformance->InitAudio( NULL, NULL, NULL,
        //	DMUS_APATH_SHARED_STEREOPLUSREVERB,
        //        24, DMUS_AUDIOF_ALL, NULL );

        //	Set the search directory for audio samples
        //lpLoader->SetSearchDirectory( GUID_DirectMusicAllTypes, AUDIO_SEARCH_PATH, false );

	//	Set the channels up

        for ( int i=0; i<MAX_CHANNELS; i++ ) {

        	//	Set memory reference addresses
        	lpChannels[ i ].dwStartAddr = 0xFFFFFFFF;
                lpChannels[ i ].dwEndAddr = 0xFFFFFFFF;

                //	Set volume
                lpChannels[ i ].Volume.Left = 127;
                lpChannels[ i ].Volume.Right = 127;

                //	Set pitch
                lpChannels[ i ].dwPitch = 22500;

                //	Set flags
                lpChannels[ i ].dwFlags = SPUF_CPUMODE | SPUF_STOPPED;
        }

	return( 0 );
}
//---------------------------------------------------------------------------
//	Terminate all audio properties
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: Terminate( void )
{
	//if ( fAudioPlaying )
        //	lpPerformance->Stop( NULL, NULL, 0, 0 );

        //lpPerformance->CloseDown();

	//lpSegment->Release();
        //lpPerformance->Release();
        //lpLoader->Release();
        Suspend();
        fCreated = false;
}
//---------------------------------------------------------------------------
//	Reset audio state
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: Reset( void )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: Execute( void )
{
	while ( ( !Terminated ) && ( !Suspended ) ) {
        	MSG	Msg;
                while ( PeekMessage( &Msg, NULL, NULL, NULL, PM_REMOVE ) != NULL ) {
                	TranslateMessage( &Msg );
                        DispatchMessage( &Msg );
                }
                ZeroMemory( &Msg, sizeof( MSG ) );
                for ( int i=0; i<MAX_CHANNELS; i++ ) {
                	if ( lpChannels[ i ].dwFlags & SPUF_PLAYING ) {	//	Sound playing
                        	//	process it
                        } else {
                        	continue;
                                // with other channels
                        }
                }
	}
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: StartChannel( int Channel, DWORD dwOffset )
{
	//	Start the channel playing from memory at dwOffset
        if ( ( Channel > 0 ) || ( Channel < MAX_CHANNELS ) )	{	//	Valid channel ?
        	lpChannels[ Channel ].dwStartAddr = dwOffset;
                lpChannels[ Channel ].dwEndAddr = 0x80000;
                lpChannels[ Channel ].dwFlags |= SPUF_PLAYING;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: StopChannel( int Channel, DWORD dwOffset )
{
	//	Stop the channel playing at memory dwOffset
        if ( ( Channel > 0 ) || ( Channel < MAX_CHANNELS ) )	{
        	lpChannels[ Channel ].dwEndAddr = dwOffset & 0x7FFFF;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: SetVolume( bool LorR, DWORD Value )
{
	if ( LorR ) {	//	Set Volume Left
        } else {	//	Set Volume Right
        }
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: SetPitch( DWORD Value )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: SetAddr( DWORD Value )
{
}
//---------------------------------------------------------------------------
void 	__fastcall	PSX_SPU :: PlayADPCMChannel( LPVOID lpbuffer /*xa_decode_t *xap*/ )
{
}
//---------------------------------------------------------------------------
void	__fastcall	PSX_SPU :: RegisterCallBack( LPVOID (lpCallBack)(void) )
{
	if ( lpCallBack == NULL )
        	return;
}
//---------------------------------------------------------------------------
