//---------------------------------------------------------------------------
//
//	$FileName	::	SPU.h
//	$Author		::	Chris Brooklyn
//	$Purpose	::	To provide an interface between the audio
//				interface on the PC to the hardware emulation layer
//				for the SPU
//
//---------------------------------------------------------------------------
#ifndef SPUH
#define SPUH
//---------------------------------------------------------------------------
#ifndef EMULATOR_DLL_H
#include "EmulatorDLL.h"
#endif
//---------------------------------------------------------------------------
#ifndef HWH
#include "hw.h"
#endif
//---------------------------------------------------------------------------
//	Messages that are sent through the windows messaging sub-system, that
// the SPU Emulation Layer Thread understands.
//---------------------------------------------------------------------------
#define	WM_SPU_CREATE		WM_CREATE
#define WM_SPU_DESTROY		WM_DESTROY
#define WM_SPU_INITIALISE       MSG_INITIALISE
#define WM_SPU_TERMINATE	MSG_TERMINATE
#define WM_SPU_RESET		MSG_RESET
#define WM_SPU_EXECUTE		MSG_EXECUTE
#define WM_SPU_SUSPEND		MSG_EXECUTE
#define WM_SPU_RESUME		MSG_RESUME
#define WM_SPU_OPEN             WM_USER + 0x0000000A
#define WM_SPU_CLOSE		WM_USER + 0x0000000B
#define WM_SPU_READ		MSG_TRANSFER_READ
#define WM_SPU_WRITE		MSG_TRANSFER_WRITE
#define WM_SPU_PLAY		WM_USER + 0x0000000C
#define WM_SPU_STOP             WM_USER + 0x0000000D
#define WM_SPU_PLAYXA           WM_USER + 0x0000000E
#define WM_SPU_LOAD             WM_USER + 0x0000000F
//---------------------------------------------------------------------------
#define SPU_BUFFER_SIZE	0x80000		//	512K internal sound work space
//---------------------------------------------------------------------------
//	PlayStation addresses for hardware access to the SPU Registers
//---------------------------------------------------------------------------
#define H_SPUirqAddr    0x0da4		//	SPU IRQ Control
#define H_SPUaddr       0x0da6		//	SPU DMA Control
#define H_SPUdata       0x0da8		//	SPU Data Port
#define H_SPUctrl       0x0daa		//	SPU Control
#define H_SPUstat       0x0dae		//	SPU Status
#define H_SPUon1        0x0d88		//	Channels 0-15
#define H_SPUon2        0x0d8a		//	Channels 16-24
#define H_SPUoff1       0x0d8c		//	Channels 0-15
#define H_SPUoff2       0x0d8e		//	Channels 16-24
//---------------------------------------------------------------------------
#ifndef lpHW
//---------------------------------------------------------------------------
__declspec( dllimport )	class HW *lpHW;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
#define SPUF_PLAYING	0x01000000
#define	SPUF_STOPPED	~SPUF_PLAYING
#define SPUF_PAUSED	0x02000000
#define SPUF_XAMODE	0x00010000
//---------------------------------------------------------------------------
#define SPUF_CPUMODE	0x10000000	//	Use CPU to transfer the data
#define SPUF_DMAMODE	0x20000000	//	Use DMA to transfer the data
#define SPUF_IRQMODE	0x40000000	//	Use IRQ to transfer the data
//---------------------------------------------------------------------------
#define	MAX_CHANNELS	25
//---------------------------------------------------------------------------
//#define	IDMSegment	IDirectMusicSegment8
//---------------------------------------------------------------------------
typedef	struct	_spu_audio_channel_def_	{	//	SPU Audio Channel 25 are supported
        DWORD		dwStartAddr;	//	Start address for reading / writing audio data for this channel
        DWORD		dwEndAddr;	//	end address for reading / writing audio data for this channel
        struct	{		//	Volume
        	int	Left;	//	Left
                int	Right;	//	Right
        } Volume;
        DWORD		dwPitch;	//	Channel Pitch (Frequency)
        DWORD		dwFlags;	//	Flags, signal IRQ, using DMA, etc
        //IDMSegment	*lpSegment;
	} SPUAudioChannel, *lpSPUAudioChannel;
//---------------------------------------------------------------------------
class	PSX_SPU : public TThread
	{
        	protected:
                	void	__fastcall	hwDMA4		( void );

                        void	__fastcall	Interrupt	( void );
                        void	__fastcall	Execute		( void );

                        void	__fastcall	StartChannel	( int Channel, DWORD dwOffset );
                        void	__fastcall	StopChannel	( int Channel, DWORD dwOffset );

                        void	__fastcall	SetVolume	( bool LorR, DWORD Value );
                        void	__fastcall	SetPitch	( DWORD Value );
                        void	__fastcall	SetAddr		( DWORD Value );
                        void 	__fastcall	PlayADPCMChannel( LPVOID lpBuffer /*xa_decode_t *xap*/ );

                        int	__fastcall	InitialiseChannel	( AnsiString FilePath, int Volume, int Pitch );
                private:
                	//	Windows Specific
                		HINSTANCE	hInstance;
                        	WNDCLASS	WndClass;
                                HANDLE		hEvent;		//	WIN32 Event for SPU Emulation Layer
                        //	Direct Audio specific
                        	//IDirectMusicLoader8*		lpLoader;
                                //IDirectMusicPerformance8*	lpPerformance;
                                //IDirectMusicSegment8*		lpSegment;
                        //	SPU Specific
                		DWORD	*pBuffer;	//	points to regArea[ 0x1000 ]
                        	DWORD	dwCommandReg;
                                DWORD	dwStatusReg;
                                DWORD	dwIrqReg;
                                DWORD	dwAddrReg;

                                lpSPUAudioChannel	lpChannels;

                                WORD	regArea[ 0x80000 ];
				WORD	spuCtrl, spuStat, spuIrq;
				DWORD	spuAddr;

                                bool	fCreated;

                                void	__fastcall	RegisterCallBack	( LPVOID (lpCallBack)(void) );


                        DWORD	__fastcall	GReg	( DWORD Index );
                        void	__fastcall	SReg	( DWORD Index, DWORD Value );

                        DWORD	__fastcall	GMem	( DWORD Index );
                        void	__fastcall	SMem	( DWORD Index, WORD Value );
                public:
                		__fastcall	PSX_SPU	( bool CreateSuspended );
                        	__fastcall	~PSX_SPU	( void );
                        int	__fastcall	Initialise	( void );
                        void	__fastcall	Terminate	( void );
                        void	__fastcall	Reset		( void );
                        friend	LRESULT	CALLBACK	WndProc	( HWND wHandle, UINT uMsg, WPARAM wParam, LPARAM lParam );
                        friend	void	__fastcall	EmulatorExecute( void );
                __published:
                	__property	DWORD	Reg[ DWORD Index ] = { read=GReg, write=SReg };
                        __property	WORD	Mem[ DWORD Index ] = { read=GMem, write=SMem };
        };
//---------------------------------------------------------------------------
__declspec( dllimport )	PSX_SPU	*PsxSPU;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
