//---------------------------------------------------------------------------
//
//	$FileName	::	ChildWin.cpp
//	$Author		::	Chris Brooklyn
//	$Version	::	0.2.0.214
//	$Purpose	::	This is the actual editor form used when editing
//				any file, or module from a project
//
//---------------------------------------------------------------------------
#include <vcl.h>
#include "ChildWin.h"
#include "ProjectManager.h"
#include "main.h"
#include "EnviromentOptionsUnit.h"
#include "Utilities.h"
#include "shellapi.h"
//---------------------------------------------------------------------------
#pragma hdrstop				//	Force BCB to create pre-compiled headers for my code as well
//---------------------------------------------------------------------------
#pragma link "SynEdit"
#pragma link "SynEditHighlighter"
#pragma link "SynHighlighterCpp"
#pragma link "SynHighlighterDfm"
//---------------------------------------------------------------------------
#pragma link "SynEditPrint"
#pragma link "SynAutoCorrect"
#pragma link "SynCompletionProposal"
#pragma link "SynHighlighterAsm"
#pragma link "SynHighlighterBat"
#pragma link "SynHighlighterPas"
#pragma resource "*.dfm"		//	Link with the form itself
//---------------------------------------------------------------------------
//	TMDIChild	Constructor
//---------------------------------------------------------------------------
__fastcall	TMDIChild :: TMDIChild( TComponent *Owner ) : TForm( Owner )
{
	asFileName = "Untitled.cpp";    //	Default file name for this module
        Index = -1;			//	Not part of this project
        Modified = false;		//	has not been modified
}
//---------------------------------------------------------------------------
//	Close MDI Child. Called from the IDE. FOR USE BY THE IDE ONLY
//---------------------------------------------------------------------------
void	__fastcall	TMDIChild :: IDEClose( TObject *Sender )
{	TComponent *Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )
        	return;

        if ( Index >= 0 )	{
        	if ( ProjectManagerForm->ActiveProject )	{
                	ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].Modified = false;
                        SynEdit1->Lines->SaveToFile( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File );
                        if ( FileName != ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File )
                        	FileName = ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File;
                }
        }

        if ( EnviromentOptionsForm->AutoSaveEditorFiles ) {			//	Auto saving being used ?
        	if ( Modified ) {						//	Do we need to bother auto saving ?
                	SystemMessage( "File Modified, auto saving" );		//	Output a nice message to the user
                	IDESave( Sender );					//	Yes, grr
                }
                Modified = false;
        }

	if ( EnviromentOptionsForm->CreateBackupFiles ) {			//	Do we need to make a backup copy ?
        	SystemMessage( "Creating backup for file " + FileName );        //	Yes, we need to implement this
                int p = FileName.Pos( "." );
                if ( p > 0 ) {	//	Found extension
                	p++;
                	SynEdit1->Lines->SaveToFile( FileName.SubString( p, FileName.Length() - p + 1 ) + ".bak" );
                        SystemMessage( "Auto-backing up to file " + FileName.SubString( p - 1, FileName.Length() - p ) + ".bak" );
                }
        }

        Index = -1;
        Close();
}
//---------------------------------------------------------------------------
void	__fastcall	TMDIChild :: IDESave( TObject *Sender )
{
        SystemMessage( "IDE Save Executing saving current editor window as " + FileName );
        Modified = false;
        SynEdit1->Modified = Modified;
        if ( Index >= 0 ) {
        	if ( ProjectManagerForm->ActiveProject )
                	ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].Modified = false;

                FileName = ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File;
        }

        SynEdit1->Lines->SaveToFile( FileName );
}
//---------------------------------------------------------------------------
//	Allows the IDE to write information to an MDI Child form
//	Used for diagnostics/development purposes only will be removed
//	in the final release
//---------------------------------------------------------------------------
void	__fastcall	TMDIChild :: ChildWrite( AnsiString Value )
{
	if ( Value.Length() > 0 ) {
        	SynEdit1->Lines->Add( Value );
                Modified = SynEdit1->Modified;
        }

        SystemMessage( "Child Write [ " + Value + " ]" );
}
//---------------------------------------------------------------------------
//	Property Method returns the filename for the MDI Child, well actually
//	the filename that the richedit will load/save
//---------------------------------------------------------------------------
AnsiString	__fastcall	TMDIChild :: GFileName( void )
{
	if ( Index >= 0 )	{
        	if ( ProjectManagerForm->ActiveProject )
                	if ( FileName != ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File )
                        	FileName = ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File;
        }
        if ( asFileName.Length() > 0 )	return( asFileName );
        else return( NULL );
}
//---------------------------------------------------------------------------
//	Opposite of above
//
//	Note that the filename is usually the same as the title of the mdi child
//	form, but not always
//---------------------------------------------------------------------------
void	__fastcall	TMDIChild :: SFileName( AnsiString Value )
{
	if ( asFileName != Value ) {
        	if ( Index > 0 ) {
                	if ( ProjectManagerForm->ActiveProject ) {
                        	if ( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File != Value ) {
                                	ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File = Value;
                                        ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].Modified = true;
                                        ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].Modified = true;
                                        ProjectManagerForm->lpGroup->Modified = true;
                                }
                        }
                }
                asFileName = Value;
        }
}
//---------------------------------------------------------------------------
//	Property Read Method
//
//	Allows you to determine if the richedit has been modified or not
//	Rather obvious eh?
//---------------------------------------------------------------------------
bool	__fastcall	TMDIChild :: GModified( void )
{
	if ( Index >= 0 ) {
        	if ( ProjectManagerForm->ActiveProject ) {
                	if ( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].Modified )
                        	return( true );
			if ( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].Modified )
                        	return( true );
                }
                if ( ProjectManagerForm->ActiveGroup )
                	if ( ProjectManagerForm->lpGroup->Modified )
                        	return( true );
        }

	return( SynEdit1->Modified );
}
//---------------------------------------------------------------------------
//	Opposite of above Property Modified Write Method
//---------------------------------------------------------------------------
void	__fastcall	TMDIChild :: SModified( bool Value )
{
	if ( Index >= 0 ) {
        	if ( ProjectManagerForm->ActiveProject ) {
                	if ( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].Modified != Value ) {
                        	ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].Modified = Value;
                        }
                        if ( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].Modified != Value )
                        	ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].Modified = Value;
                }
                if ( ProjectManagerForm->ActiveGroup )
                	if ( ProjectManagerForm->lpGroup->Modified != Value )
                        	ProjectManagerForm->lpGroup->Modified = Value;
        }

	if ( SynEdit1->Modified != Value )
        	SynEdit1->Modified = Value;
}
//---------------------------------------------------------------------
//	Properties Click Menu
//	Responds to the click on the pop menu option 'Properties'
//	Opens the Enviroment Options Form and switches to the 'Display' tab
//	Allows for modification of the editor properties, ie font face, size
//	colour etc
//---------------------------------------------------------------------------
void __fastcall TMDIChild::Properties1Click(TObject *Sender)
{
        EnviromentOptionsForm->PageControl1->ActivePage = EnviromentOptionsForm->Display;
        EnviromentOptionsForm->ShowModal();

        if ( EnviromentOptionsForm->ModalResult != mrOk ) return;

        EnviromentOptionsForm->PageControl1->ActivePage = EnviromentOptionsForm->Preferences;

        SynEdit1->Font->Assign( EnviromentOptionsForm->Label6->Font );
        SynEdit1->SetFocus();
        SynEdit1->Update();
        Update();
}
//---------------------------------------------------------------------------
void __fastcall TMDIChild::ReadOnly1Click(TObject *Sender)
{
	ReadOnly1->Checked = !( ReadOnly1->Checked );
        SynEdit1->ReadOnly = ReadOnly1->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TMDIChild::FontDialog1Apply(TObject *Sender, HWND Wnd)
{
	SynEdit1->Font->Assign( FontDialog1->Font );
}
//---------------------------------------------------------------------------
void __fastcall TMDIChild::FormKeyPress(TObject *Sender, char &Key)
{
	//	Insert Key Pressed ?
	if ( Key == VK_INSERT ) {
        	SynEdit1->InsertMode = !( SynEdit1->InsertMode );
        }

        //	NumLock Key Pressed ?
        if ( Key == VK_NUMLOCK ) {
        	MainForm->NumLock = !( MainForm->NumLock );
        }

        if ( Key == VK_F1 ) {	//	F1 Pressed, user wants help goto help section of site
        	ShellExecute( Handle, "open", "iexplore.exe",
                	"http://localhost/Help/", NULL, SW_SHOWNORMAL );
        }

        //	Here will be located some, most, of the key bindings,
        //	which need to be implemented at some point, ho hum :)
        //
        //	We currently only use the default key mapping provided
        //	by the TSynEdit component's

}
//---------------------------------------------------------------------------
void __fastcall TMDIChild::SynEdit1Paint(TObject *Sender, TCanvas *ACanvas)
{
	SynEdit1->Gutter->Visible = EnviromentOptionsForm->VisibleGutter;
        SynEdit1->Gutter->Width = EnviromentOptionsForm->GutterWidth;
        SynEdit1->Update();
}
//---------------------------------------------------------------------------
void __fastcall TMDIChild::FormClose(TObject *Sender, TCloseAction &Action)
{
	IDEClose( Sender );	//	Check for IDE closure
	if ( Modified )	{	//	Contents of this form been modified ?
        	if ( Index >= 0 ) {
                	if ( ProjectManagerForm->ActiveProject )
                        	SynEdit1->Lines->SaveToFile( ProjectManagerForm->lpGroup->lpProjects[ ProjectManagerForm->CurrProject ].lpModules[ Index ].File );
                }
        	int mrCode = MessageBox( Application->Handle, AnsiString( "File modified save?" ).c_str(),
                		Application->Title.c_str(), MB_YESNO | MB_ICONQUESTION );
                if ( mrCode == mrYes )
                	IDESave( this );

                Modified = false;
        }
        Action = caFree;
}
//---------------------------------------------------------------------------
//	This attempts to open to file specified by the text under the current
// cursor position
//---------------------------------------------------------------------------
void 	__fastcall 	TMDIChild :: OpenFileAtCursor1Click( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )	return;

        ClearSysMessages();

        SystemMessage( "Open File At Cursor" );
}
//---------------------------------------------------------------------------
void __fastcall TMDIChild::FormCreate(TObject *Sender)
{
	//	Make sure that the SynEdit options match the ones that are
        //	either stored in the registry and/or the enviroment options
        //	form

        if ( Sender == NULL )
        	return;

        LogMessage( "[TMDICHILD::FORMCREATE]" );

        if ( !EnviromentOptionsForm ) {
        	SystemMessage( "No enviroment options form available for options setting." );
                return;
        }

        //	Form exists make sure that the currently selected editor enviroment
        // is mapped onto the SynEdit1 component

        SynEdit1->InsertMode = EnviromentOptionsForm->InsertMode;

        SynEdit1->Gutter->Visible = EnviromentOptionsForm->VisibleGutter;

        if ( SynEdit1->Gutter->Visible )
        	SynEdit1->Gutter->Width = EnviromentOptionsForm->GutterWidth;

        //if ( EnviromentOptionsForm->AutoIndentMode )
        //	SynEdit1->Options << eoAutoIndentMode;
        //else	SynEdit1->Options >> eoAutoIndentMode;

        //SynEdit1->Options.eoScrollPastEof      = EnviromentOptionsForm->CursorBeyondEOF;
        //SynEdit1->Options.eoSmartTabs          = EnviromentOptionsForm->SmartTab;
        //SynEdit1->Options.eoTabsToSpaces       = !( EnviromentOptionsForm->UseTabChar );
        //SynEdit1->Options.eoTrimTrailingSpaces = EnviromentOptionsForm->KeepTrailingBlanks;

        if ( EnviromentOptionsForm->UseSyntaxHighlighting )
        	SynEdit1->Highlighter = SynCppSyn1;
        else	SynEdit1->Highlighter = NULL;

        if ( EnviromentOptionsForm->ZoomToFullScreen )
        	WindowState = wsMaximized;

        if ( EnviromentOptionsForm->VisibleRightMargin ) {
        	SynEdit1->RightEdgeColor = clSilver;
                SynEdit1->RightEdge = EnviromentOptionsForm->RightMargin;
        } else	{
        	SynEdit1->RightEdgeColor = Color;
                SynEdit1->RightEdge = 80;
        }

        if ( EnviromentOptionsForm->VisibleGutter ) {
        	SynEdit1->Gutter->Visible = EnviromentOptionsForm->VisibleGutter;
        	SynEdit1->Gutter->Width = EnviromentOptionsForm->GutterWidth;
        } else  SynEdit1->Gutter->Visible = false;

        SynEdit1->Font->Assign( EnviromentOptionsForm->Label6->Font );

}
//---------------------------------------------------------------------------
