//---------------------------------------------------------------------------
//
//	$FileName	::	ClassExplorer.h
//	$Author		::	Chris Brooklyn
//	$Language	::	C++ with VCL extensions
//
//--------------------------------------------------------------------------- 
#ifndef ClassExplorerH
#define ClassExplorerH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Buttons.hpp>
#include <ExtCtrls.hpp>
#include <ComCtrls.hpp>
#include <ImgList.hpp>
#include <ToolWin.hpp>
#include <Menus.hpp>
//---------------------------------------------------------------------------
//	Access Methods	:	PUBLIC, PRIVATE, PROTECTED
//---------------------------------------------------------------------------
#define	ACCESS_METHOD_PUBLIC	0x00000001	//	Object is public or published
#define ACCESS_METHOD_PRIVATE	0x00000002	//	Object is private
#define ACCESS_METHOD_PROTECTED	0x00000004	//	Object is protected, ie inheriter callable
#define ACCESS_METHOD_READONLY	0x00000008      //	Object is a read only object, usually properties
#define	ACCESS_METHOD_READWRITE	0x00000010	//	Object is read/write, usually properties
//---------------------------------------------------------------------------
//	Object Types
//---------------------------------------------------------------------------
#define	OBJECT_TYPE_VARIABLE	0x00000100	//	Object is a Global Variable
#define OBJECT_TYPE_ROUTINE	0x00000200	//	Object is a Global Method
#define OBJECT_TYPE_METHOD	0x00000400	//	Object is a Class Method
#define	OBJECT_TYPE_FIELD	0x00000800	//	Object is a Class Field (Property)
#define OBJECT_TYPE_PROPERTY	OBJECT_TYPE_FIELD
#define OBJECT_TYPE_INTERFACE	0x00001000	//	Object is an Object Pascal Interface (Pascal object)
#define OBJECT_TYPE_CLASS	0x00002000	//	Object is a Class
//---------------------------------------------------------------------------
//	We need to define some structures to hold the information that we require
// to be generated when a new view is need in the ClassExplorer TreeView1 component
// these are various structures that allow all the information needed for the tree
// view but also things like what line the item is at in definition form and in the
// actual source file.
//---------------------------------------------------------------------------
typedef	struct		_common_header_def_	{	//	common interface
	AnsiString	Name;				//	Name of this object
        AnsiString	HeaderFile;			//	header file for this object
        AnsiString	SourceFile;			//	duh!
        DWORD		dwDefLine;			//	Definition line in header file
        DWORD		dwSrcLine;			//	Source line in source file
        DWORD		dwFlags;			//	What type of object is this? See above.
        } ObjectHeader, *lpObjectHeader;
//---------------------------------------------------------------------------
class TClassExplorerForm : public TForm
	{
		__published:	// IDE-managed Components
			TPanel 		*Panel1;
			TBitBtn 	*BitBtn1;
			TBitBtn 	*BitBtn2;
			TBitBtn 	*BitBtn3;
			TPanel 		*Panel2;
			TTreeView 	*TreeView1;
			TToolBar 	*ToolBar1;
			TImageList 	*ImageList1;
			TRichEdit 	*RichEdit1;
			TPopupMenu 	*PopupMenu1;
			TMenuItem 	*GotoDefinition1;
			TMenuItem 	*GotoCode1;
                        TLabel 		*Label1;

			void 	__fastcall 	FormCreate		( TObject *Sender );
			void 	__fastcall 	BitBtn3Click		( TObject *Sender );
			void 	__fastcall 	GotoDefinition1Click	( TObject *Sender );
	void __fastcall FormPaint(TObject *Sender);
		private:	// User declarations
                	// .... global class list, contains all classes in project
                	TList		*lpClassList;
                        DWORD		dwNumClasses;
                        // .... and for routines
                        TList		*lpRoutinesList;
                        DWORD		dwNumRoutines;
                        // .... and variables
                        TList		*lpVariablesList;
                        DWORD		dwNumVariables;
                        // ....	properties
                        TList		*lpPropertiesList;
                        DWORD		dwNumProperties;
		public:		// User declarations
				__fastcall 	TClassExplorerForm	( TComponent* Owner );

                	void	__fastcall	AddType	( DWORD dwType, LPSTR lpszHFile, LPSTR lpszSFile, DWORD dLine, DWORD sLine );
                __published:
                	__property	DWORD	NumClasses    = { read=dwNumClasses,    stored=true, default=0 };
                        __property	DWORD	NumRoutines   = { read=dwNumRoutines,   stored=true, default=0 };
                        __property	DWORD	NumVariables  = { read=dwNumVariables,  stored=true, default=0 };
                        __property	DWORD	NumProperties = { read=dwNumProperties, stored=true, default=0 };
	};
//---------------------------------------------------------------------------
extern PACKAGE TClassExplorerForm *ClassExplorerForm;
//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
