//---------------------------------------------------------------------------
//
//	$FileName	::	DebuggingUnit.h
//	$Author		::	Chris Brooklyn
//	$Version	::	0.0.0.1
//	$Purpose	::	To provide an interface to allow the user
//				to select which particular debugger they wish
//				to use, ie internal or external
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#include "DebuggingUnit.h"
#include "ProjectOptionsFormUnit.h"
#include "Utilities.h"
#include "main.h"
#include "shellapi.h"
//---------------------------------------------------------------------------
#pragma hdrstop
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TDebugOptionsForm       *DebugOptionsForm;
//---------------------------------------------------------------------------
//	Debugger Form Constructor
//---------------------------------------------------------------------------
__fastcall      TDebugOptionsForm :: TDebugOptionsForm( TComponent* Owner ) : TForm( Owner )
{

}
//---------------------------------------------------------------------------
void	__fastcall	TDebugOptionsForm :: Execute( void )
{
	if ( ProjectOptionsForm )	{
        	if ( !ProjectOptionsForm->DebugMode )	{
                	MessageBox( NULL, "Not in Debug Mode, reverting to standard execution process.", Application->Title.c_str(), MB_OK );
                        EmulatorExecute();
                } else {
                	//	EmulatorExecuteDEBUG();
                }
        }
}
//---------------------------------------------------------------------------
int	__fastcall	TDebugOptionsForm :: GDebuggerMode( void )
{
	return( RadioGroup1->ItemIndex );
}
//---------------------------------------------------------------------------
int	__fastcall	TDebugOptionsForm :: GDebuggerType( void )
{
	return( RadioGroup2->ItemIndex );
}
//---------------------------------------------------------------------------
void	__fastcall	TDebugOptionsForm :: SDebuggerMode( int Value )
{
	if ( RadioGroup1->ItemIndex != Value ) {
        	RadioGroup1->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TDebugOptionsForm :: SDebuggerType( int Value )
{
	if ( RadioGroup2->ItemIndex != Value ) {
        	RadioGroup2->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TDebugOptionsForm :: GDebugPath( void )
{
        return( Edit1->Text );
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TDebugOptionsForm :: GDebugFlags( int Index )
{
        if ( ( Index < 0 ) || ( Index > ListView1->Items->Count ) ) return( NULL );

        return( ListView1->Items->Item[ Index ]->Caption );
}
//---------------------------------------------------------------------------
int     __fastcall      TDebugOptionsForm :: GPort( void )
{
        return( ComboBox1->ItemIndex );
}
//---------------------------------------------------------------------------
int     __fastcall      TDebugOptionsForm :: GSpeed( void )
{
        return( ComboBox2->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TDebugOptionsForm :: SDebugPath( AnsiString Value )
{
        if ( Edit1->Text != Value ) {
        	Edit1->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void    __fastcall      TDebugOptionsForm :: SDebugFlags( int Index, AnsiString Value )
{
	if ( ( Index < 0 ) || ( Index > ListView1->Items->Count ) ) return;
        if ( ListView1->Items->Item[ Index ]->Caption != Value ) {
        	ListView1->Items->Item[ Index ]->Caption = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void    __fastcall      TDebugOptionsForm :: SPort( int Value )
{
        if ( ComboBox1->ItemIndex != Value ) {
        	ComboBox1->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void    __fastcall      TDebugOptionsForm :: SSpeed( int Value )
{
        if ( ComboBox2->ItemIndex != Value ) {
        	ComboBox2->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::BitBtn1Click(TObject *Sender)
{
        ModalResult = mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::BitBtn2Click(TObject *Sender)
{
        ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::SpeedButton1Click(TObject *Sender)
{
        OpenDialog1->Filter = "Program Files ( *.exe;*.com;*.bat )|*.exe; *.com; *.bat|DLL Files ( *.dll )|*.dll|Library files ( *.a )|*.a|All Files ( *.* )|*.*";
        OpenDialog1->Title = "Please select a debugger application";
        if ( OpenDialog1->Execute() ) {
                Edit1->Text = OpenDialog1->FileName;
        }
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::ComboBox1Change(TObject *Sender)
{
	switch ( ComboBox1->ItemIndex ) {
                case ( 0 ) :	// Automatic Port Selection
                case ( 7 ) :	// USB Port
                case ( 8 ) : {	// NET/LAN Port
                	ComboBox2->ItemIndex = -1;	// Force to Auto only
                	break;
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::ComboBox2Change(TObject *Sender)
{
	if ( ( ComboBox1->ItemIndex <= 0 ) && ( ComboBox1->ItemIndex >= 7 ) ) {
        	return;
        } else ComboBox2->ItemIndex = -1;
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::RadioGroup2Click(TObject *Sender)
{
	Panel2->Enabled = ( RadioGroup2->ItemIndex == 1 );
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::FormCreate(TObject *Sender)
{
	if ( MainForm->hDebuggerHandle != INVALID_HANDLE_VALUE ) {
        	Panel1->Visible = true;
                Label5->Visible = false;
        }	 else {
        	Panel1->Visible = false;
                Label5->Visible = true;
        }
}
//---------------------------------------------------------------------------
//	Write the current debugging options to the system registry, stored in
// HKEY_CURRENT_USER\Liquid Silicon Developments\PSX IDE For Windows\Debugger
//---------------------------------------------------------------------------
void	__fastcall	TDebugOptionsForm :: WriteToRegistry( TObject *Sender )
{       TComponent	*pComponent = dynamic_cast< TComponent* >(Sender);

	if ( Sender == NULL )	return;

        SystemMessage( "[TDebugOptionsForm::WriteToRegistry] " + pComponent->Name );

        Registry = new TRegistry;

        if ( Registry == NULL )	{
        	SystemMessage( "Registry access failure." );
                return;
        }

        Registry->LazyWrite = false;

        Registry->OpenKey( DEBUGGER, true );

        Registry->WriteInteger( "Mode", DebuggerMode );	//	Integrated or standalone
        Registry->WriteInteger( "Type", DebuggerType );	//	Load or Remote

        Registry->WriteInteger( "Port", DebugPort );
        Registry->WriteInteger( "Speed", DebugSpeed );

        Registry->WriteString( "DebugPath", DebugPath );

        //	Write flags
        Registry->WriteInteger( "NumFlags", ListView1->Items->Count );

        for ( int i=0; i<=ListView1->Items->Count; i++ ) {
        	Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) ), DebugFlags[ i ] );
        }

        Registry->CloseKey();

        //	delete registry access class

        delete Registry;
}
//---------------------------------------------------------------------------
void	__fastcall	TDebugOptionsForm :: ReadFromRegistry( TObject *Sender )
{
}
//---------------------------------------------------------------------------
void __fastcall TDebugOptionsForm::BitBtn3Click(TObject *Sender)
{
	ShellExecute( Handle, "open", "iexplore.exe",
        	"http://localhost/ide/help/HELP_DebugOptions.html",
                NULL, SW_SHOWNORMAL );
}
//---------------------------------------------------------------------------
