//---------------------------------------------------------------------------
//
//      $FileName       ::      EnviromentOptions.h
//      $Author         ::      Chris Brooklyn
//      $Version        ::      0.2.0.220
//      $TimeStamp      ::      28/05/02 12:39
//      $Description    ::      IDE Enviroment Options Configuration Form Code File
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#include "EnviromentOptionsUnit.h"
#include "ChildWin.h"
#include "main.h"
#include "shellapi.h"
#include "TDirSelectDialog.h"
#include "DebuggingUnit.h"
#include "RegEntries.h"
#include "Utilities.h"
#include "ListDialog.h"
#include "ProjectOptionsFormUnit.h"
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "CGRID"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TEnviromentOptionsForm *EnviromentOptionsForm;
//---------------------------------------------------------------------------
const	int	DefaultForeGround = clWhite;
const	int	DefaultBackGround = clBlack;
const   bool    DefaultBold     = false;
const   bool    DefaultItalic   = false;
const   bool    DefaultUnderLine = false;
//const	Element	Default = ( "Default", clBlack, clWhite, false, false, false, true, true );
//---------------------------------------------------------------------------
//	Default Elements
//---------------------------------------------------------------------------
/*
const	Element	ElementWhiteSpace	= { "WhiteSpace",    	clBlack,  clWhite, false, false, false, false, false };
const	Element ElementComment		= { "Comment",       	clBlue,   clWhite, false, false, false, false, false };
const	Element	ElementReservedWord	= { "Reserved Word", 	clBlack,  clWhite, true,  false, false, false, false };
const	Element	ElementIdentifier	= { "Identifier", 	clBlack,  clWhite, true,  false, false, false, false };
const	Element	ElementSymbol		= { "Symbol", 		clBlack,  clWhite, false, false, false, false, false };
const	Element	ElementString		= { "String", 		clBlack,  clWhite, false, false, false, false, false };
const	Element	ElementInteger		= { "Integer", 		clRed,    clWhite, false, false, false, false, false };
const	Element	ElementFloat		= { "Float", 		clRed,    clWhite, false, false, false, false, false };
const	Element	ElementOctal		= { "Octal", 		clRed,    clWhite, false, false, false, false, false };
const	Element	ElementHex		= { "Hex", 		clRed,    clWhite, false, false, false, false, false };
const	Element	ElementCharacter	= { "Character", 	clBlack,  clWhite, false, false, false, false, false };
const	Element	ElementPreprocessor	= { "Preprocessor", 	glGreen,  clWhite, false, false, false, false, false };
const	Element	ElementIllegalChar	= { "Illegal Char", 	clBlack,  clRed,   true,  false, true,  false, false };
const	Element	ElementAssembler	= { "Assembler", 	clYellow, clWhite, false, true,  false, false, false };
const	Element	ElementHotLink		= { "Hot Link", 	clPurple, clWhite, false, false, true,  false, false };
const	Element	ElementPlainText	= { "Plain Text", 	clBlack,  clWhite, false, false, false, false, false };
const	Element	ElementSearchMatch	= { "Search Match", 	clWhite,  clBlack, true,  false, false, false, false };
const	Element	ElementExecutionPoint	= { "Execution Point", 	clWhite,  clRed,   true,  false, true,  false, false };
const	Element	ElementEnabledBreak	= { "Enabled Break", 	clGreen,  clRed,   false, false, false, false, false };
const	Element	ElementDisabledBreak	= { "Disabled Break", 	clYellow, clRed,   false, false, false, false, false };
const	Element	ElementErrorLine	= { "Error Line", 	clRed,    clBlack, true,  true,  false, false, false };
const	Element	ElementRightMargin	= { "Right Margin", 	clGray,   clWhite, false, false, false, false, false };
//---------------------------------------------------------------------------
const	LPElement	DefaultElements[] = {
				ElementWhiteSpace, ElementComment, ElementReservedWord, ElementIdentifier,
                                ElementSymbol, ElementString, ElementInteger, ElementFloat, ElementOctal,
                                ElementHex, ElementCharacter, ElementPreprocessor, ElementIllegalChar,
                                ElementAssembler, ElementHotLink, ElementPlainText, ElementExecutionPoint,
                                ElementEnabledBreak, ElmentDisabledBreak, ElementErrorLine, ElementRightMargin
			};
*/
//---------------------------------------------------------------------------
//      Enviroment options form constructor. Does not do anything, most on the
// work is done when the form is created. See the FormCreate event method
//---------------------------------------------------------------------------
__fastcall 	TEnviromentOptionsForm :: TEnviromentOptionsForm( TComponent* Owner ) : TForm( Owner )
{
}
//---------------------------------------------------------------------------
//	Writes the current enviroment settings to the system registry
//---------------------------------------------------------------------------
int	__fastcall	TEnviromentOptionsForm :: WriteToRegistry( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )
                return( E_BAD_SENDER );	        //	Bad call

        SystemMessage( "Writing Enviroment Options to system registry" );

        Registry = new TRegistry;
         Registry->LazyWrite = false;	        //	Forces quick updating of registry, slows app down though :(

        //	Write the preferences tab sheet to the registry
        Registry->OpenKey( ENV_OPTIONS_PREFERENCES, true );	//	Open key, create if not already exists

	Registry->WriteBool( "ShowCompilerProgress", ShowCompilerProgress );
        Registry->WriteBool( "BeepOnCompletion", BeepOnCompletion );
        Registry->WriteBool( "CacheHeadersOnStartup", CacheHeadersOnStartup );
        Registry->WriteBool( "WarnOnPackageRebuild", WarnOnPackageRebuild );
        Registry->WriteBool( "AutoSaveEditorFiles", AutoSaveEditorFiles );
        Registry->WriteBool( "AutoSaveDesktopFiles", AutoSaveDesktopFiles );
        Registry->WriteBool( "HideOnRun", HideOnRun );
        Registry->WriteBool( "MinimiseOnRun", MinimiseOnRun );

        Registry->WriteString( "SharedRepository", SharedRepository );

        Registry->WriteString( "ToolsRepository", ToolsRepository );

        Registry->CloseKey();

        //	Write the Library Tabsheet
        Registry->OpenKey( ENV_OPTIONS_LIBRARY, true );	//	Open key

        Registry->WriteString( "LibraryPath", LibraryPath );
        Registry->WriteString( "PackageOutputPath", PackageOutput );
        Registry->WriteString( "LibraryOutputPath", LibraryOutput );
        Registry->WriteString( "BrowsingPath", BrowsingPath );

        Registry->WriteString( "Emulator", Emulator );
        Registry->WriteString( "AudioEditor", AudioEditor );
        Registry->WriteString( "Gfx2DEditor", Gfx2DEditor );
        Registry->WriteString( "Gfx3DEditor", Gfx3DEditor );
        Registry->WriteString( "MovieEditor", MovieEditor );
        Registry->WriteString( "CDBurner", CDBurner );

        Registry->CloseKey();

        //	Write the editor tabsheet
        Registry->OpenKey( ENV_OPTIONS_EDITOR, true );

        Registry->WriteBool( "AutoIndentMode", AutoIndentMode );
        Registry->WriteBool( "InsertMode", InsertMode );
        Registry->WriteBool( "UseTabCharacter", UseTabCharacter );
        Registry->WriteBool( "SmartTab", SmartTab );
        Registry->WriteBool( "OptimalFill", OptimalFill );
        Registry->WriteBool( "BackspaceUnindents", BackspaceUnindents );
        Registry->WriteBool( "CursorThroughTabs", CursorThroughTabs );
        Registry->WriteBool( "GroupUndo", GroupUndo );
        Registry->WriteBool( "CursorBeyondEOF", CursorBeyondEOF );
        Registry->WriteBool( "UndoAfterSave", UndoAfterSave );
        Registry->WriteBool( "KeepTrailingBlanks", KeepTrailingBlanks );
        Registry->WriteBool( "PersistentBlocks", PersistentBlocks );
        Registry->WriteBool( "OverwriteBlocks", OverwriteBlocks );
        Registry->WriteBool( "DoubleClickLine", DoubleClickLine );
        Registry->WriteBool( "FindTextAtCursor", FindTextAtCursor );
        Registry->WriteBool( "ForceCutCopyEnabled", ForceCutCopyEnabled );
        Registry->WriteBool( "UseSyntaxHighlighting", UseSyntaxHighlighting );

        Registry->CloseKey();

        //	Write the display tabsheet
        Registry->OpenKey( ENV_OPTIONS_DISPLAY, true );

        Registry->WriteBool( "BRIEFCursorShapes", BRIEFCursorShapes );
        Registry->WriteBool( "CreateBackupFiles", CreateBackupFiles );
        Registry->WriteBool( "PreserveLineFeeds", KeepTrailingBlanks );
        Registry->WriteBool( "ZoomToFullScreen", ZoomToFullScreen );
        Registry->WriteBool( "VisibleRightMargin", VisibleRightMargin );
        Registry->WriteBool( "VisibleGutter", VisibleGutter );
        Registry->WriteInteger( "RightMargin", RightMargin );
        Registry->WriteInteger( "GutterWidth", GutterWidth );
        Registry->WriteInteger( "FontSize", FontSize );
        Registry->WriteString( "FontFace", FontFace );

        Registry->CloseKey();

        //	Write the colours tabsheet

        Registry->OpenKey( ENV_OPTIONS_ELEMENTS, true );

        Registry->WriteInteger( "NumElements", NumElements );

        if ( StyleName != ComboBox3->Text )     {
                //      Style is wrong
                StyleName = ComboBox3->Text;
        }

        Registry->WriteString( "DefaultStyle", StyleName );

        Registry->CloseKey();


        for ( int i=0; i<=NumElements; i++ ) {
        	AnsiString	asKey = AnsiString( ENV_OPTIONS_ELEMENTS + "\\" + StyleName + "\\" +
                                                ListView4->Items->Item[ i ]->Caption );

		Registry->OpenKey( asKey, true );

                SystemMessage( "Writing values for element " + ListView4->Items->Item[ i ]->Caption );

	        //Registry->WriteString( "Name", pElements[ i ].Name );

	        Registry->WriteInteger( "ForeGround", pElements[ i ].ForeGround );
	        Registry->WriteInteger( "BackGround", pElements[ i ].BackGround );

	        Registry->WriteBool( "Bold", pElements[ i ].Bold );
	        Registry->WriteBool( "Italic", pElements[ i ].Italic );
	        Registry->WriteBool( "Underline", pElements[ i ].UnderLine );
	        Registry->WriteBool( "ForeDefault", pElements[ i ].DefaultFore );
	        Registry->WriteBool( "BackDefault", pElements[ i ].DefaultBack );

                Registry->CloseKey();
                asKey = NULL;
        }

        //	palette
        Registry->OpenKey( ENV_OPTIONS_PALETTE, true );

        //	PackageManager->SaveToRegistry( Sender );

        Registry->CloseKey();

        //	code insight
        Registry->OpenKey( ENV_OPTIONS_CODE_INSIGHT, true );

        Registry->WriteBool( "CodeCompletion", CodeCompletion );
        Registry->WriteBool( "CodeParameters", CodeParameters );
        Registry->WriteBool( "TooltipExpressions", TooltipExpressions );
        Registry->WriteInteger( "AutoCodeDelay", AutoCodeDelay );

        Registry->CloseKey();

        Registry->OpenKey( ENV_OPTIONS_CODE_TEMPLATES, true );

        Registry->WriteInteger( "NumTemplates", NumCodeTemplates );

        Registry->CloseKey();

        for ( int i=0; i<=NumCodeTemplates; i++ ) {
        	TListItem	*pItem = ListView1->Items->Item[ i ];

                if ( pItem != NULL ) {
                	AnsiString	KeyName = AnsiString( ENV_OPTIONS_CODE_TEMPLATES + "\\" + pItem->Caption );

                        if ( KeyName.Length() > 0 ) {
                        	Registry->OpenKey( KeyName, true );

                                Registry->WriteString( "Description", pItem->SubItems->Strings[ 0 ] );
                                Registry->WriteString( "File", pItem->SubItems->Strings[ 1 ] );

                                Registry->CloseKey();
                                KeyName = NULL;
                        }
                }
        }

        SystemMessage( "Enviroment options written to system registry. OK." );

        delete Registry;

        return( E_NOERROR );
}
//---------------------------------------------------------------------------
int	__fastcall	TEnviromentOptionsForm :: ReadFromRegistry( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( !Originator ) 	return( E_BAD_SENDER );	//	Bad call

        SystemMessage( "Reading Enviroment Options from system registry" );

        Registry = new TRegistry;
        Registry->LazyWrite = false;	//	Forces quick updating of registry, slows app down though :(

        //	Write the preferences tab sheet to the registry
        if ( !Registry->OpenKey( ENV_OPTIONS_PREFERENCES, false ) ) {	//	Open key, create if not already exists
        	SystemMessage( "Unable to open registry key " + ENV_OPTIONS_PREFERENCES );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

	ShowCompilerProgress  = Registry->ReadBool( "ShowCompilerProgress" );
	BeepOnCompletion      = Registry->ReadBool( "BeepOnCompletion" );
	CacheHeadersOnStartup = Registry->ReadBool( "CacheHeadersOnStartup" );
	WarnOnPackageRebuild  = Registry->ReadBool( "WarnOnPackageRebuild" );
	AutoSaveEditorFiles   = Registry->ReadBool( "AutoSaveEditorFiles" );
	AutoSaveDesktopFiles  = Registry->ReadBool( "AutoSaveDesktopFiles" );
	HideOnRun             = Registry->ReadBool( "HideOnRun" );
        MinimiseOnRun 	      = Registry->ReadBool( "MinimiseOnRun" );

        SharedRepository = Registry->ReadString( "SharedRepository" );

        ToolsRepository = Registry->ReadString( "ToolsRepository" );

        Registry->CloseKey();

        //	Write the Library Tabsheet
        if ( !Registry->OpenKey( ENV_OPTIONS_LIBRARY, false ) ) {	//	Open key
        	SystemMessage( "Unable to open system registry key " + ENV_OPTIONS_LIBRARY );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

        LibraryPath   = Registry->ReadString( "LibraryPath" );
        PackageOutput = Registry->ReadString( "PackageOutputPath" );
        LibraryOutput = Registry->ReadString( "LibraryOutputPath" );
        BrowsingPath  = Registry->ReadString( "BrowsingPath" );

        Emulator = Registry->ReadString( "Emulator" );
        AudioEditor = Registry->ReadString( "AudioEditor" );
        Gfx2DEditor = Registry->ReadString( "Gfx2DEditor" );
        Gfx3DEditor = Registry->ReadString( "Gfx3DEditor" );
        MovieEditor = Registry->ReadString( "MovieEditor" );
        CDBurner = Registry->ReadString( "CDBurner" );

        Registry->CloseKey();

        //	Write the editor tabsheet
        if ( !Registry->OpenKey( ENV_OPTIONS_EDITOR, true ) ) {	//	Open Key for editor options
        	SystemMessage( "Unable to open system registry key " + ENV_OPTIONS_EDITOR );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

	AutoIndentMode        = Registry->ReadBool( "AutoIndentMode" );
	InsertMode            = Registry->ReadBool( "InsertMode" );
	UseTabCharacter       = Registry->ReadBool( "UseTabCharacter" );
	SmartTab              = Registry->ReadBool( "SmartTab" );
	OptimalFill           = Registry->ReadBool( "OptimalFill" );
	BackspaceUnindents    = Registry->ReadBool( "BackspaceUnindents" );
	CursorThroughTabs     = Registry->ReadBool( "CursorThroughTabs" );
	GroupUndo             = Registry->ReadBool( "GroupUndo" );
	CursorBeyondEOF       = Registry->ReadBool( "CursorBeyondEOF" );
	UndoAfterSave         = Registry->ReadBool( "UndoAfterSave" );
	KeepTrailingBlanks    = Registry->ReadBool( "KeepTrailingBlanks" );
	PersistentBlocks      = Registry->ReadBool( "PersistentBlocks" );
	OverwriteBlocks       = Registry->ReadBool( "OverwriteBlocks" );
	DoubleClickLine       = Registry->ReadBool( "DoubleClickLine" );
        FindTextAtCursor      = Registry->ReadBool( "FindTextAtCursor" );
	ForceCutCopyEnabled   = Registry->ReadBool( "ForceCutCopyEnabled" );
	UseSyntaxHighlighting = Registry->ReadBool( "UseSyntaxHighlighting" );

        Registry->CloseKey();

        //	Write the display tabsheet
        if ( !Registry->OpenKey( ENV_OPTIONS_DISPLAY, true ) )	{	//	Open key for display settings
		SystemMessage( "Unable to open system registry key " + ENV_OPTIONS_DISPLAY );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

	BRIEFCursorShapes  = Registry->ReadBool( "BRIEFCursorShapes" );
	CreateBackupFiles  = Registry->ReadBool( "CreateBackupFiles" );
	KeepTrailingBlanks = Registry->ReadBool( "PreserveLineFeeds" );
	ZoomToFullScreen   = Registry->ReadBool( "ZoomToFullScreen" );
	VisibleRightMargin = Registry->ReadBool( "VisibleRightMargin" );
	VisibleGutter      = Registry->ReadBool( "VisibleGutter" );
	RightMargin        = Registry->ReadInteger( "RightMargin" );
	GutterWidth        = Registry->ReadInteger( "GutterWidth" );
	FontSize           = Registry->ReadInteger( "FontSize" );
	FontFace           = Registry->ReadString( "FontFace" );

        Registry->CloseKey();

        //	Write the colours tabsheet
        if ( !Registry->OpenKey( ENV_OPTIONS_ELEMENTS, true ) ) {	//	Open key for colour settings for syntax highlighting
        	SystemMessage( "Unable to open system registry key " + ENV_OPTIONS_ELEMENTS );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

        NumElements = Registry->ReadInteger( "NumElements" );

        // For some reason the compiler thinks that we are trying to call
        // this->WriteToRegistry() for some stupid reason ?

        AnsiString Style = Registry->ReadString( "DefaultStyle" );

        if ( ComboBox3->Items->IndexOf( Style ) == -1 ) {
        	SystemMessage( "Style is not known, skipping." );
                ComboBox3->ItemIndex = -1;
        } else {
        	SystemMessage( "configuring for style " + Style );

                ComboBox3->ItemIndex = ComboBox3->Items->IndexOf( Style );

        if ( NumElements <= 0 ) {
        	SystemMessage( "Registry failure on NumElements @ " + ENV_OPTIONS_ELEMENTS );
                Registry->CloseKey();
                delete Registry;
                return( E_BAD_REGISTRY );
        }

        Registry->CloseKey();

        if ( pElements == NULL ) {
        	pElements = new Element[ NumElements ];
        }

        ZeroMemory( pElements, sizeof( Element ) * NumElements );

	for ( int i=0; i<NumElements; i++ ) {
		AnsiString	asKey = AnsiString( ENV_OPTIONS_ELEMENTS + "\\" + StyleName + "\\" + ListView4->Items->Item[ i ]->Caption );

		Registry->OpenKey( asKey, true );

                asKey.Delete( 0, asKey.Length() );

		pElements[ i ].Name        = ListView4->Items->Item[ i ]->Caption;

		pElements[ i ].ForeGround  = Registry->ReadInteger( "ForeGround" );
		pElements[ i ].BackGround  = Registry->ReadInteger( "BackGround" );
		pElements[ i ].Bold        = Registry->ReadBool( "Bold" );
		pElements[ i ].Italic      = Registry->ReadBool( "Italic" );
		pElements[ i ].UnderLine   = Registry->ReadBool( "Underline" );
		pElements[ i ].DefaultFore = Registry->ReadBool( "ForeDefault" );
		pElements[ i ].DefaultBack = Registry->ReadBool( "BackDefault" );

		Registry->CloseKey();
	}

        }

        //	palette
        if ( !Registry->OpenKey( ENV_OPTIONS_PALETTE, true ) ) {
        	SystemMessage( "Unable to open system registry key " + ENV_OPTIONS_PALETTE );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

        Registry->CloseKey();

        //	code insight
        if ( !Registry->OpenKey( ENV_OPTIONS_CODE_INSIGHT, true ) ) {
        	SystemMessage( "Unable to open system registry key " + ENV_OPTIONS_CODE_INSIGHT );
                delete Registry;
                return( E_BAD_REGISTRY );
        }

        CodeCompletion = Registry->ReadBool( "CodeCompletion" );
        CodeParameters = Registry->ReadBool( "CodeParameters" );
        TooltipExpressions = Registry->ReadBool( "TooltipExpressions" );
        AutoCodeDelay = Registry->ReadInteger( "AutoCodeDelay" );

        Registry->CloseKey();

        //	code insight templates
        Registry->OpenKey( ENV_OPTIONS_CODE_TEMPLATES, true );

        //ListView1->Items->Count = Registry->ReadInteger( "NumTemplates" );

        //for ( int i=0; i<=NumCodeTemplates; i++ ) {
        //	TListItem *pItem = ListView1->Items->Add();

        //        if ( pItem != NULL ) {
        //        	pItem->Caption = Registry->ReadString( "CTemplate#" + IntToStr( i ) + "Name" );
        //                pItem->SubItems->Add( Registry->ReadString( "CTemplate#" + IntToStr( i ) + "Text" ) );
        //        }
        //}

        Registry->CloseKey();

        delete Registry;

        return( E_NOERROR );
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GBold( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].Bold );
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GItalic( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].Italic );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GCodeCompletion( void )
{
	return( CheckBox31->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GCodeParameters( void )
{
	return( CheckBox32->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GTooltipExpressions( void )
{
	return( CheckBox33->Checked );
}
//---------------------------------------------------------------------------
WORD		__fastcall	TEnviromentOptionsForm :: GAutoCodeDelay( void )
{
	return( TrackBar1->Position );
}
//---------------------------------------------------------------------------
int		__fastcall	TEnviromentOptionsForm :: GNumTemplates	( void )
{
	return( ListView1->Items->Count );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GTemplate( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumCodeTemplates ) )
        	return( NULL );

        return( ListView1->Items->Item[ Index ]->Caption );
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SCodeCompletion( bool Value )
{
	if ( CheckBox31->Checked != Value ) {
        	CheckBox31->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SCodeParameters( bool Value )
{
	if ( CheckBox32->Checked != Value ) {
        	CheckBox32->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: STooltipExpressions( bool Value )
{
	if ( CheckBox33->Checked != Value ) {
        	CheckBox33->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SAutoCodeDelay( WORD Value )
{
	if ( Value > TrackBar1->Max ) return;

        if ( TrackBar1->Position != Value ) {
        	TrackBar1->Position = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GUnderline( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].UnderLine );
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GForeDefault( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].DefaultFore );
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GBackDefault( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].DefaultBack );
}
//---------------------------------------------------------------------------
int	__fastcall	TEnviromentOptionsForm :: GForeColour( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].ForeGround );
}
//---------------------------------------------------------------------------
int	__fastcall	TEnviromentOptionsForm :: GBackColour( int Index )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return( NULL );

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return( NULL );
        }

        return( pElements[ Index ].BackGround );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SBold( int Index, bool Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].Bold != Value )	pElements[ Index ].Bold = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SItalic( int Index, bool Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].Italic != Value ) pElements[ Index ].Italic = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SUnderline( int Index, bool Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].UnderLine != Value )	pElements[ Index ].UnderLine = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SForeDefault( int Index, bool Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].DefaultFore != Value )	pElements[ Index ].DefaultFore = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SBackDefault( int Index, bool Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].DefaultBack != Value )	pElements[ Index ].DefaultBack = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SForeColour( int Index, int Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].ForeGround != Value )	pElements[ Index ].ForeGround = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SBackColour( int Index, int Value )
{
	if ( ( Index < 0 ) || ( Index > NumElements ) )	return;

        if ( !pElements )	{
        	SystemMessage( "MAJOR INTERNAL ERROR 'pElements' NOT DEFINED." );
                return;
        }

        if ( pElements[ Index ].BackGround != Value )	pElements[ Index ].BackGround = Value;
}
//---------------------------------------------------------------------------
bool    __fastcall      TEnviromentOptionsForm :: GModified( void )
{
        if ( EnviromentOptionsForm )	return( bModified );
        else 	return( NULL );
}
//---------------------------------------------------------------------------
void    __fastcall      TEnviromentOptionsForm :: SModified( bool Value )
{
        if ( bModified != Value )	bModified = Value;
}
//---------------------------------------------------------------------------
//	PREFERENCES TABSHEET	Settings
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GShowCompilerProgress( void )
{
	return( CheckBox1->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SShowCompilerProgress( bool Value )
{
	if ( ShowCompilerProgress != Value ) {
        	CheckBox1->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GBeepOnCompletion( void )
{
	return( CheckBox2->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SBeepOnCompletion( bool Value )
{
	if ( CheckBox2->Checked != Value ) {
        	CheckBox2->Checked = Value;
                Modified = false;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GCacheHeadersOnStartup( void )
{
	return( CheckBox3->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SCacheHeadersOnStartup( bool Value )
{
	if ( CheckBox3->Checked != Value ) {
        	CheckBox3->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GWarnOnPackageRebuild( void )
{
	return( CheckBox4->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SWarnOnPackageRebuild( bool Value )
{
	if ( CheckBox4->Checked != Value ) {
        	CheckBox4->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GAutoSaveEditorFiles( void )
{
	return( CheckBox5->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SAutoSaveEditorFiles( bool Value )
{
	if ( CheckBox5->Checked != Value ) {
        	CheckBox5->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GAutoSaveDesktopFiles( void )
{
        if ( CheckBox6 )	return( CheckBox6->Checked );
        else 			return( false );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SAutoSaveDesktopFiles( bool Value )
{
	if ( CheckBox6->Checked != Value ) {
        	CheckBox6->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GMinimiseOnRun( void )
{
	return( CheckBox7->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SMinimiseOnRun( bool Value )
{
	if ( CheckBox7->Checked != Value ) {
        	CheckBox7->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GHideOnRun( void )
{
	return( CheckBox8->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SHideOnRun( bool Value )
{
	if ( CheckBox8->Checked != false ) {
        	CheckBox8->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GSharedRepository( void )
{
	return( Edit1->Text );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SSharedRepository( AnsiString Value )
{
	if ( Edit1->Text != Value ) {
        	Edit1->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
//	LIBRARY TABSHEET	Settings
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GSystemLibraryPath( void )
{
	return( Edit2->Text );
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SSystemLibraryPath( AnsiString Value )
{
	if ( Edit2->Text != Value ) {
        	Edit2->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GSystemPackageOutputPath( void )
{
	return( Edit3->Text );
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SSystemPackageOutputPath( AnsiString Value )
{
	if ( Edit3->Text != Value ) {
        	Edit3->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GSystemLibraryOutputPath( void )
{
	return( Edit4->Text );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SSystemLibraryOutputPath( AnsiString Value )
{
	if ( Edit4->Text != Value ) {
        	Edit4->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GSystemBrowsingPath( void )
{
	return( Edit5->Text );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SSystemBrowsingPath( AnsiString Value )
{
	if ( Edit5->Text != Value ) {
        	Edit5->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GAutoIndentMode( void )
{
	return( CheckBox9->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SAutoIndentMode( bool Value )
{
	if ( CheckBox9->Checked != Value ) {
        	CheckBox9->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GInsertMode( void )
{
	if ( EnviromentOptionsForm )	return( CheckBox10->Checked );
        else return( false );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SInsertMode( bool Value )
{
	if ( CheckBox10->Checked != Value ) {
        	CheckBox10->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm ::GUseTabChar( void )
{
	return( CheckBox11->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SUseTabChar( bool Value )
{
	if ( CheckBox11->Checked != Value ) {
        	CheckBox11->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GSmartTab( void )
{
	return( CheckBox12->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SSmartTab( bool Value )
{
	if ( CheckBox12->Checked != Value ) {
        	CheckBox12->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GOptimalFill( void )
{
	return( CheckBox13->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SOptimalFill( bool Value )
{
	if ( CheckBox13->Checked != Value ) {
        	CheckBox13->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GBackspaceUnindents( void )
{
	return( CheckBox14->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SBackspaceUnindents( bool Value )
{
	if ( CheckBox14->Checked != Value ) {
        	CheckBox14->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GCursorThroughTabs( void )
{
	return( CheckBox15->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SCursorThroughTabs( bool Value )
{
	if ( CheckBox15->Checked != Value ) {
        	CheckBox15->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GGroupUndo( void )
{
	return( CheckBox15->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SGroupUndo( bool Value )
{
	if ( CheckBox15->Checked != Value ) {
        	CheckBox15->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GCursorBeyondEOF( void )
{
	return( CheckBox16->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SCursorBeyondEOF( bool Value )
{
	if ( CheckBox17->Checked != Value ) {
        	CheckBox17->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GUndoAfterSave( void )
{
	return( CheckBox18->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SUndoAfterSave( bool Value )
{
	if ( CheckBox18->Checked != Value ) {
        	CheckBox18->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GKeepTrailingBlanks( void )
{
	return( CheckBox19->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SKeepTrailingBlanks( bool Value )
{
	if ( CheckBox19->Checked != Value ) {
        	CheckBox19->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GPersistentBlocks( void )
{
	return( CheckBox20->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SPersistentBlocks( bool Value )
{
	if ( CheckBox20->Checked != Value ) {
        	CheckBox20->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GOverwriteBlocks( void )
{
	return( CheckBox21->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SOverwriteBlocks( bool Value )
{
	if ( CheckBox21->Checked != Value ) {
        	CheckBox21->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GDoubleClickLine( void )
{
	return( CheckBox22->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SDoubleClickLine( bool Value )
{
	if ( CheckBox22->Checked != Value ) {
        	CheckBox22->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GFindTextAtCursor( void )
{
	return( CheckBox23->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SFindTextAtCursor( bool Value )
{
	if ( CheckBox23->Checked != Value ) {
        	CheckBox23->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GForceCutCopyEnabled( void )
{
	return( CheckBox24->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SForceCutCopyEnabled( bool Value )
{
	if ( CheckBox24->Checked != Value ) {
        	CheckBox24->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TEnviromentOptionsForm :: GUseSyntaxHighlighting( void )
{
	return( CheckBox25->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SUseSyntaxHighlighting( bool Value )
{
	if ( CheckBox25->Checked != Value ) {
        	CheckBox25->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GBRIEFCursorShapes	( void )
{
	return( CheckBox26->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GCreateBackupFiles	( void )
{
	return( CheckBox27->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GPreserveLineEnds	( void )
{
	return( CheckBox28->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GZoomToFullScreen	( void )
{
	return( CheckBox29->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GVisibleRightMargin	( void )
{
	return( CheckBox30->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TEnviromentOptionsForm :: GVisibleGutter		( void )
{
	return( CheckBox31->Checked );
}
//---------------------------------------------------------------------------
int		__fastcall	TEnviromentOptionsForm :: GRightMargin		( void )
{
	return( StrToInt( Edit8->Text ) );
}
//---------------------------------------------------------------------------
int		__fastcall	TEnviromentOptionsForm :: GGutterWidth		( void )
{
	return( UpDown1->Position );
}
//---------------------------------------------------------------------------
int		__fastcall	TEnviromentOptionsForm :: GFontSize		( void )
{
	return( UpDown2->Position );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GFontFace		( void )
{
	return( Edit7->Text );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GKeystrokeMapping	( void )
{
	return( "Default" );
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SBRIEFCursorShapes	( bool Value )
{
	if ( CheckBox26->Checked != Value ) {
        	CheckBox26->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SCreateBackupFiles	( bool Value )
{
	if ( CheckBox27->Checked != Value ) {
        	CheckBox27->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SPreserveLineEnds	( bool Value )
{
	if ( CheckBox28->Checked != Value ) {
        	CheckBox28->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SZoomToFullScreen	( bool Value )
{
	if ( CheckBox29->Checked != Value ) {
        	CheckBox29->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SVisibleRightMargin	( bool Value )
{
	if ( CheckBox30->Checked != Value ) {
        	CheckBox30->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SVisibleGutter		( bool Value )
{	TMDIChild	*pForm = (TMDIChild*)MDIChildren[ 0 ];
	if ( CheckBox31->Checked != Value ) {
        	CheckBox31->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SRightMargin		( int Value )
{
	if ( UpDown1->Position != Value ) {
        	UpDown1->Position = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SGutterWidth		( int Value )
{
	if ( UpDown2->Position != Value ) {
        	UpDown2->Position = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SFontSize		( int Value )
{
	if ( UpDown3->Position != Value ) {
        	UpDown3->Position = Value;
                Font->Size = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SFontFace		( AnsiString Value )
{
	if ( Font->Name != Value ) {
        	Font->Name = Value;
                Edit7->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TEnviromentOptionsForm :: SKeystrokeMapping	( AnsiString Value )
{
}
//---------------------------------------------------------------------------
//      Called for TPageControl->ActivePage = DisplayTab
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::DisplayShow(TObject *Sender)
{
        if ( Label6->Font != FontDialog1->Font ) {
                Modified = true;
                Label6->Font = FontDialog1->Font;
        }
        Update();
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::UpDown1Click(TObject *Sender,
      TUDBtnType Button)
{
        if ( Label6->Font->Size != UpDown1->Position ) {
                Label6->Font->Size = UpDown1->Position;
                Modified = true;
        }
        Update();
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton2Click(TObject *Sender)
{
        ListEditDialog->Caption = "Please select an include directory for Libraries";

        ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = LibraryPath;
        ListEditDialog->ShowModal();

        if ( ListEditDialog->ModalResult != mrOk ) 	return;

        LibraryPath = ListEditDialog->Items;
        Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton3Click(TObject *Sender)
{
        ListEditDialog->Caption = "Please select an include directory for Package Output";

        ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = PackageOutput;
        ListEditDialog->ShowModal();

        if ( ListEditDialog->ModalResult != mrOk ) 	return;

        PackageOutput = ListEditDialog->Items;
        Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton4Click(TObject *Sender)
{
        ListEditDialog->Caption = "Please select an include directory for Library Output";

        ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = LibraryOutput;
        ListEditDialog->ShowModal();

        if ( ListEditDialog->ModalResult != mrOk ) 	return;

        LibraryOutput = ListEditDialog->Items;
        Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton5Click(TObject *Sender)
{

        ListEditDialog->Caption = "Please select an include directory for Browsing Path";

        ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = BrowsingPath;
        ListEditDialog->ShowModal();

        if ( ListEditDialog->ModalResult != mrOk ) 	return;

        BrowsingPath = ListEditDialog->Items;
        Modified = true;

}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton1Click(TObject *Sender)
{
	if ( !Sender )	return;

        ListEditDialog->Caption = "Please select a directory or set of directories for shared Repository";

        ListEditDialog->ListView1->Items->Clear();			//	Clear any previous junk from array
        ListEditDialog->Items = SharedRepository;			//	Set to any current directory
        ListEditDialog->ShowModal();					//	Display the form

        if ( ListEditDialog->ModalResult != mrOk )	return;		//	User cancelled request

        SharedRepository = ListEditDialog->Items;			//	else change the entries

        Modified = true;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::BitBtn1Click(TObject *Sender)
{
        ModalResult = mrOk;
        Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::BitBtn2Click(TObject *Sender)
{
        ModalResult = mrCancel;
        Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::FormCreate(TObject *Sender)
{
	//	Read the current configuration from the registry
        //      Installation Check, used because application will critical fail if
        // certain keys are not present in the system registry
        Registry = new TRegistry;

        if ( !Registry->OpenKey( PSXIDE_REGKEY, false ) )       {
                RepairRegistry();       //      Key not found, repair
        } else
                Registry->CloseKey();

        delete Registry;

	if ( ReadFromRegistry( Sender ) != E_NOERROR ) {
		MessageBox( Application->Handle,
			AnsiString( "Unable to read 'Enviroment Options' from system registry." ).c_str(),
			Application->Title.c_str(), MB_OK | MB_ICONEXCLAMATION );
	}

	Modified = true;

	Font = MainForm->Font;
	Font->Size = 9;

	if ( !pElements )	{
		NumElements = ListView4->Items->Count;	//	Number of elements in the colours listview component
		pElements = new Element[ NumElements ];
		memset( pElements, 0, sizeof( Element ) * NumElements );
	}

	//	Check for the presence of the debugger and emulator

	if ( MainForm->hDebuggerHandle != INVALID_HANDLE_VALUE )	{	//	Debugger present ?
		TListItem *pItemDEBUG = ListView3->Items->Add();
		pItemDEBUG->Caption = "Internal Target Debugger";
	}

	if ( MainForm->hEmulatorHandle != INVALID_HANDLE_VALUE )	{	//	Emulator present ?
		TListItem *pItemEMU = ListView3->Items->Add();
		pItemEMU->Caption = "Internal Target Emulator";
		Emulator = "Internal";
	} else  Emulator = "";

        if ( MainForm->hCompilerHandle != INVALID_HANDLE_VALUE )	{	//	Compiler present ?
        	TListItem *pItemCC = ListView3->Items->Add();
                pItemCC->Caption = "GNU Compiler Set";
        }

        Modified = false;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::FormDestroy(TObject *Sender)
{
	//	Save current configuration to registry
        if ( ( AutoSaveDesktopFiles ) && ( Modified ) )	{
        	//WriteToRegistry( Sender );
                Modified = false;
        }
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton6Click(TObject *Sender)
{
        if ( FontDialog1->Execute() ) {
                Label6->Font->Assign( FontDialog1->Font );
                Edit7->Text = FontDialog1->Font->Name;
                UpDown1->Position = (short)FontDialog1->Font->Size;
                Modified = true;
        } else Modified = false;
        Update();
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::BitBtn7Click(TObject *Sender)
{
	//	User wishes to add a plugin (dll)
        if ( OpenDialog->Execute() )	{
        	TListItem	*pItem = ListView3->Items->Add();

        	SystemMessage( "NOT IMPLEMENTED YET.\nUser adding library " + OpenDialog->FileName );
                pItem->Caption = OpenDialog->FileName;
        }
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::BitBtn8Click(TObject *Sender)
{
	if ( !ListView3->Selected )	{
        	SystemMessage( "To delete a plugin select the desired plugin and click on the 'DELETE' button." );
                return;
        }

        int mrCode = MessageBox( Application->Handle,
        	AnsiString( "Delete plugin '" + ListView3->Selected->Caption + "' ?" ).c_str(),
                Application->Title.c_str(), MB_YESNO | MB_ICONQUESTION );

        if ( mrCode == mrNo )	return;

        ListView3->Items->Delete( ListView3->Selected->Index );
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::ListView4Click(TObject *Sender)
{
	if ( !ListView4->Selected ) return;

        CColorGrid1->ForegroundIndex = pElements[ ListView4->Selected->Index ].ForeGround;
        CColorGrid1->BackgroundIndex = pElements[ ListView4->Selected->Index ].BackGround;

        CheckBox30->Checked = pElements[ ListView4->Selected->Index ].Bold;
        CheckBox36->Checked = pElements[ ListView4->Selected->Index ].Italic;
        CheckBox37->Checked = pElements[ ListView4->Selected->Index ].UnderLine;

        CheckBox38->Checked = pElements[ ListView4->Selected->Index ].DefaultFore;
        CheckBox39->Checked = pElements[ ListView4->Selected->Index ].DefaultBack;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton7Click(TObject *Sender)
{	//	Changes the tools repository, ie where the tools are located
	//	note these are user supplied tools
        if ( !Sender )	return;
        ListEditDialog->Caption = "Please select a directory or set of directories for Tools Repository";
        ListEditDialog->ListView1->Items->Clear();			//	Clear any previous junk from array
        ListEditDialog->Items = ToolsRepository;			//	Set to any current directory
        ListEditDialog->ShowModal();					//	Display the form
        if ( ListEditDialog->ModalResult != mrOk )	return;		//	User cancelled request
        ToolsRepository = ListEditDialog->Items;			//	else change the entries
        Modified = true;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SToolsRepository( AnsiString Value )
{
	if ( Edit10->Text != Value )
        	Edit10->Text = Value;
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GToolsRepository( void )
{
	return( Edit10->Text );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SAudioEditor( AnsiString Value )
{
	if ( Edit12->Text != Value )	Edit12->Text = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SGfx2DEditor( AnsiString Value )
{
	if ( Edit13->Text != Value )	Edit13->Text = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SGfx3DEditor( AnsiString Value )
{
	if ( Edit14->Text != Value )	Edit14->Text = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SGfxMovieEditor( AnsiString Value )
{
	if ( Edit15->Text != Value )	Edit15->Text = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SCDBurner( AnsiString Value )
{
	if ( Edit16->Text != Value )	Edit16->Text = Value;
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SEmulator( AnsiString Value )
{
	if ( Edit11->Text != Value )	Edit11->Text = Value;
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GAudioEditor( void )
{
	return( Edit12->Text );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GGfx2DEditor( void )
{
	return( Edit13->Text );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GGfx3DEditor( void )
{
	return( Edit14->Text );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GGfxMovieEditor( void )
{
	return( Edit15->Text );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GCDBurner( void )
{
	return( Edit16->Text );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GEmulator( void )
{
	return( Edit11->Text );
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton8Click(TObject *Sender)
{
	//	Change from the internal emulator to an external one
        OpenDialog->Title = "Select Emulator";
        OpenDialog->Filter = "Applications ( *.exe )|*.exe|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( !OpenDialog->Execute() ) return;

        //	Remove the internal emulator handle, and references in the
        // project options, enviroment options and the system registry

        //	First change this form and force save to registry, thus doing
        // 2 changes at once

        Emulator = OpenDialog->FileName;

        //       Remove it from the enviroment options
        for ( int i=0; i<=ListView3->Items->Count; i++ ) {
            if ( ListView3->Items->Item[ i ]->Caption == "Internal Target Emulator" ) {
               ListView3->Items->Delete( i );
               break;
            }
        }

        if ( MainForm->hEmulatorHandle != INVALID_HANDLE_VALUE ) { //     if internal emulator loaded, free it
           FreeLibrary( MainForm->hEmulatorHandle );
        }

        // remove it from the project options form
        for ( int i=0; i<=ProjectOptionsForm->ListView8->Items->Count; i++ ) {
            if ( ProjectOptionsForm->ListView8->Items->Item[ i ]->Caption == "Emulator" ) {
               ProjectOptionsForm->ListView8->Items->Delete( i );
               break;
            }
        }

}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton9Click(TObject *Sender)
{
	//	change the audio editor
        OpenDialog->Title = "Select Audio Editor";
        OpenDialog->Filter = "Applications ( *.exe )|*.exe|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( !OpenDialog->Execute() )      return;

        AudioEditor = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton10Click(TObject *Sender)
{
	//	change the 2d graphics editor
        OpenDialog->Title = "Select 2D Editor";
        OpenDialog->Filter = "Applications ( *.exe )|*.exe|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( !OpenDialog->Execute() )      return;

        Gfx2DEditor = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton11Click(TObject *Sender)
{
	//	change the 3d graphics editor
        OpenDialog->Title = "Select 3D Editor";
        OpenDialog->Filter = "Applications ( *.exe )|*.exe|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( !OpenDialog->Execute() )      return;

        Gfx3DEditor = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton12Click(TObject *Sender)
{
	//	change the movie editor
        OpenDialog->Title = "Select Movie Editor";
        OpenDialog->Filter = "Applications ( *.exe )|*.exe|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( !OpenDialog->Execute() )      return;

        MovieEditor = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::SpeedButton13Click(TObject *Sender)
{
	//	change the cd burner application
        OpenDialog->Title = "Select CD Burner";
        OpenDialog->Filter = "Applications ( *.exe )|*.exe|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( !OpenDialog->Execute() )      return;

        CDBurner = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::CColorGrid1Change(TObject *Sender)
{
     if ( !ListView4->Selected ) return;

     pElements[ ListView4->Selected->Index ].ForeGround = CColorGrid1->ForegroundIndex;
     pElements[ ListView4->Selected->Index ].BackGround = CColorGrid1->BackgroundIndex;
     pElements[ ListView4->Selected->Index ].Bold = CheckBox30->Checked;
     pElements[ ListView4->Selected->Index ].Italic = CheckBox36->Checked;
     pElements[ ListView4->Selected->Index ].UnderLine = CheckBox37->Checked;
     pElements[ ListView4->Selected->Index ].DefaultFore = CheckBox38->Checked;
     pElements[ ListView4->Selected->Index ].DefaultBack = CheckBox39->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TEnviromentOptionsForm::ColoursShow(TObject *Sender)
{
     ListView4->Selected = NULL;
     CheckBox30->Checked = false;
     CheckBox36->Checked = false;
     CheckBox37->Checked = false;
     CheckBox38->Checked = false;
     CheckBox39->Checked = false;
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TEnviromentOptionsForm :: GStyleName( void )
{
	return( ComboBox3->Text );
}
//---------------------------------------------------------------------------
void	__fastcall	TEnviromentOptionsForm :: SStyleName( AnsiString Value )
{
	if ( Value.Length() <= 0 )	return;

        ComboBox3->Text = Value;

        Registry = new TRegistry;
        Registry->OpenKey( ENV_OPTIONS_ELEMENTS, false );

        Registry->WriteString( "Style", ComboBox3->Text );

        Registry->CloseKey();

        delete Registry;

        WriteToRegistry( this );

}
//---------------------------------------------------------------------------
//	This is called after the text has changed in the combobox, and is used
// to change the editor style, well colours anyhow.
//---------------------------------------------------------------------------
void 	__fastcall 	TEnviromentOptionsForm :: ComboBox3Change( TObject *Sender )
{	AnsiString	asKeyName = NULL;

	if ( Sender == NULL )	return;

        ClearSysMessages();

        SystemMessage( "Changing the editor's colours style." );

        asKeyName = AnsiString( ENV_OPTIONS_ELEMENTS + "\\" + ComboBox3->Text );

        Registry = new TRegistry;

        if ( Registry == NULL ) {
        	SystemMessage( "Bad registry." );
        	delete Registry;
                return;
        }

        if ( !Registry->OpenKey( asKeyName, false ) ) {	// style key not found
        	SystemMessage( "Style not found in system registry." );
                delete Registry;
                return;
        }

        Registry->CloseKey();

        SystemMessage( "Changing editor style to " + ComboBox3->Text );

        Registry->OpenKey( ENV_OPTIONS_ELEMENTS, false );

        Registry->WriteString( "Style", ComboBox3->Text );

        Registry->CloseKey();

        //	Now load the elements from the registry
        for ( int i=0; i<=NumElements; i++ ) {
        	AnsiString Element = AnsiString( ENV_OPTIONS_ELEMENTS + "\\" +
                	ComboBox3->Text + "\\" + ListView4->Items->Item[ i ]->Caption );

                if ( !Registry->OpenKey( Element, false ) ) {
                	SystemMessage( "Element " + ListView4->Items->Item[ i ]->Caption +
                        	" is missing, please please correct." );
                        continue;
                } else {
                        pElements[ i ].Bold = Registry->ReadBool( "Bold" );
                        pElements[ i ].Italic = Registry->ReadBool( "Italic" );
                        pElements[ i ].UnderLine = Registry->ReadBool( "UnderLine" );
                        pElements[ i ].DefaultFore = Registry->ReadBool( "ForeDefault" );
                        pElements[ i ].DefaultBack = Registry->ReadBool( "BackDefault" );

                        pElements[ i ].ForeGround = Registry->ReadInteger( "ForeGround" );
                        pElements[ i ].BackGround = Registry->ReadInteger( "BackGround" );

                        Registry->CloseKey();
                }
        }

        delete Registry;

}
//---------------------------------------------------------------------------
