//------------------------------------------------------------------------------
//
//	$FileName	::	GetAppFileVersionInfoStrings.cpp
//	$Author		::	Unknown
//	$Modified	::	Chris Brooklyn
//	$Purpose	::	Obtain file version information strings
//
//------------------------------------------------------------------------------
/*******************************************************************************
*									       *
* The class retrieves the application's own EXE-file's Version Info.	       *
*									       *
* Constructor:								       *
*  Takes no parameter. While the class is being constructed the file Version   *
*  Info size, data pointer and language ID are retrieved.		       *
* Public member functions:						       *
*  AnsiString GetAppFileVersionInfoString(AnsiString KeyId):		       *
*   KeyId is the name of the Version Info key whose value should be picked up. *
*   Returns the key's value.						       *
*  bool GetAnyErrFlag():						       *
*   Returns "true" if any error occurred since the class has been constructed; *
*   otherwise, returns "false".						       *
*  bool GetLastErrFlag():						       *
*   Returns "true" if an error occurred upon the last execution of	       *
*   the function GetAppFileVersionInfoString(); otherwise, returns "false".    *
*  version number formatting function:					       *
*   (I have designated the 4 segments of the version number as A, B, C, D from *
*   left to right. The functions have no parameter to be easily used.)	       *
*   AnsiString GetAppFileVerAStr():					       *
*    Returns A-segment of version number as ansi-string.		       *
*   AnsiString GetAppFileVerBStr():					       *
*    Returns B-segment of version number as ansi-string.		       *
*   AnsiString GetAppFileVerCStr():					       *
*    Returns C-segment of version number as ansi-string.		       *
*   AnsiString GetAppFileVerDStr():					       *
*    Returns D-segment of version number as ansi-string.		       *
*   int GetAppFileVerAInt():						       *
*    Returns A-segment of version number as integer.			       *
*   int GetAppFileVerBInt():						       *
*    Returns B-segment of version number as integer.			       *
*   int GetAppFileVerCInt():						       *
*    Returns C-segment of version number as integer.			       *
*   int GetAppFileVerDInt():						       *
*    Returns D-segment of version number as integer.			       *
* Destructor:								       *
*  Releases allocated memory if any has been allocated.			       *
*									       *
*******************************************************************************/

#include "GetAppFileVersionInfoStrings.h"
#include "Utilities.h"

//------------------------------------------------------------------------------

__fastcall GetApplicationFileVersionInfo::GetApplicationFileVersionInfo() :

AnyErrDetected(false),
LastErrDetected(false),
PickUpKeyVal("\\StringFileInfo\\%04X%04X\\%s")

{
 FileVerInfoBuff = NULL; // If not NULL at the end memory has to be released.
 do { // This is a fake loop.
  ExeFilename = Application->ExeName;

  /*>> Get the Version Info size in order to:
  1) check if there's any Version Info included in the file,
  2) learn how much memory has to be allocated to hold the Version Info data. */
  unsigned long NoUse;
  FileVerInfoSize = GetFileVersionInfoSize(ExeFilename.c_str(),
  &NoUse); // NoUse receives always zero.
  if (FileVerInfoSize == 0) {
   LastErrDetected = true;
   break;
  }

  //>> Get the Version Info data pointer:
  FileVerInfoBuff = new char[FileVerInfoSize];
  if (GetFileVersionInfo(ExeFilename.c_str(),0,FileVerInfoSize,FileVerInfoBuff) == 0) {
   LastErrDetected = true;
   break;
  }

  //>> Get the code of the language in which the Version Info is written:
  if (VerQueryValue(FileVerInfoBuff,"\\VarFileInfo\\Translation",
  &LanguageCodePtr,&ExtractedDataSize) == 0) {
   LastErrDetected = true;
   break;
  }
  AppVerStrg = GetAppFileVersionInfoString("FileVersion");
  UShort EndSubSg,
	 SgLng,
	 BegSubSg = 1;
  SgLng = (UShort)AppVerStrg.Length();
  EndSubSg = (UShort)AppVerStrg.Pos(".");
  AppVerA = AppVerStrg.SubString(BegSubSg,EndSubSg - 1);
  SgLng -= EndSubSg;
  BegSubSg += EndSubSg;
  EndSubSg = (UShort)AppVerStrg.SubString(BegSubSg,SgLng).Pos(".");
  AppVerB = AppVerStrg.SubString(BegSubSg,EndSubSg - 1);
  SgLng -= (UShort)EndSubSg;
  BegSubSg += (UShort)EndSubSg;
  EndSubSg = (UShort)AppVerStrg.SubString(BegSubSg,SgLng).Pos(".");
  AppVerC = AppVerStrg.SubString(BegSubSg,EndSubSg - 1);
  SgLng -= (UShort)EndSubSg;
  BegSubSg += (UShort)EndSubSg;
  AppVerD = AppVerStrg.SubString(BegSubSg,SgLng);
 } while (0);
 AnyErrDetected |= LastErrDetected;
}
//------------------------------------------------------------------------------

__fastcall	GetApplicationFileVersionInfo :: GetApplicationFileVersionInfo( AnsiString PathName )
{
	SystemMessage( "Looking for version information in the file " + PathName );

        ExeFilename = PathName;

        AnyErrDetected = false;

        LastErrDetected = false;

        PickUpKeyVal = "\\StringFileInfo\\%04x%04x\\%s";

 FileVerInfoBuff = NULL; // If not NULL at the end memory has to be released.
 do { // This is a fake loop.
  ExeFilename = Application->ExeName;

  /*>> Get the Version Info size in order to:
  1) check if there's any Version Info included in the file,
  2) learn how much memory has to be allocated to hold the Version Info data. */
  unsigned long NoUse;
  FileVerInfoSize = GetFileVersionInfoSize(ExeFilename.c_str(),
  &NoUse); // NoUse receives always zero.
  if (FileVerInfoSize == 0) {
   LastErrDetected = true;
   break;
  }

  //>> Get the Version Info data pointer:
  FileVerInfoBuff = new char[FileVerInfoSize];
  if (GetFileVersionInfo(ExeFilename.c_str(),0,FileVerInfoSize,FileVerInfoBuff) == 0) {
   LastErrDetected = true;
   break;
  }

  //>> Get the code of the language in which the Version Info is written:
  if (VerQueryValue(FileVerInfoBuff,"\\VarFileInfo\\Translation",
  &LanguageCodePtr,&ExtractedDataSize) == 0) {
   LastErrDetected = true;
   break;
  }
  AppVerStrg = GetAppFileVersionInfoString("FileVersion");
  UShort EndSubSg,
	 SgLng,
	 BegSubSg = 1;
  SgLng = (UShort)AppVerStrg.Length();
  EndSubSg = (UShort)AppVerStrg.Pos(".");
  AppVerA = AppVerStrg.SubString(BegSubSg,EndSubSg - 1);
  SgLng -= EndSubSg;
  BegSubSg += EndSubSg;
  EndSubSg = (UShort)AppVerStrg.SubString(BegSubSg,SgLng).Pos(".");
  AppVerB = AppVerStrg.SubString(BegSubSg,EndSubSg - 1);
  SgLng -= (UShort)EndSubSg;
  BegSubSg += (UShort)EndSubSg;
  EndSubSg = (UShort)AppVerStrg.SubString(BegSubSg,SgLng).Pos(".");
  AppVerC = AppVerStrg.SubString(BegSubSg,EndSubSg - 1);
  SgLng -= (UShort)EndSubSg;
  BegSubSg += (UShort)EndSubSg;
  AppVerD = AppVerStrg.SubString(BegSubSg,SgLng);
 } while (0);
 AnyErrDetected |= LastErrDetected;


}

//------------------------------------------------------------------------------

__fastcall GetApplicationFileVersionInfo::~GetApplicationFileVersionInfo()
{
 if (FileVerInfoBuff != NULL) {
  delete[] FileVerInfoBuff;
 }
}
//------------------------------------------------------------------------------

AnsiString __fastcall
GetApplicationFileVersionInfo::GetAppFileVersionInfoString(AnsiString KeyId)
{
 AnsiString	WkSg,
		FakeSg;
 void* 		RetVal;

 LastErrDetected = false;
 FakeSg = "";
 RetVal = WkSg.c_str();

 //>> Extract and return the value the key given by the parameter:
 WkSg.sprintf(PickUpKeyVal,*(UShort*)LanguageCodePtr,
 *((UShort*)LanguageCodePtr + 1),KeyId.c_str());
 if (VerQueryValue(FileVerInfoBuff,WkSg.c_str(),&InfoDataPtr,
 &ExtractedDataSize) == 0) {
  LastErrDetected = true;
 } else {
  RetVal = InfoDataPtr;
 }
 AnyErrDetected |= LastErrDetected;
 return((AnsiString)(char*)RetVal);
}
//------------------------------------------------------------------------------

AnsiString __fastcall GetApplicationFileVersionInfo::GetAppFileVerNumStr() const
{
 return(AppVerStrg);
}
//------------------------------------------------------------------------------

AnsiString __fastcall GetApplicationFileVersionInfo::GetAppFileVerAStr() const
{
 return(AppVerA);
}
//------------------------------------------------------------------------------

AnsiString __fastcall GetApplicationFileVersionInfo::GetAppFileVerBStr() const
{
 return(AppVerB);
}
//------------------------------------------------------------------------------

AnsiString __fastcall GetApplicationFileVersionInfo::GetAppFileVerCStr() const
{
 return(AppVerC);
}
//------------------------------------------------------------------------------

AnsiString __fastcall GetApplicationFileVersionInfo::GetAppFileVerDStr() const
{
 return(AppVerD);
}
//------------------------------------------------------------------------------

int __fastcall GetApplicationFileVersionInfo::GetAppFileVerAInt() const
{
 return(StrToInt(AppVerA));
}
//------------------------------------------------------------------------------

int __fastcall GetApplicationFileVersionInfo::GetAppFileVerBInt() const
{
 return(StrToInt(AppVerB));
}
//------------------------------------------------------------------------------

int __fastcall GetApplicationFileVersionInfo::GetAppFileVerCInt() const
{
 return(StrToInt(AppVerC));
}
//------------------------------------------------------------------------------

int __fastcall GetApplicationFileVersionInfo::GetAppFileVerDInt() const
{
 return(StrToInt(AppVerD));
}
//------------------------------------------------------------------------------

bool __fastcall GetApplicationFileVersionInfo::GetAnyErrFlag() const
{
 return(AnyErrDetected);
}
//------------------------------------------------------------------------------

bool __fastcall GetApplicationFileVersionInfo::GetLastErrFlag() const
{
 return(LastErrDetected);
}
//------------------------------------------------------------------------------

