//---------------------------------------------------------------------------
//
//	$FileName	::	PackageManager.cpp
//	$Author		::	Chris Brooklyn
//	$Purpose	::	Allows for loading, unloading and editing of
//				any external packages
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "PackageManager.h"
#include "Utilities.h"
#include "main.h"
#include "PackageDialogUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TPackagesForm *PackagesForm;
//---------------------------------------------------------------------------
__fastcall 	TPackagesForm :: TPackagesForm( TComponent* Owner ) : TForm( Owner )
{

	if ( pPackages == NULL )
        	*pPackages = new TPackage[ MAX_PACKAGES ];

	iNumPackages = 0;

}
//---------------------------------------------------------------------------
__fastcall	TPackagesForm :: ~TPackagesForm( void )
{
	// WriteToRegistry( this );
	delete this;
}
//---------------------------------------------------------------------------
void 	__fastcall 	TPackagesForm :: BitBtn2Click( TObject *Sender )
{
	if ( !ListView1->Selected )	{
        	SystemMessage( "Select a package for removal" );
                return;
        }

        Label1->Caption = AnsiString( "Removal of package " + ListView1->Selected->Caption );

        int mrCode = MessageBox( Application->Handle, AnsiString( "Confirm removal of package " + ListView1->Selected->Caption ).c_str(),
        	Application->Title.c_str(), MB_YESNO | MB_ICONEXCLAMATION );

        if ( mrCode == mrNo )	{
		Label1->Caption = "Package NOT removed";
		return;
        }

        Label1->Caption = AnsiString( "Package " + ListView1->Selected->Caption + " removed." );

	ListView1->Items->Delete( ListView1->Selected->Index );
        iNumPackages--;
}
//---------------------------------------------------------------------------
void 	__fastcall 	TPackagesForm :: BitBtn1Click( TObject *Sender )
{	TPackage	*Package;
	AnsiString	asKeyName;

        if ( iNumPackages > MAX_PACKAGES )	{
        	SystemMessage( "Unable to load package, ide full." );
                return;
        }

        Package = pPackages[ iNumPackages ];

        if ( OpenDialog->Execute() ) {
        	TListItem *pItem = ListView1->Items->Add();

        	SystemMessage( "Loading package " + OpenDialog->FileName );

                if ( !Package->LoadFromFile( OpenDialog->FileName ) ) {
                        SystemMessage( "Unable to load package " + OpenDialog->FileName );
                        return;
                }

                pItem->Caption = Package->PackageName;

                pItem->SubItems->Add( "0.0.0.0" );

                switch ( Package->PackageType ) {

                	case ( PACKAGE_DESIGNER ) : {
                        	pItem->SubItems->Add( "Designer" );
                                break;
                        }

                        case ( PACKAGE_RUNTIME ) : {
                        	pItem->SubItems->Add( "RunTime" );
                                break;
                        }

                        case ( PACKAGE_PLUGIN ) : {
                        	pItem->SubItems->Add( "Plugin" );
                                break;
                        }

                        case ( PACKAGE_DEFAULT ) : {
                        	pItem->SubItems->Add( "Design / RunTime" );
                                break;
                        }

                        default : {
                        	pItem->SubItems->Add( AnsiString( "Unknown id : " + IntToStr( pPackages[ iNumPackages ]->PackageType ) ) );
                                break;
                        }
                }

                pItem->SubItems->Add( Package->Description );

		iNumPackages++;

		//	Write data to system registry
		Registry = new TRegistry;

		asKeyName = AnsiString( PSXIDE_PACKAGES + "\\" + IntToStr( iNumPackages ) );

		Registry->OpenKey( asKeyName, true );

		Registry->WriteString( "PackageFile", OpenDialog->FileName );

		Registry->CloseKey();

		delete Registry;

	}

}
//---------------------------------------------------------------------------
void __fastcall TPackagesForm::FormCreate(TObject *Sender)
{

	if ( Sender == NULL )
        	return;

        if ( pPackages == NULL ) {
        	iNumPackages = 0;
        	return;
        }

	if ( MainForm->hEmulatorHandle != INVALID_HANDLE_VALUE ) {
		TListItem 	*pItem = ListView1->Items->Add();
		char		*pText = new char[ MAX_PATH ];

		pItem->Caption = "Emulator";
		pItem->SubItems->Add( "0.0.0.0" );
		pItem->SubItems->Add( "RunTime" );
		pItem->SubItems->Add( "Internal PlayStation Emulator" );

                //if ( pPackages[ 0 ] == NULL )
                //	pPackages[ 0 ] = new TPackage;

		//pPackages[ 0 ]->PackageName = pItem->Caption;
		//pPackages[ 0 ]->Description = pItem->SubItems->Strings[ 2 ];
		//pPackages[ 0 ]->PackageType = PACKAGE_RUNTIME;
		//pPackages[ 0 ]->asFileName = "lib\\Emulator.dll";

		memset( pText, 0, MAX_PATH );
		//if ( GetModuleFileName( MainForm->hEmulatorHandle, pText, MAX_PATH ) == 0 ) {
		//	pPackages[ 0 ]->asFileName = "Unknown or internal";
		//} else {
		//	pPackages[ 0 ]->asFileName = AnsiString( pText );
		//}

		iNumPackages++;
	}

	if ( MainForm->hDebuggerHandle != INVALID_HANDLE_VALUE ) {
        	TListItem 	*pItem = ListView1->Items->Add();
		char		*pText = new char[ MAX_PATH ];

                pItem->Caption = "Debugger";
                pItem->SubItems->Add( "0.0.0.0" );
                pItem->SubItems->Add( "RunTime" );
                pItem->SubItems->Add( "Internal Debugger" );

                //if ( pPackages[ 1 ] == NULL )
                //	pPackages[ 1 ] = new TPackage;

		//pPackages[ 1 ]->PackageName = pItem->Caption;
		//pPackages[ 1 ]->Description = pItem->SubItems->Strings[ 2 ];
		//pPackages[ 1 ]->PackageType = PACKAGE_RUNTIME;
		//pPackages[ 1 ]->asFileName = "lib\\Debugger.dll";

		//memset( pText, 0, MAX_PATH );

		//if ( GetModuleFileName( MainForm->hDebuggerHandle, pText, MAX_PATH ) == 0 ) {
		//	pPackages[ 1 ]->asFileName = "Unknown or internal";
		//} else {
		//	pPackages[ 1 ]->asFileName = AnsiString( pText );
		//}

		iNumPackages++;
	}

	//ReadFromRegistry( this );

	Label1->Caption = AnsiString( "Number of installed packages : " + IntToStr( iNumPackages ) );
}
//---------------------------------------------------------------------------
void __fastcall TPackagesForm::FormDestroy(TObject *Sender)
{	AnsiString	asKeyName;

	//	Write data about the packages to the system registry
	// note that the package's 0 and 1 are reserved for the internal
	// emulator and debugger respectively

	if ( NumPackages > 0 ) {

		if ( MainForm->hEmulatorHandle != INVALID_HANDLE_VALUE ) {

			asKeyName = AnsiString( PSXIDE_PACKAGES + "\\0" );

			Registry->OpenKey( asKeyName, true );

			Registry->WriteString( "PackageFile", pPackages[ 0 ]->asFileName );

			Registry->CloseKey();
		}

		if ( MainForm->hDebuggerHandle != INVALID_HANDLE_VALUE ) {

			asKeyName = AnsiString( PSXIDE_PACKAGES + "\\1" );

			Registry->OpenKey( asKeyName, true );

			Registry->WriteString( "PackageFile", pPackages[ 1 ]->asFileName );

			Registry->CloseKey();
		}

		//	write the packages out
		Registry = new TRegistry;

		Registry->LazyWrite = false;


		if ( iNumPackages > 2 ) {

			Registry->OpenKey( PSXIDE_PACKAGES, true );

			Registry->WriteInteger( "NumPackages", iNumPackages );

			Registry->CloseKey();

			for ( int i=2; i<=(iNumPackages-1); i++ ) {
				asKeyName = AnsiString( PSXIDE_PACKAGES + "\\" + IntToStr( i ) );

				Registry->OpenKey( asKeyName, true );

				if ( pPackages[ i ]->asFileName.Length() > 0 )
					Registry->WriteString( "PackageFile", pPackages[ i ]->asFileName );
				else
					Registry->WriteString( "PackageFile", "" );

				Registry->CloseKey();
			}
		}

		delete Registry;
	}

}
//---------------------------------------------------------------------------
void __fastcall TPackagesForm::Add1Click(TObject *Sender)
{
	BitBtn1Click( Sender );
}
//---------------------------------------------------------------------------
void __fastcall TPackagesForm::Remove1Click(TObject *Sender)
{
	BitBtn2Click( Sender );
}
//---------------------------------------------------------------------------
void 	__fastcall 	TPackagesForm :: Edit1Click( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	//	Set the PackageDialog for read / write mode

        if ( !Originator )	return;	//	Bad call ?

        if ( !ListView1->Selected )	{
        	SystemMessage( "Please selected a package prior to editing it" );
                return;
        }

        if ( !pPackages[ ListView1->Selected->Index ] )	{
		SystemMessage( "INTERNAL ERROR, NO PACKAGE FOR " + IntToStr( ListView1->Selected->Index ) );
		return;
        }

        PackageDialog->Edit1->Text = pPackages[ ListView1->Selected->Index ]->PackageName;
        PackageDialog->Edit2->Text = pPackages[ ListView1->Selected->Index ]->Description;
        PackageDialog->Edit3->Text = pPackages[ ListView1->Selected->Index ]->asFileName;

	switch ( pPackages[ ListView1->Selected->Index ]->PackageType ) {
        	case ( PACKAGE_DESIGNER ) : {
                	PackageDialog->ComboBox1->ItemIndex = 0;
                        break;
                }
                case ( PACKAGE_RUNTIME ) : {
                	PackageDialog->ComboBox1->ItemIndex = 1;
                        break;
		}
                case ( PACKAGE_PLUGIN ) : {
                	PackageDialog->ComboBox1->ItemIndex = 2;
                        break;
		}
                case ( PACKAGE_DESIGN_RUNTIME ) : {
                	PackageDialog->ComboBox1->ItemIndex = 3;
                        break;
                }
	}

        PackageDialog->Edit1->Enabled = true;
        PackageDialog->Edit2->Enabled = true;
        PackageDialog->Edit3->Enabled = false;

        PackageDialog->ComboBox1->Enabled = true;

        PackageDialog->SpeedButton1->Enabled = true;

        PackageDialog->Modified = false;

        PackageDialog->ShowModal();

        if ( !PackageDialog->Modified )	return;

        if ( PackageDialog->ModalResult != mrOk )	return;

        pPackages[ ListView1->Selected->Index ]->PackageName = PackageDialog->Edit1->Text;

        pPackages[ ListView1->Selected->Index ]->Description = PackageDialog->Edit2->Text;

}
//---------------------------------------------------------------------------
void 	__fastcall 	TPackagesForm :: Info1Click( TObject *Sender )
{       TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )
		return;

	if ( !ListView1->Selected )	{
		SystemMessage( "Please select a package to display information for" );
		return;
	}

	PackageDialog->Edit1->Text = pPackages[ ListView1->Selected->Index ]->PackageName;
	PackageDialog->Edit2->Text = pPackages[ ListView1->Selected->Index ]->Description;
	//PackageDialog->Edit3->Text = pPackages[ ListView1->Selected->Index ]->FileName;

	switch ( pPackages[ ListView1->Selected->Index ]->PackageType ) {

		case ( PACKAGE_DESIGNER ) : {
			PackageDialog->ComboBox1->ItemIndex = 0;
			break;
		}

		case ( PACKAGE_RUNTIME ) : {
			PackageDialog->ComboBox1->ItemIndex = 1;
			break;
		}

		case ( PACKAGE_PLUGIN ) : {
			PackageDialog->ComboBox1->ItemIndex = 2;
			break;
		}

		case ( PACKAGE_DESIGN_RUNTIME ) : {
			PackageDialog->ComboBox1->ItemIndex = 3;
			break;
		}

		default : {
			PackageDialog->ComboBox1->ItemIndex = -1;
			break;
		}

	}

	PackageDialog->Edit1->Enabled = false;
	PackageDialog->Edit2->Enabled = false;
	PackageDialog->Edit3->Enabled = false;

	PackageDialog->ComboBox1->Enabled = false;

	PackageDialog->SpeedButton1->Enabled = false;

	PackageDialog->ShowModal();

	PackageDialog->Modified = false;
}
//---------------------------------------------------------------------------
int	__fastcall	TPackagesForm :: WriteToRegistry( TObject *Sender )
{
	if ( Sender == NULL )
		return( E_BAD_SENDER );

	return( E_NOERROR );
}
//----------------------------------------------------------------------------
int	__fastcall	TPackagesForm :: ReadFromRegistry( TObject *Sender )
{
	if ( Sender == NULL )
		return( E_BAD_SENDER );

	return( E_NOERROR );
}
//----------------------------------------------------------------------------
