//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "PackageUnit.h"
#include "Utilities.h"
#include "PackageDialogUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
__fastcall	TPackage :: TPackage( TObject *Owner ) : TObject()
{
	//	Set the owner of this package, should usually be the PackageManager
	if ( Owner != NULL ) pOwner = Owner;
	else pOwner = dynamic_cast< TObject* >(this);

	//	Set package type and name to a known default state, should really ask
	// for the information, and possibly source files?

	dwPackageType = PACKAGE_DEFAULT;
	asPackageName = "Untitled";

	//	set internal variables to NULL, to specify that the library
	// has not been loaded yet and no resources/information is available
	// about the library
	hInstance = NULL;

	hrcName	= NULL;		//	Name resource if present
	hrcDescription = NULL;	//	Description resource
	hrcVersion = NULL;	//	Version resource

	FOnLeftClick = NULL;
	FOnRightClick = NULL;
	FOnEdit = NULL;

}
//---------------------------------------------------------------------------
__fastcall	TPackage :: TPackage( AnsiString FileName ) : TObject()
{
	pOwner = dynamic_cast< TObject* >(this);

	//	Set package type and name to a known default state, should really ask
	// for the information, and possibly source files?

	dwPackageType = PACKAGE_DEFAULT;
	asPackageName = "Untitled";

	//	set internal variables to NULL, to specify that the library
	// has not been loaded yet and no resources/information is available
	// about the library, nor is any information stored in the system
        // registry about the package
	hInstance = NULL;

	hrcName	= NULL;		//	Name resource if present
	hrcDescription = NULL;	//	Description resource
	hrcVersion = NULL;	//	Version resource

	FOnLeftClick = NULL;
	FOnRightClick = NULL;
	FOnEdit = NULL;

	this->LoadFromFile( FileName );
}
//---------------------------------------------------------------------------
__fastcall	TPackage :: ~TPackage( void )
{
	if ( this )	{	//	Destroy this instance of the class ?
		//	Pre destruction cleanup
		if ( pOwner != dynamic_cast< TObject* >(this) ) {
			pOwner = NULL;	//	Remove connection
		}

		if ( !hInstance )	FreeLibrary( hInstance );

		if ( hrcName )		FreeResource( hrcName );

		if ( hrcDescription )	FreeResource( hrcName );

		if ( hrcVersion )	FreeResource( hrcVersion );

	}

	delete this;
}
//---------------------------------------------------------------------------
int	__fastcall	TPackage :: LoadFromFile( AnsiString FileName )
{
	if ( FileName.Length() <= 0 ) return( false );

	if ( !FileExists( FileName ) )	{
		SystemMessage( FileName + " is not found." );
		hInstance = NULL;
		return( false );
	}

	SystemMessage( "Loading " + FileName );

	hInstance = LoadLibrary( FileName.c_str() );	//	Load the library

	if ( hInstance == NULL ) {	//	Unable to load
		SystemMessage( "Unable to load package from file " + FileName );
		return( false );
	}

	SystemMessage( "Loaded. Querying." );

	//	file loaded OK, decode the resources to determine the package
	//	type, use FindResourceEx to obtain handles to the information

	hrcName = FindResourceEx( hInstance, RT_STRING, "PackageName", MAKELANGID( LANG_NEUTRAL, SUBLANG_NEUTRAL ) );

	if ( hrcName != NULL )	{	//	Managed to locate the resource ?
		char	*pText = new char[ MAX_PATH ];

		memset( pText, 0, MAX_PATH );

		if ( LoadString( hInstance, (UINT)hrcName, pText, MAX_PATH ) == 0 ) {
                	SystemMessage( "Unable to find package name resource." );
                        //return( false );
                }

                if ( strlen( pText ) <= 0 ) {
                	SystemMessage( "Bad resource located, null length." );
                        //return( false );
                }

                asPackageName = AnsiString( pText );
        } else {
        	SystemMessage( "Resource 'PackageName' is NOT present in file " + FileName );
                //return( false );
                asPackageName = "Untitled";
        }

        //	find description
	hrcDescription = FindResourceEx( hInstance, RT_STRING, "PackageDescription", MAKELANGID( LANG_NEUTRAL, SUBLANG_NEUTRAL ) );

        if ( hrcDescription != NULL )	{	//	Managed to locate the resource ?
		char	*pText = new char[ MAX_PATH ];

                memset( pText, 0, MAX_PATH );

                if ( LoadString( hInstance, (UINT)hrcDescription, pText, MAX_PATH ) == 0 ) {
                	SystemMessage( "Unable to find package description resource." );
                        return( false );
                }

                if ( strlen( pText ) <= 0 ) {
                	SystemMessage( "Bad resource located, null length." );
                        return( false );
                }

                asPackageDescription = AnsiString( pText );

        } else {
        	SystemMessage( "Resource 'PackageDescription' is NOT present in file " + FileName );
                //return( false );
                asPackageDescription = "Unknown capabilities";
        }

        //	find version info, to do

	//	Get the address of any current events
        //	FOnLeftClick = dynamic_cast< TPackageEvent >(GetProcAddress( hInstance, "OnLeftClick" ));
        //	FOnRightClick = dynamic_cast< TPackageEvent >(GetProcAddress( hInstance, "OnRightClick" ));
        //	FOnEdit = dynamic_cast< TPackageEvent >(GetProcAddress( hInstance, "OnEdit" ));

        //	Set event handlers to NULL
	FOnRightClick = NULL;
        FOnLeftClick = NULL;
        FOnEdit = NULL;

        SystemMessage( "File " + FileName + " loaded. OK" );
        SystemMessage( "Name " + PackageName );
        SystemMessage( "Description " + Description );

        asFileName = FileName;

        return( true );
}
//---------------------------------------------------------------------------
int	__fastcall	TPackage :: LoadFromRegistry( AnsiString PackageName )
{	TRegistry	*Registry = NULL;
        AnsiString	asKeyName = NULL;

	if ( PackageName.Length() <= 0 )
        	return( E_BAD_PARAMS );

        Registry = new TRegistry;

        asKeyName = AnsiString( PSXIDE_PACKAGES + "\\" + PackageName );

        if ( !Registry->OpenKey( asKeyName, false ) ) {
        	SystemMessage( "Unable to open registry for " + PackageName );
                delete Registry;
                asKeyName = NULL;
                return( E_BAD_REGISTRY );
        }

        //	Key was found, should mean that some informatin MAY be available to us

        asPackageName 		= Registry->ReadString( "PackageName" );
        asPackageFile 		= Registry->ReadString( "PackageFile" );
        asPackageDescription 	= Registry->ReadString( "Description" );
        dwPackageType 		= Registry->ReadInteger( "PackageType" );

        Registry->CloseKey();

        SystemMessage( "Data read from the registry, OK. Now loading from file " + asPackageFile );

	this->LoadFromFile( asPackageFile );

        delete Registry;

        asKeyName = NULL;

        return( E_NOERROR );
}
//----------------------------------------------------------------------------
int	__fastcall	TPackage :: SaveToRegistry( void )
{	TRegistry	*Registry = new TRegistry;
	AnsiString	asKeyName = AnsiString( PSXIDE_PACKAGES + "\\" + asPackageName );

        if ( ( Registry == NULL ) || ( asKeyName.Length() <= 0 ) || ( asPackageName.Length() <= 0 ) )
        	return( E_ABORTED );

        //	Handle special packages 0 and 1, Emulator and Debugger respectively
        if ( ( asPackageName == "Emulator" ) || ( asPackageName == "Debugger" ) )
        	return( E_NOERROR );

        SystemMessage( "Saving package to registry " + asPackageName + " ... " );

        Registry->OpenKey( asKeyName, true );

        Registry->WriteString( "PackageName", asPackageName );
        Registry->WriteString( "PackageFile", asPackageFile );
        Registry->WriteString( "Description", asPackageDescription );
        Registry->WriteInteger( "PackageType", dwPackageType );

        Registry->CloseKey();

        asKeyName = NULL;
        delete Registry;

        SystemMessage( "... saved OK." );
        return( E_NOERROR );
}
//----------------------------------------------------------------------------
