//---------------------------------------------------------------------------
//
//	$FileName	::	ProgInfoForm.cpp
//	$Author		::	Chris Brooklyn
//	$Purpose	::	Project Information Dialog
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "ProgInfoForm.h"
#include "ProjectManager.h"
#include "Main.h"
#include "ProjectOptionsFormUnit.h"
#include "shellapi.h"
#include "Utilities.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TProgInfoDialog *ProgInfoDialog;
//---------------------------------------------------------------------------
__fastcall TProgInfoDialog::TProgInfoDialog(TComponent* Owner)
	: TForm(Owner)
{
	lpSysInfo = new SYSTEM_INFO;		//	CPU Core info
	lpOsVersion = new OSVERSIONINFO;	//	OS Core info

	ZeroMemory( lpSysInfo, sizeof( SYSTEM_INFO ) );
        ZeroMemory( lpOsVersion, sizeof( OSVERSIONINFO ) );

        lpOsVersion->dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
}
//---------------------------------------------------------------------------
void __fastcall TProgInfoDialog::FormCreate(TObject *Sender)
{

	if ( ProjectManagerForm != NULL ) {
        	if ( !ProjectManagerForm->ActiveProject )
                	Caption = "Application Information";
                else
                	if ( ProjectManagerForm->ProjectName.Length() > 0 )
                        	Caption = "Project " + ProjectManagerForm->ProjectName + " application information";
                        else Caption = "Application Information";
        }

	if ( ProjectOptionsForm->DebugMode )
        	TabSheet4->Visible = true;
        else	TabSheet4->Visible = false;

        PageControl1->ActivePage = TabSheet1;	//	Make sure that the first tabsheet is displayed first
}
//---------------------------------------------------------------------------
void __fastcall TProgInfoDialog::FormShow(TObject *Sender)
{       AnsiString asLine;

	//	Update the System Information Text

        RichEdit1->Lines->Clear();
        //	OS Core information
        RichEdit1->Lines->Add( "Operating System Core Information." );

        if ( !GetVersionEx( lpOsVersion ) )	{
        	RichEdit1->Lines->Add( "Unable to determine OS Core." );
        } else {
        	switch ( lpOsVersion->dwPlatformId ) {
                	case ( VER_PLATFORM_WIN32s ) : {	//	Windows 3.1 with WIN32 extensions
                        	RichEdit1->Lines->Add( "WIN32 on Windows 3.1" );
                                break;
                        }
                        case ( VER_PLATFORM_WIN32_WINDOWS ) : {	//	Windows 95
                        	RichEdit1->Lines->Add( "Windows 95" );
                                break;
                        }
                        case ( VER_PLATFORM_WIN32_NT ) : {	//	Windows NT
                        	RichEdit1->Lines->Add( "Windows NT" );
                                break;
                        }
                }
        }

        if ( lpOsVersion->dwPlatformId == VER_PLATFORM_WIN32_NT ) {
        	RichEdit1->Lines->Add( "Version : " + IntToStr( lpOsVersion->dwMajorVersion ) +
                	"." + IntToStr( lpOsVersion->dwMinorVersion ) +
                        "." + IntToStr( lpOsVersion->dwBuildNumber )
                );
        } else {
        	RichEdit1->Lines->Add( "Version : " + IntToStr( lpOsVersion->dwMajorVersion ) +
                	"." + IntToStr( lpOsVersion->dwMinorVersion ) +
                        "." + IntToStr( LOWORD( lpOsVersion->dwBuildNumber ) )
                );
        }

        RichEdit1->Lines->Add( AnsiString( lpOsVersion->szCSDVersion ) );

        //	Processor Core information
        RichEdit1->Lines->Add( "Processor Core Information." );

        GetSystemInfo( lpSysInfo );

        asLine = "Processor Architecture : ";

        switch ( lpSysInfo->wProcessorArchitecture ) {

        	case ( PROCESSOR_ARCHITECTURE_INTEL ) : {	//	Intel x86, or compatible based processor
                	asLine += " INTEL ";

                        switch ( lpSysInfo->wProcessorLevel ) {

                        	case ( 3 ) : {	//	386
                                	asLine += " 386 ";
                                        break;
                                }

                                case ( 4 ) : {	//	486
                                	asLine += " 486 ";
                                        break;
                                }

                                case ( 5 ) : {	//	Pentium
                                	asLine += " Pentium ";
                                        break;
                                }

                                case ( 6 ) : {	//	Pentium Pro/II/II
                                	asLine += " Pentium Pro/II/III";
                                        break;
                                }
                                default : {
                                	asLine += " Unknown " + IntToStr( lpSysInfo->wProcessorLevel );
                                        break;
                                }
                        }

                        if ( ( lpSysInfo->wProcessorRevision && 0xFF00 ) == 0xFF )	{

                        	asLine += " Model " + IntToStr( ( lpSysInfo->wProcessorRevision && 0x00F0 ) );
                                asLine += " Stepping " + IntToStr( ( lpSysInfo->wProcessorRevision && 0x000F ) );

                        } else {

                        	asLine += " Stepping " + IntToStr( ( lpSysInfo->wProcessorRevision && 0xFF00 ) );
                                asLine += " Minor " + IntToStr( ( lpSysInfo->wProcessorRevision && 0x00FF ) );
                        }

                        break;

                }

                case ( PROCESSOR_ARCHITECTURE_MIPS ) : {
                	asLine += " MIPS ";
                        break;
                }

                case ( PROCESSOR_ARCHITECTURE_ALPHA ) : {
                	asLine += " ALPHA ";
                        break;
                }

                case ( PROCESSOR_ARCHITECTURE_PPC ) : {
                	asLine += " PPC ";
                        break;
                }

                case ( PROCESSOR_ARCHITECTURE_UNKNOWN ) : {
                	asLine += " UNKNOWN " + IntToStr( lpSysInfo->wProcessorArchitecture );
                        break;
                }

                default : {
                	asLine += " UNKNOWN " + IntToStr( lpSysInfo->wProcessorArchitecture );
                        break;
                }

        }

        RichEdit1->Lines->Add( asLine );

        //	Application Information
        RichEdit2->Lines->Clear();

        RichEdit2->Lines->Add( "Project Name   : " + ProjectManagerForm->ProjectName );
        RichEdit2->Lines->Add( "Project Title  : " + ProjectOptionsForm->ProjectTitle );
        RichEdit2->Lines->Add( "Home Directory : " + ProjectOptionsForm->ProjectHomeDir );
        RichEdit2->Lines->Add( "Platform       : " + PlatFormStrings[ ProjectOptionsForm->PlatformId ] );

        //	Executable Information
        RichEdit3->Lines->Clear();

        if ( FileExists( ProjectOptionsForm->ProjectHomeDir + "\\" + ProjectManagerForm->ProjectName + ".exe" ) )
        	RichEdit3->Lines->Add( "Executable is present on disk." );
        else	RichEdit3->Lines->Add( "Executable is not present." );

        //	Debug Information

	if ( ProjectOptionsForm->DebugMode )
        	TabSheet4->Visible = true;
        else	TabSheet4->Visible = false;

        //	Map File information, not implemented in any revision at present
        // as this relies on the compiler's .map file to be present, if any is
        // produced at all

        
}
//---------------------------------------------------------------------------
