//---------------------------------------------------------------------------
//
//      $FileName       ::      ProjectManager.cpp
//      $Author         ::      Chris Brooklyn
//      $Description    ::      IDE Controllable Project Management
//      $Version        ::      0.1.0.0
//
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#include <stdlib.h>
//---------------------------------------------------------------------------
#include "ProjectManager.h"
#include "main.h"
#include "Utilities.h"
#include "EnviromentOptionsUnit.h"
#include "WarningSelect.h"
#include "CodeOptimisationSelect.h"
#include "DebuggingUnit.h"
#include "main.h"
#include "ValueDialogUnit.h"
#include "TDirSelectDialog.h"
#include "ListSelectDialog.h"
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TProjectManagerForm *ProjectManagerForm;
//---------------------------------------------------------------------------
//	Form Constructor
//---------------------------------------------------------------------------
__fastcall TProjectManagerForm::TProjectManagerForm(TComponent* Owner)
        : TForm(Owner)
{
	iCurrModule = iCurrProject = -1;
        bActiveGroup = bActiveProject = false;

#ifdef	DEBUGMODE
	RichEdit1->Visible = true;
        RichEdit1->Enabled = true;
        RichEdit1->Align = alRight;
        RichEdit1->ReadOnly = true;
#endif

}
//---------------------------------------------------------------------------
//	Quick group constructor
//---------------------------------------------------------------------------
LPGROUP		__fastcall	TProjectManagerForm :: NewGroup	( AnsiString GName, AnsiString GHome )
{	LPGROUP	pGroup = new GROUP;

	if ( ( pGroup == NULL )	|| ( GName.Length() <= 0 ) )
        	return( NULL );

        ZeroMemory( pGroup, sizeof( GROUP ) );

        pGroup->Name = GName;
        pGroup->Home = GHome;
        pGroup->Modified = true;

        return( pGroup );
}
//---------------------------------------------------------------------------
//	Quick new project
//---------------------------------------------------------------------------
LPPROJECT	__fastcall	TProjectManagerForm :: NewProject	( AnsiString PName, AnsiString PHome, WORD PType )
{	LPPROJECT	pProject = new PROJECT;

	if ( ( pProject == NULL ) || ( PName.Length() <= 0 ) )
        	return( NULL );

        ZeroMemory( pProject, sizeof( PROJECT ) );

        pProject->Name = PName;
        pProject->Home = PHome;
        pProject->wType = PType;
        pProject->Modified = true;

        return( pProject );
}
//---------------------------------------------------------------------------
LPPROJECT*	__fastcall	TProjectManagerForm :: NewProjects( int NumProjects )
{	LPPROJECT	*lpProjects;

	if ( NumProjects <= 0 )
        	return( NULL );

        lpProjects = new LPPROJECT[ NumProjects + 1 ];

        if ( lpProjects == NULL ) {
        	return( NULL );
        }

        ZeroMemory( lpProjects, sizeof( LPPROJECT ) + ( NumProjects + 1 ) );

        for ( int i=0; i<=NumProjects; i++ )
        	lpProjects[ i ] = NewProject( "Untitled", "C:\\My Documents", PROJECT_TYPE_APPLICATION );

        return( lpProjects );
}
//---------------------------------------------------------------------------
//	quick new module
//---------------------------------------------------------------------------
LPMODULE	__fastcall	TProjectManagerForm :: NewModule	( AnsiString MName, AnsiString MFile )
{	LPMODULE	pModule = new MODULE;

	if ( ( pModule == NULL ) || ( MName.Length() <= 0 ) )
        	return( NULL );

        ZeroMemory( pModule, sizeof( MODULE ) );

        pModule->Name = MName;
        pModule->File = MFile;
        pModule->Modified = true;

        return( pModule );
}
//---------------------------------------------------------------------------
//	property modifiers	read/write pairs, if available, for all of the
// __published properties, note note all are read/write some are read-only
//---------------------------------------------------------------------------
bool	__fastcall	TProjectManagerForm :: GModified( void )
{
	if ( lpGroup->Modified ) {
        	bModified = true;
                goto out;
        }

        if ( lpGroup->lpProjects[ CurrProject ].Modified ) {
        	bModified = true;
                goto out;
        }

        for ( WORD w=0; w<=lpGroup->lpProjects[ CurrProject ].wNumModules; w++ )
        	if ( lpGroup->lpProjects[ CurrProject ].lpModules[ w ].Modified ) {
                	bModified = true;
                        goto out;
                }
out:
        return( bModified );
}
//---------------------------------------------------------------------------
//	Called from external forms, to signal to the project manager that an
// external section of the application modified the project in some form or
// another.
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SModified( bool Value )
{

	//	Is there a currently active group

	if ( bModified != Value )
        	bModified = Value;
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectManagerForm :: GActiveGroup( void )
{
	return( false );//bActiveGroup
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectManagerForm :: GActiveProject( void )
{
	if ( ( !ActiveGroup ) || ( lpGroup->lpProjects == NULL ) || ( lpGroup->wNumProjects <= 0 ) || ( lpGroup->wCurrProject <= 0 ) )
        	return( false );
        else	bActiveProject = true;

        return( bActiveProject );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SActiveProject( bool Value )
{
        if ( bActiveProject != Value )
        	bActiveProject = Value;
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: GCurrProject( void )
{
	if ( ( !ActiveGroup ) && ( !ActiveProject ) )
        	iCurrProject = -1;

        if ( ( lpGroup == NULL ) || ( lpGroup->wNumProjects <= 0 ) )
        	iCurrProject = -1;

        iCurrProject = (int)lpGroup->wCurrProject;

        return( iCurrProject );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SCurrProject( int Value )
{
	if ( !ActiveProject )
        	return;

	if ( Value < 0 )	return;

        if ( ( lpGroup == NULL ) || ( lpGroup->wNumProjects <= 0 ) || ( Value > lpGroup->wNumProjects ) )
        	return;

        if ( lpGroup->wCurrProject != (WORD)Value ) {

        	if ( ActiveProject )
                	IDECloseProject( this );	//	Ask the project manager to close this project

        	lpGroup->wCurrProject = (WORD)Value;
                lpGroup->Modified = true;

                iCurrProject = Value;
                bActiveProject = true;
        }
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: GCurrModule( void )
{
	if ( ( !ActiveGroup ) && ( !ActiveProject ) )
        	iCurrModule = -1;

        if ( ( lpGroup == NULL ) || ( CurrProject == -1 ) )
                iCurrModule = -1;

        iCurrModule = (int)lpGroup->lpProjects[ CurrProject ].wCurrModule;

        return( iCurrModule );
}
//---------------------------------------------------------------------------
//void	__fastcall	TProjectManagerForm :: SCurrModule( int Value )
//{
//	if ( ( !ActiveGroup ) && ( !ActiveProject ) ) {
//        	iCurrModule = -1;
//                return;
//        }
//
//        if ( ( lpGroup == NULL ) || ( CurrProject == -1 ) ) {
//        	iCurrModule = -1;
//                return;
//        }
//
//        lpGroup->lpProjects[ CurrProject ].wCurrModule = (WORD)Value;
//        iCurrModule = (WORD)Value;
//
//        ActivteModule( this );
//}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectManagerForm :: GGroupName	( void )
{
	if ( ( !ActiveGroup ) || ( lpGroup == NULL ) )
        	return( NULL );

        return( lpGroup->Name );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectManagerForm :: GGroupHome	( void )
{
	if ( ( !ActiveGroup ) || ( lpGroup == NULL ) )
        	return( NULL );
        return( lpGroup->Home );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectManagerForm :: GProjectName	( void )
{
	if ( ( !ActiveProject ) || ( lpGroup == NULL ) ||
        	( lpGroup->wNumProjects <= 0 ) || ( lpGroup->wCurrProject <= 0 ) ||
                ( lpGroup->lpProjects == NULL ) || ( iCurrProject != -1 ) )
        	return( NULL );
        return( lpGroup->lpProjects[ CurrProject ].Name );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectManagerForm :: GProjectHome	( void )
{
	if ( ( !ActiveProject ) || ( lpGroup == NULL ) )
        	return( AnsiString( "" ) );
        return( lpGroup->lpProjects[ CurrProject ].Home );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectManagerForm :: GModuleName	( void )
{
	if ( ( !ActiveProject ) || ( CurrModule == -1 ) || ( lpGroup == NULL ) )
        	return( NULL );
        return( lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].Name );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectManagerForm :: GModuleFile	( void )
{
	if ( ( !ActiveProject ) || ( CurrModule == -1 ) || ( lpGroup == NULL ) )
        	return( NULL );

        return( lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].File );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SGroupName		( AnsiString Value )
{
	if ( ( !ActiveGroup ) || ( lpGroup == NULL ) || ( Value.Length() <= 0 ) )
        	return;

	if ( lpGroup->Name != Value ) {

        	lpGroup->Name = Value;
                lpGroup->Modified = true;

                bModified = true;

                Caption = "Group [ " + Value + " ] in " + GroupHome;

                Update();
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SGroupHome		( AnsiString Value )
{
	if ( ( !ActiveGroup ) || ( lpGroup == NULL ) || ( Value.Length() <= 0 ) )
        	return;

        if ( lpGroup->Home != Value ) {
                lpGroup->Home = Value;
                lpGroup->Modified = true;
                Modified = true;

                Caption = "Group [ " + GroupName + " ] in " + GroupHome;

                Update();
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SProjectName		( AnsiString Value )
{
	if ( ( !ActiveProject ) || ( lpGroup == NULL ) || ( lpGroup->lpProjects == NULL ) )
        	return;

        if ( lpGroup->lpProjects[ CurrProject ].Name != Value ) {

        	lpGroup->Modified = true;
                lpGroup->lpProjects[ CurrProject ].Name = Value;
                lpGroup->lpProjects[ CurrProject ].Modified = true;

                RebuildTreeView( this );
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SProjectHome		( AnsiString Value )
{
	if ( ( !ActiveProject ) || ( lpGroup == NULL ) || ( lpGroup->lpProjects == NULL ) )
        	return;

        if ( lpGroup->lpProjects[ CurrProject ].Name != Value ) {
        	lpGroup->Modified = true;

                lpGroup->lpProjects[ CurrProject ].Home = Value;
                lpGroup->lpProjects[ CurrProject ].Modified = true;

                RebuildTreeView( this );

                if ( ProjectOptionsForm != NULL ) {
                	ProjectOptionsForm->ProjectHomeDir = Value;
                        if ( ProjectOptionsForm->Visible )
                        	ProjectOptionsForm->Update();
                }
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SModuleName		( AnsiString Value )
{
	if ( ( !ActiveProject ) || ( lpGroup == NULL ) ||
        	( lpGroup->lpProjects == NULL ) || ( lpGroup->lpProjects[ CurrProject ].lpModules != NULL ) )
                return;

        if ( lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].Name != Value ) {

        	lpGroup->Modified = true;
                lpGroup->lpProjects[ CurrProject ].Modified = true;

                lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].Name = Value;
                lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].Modified = true;

                RebuildTreeView( this );

                Update();
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: SModuleFile		( AnsiString Value )
{
	if ( ( !ActiveProject ) || ( lpGroup == NULL ) || ( lpGroup->lpProjects == NULL )
        	|| ( lpGroup->lpProjects[ CurrProject ].lpModules == NULL ) )
                return;

        if ( lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].File != Value ) {
        	lpGroup->Modified = true;

                lpGroup->lpProjects[ CurrProject ].Modified = true;
                lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].Modified = true;

                lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].File = Value;

                RebuildTreeView( this );

                Update();
        }

}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectManagerForm :: InProject( AnsiString FilePath )
{
	if ( ( !ActiveGroup ) || ( !ActiveProject ) )
        	return( false );

        if ( FilePath.Length() <= 0 )
        	return( false );

        if ( CurrProject <= 0 )
        	return( false );

        for ( int i=1; i<=lpGroup->lpProjects[ CurrProject ].wNumModules; i++ ) {
        	if ( lpGroup->lpProjects[ CurrProject ].lpModules[ i ].File == FilePath )
                	return( true );
        }

        return( false );
}
//---------------------------------------------------------------------------
//	Helper routines
//---------------------------------------------------------------------------
//	Activate the module in the current project determined by CurrModule
// and CurrProject
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: ActivateModule( TObject *Sender )
{       TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )
        	return;

        ClearSysMessages();

        SystemMessage( "Activate Module" );

        if ( !ActiveProject ) {
        	SystemMessage( "No active project, aborting module activation." );
        	return;
        }

        //	now we need to activate the module

        ListSelection->ListView1->Items->Clear();

        for ( int i=0; i<=lpGroup->lpProjects[ CurrProject ].wNumModules; i++ ) {
        	ListSelection->Add( lpGroup->lpProjects[ CurrProject ].lpModules[ i ].Name );
        }

        ListSelection->ShowModal();

        if ( ListSelection->ModalResult != mrOk ) {
        	SystemMessage( "Operation aborted." );
        	return;
        }

        if ( ListSelection->Selected.Length() <= 0 ) {
        	SystemMessage( "Bad selection." );
                return;
        }

        SystemMessage( "User requests module '" + ListSelection->Selected + "' is to be activated." );

        iCurrModule = ListSelection->ListView1->Selected->Index;

        MainForm->CreateMDIChild( lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].File );

        TMDIChild *pForm = (TMDIChild*)MDIChildren[ 0 ];

        if ( pForm == NULL )
        	return;

        pForm->Caption = lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].Name;
        pForm->FileName = lpGroup->lpProjects[ CurrProject ].lpModules[ CurrModule ].File;
        pForm->Index = CurrModule;

	SystemMessage( "Module activation completed. OK." );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: ActivateProject( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )	{
        	return;
        }

        ClearSysMessages();

        SystemMessage( "Project Activation" );

        if ( !ActiveGroup ) {
        	SystemMessage( "No active project group, please open or create one." );
                return;
        }

        ListSelection->ListView1->Items->Clear();

        for ( int i=0; i<=lpGroup->wNumProjects; i++ ) {
        	TListItem *pItem = ListSelection->ListView1->Items->Add();

                if ( pItem != NULL ) {
                	pItem->Caption = lpGroup->lpProjects[ i ].Name;
                }
        }

        ListSelection->ShowModal();

        if ( ListSelection->ModalResult != mrOk ) {
        	SystemMessage( "Project activation aborted." );
                return;
        }

        if ( ListSelection->Selected.Length() <= 0 ) {
        	SystemMessage( "Bad selection." );
                return;
        }

        iCurrProject = ListSelection->ListView1->Selected->Index;
        bActiveProject = true;

        Caption = lpGroup->Name + " : " + lpGroup->lpProjects[ CurrProject ].Name;

}
//---------------------------------------------------------------------------
//	Form creation event method
//---------------------------------------------------------------------------
void 	__fastcall 	TProjectManagerForm :: FormCreate( TObject *Sender )
{	int	ECode = E_NOERROR;
	//	These just clear the captions from the speed buttons on the form
        //	so that the icons can be shown, as they are direct links to the
        //	associated actions in the mainform

        SpeedButton1->Caption = "";	SpeedButton2->Caption = "";
        SpeedButton3->Caption = "";     SpeedButton4->Caption = "";
        SpeedButton5->Caption = "";

        //	Make sure the list view is blank before we proceed any further
        ListView1->Items->Clear();

        //	Now check the integrity of the lpGroup structure
        if ( lpGroup == NULL )
        	lpGroup = NewGroup( "Untitled", "C:\\My Documents" );

        bActiveGroup = bActiveProject = bModified = false;

        //	Need to read the last project information if available and then load the
        // project from disk

        if ( ( ECode = LoadFromRegistry( this ) ) != E_NOERROR ) {
        	SystemMessage( "Error obtaining project manager options from system registry." );
                SystemMessage( "ErrorCode " + IntToStr( ECode ) );
        } else {
        	SystemMessage( "Project Manager Options obtained OK from the system registry." );
        }
}
//---------------------------------------------------------------------------
//	This closes the current project
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: IDECloseProject( TObject *Sender )
{       TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )	{
        	bModified = false;
                bActiveProject = false;
                iCurrProject = -1;
                return;
        }

        if ( !ActiveGroup )	return;	//	no active group, so there are no projects at all

        if ( !ActiveProject )	return;	//	no active project in group, so bye!

        ClearSysMessages();

        SystemMessage( "Closing project " + ProjectName );

        if ( Modified ) {	//	Project modified ?

        	if ( PRJMAN_SaveProject( this ) != E_NOERROR ) {
                	//	Failed
                	SystemMessage( "Project NOT saved." );
                }
        }

        bModified = false;

        lpGroup->wCurrProject = 0;

        iCurrProject = -1;

        bActiveProject = false;

}
//---------------------------------------------------------------------------
//
//	Method		::	IDESaveProject
//	Parameter	::	TObject *Sender, originator of the call
//	Purpose		::	This is called from the MainFrom only. It saves
//				the project using the current Name ie
//				MainForm->Name property. It also makes sure that
//				the current directory is the same as the
//				project's home directory
//
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: PRJMAN_SaveProject( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )	{
        	return( E_BAD_SENDER );
        }

        if ( !ActiveProject )	{
        	return( E_NOPROJECT );
        }

        ClearSysMessages();

        SystemMessage( "PRJMAN_SaveProject" );

        if ( !SaveDialog->Execute() ) {
        	SystemMessage( "Save aborted." );
                return( E_ABORTED );
        }

        if ( SaveDialog->FileName.Length() <= 0 )
        	return( E_BAD_FILE );

        ClearSysMessages();

        SystemMessage( "Save project" );

        RichEdit1->Lines->Clear();

        RichEdit1->Lines->Add( "#\n#\tKonDev Project File\n#\n" );
        RichEdit1->Lines->Add( "#\n#\tGenerated on : " + SystemTimeDate() + "\n#\n" );

        RichEdit1->Lines->Add( "[PSXIDE]" );
        RichEdit1->Lines->Add( "Name=" + lpGroup->lpProjects->Name );
        RichEdit1->Lines->Add( "Home=" + lpGroup->lpProjects->Home );
        RichEdit1->Lines->Add( "Type=" + IntToStr( lpGroup->lpProjects->wType ) );
        RichEdit1->Lines->Add( "ModuleCount=" + IntToStr( lpGroup->lpProjects->wNumModules ) );

        RichEdit1->Lines->Add( "\n[MODULES]\n" );

        for ( WORD w=0; w<=lpGroup->lpProjects->wNumModules; w++ ) {

		RichEdit1->Lines->Add( "Module#" + IntToStr( w ) + "Name=" + lpGroup->lpProjects->lpModules[ w ].Name );
                RichEdit1->Lines->Add( "Module#" + IntToStr( w ) + "Home=" + lpGroup->lpProjects->lpModules[ w ].File );

                lpGroup->lpProjects->lpModules[ w ].Modified = false;
        }

        RichEdit1->Lines->SaveToFile( SaveDialog->FileName );

        SystemMessage( "Project saved to file " + SaveDialog->FileName + ". OK." );

        lpGroup->lpProjects[ CurrProject ].Modified = false;
        lpGroup->Modified = false;
        bModified = false;

	return( E_NOERROR );
}
//---------------------------------------------------------------------------
//	PRJMAN_OpenProject	Project Manager Action 'Open Project File'
//
//	*Sender		Component that wants this action to take place
//	returns		integer code dependant on parameters and file contents
//
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: PRJMAN_OpenProject( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);
	AnsiString	FileName = "Untitled";

        AnsiString	PName = NULL;
        AnsiString	PHome = NULL;
        WORD		PType = NULL;
        WORD		PCount = NULL;

        AnsiString	MName = NULL;
        AnsiString	MFile = NULL;

        int		iCount = -1;

        //	Valid call
        if ( Originator == NULL ) {
        	return( E_BAD_SENDER );
        }

        //	Clear message view and output which method this is
        ClearSysMessages();
        SystemMessage( "PRJMAN Open Project" );

        //	check for active modified groups and projects

	if ( ( ActiveGroup ) && ( Modified ) )	{
        	//	Active group and has been modified save it ?
                if ( EnviromentOptionsForm->AutoSaveEditorFiles )
                	IDECloseProjectGroup( this );

        }

        if ( ( ActiveProject ) && ( lpGroup->lpProjects[ CurrProject ].Modified ) ) {
        	if ( EnviromentOptionsForm->AutoSaveEditorFiles )
                	IDECloseProject( this );
        }

        //	Who called us ?
        if ( Originator->Name == "ProjectManager" )
        	FileName = OpenDialog->FileName;

        if ( Originator->Name == "MainForm" )
        	FileName = MainForm->OpenDialog->FileName;

        if ( FileName == "Untitled" )
        	return( E_BAD_FILE );

        //	Clear the group and recreate untitled for project addition
        //PRJMAN_CloseGroup( Sender );

        //	Load the file for processing
        RichEdit1->Lines->Clear();

        SystemMessage( "Loading project file [ " + FileName + " ]" );

        RichEdit1->Lines->LoadFromFile( FileName );

        //	process it, check for the presence of the header line
        // '[PSXIDE]'. This denotes the file is a project file.

        iCount = RichEdit1->Lines->IndexOf( "[PSXIDE]" );

        if ( iCount == -1 )	{
        	SystemMessage( "Header '[PSXIDE]' not found" );
                RichEdit1->Lines->Clear();
                return( E_BAD_HEADER );
        }

        //	Header was found, find the project name, home directory, type,
        // and number of modules, the current project is not stored in the file

        iCount = RichEdit1->Lines->IndexOfName( "Name" );
        PName = GetValue( RichEdit1->Lines->Strings[ iCount ] );

        if ( ( iCount == -1 ) || ( PName.Length() <= 0 ) ) {
        	SystemMessage( "Bad project name" );
                RichEdit1->Lines->Clear();
                return( E_BAD_NAME );
        }

        //	home directory

        iCount = RichEdit1->Lines->IndexOfName( "Home" );
        PHome = GetValue( RichEdit1->Lines->Strings[ iCount ] );

        if ( ( iCount == -1 ) || ( PName.Length() <= 0 ) ) {
        	SystemMessage( "Bad home directory" );
                RichEdit1->Lines->Clear();
                return( E_BAD_HOME );
        }

        //	type
        iCount = RichEdit1->Lines->IndexOfName( "Type" );
        PType = (WORD)StrToInt( GetValue( RichEdit1->Lines->Strings[ iCount ] ) );

        if ( ( iCount == -1 ) || ( PType == NULL ) ) {
        	SystemMessage( "Bad type" );
                RichEdit1->Lines->Clear();
                return( E_BAD_TYPE );
        }

        //	number of modules
        iCount = RichEdit1->Lines->IndexOfName( "ModuleCount" );
        PCount = (WORD)StrToInt( GetValue( RichEdit1->Lines->Strings[ iCount ] ) );

        if ( ( iCount == -1 ) || ( PCount == NULL ) ) {
        	SystemMessage( "Bad module count" );
                RichEdit1->Lines->Clear();
                return( E_BAD_COUNT );
        }

        lpGroup = NewGroup( "Untitled", "C:\\My Documents" );

        lpGroup->Modified = true;
        lpGroup->wNumProjects = 1;
        lpGroup->wCurrProject = 1;

        lpGroup->lpProjects = NewProject( PName, PHome, PType );

        lpGroup->lpProjects->wNumModules = PCount;
        lpGroup->lpProjects->Modified = true;

        lpGroup->lpProjects->lpModules = new MODULE[ PCount + 1 ];
        ZeroMemory( lpGroup->lpProjects->lpModules, sizeof( MODULE ) * ( PCount + 1 ) );

	for ( WORD w=0; w<=lpGroup->lpProjects->wNumModules; w++ ) {

        	iCount = RichEdit1->Lines->IndexOfName( "Module#" + IntToStr( w ) + "Name" );
                MName = GetValue( RichEdit1->Lines->Strings[ iCount ] );

                if ( ( iCount == -1 ) || ( MName.Length() <= 0 ) && ( w <= lpGroup->lpProjects->wNumModules ) ) {
                	SystemMessage( "Module " + IntToStr( w ) + " Name not found" );
                        continue;
                }

                lpGroup->lpProjects->lpModules[ w ].Name = PName;

                iCount = RichEdit1->Lines->IndexOfName( "Module#" + IntToStr( w ) + "File" );
                MFile = GetValue( RichEdit1->Lines->Strings[ iCount ] );

                if ( ( iCount == -1 ) || ( MFile.Length() <= 0 ) && ( w <= lpGroup->lpProjects->wNumModules ) ) {
                	SystemMessage( "Module " + IntToStr( w ) + " File not found" );
                        continue;
                }

                lpGroup->lpProjects->lpModules[ w ].File = MFile;

                SystemMessage( "Module : " + lpGroup->lpProjects->lpModules[ w ].Name + " loaded." );
        }

        SystemMessage( "Project " + lpGroup->lpProjects->Name + " loaded. OK." );

        bActiveProject = bActiveGroup = true;

        iCurrProject = 1;
        iCurrModule = 0;

        RichEdit1->Lines->Clear();

        return( E_NOERROR );
}
//---------------------------------------------------------------------------
//
//	Method		::	IDEGenerateMakeFile
//	Parameters	::	TObject *Sender, originator of the call
//	Purpose		::	Creates the makefile for the current project
//				NOTE is highly dependant on the contents of the
//				"ProjectOptionsForm" as it takes the make file
//				variables from it.
//
//      NOTE THAT THIS INFORMATION IS VALID FOR GNU MAKE 3.79 ONLY, IF YOU WISH TO
//	USE ANOTHER VERSION OF MAKE, IE BORLAND, MS, WATCOM THEN THIS FILE "MAY"
//	NEED TO BE CHANGED. AS LONG AS THE MAKE, AND ASSOCIATED BIN UTILS ARE "GNU"
//	THEN IT SHOULD WORK OK. AS I DO NOT WANT TO WRITE SEVERAL VERSIONS OF THIS
//	OR ANY OTHER METHOD TO WRITE BORLAND OR MS SPECIFIC DATA, MAYBE IN A LATER
//	VERSION EH?
//
//	This is called from the MainForm and when the options are changed, from
//	various parts with in the mainform's application generation processes,
//	it MUST be called after the project options have been set, as it uses
//	the details stored in the Project Options Form to create the make file
//	so that make can process it correctly for the given target(s)
//
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: IDEGenerateMakeFile( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( !Originator )	return;	//	null caller

        if ( !ActiveProject )	{	//	No active project
        	SystemMessage( "No active project." );
                return;
        }

        if ( GetCurrentDir() != ProjectOptionsForm->ProjectHomeDir ) {
        	SetCurrentDir( ProjectOptionsForm->ProjectHomeDir );
        }

        SystemMessage( "Auto-Generating project's " + MainForm->Name + " makefile on " + SystemDate() + " @ " + SystemTime() );

        RichEdit1->Lines->Clear();

        RichEdit1->Lines->Add( "#\n#\tPSX IDE Generated Makefile\n#\n" );
        RichEdit1->Lines->Add( "#\n#\tGenerated on " + SystemDate() + " @ " + SystemTime() );

        RichEdit1->Lines->Add( "PROJECT\t=\t" + MainForm->Name );
        RichEdit1->Lines->Add( "TITLE\t=\t" + ProjectOptionsForm->ProjectTitle );
        RichEdit1->Lines->Add( "HOMEDIR\t=\t" + ProjectOptionsForm->ProjectHomeDir );
        RichEdit1->Lines->Add( "SRCDIR\t=\t$(HOMEDIR)\\src" );
        RichEdit1->Lines->Add( "INCDIR\t=\t$(HOMEDIR)\\include" );
        RichEdit1->Lines->Add( "LIBDIR\t=\t$(HOMEDIR)\\lib" );

        AnsiString as = "MODULES\t=\t";
        for ( int i=0; i<=ListView1->Items->Count; i++ ) {
        	if ( ( i < ListView1->Items->Count ) && ( ListView1->Items->Item[ i ]->Caption.Length() > 0 ) &&
			( ListView1->Items->Item[ i ]->SubItems->Strings[ 0 ].Length() > 0 ) )
	        	as += ListView1->Items->Item[ i ]->SubItems->Strings[ 0 ] + " ";
        }

        RichEdit1->Lines->Add( as );

        RichEdit1->Lines->Add( "CFLAGS\t+=\t-I $(INCDIR)" );

        if ( ProjectOptionsForm->MakeApplication.Length() <= 0 )
        	RichEdit1->Lines->Add( "MAKE\t=" );
        else	RichEdit1->Lines->Add( "MAKE\t=\t" + ProjectOptionsForm->MakeApplication );

        if ( ProjectOptionsForm->CompilerPath.Length() <= 0 )
        	RichEdit1->Lines->Add( "GCC\t=" );
        else	RichEdit1->Lines->Add( "GCC\t=\t" + ProjectOptionsForm->CompilerPath );

        if ( ProjectOptionsForm->AssemblerPath.Length() <= 0 )
        	RichEdit1->Lines->Add( "AS\t=" );
        else	RichEdit1->Lines->Add( "AS\t=\t" + ProjectOptionsForm->AssemblerPath );

        if ( ProjectOptionsForm->LinkerPath.Length() <= 0 )
        	RichEdit1->Lines->Add( "LD\t=" );
        else	RichEdit1->Lines->Add( "LD\t=\t" + ProjectOptionsForm->LinkerPath );


        //	PHONY TARGET

        RichEdit1->Lines->Add( ".PHONY\t:\tDefault" );


        RichEdit1->Lines->Add( "Default\t:" );
        RichEdit1->Lines->Add( "\t@echo Usage for project " + MainForm->Name );
        RichEdit1->Lines->Add( "\t@echo make clean\t\tRemoves all object and executable files, ie 'cleans' up" );
        RichEdit1->Lines->Add( "\t@echo make config\t\tCreates a configuration binary and executes it" );
        RichEdit1->Lines->Add( "\t@echo \t\t\tNote that this is for LINUX and not WINDOWS yet. " );
        RichEdit1->Lines->Add( "\t@echo make make\t\tBuilds the project, compiling only files that have changed" );
        RichEdit1->Lines->Add( "\t@echo make build\t\tBuilds the project, all files are remade" );
        RichEdit1->Lines->Add( "\t@echo make install\t\tBuilds the project, then copies file to install dir" );
        RichEdit1->Lines->Add( "\t@echo make uninstall\t\tRemoves the installed project from the system" );
        RichEdit1->Lines->Add( "\t@echo make $(PROJECT)\t\tSame as build, but with a different name" );



        RichEdit1->Lines->Add( "$(PROJECT).exe\t:\t$(PROJECT).cpp $(MODULES)" );
        RichEdit1->Lines->Add( "\t$(make)" );

        RichEdit1->Lines->Add( "$(PROJECT).cpp\t:\t" );
        RichEdit1->Lines->Add( "\t$(GCC) $(CFLAGS) $(MODULES) $(PROJECT).cpp -o $(PROJECT).exe" );

        RichEdit1->Lines->Add( "$(MODULES)\t:" );
        RichEdit1->Lines->Add( "\t$(GCC) -c $(CFLAGS) $(MODULES) -o $(subst $(MODULES) %.cpp, %.o )" );

        RichEdit1->Lines->Add( "Modules\t:" );
        RichEdit1->Lines->Add( "\t@echo Compiling $(MODULES)" );
        RichEdit1->Lines->Add( "\t$(MODULES)" );

        //	CLEAN TARGET

        RichEdit1->Lines->Add( "clean\t:" );
        RichEdit1->Lines->Add( "\t@del $(PROJECT).exe" );
        RichEdit1->Lines->Add( "\t@del $(wildcard *.o)" );
        RichEdit1->Lines->Add( "\t@del $(SRCDIR)\\$(wildcard *.o)" );

        RichEdit1->Lines->Add( "config\t:" );
        RichEdit1->Lines->Add( "\t@$(clean)" );
        RichEdit1->Lines->Add( "\t@echo \"Configure options not relevent, yet, to Windows.\"" );

        RichEdit1->Lines->Add( "make\t:\t$(PROJECT).cpp $(MODULES)" );
        RichEdit1->Lines->Add( "\t@$(GCC) $(CFLAGS) $(MODULES) $(PROJECT).cpp -o $(PROJECT).exe" );

        RichEdit1->Lines->Add( "build\t:\t$(PROJECT).cpp $(MODULES)" );
        RichEdit1->Lines->Add( "\t@clean" );
        RichEdit1->Lines->Add( "\t@$(make)" );

        RichEdit1->Lines->Add( "install\t:\t$(PROJECT).exe" );
        RichEdit1->Lines->Add( "\t@$(make)" );
        RichEdit1->Lines->Add( "\t@md c:\\progra~1\\$(PROJECT)" );
        RichEdit1->Lines->Add( "\t@copy $(PROJECT).exe C:\\progra~1\\$(PROJECT)\\$(project).exe" );

        RichEdit1->Lines->Add( "uninstall\t:\tc:\\progra~1\\$(PROJECT)\\$(PROJECT).exe" );
        RichEdit1->Lines->Add( "\t@$(clean)" );
        RichEdit1->Lines->Add( "\t@del c:\\progra~1\\$(PROJECT)\\$(PROJECT).exe" );
        RichEdit1->Lines->Add( "\t@rd c:\\progra~1\\$(PROJECT)" );

        RichEdit1->Lines->Add( "$(PROJECT)\t:\t$(PROJECT).cpp $(HOMEDIR)\\$(wildcard *.cpp)" );
        RichEdit1->Lines->Add( "\t@$(build)" );

        RichEdit1->Lines->SaveToFile( "Makefile" );

        SystemMessage( "Generation completed : @ " + SystemTime() );
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: IDESaveProjectGroup( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )	return( false );

        if ( lpGroup == NULL )	{
        	SystemMessage( "[ INTERNAL ERROR : lpGroup == NULL ]" );
                return( false );
        }

        ClearSysMessages();

        SystemMessage( "[ IDESaveProjectGroup ]" );

        //	Clear workarea and add the header
        RichEdit1->Lines->Clear();
        RichEdit1->Lines->Add( "[KONDEV]" );

        RichEdit1->Lines->Add( "Name=" + lpGroup->Name );
        RichEdit1->Lines->Add( "Home=" + lpGroup->Home );
        RichEdit1->Lines->Add( "NumProjects=" + IntToStr( lpGroup->wNumProjects ) );

        RichEdit1->Lines->Add ( "[PROJECTS]" );

        for ( WORD w=0; w<=lpGroup->wNumProjects; w++ ) {
        	RichEdit1->Lines->Add( "Project#" + IntToStr( w ) + "Name=" + lpGroup->lpProjects[ w ].Name );
                RichEdit1->Lines->Add( "Project#" + IntToStr( w ) + "Home=" + lpGroup->lpProjects[ w ].Home );
        }

        lpGroup->Modified = false;

        if ( GetCurrentDir() != lpGroup->Home )
        	SetCurrentDir( lpGroup->Home );

        RichEdit1->Lines->SaveToFile( lpGroup->Name + ".psxgrp" );

        RichEdit1->Lines->Clear();

        return( true );

}
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: PRJMAN_OpenGroup( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);
	AnsiString	asTemp = NULL;
	int		iCount = -1;

	if ( Originator == NULL )	{
        	bActiveProject = bActiveGroup = false;
                iCurrModule = iCurrProject = -1;
        	return( E_BAD_SENDER );
        }

        ClearSysMessages();

        SystemMessage( "IDELoadProjectGroup" );

        if ( ActiveGroup )	{
        	SystemMessage( "Closing current project group." );
                IDECloseProjectGroup( this );
        }

        SystemMessage( "Continuing load operation." );

        if ( !OpenDialog->Execute() ) {
        	SystemMessage( "Load aborted by user." );
                return( E_ABORTED );
        }

        SystemMessage( "Loading from file " + OpenDialog->FileName );

        RichEdit1->Lines->Clear();

        RichEdit1->Lines->LoadFromFile( OpenDialog->FileName );

        if ( RichEdit1->Lines->IndexOf( "[KONDEV]" ) == -1 ) {
        	SystemMessage( "This is not a group file." );
                RichEdit1->Lines->Clear();

                bActiveGroup = false;

                iCurrProject = -1;

                return( E_BAD_FILE );
        }

        //	Header found decode name and home, then change to home if needed

        iCount = RichEdit1->Lines->IndexOfName( "Name" );

        if ( iCount == -1 ) {	//	Not found
        	SystemMessage( "No group name found." );
                RichEdit1->Lines->Clear();
                bActiveGroup = false;
                iCurrProject = -1;
                return( E_BAD_FILE );
        }

        asTemp = GetValue( RichEdit1->Lines->Strings[ iCount ] );
        if ( asTemp.Length() <= 0 ) {
        	SystemMessage( "Bad name." );
                RichEdit1->Lines->Clear();
                bActiveGroup = false;
                iCurrProject = -1;
                return( E_BAD_FILE );
        }


	if ( lpGroup == NULL )
        	lpGroup = new GROUP;

        ZeroMemory( lpGroup, sizeof( GROUP ) );

        lpGroup->Name = asTemp;

        asTemp = NULL;

        iCount = RichEdit1->Lines->IndexOfName( "Home" );
        if ( iCount == -1 ) {
        	SystemMessage( "Bad home" );
                RichEdit1->Lines->Clear();
                bActiveGroup = false;
                iCurrProject = -1;
                return( E_BAD_FILE );
        }

        asTemp = GetValue( RichEdit1->Lines->Strings[ iCount ] );

        if ( asTemp.Length() <= 0 ) {
        	SystemMessage( "Bad home" );
                RichEdit1->Lines->Clear();
                bActiveGroup = false;
                iCurrProject = -1;
                return( E_BAD_FILE );
        }

        lpGroup->Home = asTemp;

        if ( GetCurrentDir() != lpGroup->Home ) {
        	SystemMessage( "Change to " + lpGroup->Home );
        	SetCurrentDir( lpGroup->Home );
        }

        //	0 'zero' projects are valid for project group files
        iCount = RichEdit1->Lines->IndexOfName( "NumProjects" );
        if ( iCount == -1 ) {
        	SystemMessage( "No projects in group" );
                RichEdit1->Lines->Clear();
                iCurrProject = -1;
                bActiveGroup = true;
                return( E_NOERROR );
        }

        asTemp = GetValue( RichEdit1->Lines->Strings[ iCount ] );

        if ( asTemp.Length() <= 0 ) {
        	SystemMessage( "No projects in group" );
        	RichEdit1->Lines->Clear();
                iCurrProject = -1;
                bActiveGroup = true;
                return( E_NOERROR );
        }

        SystemMessage( "Number of projects : " + StrToInt( asTemp ) );

        iCount = StrToInt( asTemp );

        lpGroup->wNumProjects = (WORD)iCount;
        lpGroup->wCurrProject = 0;
        lpGroup->Modified = true;

        iCurrProject = -1;

        SystemMessage( "Locating project header section" );

        if ( RichEdit1->Lines->IndexOf( "[PROJECTS]" ) == - 1 ) {
        	SystemMessage( "Project header section not found." );
                RichEdit1->Lines->Clear();
                bActiveGroup = true;
                iCurrProject = -1;
                lpGroup->wNumProjects = 0;
                lpGroup->wCurrProject = 0;
                return( E_NOERROR );
        }

        lpGroup->lpProjects = *NewProjects( iCount );

        for ( int i=0; i<=iCount; i++ ) {
        	AnsiString	asLine = NULL;
                int		iLine = -1;

                //	Project Name
                iLine = RichEdit1->Lines->IndexOfName( "Project#" + IntToStr( i ) + "Name" );
                if ( iLine == -1 ) {
                	SystemMessage( "Project " + IntToStr( i ) + " Name not found." );
                        continue;
                } else {
                	asLine = GetValue( RichEdit1->Lines->Strings[ i ] );
                        if ( asLine.Length() <= 0 ) {
                        	SystemMessage( "Bad name for project " + IntToStr( i ) );
                                continue;
                        } else {
                        	SystemMessage( "Project " + IntToStr( i ) + " named " + asLine );
                        	lpGroup->lpProjects[ i ].Name = asLine;
                        }
                }

                //	Project home directory
                iLine = RichEdit1->Lines->IndexOfName( "Project#" + IntToStr( i ) + "Home" );
                if ( iLine == -1 ) {
                	SystemMessage( "Project " + IntToStr( i ) + " home not found." );
                        continue;
                } else {
                	asLine = GetValue( RichEdit1->Lines->Strings[ iLine ] );
                        if ( asLine.Length() <= 0 ) {
                        	SystemMessage( "Bad home for project " + IntToStr( i ) );
                                continue;
                        } else {
                        	SystemMessage( "Project " + IntToStr( i ) + " home " + asLine );
                                lpGroup->lpProjects[ i ].Name = asLine;
                        }
                }

        }


/*

	//	Load the project 'i', note disabled due to the fact that the
        //	RichEdit1 component is used during all load and save operations
        //	and is cleared prior to each, so calling IDELoadProject clears
        //	the project group file, so DONT CALL IT YET! or change IDELoadProject

        if ( ( lpGroup->lpProjects[ i ]->Name.Length() > 0 ) && ( lpGroup->lpProjects[ i ]->Home.Length() > 0 ) ) {

        	OpenDialog->FileName = lpGroup->lpProjects[ i ]->Home + lpGroup->lpProjects[ i ]->Name + ".psxide";

                IDELoadProject( this );

        }

*/

	bActiveGroup = true;
        bActiveProject = true;
        iCurrProject = 0;
        iCurrModule = -1;

        return( E_NOERROR );

}
//---------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: IDECloseProjectGroup( TObject *Sender )
{
	if ( Sender == NULL )
        	return;

	if ( lpGroup == NULL ) {
        	bActiveProject = bActiveGroup = false;
                iCurrProject = iCurrModule = -1;
                return;
        }

        ClearSysMessages();

        SystemMessage( "IDECloseProject" );

        if ( lpGroup->Modified ) {
        	if ( EnviromentOptionsForm->AutoSaveEditorFiles ) {
                	SystemMessage( "Auto-saving." );
                	IDESaveProjectGroup( this );
                }
        }

        bActiveGroup = bActiveProject = false;
        iCurrProject = iCurrModule = -1;

        lpGroup->Modified = false;

        ListView1->Items->Clear();
        bModified = false;

        Caption = "No active projects and/or groups";
}
//----------------------------------------------------------------------------
void	__fastcall	TProjectManagerForm :: RebuildTreeView( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( ( Originator == NULL ) || ( lpGroup == NULL ) )	//	Nothing to rebuild or bad call
        	return;

        ClearSysMessages();

        SystemMessage( "Rebuilding project view" );

        Cursor = crHourGlass;	//	Change cursor to hourglass

        ListView1->Items->Clear();	//	clear the current view and rebuild

        for ( WORD w=0; w<=lpGroup->wNumProjects; w++ )	{
        	TListItem	*pProject = ListView1->Items->Add();

                pProject->Caption = lpGroup->lpProjects[ w ].Name;
                for ( WORD x=0; x<=lpGroup->lpProjects[ w ].wNumModules; x++ ) {
                	if ( w == 0 )
                        	pProject->SubItems->Add( "Main project module" );
                        else pProject->SubItems->Add( "" );
                	pProject->SubItems->Add( lpGroup->lpProjects[ w ].lpModules[ x ].Name );
                        pProject->SubItems->Add( lpGroup->lpProjects[ w ].lpModules[ x ].File );
                }
        }
        ListView1->Selected = NULL;

        Cursor = crDefault;

        SystemMessage( "Rebuild complete. OK." );

        Update();
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: SaveToRegistry( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);

	if ( Originator == NULL )
        	return( E_BAD_SENDER );

        Registry = new TRegistry;

        if ( Registry == NULL )
        	return( E_BAD_REGISTRY );

        Registry->LazyWrite = false;

        Registry->OpenKey( PRJ_MANAGER_OPTIONS, true );

        if ( ActiveGroup ) {
        	Registry->WriteString( "LastGroup", lpGroup->Name );
                Registry->WriteString( "GroupHome", lpGroup->Home );
        } else {
        	Registry->WriteString( "LastGroup", NULL );
                Registry->WriteString( "GroupHome", NULL );
        }

        if ( ActiveProject ) {
        	Registry->WriteString( "LastProject", lpGroup->lpProjects[ CurrProject ].Name );
                Registry->WriteString( "ProjectHome", lpGroup->lpProjects[ CurrProject ].Home );
        } else {
        	Registry->WriteString( "LastProject", NULL );
                Registry->WriteString( "ProjectHome", NULL );
        }

        Registry->CloseKey();

        delete Registry;

	return( E_NOERROR );
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectManagerForm :: LoadFromRegistry( TObject *Sender )
{
	if ( Sender == NULL )
        	return( E_BAD_SENDER );

        Registry = new TRegistry;

        if ( Registry == NULL )
        	return( E_BAD_REGISTRY );

        if ( !Registry->OpenKey( PRJ_MANAGER_OPTIONS, false ) ) {
        	delete Registry;
                return( E_BAD_REGISTRY );
        }

        asLastGroupName = Registry->ReadString( "LastGroup" );
        asLastGroupHome = Registry->ReadString( "GroupHome" );

        asLastProjectName = Registry->ReadString( "LastProject" );
        asLastProjectHome = Registry->ReadString( "ProjectHome" );

        Registry->CloseKey();

        delete Registry;

	return( E_NOERROR );
}
//---------------------------------------------------------------------------
void __fastcall TProjectManagerForm::FormDestroy(TObject *Sender)
{	int	ECode = E_NOERROR;

	if ( !Modified )	return;
/*
	if ( ( ECode = SaveToRegistry( this ) ) != E_NOERROR ) {
        	SystemMessage( "Unable to save project manager options to system registry." );
        } else {
        	SystemMessage( "Project Manager Options saved OK to system registry." );
        }*/
}
//---------------------------------------------------------------------------
