//---------------------------------------------------------------------------
//
//      $FileName       ::      ProjectManager.h
//      $Author         ::      Chris Brooklyn
//      $Description    ::      Include Header for project management
//
//---------------------------------------------------------------------------
#ifndef ProjectManagerH
#define ProjectManagerH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <Menus.hpp>
#include <ToolWin.hpp>
#include <Buttons.hpp>
#include <Dialogs.hpp>
#include <Windows.hpp>
#include <Registry.hpp>
#include <SysUtils.hpp>
#include <Registry.hpp>
//----------------------------------------------------------------------------
#include "Defines.h"
#include "RegEntries.h"
#include "ProjectOptionsFormUnit.h"
#include <ActnList.hpp>
//----------------------------------------------------------------------------
//	The following three structures are for producing a linked list of the
// current project group, it projects and their individual modules
//---------------------------------------------------------------------------
//	Project Module definition, very simple
//----------------------------------------------------------------------------
typedef	struct	_prj_module_	{
	AnsiString	Name;
        AnsiString	File;
        bool		Modified;
        } MODULE, *LPMODULE;
//----------------------------------------------------------------------------
//	Project definition, just a simple structure
//----------------------------------------------------------------------------
typedef	struct	_project_	{
	AnsiString	Name;
        AnsiString	Home;
        WORD		wNumModules;
        WORD		wCurrModule;
        WORD		wType;
        LPMODULE	lpModules;
        bool		Modified;
        } PROJECT, *LPPROJECT;
//----------------------------------------------------------------------------
typedef	struct	_group_	{
	AnsiString	Name;
        AnsiString	Home;
        WORD		wNumProjects;
        WORD		wCurrProject;
        LPPROJECT	lpProjects;
        bool		Modified;
        } GROUP, *LPGROUP;
//----------------------------------------------------------------------------
class TProjectManagerForm : public TForm
        {

        friend  class   TMainForm;
        friend  class   TMDIChild;
        protected:

        	int	__fastcall	SaveToRegistry		( TObject *Sender );
                int	__fastcall	LoadFromRegistry	( TObject *Sender );

        	bool	__fastcall	InProject		( AnsiString FilePath );

        	//	Quick methods for creating groups, projects and modules
        	LPGROUP		__fastcall	NewGroup	( AnsiString GName, AnsiString GHome );
                LPPROJECT	__fastcall	NewProject	( AnsiString PName, AnsiString PHome, WORD PType );
                LPPROJECT*	__fastcall	NewProjects	( int NumProjects );
                LPMODULE	__fastcall	NewModule	( AnsiString MName, AnsiString MFile = NULL );

                //	Generates the makefile for the current project
        	void	__fastcall	IDEGenerateMakeFile	( TObject *Sender );

                //	Save the current project
                int	__fastcall	PRJMAN_SaveProject	( TObject *Sender );

                //	Loads an existing project
                int	__fastcall	PRJMAN_OpenProject	( TObject *Sender );

                //	Closes the current project
                void	__fastcall	IDECloseProject		( TObject *Sender );

                //	Activate a project
                void	__fastcall	ActivateProject		( TObject *Sender );

                //	Activate a module
                void	__fastcall	ActivateModule		( TObject *Sender );

                //	Save, Load, close for project groups

                //	Method to open a project group file
                int	__fastcall	PRJMAN_OpenGroup	( TObject *Sender );

                int	__fastcall	IDESaveProjectGroup	( TObject *Sender );
                void	__fastcall	IDECloseProjectGroup	( TObject *Sender );

                void	__fastcall	RebuildTreeView		( TObject *Sender );

        __published:	                                // IDE-managed Components
                TToolBar        *ToolBar1;
                TListView       *ListView1;
                TSpeedButton    *SpeedButton1;
                TToolButton     *ToolButton1;
                TToolButton     *ToolButton2;
                TToolButton     *ToolButton3;
                TSpeedButton    *SpeedButton3;
                TToolButton     *ToolButton4;
                TSpeedButton    *SpeedButton4;
                TSpeedButton    *SpeedButton5;
                TEdit           *Edit1;
		TSpeedButton 	*SpeedButton2;
		TStatusBar 	*StatusBar1;
		TRichEdit 	*RichEdit1;
	TOpenDialog *OpenDialog;
	TSaveDialog *SaveDialog;
	TPopupMenu *PopupMenu1;
	TMenuItem *AddToProject;
	TActionList *ActionList1;
	TAction *PRJMAN_OpenGroupAction;
                void    __fastcall      FormCreate              ( TObject *Sender );
	void __fastcall FormDestroy(TObject *Sender);

        private:// User declarations
        	TRegistry	*Registry;

                AnsiString	asLastProjectName;
                AnsiString	asLastProjectHome;

                AnsiString	asLastGroupName;
                AnsiString	asLastGroupHome;

                LPGROUP		lpGroup;

        	//	Property	'Modified'
        	bool		bModified;
                bool	__fastcall	GModified	( void );
                void	__fastcall	SModified	( bool Value );

                //	Property	'ActiveGroup'
                bool	bActiveGroup;
                bool	__fastcall	GActiveGroup	( void );
                void	__fastcall	SActiveGroup	( bool Value );

                //	Property	'ActiveProject'
                bool	bActiveProject;
                bool	__fastcall	GActiveProject	( void );
                void	__fastcall	SActiveProject	( bool Value );

                //	Property	'CurrProject'
                int	iCurrProject;
                int	__fastcall	GCurrProject	( void );
                void	__fastcall	SCurrProject	( int Value );

                //	Property	'CurrModule'
                int	iCurrModule;
                int	__fastcall	GCurrModule	( void );
                int	__fastcall	SCurrModule	( int Value );

                AnsiString	asGroupName;
                AnsiString	asGroupHome;
                AnsiString	asProjectName;
                AnsiString	asProjectHome;
                AnsiString	asModuleName;
                AnsiString	asModuleFile;

                AnsiString	__fastcall	GGroupName	( void );
                AnsiString	__fastcall	GGroupHome	( void );

                AnsiString	__fastcall	GProjectName	( void );
                AnsiString	__fastcall	GProjectHome	( void );

                AnsiString	__fastcall	GModuleName	( void );
                AnsiString	__fastcall	GModuleFile	( void );

                void	__fastcall	SGroupName		( AnsiString Value );
                void	__fastcall	SGroupHome		( AnsiString Value );

                void	__fastcall	SProjectName		( AnsiString Value );
                void	__fastcall	SProjectHome		( AnsiString Value );

                void	__fastcall	SModuleName		( AnsiString Value );
                void	__fastcall	SModuleFile		( AnsiString Value );

        public:		                                        // User declarations
                        __fastcall      TProjectManagerForm     ( TComponent* Owner );

        __published:
        	__property 	bool 	Modified 	= { read=GModified, write=SModified, stored=true, default=false };

                __property	bool	ActiveGroup	= { read=GActiveGroup, stored=true, default=false };
                __property	bool	ActiveProject	= { read=GActiveProject, stored=true, default=false };

                __property	int	CurrProject	= { read=GCurrProject, stored=true, default=-1 };
                __property	int	CurrModule	= { read=GCurrModule, stored=true, default=-1 };

                __property	AnsiString	GroupName = { read=GGroupName, write=SGroupName };
                __property	AnsiString	GroupHome = { read=GGroupHome, write=SGroupHome };

                __property	AnsiString	ProjectName = { read=GProjectName, write=SProjectName };
                __property	AnsiString	ProjectHome = { read=GProjectHome, write=SProjectHome };

                __property	AnsiString	ModuleName = { read=GModuleName, write=SModuleName };
                __property	AnsiString	ModuleFile = { read=GModuleFile, write=SModuleFile };

};
//---------------------------------------------------------------------------
extern PACKAGE  TProjectManagerForm     *ProjectManagerForm;
//----------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------

