//---------------------------------------------------------------------------
//
//      $FileName       ::      ProjectOptionsForm.cpp
//      $Author         ::      Chris Brooklyn
//      $Description    ::      Project Options Form Class Handling Code
//      $Version        ::      0.2.0.220
//
//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
//---------------------------------------------------------------------------
#include "ProjectOptionsFormUnit.h"
#include "EnviromentOptionsUnit.h"
#include "ProjectManager.h"
#include "main.h"
#include "DebuggingUnit.h"
#include "TDirSelectDialog.h"
#include "Utilities.h"
#include "CodeOptimisationSelect.h"
#include "WarningSelect.h"
#include "ListDialog.h"
#include "shellapi.h"
#include "GetAppFileVersionInfoStrings.h"
#include "ValueDialogUnit.h"
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TProjectOptionsForm 		*ProjectOptionsForm;
GetApplicationFileVersionInfo 	*dll_version;
//---------------------------------------------------------------------------
//
//	Method		::	TProjectOptionsForm ( CONSTRUCTOR )
//	Parameters	::	TObject *Owner, creator of the form, usually Application
//
//---------------------------------------------------------------------------
__fastcall TProjectOptionsForm::TProjectOptionsForm(TComponent* Owner)
        : TForm(Owner)
{
        asProjectIconFile = "";
        ConfigureForDefault( this );
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectOptionsForm :: SaveToRegistry( TObject *Sender )
{	TComponent	*Originator = dynamic_cast< TComponent* >(Sender);
	AnsiString	MainKeyName = NULL;
        AnsiString	KeyName;

        if ( Originator == NULL )
        	return( E_BAD_SENDER );

        ClearSysMessages();

        SystemMessage( "PRJOPT SaveToRegistry" );

        Registry = new TRegistry;

        if ( Registry == NULL ) {
        	SystemMessage( "Unable to access registry." );
                return( E_BAD_REGISTRY );
        }

        Registry->LazyWrite = false;

        if ( !ProjectManagerForm->ActiveProject ) {
        	SystemMessage( "No active project, please create or open one." );
                delete Registry;
                return( E_BAD_PROJECT );
        }

        if ( ProjectManagerForm->ProjectName.Length() > 0 )
        	MainKeyName = AnsiString( PRJ_OPTIONS_DEFAULT + "\\" + ProjectManagerForm->ProjectName );

        if ( MainKeyName.Length() <= 0 ) {
        	SystemMessage( "Bad registry key name." );
                return( E_BAD_REGISTRY );
        }

        SystemMessage( "Save project " + ProjectManagerForm->ProjectName + "'s options to " + MainKeyName );

        //	Main project options key-path
	Registry->OpenKey( MainKeyName, true );	//	Force create the key
        Registry->CloseKey();

        //	project options 'application'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_APPLICATION );
        Registry->OpenKey( KeyName, true );	//	Force create

        Registry->WriteString( "ProjectTitle", ProjectTitle );
        Registry->WriteString( "ProjectHome", ProjectHomeDir );
        Registry->WriteInteger( "PlatformId", PlatformId );
        Registry->WriteString( "HelpFile", HelpFile );
        Registry->WriteString( "IconFile", IconFile );
        Registry->WriteInteger( "Language", Language );
        Registry->WriteInteger( "Interface", Interface );

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'compiler'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_COMPILER );
        Registry->OpenKey( KeyName, true );

        Registry->WriteInteger( "OptimisationLevel", OptimisationLevel );
        Registry->WriteInteger( "WarningLevel", WarningLevel );
        Registry->WriteInteger( "CallingConvention", TargetCallingConvention );
        Registry->WriteInteger( "MakeTarget", MakeTarget );

        Registry->WriteBool( "DebugMode", DebugMode );
        Registry->WriteBool( "LineNumbering", LineNumbering );
        Registry->WriteBool( "InlineExpansion", InlineExpansion );

        Registry->WriteString( "MakePath", MakeApplication );
        Registry->WriteString( "MakeFile", MakeFileName );

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'advanced compiler'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_ADVANCED_COMPILER );
        Registry->OpenKey( KeyName, true );

        Registry->WriteInteger( "InstructionSet", TargetInstructionSet );
        Registry->WriteInteger( "DataAlignment", TargetDataAlignment );
        Registry->WriteInteger( "RegisterVars", TargetRegisterVariables );

        Registry->WriteBool( "FPUNone", FPUNone );
        Registry->WriteBool( "FPUFast", FPUFast );
        Registry->WriteBool( "FPUSoft", FPUSoft );
        Registry->WriteBool( "FPUFix", FPUFix );

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'C++'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_CPP );
        Registry->OpenKey( KeyName, true );

        Registry->WriteString( "CompilerPath", CompilerPath );

        //	Flags
        for ( int i=0; i<=ListView2->Items->Count; i++ ) {
        	Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Caption" ),
                	CompilerFlags[ i ] );
                Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Description" ),
                	ListView2->Items->Item[ i ]->Caption );
                if ( ListView2->Items->Item[ i ]->Checked )
                	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), true );
                else	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), false );
        }

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'assembler'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_ASSEMBLER );
        Registry->OpenKey( KeyName, true );

        Registry->WriteString( "AssemblerPath", AssemblerPath );

        //	Flags
        for ( int i=0; i<=ListView3->Items->Count; i++ ) {
        	Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Caption" ),
                	CompilerFlags[ i ] );
                Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Description" ),
                	ListView3->Items->Item[ i ]->Caption );
                if ( ListView3->Items->Item[ i ]->Checked )
                	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), true );
                else	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), false );
        }

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'linker'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_LD );
        Registry->OpenKey( KeyName, true );

        Registry->WriteString( "LinkerPath", LinkerPath );

        //	Flags
        for ( int i=0; i<=ListView3->Items->Count; i++ ) {
        	Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Caption" ),
                	CompilerFlags[ i ] );
                Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Description" ),
                	ListView3->Items->Item[ i ]->Caption );
                if ( ListView3->Items->Item[ i ]->Checked )
                	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), true );
                else	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), false );
        }

        //	extra libraries
        for ( int i=0; i<=ListView4->Items->Count; i++ ) {
        	Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Caption" ),
                	CompilerFlags[ i ] );
                Registry->WriteString( AnsiString( "Flag#" + IntToStr( i ) + "Description" ),
                	ListView4->Items->Item[ i ]->Caption );
                if ( ListView4->Items->Item[ i ]->Checked )
                	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), true );
                else	Registry->WriteBool( AnsiString( "Flag#" + IntToStr( i ) + "Checked" ), false );
        }

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'Directories'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_DIRECTORIES );
        Registry->OpenKey( KeyName, true );

        Registry->WriteString( "IncludePath", IncludePath );
        Registry->WriteString( "LibraryPath", LibraryPath );
        Registry->WriteString( "DebugSource", DebugSourcePath );

        if ( TempOutputPiping ) {
        	Registry->WriteBool( "TempIOPipes", true );
                Registry->WriteString( "TempIOPath", "Pipe" );
        } else {
        	Registry->WriteBool( "TempIOPipes", false );
                Registry->WriteString( "TempIOPath", TempOutputPath );
        }

        Registry->WriteString( "FinalOutputPath", FinalOutputPath );
        Registry->WriteString( "LIBOutputPath", LibraryOutputPath );

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'conditionals'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_CONDITIONALS );
        Registry->OpenKey( KeyName, true );

        Registry->WriteString( "Conditionals", Conditionals );

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'unit aliases'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_ALIASES );
        Registry->OpenKey( KeyName, true );

        Registry->WriteString( "Aliases", UnitAliases );

        Registry->CloseKey();
        KeyName = NULL;

        //	project options 'version info'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_VERSION_INFORMATION );
        Registry->OpenKey( KeyName, true );

        if ( !IncludeVerInfo ) {
        	Registry->WriteBool( "VersionInformation", false );
                Registry->CloseKey();
                KeyName = NULL;
        } else {
        	Registry->WriteBool( "VersionInformation", true );
                Registry->WriteBool( "AutoIncrementVersion", AutoIncVersion );

                Registry->WriteBool( "AttrDebug", DebugAttr );
                Registry->WriteBool( "AttrPre", PreReleaseAttr );
                Registry->WriteBool( "AttrDLL", DLLAttr );
                Registry->WriteBool( "AttrSpecial", SpecialAttr );
                Registry->WriteBool( "AttrPrivate", PrivateAttr );
                Registry->WriteBool( "GPL", GPL );

                Registry->WriteInteger( "Major", Major );
                Registry->WriteInteger( "Minor", Minor );
                Registry->WriteInteger( "Revision", Revision );
                Registry->WriteInteger( "Build", Build );

                Registry->CloseKey();

                KeyName += AnsiString( "\\" + PRJ_OPTIONS_KEYS );
                Registry->OpenKey( KeyName, true );

                //	Write the keys
                for ( int i=0; i<=ListView1->Items->Count; i++ ) {
                	Registry->WriteString( AnsiString( "Key#" + IntToStr( i ) + "Name" ),
                        	ListView1->Items->Item[ i ]->Caption );
                        Registry->WriteString( AnsiString( "Key#" + IntToStr( i ) + "Value" ),
                        	ListView1->Items->Item[ i ]->SubItems->Strings[ 0 ] );
                }

                Registry->CloseKey();
                KeyName = NULL;
        }

        //	project options 'packages'
        KeyName = AnsiString( MainKeyName + "\\" + PRJ_OPTIONS_PACKAGES );
        Registry->OpenKey( KeyName, true );

        Registry->CloseKey();
        KeyName = NULL;

        delete Registry;
        return( E_NOERROR );

}
//---------------------------------------------------------------------------
int	__fastcall	TProjectOptionsForm :: LoadFromRegistry( TObject *Sender )
{
	return( false );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectOptionsForm :: GCFlags( int Index )
{
	if ( ( Index < 0 ) || ( Index > ListView2->Items->Count ) ) return( NULL );
        else return( ListView2->Items->Item[ Index ]->Caption );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectOptionsForm :: GASFlags( int Index )
{
	if ( ( Index < 0 ) || ( Index > ListView3->Items->Count ) ) return( NULL );
        else return( ListView3->Items->Item[ Index ]->Caption );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectOptionsForm :: GLDFlags( int Index )
{
	if ( ( Index < 0 ) || ( Index > ListView4->Items->Count ) ) return( NULL );
        else return( ListView4->Items->Item[ Index ]->Caption );
}
//---------------------------------------------------------------------------
AnsiString	__fastcall	TProjectOptionsForm :: GDBFlags( int Index )
{
	return( NULL );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SCFlags( int Index, AnsiString Value )
{
	if ( ( Index < 0 ) || ( Index > ListView2->Items->Count ) )	return;

        if ( ListView2->Items->Item[ Index ] ->Caption != Value ) {
        	ListView2->Items->Item[ Index ]->Caption = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SASFlags( int Index, AnsiString Value )
{
	if ( ( Index < 0 ) || ( Index > ListView3->Items->Count ) )	return;

        if ( ListView3->Items->Item[ Index ]->Caption != Value ) {
        	ListView3->Items->Item[ Index ]->Caption = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SLDFlags( int Index, AnsiString Value )
{
	if ( ( Index < 0 ) || ( Index > ListView4->Items->Count ) )	return;

        if ( ListView4->Items->Item[ Index ]->Caption != Value ) {
        	ListView4->Items->Item[ Index ]->Caption = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SDBFlags( int Index, AnsiString Value )
{
	if ( Index < 0 )	return;
}
//---------------------------------------------------------------------------
//	Returns the type of interface, console, gui, open gl, direct x
//	Depends on the platform id setting of the project
//---------------------------------------------------------------------------
int	__fastcall	TProjectOptionsForm :: GLanguage( void )
{
	return( RadioGroup2->ItemIndex );
}
//---------------------------------------------------------------------------
int	__fastcall	TProjectOptionsForm :: GInterface( void )
{
	return( RadioGroup3->ItemIndex );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SLanguage( int Value )
{
	if ( ( Value < -1 ) || ( Value > RadioGroup2->Items->Count ) ) return;

        if ( RadioGroup2->ItemIndex != Value ) {
        	RadioGroup2->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SInterface( int Value )
{
	if ( ( Value < -1 ) || ( Value > RadioGroup3->Items->Count ) ) return;

        if ( RadioGroup3->ItemIndex != Value ) {
        	RadioGroup3->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SIncludeVerInfo		( bool Value )
{
	if ( CheckBox7->Checked != Value ) {
        	CheckBox7->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SAutoIncVersion		( bool Value )
{
	if ( CheckBox13->Checked != Value ) {
        	CheckBox13->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SDebugAttr		( bool Value )
{
	if ( CheckBox8->Checked != Value ) {
        	CheckBox8->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SPreRelease	( bool Value )
{
	if ( CheckBox9->Checked != Value ) {
        	CheckBox9->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SDLLAttr		( bool Value )
{
	if ( CheckBox10->Checked != Value ) {
        	CheckBox10->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SSpecialAttr		( bool Value )
{
	if ( CheckBox11->Checked != Value ) {
        	CheckBox11->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SPrivateAttr		( bool Value )
{
	if ( CheckBox12->Checked != Value ) {
        	CheckBox12->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SMajor			( int Value )
{
	if ( UpDown1->Position != Value ) {
        	UpDown1->Position = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SMinor			( int Value )
{
	if ( UpDown2->Position != Value ) {
        	UpDown2->Position = Value;
                Modified = Value;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SRevision		( int Value )
{
	if ( UpDown3->Position != Value ) {
        	UpDown3->Position = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void		__fastcall	TProjectOptionsForm :: SBuild			( int Value )
{
	if ( UpDown4->Position != Value ) {
        	UpDown4->Position = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GIncludeVerInfo		( void )
{
	return( CheckBox7->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GAutoIncVersion		( void )
{
	return( CheckBox13->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GDebugAttr		( void )
{
	return( CheckBox8->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GPreRelease		( void )
{
	return( CheckBox9->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GDLLAttr			( void )
{
	return( CheckBox10->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GSpecialAttr		( void )
{
	return( CheckBox11->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GPrivateAttr		( void )
{
	return( CheckBox12->Checked );
}
//---------------------------------------------------------------------------
bool		__fastcall	TProjectOptionsForm :: GGPL			( void )
{
	return( true );
}
//---------------------------------------------------------------------------
int 		__fastcall	TProjectOptionsForm :: GMajor			( void )
{
	return( UpDown1->Position );
}
//---------------------------------------------------------------------------
int 		__fastcall	TProjectOptionsForm :: GMinor			( void )
{
	return( UpDown2->Position );
}
//---------------------------------------------------------------------------
int 		__fastcall	TProjectOptionsForm :: GRevision		( void )
{
	return( UpDown3->Position );
}
//---------------------------------------------------------------------------
int 		__fastcall	TProjectOptionsForm :: GBuild			( void )
{
	return( UpDown4->Position );
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectOptionsForm :: GFPUNone( void )
{
	return( CheckBox4->Checked );
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectOptionsForm :: GFPUFast( void )
{
	return( CheckBox5->Checked );
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectOptionsForm :: GFPUFix( void )
{
	return( CheckBox6->Checked );
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectOptionsForm :: GFPUSoft( void )
{
	return( CheckBox15->Checked );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SFPUNone( bool Value )
{
	if ( CheckBox4->Checked != Value ) {
        	CheckBox4->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SFPUFast( bool Value )
{
	if ( CheckBox5->Checked != Value ) {
        	CheckBox5->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SFPUFix( bool Value )
{
	if ( CheckBox6->Checked != Value ) {
        	CheckBox6->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SFPUSoft( bool Value )
{
	if ( CheckBox15->Checked != Value ) {
        	CheckBox15->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool	__fastcall	TProjectOptionsForm :: GModified( void )
{
        return( bModified );
}
//---------------------------------------------------------------------------
void	__fastcall	TProjectOptionsForm :: SModified( bool Value )
{
	if ( ProjectOptionsForm )
        	if ( bModified != Value ) bModified = Value;
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GIconFile( void )
{
        if ( asProjectIconFile.Length() > 0 ) return( asProjectIconFile );
        else return( NULL );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SIconFile( AnsiString Value )
{
        if ( ( asProjectIconFile != Value ) && ( Value.Length() > 0 ) ) {
                if ( FileExists( Value ) ) {
                        Image1->Picture->LoadFromFile( Value );
                        asProjectIconFile = Value;
                } else {
                	SystemMessage( "Unable to load file for icon image " + Value );
                        asProjectIconFile = "";
                }
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GAlignMode( void )
{
        return( DataAlignment->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SAlignMode( int Value )
{
        if ( DataAlignment->ItemIndex != Value ) {
                if ( ( Value < 0 ) || ( Value > DataAlignment->Items->Count ) ) return;
                DataAlignment->ItemIndex = Value;

                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GPlatformId( void )
{
        return( RadioGroup1->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SPlatformId( int Value )
{
        if ( RadioGroup1->ItemIndex != Value ) {
                RadioGroup1->ItemIndex = Value;
                ConfigureForPlatform( RadioGroup1->ItemIndex );
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GProjectTitle( void )
{
        return( Edit1->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SProjectTitle( AnsiString Value )
{
        if ( Edit1->Text != Value ) {
                Edit1->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString __fastcall   TProjectOptionsForm :: GHomeDir( void )
{
        return( Edit20->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SHomeDir( AnsiString Value )
{
        if ( Edit20->Text != Value ) {
                Edit20->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString __fastcall   TProjectOptionsForm :: GHelpFile( void )
{
        return( Edit2->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SHelpFile( AnsiString Value )
{
        if ( Edit2->Text != Value ) {
                Edit2->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GOptimisation( void )
{
        return( CodeOptimisation->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SOptimisation( int Value )
{
        if ( CodeOptimisation->ItemIndex != Value ) {
                CodeOptimisation->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GWarnings( void )
{
        return( WarningsLevel->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SWarnings( int Value )
{
        if ( WarningsLevel->ItemIndex != Value ) {
                WarningsLevel->ItemIndex = Value;
                Modified = true;
        }
}
//--------------------------------------------------------------------------
bool    __fastcall      TProjectOptionsForm :: GDebugMode( void )
{
        return( CheckBox1->Checked );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SDebugMode( bool Value )
{
        if ( CheckBox1->Checked != Value ) {
                CheckBox1->Checked = Value;
                DebugAttr = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool    __fastcall      TProjectOptionsForm :: GLineNumbering( void )
{
        return( CheckBox2->Checked );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SLineNumbering( bool Value )
{
        if ( CheckBox2->Checked != Value ) {
                CheckBox2->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool    __fastcall      TProjectOptionsForm :: GInlineExpanding( void )
{
        return( CheckBox3->Checked );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SInlineExpanding( bool Value )
{
        if ( CheckBox3->Checked != Value ) {
                CheckBox3->Checked = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GMakePath( void )
{
        return( Edit18->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SMakePath( AnsiString Value )
{
        if ( Edit18->Text != Value ) {
                Edit18->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GMakeFileName( void )
{
        return( Edit19->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SMakeFileName( AnsiString Value )
{
        if ( Edit19->Text != Value ) {
                Edit19->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GMakeTarget( void )
{
        return( ComboBox1->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SMakeTarget( int Value )
{
        if ( ComboBox1->ItemIndex != Value ) {
                ComboBox1->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GInstructionSet( void )
{
        return( InstructionSet->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SInstructionSet( int Value )
{
        if ( InstructionSet->ItemIndex != Value ) {
                InstructionSet->Items->Clear();
                switch ( Value ) {
                        case ( 0 ) : {  // PSX
                                InstructionSet->Items->Add( "R&3000 ( PSX )" );
                                break;
                        }
                        case ( 1 ) :
                        case ( 3 ) : {  // PS 2
                                InstructionSet->Items->Add( "R&3000 ( IOP )" );
                                InstructionSet->Items->Add( "R&5900 ( EE )" );
                                break;
                        }
                        case ( 2 ) : {  // N64
                                InstructionSet->Items->Add( "R&4000 ( N64 )" );
                                break;
                        }
                        case ( 5 ) :    // PC Linux
                        case ( 6 ) : {  // PC Windows
                                InstructionSet->Items->Add( "i&486" );
                                InstructionSet->Items->Add( "&Pentium" );
                                InstructionSet->Items->Add( "P&entium Pro" );
                                InstructionSet->Items->Add( "Pe&ntium II" );
                                InstructionSet->Items->Add( "Pen&tium III" );
                                break;
                        }
                        default : {
                                MessageBox( Application->Handle, "Unable to determine the platform", "PSX IDE", MB_OK );
                                return;
                        }
                }
                InstructionSet->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GDataAlignment( void )
{
        return( DataAlignment->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SDataAlignment( int Value )
{
        if ( DataAlignment->ItemIndex != Value ) {
                DataAlignment->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GRegVariables( void )
{
        return( RegisterVariables->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SRegVariables( int Value )
{
        if ( RegisterVariables->ItemIndex != Value ) {
                RegisterVariables->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
int     __fastcall      TProjectOptionsForm :: GCallingConvention( void )
{
        return( CallingConvention->ItemIndex );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SCallingConvention( int Value )
{
        if ( CallingConvention->ItemIndex != Value ) {
                CallingConvention->ItemIndex = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GCPPCompiler( void )
{
        return( Edit15->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SCPPCompiler( AnsiString Value )
{
        if ( Edit15->Text != Value ) {
                Edit15->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GASPath( void )
{
        return( Edit16->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SASPath( AnsiString Value )
{
        if ( Edit16->Text != Value ) {
                Edit16->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GLDPath( void )
{
        return( Edit17->Text );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SLDPath( AnsiString Value )
{
        if ( Edit17->Text != Value ) {
                Edit17->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString __fastcall   TProjectOptionsForm :: GLibrary( int Index )
{
        if ( ListView5->Items->Count <= 0 ) return( NULL );
        if ( ( Index < 0 ) || ( Index > ListView5->Items->Count ) ) return( NULL );
        return( ListView5->Items->Item[ Index ]->Caption );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SLibrary( int Index, AnsiString Value )
{
        if ( ListView5->Items->Count <= 0 ) return;
        if ( ( Index < 0 ) || ( Index > ListView5->Items->Count ) ) return;
        if ( ListView5->Items->Item[ Index ]->Caption != Value ) {
                ListView5->Items->Item[ Index ]->Caption = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GIncludePath    ( void )
{
        return( Edit7->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SIncludePath    ( AnsiString Value )
{
        if ( Edit7->Text != Value ) {
                Edit7->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GLibraryPath    ( void )
{
        return( Edit8->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SLibraryPath    ( AnsiString Value )
{
        if ( Edit8->Text != Value ) {
                Edit8->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GDebugSourcePath( void )
{
        return( Edit9->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SDebugSourcePath( AnsiString Value )
{
        if ( Edit9->Text != Value ) {
                Edit9->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
bool            __fastcall      TProjectOptionsForm :: GTempPipe       ( void )
{
        return( CheckBox14->Checked );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: STempPipe       ( bool Value )
{
        if ( CheckBox14->Checked != Value ) {
                CheckBox14->Checked = true;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GTempOutput     ( void )
{
        return( Edit10->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: STempOutput     ( AnsiString Value )
{
        if ( Edit10->Text != Value ) {
                Edit10->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GFinalOutput    ( void )
{
        return( Edit11->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SFinalOutput    ( AnsiString Value )
{
        if ( Edit11->Text != Value ) {
                Edit11->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GLibraryOutput  ( void )
{
        return( Edit12->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SLibraryOutput  ( AnsiString Value )
{
        if ( Edit12->Text != Value ) {
                Edit12->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GConditionals   ( void )
{
        return( Edit13->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SConditionals   ( AnsiString Value )
{
        if ( Edit13->Text != Value ) {
                Edit13->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GAliases        ( void )
{
        return( Edit14->Text );
}
//---------------------------------------------------------------------------
void            __fastcall      TProjectOptionsForm :: SAliases        ( AnsiString Value )
{
        if ( Edit14->Text != Value ) {
                Edit14->Text = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
AnsiString      __fastcall      TProjectOptionsForm :: GDBPath( void )
{
        return( DebugOptionsForm->DebugPath );
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: SDBPath( AnsiString Value )
{
        if ( DebugOptionsForm->DebugPath != Value ) {
                DebugOptionsForm->DebugPath = Value;
                Modified = true;
        }
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForPlatform     ( int PlatFormType )
{
	if ( PlatFormType == -1 ) return;
        switch ( PlatFormType ) {
                case ( PLATFORM_PSX ) : {
                        ConfigureForPlayStation( this );
                        break;
                }
                case ( PLATFORM_PS2 ) : {
                        ConfigureForPlayStation2( this );
                        break;
                }
                case ( PLATFORM_N64 ) : {
                        ConfigureForNintendo64( this );
                        break;
                }
                case ( PLATFORM_PS2L ) : {
                        ConfigureForPlayStation2Linux( this );
                        break;
                }
                case ( PLATFORM_PCL ) : {
                        ConfigureForPCLinux( this );
                        break;
                }
                case ( PLATFORM_PCW ) : {
                        ConfigureForPCWindows( this );
                        break;
                }
                default : {
                        ConfigureForDefault( this );
                        break;
                }
        }
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForDefault             ( TObject *Sender )
{
        if ( PLATFORM_DEFAULT == PLATFORM_PSX ) {     		//	PlayStation 1
                ConfigureForPlayStation( Sender );
        } else if ( PLATFORM_DEFAULT == PLATFORM_PS2 ) {     	//	PlayStation 2
                ConfigureForPlayStation2( Sender );
        } else if ( PLATFORM_DEFAULT == PLATFORM_N64 ) {     	//	Nintendo 64
                ConfigureForNintendo64( Sender );
        } else if ( PLATFORM_DEFAULT == PLATFORM_PS2L ) {    	//	PlayStation 2 Linux
                ConfigureForPlayStation2Linux( Sender );
        } else if ( PLATFORM_DEFAULT == PLATFORM_PCL ) {     	//	PC Linux
                ConfigureForPCLinux( Sender );
        } else if ( PLATFORM_DEFAULT == PLATFORM_PCW ) {     	//	PC Windows
                ConfigureForPCWindows( Sender );
        }
}
//---------------------------------------------------------------------------
//	Sets the project options suitable for target platform of PlayStation
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForPlayStation         ( TObject *Sender )
{
	if ( !Sender ) return;

        Label2->Enabled = false;
        Edit2->Enabled = false;
        HelpFileBrowse->Enabled = false;

        Label3->Enabled = false;
        IconBrowse->Enabled = false;
        Image1->Enabled = false;

        RadioGroup3->Enabled = false;

        //	We need to change the available libraries as this will only
        //	have 1 (one) installed, ie the standard run time library
        //	supplied with the gcc compiler for this target platform
        ListView4->Items->Clear();

        TListItem	*pItem = ListView4->Items->Add();

        pItem->Caption = "SRTL (PS)";
        pItem->Checked = true;
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForPlayStation2        ( TObject *Sender )
{
	if ( !Sender ) return;

        Label2->Enabled = false;
        Edit2->Enabled = false;
        HelpFileBrowse->Enabled = false;

        Label3->Enabled = true;
        Image1->Enabled = true;
        IconBrowse->Enabled = true;

        RadioGroup3->Enabled = false;

        ListView4->Items->Clear();
        TListItem	*pItem = ListView4->Items->Add();

        pItem->Caption = "SRTL (PS2)";
        pItem->Checked = true;
}
//---------------------------------------------------------------------------
//	At present the only thing that I know about the N64 is that is uses
//	the MIPS R4000 CPU, no other information is known about any of the
//	other internal chips that Nintendo uses
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForNintendo64          ( TObject *Sender )
{
	if ( !Sender ) return;

        Label2->Enabled = false;
        Edit2->Enabled = false;
        HelpFileBrowse->Enabled = false;

        Label3->Enabled = false;
        Image1->Enabled = false;
        IconBrowse->Enabled = false;

        RadioGroup3->Enabled = false;

        ListView4->Items->Clear();
        TListItem	*pItem = ListView4->Items->Add();
        pItem->Caption =  "SRTL (N64)";
        pItem->Checked = true;
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForPlayStation2Linux   ( TObject *Sender )
{
	if ( !Sender ) return;

        Label2->Enabled = true;
        Edit2->Enabled = true;
        HelpFileBrowse->Enabled = true;

        Label3->Enabled = true;
        Image1->Enabled = true;
        IconBrowse->Enabled = true;

        RadioGroup3->Enabled = true;

        RadioGroup3->Items->Clear();
        RadioGroup3->Items->Add( "Console" );
        RadioGroup3->Items->Add( "GUI" );
        RadioGroup3->Items->Add( "Open GL" );

        Interface = INTERFACE_CLI;	//	CLI INTERFACE

        ListView4->Items->Clear();
        TListItem	*pItem = ListView4->Items->Add();

        pItem->Caption = "SRTL-C (PS2L)";
        pItem->Checked = true;

        TListItem *pItem1 = ListView4->Items->Add();

        pItem1->Caption =  "SRTL-G (PS2L)";
        pItem1->Checked = false;

        TListItem *pItem2 = ListView4->Items->Add();

        pItem2->Caption = "SRTL-OGL (PS2L)";
        pItem2->Checked = false;
}
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForPCLinux             ( TObject *Sender )
{
	if ( !Sender ) return;

        Label2->Enabled = true;
        Edit2->Enabled = true;
        HelpFileBrowse->Enabled = true;

        Label3->Enabled = true;
        Image1->Enabled = true;
        IconBrowse->Enabled = true;

        RadioGroup3->Enabled = true;

        RadioGroup3->Items->Clear();
        RadioGroup3->Items->Add( "Console" );
        RadioGroup3->Items->Add( "GUI" );
        RadioGroup3->Items->Add( "Open GL" );

        Interface = INTERFACE_GUI;	//	GUI INTERFACE

        ListView4->Items->Clear();

        TListItem *pItem = ListView4->Items->Add();
        pItem->Caption = "SRTL-C (PCL)";
        pItem->Checked = true;

        TListItem *pItem1 = ListView4->Items->Add();
        pItem1->Caption = "SRTL-G (PCL)";
        pItem1->Checked = false;

        TListItem *pItem2 = ListView4->Items->Add();
        pItem2->Caption = "SRTL-OGL (PCL)";
        pItem2->Checked = false;
}
//---------------------------------------------------------------------------
//
//	$Method		::	ConfigureForPCWindows
//	$Parameters	::	TObject *Sender, originator of the call, not used
//	$Purpose	::	Configures the project options suitable for use with
//				a PC running MS Windows 95 or higher ( ie Win32 )
//				It does not alter any paths, etc just flags
//
//---------------------------------------------------------------------------
void    __fastcall      TProjectOptionsForm :: ConfigureForPCWindows           ( TObject *Sender )
{
	if ( !Sender ) return;

        SystemMessage( "Configuring for PC Windows platform" );

        //	We will start at the Application Tab

        //	Re-enable the help file edit box and browse button
        Label2->Enabled = true;
        Edit2->Enabled = true;		//	Help files are valid for windows, usually *.hlp
        HelpFileBrowse->Enabled = true;	//	Browse button for above

        //	Re-enable the icon image view and browse button
        Label3->Enabled = true;
        Image1->Enabled = true;
        IconBrowse->Enabled = true;

        //	Re-enable the Interface selection group
        RadioGroup3->Enabled = true;

        RadioGroup3->Items->Clear();
        RadioGroup3->Items->Add( "Console" );	//	Standard int main( ... )
        RadioGroup3->Items->Add( "GUI" );	//	int WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR, int )
        RadioGroup3->Items->Add( "Open GL" );	//	As Above, but includes the gl.h, glu.h, glaux.h headers and libraries
        RadioGroup3->Items->Add( "Direct X" );	//	As Above, but for Direct X 8.1
        RadioGroup3->Items->Add( "DLL" );	//	int WINAPI DLLEntryPoint( ... )

        Interface = INTERFACE_GUI;	//	GUI INTERFACE

        //	Nothing else needs to be re-enabled, the only thing that needs
        //	modifiying now is the flags presented to the compiler on a
        //	compilation request, ie C++ flags, and the linker section
        //	maybe also Assembler

        ListView4->Items->Clear();
        TListItem *pItem  = ListView4->Items->Add();
        pItem->Caption =  "SRTL-C (PCW)";
        pItem->Checked = true;

        TListItem *pItem1 = ListView4->Items->Add();
        pItem1->Caption =  "SRTL-G (PCW)";
        pItem1->Checked = false;

        TListItem *pItem2 = ListView4->Items->Add();
        pItem2->Caption =  "SRTL-DX (PCW)";
        pItem2->Checked = false;

        TListItem *pItem3 = ListView4->Items->Add();
        pItem3->Caption = "SRTL-OGL (PCW)";
        pItem3->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::CancelButtonClick(TObject *Sender)
{
        ModalResult = mrCancel;
        Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::HelpFileBrowseClick(TObject *Sender)
{
        OpenDialog1->Filter = "PDF Files ( *.pdf )|*.pdf|HTML Files (*.html;*.htm;*.chm)|*.html;*.htm;*.chm|Help Files (*.hlp)|*.hlp|All files (*.*)|*.*";
        OpenDialog1->InitialDir = ProjectHomeDir;
        if ( OpenDialog1->Execute() ) {
        	HelpFile = OpenDialog1->FileName;
                Modified = true;
        } else {
        	HelpFile = "";
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::IconBrowseClick(TObject *Sender)
{
        OpenDialog1->Filter = "Bitmap Files (*.bmp)|*.bmp|Icon Files (*.ico)|*.ico";
        OpenDialog1->InitialDir = ProjectHomeDir;

        if ( OpenDialog1->Execute() ) {
                Image1->Picture->LoadFromFile( OpenDialog1->FileName );
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::OKButtonClick(TObject *Sender)
{

	if ( Modified ) {
        	SaveToRegistry( Sender );
                Modified = false;
        }

        ModalResult = mrOk;

        Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton1Click(TObject *Sender)
{       // Selects a directory for header INCLUDE PATH

	if ( !Sender ) return;

	ListEditDialog->Caption = "Modifying Project's Include Directories";

	ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = IncludePath;
        ListEditDialog->ShowModal();
        if ( ListEditDialog->ModalResult != mrOk ) return;
        IncludePath = ListEditDialog->Items;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton2Click(TObject *Sender)
{
	if ( !Sender ) return;
	ListEditDialog->Caption = "Modifying Project's Library Directories";
	ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = LibraryPath;
        ListEditDialog->ShowModal();
        if ( ListEditDialog->ModalResult != mrOk ) return;
        LibraryPath = ListEditDialog->Items;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton3Click(TObject *Sender)
{
        // Selects a directory for DEBUG SOURCE PATH
        SelectDirectoryDialog->Caption = "Select the Debug Source Path";
        SelectDirectoryDialog->ShowModal();
        if ( SelectDirectoryDialog->ModalResult == mrCancel ) return;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton8Click(TObject *Sender)
{       // Select a program file for use as the c++ compiler
        OpenDialog1->Title = "Please select C++ Compiler";
        OpenDialog1->Filter = "Program Files ( *.com; *.exe )|*.com;*.exe|All Files ( *.* )|*.*";
        if ( OpenDialog1->Execute() ) {
                Edit15->Text = OpenDialog1->FileName;
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton7Click(TObject *Sender)
{
        OpenDialog1->Title = "Please select Assembler";
        OpenDialog1->Filter = "Program Files ( *.com; *.exe )|*.com;*.exe|All Files ( *.* )|*.*";
        if ( OpenDialog1->Execute() ) {
                Edit16->Text = OpenDialog1->FileName;
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton6Click(TObject *Sender)
{
        OpenDialog1->Title = "Please select a Linker";
        OpenDialog1->Filter = "Program Files ( *.com; *.exe )|*.com;*.exe|All Files ( *.* )|*.*";
        if ( OpenDialog1->Execute() ) {
                Edit17->Text = OpenDialog1->FileName;
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::CheckBox14Click(TObject *Sender)
{
        CheckBox14->Checked |= CheckBox14->Checked;	//	Toggle option

        if ( CheckBox14->Checked ) {			//	Change data as needed to
                Edit10->Text = "Using Pipes";
                Edit10->Enabled = false;
                // We need to scan through the cflags list to enable the -pipe option
                for ( int i=0; i<=ListView2->Items->Count; i++ ) {
                	if ( ListView2->Items->Item[ i ]->Caption == "-pipe" )
                        	ListView2->Items->Item[ i ]->Checked = true;
                }
        } else {
                Edit10->Text = "C:\\Windows\\Temp";
                Edit10->Enabled = true;
                // We need to scan through the cflags list to disable the -pipe option
                for ( int i=0; i<=ListView2->Items->Count; i++ ) {
                	if ( ListView2->Items->Item[ i ]->Caption == "-pipe" )
                        	ListView2->Items->Item[ i ]->Checked = false;
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton10Click(TObject *Sender)
{
        OpenDialog1->Title = "Select the make application you require";
        OpenDialog1->Filter = "Program Files ( *.exe;*.com;*.bat )|*.exe;*.com;*.bat|All Files ( *.* )|*.*";
        if ( OpenDialog1->Execute() ) {
                MakeApplication = OpenDialog1->FileName;
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton9Click(TObject *Sender)
{       // Selects the tempory output path to be used during compilation
        SelectDirectoryDialog->Caption = "Select temp output path";
        SelectDirectoryDialog->ShowModal();
        if ( SelectDirectoryDialog->ModalResult == mrCancel ) return;
        TempOutputPath = SelectDirectoryDialog->DirectoryPath;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::AdvancedCompilerTabShow(
      TObject *Sender)
{
	// Setup the instruction suitable for the current platform
	InstructionSet->Items->Clear();
	switch ( PlatformId ) {
        	case ( 0 ) : {	// PSX
                	InstructionSet->Items->Add( "&R3000 ( PlayStation" );
                        break;
                }
                case ( 1 ) : {	// PS2
                	InstructionSet->Items->Add( "&Emotion Engine ( PS2 R5900 )" );
                        InstructionSet->Items->Add( "&IO Processor   ( PS2 R3400 )" );
                        break;
                }
                case ( 2 ) : {	// N64
                        InstructionSet->Items->Add( "&R4000	( Nintendo 64 )" );
                        break;
                }
                case ( 3 ) : {	// PS2 Linux
                        InstructionSet->Items->Add( "&Emotion Engine ( PS2 R5900 LINUX )" );
                        InstructionSet->Items->Add( "&IO Processor   ( PS2 R3400 LINUX )" );
                        break;
                }
                case ( 4 ) : {	// PC Linux
                        InstructionSet->Items->Add( "&486" );
                        InstructionSet->Items->Add( "&Pentium" );
                        InstructionSet->Items->Add( "P&entium Pro" );
                        InstructionSet->Items->Add( "Pentium &II" );
                        InstructionSet->Items->Add( "Pe&ntium III" );
                        break;
                }
                case ( 5 ) : {	// PC Windows
                        InstructionSet->Items->Add( "&486" );
                        InstructionSet->Items->Add( "&Pentium" );
                        InstructionSet->Items->Add( "P&entium Pro" );
                        InstructionSet->Items->Add( "Pentium &II" );
                        InstructionSet->Items->Add( "Pe&ntium III" );
                        break;
                }
                default : {
                	InstructionSet->Items->Add( "Unable to determine the platform id" );
                        break;
                }
        }
        InstructionSet->ItemIndex = 0;

        // setup the fpu section

        if ( FPUNone ) {			//	No FPU required
        	CheckBox5->Enabled = false;	//	Fast
                CheckBox6->Enabled = false;	//	Fix
                CheckBox7->Enabled = false;	//	Soft
        } else {
        	CheckBox5->Enabled = true;
                CheckBox6->Enabled = true;
                CheckBox7->Enabled = true;
        }
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::BitBtn1Click(TObject *Sender)
{
        SelectDirectoryDialog->Caption = "Select the HOME directory for this project";
        SelectDirectoryDialog->ShowModal();
        if ( SelectDirectoryDialog->ModalResult == mrCancel ) return;
        if ( !DirExists( SelectDirectoryDialog->DirectoryPath ) ) {
        	if ( MessageBox( Application->Handle, AnsiString( "Directory " + SelectDirectoryDialog->DirectoryPath + " does not exist. Create ?" ).c_str(),
                	Application->Title.c_str(), MB_YESNO ) == ID_YES ) {
                        	CreateDir( SelectDirectoryDialog->DirectoryPath );
                }
        }
        ProjectHomeDir = SelectDirectoryDialog->DirectoryPath;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::DirectoriesTabShow(TObject *Sender)
{
	Edit8->Text = EnviromentOptionsForm->LibraryPath;
        Edit12->Text = EnviromentOptionsForm->LibraryOutput;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::CodeOptimisationButtonClick(
      TObject *Sender)
{
	CodeOptimisationSelection->ShowModal();
        if ( CodeOptimisationSelection->ModalResult != mrOk ) return;	//	should really switch back to previous options
	// Adjust the makefile accordingly, ie re-generate it
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::WarningsButtonClick(TObject *Sender)
{
	// Show the warnings modification form
        WarningSelection->ShowModal();
        if ( WarningSelection->ModalResult != mrOk ) return;	//	should really revert to previous options
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton4Click(TObject *Sender)
{
	if ( !Sender ) return;
	ListEditDialog->Caption = "Modifying Project's Conditionals";
	ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = Edit13->Text;
        ListEditDialog->ShowModal();
        if ( ListEditDialog->ModalResult != mrOk ) return;
        Edit13->Text = ListEditDialog->Items;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::SpeedButton5Click(TObject *Sender)
{
	if ( !Sender ) return;
        ListEditDialog->Caption = "Modifying Project's Aliases";
        ListEditDialog->ListView1->Items->Clear();
        ListEditDialog->Items = Edit14->Text;
        ListEditDialog->ShowModal();
        if ( ListEditDialog->ModalResult != mrOk ) return;
        Edit14->Text = ListEditDialog->Items;
}
//---------------------------------------------------------------------------
//
//	$Method		::	RadioGroup1Click
//	$Parameters	::	TObject *Sender, originator of call, not used
//	$Purpose	::	Provides a way for the options to be modified
//				for the target platform, whilst application is
//				running.
//
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::RadioGroup1Click(TObject *Sender)
{
	if ( !Sender ) return;
        ConfigureForPlatform( PlatformId );
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::BitBtn3Click(TObject *Sender)
{	TComponent *pComponent = dynamic_cast< TComponent* >(Sender);

	if ( !pComponent ) {
        	SystemMessage( "[ TProjectOptionsForm :: BitBtn3Click ] Bad Sender" );
                return;
        }

        if ( !ListView5->Selected ) {
        	SystemMessage( "No library selected" );
                return;
        }

        int	mrCode = MessageBox( Application->Handle, AnsiString( "Remove library " + ListView5->Selected->Caption + " from list?" ).c_str(),
        	Application->Title.c_str(), MB_YESNO );

        if ( mrCode == ID_YES ) {	//	Yeah, delete it
        	ListView5->Items->Delete( ListView5->Selected->Index );
        }
}
//---------------------------------------------------------------------------
//	$Method		::	Add Library ( Extra Linker Libraries )
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::BitBtn2Click(TObject *Sender)
{       TComponent *pComponent = dynamic_cast< TComponent* >(Sender);

	if ( !pComponent ) {
        	SystemMessage( "[ TProjectOptionsForm :: BitBtn2Click ] Bad Sender" );
                return;
        }

        OpenDialog1->Title = "Please select additional libraries";
        OpenDialog1->Filter = "Static Libraries ( *.a )|*.a|Dynamic Libraries ( *.dll )|*.dll|All Files ( *.* )|*.*";

        if ( OpenDialog1->Execute() ) {
        	TListItem	*pItem = ListView5->Items->Add();
                //	NOTE WE NEED TO ADD THE FACILITY TO ADD MULTIPLE FILES
                //	AS AT PRESENT IT ONLY ADDS ONE FILE SELECTED
                if ( pItem ) {
                	AnsiString	as;

                	pItem->Caption = ExtractFileName( OpenDialog1->FileName );
                        pItem->Checked = true;

                        if ( !dll_version )	dll_version = new GetApplicationFileVersionInfo( pItem->Caption );

                        as = dll_version->GetAppFileVerNumStr();
                        pItem->SubItems->Add( as );
                        pItem->SubItems->Add( dll_version->GetAppFileVersionInfoString( AnsiString( "FileDescription" ) ) );
                        SystemMessage( "Extra Library added " + pItem->Caption + " from file " + OpenDialog1->FileName );
                }
        }
}
//---------------------------------------------------------------------------
//	$Method		::	DebugButtonClick
//	$Purpose	::	Allows quick setup for 'DEBUG' mode
//				ie default debugging options are enabled
//				and the debugging options form modified
//				accordingly
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::DebugButtonClick(TObject *Sender)
{	TComponent	*Originator = dynamic_cast< TComponent*>(Sender);

	if ( !Originator )	return;		//	Bad call ?

        ClearSysMessages();

        SystemMessage( "Configuring Project " + MainForm->Name + " for 'DEBUG MODE'" );

        OptimisationLevel = CODE_OPTIMISE_NONE;			//	No Code Optimisation
        WarningLevel = WARNINGS_ALL;				//	Display all warnings

        //	NOTE UPDATE THE WARNINGS SELECTION FORM ?

        DebugMode = true;					//	Yes, we are in DEBUG MODE
        LineNumbering = true;                                   //	Yes, to line numbering
        InlineExpansion = false;                                //	No, inline expansion

        TargetRegisterVariables = REGISTER_VARIABLES_NONE;	//	No cpu register <-> variable access method

        if ( DebugOptionsForm->DebuggerType == DEBUGGER_STANDALONE ) {	//	External debugger specified ?
        	if ( DebugOptionsForm->DebugPath.Length() > 0 )
                	DebuggerPath = DebugOptionsForm->DebugPath;
                else	DebuggerPath = "GDB.EXE";				//	Set the debugger path
        } else DebuggerPath = "Internal";

        if ( Conditionals.Length() >0 )
        	Conditionals += " DEBUG ";
        else Conditionals = "DEBUG";

        if ( DebugSourcePath.Length() <= 0 )
        	DebugSourcePath = ProjectHomeDir;

        SystemMessage( "Configuration changed OK. Returning to IDE." );
}
//---------------------------------------------------------------------------
//	$Method		::	BitBtn9Click ( Release Mode Button )
//	$Purpose	::	Quick release mode settings
//---------------------------------------------------------------------------
//	$Method		::	( Speed Mode Button Click )
//	$Purpose	::	Speed release mode settings
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::BitBtn4Click(TObject *Sender)
{
	//	Speed Mode
        SystemMessage( "Configuring for Speed Mode" );
}
//---------------------------------------------------------------------------
//	$Method		::	( Size Mode Button Click )
//	$Purpose	::	Size release mode settings
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::BitBtn5Click(TObject *Sender)
{
	//	Size Mode
        SystemMessage( "Configuring for Size Mode" );
}
//---------------------------------------------------------------------------
//	Add a new key to the project
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::Add1Click(TObject *Sender)
{
	ValueDialog->Caption = "Please enter new Key Name";
        ValueDialog->Value = "KeyName";
        ValueDialog->ShowModal();

        if ( ValueDialog->ModalResult != mrOk )	{
        	SystemMessage( "Cancelled." );
                return;
        }

        TListItem *pItem = ListView1->Items->Add();
        pItem->Caption = ValueDialog->Value;

        ValueDialog->Value = "Key Value";
        ValueDialog->Caption = "Enter value for new key " + pItem->Caption;
        ValueDialog->ShowModal();

        if ( ValueDialog->ModalResult != mrOk )	{
        	SystemMessage( "Cancelled." );
                ListView1->Items->Delete( pItem->Index );
                return;
        }
        pItem->SubItems->Add( ValueDialog->Value );
        Update();
}
//---------------------------------------------------------------------------
//	Edit an existing key
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::Edit21Click(TObject *Sender)
{
	if ( !ListView1->Selected )	{
        	SystemMessage( "Nothing to do for edit application version information key." );
                return;
        }

	ValueDialog->Value = ListView1->Selected->SubItems->Strings[ 0 ];
        ValueDialog->Caption = "Enter new value for " + ListView1->Selected->Caption;
        ValueDialog->ShowModal();

        if ( ValueDialog->ModalResult != mrOk )	{
        	SystemMessage( "Not changing key " + ListView1->Selected->Caption );
                return;
        }

        ListView1->Selected->SubItems->Strings[ 0 ] = ValueDialog->Value;
        Update();
}
//---------------------------------------------------------------------------
//	Delete a key
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::Delete1Click(TObject *Sender)
{
	if ( !ListView1->Selected )	{
        	SystemMessage( "Nothing to do for Application Version Info Key delete." );
                return;
        }

        int mrCode = MessageBox( Application->Handle,
        	AnsiString( "Are you sure you wish to delete the key " + ListView1->Selected->Caption + " ?"  ).c_str(),
                Application->Title.c_str(), MB_YESNO );

        if ( mrCode == ID_NO )	{
        	SystemMessage( "NOT deleting." );
                return;
        }

        SystemMessage( "Deleting key " + ListView1->Selected->Caption );

        ListView1->Items->Delete( ListView1->Selected->Index );

        SystemMessage( "Done." );
        Update();

}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::CheckBox7Click(TObject *Sender)
{
	CheckBox7->Checked |= CheckBox7->Checked;	//	Flip value
        Update();
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::FormCreate(TObject *Sender)
{
	if ( MainForm->hEmulatorHandle != INVALID_HANDLE_VALUE ) {
        	TListItem *pItem = ListView8->Items->Add();
                pItem->Caption = "Emulator";
        }

        if ( MainForm->hDebuggerHandle != INVALID_HANDLE_VALUE ) {
        	TListItem *pItem = ListView8->Items->Add();
                pItem->Caption = "Debugger";
        }

        if ( MainForm->hCompilerHandle != INVALID_HANDLE_VALUE ) {
        	TListItem *pItem = ListView8->Items->Add();
                pItem->Caption = "Compiler Set";
        }

        PageControl1->ActivePage = ApplicationTab;

        //	LoadFromRegistry( this );

        Modified = false;
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::CodeOptimisationClick(TObject *Sender)
{
	CodeOptimisationButton->Enabled = ( OptimisationLevel == CODE_OPTIMISE_SELECTED );
}
//---------------------------------------------------------------------------
void __fastcall TProjectOptionsForm::WarningsLevelClick(TObject *Sender)
{
	WarningsLevel->Enabled = ( WarningLevel == WARNINGS_SELECTED );
}
//---------------------------------------------------------------------------
