//---------------------------------------------------------------------------
//
//	$FileName	::	ToolsAPI.cpp
//	$Author		::	Chris Brooklyn
//	$Version	::	0.0.0.1
//	$Purpose	::	Allows custom programs to be installed in the 'tools'
//				menu of the ide
//
//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "ToolsAPI.h"
#include "EditToolDialog.h"
#include "shellapi.h"
#include "main.h"
#include "Utilities.h"
#include <Registry.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//	TOOLSAPI	::	GLOBAL VARIABLES
//---------------------------------------------------------------------------
__declspec( dllexport ) TIDETool        *IdeTools = (TIDETool*)malloc( sizeof( TIDETool ) * MAX_IDE_TOOLS );
__declspec( dllexport )	DWORD		dwNumTools = 0;
//---------------------------------------------------------------------------
//	TOOLSAPI	::	GLOBAL METHODS
//---------------------------------------------------------------------------
__declspec( dllexport ) AnsiString	__fastcall	DecodeMacro( AnsiString asLine )
//---------------------------------------------------------------------------
//	Decodes a line containing one or more macro strings, ie words that begin
// with the character '$' and are terminated by the first whitespace character
//---------------------------------------------------------------------------
{	int		s, e;
	static	int	p = 0;
        AnsiString 	asMacro;

	if ( asLine.Length() <= 0 )
        	return( NULL );

        if ( asLine.Pos( '$' ) == 0 )
        	return( NULL );

        e = asLine.SubString( (s+1), asLine.Length() ).Pos( ' ' );

        if ( e == 0 )
        	return( NULL );

        asMacro = asLine.SubString( (s+1), (e-1) );

        if ( asMacro.Length() <= 0 )
        	return( NULL );

        p = asLine.SubString( e, asLine.Length() ).Pos( '$' );

        return( asMacro );
}
//---------------------------------------------------------------------------
__fastcall      TIDETool :: TIDETool( TObject *Owner ) : TObject()
{
	asCaption = NULL;
        asToolPath = NULL;
        asWorkDir = NULL;
        asParameters = NULL;
        Name = NULL;

        bModified = false;

        pMenuItem = NULL;
        IdNum = 0xffffffff;
        Registry = NULL;

        if ( dwNumTools != 0 )
        	dwNumTools = 0;

        if ( Owner != NULL )
        	pOwner = Owner;
        else
        	pOwner = (TObject*)this;
}
//---------------------------------------------------------------------------
__fastcall	TIDETool :: ~TIDETool( void )
{
	if ( Modified )
        	SaveToRegistry( this );

        dwNumTools--;
        delete this;
}
//---------------------------------------------------------------------------
int	__fastcall      TIDETool :: LoadFromRegistry( TObject *Sender )
{
	if ( Sender == NULL )
        	return( false );
        else
        	return( true );
}
//---------------------------------------------------------------------------
int	__fastcall      TIDETool :: SaveToRegistry( TObject *Sender )
{
	if ( Sender == NULL )
        	return( false );
        else
        	return( true );
}
//---------------------------------------------------------------------------
//	This is the actual method that launches the tool selected by the user
//---------------------------------------------------------------------------
void    __fastcall      TIDETool :: DoExecute( TObject *Sender )
{       HINSTANCE       hInstance;

        if ( asToolPath.Length() <= 0 ) return;

        hInstance = ShellExecute( Application->Handle, "open", asToolPath.c_str(), asParameters.c_str(), asWorkDir.c_str(), SW_SHOWNORMAL );

        if ( !hInstance ) {
           SystemMessage( "Unable to execute tool '" + Caption + "' from path " + ToolPath );
        } else {
        	SystemMessage( "Tool : " + Caption + " : launched OK." );
        }
}
//---------------------------------------------------------------------------
//      Routine that installs the tool pointed to by pTool, adds a menu item
// for it, and write some information into the registry
//---------------------------------------------------------------------------
void       __fastcall      TIDETool :: Install( TObject *Sender )
{       TIDETool	*p = this;

        if ( !Sender ) return;

        pMenuItem = new TMenuItem( MainForm->MainMenu1 );

        if ( !pMenuItem ) return;

        pMenuItem->Caption = Caption;
        pMenuItem->OnClick = Execute;

        IdNum = pMenuItem->MenuIndex;

        Modified = false;

        MainForm->Tools1->Add( pMenuItem );

        dwNumTools++;

        SaveToRegistry( Sender );
}
//---------------------------------------------------------------------------
void    __fastcall      TIDETool :: Remove( TObject *Sender )
{
        MainForm->Tools1->Remove( pMenuItem );
        delete pMenuItem;
        dwNumTools--;
        delete this;
}
//---------------------------------------------------------------------------
void    __fastcall      TIDETool :: Execute( TObject *Sender )
{
        if ( !Sender ) return;
        DoExecute( Sender );
}
//---------------------------------------------------------------------------
