//---------------------------------------------------------------------------
//
//	$FileName	::	Utilities.cpp
//	$Author		::	Chris Brooklyn
//	$Purpose	::	Various utility methods used throughout the ide
//
//----------------------------------------------------------------------------
#include <vcl.h>
#include <string.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "Utilities.h"
#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
__declspec( dllexport ) SysLogger	*SystemLog = NULL;
//---------------------------------------------------------------------------
__fastcall	SysLogger :: SysLogger( void ) : TObject()
{
	fHandle = FileOpen( "LOGFILE.TXT", fmOpenWrite );

        if ( !fHandle ) {
        	SystemMessage( "Unable to open logger file." );
                return;
        }
}
//---------------------------------------------------------------------------
__fastcall	SysLogger :: ~SysLogger( void )
{
	if ( fHandle )
        	FileClose( fHandle );

	delete this;
}
//---------------------------------------------------------------------------
void	__fastcall	SysLogger :: Message( AnsiString asMessage )
{
	if ( asMessage.Length() <= 0 )
        	return;

        if ( !fHandle )
        	return;

        FileWrite( fHandle, asMessage.c_str(), asMessage.Length() );
        FlushFileBuffers( (void*)fHandle );

}
//---------------------------------------------------------------------------
//	Inserts a delay for approximately TimeDelay seconds
//----------------------------------------------------------------------------
void	__fastcall	Delay( float TimeDelay )
{
	for ( int i=0; i<=(int)(TimeDelay*1000); i++ ) {
        ;
        }
}
//---------------------------------------------------------------------------
//	Attempts to read a line from a file until \n is read or non null
//----------------------------------------------------------------------------
char*	__fastcall	ReadLine( FILE *fp )
{	char	*p = (char*)malloc( 8192 );
	int	i=0;

	if ( !fp ) return( NULL );
        do {
        	p[ i ] = fgetc( fp );
                i++;
        } while ( ( p[ i ] !='\n' ) || ( p[ i ]!='\0' ) );

        p[ i + 1 ] = '\0';
        return( p );
}
//---------------------------------------------------------------------------
__declspec( dllexport ) AnsiString	__fastcall	asReadLine( FILE *fp )
{
	if ( fp == NULL )
        	return( NULL );
        else
        	return( AnsiString( ReadLine( fp ) ) );
}
//---------------------------------------------------------------------------
//	Returns an enviroment variable value ie GetEnvVar( "PATH" );
//----------------------------------------------------------------------------
AnsiString      __fastcall      GetEnvVar( AnsiString EnvVar )
{       AnsiString      as;
        if ( EnvVar.Length() <= 0 ) return( NULL );
        as = getenv( EnvVar.c_str() );
        if ( as.Length() <= 0 ) return( NULL );
        else return( as );
}
//----------------------------------------------------------------------------
//	Opposite of above, allows for setting of enviroment variables. Note that
// any variables set are not permenant, to do this you must add them to the
// autoexec.bat file
//----------------------------------------------------------------------------
void    __fastcall      SetEnvVar( AnsiString EnvVar, AnsiString Value )
{       AnsiString      as;
        if ( (EnvVar.Length()<=0) || (Value.Length()<=0) ) return;
        as = EnvVar + "=" + Value;
        putenv( as.c_str() );
}
//----------------------------------------------------------------------------
//	Returns the system time as an ansistring
//----------------------------------------------------------------------------
AnsiString      __fastcall      SystemTime( void )
{       SYSTEMTIME      *pTime = new SYSTEMTIME;
        AnsiString      as;
        GetLocalTime( pTime );
        if ( !pTime ) return( NULL );
        if ( pTime->wHour < 10 ) as = "0" + IntToStr( pTime->wHour );
        else as = IntToStr( pTime->wHour );
        as += ":";
        if ( pTime->wMinute < 10 ) as += "0" + IntToStr( pTime->wMinute );
        else as += IntToStr( pTime->wMinute );
        as += ":";
        if ( pTime->wSecond < 10 ) as += "0" + IntToStr( pTime->wSecond );
        else as += IntToStr( pTime->wSecond ); 
        return( as );
}
//---------------------------------------------------------------------------
//	Like above but for the system date
//----------------------------------------------------------------------------
AnsiString      __fastcall      SystemDate( void )
{       SYSTEMTIME      *pTime = new SYSTEMTIME;
        AnsiString      as;
        GetSystemTime( pTime );
        if ( !pTime ) return( NULL );
        if ( pTime->wDay < 10 ) as = "0" + IntToStr( pTime->wDay );
        else as = IntToStr( pTime->wDay );
        as += "/";
        if ( pTime->wMonth < 10 ) as += "0" + IntToStr( pTime->wMonth );
        else as += IntToStr( pTime->wMonth );
        as += "/" + IntToStr( pTime->wYear );
        return( as );
}
//---------------------------------------------------------------------------
//	Calls both of the above together returning the concated string
//----------------------------------------------------------------------------
AnsiString      __fastcall      SystemTimeDate( void )
{       AnsiString as = SystemTime() + " @ " + SystemDate();
        return( as );
}
//---------------------------------------------------------------------------
//	Transfer, via RAM, the source to the destination
//----------------------------------------------------------------------------
bool    __fastcall      FileCopy( AnsiString Source, AnsiString Destination )
{       TMemoryStream   *mStream;
        if ( ( Source.Length() <= 0 ) || ( Destination.Length() <= 0 ) ) return( false );
        if ( !FileExists( Source ) ) return( false );
        mStream = new TMemoryStream;
        mStream->LoadFromFile( Source );
        mStream->SaveToFile( Destination );
        delete mStream;
        return( true );
}
//---------------------------------------------------------------------------
//	returns the filename part of a filepath, note includes the extension
//----------------------------------------------------------------------------
AnsiString	__fastcall	GetFileName( AnsiString FilePath )
{	AnsiString	as;
	char		*token;

        token = strtok( FilePath.c_str(), "\\\n\0" );
        while ( token ) {
        	as = token;
                token = strtok( NULL, "\\\n\0" );
        }

        token = StrPos( FilePath.c_str(), as.c_str() );
        return( AnsiString( token ) );
}
//---------------------------------------------------------------------------
//	Like above but just the 3 letter extension is returned
//----------------------------------------------------------------------------
AnsiString	__fastcall	GetFileExtension( AnsiString FilePath )
{
	return( StrPos( FilePath.c_str(), "." ) );
}
//---------------------------------------------------------------------------
//	Quits the application after displaying a message dialog box with
// the given text to display
//----------------------------------------------------------------------------
void	__fastcall	ErrorExit( AnsiString Msg )
{
	MessageBox( Application->Handle, Msg.c_str(), Application->Title.c_str(), MB_OK | MB_ICONEXCLAMATION );
        Application->Terminate();
}
//---------------------------------------------------------------------------
//	Displays a line of text in the main ide form's system message window
//----------------------------------------------------------------------------
void	__fastcall	SystemMessage( AnsiString Msg )
{
	if ( Msg.Length() <= 0 ) return;
        MainForm->RichEdit1->Lines->Add( Msg );
}
//---------------------------------------------------------------------------
//	Checks to make sure that the specified directory path is actually a
// directory, ie the directory attribute is set
//---------------------------------------------------------------------------
bool	__fastcall	IsDirPath( AnsiString DirPath )
{	int	fAttr;
	if ( DirPath.Length() <= 0 ) return( false );
        fAttr = FileGetAttr( DirPath );
        if ( fAttr && 0x10 ) return( true );
        else return( false );
}
//---------------------------------------------------------------------------
//	Checks to make sure that the specified directory path exists, does not
// determine whether the specified path is file or directory, or other object
//---------------------------------------------------------------------------
bool	__fastcall	DirExists( AnsiString DirPath )
{	AnsiString	CurDir = GetCurrentDir();			//	Get current directory

	if ( DirPath.Length() <= 0 ) return( false );	//	Bad directory name
        if ( SetCurrentDir( DirPath ) ) {		//	Can we changed to this path, ie is dir and not file
        	SetCurrentDir( CurDir );		//	Restore original directory
                return( true );				//	Yes
        }
        return( false );				//	No, not found, or not a valid dir path
}
//---------------------------------------------------------------------------
//	Clears the system message view
//----------------------------------------------------------------------------
void	__fastcall	ClearSysMessages( void )
{
	MainForm->RichEdit1->Lines->Clear();	//	Disabled for debugging purposes
}
//---------------------------------------------------------------------------
//	returns the size in bytes of a file
//----------------------------------------------------------------------------
long 	__fastcall	FileSize( FILE *fStream )
{	long curpos, length;
	curpos = ftell( fStream );
        fseek( fStream, 0L, SEEK_END );
   	length = ftell( fStream );
   	fseek( fStream, curpos, SEEK_SET );
   	return( length );
}
//---------------------------------------------------------------------------
//	Expands macros for the tools
//----------------------------------------------------------------------------
AnsiString      __fastcall      ExpandMacro( AnsiString asMacro )
{       
        SystemMessage( "Macro passed '" + asMacro + "'" );
}
//---------------------------------------------------------------------------
//	returns just the drive portion of a filepath or null
//----------------------------------------------------------------------------
AnsiString	__fastcall	GetDrivePath( AnsiString FilePath )
{
	if ( ( FilePath.Length() <= 0 ) && ( FilePath.Pos( ':' ) <= 0 ) )
        	return( NULL );

        return( FilePath.SubString( 0, FilePath.Pos( ':' ) ) );
}
//----------------------------------------------------------------------------
//	returns just the directory path of a filepath or null
//----------------------------------------------------------------------------
AnsiString	__fastcall	GetDirPath( AnsiString FilePath )
{
	if ( ( FilePath.Length() <= 0 ) && ( FilePath.Pos( ':' ) <= 0 ) )
        	return( NULL );

        return( FilePath.SubString( FilePath.Pos( ':' ) + 1, FilePath.Length() ) );
}
//----------------------------------------------------------------------------
AnsiString	__fastcall	GetValue( AnsiString aLine )
{	AnsiString	aValue;

	if ( aLine.Length() <= 0 )
        	return( NULL );

        if ( aLine.Pos( '=' ) == 0 )
        	return( NULL );

        aValue = aLine.SubString( aLine.Pos( '=' ) + 1, ( aLine.Length() - aLine.Pos( '=' ) + 1 ) );

        return( aValue );
}
//---------------------------------------------------------------------------
